@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2014, 2020, Red Hat Inc. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -46,6 +46,7 @@ protected:
   static int _dcache_line_size;
   static int _icache_line_size;
   static int _initial_sve_vector_length;
+  static int _max_supported_sve_vector_length;
   static bool _rop_protection;
   static uintptr_t _pac_mask;
 
@@ -124,6 +125,8 @@ enum Ampere_CPU_Model {
     decl(SHA2,          sha256,        6)     \
     decl(CRC32,         crc32,         7)     \
     decl(LSE,           lse,           8)     \
+    decl(FPHP,          fphp,          9)     \
+    decl(ASIMDHP,       asimdhp,       10)    \
     decl(DCPOP,         dcpop,         16)    \
     decl(SHA3,          sha3,          17)    \
     decl(SHA512,        sha512,        21)    \
@@ -164,15 +167,18 @@ enum Ampere_CPU_Model {
 
   static int icache_line_size() { return _icache_line_size; }
   static int dcache_line_size() { return _dcache_line_size; }
-  static int get_initial_sve_vector_length()  { return _initial_sve_vector_length; };
+  static int get_initial_sve_vector_length()        { return _initial_sve_vector_length; };
+  static int get_max_supported_sve_vector_length()  { return _max_supported_sve_vector_length; };
 
+  // Aarch64 supports fast class initialization checks
   static bool supports_fast_class_init_checks() { return true; }
   constexpr static bool supports_stack_watermark_barrier() { return true; }
-
-  static void get_compatible_board(char *buf, int buflen);
+  constexpr static bool supports_recursive_lightweight_locking() { return true; }
 
   constexpr static bool supports_secondary_supers_table() { return true; }
 
+  static void get_compatible_board(char *buf, int buflen);
+
   static const SpinWait& spin_wait_desc() { return _spin_wait; }
 
   static bool supports_on_spin_wait() { return _spin_wait.inst() != SpinWait::NONE; }
