@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -29,18 +29,20 @@
 #include "gc/g1/g1BarrierSetRuntime.hpp"
 #include "gc/g1/g1ThreadLocalData.hpp"
 #include "gc/g1/g1CardTable.hpp"
+#include "gc/g1/g1HeapRegion.hpp"
 #include "gc/g1/g1ThreadLocalData.hpp"
-#include "gc/g1/heapRegion.hpp"
 #include "interpreter/interp_masm.hpp"
+#include "runtime/javaThread.hpp"
 #include "runtime/sharedRuntime.hpp"
-#include "runtime/thread.hpp"
 #include "utilities/macros.hpp"
 #ifdef COMPILER1
 #include "c1/c1_LIRAssembler.hpp"
 #include "c1/c1_MacroAssembler.hpp"
 #include "gc/g1/c1/g1BarrierSetC1.hpp"
-#endif
-
+#endif // COMPILER1
+#ifdef COMPILER2
+#include "gc/g1/c2/g1BarrierSetC2.hpp"
+#endif // COMPILER2
 #define __ masm->
 
 #ifdef PRODUCT
@@ -106,70 +108,87 @@ void G1BarrierSetAssembler::gen_write_ref_array_post_barrier(MacroAssembler* mas
 #endif // !R9_IS_SCRATCHED
 }
 
+static void generate_queue_test_and_insertion(MacroAssembler* masm, ByteSize index_offset, ByteSize buffer_offset, Label& runtime,
+                                              const Register thread, const Register value, const Register temp1, const Register temp2) {
+  assert_different_registers(value, temp1, temp2);
+  // Can we store original value in the thread's buffer?
+  // (The index field is typed as size_t.)
+  __ ldr(temp1, Address(thread, in_bytes(index_offset)));  // temp1 := *(index address)
+  __ cbz(temp1, runtime);                                  // jump to runtime if index == 0 (full buffer)
+  // The buffer is not full, store value into it.
+  __ sub(temp1, temp1, wordSize);                          // temp1 := next index
+  __ str(temp1, Address(thread, in_bytes(index_offset)));  // *(index address) := next index
+  __ ldr(temp2, Address(thread, in_bytes(buffer_offset))); // temp2 := buffer address
+  // Record the previous value
+  __ str(value, Address(temp2, temp1));                    // *(buffer address + next index) := value
+ }
+
+static void generate_pre_barrier_fast_path(MacroAssembler* masm,
+                                           const Register thread,
+                                           const Register tmp1) {
+  Address in_progress(thread, in_bytes(G1ThreadLocalData::satb_mark_queue_active_offset()));
+  // Is marking active?
+  assert(in_bytes(SATBMarkQueue::byte_width_of_active()) == 1, "adjust this code");
+  __ ldrb(tmp1, in_progress);
+}
+
+static void generate_pre_barrier_slow_path(MacroAssembler* masm,
+                                           const Register obj,
+                                           const Register pre_val,
+                                           const Register thread,
+                                           const Register tmp1,
+                                           const Register tmp2,
+                                           Label& done,
+                                           Label& runtime) {
+  // Do we need to load the previous value?
+  if (obj != noreg) {
+    __ load_heap_oop(pre_val, Address(obj, 0));
+  }
+
+  // Is the previous value null?
+  __ cbz(pre_val, done);
+
+  generate_queue_test_and_insertion(masm,
+                                    G1ThreadLocalData::satb_mark_queue_index_offset(),
+                                    G1ThreadLocalData::satb_mark_queue_buffer_offset(),
+                                    runtime,
+                                    thread, pre_val, tmp1, tmp2);
+  __ b(done);
+}
+
 // G1 pre-barrier.
-// Blows all volatile registers R0-R3, Rtemp, LR).
-// If store_addr != noreg, then previous value is loaded from [store_addr];
-// in such case store_addr and new_val registers are preserved;
+// Blows all volatile registers R0-R3, LR).
+// If obj != noreg, then previous value is loaded from [obj];
+// in such case obj and pre_val registers is preserved;
 // otherwise pre_val register is preserved.
 void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm,
-                                          Register store_addr,
-                                          Register new_val,
+                                          Register obj,
                                           Register pre_val,
                                           Register tmp1,
                                           Register tmp2) {
   Label done;
   Label runtime;
 
-  if (store_addr != noreg) {
-    assert_different_registers(store_addr, new_val, pre_val, tmp1, tmp2, noreg);
-  } else {
-    assert (new_val == noreg, "should be");
-    assert_different_registers(pre_val, tmp1, tmp2, noreg);
-  }
+  assert_different_registers(obj, pre_val, tmp1, tmp2, noreg);
 
-  Address in_progress(Rthread, in_bytes(G1ThreadLocalData::satb_mark_queue_active_offset()));
-  Address index(Rthread, in_bytes(G1ThreadLocalData::satb_mark_queue_index_offset()));
-  Address buffer(Rthread, in_bytes(G1ThreadLocalData::satb_mark_queue_buffer_offset()));
-
-  // Is marking active?
-  assert(in_bytes(SATBMarkQueue::byte_width_of_active()) == 1, "adjust this code");
-  __ ldrb(tmp1, in_progress);
+  generate_pre_barrier_fast_path(masm, Rthread, tmp1);
+  // If marking is not active (*(mark queue active address) == 0), jump to done
   __ cbz(tmp1, done);
 
-  // Do we need to load the previous value?
-  if (store_addr != noreg) {
-    __ load_heap_oop(pre_val, Address(store_addr, 0));
-  }
-
-  // Is the previous value null?
-  __ cbz(pre_val, done);
-
-  // Can we store original value in the thread's buffer?
-  // Is index == 0?
-  // (The index field is typed as size_t.)
-
-  __ ldr(tmp1, index);           // tmp1 := *index_adr
-  __ ldr(tmp2, buffer);
-
-  __ subs(tmp1, tmp1, wordSize); // tmp1 := tmp1 - wordSize
-  __ b(runtime, lt);             // If negative, goto runtime
-
-  __ str(tmp1, index);           // *index_adr := tmp1
-
-  // Record the previous value
-  __ str(pre_val, Address(tmp2, tmp1));
-  __ b(done);
+   generate_pre_barrier_slow_path(masm, obj, pre_val, Rthread, tmp1, tmp2, done, runtime);
 
   __ bind(runtime);
 
   // save the live input values
-  if (store_addr != noreg) {
-    // avoid raw_push to support any ordering of store_addr and new_val
-    __ push(RegisterSet(store_addr) | RegisterSet(new_val));
-  } else {
-    __ push(pre_val);
+  RegisterSet set = RegisterSet(pre_val) | RegisterSet(R0, R3) | RegisterSet(R12);
+  // save the live input values
+  if (obj != noreg) {
+    // avoid raw_push to support any ordering of store_addr and pre_val
+    set = set | RegisterSet(obj);
   }
 
+  __ push(set);
+
   if (pre_val != R0) {
     __ mov(R0, pre_val);
   }
@@ -177,86 +196,189 @@ void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm,
 
   __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_pre_entry), R0, R1);
 
-  if (store_addr != noreg) {
-    __ pop(RegisterSet(store_addr) | RegisterSet(new_val));
-  } else {
-    __ pop(pre_val);
-  }
-
+  __ pop(set);
   __ bind(done);
 }
 
+static void generate_post_barrier_fast_path(MacroAssembler* masm,
+                                            const Register store_addr,
+                                            const Register new_val,
+                                            const Register tmp1,
+                                            const Register tmp2,
+                                            Label& done,
+                                            bool new_val_may_be_null) {
+  // Does store cross heap regions?
+
+  __ eor(tmp1, store_addr, new_val);
+  __ movs(tmp1, AsmOperand(tmp1, lsr, G1HeapRegion::LogOfHRGrainBytes));
+  __ b(done, eq);
+
+  // crosses regions, storing null?
+  if (new_val_may_be_null) {
+    __ cbz(new_val, done);
+  }
+  // storing region crossing non-null, is card already dirty?
+  const Register card_addr = tmp1;
+
+  CardTableBarrierSet* ct = barrier_set_cast<CardTableBarrierSet>(BarrierSet::barrier_set());
+  __ mov_address(tmp2, (address)ct->card_table()->byte_map_base());
+  __ add(card_addr, tmp2, AsmOperand(store_addr, lsr, CardTable::card_shift()));
+
+  __ ldrb(tmp2, Address(card_addr));
+  __ cmp(tmp2, (int)G1CardTable::g1_young_card_val());
+}
+
+static void generate_post_barrier_slow_path(MacroAssembler* masm,
+                                            const Register thread,
+                                            const Register tmp1,
+                                            const Register tmp2,
+                                            const Register tmp3,
+                                            Label& done,
+                                            Label& runtime) {
+  __ membar(MacroAssembler::Membar_mask_bits(MacroAssembler::StoreLoad), tmp2);
+  assert(CardTable::dirty_card_val() == 0, "adjust this code");
+  // card_addr is loaded by generate_post_barrier_fast_path
+  const Register card_addr = tmp1;
+  __ ldrb(tmp2, Address(card_addr));
+  __ cbz(tmp2, done);
+
+  // storing a region crossing, non-null oop, card is clean.
+  // dirty card and log.
+
+  __ strb(__ zero_register(tmp2), Address(card_addr));
+  generate_queue_test_and_insertion(masm,
+                                    G1ThreadLocalData::dirty_card_queue_index_offset(),
+                                    G1ThreadLocalData::dirty_card_queue_buffer_offset(),
+                                    runtime,
+                                    thread, card_addr, tmp2, tmp3);
+  __ b(done);
+}
+
+
 // G1 post-barrier.
-// Blows all volatile registers R0-R3, Rtemp, LR).
+// Blows all volatile registers R0-R3,  LR).
 void G1BarrierSetAssembler::g1_write_barrier_post(MacroAssembler* masm,
                                            Register store_addr,
                                            Register new_val,
                                            Register tmp1,
                                            Register tmp2,
                                            Register tmp3) {
-
-  Address queue_index(Rthread, in_bytes(G1ThreadLocalData::dirty_card_queue_index_offset()));
-  Address buffer(Rthread, in_bytes(G1ThreadLocalData::dirty_card_queue_buffer_offset()));
-
-  BarrierSet* bs = BarrierSet::barrier_set();
-  CardTableBarrierSet* ctbs = barrier_set_cast<CardTableBarrierSet>(bs);
-  CardTable* ct = ctbs->card_table();
   Label done;
   Label runtime;
 
-  // Does store cross heap regions?
-
-  __ eor(tmp1, store_addr, new_val);
-  __ movs(tmp1, AsmOperand(tmp1, lsr, HeapRegion::LogOfHRGrainBytes));
-  __ b(done, eq);
-
-  // crosses regions, storing NULL?
-
-  __ cbz(new_val, done);
-
-  // storing region crossing non-NULL, is card already dirty?
+  generate_post_barrier_fast_path(masm, store_addr, new_val, tmp1, tmp2, done, true /* new_val_may_be_null */);
+  // If card is young, jump to done
+  // card_addr and card are loaded by generate_post_barrier_fast_path
+  const Register card      = tmp2;
   const Register card_addr = tmp1;
-
-  __ mov_address(tmp2, (address)ct->byte_map_base());
-  __ add(card_addr, tmp2, AsmOperand(store_addr, lsr, CardTable::card_shift));
-
-  __ ldrb(tmp2, Address(card_addr));
-  __ cmp(tmp2, (int)G1CardTable::g1_young_card_val());
-  __ b(done, eq);
-
-  __ membar(MacroAssembler::Membar_mask_bits(MacroAssembler::StoreLoad), tmp2);
-
-  assert(CardTable::dirty_card_val() == 0, "adjust this code");
-  __ ldrb(tmp2, Address(card_addr));
-  __ cbz(tmp2, done);
-
-  // storing a region crossing, non-NULL oop, card is clean.
-  // dirty card and log.
-
-  __ strb(__ zero_register(tmp2), Address(card_addr));
-
-  __ ldr(tmp2, queue_index);
-  __ ldr(tmp3, buffer);
-
-  __ subs(tmp2, tmp2, wordSize);
-  __ b(runtime, lt); // go to runtime if now negative
-
-  __ str(tmp2, queue_index);
-
-  __ str(card_addr, Address(tmp3, tmp2));
-  __ b(done);
+   __ b(done, eq);
+  generate_post_barrier_slow_path(masm, Rthread, card_addr, tmp2, tmp3, done, runtime);
 
   __ bind(runtime);
 
+  RegisterSet set = RegisterSet(store_addr) | RegisterSet(R0, R3) | RegisterSet(R12);
+  __ push(set);
+
   if (card_addr != R0) {
     __ mov(R0, card_addr);
   }
   __ mov(R1, Rthread);
   __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_post_entry), R0, R1);
 
+  __ pop(set);
+
   __ bind(done);
 }
 
+#if defined(COMPILER2)
+
+static void generate_c2_barrier_runtime_call(MacroAssembler* masm, G1BarrierStubC2* stub, const Register arg, const address runtime_path, Register tmp1) {
+  SaveLiveRegisters save_registers(masm, stub);
+  if (c_rarg0 != arg) {
+    __ mov(c_rarg0, arg);
+  }
+  __ mov(c_rarg1, Rthread);
+  __ call_VM_leaf(runtime_path, R0, R1);
+}
+
+void G1BarrierSetAssembler::g1_write_barrier_pre_c2(MacroAssembler* masm,
+                                                    Register obj,
+                                                    Register pre_val,
+                                                    Register thread,
+                                                    Register tmp1,
+                                                    Register tmp2,
+                                                    G1PreBarrierStubC2* stub) {
+  assert(thread == Rthread, "must be");
+  assert_different_registers(obj, pre_val, tmp1, tmp2);
+  assert(pre_val != noreg && tmp1 != noreg && tmp2 != noreg, "expecting a register");
+
+  stub->initialize_registers(obj, pre_val, thread, tmp1, tmp2);
+
+  generate_pre_barrier_fast_path(masm, thread, tmp1);
+  // If marking is active (*(mark queue active address) != 0), jump to stub (slow path)
+  __ cbnz(tmp1, *stub->entry());
+
+  __ bind(*stub->continuation());
+}
+
+void G1BarrierSetAssembler::generate_c2_pre_barrier_stub(MacroAssembler* masm,
+                                                         G1PreBarrierStubC2* stub) const {
+  Assembler::InlineSkippedInstructionsCounter skip_counter(masm);
+  Label runtime;
+  Register obj = stub->obj();
+  Register pre_val = stub->pre_val();
+  Register thread = stub->thread();
+  Register tmp1 = stub->tmp1();
+  Register tmp2 = stub->tmp2();
+
+  __ bind(*stub->entry());
+  generate_pre_barrier_slow_path(masm, obj, pre_val, thread, tmp1, tmp2, *stub->continuation(), runtime);
+
+  __ bind(runtime);
+  generate_c2_barrier_runtime_call(masm, stub, pre_val, CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_pre_entry), tmp1);
+  __ b(*stub->continuation());
+}
+
+void G1BarrierSetAssembler::g1_write_barrier_post_c2(MacroAssembler* masm,
+                                                     Register store_addr,
+                                                     Register new_val,
+                                                     Register thread,
+                                                     Register tmp1,
+                                                     Register tmp2,
+                                                     Register tmp3,
+                                                     G1PostBarrierStubC2* stub) {
+  assert(thread == Rthread, "must be");
+  assert_different_registers(store_addr, new_val, thread, tmp1, tmp2, noreg);
+
+  stub->initialize_registers(thread, tmp1, tmp2, tmp3);
+
+  bool new_val_may_be_null = (stub->barrier_data() & G1C2BarrierPostNotNull) == 0;
+  generate_post_barrier_fast_path(masm, store_addr, new_val, tmp1, tmp2, *stub->continuation(), new_val_may_be_null);
+  // If card is not young, jump to stub (slow path)
+  __ b(*stub->entry(), ne);
+
+  __ bind(*stub->continuation());
+}
+
+void G1BarrierSetAssembler::generate_c2_post_barrier_stub(MacroAssembler* masm,
+                                                          G1PostBarrierStubC2* stub) const {
+  Assembler::InlineSkippedInstructionsCounter skip_counter(masm);
+  Label runtime;
+  Register thread = stub->thread();
+  Register tmp1 = stub->tmp1(); // tmp1 holds the card address.
+  Register tmp2 = stub->tmp2();
+  Register tmp3 = stub->tmp3();
+
+  __ bind(*stub->entry());
+  generate_post_barrier_slow_path(masm, thread, tmp1, tmp2, tmp3,  *stub->continuation(), runtime);
+
+  __ bind(runtime);
+  generate_c2_barrier_runtime_call(masm, stub, tmp1, CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_post_entry), tmp2);
+  __ b(*stub->continuation());
+}
+
+#endif // COMPILER2
+
 void G1BarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorators, BasicType type,
                                     Register dst, Address src, Register tmp1, Register tmp2, Register tmp3) {
   bool on_oop = type == T_OBJECT || type == T_ARRAY;
@@ -268,7 +390,7 @@ void G1BarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorator
   if (on_oop && on_reference) {
     // Generate the G1 pre-barrier code to log the value of
     // the referent field in an SATB buffer.
-    g1_write_barrier_pre(masm, noreg, noreg, dst, tmp1, tmp2);
+    g1_write_barrier_pre(masm, noreg, dst, tmp1, tmp2);
   }
 }
 
@@ -295,7 +417,7 @@ void G1BarrierSetAssembler::oop_store_at(MacroAssembler* masm, DecoratorSet deco
   }
 
   if (needs_pre_barrier) {
-    g1_write_barrier_pre(masm, store_addr, new_val, tmp1, tmp2, tmp3);
+    g1_write_barrier_pre(masm, store_addr, tmp3 /*pre_val*/, tmp1, tmp2);
   }
 
   if (is_null) {
@@ -452,7 +574,7 @@ void G1BarrierSetAssembler::generate_c1_post_barrier_runtime_stub(StubAssembler*
   // explicitly specify that 'cardtable' has a relocInfo::none
   // type.
   __ lea(r_card_base_1, cardtable);
-  __ add(r_card_addr_0, r_card_base_1, AsmOperand(r_obj_0, lsr, CardTable::card_shift));
+  __ add(r_card_addr_0, r_card_base_1, AsmOperand(r_obj_0, lsr, CardTable::card_shift()));
 
   // first quick check without barrier
   __ ldrb(r_tmp2, Address(r_card_addr_0));
@@ -476,7 +598,7 @@ void G1BarrierSetAssembler::generate_c1_post_barrier_runtime_stub(StubAssembler*
   assert(CardTable::dirty_card_val() == 0, "adjust this code");
   __ cbz(r_tmp2, done);
 
-  // storing region crossing non-NULL, card is clean.
+  // storing region crossing non-null, card is clean.
   // dirty card and log.
 
   assert(0 == (int)CardTable::dirty_card_val(), "adjust this code");
