@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1999, 2024, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2012, 2024 SAP SE. All rights reserved.
+ * Copyright (c) 1999, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2025 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,7 +23,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "asm/macroAssembler.inline.hpp"
 #include "c1/c1_MacroAssembler.hpp"
 #include "c1/c1_Runtime1.hpp"
@@ -87,12 +86,12 @@ void C1_MacroAssembler::lock_object(Register Rmark, Register Roop, Register Rbox
   if (DiagnoseSyncOnValueBasedClasses != 0) {
     load_klass(Rscratch, Roop);
     lbz(Rscratch, in_bytes(Klass::misc_flags_offset()), Rscratch);
-    testbitdi(CCR0, R0, Rscratch, exact_log2(KlassFlags::_misc_is_value_based_class));
-    bne(CCR0, slow_int);
+    testbitdi(CR0, R0, Rscratch, exact_log2(KlassFlags::_misc_is_value_based_class));
+    bne(CR0, slow_int);
   }
 
   if (LockingMode == LM_LIGHTWEIGHT) {
-    lightweight_lock(Roop, Rmark, Rscratch, slow_int);
+    lightweight_lock(Rbox, Roop, Rmark, Rscratch, slow_int);
   } else if (LockingMode == LM_LEGACY) {
     // ... and mark it unlocked.
     ori(Rmark, Rmark, markWord::unlocked_value);
@@ -102,7 +101,7 @@ void C1_MacroAssembler::lock_object(Register Rmark, Register Roop, Register Rbox
 
     // Compare object markWord with Rmark and if equal exchange Rscratch with object markWord.
     assert(oopDesc::mark_offset_in_bytes() == 0, "cas must take a zero displacement");
-    cmpxchgd(/*flag=*/CCR0,
+    cmpxchgd(/*flag=*/CR0,
              /*current_value=*/Rscratch,
              /*compare_value=*/Rmark,
              /*exchange_value=*/Rbox,
@@ -114,6 +113,8 @@ void C1_MacroAssembler::lock_object(Register Rmark, Register Roop, Register Rbox
              /*check without membar and ldarx first*/true);
     // If compare/exchange succeeded we found an unlocked object and we now have locked it
     // hence we are done.
+  } else {
+    assert(false, "Unhandled LockingMode:%d", LockingMode);
   }
   b(done);
 
@@ -127,11 +128,13 @@ void C1_MacroAssembler::lock_object(Register Rmark, Register Roop, Register Rbox
     load_const_optimized(R0, (~(os::vm_page_size()-1) | markWord::lock_mask_in_place));
     and_(R0/*==0?*/, Rscratch, R0);
     std(R0/*==0, perhaps*/, BasicLock::displaced_header_offset_in_bytes(), Rbox);
-    bne(CCR0, slow_int);
+    bne(CR0, slow_int);
   }
 
   bind(done);
-  inc_held_monitor_count(Rmark /*tmp*/);
+  if (LockingMode == LM_LEGACY) {
+    inc_held_monitor_count(Rmark /*tmp*/);
+  }
 }
 
 
@@ -146,8 +149,8 @@ void C1_MacroAssembler::unlock_object(Register Rmark, Register Roop, Register Rb
   if (LockingMode != LM_LIGHTWEIGHT) {
     // Test first if it is a fast recursive unlock.
     ld(Rmark, BasicLock::displaced_header_offset_in_bytes(), Rbox);
-    cmpdi(CCR0, Rmark, 0);
-    beq(CCR0, done);
+    cmpdi(CR0, Rmark, 0);
+    beq(CR0, done);
   }
 
   // Load object.
@@ -159,7 +162,7 @@ void C1_MacroAssembler::unlock_object(Register Rmark, Register Roop, Register Rb
   } else if (LockingMode == LM_LEGACY) {
     // Check if it is still a light weight lock, this is is true if we see
     // the stack address of the basicLock in the markWord of the object.
-    cmpxchgd(/*flag=*/CCR0,
+    cmpxchgd(/*flag=*/CR0,
              /*current_value=*/R0,
              /*compare_value=*/Rbox,
              /*exchange_value=*/Rmark,
@@ -168,6 +171,8 @@ void C1_MacroAssembler::unlock_object(Register Rmark, Register Roop, Register Rb
              MacroAssembler::cmpxchgx_hint_release_lock(),
              noreg,
              &slow_int);
+  } else {
+    assert(false, "Unhandled LockingMode:%d", LockingMode);
   }
   b(done);
   bind(slow_int);
@@ -175,7 +180,9 @@ void C1_MacroAssembler::unlock_object(Register Rmark, Register Roop, Register Rb
 
   // Done
   bind(done);
-  dec_held_monitor_count(Rmark /*tmp*/);
+  if (LockingMode == LM_LEGACY) {
+    dec_held_monitor_count(Rmark /*tmp*/);
+  }
 }
 
 
@@ -197,12 +204,19 @@ void C1_MacroAssembler::try_allocate(
 
 void C1_MacroAssembler::initialize_header(Register obj, Register klass, Register len, Register t1, Register t2) {
   assert_different_registers(obj, klass, len, t1, t2);
-  load_const_optimized(t1, (intx)markWord::prototype().value());
-  std(t1, oopDesc::mark_offset_in_bytes(), obj);
-  store_klass(obj, klass);
+
+  if (UseCompactObjectHeaders) {
+    ld(t1, in_bytes(Klass::prototype_header_offset()), klass);
+    std(t1, oopDesc::mark_offset_in_bytes(), obj);
+  } else {
+    load_const_optimized(t1, (intx)markWord::prototype().value());
+    std(t1, oopDesc::mark_offset_in_bytes(), obj);
+    store_klass(obj, klass);
+  }
+
   if (len->is_valid()) {
     stw(len, arrayOopDesc::length_offset_in_bytes(), obj);
-  } else if (UseCompressedClassPointers) {
+  } else if (UseCompressedClassPointers && !UseCompactObjectHeaders) {
     // Otherwise length is in the class gap.
     store_klass_gap(obj);
   }
@@ -271,9 +285,9 @@ void C1_MacroAssembler::initialize_object(
   {
     lwz(t1, in_bytes(Klass::layout_helper_offset()), klass);
     if (var_size_in_bytes != noreg) {
-      cmpw(CCR0, t1, var_size_in_bytes);
+      cmpw(CR0, t1, var_size_in_bytes);
     } else {
-      cmpwi(CCR0, t1, con_size_in_bytes);
+      cmpwi(CR0, t1, con_size_in_bytes);
     }
     asm_assert_eq("bad size in initialize_object");
   }
@@ -326,8 +340,8 @@ void C1_MacroAssembler::allocate_array(
     if (max_tlab < max_length) { max_length = max_tlab; }
   }
   load_const_optimized(t1, max_length);
-  cmpld(CCR0, len, t1);
-  bc_far_optimized(Assembler::bcondCRbiIs1, bi0(CCR0, Assembler::greater), slow_case);
+  cmpld(CR0, len, t1);
+  bc_far_optimized(Assembler::bcondCRbiIs1, bi0(CR0, Assembler::greater), slow_case);
 
   // compute array size
   // note: If 0 <= len <= max_length, len*elt_size + header + alignment is
@@ -385,8 +399,8 @@ void C1_MacroAssembler::verify_stack_oop(int stack_offset) {
 
 void C1_MacroAssembler::verify_not_null_oop(Register r) {
   Label not_null;
-  cmpdi(CCR0, r, 0);
-  bne(CCR0, not_null);
+  cmpdi(CR0, r, 0);
+  bne(CR0, not_null);
   stop("non-null oop required");
   bind(not_null);
   verify_oop(r, FILE_AND_LINE);
@@ -400,7 +414,7 @@ void C1_MacroAssembler::null_check(Register r, Label* Lnull) {
   } else { // explicit
     //const address exception_entry = Runtime1::entry_for(C1StubId::throw_null_pointer_exception_id);
     assert(Lnull != nullptr, "must have Label for explicit check");
-    cmpdi(CCR0, r, 0);
-    bc_far_optimized(Assembler::bcondCRbiIs1, bi0(CCR0, Assembler::equal), *Lnull);
+    cmpdi(CR0, r, 0);
+    bc_far_optimized(Assembler::bcondCRbiIs1, bi0(CR0, Assembler::equal), *Lnull);
   }
 }
