@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2012, 2020 SAP SE. All rights reserved.
+ * Copyright (c) 2002, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2024 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -34,7 +34,9 @@
 
 define_pd_global(bool, ImplicitNullChecks,    true);  // Generate code for implicit null checks.
 define_pd_global(bool, TrapBasedNullChecks,   true);
-define_pd_global(bool, UncommonNullCast,      true);  // Uncommon-trap NULLs passed to check cast.
+define_pd_global(bool, UncommonNullCast,      true);  // Uncommon-trap nulls passed to check cast.
+
+define_pd_global(bool, DelayCompilerStubsGeneration, COMPILER2_OR_JVMCI);
 
 #define DEFAULT_STACK_YELLOW_PAGES (2)
 #define DEFAULT_STACK_RED_PAGES (1)
@@ -54,11 +56,11 @@ define_pd_global(intx, StackRedPages,         DEFAULT_STACK_RED_PAGES);
 define_pd_global(intx, StackShadowPages,      DEFAULT_STACK_SHADOW_PAGES);
 define_pd_global(intx, StackReservedPages,    DEFAULT_STACK_RESERVED_PAGES);
 
-define_pd_global(bool,  VMContinuations, false);
+define_pd_global(bool,  VMContinuations, true);
 
 // Use large code-entry alignment.
 define_pd_global(uintx, CodeCacheSegmentSize,  128);
-define_pd_global(intx,  CodeEntryAlignment,    128);
+define_pd_global(intx,  CodeEntryAlignment,    64);
 define_pd_global(intx,  OptoLoopAlignment,     16);
 define_pd_global(intx,  InlineSmallCode,       1500);
 
@@ -78,7 +80,6 @@ define_pd_global(intx, InitArrayShortSize, 9*BytesPerLong);
 // Platform dependent flag handling: flags only defined on this platform.
 #define ARCH_FLAGS(develop,                                                 \
                    product,                                                 \
-                   notproduct,                                              \
                    range,                                                   \
                    constraint)                                              \
                                                                             \
@@ -114,7 +115,7 @@ define_pd_global(intx, InitArrayShortSize, 9*BytesPerLong);
           "Use static branch prediction hints for uncommon paths.")         \
                                                                             \
   /* special instructions */                                                \
-  product(bool, SuperwordUseVSX, false,                                     \
+  product(bool, SuperwordUseVSX, true,                                      \
           "Use Power8 VSX instructions for superword optimization.")        \
                                                                             \
   product(bool, UseByteReverseInstructions, false, DIAGNOSTIC,              \
@@ -149,50 +150,7 @@ define_pd_global(intx, InitArrayShortSize, 9*BytesPerLong);
           "Trace all traps the signal handler handles.")                    \
                                                                             \
   develop(bool, ZapMemory, false,                                           \
-          "Write 0x0101... to empty memory. Use this to ease debugging.")   \
-                                                                            \
-  /* Use Restricted Transactional Memory for lock elision */                \
-  product(bool, UseRTMLocking, false,                                       \
-          "Enable RTM lock eliding for inflated locks in compiled code")    \
-                                                                            \
-  product(bool, UseRTMForStackLocks, false, EXPERIMENTAL,                   \
-          "Enable RTM lock eliding for stack locks in compiled code")       \
-                                                                            \
-  product(bool, UseRTMDeopt, false,                                         \
-          "Perform deopt and recompilation based on RTM abort ratio")       \
-                                                                            \
-  product(int, RTMRetryCount, 5,                                            \
-          "Number of RTM retries on lock abort or busy")                    \
-          range(0, max_jint)                                                \
-                                                                            \
-  product(int, RTMSpinLoopCount, 100, EXPERIMENTAL,                         \
-          "Spin count for lock to become free before RTM retry")            \
-          range(0, 32767) /* immediate operand limit on ppc */              \
-                                                                            \
-  product(int, RTMAbortThreshold, 1000, EXPERIMENTAL,                       \
-          "Calculate abort ratio after this number of aborts")              \
-          range(0, max_jint)                                                \
-                                                                            \
-  product(int, RTMLockingThreshold, 10000, EXPERIMENTAL,                    \
-          "Lock count at which to do RTM lock eliding without "             \
-          "abort ratio calculation")                                        \
-          range(0, max_jint)                                                \
-                                                                            \
-  product(int, RTMAbortRatio, 50, EXPERIMENTAL,                             \
-          "Lock abort ratio at which to stop use RTM lock eliding")         \
-          range(0, 100) /* natural range */                                 \
-                                                                            \
-  product(int, RTMTotalCountIncrRate, 64, EXPERIMENTAL,                     \
-          "Increment total RTM attempted lock count once every n times")    \
-          range(1, 32767) /* immediate operand limit on ppc */              \
-          constraint(RTMTotalCountIncrRateConstraintFunc,AfterErgo)         \
-                                                                            \
-  product(intx, RTMLockingCalculationDelay, 0, EXPERIMENTAL,                \
-          "Number of milliseconds to wait before start calculating aborts " \
-          "for RTM locking")                                                \
-                                                                            \
-  product(bool, UseRTMXendForLockBusy, true, EXPERIMENTAL,                  \
-          "Use RTM Xend instead of Xabort when lock busy")
+          "Write 0x0101... to empty memory. Use this to ease debugging.")
 
 // end of ARCH_FLAGS
 
