@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2014, 2024, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2015, 2024 SAP SE. All rights reserved.
+ * Copyright (c) 2014, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2025 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,7 +23,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "asm/macroAssembler.inline.hpp"
 #include "classfile/javaClasses.hpp"
 #include "compiler/disassembler.hpp"
@@ -144,12 +143,12 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
     // TODO PPC port: requires change in shared code.
     //assert(in_bytes(AccessFlags::flags_offset()) == 0,
     //       "MethodDesc._access_flags == MethodDesc._access_flags._flags");
-    // _access_flags must be a 32 bit value.
-    assert(sizeof(AccessFlags) == 4, "wrong size");
-    __ lwa(R11_scratch1/*access_flags*/, method_(access_flags));
+    // _access_flags must be a 16 bit value.
+    assert(sizeof(AccessFlags) == 2, "wrong size");
+    __ lhz(R11_scratch1/*access_flags*/, method_(access_flags));
     // testbit with condition register.
-    __ testbitdi(CCR0, R0, R11_scratch1/*access_flags*/, JVM_ACC_STATIC_BIT);
-    __ btrue(CCR0, L);
+    __ testbitdi(CR0, R0, R11_scratch1/*access_flags*/, JVM_ACC_STATIC_BIT);
+    __ btrue(CR0, L);
     // For non-static functions, pass "this" in R4_ARG2 and copy it
     // to 2nd C-arg slot.
     // We need to box the Java object here, so we use arg_java
@@ -176,8 +175,8 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
   // signature points to '(' at entry
 #ifdef ASSERT
   __ lbz(sig_byte, 0, signature);
-  __ cmplwi(CCR0, sig_byte, '(');
-  __ bne(CCR0, do_dontreachhere);
+  __ cmplwi(CR0, sig_byte, '(');
+  __ bne(CR0, do_dontreachhere);
 #endif
 
   __ bind(loop_start);
@@ -185,41 +184,41 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
   __ addi(argcnt, argcnt, 1);
   __ lbzu(sig_byte, 1, signature);
 
-  __ cmplwi(CCR0, sig_byte, ')'); // end of signature
-  __ beq(CCR0, loop_end);
+  __ cmplwi(CR0, sig_byte, ')'); // end of signature
+  __ beq(CR0, loop_end);
 
-  __ cmplwi(CCR0, sig_byte, 'B'); // byte
-  __ beq(CCR0, do_int);
+  __ cmplwi(CR0, sig_byte, 'B'); // byte
+  __ beq(CR0, do_int);
 
-  __ cmplwi(CCR0, sig_byte, 'C'); // char
-  __ beq(CCR0, do_int);
+  __ cmplwi(CR0, sig_byte, 'C'); // char
+  __ beq(CR0, do_int);
 
-  __ cmplwi(CCR0, sig_byte, 'D'); // double
-  __ beq(CCR0, do_double);
+  __ cmplwi(CR0, sig_byte, 'D'); // double
+  __ beq(CR0, do_double);
 
-  __ cmplwi(CCR0, sig_byte, 'F'); // float
-  __ beq(CCR0, do_float);
+  __ cmplwi(CR0, sig_byte, 'F'); // float
+  __ beq(CR0, do_float);
 
-  __ cmplwi(CCR0, sig_byte, 'I'); // int
-  __ beq(CCR0, do_int);
+  __ cmplwi(CR0, sig_byte, 'I'); // int
+  __ beq(CR0, do_int);
 
-  __ cmplwi(CCR0, sig_byte, 'J'); // long
-  __ beq(CCR0, do_long);
+  __ cmplwi(CR0, sig_byte, 'J'); // long
+  __ beq(CR0, do_long);
 
-  __ cmplwi(CCR0, sig_byte, 'S'); // short
-  __ beq(CCR0, do_int);
+  __ cmplwi(CR0, sig_byte, 'S'); // short
+  __ beq(CR0, do_int);
 
-  __ cmplwi(CCR0, sig_byte, 'Z'); // boolean
-  __ beq(CCR0, do_int);
+  __ cmplwi(CR0, sig_byte, 'Z'); // boolean
+  __ beq(CR0, do_int);
 
-  __ cmplwi(CCR0, sig_byte, 'L'); // object
-  __ beq(CCR0, do_object);
+  __ cmplwi(CR0, sig_byte, 'L'); // object
+  __ beq(CR0, do_object);
 
-  __ cmplwi(CCR0, sig_byte, '['); // array
-  __ beq(CCR0, do_array);
+  __ cmplwi(CR0, sig_byte, '['); // array
+  __ beq(CR0, do_array);
 
-  //  __ cmplwi(CCR0, sig_byte, 'V'); // void cannot appear since we do not parse the return type
-  //  __ beq(CCR0, do_void);
+  //  __ cmplwi(CR0, sig_byte, 'V'); // void cannot appear since we do not parse the return type
+  //  __ beq(CR0, do_void);
 
   __ bind(do_dontreachhere);
 
@@ -232,16 +231,16 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
 
     __ bind(start_skip);
     __ lbzu(sig_byte, 1, signature);
-    __ cmplwi(CCR0, sig_byte, '[');
-    __ beq(CCR0, start_skip); // skip further brackets
-    __ cmplwi(CCR0, sig_byte, '9');
-    __ bgt(CCR0, end_skip);   // no optional size
-    __ cmplwi(CCR0, sig_byte, '0');
-    __ bge(CCR0, start_skip); // skip optional size
+    __ cmplwi(CR0, sig_byte, '[');
+    __ beq(CR0, start_skip); // skip further brackets
+    __ cmplwi(CR0, sig_byte, '9');
+    __ bgt(CR0, end_skip);   // no optional size
+    __ cmplwi(CR0, sig_byte, '0');
+    __ bge(CR0, start_skip); // skip optional size
     __ bind(end_skip);
 
-    __ cmplwi(CCR0, sig_byte, 'L');
-    __ beq(CCR0, do_object);  // for arrays of objects, the name of the object must be skipped
+    __ cmplwi(CR0, sig_byte, 'L');
+    __ beq(CR0, do_object);  // for arrays of objects, the name of the object must be skipped
     __ b(do_boxed);          // otherwise, go directly to do_boxed
   }
 
@@ -250,8 +249,8 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
     Label L;
     __ bind(L);
     __ lbzu(sig_byte, 1, signature);
-    __ cmplwi(CCR0, sig_byte, ';');
-    __ bne(CCR0, L);
+    __ cmplwi(CR0, sig_byte, ';');
+    __ bne(CR0, L);
    }
   // Need to box the Java object here, so we use arg_java (address of
   // current Java stack slot) as argument and don't dereference it as
@@ -259,16 +258,16 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
   Label do_null;
   __ bind(do_boxed);
   __ ld(R0,0, arg_java);
-  __ cmpdi(CCR0, R0, 0);
+  __ cmpdi(CR0, R0, 0);
   __ li(intSlot,0);
-  __ beq(CCR0, do_null);
+  __ beq(CR0, do_null);
   __ mr(intSlot, arg_java);
   __ bind(do_null);
   __ std(intSlot, 0, arg_c);
   __ addi(arg_java, arg_java, -BytesPerWord);
   __ addi(arg_c, arg_c, BytesPerWord);
-  __ cmplwi(CCR0, argcnt, max_int_register_arguments);
-  __ blt(CCR0, move_intSlot_to_ARG);
+  __ cmplwi(CR0, argcnt, max_int_register_arguments);
+  __ blt(CR0, move_intSlot_to_ARG);
   __ b(loop_start);
 
   __ bind(do_int);
@@ -276,8 +275,8 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
   __ std(intSlot, 0, arg_c);
   __ addi(arg_java, arg_java, -BytesPerWord);
   __ addi(arg_c, arg_c, BytesPerWord);
-  __ cmplwi(CCR0, argcnt, max_int_register_arguments);
-  __ blt(CCR0, move_intSlot_to_ARG);
+  __ cmplwi(CR0, argcnt, max_int_register_arguments);
+  __ blt(CR0, move_intSlot_to_ARG);
   __ b(loop_start);
 
   __ bind(do_long);
@@ -285,8 +284,8 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
   __ std(intSlot, 0, arg_c);
   __ addi(arg_java, arg_java, - 2 * BytesPerWord);
   __ addi(arg_c, arg_c, BytesPerWord);
-  __ cmplwi(CCR0, argcnt, max_int_register_arguments);
-  __ blt(CCR0, move_intSlot_to_ARG);
+  __ cmplwi(CR0, argcnt, max_int_register_arguments);
+  __ blt(CR0, move_intSlot_to_ARG);
   __ b(loop_start);
 
   __ bind(do_float);
@@ -294,8 +293,8 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
   __ stfs(floatSlot, Argument::float_on_stack_offset_in_bytes_c, arg_c);
   __ addi(arg_java, arg_java, -BytesPerWord);
   __ addi(arg_c, arg_c, BytesPerWord);
-  __ cmplwi(CCR0, fpcnt, max_fp_register_arguments);
-  __ blt(CCR0, move_floatSlot_to_FARG);
+  __ cmplwi(CR0, fpcnt, max_fp_register_arguments);
+  __ blt(CR0, move_floatSlot_to_FARG);
   __ b(loop_start);
 
   __ bind(do_double);
@@ -303,8 +302,8 @@ address TemplateInterpreterGenerator::generate_slow_signature_handler() {
   __ stfd(floatSlot, 0, arg_c);
   __ addi(arg_java, arg_java, - 2 * BytesPerWord);
   __ addi(arg_c, arg_c, BytesPerWord);
-  __ cmplwi(CCR0, fpcnt, max_fp_register_arguments);
-  __ blt(CCR0, move_floatSlot_to_FARG);
+  __ cmplwi(CR0, fpcnt, max_fp_register_arguments);
+  __ blt(CR0, move_floatSlot_to_FARG);
   __ b(loop_start);
 
   __ bind(loop_end);
@@ -511,8 +510,8 @@ address TemplateInterpreterGenerator::generate_Reference_get_entry(void) {
   __ ld(R3_RET, Interpreter::stackElementSize, R15_esp); // get receiver
 
   // Check if receiver == nullptr and go the slow path.
-  __ cmpdi(CCR0, R3_RET, 0);
-  __ beq(CCR0, slow_path);
+  __ cmpdi(CR0, R3_RET, 0);
+  __ beq(CR0, slow_path);
 
   __ load_heap_oop(R3_RET, referent_offset, R3_RET,
                    /* non-volatile temp */ R31, R11_scratch1,
@@ -696,6 +695,17 @@ address TemplateInterpreterGenerator::generate_safept_entry_for(TosState state,
   return entry;
 }
 
+address TemplateInterpreterGenerator::generate_cont_resume_interpreter_adapter() {
+  if (!Continuations::enabled()) return nullptr;
+  address start = __ pc();
+
+  __ load_const_optimized(R25_templateTableBase, (address)Interpreter::dispatch_table((TosState)0), R12_scratch2);
+  __ restore_interpreter_state(R11_scratch1, false, true /*restore_top_frame_sp*/);
+  __ blr();
+
+  return start;
+}
+
 // Helpers for commoning out cases in the various type of method entries.
 
 // Increment invocation count & check for overflow.
@@ -715,8 +725,8 @@ void TemplateInterpreterGenerator::generate_counter_incr(Label* overflow) {
   if (ProfileInterpreter) {
     const Register Rmdo = R3_counters;
     __ ld(Rmdo, in_bytes(Method::method_data_offset()), R19_method);
-    __ cmpdi(CCR0, Rmdo, 0);
-    __ beq(CCR0, no_mdo);
+    __ cmpdi(CR0, Rmdo, 0);
+    __ beq(CR0, no_mdo);
 
     // Increment invocation counter in the MDO.
     const int mdo_ic_offs = in_bytes(MethodData::invocation_counter_offset()) + in_bytes(InvocationCounter::counter_offset());
@@ -725,7 +735,7 @@ void TemplateInterpreterGenerator::generate_counter_incr(Label* overflow) {
     __ addi(Rscratch2, Rscratch2, increment);
     __ stw(Rscratch2, mdo_ic_offs, Rmdo);
     __ and_(Rscratch1, Rscratch2, Rscratch1);
-    __ bne(CCR0, done);
+    __ bne(CR0, done);
     __ b(*overflow);
   }
 
@@ -738,7 +748,7 @@ void TemplateInterpreterGenerator::generate_counter_incr(Label* overflow) {
   __ addi(Rscratch2, Rscratch2, increment);
   __ stw(Rscratch2, mo_ic_offs, R3_counters);
   __ and_(Rscratch1, Rscratch2, Rscratch1);
-  __ beq(CCR0, *overflow);
+  __ beq(CR0, *overflow);
 
   __ bind(done);
 }
@@ -779,8 +789,8 @@ void TemplateInterpreterGenerator::generate_stack_overflow_check(Register Rmem_f
   BLOCK_COMMENT("stack_overflow_check_with_compare {");
   __ sub(Rmem_frame_size, R1_SP, Rmem_frame_size);
   __ ld(Rscratch1, thread_(stack_overflow_limit));
-  __ cmpld(CCR0/*is_stack_overflow*/, Rmem_frame_size, Rscratch1);
-  __ bgt(CCR0/*is_stack_overflow*/, done);
+  __ cmpld(CR0/*is_stack_overflow*/, Rmem_frame_size, Rscratch1);
+  __ bgt(CR0/*is_stack_overflow*/, done);
 
   // The stack overflows. Load target address of the runtime stub and call it.
   assert(SharedRuntime::throw_StackOverflowError_entry() != nullptr, "generated in wrong order");
@@ -789,13 +799,13 @@ void TemplateInterpreterGenerator::generate_stack_overflow_check(Register Rmem_f
   // Restore caller_sp (c2i adapter may exist, but no shrinking of interpreted caller frame).
 #ifdef ASSERT
   Label frame_not_shrunk;
-  __ cmpld(CCR0, R1_SP, R21_sender_SP);
-  __ ble(CCR0, frame_not_shrunk);
+  __ cmpld(CR0, R1_SP, R21_sender_SP);
+  __ ble(CR0, frame_not_shrunk);
   __ stop("frame shrunk");
   __ bind(frame_not_shrunk);
   __ ld(Rscratch1, 0, R1_SP);
   __ ld(R0, 0, R21_sender_SP);
-  __ cmpd(CCR0, R0, Rscratch1);
+  __ cmpd(CR0, R0, Rscratch1);
   __ asm_assert_eq("backlink");
 #endif // ASSERT
   __ mr(R1_SP, R21_sender_SP);
@@ -812,15 +822,15 @@ void TemplateInterpreterGenerator::lock_method(Register Rflags, Register Rscratc
 
   {
     if (!flags_preloaded) {
-      __ lwz(Rflags, method_(access_flags));
+      __ lhz(Rflags, method_(access_flags));
     }
 
 #ifdef ASSERT
     // Check if methods needs synchronization.
     {
       Label Lok;
-      __ testbitdi(CCR0, R0, Rflags, JVM_ACC_SYNCHRONIZED_BIT);
-      __ btrue(CCR0,Lok);
+      __ testbitdi(CR0, R0, Rflags, JVM_ACC_SYNCHRONIZED_BIT);
+      __ btrue(CR0,Lok);
       __ stop("method doesn't need synchronization");
       __ bind(Lok);
     }
@@ -832,8 +842,8 @@ void TemplateInterpreterGenerator::lock_method(Register Rflags, Register Rscratc
     Label Lstatic;
     Label Ldone;
 
-    __ testbitdi(CCR0, R0, Rflags, JVM_ACC_STATIC_BIT);
-    __ btrue(CCR0, Lstatic);
+    __ testbitdi(CR0, R0, Rflags, JVM_ACC_STATIC_BIT);
+    __ btrue(CR0, Lstatic);
 
     // Non-static case: load receiver obj from stack and we're done.
     __ ld(Robj_to_lock, R18_locals);
@@ -940,8 +950,8 @@ void TemplateInterpreterGenerator::generate_fixed_frame(bool native_call, Regist
     // environment and one for a possible native mirror.
     Label skip_native_calculate_max_stack;
     __ addi(Rtop_frame_size, Rsize_of_parameters, 2);
-    __ cmpwi(CCR0, Rtop_frame_size, Argument::n_int_register_parameters_c);
-    __ bge(CCR0, skip_native_calculate_max_stack);
+    __ cmpwi(CR0, Rtop_frame_size, Argument::n_int_register_parameters_c);
+    __ bge(CR0, skip_native_calculate_max_stack);
     __ li(Rtop_frame_size, Argument::n_int_register_parameters_c);
     __ bind(skip_native_calculate_max_stack);
     __ sldi(Rsize_of_parameters, Rsize_of_parameters, Interpreter::logStackElementSize);
@@ -989,8 +999,8 @@ void TemplateInterpreterGenerator::generate_fixed_frame(bool native_call, Regist
   if (ProfileInterpreter) {
     Label zero_continue;
     __ ld(R28_mdx, method_(method_data));
-    __ cmpdi(CCR0, R28_mdx, 0);
-    __ beq(CCR0, zero_continue);
+    __ cmpdi(CR0, R28_mdx, 0);
+    __ beq(CR0, zero_continue);
     __ addi(R28_mdx, R28_mdx, in_bytes(MethodData::data_offset()));
     __ bind(zero_continue);
   }
@@ -1078,6 +1088,7 @@ address TemplateInterpreterGenerator::generate_math_entry(AbstractInterpreter::M
     case Interpreter::java_lang_math_sin  : runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dsin);   break;
     case Interpreter::java_lang_math_cos  : runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dcos);   break;
     case Interpreter::java_lang_math_tan  : runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dtan);   break;
+    case Interpreter::java_lang_math_tanh : /* run interpreted */ break;
     case Interpreter::java_lang_math_abs  : /* run interpreted */ break;
     case Interpreter::java_lang_math_sqrt : runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dsqrt);  break;
     case Interpreter::java_lang_math_log  : runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dlog);   break;
@@ -1143,6 +1154,44 @@ address TemplateInterpreterGenerator::generate_math_entry(AbstractInterpreter::M
   return entry;
 }
 
+address TemplateInterpreterGenerator::generate_Float_floatToFloat16_entry() {
+  if (!VM_Version::supports_float16()) return nullptr;
+
+  address entry = __ pc();
+
+  __ lfs(F1, Interpreter::stackElementSize, R15_esp);
+  __ f2hf(R3_RET, F1, F0);
+
+  // Restore caller sp for c2i case (from compiled) and for resized sender frame (from interpreted).
+  __ resize_frame_absolute(R21_sender_SP, R11_scratch1, R0);
+  __ blr();
+
+  __ flush();
+
+  return entry;
+}
+
+address TemplateInterpreterGenerator::generate_Float_float16ToFloat_entry() {
+  if (!VM_Version::supports_float16()) return nullptr;
+
+  address entry = __ pc();
+
+  // Note: Could also use:
+  //__ li(R3, Interpreter::stackElementSize);
+  //__ lfiwax(F1_RET, R15_esp, R3); // short stored as 32 bit integer
+  //__ xscvhpdp(F1_RET->to_vsr(), F1_RET->to_vsr());
+  __ lwa(R3, Interpreter::stackElementSize, R15_esp);
+  __ hf2f(F1_RET, R3);
+
+  // Restore caller sp for c2i case (from compiled) and for resized sender frame (from interpreted).
+  __ resize_frame_absolute(R21_sender_SP, R11_scratch1, R0);
+  __ blr();
+
+  __ flush();
+
+  return entry;
+}
+
 void TemplateInterpreterGenerator::bang_stack_shadow_pages(bool native_call) {
   // Quick & dirty stack overflow checking: bang the stack & handle trap.
   // Note that we do the banging after the frame is setup, since the exception
@@ -1196,7 +1245,7 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
   const Register signature_handler_fd = R11_scratch1;
   const Register pending_exception    = R0;
   const Register result_handler_addr  = R31;
-  const Register native_method_fd     = R11_scratch1;
+  const Register native_method_fd     = R12_scratch2; // preferred in MacroAssembler::branch_to
   const Register access_flags         = R22_tmp2;
   const Register active_handles       = R11_scratch1; // R26_monitor saved to state.
   const Register sync_state           = R12_scratch2;
@@ -1210,10 +1259,6 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
   Label exception_return_sync_check;
   Label stack_overflow_return;
 
-  // Generate new interpreter state and jump to stack_overflow_return in case of
-  // a stack overflow.
-  //generate_compute_interpreter_state(stack_overflow_return);
-
   Register size_of_parameters = R22_tmp2;
 
   generate_fixed_frame(true, size_of_parameters, noreg /* unused */);
@@ -1252,11 +1297,11 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
 
   // access_flags = method->access_flags();
   // Load access flags.
-  assert(access_flags->is_nonvolatile(),
-         "access_flags must be in a non-volatile register");
+  assert(__ nonvolatile_accross_vthread_preemtion(access_flags),
+         "access_flags not preserved");
   // Type check.
-  assert(4 == sizeof(AccessFlags), "unexpected field size");
-  __ lwz(access_flags, method_(access_flags));
+  assert(2 == sizeof(AccessFlags), "unexpected field size");
+  __ lhz(access_flags, method_(access_flags));
 
   // We don't want to reload R19_method and access_flags after calls
   // to some helper functions.
@@ -1285,8 +1330,8 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
   __ ld(signature_handler_fd, method_(signature_handler));
   Label call_signature_handler;
 
-  __ cmpdi(CCR0, signature_handler_fd, 0);
-  __ bne(CCR0, call_signature_handler);
+  __ cmpdi(CR0, signature_handler_fd, 0);
+  __ bne(CR0, call_signature_handler);
 
   // Method has never been called. Either generate a specialized
   // handler or point to the slow one.
@@ -1297,8 +1342,8 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
   // Check for an exception while looking up the target method. If we
   // incurred one, bail.
   __ ld(pending_exception, thread_(pending_exception));
-  __ cmpdi(CCR0, pending_exception, 0);
-  __ bne(CCR0, exception_return_sync_check); // Has pending exception.
+  __ cmpdi(CR0, pending_exception, 0);
+  __ bne(CR0, exception_return_sync_check); // Has pending exception.
 
   // Reload signature handler, it may have been created/assigned in the meanwhile.
   __ ld(signature_handler_fd, method_(signature_handler));
@@ -1314,8 +1359,12 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
   // convenient and the slow signature handler can use this same frame
   // anchor.
 
+  bool support_vthread_preemption = Continuations::enabled() && LockingMode != LM_LEGACY;
+
   // We have a TOP_IJAVA_FRAME here, which belongs to us.
-  __ set_top_ijava_frame_at_SP_as_last_Java_frame(R1_SP, R12_scratch2/*tmp*/);
+  Label last_java_pc;
+  Label *resume_pc = support_vthread_preemption ? &last_java_pc : nullptr;
+  __ set_top_ijava_frame_at_SP_as_last_Java_frame(R1_SP, R3_ARG1/*tmp*/, resume_pc);
 
   // Now the interpreter frame (and its call chain) have been
   // invalidated and flushed. We are now protected against eager
@@ -1334,16 +1383,11 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
 
   __ call_stub(signature_handler_fd);
 
-  // Remove the register parameter varargs slots we allocated in
-  // compute_interpreter_state. SP+16 ends up pointing to the ABI
-  // outgoing argument area.
-  //
-  // Not needed on PPC64.
-  //__ add(SP, SP, Argument::n_int_register_parameters_c*BytesPerWord);
-
-  assert(result_handler_addr->is_nonvolatile(), "result_handler_addr must be in a non-volatile register");
+  assert(__ nonvolatile_accross_vthread_preemtion(result_handler_addr),
+         "result_handler_addr not preserved");
   // Save across call to native method.
   __ mr(result_handler_addr, R3_RET);
+  __ ld(R11_scratch1, _abi0(callers_sp), R1_SP); // load FP
 
   __ isync(); // Acquire signature handler before trying to fetch the native entry point and klass mirror.
 
@@ -1354,15 +1398,14 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
     // Access_flags is non-volatile and still, no need to restore it.
 
     // Restore access flags.
-    __ testbitdi(CCR0, R0, access_flags, JVM_ACC_STATIC_BIT);
-    __ bfalse(CCR0, method_is_not_static);
+    __ testbitdi(CR0, R0, access_flags, JVM_ACC_STATIC_BIT);
+    __ bfalse(CR0, method_is_not_static);
 
-    __ ld(R11_scratch1, _abi0(callers_sp), R1_SP);
-    // Load mirror from interpreter frame.
-    __ ld(R12_scratch2, _ijava_state_neg(mirror), R11_scratch1);
+    // Load mirror from interpreter frame (FP in R11_scratch1)
+    __ ld(R21_tmp1, _ijava_state_neg(mirror), R11_scratch1);
     // R4_ARG2 = &state->_oop_temp;
     __ addi(R4_ARG2, R11_scratch1, _ijava_state_neg(oop_tmp));
-    __ std(R12_scratch2/*mirror*/, _ijava_state_neg(oop_tmp), R11_scratch1);
+    __ std(R21_tmp1/*mirror*/, _ijava_state_neg(oop_tmp), R11_scratch1);
     BIND(method_is_not_static);
   }
 
@@ -1396,7 +1439,18 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
   // Call the native method. Argument registers must not have been
   // overwritten since "__ call_stub(signature_handler);" (except for
   // ARG1 and ARG2 for static methods).
+
+  if (support_vthread_preemption) {
+    // result_handler_addr is a nonvolatile register. Its value will be preserved across
+    // the native call but only if the call isn't preempted. To preserve its value even
+    // in the case of preemption we save it in the lresult slot. It is restored at
+    // resume_pc if, and only if the call was preempted. This works because only
+    // j.l.Object::wait calls are preempted which don't return a result.
+    __ std(result_handler_addr, _ijava_state_neg(lresult), R11_scratch1);
+  }
+  __ push_cont_fastpath();
   __ call_c(native_method_fd);
+  __ pop_cont_fastpath();
 
   __ li(R0, 0);
   __ ld(R11_scratch1, 0, R1_SP);
@@ -1454,8 +1508,8 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
   // Not suspended.
   // TODO PPC port assert(4 == Thread::sz_suspend_flags(), "unexpected field size");
   __ lwz(suspend_flags, thread_(suspend_flags));
-  __ cmpwi(CCR1, suspend_flags, 0);
-  __ beq(CCR1, sync_check_done);
+  __ cmpwi(CR1, suspend_flags, 0);
+  __ beq(CR1, sync_check_done);
 
   __ bind(do_safepoint);
   __ isync();
@@ -1494,6 +1548,35 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
   __ lwsync(); // Acquire safepoint and suspend state, release thread state.
   __ stw(R0/*thread_state*/, thread_(thread_state));
 
+  if (support_vthread_preemption) {
+    // Check preemption for Object.wait()
+    Label not_preempted;
+    __ ld(R0, in_bytes(JavaThread::preempt_alternate_return_offset()), R16_thread);
+    __ cmpdi(CR0, R0, 0);
+    __ beq(CR0, not_preempted);
+    __ mtlr(R0);
+    __ li(R0, 0);
+    __ std(R0, in_bytes(JavaThread::preempt_alternate_return_offset()), R16_thread);
+    __ blr();
+
+    // Execution will be resumed here when the vthread becomes runnable again.
+    __ bind(*resume_pc);
+    __ restore_after_resume(R11_scratch1 /* fp */);
+    // We saved the result handler before the call
+    __ ld(result_handler_addr, _ijava_state_neg(lresult), R11_scratch1);
+#ifdef ASSERT
+    // Clobber result slots. Only native methods returning void can be preemted currently.
+    __ load_const(R3_RET, UCONST64(0xbad01001));
+    __ std(R3_RET, _ijava_state_neg(lresult), R11_scratch1);
+    __ std(R3_RET, _ijava_state_neg(fresult), R11_scratch1);
+    // reset_last_Java_frame() below asserts that a last java sp is set
+    __ asm_assert_mem8_is_zero(in_bytes(JavaThread::last_Java_sp_offset()),
+        R16_thread, FILE_AND_LINE ": Last java sp should not be set when resuming");
+    __ std(R3_RET, in_bytes(JavaThread::last_Java_sp_offset()), R16_thread);
+#endif
+    __ bind(not_preempted);
+  }
+
   if (CheckJNICalls) {
     // clear_pending_jni_exception_check
     __ load_const_optimized(R0, 0L);
@@ -1528,8 +1611,8 @@ address TemplateInterpreterGenerator::generate_native_entry(bool synchronized) {
 
   Label exception_return_sync_check_already_unlocked;
   __ ld(R0/*pending_exception*/, thread_(pending_exception));
-  __ cmpdi(CCR0, R0/*pending_exception*/, 0);
-  __ bne(CCR0, exception_return_sync_check_already_unlocked);
+  __ cmpdi(CR0, R0/*pending_exception*/, 0);
+  __ bne(CR0, exception_return_sync_check_already_unlocked);
 
   //-----------------------------------------------------------------------------
   // No exception pending.
@@ -1623,7 +1706,7 @@ address TemplateInterpreterGenerator::generate_normal_entry(bool synchronized) {
   __ subf(Rnum, Rsize_of_parameters, Rsize_of_locals);
   __ subf(Rslot_addr, Rsize_of_parameters, R18_locals);
   __ srdi_(Rnum, Rnum, Interpreter::logStackElementSize);
-  __ beq(CCR0, Lno_locals);
+  __ beq(CR0, Lno_locals);
   __ li(R0, 0);
   __ mtctr(Rnum);
 
@@ -1685,7 +1768,7 @@ address TemplateInterpreterGenerator::generate_normal_entry(bool synchronized) {
 #ifdef ASSERT
   else {
     Label Lok;
-    __ lwz(R0, in_bytes(Method::access_flags_offset()), R19_method);
+    __ lhz(R0, in_bytes(Method::access_flags_offset()), R19_method);
     __ andi_(R0, R0, JVM_ACC_SYNCHRONIZED);
     __ asm_assert_eq("method needs synchronization");
     __ bind(Lok);
@@ -1913,14 +1996,24 @@ address TemplateInterpreterGenerator::generate_CRC32C_updateBytes_entry(Abstract
   return start;
 }
 
+address TemplateInterpreterGenerator::generate_currentThread() {
+  address entry_point = __ pc();
+
+  __ ld(R3_RET, JavaThread::vthread_offset(), R16_thread);
+  __ resolve_oop_handle(R3_RET, R11_scratch1, R12_scratch2, MacroAssembler::PRESERVATION_FRAME_LR);
+
+  // restore caller sp for c2i case (from compiled) and for resized sender frame (from interpreted).
+  __ resize_frame_absolute(R21_sender_SP, R11_scratch1, R0);
+  __ blr();
+
+  return entry_point;
+}
+
 // Not supported
-address TemplateInterpreterGenerator::generate_currentThread() { return nullptr; }
 address TemplateInterpreterGenerator::generate_Float_intBitsToFloat_entry() { return nullptr; }
 address TemplateInterpreterGenerator::generate_Float_floatToRawIntBits_entry() { return nullptr; }
 address TemplateInterpreterGenerator::generate_Double_longBitsToDouble_entry() { return nullptr; }
 address TemplateInterpreterGenerator::generate_Double_doubleToRawLongBits_entry() { return nullptr; }
-address TemplateInterpreterGenerator::generate_Float_float16ToFloat_entry() { return nullptr; }
-address TemplateInterpreterGenerator::generate_Float_floatToFloat16_entry() { return nullptr; }
 
 // =============================================================================
 // Exceptions
@@ -1999,8 +2092,8 @@ void TemplateInterpreterGenerator::generate_throw_exception() {
     __ ld(return_pc, 0, R1_SP);
     __ ld(return_pc, _abi0(lr), return_pc);
     __ call_VM_leaf(CAST_FROM_FN_PTR(address, InterpreterRuntime::interpreter_contains), return_pc);
-    __ cmpdi(CCR0, R3_RET, 0);
-    __ bne(CCR0, Lcaller_not_deoptimized);
+    __ cmpdi(CR0, R3_RET, 0);
+    __ bne(CR0, Lcaller_not_deoptimized);
 
     // The deoptimized case.
     // In this case, we can't call dispatch_next() after the frame is
@@ -2046,16 +2139,16 @@ void TemplateInterpreterGenerator::generate_throw_exception() {
     Label L_done;
 
     __ lbz(R11_scratch1, 0, R14_bcp);
-    __ cmpwi(CCR0, R11_scratch1, Bytecodes::_invokestatic);
-    __ bne(CCR0, L_done);
+    __ cmpwi(CR0, R11_scratch1, Bytecodes::_invokestatic);
+    __ bne(CR0, L_done);
 
     // The member name argument must be restored if _invokestatic is re-executed after a PopFrame call.
     // Detect such a case in the InterpreterRuntime function and return the member name argument, or null.
     __ ld(R4_ARG2, 0, R18_locals);
     __ call_VM(R4_ARG2, CAST_FROM_FN_PTR(address, InterpreterRuntime::member_name_arg_or_null), R4_ARG2, R19_method, R14_bcp);
 
-    __ cmpdi(CCR0, R4_ARG2, 0);
-    __ beq(CCR0, L_done);
+    __ cmpdi(CR0, R4_ARG2, 0);
+    __ beq(CR0, L_done);
     __ std(R4_ARG2, wordSize, R15_esp);
     __ bind(L_done);
 #endif // INCLUDE_JVMTI
@@ -2235,13 +2328,13 @@ address TemplateInterpreterGenerator::generate_trace_code(TosState state) {
   // Support short-cut for TraceBytecodesAt.
   // Don't call into the VM if we don't want to trace to speed up things.
   Label Lskip_vm_call;
-  if (TraceBytecodesAt > 0 && TraceBytecodesAt < max_intx) {
+  if (TraceBytecodesAt > 0) {
     int offs1 = __ load_const_optimized(R11_scratch1, (address) &TraceBytecodesAt, R0, true);
     int offs2 = __ load_const_optimized(R12_scratch2, (address) &BytecodeCounter::_counter_value, R0, true);
     __ ld(R11_scratch1, offs1, R11_scratch1);
-    __ lwa(R12_scratch2, offs2, R12_scratch2);
-    __ cmpd(CCR0, R12_scratch2, R11_scratch1);
-    __ blt(CCR0, Lskip_vm_call);
+    __ ld(R12_scratch2, offs2, R12_scratch2);
+    __ cmpd(CR0, R12_scratch2, R11_scratch1);
+    __ blt(CR0, Lskip_vm_call);
   }
 
   __ push(state);
@@ -2253,7 +2346,7 @@ address TemplateInterpreterGenerator::generate_trace_code(TosState state) {
   __ mtlr(R31);
   __ pop(state);
 
-  if (TraceBytecodesAt > 0 && TraceBytecodesAt < max_intx) {
+  if (TraceBytecodesAt > 0) {
     __ bind(Lskip_vm_call);
   }
   __ blr();
@@ -2263,9 +2356,9 @@ address TemplateInterpreterGenerator::generate_trace_code(TosState state) {
 
 void TemplateInterpreterGenerator::count_bytecode() {
   int offs = __ load_const_optimized(R11_scratch1, (address) &BytecodeCounter::_counter_value, R12_scratch2, true);
-  __ lwz(R12_scratch2, offs, R11_scratch1);
+  __ ld(R12_scratch2, offs, R11_scratch1);
   __ addi(R12_scratch2, R12_scratch2, 1);
-  __ stw(R12_scratch2, offs, R11_scratch1);
+  __ std(R12_scratch2, offs, R11_scratch1);
 }
 
 void TemplateInterpreterGenerator::histogram_bytecode(Template* t) {
@@ -2314,9 +2407,9 @@ void TemplateInterpreterGenerator::stop_interpreter_at() {
   int offs1 = __ load_const_optimized(R11_scratch1, (address) &StopInterpreterAt, R0, true);
   int offs2 = __ load_const_optimized(R12_scratch2, (address) &BytecodeCounter::_counter_value, R0, true);
   __ ld(R11_scratch1, offs1, R11_scratch1);
-  __ lwa(R12_scratch2, offs2, R12_scratch2);
-  __ cmpd(CCR0, R12_scratch2, R11_scratch1);
-  __ bne(CCR0, L);
+  __ ld(R12_scratch2, offs2, R12_scratch2);
+  __ cmpd(CR0, R12_scratch2, R11_scratch1);
+  __ bne(CR0, L);
   __ illtrap();
   __ bind(L);
 }
