@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2025, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2012, 2022 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -23,7 +23,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "asm/assembler.hpp"
 #include "code/vmreg.hpp"
 
@@ -33,7 +32,7 @@ void VMRegImpl::set_regName() {
   for (i = 0; i < ConcreteRegisterImpl::max_gpr; ) {
     regName[i++] = reg->name();
     regName[i++] = reg->name();
-    if (reg->encoding() < RegisterImpl::number_of_registers-1)
+    if (reg->encoding() < Register::number_of_registers-1)
       reg = reg->successor();
   }
 
@@ -41,7 +40,7 @@ void VMRegImpl::set_regName() {
   for ( ; i < ConcreteRegisterImpl::max_fpr; ) {
     regName[i++] = freg->name();
     regName[i++] = freg->name();
-    if (reg->encoding() < FloatRegisterImpl::number_of_registers-1)
+    if (reg->encoding() < FloatRegister::number_of_registers-1)
       freg = freg->successor();
   }
 
