@@ -81,8 +81,6 @@ define_pd_global(intx, InlineSmallCode,          1000);
                    range,                                                        \
                    constraint)                                                   \
                                                                                  \
-  product(bool, NearCpool, true,                                                 \
-         "constant pool is close to instructions")                               \
   product(bool, UseBlockZeroing, false,                                          \
           "Use Zicboz for block zeroing")                                        \
   product(intx, BlockZeroingLowLimit, 256,                                       \
@@ -100,28 +98,31 @@ define_pd_global(intx, InlineSmallCode,          1000);
   product(bool, UseRVA20U64, true, "Use RVA20U64 profile")                       \
   product(bool, UseRVA22U64, false, EXPERIMENTAL, "Use RVA22U64 profile")        \
   product(bool, UseRVA23U64, false, EXPERIMENTAL, "Use RVA23U64 profile")        \
-  product(bool, UseRVC, false, "Use RVC instructions")                           \
-  product(bool, UseRVV, false, "Use RVV instructions")                           \
-  product(bool, UseZba, false, "Use Zba instructions")                           \
-  product(bool, UseZbb, false, "Use Zbb instructions")                           \
-  product(bool, UseZbs, false, "Use Zbs instructions")                           \
-  product(bool, UseZfh, false, "Use Zfh instructions")                           \
+  product(bool, UseRVC, false, DIAGNOSTIC, "Use RVC instructions")               \
+  product(bool, UseRVV, false, DIAGNOSTIC, "Use RVV instructions")               \
+  product(bool, UseZba, false, DIAGNOSTIC, "Use Zba instructions")               \
+  product(bool, UseZbb, false, DIAGNOSTIC, "Use Zbb instructions")               \
+  product(bool, UseZbkb, false, EXPERIMENTAL, "Use Zbkb instructions")           \
+  product(bool, UseZbs, false, DIAGNOSTIC, "Use Zbs instructions")               \
+  product(bool, UseZfa, false, EXPERIMENTAL, "Use Zfa instructions")             \
+  product(bool, UseZfh, false, DIAGNOSTIC, "Use Zfh instructions")               \
+  product(bool, UseZfhmin, false, DIAGNOSTIC, "Use Zfhmin instructions")         \
   product(bool, UseZacas, false, EXPERIMENTAL, "Use Zacas instructions")         \
   product(bool, UseZcb, false, EXPERIMENTAL, "Use Zcb instructions")             \
   product(bool, UseZic64b, false, EXPERIMENTAL, "Use Zic64b instructions")       \
   product(bool, UseZicbom, false, EXPERIMENTAL, "Use Zicbom instructions")       \
   product(bool, UseZicbop, false, EXPERIMENTAL, "Use Zicbop instructions")       \
   product(bool, UseZicboz, false, EXPERIMENTAL, "Use Zicboz instructions")       \
-  product(bool, UseZtso, false, EXPERIMENTAL, "Assume Ztso memory model")        \
+  product(bool, UseZicond, false, DIAGNOSTIC, "Use Zicond instructions")         \
   product(bool, UseZihintpause, false, EXPERIMENTAL,                             \
           "Use Zihintpause instructions")                                        \
+  product(bool, UseZtso, false, EXPERIMENTAL, "Assume Ztso memory model")        \
   product(bool, UseZvbb, false, EXPERIMENTAL, "Use Zvbb instructions")           \
-  product(bool, UseZvfh, false, EXPERIMENTAL, "Use Zvfh instructions")           \
+  product(bool, UseZvbc, false, EXPERIMENTAL, "Use Zvbc instructions")           \
+  product(bool, UseZvfh, false, DIAGNOSTIC, "Use Zvfh instructions")             \
   product(bool, UseZvkn, false, EXPERIMENTAL,                                    \
           "Use Zvkn group extension, Zvkned, Zvknhb, Zvkb, Zvkt")                \
-  product(bool, UseRVVForBigIntegerShiftIntrinsics, true,                        \
-          "Use RVV instructions for left/right shift of BigInteger")             \
-  product(bool, UseTrampolines, false, EXPERIMENTAL,                             \
-          "Far calls uses jal to trampoline.")
+  product(bool, UseCtxFencei, false, EXPERIMENTAL,                               \
+          "Use PR_RISCV_CTX_SW_FENCEI_ON to avoid explicit icache flush")
 
 #endif // CPU_RISCV_GLOBALS_RISCV_HPP
