@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2016, 2021, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2016, 2017 SAP SE. All rights reserved.
+ * Copyright (c) 2016, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2024 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,7 +23,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "c1/c1_Compilation.hpp"
 #include "c1/c1_FrameMap.hpp"
 #include "c1/c1_Instruction.hpp"
@@ -103,11 +102,11 @@ LIR_Opr LIRGenerator::rlock_byte(BasicType type) {
 
 // z/Architecture cannot inline all constants.
 bool LIRGenerator::can_store_as_constant(Value v, BasicType type) const {
-  if (v->type()->as_IntConstant() != NULL) {
+  if (v->type()->as_IntConstant() != nullptr) {
     return Immediate::is_simm16(v->type()->as_IntConstant()->value());
-  } else if (v->type()->as_LongConstant() != NULL) {
+  } else if (v->type()->as_LongConstant() != nullptr) {
     return Immediate::is_simm16(v->type()->as_LongConstant()->value());
-  } else if (v->type()->as_ObjectConstant() != NULL) {
+  } else if (v->type()->as_ObjectConstant() != nullptr) {
     return v->type()->as_ObjectConstant()->value()->is_null_object();
   } else {
     return false;
@@ -115,9 +114,9 @@ bool LIRGenerator::can_store_as_constant(Value v, BasicType type) const {
 }
 
 bool LIRGenerator::can_inline_as_constant(Value i, int bits) const {
-  if (i->type()->as_IntConstant() != NULL) {
+  if (i->type()->as_IntConstant() != nullptr) {
     return Assembler::is_simm(i->type()->as_IntConstant()->value(), bits);
-  } else if (i->type()->as_LongConstant() != NULL) {
+  } else if (i->type()->as_LongConstant() != nullptr) {
     return Assembler::is_simm(i->type()->as_LongConstant()->value(), bits);
   } else {
     return can_store_as_constant(i, as_BasicType(i->type()));
@@ -157,8 +156,10 @@ LIR_Address* LIRGenerator::generate_address(LIR_Opr base, LIR_Opr index,
     return new LIR_Address(base, index, type);
   } else {
     if (shift > 0) {
-      LIR_Opr tmp = new_pointer_register();
-      __ shift_left(index, shift, tmp);
+      // Use long register to avoid overflow when shifting large index values left.
+      LIR_Opr tmp = new_register(T_LONG);
+      __ convert(Bytecodes::_i2l, index, tmp);
+      __ shift_left(tmp, shift, tmp);
       index = tmp;
     }
     return new LIR_Address(base, index, disp, type);
@@ -191,12 +192,12 @@ LIR_Address* LIRGenerator::emit_array_address(LIR_Opr array_opr, LIR_Opr index_o
   return addr;
 }
 
-LIR_Opr LIRGenerator::load_immediate(int x, BasicType type) {
+LIR_Opr LIRGenerator::load_immediate(jlong x, BasicType type) {
   LIR_Opr r = LIR_OprFact::illegalOpr;
   if (type == T_LONG) {
     r = LIR_OprFact::longConst(x);
   } else if (type == T_INT) {
-    r = LIR_OprFact::intConst(x);
+    r = LIR_OprFact::intConst(checked_cast<jint>(x));
   } else {
     ShouldNotReachHere();
   }
@@ -225,19 +226,25 @@ void LIRGenerator::cmp_reg_mem(LIR_Condition condition, LIR_Opr reg, LIR_Opr bas
 }
 
 bool LIRGenerator::strength_reduce_multiply(LIR_Opr left, jint c, LIR_Opr result, LIR_Opr tmp) {
+  juint u_value = (juint)c;
   if (tmp->is_valid()) {
-    if (is_power_of_2(c + 1)) {
+    if (is_power_of_2(u_value + 1)) {
       __ move(left, tmp);
-      __ shift_left(left, log2i_exact(c + 1), left);
+      __ shift_left(left, log2i_exact(u_value + 1), left);
       __ sub(left, tmp, result);
       return true;
-    } else if (is_power_of_2(c - 1)) {
+    } else if (is_power_of_2(u_value - 1)) {
       __ move(left, tmp);
-      __ shift_left(left, log2i_exact(c - 1), left);
+      __ shift_left(left, log2i_exact(u_value - 1), left);
       __ add(left, tmp, result);
       return true;
     }
   }
+
+  if (c == -1) {
+    __ negate(left, result);
+    return true;
+  }
   return false;
 }
 
@@ -267,7 +274,7 @@ void LIRGenerator::do_MonitorEnter(MonitorEnter* x) {
   // "lock" stores the address of the monitor stack slot, so this is not an oop.
   LIR_Opr lock = new_register(T_INT);
 
-  CodeEmitInfo* info_for_exception = NULL;
+  CodeEmitInfo* info_for_exception = nullptr;
   if (x->needs_null_check()) {
     info_for_exception = state_for (x);
   }
@@ -326,7 +333,7 @@ void LIRGenerator::do_ArithmeticOp_FPU(ArithmeticOp* x) {
     default:
       ShouldNotReachHere();
     }
-    LIR_Opr result = call_runtime(x->x(), x->y(), entry, x->type(), NULL);
+    LIR_Opr result = call_runtime(x->x(), x->y(), entry, x->type(), nullptr);
     set_result(x, result);
   } else {
     LIR_Opr reg = rlock(x);
@@ -339,7 +346,7 @@ void LIRGenerator::do_ArithmeticOp_FPU(ArithmeticOp* x) {
 // for _ladd, _lmul, _lsub, _ldiv, _lrem
 void LIRGenerator::do_ArithmeticOp_Long(ArithmeticOp* x) {
   if (x->op() == Bytecodes::_ldiv || x->op() == Bytecodes::_lrem) {
-    // Use shifts if divisior is a power of 2 otherwise use DSGR instruction.
+    // Use shifts if divisor is a power of 2 otherwise use DSGR instruction.
     // Instruction: DSGR R1, R2
     // input : R1+1: dividend   (R1, R1+1 designate a register pair, R1 must be even)
     //         R2:   divisor
@@ -387,7 +394,7 @@ void LIRGenerator::do_ArithmeticOp_Long(ArithmeticOp* x) {
       __ cmp(lir_cond_equal, right.result(), LIR_OprFact::longConst(0));
       __ branch(lir_cond_equal, new DivByZeroStub(info));
       // Idiv/irem cannot trap (passing info would generate an assertion).
-      info = NULL;
+      info = nullptr;
     }
 
     if (x->op() == Bytecodes::_lrem) {
@@ -408,14 +415,14 @@ void LIRGenerator::do_ArithmeticOp_Long(ArithmeticOp* x) {
     left.load_item();
     right.load_nonconstant(32);
     rlock_result(x);
-    arithmetic_op_long(x->op(), x->operand(), left.result(), right.result(), NULL);
+    arithmetic_op_long(x->op(), x->operand(), left.result(), right.result(), nullptr);
   }
 }
 
 // for: _iadd, _imul, _isub, _idiv, _irem
 void LIRGenerator::do_ArithmeticOp_Int(ArithmeticOp* x) {
   if (x->op() == Bytecodes::_idiv || x->op() == Bytecodes::_irem) {
-    // Use shifts if divisior is a power of 2 otherwise use DSGFR instruction.
+    // Use shifts if divisor is a power of 2 otherwise use DSGFR instruction.
     // Instruction: DSGFR R1, R2
     // input : R1+1: dividend   (R1, R1+1 designate a register pair, R1 must be even)
     //         R2:   divisor
@@ -463,7 +470,7 @@ void LIRGenerator::do_ArithmeticOp_Int(ArithmeticOp* x) {
       __ cmp(lir_cond_equal, right.result(), LIR_OprFact::intConst(0));
       __ branch(lir_cond_equal, new DivByZeroStub(info));
       // Idiv/irem cannot trap (passing info would generate an assertion).
-      info = NULL;
+      info = nullptr;
     }
 
     if (x->op() == Bytecodes::_irem) {
@@ -494,8 +501,8 @@ void LIRGenerator::do_ArithmeticOp_Int(ArithmeticOp* x) {
     if (x->op() == Bytecodes::_imul) {
       bool use_tmp = false;
       if (right_arg->is_constant()) {
-        int iconst = right_arg->get_jint_constant();
-        if (is_power_of_2(iconst - 1) || is_power_of_2(iconst + 1)) {
+        juint u_const = (juint)right_arg->get_jint_constant();
+        if (is_power_of_2(u_const - 1) || is_power_of_2(u_const + 1)) {
           use_tmp = true;
         }
       }
@@ -519,7 +526,7 @@ void LIRGenerator::do_ArithmeticOp_Int(ArithmeticOp* x) {
 void LIRGenerator::do_ArithmeticOp(ArithmeticOp* x) {
   // If an operand with use count 1 is the left operand, then it is
   // likely that no move for 2-operand-LIR-form is necessary.
-  if (x->is_commutative() && x->y()->as_Constant() == NULL && x->x()->use_count() > x->y()->use_count()) {
+  if (x->is_commutative() && x->y()->as_Constant() == nullptr && x->x()->use_count() > x->y()->use_count()) {
     x->swap_operands();
   }
 
@@ -558,7 +565,7 @@ void LIRGenerator::do_ShiftOp(ShiftOp* x) {
 void LIRGenerator::do_LogicOp(LogicOp* x) {
   // IF an operand with use count 1 is the left operand, then it is
   // likely that no move for 2-operand-LIR-form is necessary.
-  if (x->is_commutative() && x->y()->as_Constant() == NULL && x->x()->use_count() > x->y()->use_count()) {
+  if (x->is_commutative() && x->y()->as_Constant() == nullptr && x->x()->use_count() > x->y()->use_count()) {
     x->swap_operands();
   }
 
@@ -659,7 +666,7 @@ void LIRGenerator::do_MathIntrinsic(Intrinsic* x) {
     case vmIntrinsics::_dexp: {
       assert(x->number_of_arguments() == 1, "wrong type");
 
-      address runtime_entry = NULL;
+      address runtime_entry = nullptr;
       switch (x->id()) {
         case vmIntrinsics::_dsin:
           runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dsin);
@@ -683,14 +690,14 @@ void LIRGenerator::do_MathIntrinsic(Intrinsic* x) {
           ShouldNotReachHere();
       }
 
-      LIR_Opr result = call_runtime(x->argument_at(0), runtime_entry, x->type(), NULL);
+      LIR_Opr result = call_runtime(x->argument_at(0), runtime_entry, x->type(), nullptr);
       set_result(x, result);
       break;
     }
     case vmIntrinsics::_dpow: {
       assert(x->number_of_arguments() == 2, "wrong type");
       address runtime_entry = CAST_FROM_FN_PTR(address, SharedRuntime::dpow);
-      LIR_Opr result = call_runtime(x->argument_at(0), x->argument_at(1), runtime_entry, x->type(), NULL);
+      LIR_Opr result = call_runtime(x->argument_at(0), x->argument_at(1), runtime_entry, x->type(), nullptr);
       set_result(x, result);
       break;
     }
@@ -708,7 +715,13 @@ void LIRGenerator::do_ArrayCopy(Intrinsic* x) {
   frame_map()->c_calling_convention(&sig);
 
   // Make all state_for calls early since they can emit code.
-  CodeEmitInfo* info = state_for (x, x->state());
+  CodeEmitInfo* info = nullptr;
+  if (x->state_before() != nullptr && x->state_before()->force_reexecute()) {
+    info = state_for(x, x->state_before());
+    info->set_force_reexecute();
+  } else {
+    info = state_for(x, x->state());
+  }
 
   LIRItem src(x->argument_at(0), this);
   LIRItem src_pos(x->argument_at(1), this);
@@ -733,6 +746,9 @@ void LIRGenerator::do_ArrayCopy(Intrinsic* x) {
   int flags;
   ciArrayKlass* expected_type;
   arraycopy_helper(x, &flags, &expected_type);
+  if (x->check_flag(Instruction::OmitChecksFlag)) {
+    flags = 0;
+  }
 
   __ arraycopy(src.result(), src_pos.result(), dst.result(), dst_pos.result(),
                length.result(), tmp, expected_type, flags, info); // does add_safepoint
@@ -767,7 +783,13 @@ void LIRGenerator::do_NewInstance(NewInstance* x) {
 }
 
 void LIRGenerator::do_NewTypeArray(NewTypeArray* x) {
-  CodeEmitInfo* info = state_for (x, x->state());
+  CodeEmitInfo* info = nullptr;
+  if (x->state_before() != nullptr && x->state_before()->force_reexecute()) {
+    info = state_for(x, x->state_before());
+    info->set_force_reexecute();
+  } else {
+    info = state_for(x, x->state());
+  }
 
   LIRItem length(x->length(), this);
   length.load_item();
@@ -784,7 +806,8 @@ void LIRGenerator::do_NewTypeArray(NewTypeArray* x) {
   __ metadata2reg(ciTypeArrayKlass::make(elem_type)->constant_encoding(), klass_reg);
 
   CodeStub* slow_path = new NewTypeArrayStub(klass_reg, len, reg, info);
-  __ allocate_array(reg, len, tmp1, tmp2, tmp3, tmp4, elem_type, klass_reg, slow_path);
+  __ allocate_array(reg, len, tmp1, tmp2, tmp3, tmp4, elem_type,
+                    klass_reg, slow_path, x->zero_array());
 
   LIR_Opr result = rlock_result(x);
   __ move(reg, result);
@@ -795,7 +818,7 @@ void LIRGenerator::do_NewObjectArray(NewObjectArray* x) {
   CodeEmitInfo* info = state_for (x, x->state());
   // In case of patching (i.e., object class is not yet loaded), we need to reexecute the instruction
   // and therefore provide the state before the parameters have been consumed.
-  CodeEmitInfo* patching_info = NULL;
+  CodeEmitInfo* patching_info = nullptr;
   if (!x->klass()->is_loaded() || PatchALot) {
     patching_info = state_for (x, x->state_before());
   }
@@ -826,14 +849,14 @@ void LIRGenerator::do_NewObjectArray(NewObjectArray* x) {
 void LIRGenerator::do_NewMultiArray(NewMultiArray* x) {
   Values* dims = x->dims();
   int i = dims->length();
-  LIRItemList* items = new LIRItemList(i, i, NULL);
+  LIRItemList* items = new LIRItemList(i, i, nullptr);
   while (i-- > 0) {
     LIRItem* size = new LIRItem(dims->at(i), this);
     items->at_put(i, size);
   }
 
   // Evaluate state_for early since it may emit code.
-  CodeEmitInfo* patching_info = NULL;
+  CodeEmitInfo* patching_info = nullptr;
   if (!x->klass()->is_loaded() || PatchALot) {
     patching_info = state_for (x, x->state_before());
 
@@ -867,7 +890,7 @@ void LIRGenerator::do_NewMultiArray(NewMultiArray* x) {
   args->append(rank);
   args->append(varargs);
   LIR_Opr reg = result_register_for (x->type());
-  __ call_runtime(Runtime1::entry_for (Runtime1::new_multi_array_id),
+  __ call_runtime(Runtime1::entry_for (C1StubId::new_multi_array_id),
                   LIR_OprFact::illegalOpr,
                   reg, args, info);
 
@@ -882,7 +905,7 @@ void LIRGenerator::do_BlockBegin(BlockBegin* x) {
 void LIRGenerator::do_CheckCast(CheckCast* x) {
   LIRItem obj(x->obj(), this);
 
-  CodeEmitInfo* patching_info = NULL;
+  CodeEmitInfo* patching_info = nullptr;
   if (!x->klass()->is_loaded() || (PatchALot && !x->is_incompatible_class_change_check() && !x->is_invokespecial_receiver_check())) {
     // Must do this before locking the destination register as an oop register,
     // and before the obj is loaded (the latter is for deoptimization).
@@ -897,15 +920,15 @@ void LIRGenerator::do_CheckCast(CheckCast* x) {
 
   CodeStub* stub;
   if (x->is_incompatible_class_change_check()) {
-    assert(patching_info == NULL, "can't patch this");
-    stub = new SimpleExceptionStub(Runtime1::throw_incompatible_class_change_error_id, LIR_OprFact::illegalOpr, info_for_exception);
+    assert(patching_info == nullptr, "can't patch this");
+    stub = new SimpleExceptionStub(C1StubId::throw_incompatible_class_change_error_id, LIR_OprFact::illegalOpr, info_for_exception);
   } else if (x->is_invokespecial_receiver_check()) {
-    assert(patching_info == NULL, "can't patch this");
+    assert(patching_info == nullptr, "can't patch this");
     stub = new DeoptimizeStub(info_for_exception,
                               Deoptimization::Reason_class_check,
                               Deoptimization::Action_none);
   } else {
-    stub = new SimpleExceptionStub(Runtime1::throw_class_cast_exception_id, obj.result(), info_for_exception);
+    stub = new SimpleExceptionStub(C1StubId::throw_class_cast_exception_id, obj.result(), info_for_exception);
   }
   LIR_Opr reg = rlock_result(x);
   LIR_Opr tmp1 = new_register(objectType);
@@ -920,7 +943,7 @@ void LIRGenerator::do_CheckCast(CheckCast* x) {
 
 void LIRGenerator::do_InstanceOf(InstanceOf* x) {
   LIRItem obj(x->obj(), this);
-  CodeEmitInfo* patching_info = NULL;
+  CodeEmitInfo* patching_info = nullptr;
   if (!x->klass()->is_loaded() || PatchALot) {
     patching_info = state_for (x, x->state_before());
   }
@@ -936,6 +959,11 @@ void LIRGenerator::do_InstanceOf(InstanceOf* x) {
                 x->profiled_method(), x->profiled_bci());
 }
 
+// Intrinsic for Class::isInstance
+address LIRGenerator::isInstance_entry() {
+  return CAST_FROM_FN_PTR(address, Runtime1::is_instance_of);
+}
+
 
 void LIRGenerator::do_If (If* x) {
   assert(x->number_of_sux() == 2, "inconsistency");
