@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2018, 2019 SAP SE. All rights reserved.
+ * Copyright (c) 2019, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2024 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,7 +23,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "asm/macroAssembler.inline.hpp"
 #include "registerSaver_s390.hpp"
 #include "gc/g1/g1CardTable.hpp"
@@ -31,21 +30,58 @@
 #include "gc/g1/g1BarrierSetAssembler.hpp"
 #include "gc/g1/g1BarrierSetRuntime.hpp"
 #include "gc/g1/g1DirtyCardQueue.hpp"
+#include "gc/g1/g1HeapRegion.hpp"
 #include "gc/g1/g1SATBMarkQueueSet.hpp"
 #include "gc/g1/g1ThreadLocalData.hpp"
-#include "gc/g1/heapRegion.hpp"
 #include "interpreter/interp_masm.hpp"
 #include "runtime/jniHandles.hpp"
 #include "runtime/sharedRuntime.hpp"
+#include "utilities/macros.hpp"
 #ifdef COMPILER1
 #include "c1/c1_LIRAssembler.hpp"
 #include "c1/c1_MacroAssembler.hpp"
 #include "gc/g1/c1/g1BarrierSetC1.hpp"
-#endif
+#endif // COMPILER1
+#ifdef COMPILER2
+#include "gc/g1/c2/g1BarrierSetC2.hpp"
+#endif // COMPILER2
 
 #define __ masm->
 
-#define BLOCK_COMMENT(str) if (PrintAssembly) __ block_comment(str)
+#define BLOCK_COMMENT(str) __ block_comment(str)
+
+static void generate_pre_barrier_fast_path(MacroAssembler* masm,
+                                           const Register thread,
+                                           const Register tmp1) {
+  Address in_progress(thread, in_bytes(G1ThreadLocalData::satb_mark_queue_active_offset()));
+  // Is marking active?
+  if (in_bytes(SATBMarkQueue::byte_width_of_active()) == 4) {
+    __ load_and_test_int(tmp1, in_progress);
+  } else {
+    assert(in_bytes(SATBMarkQueue::byte_width_of_active()) == 1, "Assumption");
+    __ load_and_test_byte(tmp1, in_progress);
+  }
+}
+
+static void generate_queue_test_and_insertion(MacroAssembler* masm, ByteSize index_offset, ByteSize buffer_offset, Label& runtime,
+                                              const Register Z_thread, const Register value, const Register temp) {
+  BLOCK_COMMENT("generate_queue_test_and_insertion {");
+
+  assert_different_registers(temp, value);
+  // Can we store a value in the given thread's buffer?
+  // (The index field is typed as size_t.)
+
+  __ load_and_test_long(temp, Address(Z_thread, in_bytes(index_offset))); // temp := *(index address)
+  __ branch_optimized(Assembler::bcondEqual, runtime);                    // jump to runtime if index == 0 (full buffer)
+
+  // The buffer is not full, store value into it.
+  __ add2reg(temp, -wordSize);                                            // temp := next index
+  __ z_stg(temp, in_bytes(index_offset), Z_thread);                       // *(index address) := next index
+
+  __ z_ag(temp, Address(Z_thread, in_bytes(buffer_offset)));              // temp := buffer address + next index
+  __ z_stg(value, 0, temp);                                               // *(buffer address + next index) := value
+  BLOCK_COMMENT("} generate_queue_test_and_insertion");
+}
 
 void G1BarrierSetAssembler::gen_write_ref_array_pre_barrier(MacroAssembler* masm, DecoratorSet decorators,
                                                             Register addr, Register count) {
@@ -58,13 +94,8 @@ void G1BarrierSetAssembler::gen_write_ref_array_pre_barrier(MacroAssembler* masm
     assert_different_registers(addr,  Z_R0_scratch);  // would be destroyed by push_frame()
     assert_different_registers(count, Z_R0_scratch);  // would be destroyed by push_frame()
     Register Rtmp1 = Z_R0_scratch;
-    const int active_offset = in_bytes(G1ThreadLocalData::satb_mark_queue_active_offset());
-    if (in_bytes(SATBMarkQueue::byte_width_of_active()) == 4) {
-      __ load_and_test_int(Rtmp1, Address(Z_thread, active_offset));
-    } else {
-      guarantee(in_bytes(SATBMarkQueue::byte_width_of_active()) == 1, "Assumption");
-      __ load_and_test_byte(Rtmp1, Address(Z_thread, active_offset));
-    }
+
+    generate_pre_barrier_fast_path(masm, Z_thread, Rtmp1);
     __ z_bre(filtered); // Activity indicator is zero, so there is no marking going on currently.
 
     RegisterSaver::save_live_registers(masm, RegisterSaver::arg_registers); // Creates frame.
@@ -99,6 +130,181 @@ void G1BarrierSetAssembler::gen_write_ref_array_post_barrier(MacroAssembler* mas
   }
 }
 
+#if defined(COMPILER2)
+
+#undef __
+#define __ masm->
+
+static void generate_c2_barrier_runtime_call(MacroAssembler* masm, G1BarrierStubC2* stub, const Register pre_val, const address runtime_path) {
+  BLOCK_COMMENT("generate_c2_barrier_runtime_call {");
+  SaveLiveRegisters save_registers(masm, stub);
+  __ call_VM_leaf(runtime_path, pre_val, Z_thread);
+  BLOCK_COMMENT("} generate_c2_barrier_runtime_call");
+}
+
+void G1BarrierSetAssembler::g1_write_barrier_pre_c2(MacroAssembler* masm,
+                                                    Register obj,
+                                                    Register pre_val,
+                                                    Register thread,
+                                                    Register tmp1,
+                                                    G1PreBarrierStubC2* stub) {
+
+  BLOCK_COMMENT("g1_write_barrier_pre_c2 {");
+
+  assert(thread == Z_thread, "must be");
+  assert_different_registers(obj, pre_val, tmp1);
+  assert(pre_val != noreg && tmp1 != noreg, "expecting a register");
+
+  stub->initialize_registers(obj, pre_val, thread, tmp1, noreg);
+
+  generate_pre_barrier_fast_path(masm, thread, tmp1);
+  __ branch_optimized(Assembler::bcondNotEqual, *stub->entry()); // Activity indicator is zero, so there is no marking going on currently.
+
+  __ bind(*stub->continuation());
+
+  BLOCK_COMMENT("} g1_write_barrier_pre_c2");
+}
+
+void G1BarrierSetAssembler::generate_c2_pre_barrier_stub(MacroAssembler* masm,
+                                                         G1PreBarrierStubC2* stub) const {
+
+  BLOCK_COMMENT("generate_c2_pre_barrier_stub {");
+
+  Assembler::InlineSkippedInstructionsCounter skip_counter(masm);
+
+  Label runtime;
+  Register obj     = stub->obj();
+  Register pre_val = stub->pre_val();
+  Register thread  = stub->thread();
+  Register tmp1    = stub->tmp1();
+
+  __ bind(*stub->entry());
+
+  BLOCK_COMMENT("generate_pre_val_not_null_test {");
+  if (obj != noreg) {
+    __ load_heap_oop(pre_val, Address(obj), noreg, noreg, AS_RAW);
+  }
+  __ z_ltgr(pre_val, pre_val);
+  __ branch_optimized(Assembler::bcondEqual, *stub->continuation());
+  BLOCK_COMMENT("} generate_pre_val_not_null_test");
+
+  generate_queue_test_and_insertion(masm,
+                                    G1ThreadLocalData::satb_mark_queue_index_offset(),
+                                    G1ThreadLocalData::satb_mark_queue_buffer_offset(),
+                                    runtime,
+                                    Z_thread, pre_val, tmp1);
+
+  __ branch_optimized(Assembler::bcondAlways, *stub->continuation());
+
+  __ bind(runtime);
+
+  generate_c2_barrier_runtime_call(masm, stub, pre_val, CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_pre_entry));
+
+  __ branch_optimized(Assembler::bcondAlways, *stub->continuation());
+
+  BLOCK_COMMENT("} generate_c2_pre_barrier_stub");
+}
+
+void G1BarrierSetAssembler::g1_write_barrier_post_c2(MacroAssembler* masm,
+                                                     Register store_addr,
+                                                     Register new_val,
+                                                     Register thread,
+                                                     Register tmp1,
+                                                     Register tmp2,
+                                                     G1PostBarrierStubC2* stub) {
+  BLOCK_COMMENT("g1_write_barrier_post_c2 {");
+
+  assert(thread == Z_thread, "must be");
+  assert_different_registers(store_addr, new_val, thread, tmp1, tmp2, Z_R1_scratch);
+
+  assert(store_addr != noreg && new_val != noreg && tmp1 != noreg && tmp2 != noreg, "expecting a register");
+
+  stub->initialize_registers(thread, tmp1, tmp2);
+
+  BLOCK_COMMENT("generate_region_crossing_test {");
+  if (VM_Version::has_DistinctOpnds()) {
+    __ z_xgrk(tmp1, store_addr, new_val);
+  } else {
+    __ z_lgr(tmp1, store_addr);
+    __ z_xgr(tmp1, new_val);
+  }
+  __ z_srag(tmp1, tmp1, G1HeapRegion::LogOfHRGrainBytes);
+  __ branch_optimized(Assembler::bcondEqual, *stub->continuation());
+  BLOCK_COMMENT("} generate_region_crossing_test");
+
+  // crosses regions, storing null?
+  if ((stub->barrier_data() & G1C2BarrierPostNotNull) == 0) {
+    __ z_ltgr(new_val, new_val);
+    __ branch_optimized(Assembler::bcondEqual, *stub->continuation());
+  }
+
+  BLOCK_COMMENT("generate_card_young_test {");
+  CardTableBarrierSet* ct = barrier_set_cast<CardTableBarrierSet>(BarrierSet::barrier_set());
+  // calculate address of card
+  __ load_const_optimized(tmp2, (address)ct->card_table()->byte_map_base());      // Card table base.
+  __ z_srlg(tmp1, store_addr, CardTable::card_shift());         // Index into card table.
+  __ z_algr(tmp1, tmp2);                                      // Explicit calculation needed for cli.
+
+  // Filter young.
+  __ z_cli(0, tmp1, G1CardTable::g1_young_card_val());
+
+  BLOCK_COMMENT("} generate_card_young_test");
+
+  // From here on, tmp1 holds the card address.
+  __ branch_optimized(Assembler::bcondNotEqual, *stub->entry());
+
+  __ bind(*stub->continuation());
+
+  BLOCK_COMMENT("} g1_write_barrier_post_c2");
+}
+
+void G1BarrierSetAssembler::generate_c2_post_barrier_stub(MacroAssembler* masm,
+                                                          G1PostBarrierStubC2* stub) const {
+
+  BLOCK_COMMENT("generate_c2_post_barrier_stub {");
+
+  Assembler::InlineSkippedInstructionsCounter skip_counter(masm);
+  Label runtime;
+
+  Register thread     = stub->thread();
+  Register tmp1       = stub->tmp1(); // tmp1 holds the card address.
+  Register tmp2       = stub->tmp2();
+  Register Rcard_addr = tmp1;
+
+  __ bind(*stub->entry());
+
+  BLOCK_COMMENT("generate_card_clean_test {");
+  __ z_sync(); // Required to support concurrent cleaning.
+  __ z_cli(0, Rcard_addr, 0); // Reload after membar.
+  __ branch_optimized(Assembler::bcondEqual, *stub->continuation());
+  BLOCK_COMMENT("} generate_card_clean_test");
+
+  BLOCK_COMMENT("generate_dirty_card {");
+  // Storing a region crossing, non-null oop, card is clean.
+  // Dirty card and log.
+  STATIC_ASSERT(CardTable::dirty_card_val() == 0);
+  __ z_mvi(0, Rcard_addr, CardTable::dirty_card_val());
+  BLOCK_COMMENT("} generate_dirty_card");
+
+  generate_queue_test_and_insertion(masm,
+                                    G1ThreadLocalData::dirty_card_queue_index_offset(),
+                                    G1ThreadLocalData::dirty_card_queue_buffer_offset(),
+                                    runtime,
+                                    Z_thread, tmp1, tmp2);
+
+  __ branch_optimized(Assembler::bcondAlways, *stub->continuation());
+
+  __ bind(runtime);
+
+  generate_c2_barrier_runtime_call(masm, stub, tmp1, CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_post_entry));
+
+  __ branch_optimized(Assembler::bcondAlways, *stub->continuation());
+
+  BLOCK_COMMENT("} generate_c2_post_barrier_stub");
+}
+
+#endif //COMPILER2
+
 void G1BarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorators, BasicType type,
                                     const Address& src, Register dst, Register tmp1, Register tmp2, Label *L_handle_null) {
   bool on_oop = is_reference_type(type);
@@ -106,13 +312,13 @@ void G1BarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorator
   bool on_phantom = (decorators & ON_PHANTOM_OOP_REF) != 0;
   bool on_reference = on_weak || on_phantom;
   Label done;
-  if (on_oop && on_reference && L_handle_null == NULL) { L_handle_null = &done; }
+  if (on_oop && on_reference && L_handle_null == nullptr) { L_handle_null = &done; }
   ModRefBarrierSetAssembler::load_at(masm, decorators, type, src, dst, tmp1, tmp2, L_handle_null);
   if (on_oop && on_reference) {
     // Generate the G1 pre-barrier code to log the value of
     // the referent field in an SATB buffer.
     g1_write_barrier_pre(masm, decorators | IS_NOT_NULL,
-                         NULL /* obj */,
+                         nullptr /* obj */,
                          dst  /* pre_val */,
                          noreg/* preserve */ ,
                          tmp1, tmp2 /* tmp */,
@@ -131,13 +337,10 @@ void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm, Decorator
                                                  ) {
 
   bool not_null  = (decorators & IS_NOT_NULL) != 0,
-       preloaded = obj == NULL;
+       preloaded = obj == nullptr;
 
   const Register Robj = obj ? obj->base() : noreg,
                  Roff = obj ? obj->index() : noreg;
-  const int active_offset = in_bytes(G1ThreadLocalData::satb_mark_queue_active_offset());
-  const int buffer_offset = in_bytes(G1ThreadLocalData::satb_mark_queue_buffer_offset());
-  const int index_offset  = in_bytes(G1ThreadLocalData::satb_mark_queue_index_offset());
   assert_different_registers(Rtmp1, Rtmp2, Z_R0_scratch); // None of the Rtmp<i> must be Z_R0!!
   assert_different_registers(Robj, Z_R0_scratch);         // Used for addressing. Furthermore, push_frame destroys Z_R0!!
   assert_different_registers(Rval, Z_R0_scratch);         // push_frame destroys Z_R0!!
@@ -146,14 +349,7 @@ void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm, Decorator
 
   BLOCK_COMMENT("g1_write_barrier_pre {");
 
-  // Is marking active?
-  // Note: value is loaded for test purposes only. No further use here.
-  if (in_bytes(SATBMarkQueue::byte_width_of_active()) == 4) {
-    __ load_and_test_int(Rtmp1, Address(Z_thread, active_offset));
-  } else {
-    guarantee(in_bytes(SATBMarkQueue::byte_width_of_active()) == 1, "Assumption");
-    __ load_and_test_byte(Rtmp1, Address(Z_thread, active_offset));
-  }
+  generate_pre_barrier_fast_path(masm, Z_thread, Rtmp1);
   __ z_bre(filtered); // Activity indicator is zero, so there is no marking going on currently.
 
   assert(Rpre_val != noreg, "must have a real register");
@@ -169,23 +365,23 @@ void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm, Decorator
     }
   }
 
-  // Is the previous value NULL?
+  // Is the previous value null?
   // If so, we don't need to record it and we're done.
   // Note: pre_val is loaded, decompressed and stored (directly or via runtime call).
   //       Register contents is preserved across runtime call if caller requests to do so.
   if (preloaded && not_null) {
 #ifdef ASSERT
     __ z_ltgr(Rpre_val, Rpre_val);
-    __ asm_assert_ne("null oop not allowed (G1 pre)", 0x321); // Checked by caller.
+    __ asm_assert(Assembler::bcondNotZero, "null oop not allowed (G1 pre)", 0x321); // Checked by caller.
 #endif
   } else {
     __ z_ltgr(Rpre_val, Rpre_val);
-    __ z_bre(filtered); // previous value is NULL, so we don't need to record it.
+    __ z_bre(filtered); // previous value is null, so we don't need to record it.
   }
 
-  // Decode the oop now. We know it's not NULL.
+  // Decode the oop now. We know it's not null.
   if (Robj != noreg && UseCompressedOops) {
-    __ oop_decoder(Rpre_val, Rpre_val, /*maybeNULL=*/false);
+    __ oop_decoder(Rpre_val, Rpre_val, /*maybenullptr=*/false);
   }
 
   // OK, it's not filtered, so we'll need to call enqueue.
@@ -193,24 +389,14 @@ void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm, Decorator
   // We can store the original value in the thread's buffer
   // only if index > 0. Otherwise, we need runtime to handle.
   // (The index field is typed as size_t.)
-  Register Rbuffer = Rtmp1, Rindex = Rtmp2;
-  assert_different_registers(Rbuffer, Rindex, Rpre_val);
 
-  __ z_lg(Rbuffer, buffer_offset, Z_thread);
-
-  __ load_and_test_long(Rindex, Address(Z_thread, index_offset));
-  __ z_bre(callRuntime); // If index == 0, goto runtime.
-
-  __ add2reg(Rindex, -wordSize); // Decrement index.
-  __ z_stg(Rindex, index_offset, Z_thread);
-
-  // Record the previous value.
-  __ z_stg(Rpre_val, 0, Rbuffer, Rindex);
+  generate_queue_test_and_insertion(masm,
+                                    G1ThreadLocalData::satb_mark_queue_index_offset(),
+                                    G1ThreadLocalData::satb_mark_queue_buffer_offset(),
+                                    callRuntime,
+                                    Z_thread, Rpre_val, Rtmp2);
   __ z_bru(filtered);  // We are done.
 
-  Rbuffer = noreg;  // end of life
-  Rindex  = noreg;  // end of life
-
   __ bind(callRuntime);
 
   // Save some registers (inputs and result) over runtime call
@@ -281,14 +467,14 @@ void G1BarrierSetAssembler::g1_write_barrier_post(MacroAssembler* masm, Decorato
     __ z_lgr(Rtmp1, Rstore_addr);
     __ z_xgr(Rtmp1, Rnew_val);
   }
-  __ z_srag(Rtmp1, Rtmp1, HeapRegion::LogOfHRGrainBytes);
+  __ z_srag(Rtmp1, Rtmp1, G1HeapRegion::LogOfHRGrainBytes);
   __ z_bre(filtered);
 
-  // Crosses regions, storing NULL?
+  // Crosses regions, storing null?
   if (not_null) {
 #ifdef ASSERT
     __ z_ltgr(Rnew_val, Rnew_val);
-    __ asm_assert_ne("null oop not allowed (G1 post)", 0x322); // Checked by caller.
+    __ asm_assert(Assembler::bcondNotZero, "null oop not allowed (G1 post)", 0x322); // Checked by caller.
 #endif
   } else {
     __ z_ltgr(Rnew_val, Rnew_val);
@@ -297,7 +483,7 @@ void G1BarrierSetAssembler::g1_write_barrier_post(MacroAssembler* masm, Decorato
 
   Rnew_val = noreg; // end of lifetime
 
-  // Storing region crossing non-NULL, is card already dirty?
+  // Storing region crossing non-null, is card already dirty?
   assert_different_registers(Rtmp1, Rtmp2, Rtmp3);
   // Make sure not to use Z_R0 for any of these registers.
   Register Rcard_addr = (Rtmp1 != Z_R0_scratch) ? Rtmp1 : Rtmp3;
@@ -305,7 +491,7 @@ void G1BarrierSetAssembler::g1_write_barrier_post(MacroAssembler* masm, Decorato
 
   // calculate address of card
   __ load_const_optimized(Rbase, (address)ct->card_table()->byte_map_base());      // Card table base.
-  __ z_srlg(Rcard_addr, Rstore_addr, CardTable::card_shift);         // Index into card table.
+  __ z_srlg(Rcard_addr, Rstore_addr, CardTable::card_shift());         // Index into card table.
   __ z_algr(Rcard_addr, Rbase);                                      // Explicit calculation needed for cli.
   Rbase = noreg; // end of lifetime
 
@@ -319,29 +505,22 @@ void G1BarrierSetAssembler::g1_write_barrier_post(MacroAssembler* masm, Decorato
   __ z_cli(0, Rcard_addr, G1CardTable::dirty_card_val()); // Reload after membar.
   __ z_bre(filtered);
 
-  // Storing a region crossing, non-NULL oop, card is clean.
+  // Storing a region crossing, non-null oop, card is clean.
   // Dirty card and log.
   __ z_mvi(0, Rcard_addr, G1CardTable::dirty_card_val());
 
   Register Rcard_addr_x = Rcard_addr;
   Register Rqueue_index = (Rtmp2 != Z_R0_scratch) ? Rtmp2 : Rtmp1;
-  Register Rqueue_buf   = (Rtmp3 != Z_R0_scratch) ? Rtmp3 : Rtmp1;
-  const int qidx_off    = in_bytes(G1ThreadLocalData::dirty_card_queue_index_offset());
-  const int qbuf_off    = in_bytes(G1ThreadLocalData::dirty_card_queue_buffer_offset());
-  if ((Rcard_addr == Rqueue_buf) || (Rcard_addr == Rqueue_index)) {
+  if (Rcard_addr == Rqueue_index) {
     Rcard_addr_x = Z_R0_scratch;  // Register shortage. We have to use Z_R0.
   }
   __ lgr_if_needed(Rcard_addr_x, Rcard_addr);
 
-  __ load_and_test_long(Rqueue_index, Address(Z_thread, qidx_off));
-  __ z_bre(callRuntime); // Index == 0 then jump to runtime.
-
-  __ z_lg(Rqueue_buf, qbuf_off, Z_thread);
-
-  __ add2reg(Rqueue_index, -wordSize); // Decrement index.
-  __ z_stg(Rqueue_index, qidx_off, Z_thread);
-
-  __ z_stg(Rcard_addr_x, 0, Rqueue_index, Rqueue_buf); // Store card.
+  generate_queue_test_and_insertion(masm,
+                                    G1ThreadLocalData::dirty_card_queue_index_offset(),
+                                    G1ThreadLocalData::dirty_card_queue_buffer_offset(),
+                                    callRuntime,
+                                    Z_thread, Rcard_addr_x, Rqueue_index);
   __ z_bru(filtered);
 
   __ bind(callRuntime);
@@ -379,7 +558,7 @@ void G1BarrierSetAssembler::oop_store_at(MacroAssembler* masm, DecoratorSet deco
 
   BarrierSetAssembler::store_at(masm, decorators, type, dst, val, tmp1, tmp2, tmp3);
 
-  // No need for post barrier if storing NULL
+  // No need for post barrier if storing null
   if (val != noreg) {
     const Register base = dst.base(),
                    idx  = dst.index();
@@ -394,16 +573,16 @@ void G1BarrierSetAssembler::oop_store_at(MacroAssembler* masm, DecoratorSet deco
 void G1BarrierSetAssembler::resolve_jobject(MacroAssembler* masm, Register value, Register tmp1, Register tmp2) {
   NearLabel Ldone, Lnot_weak;
   __ z_ltgr(tmp1, value);
-  __ z_bre(Ldone);          // Use NULL result as-is.
+  __ z_bre(Ldone);          // Use null result as-is.
 
-  __ z_nill(value, ~JNIHandles::weak_tag_mask);
+  __ z_nill(value, ~JNIHandles::tag_mask);
   __ z_lg(value, 0, value); // Resolve (untagged) jobject.
 
-  __ z_tmll(tmp1, JNIHandles::weak_tag_mask); // Test for jweak tag.
+  __ z_tmll(tmp1, JNIHandles::TypeTag::weak_global); // Test for jweak tag.
   __ z_braz(Lnot_weak);
   __ verify_oop(value, FILE_AND_LINE);
   DecoratorSet decorators = IN_NATIVE | ON_PHANTOM_OOP_REF;
-  g1_write_barrier_pre(masm, decorators, (const Address*)NULL, value, noreg, tmp1, tmp2, true);
+  g1_write_barrier_pre(masm, decorators, (const Address*)nullptr, value, noreg, tmp1, tmp2, true);
   __ bind(Lnot_weak);
   __ verify_oop(value, FILE_AND_LINE);
   __ bind(Ldone);
@@ -548,7 +727,7 @@ void G1BarrierSetAssembler::generate_c1_post_barrier_runtime_stub(StubAssembler*
 
   // Calculate address of card corresponding to the updated oop slot.
   AddressLiteral rs(byte_map_base);
-  __ z_srlg(addr_card, addr_oop, CardTable::card_shift);
+  __ z_srlg(addr_card, addr_oop, CardTable::card_shift());
   addr_oop = noreg; // dead now
   __ load_const_optimized(cardtable, rs); // cardtable := <card table base>
   __ z_agr(addr_card, cardtable); // addr_card := addr_oop>>card_shift + cardtable
