@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "c1/c1_Compilation.hpp"
 #include "c1/c1_FrameMap.hpp"
 #include "c1/c1_Instruction.hpp"
@@ -344,20 +343,7 @@ void LIRGenerator::do_NegateOp(NegateOp* x) {
   value.load_item();
   LIR_Opr reg = rlock(x);
 
-  LIR_Opr tmp = LIR_OprFact::illegalOpr;
-#ifdef _LP64
-  if (UseAVX > 2 && !VM_Version::supports_avx512vl()) {
-    if (x->type()->tag() == doubleTag) {
-      tmp = new_register(T_DOUBLE);
-      __ move(LIR_OprFact::doubleConst(-0.0), tmp);
-    }
-    else if (x->type()->tag() == floatTag) {
-      tmp = new_register(T_FLOAT);
-      __ move(LIR_OprFact::floatConst(-0.0), tmp);
-    }
-  }
-#endif
-  __ negate(value.result(), reg, tmp);
+  __ negate(value.result(), reg);
 
   set_result(x, round_item(reg));
 }
@@ -807,7 +793,11 @@ void LIRGenerator::do_MathIntrinsic(Intrinsic* x) {
   if (x->id() == vmIntrinsics::_dexp || x->id() == vmIntrinsics::_dlog ||
       x->id() == vmIntrinsics::_dpow || x->id() == vmIntrinsics::_dcos ||
       x->id() == vmIntrinsics::_dsin || x->id() == vmIntrinsics::_dtan ||
-      x->id() == vmIntrinsics::_dlog10) {
+      x->id() == vmIntrinsics::_dlog10
+#ifdef _LP64
+      || x->id() == vmIntrinsics::_dtanh
+#endif
+      ) {
     do_LibmIntrinsic(x);
     return;
   }
@@ -826,16 +816,8 @@ void LIRGenerator::do_MathIntrinsic(Intrinsic* x) {
   LIR_Opr calc_result = rlock_result(x);
 
   LIR_Opr tmp = LIR_OprFact::illegalOpr;
-#ifdef _LP64
-  if (UseAVX > 2 && (!VM_Version::supports_avx512vl()) &&
-      (x->id() == vmIntrinsics::_dabs)) {
-    tmp = new_register(T_DOUBLE);
-    __ move(LIR_OprFact::doubleConst(-0.0), tmp);
-  }
-#endif
   if (x->id() == vmIntrinsics::_floatToFloat16) {
     tmp = new_register(T_FLOAT);
-    __ move(LIR_OprFact::floatConst(-0.0), tmp);
   }
 
   switch(x->id()) {
@@ -989,11 +971,17 @@ void LIRGenerator::do_LibmIntrinsic(Intrinsic* x) {
       break;
     case vmIntrinsics::_dtan:
        if (StubRoutines::dtan() != nullptr) {
-      __ call_runtime_leaf(StubRoutines::dtan(), getThreadTemp(), result_reg, cc->args());
+        __ call_runtime_leaf(StubRoutines::dtan(), getThreadTemp(), result_reg, cc->args());
       } else {
         __ call_runtime_leaf(CAST_FROM_FN_PTR(address, SharedRuntime::dtan), getThreadTemp(), result_reg, cc->args());
       }
       break;
+    case vmIntrinsics::_dtanh:
+       assert(StubRoutines::dtanh() != nullptr, "tanh intrinsic not found");
+       if (StubRoutines::dtanh() != nullptr) {
+        __ call_runtime_leaf(StubRoutines::dtanh(), getThreadTemp(), result_reg, cc->args());
+      }
+      break;
     default:  ShouldNotReachHere();
   }
 #endif // _LP64
