@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,8 +28,8 @@
 #include "gc/g1/g1BarrierSetAssembler.hpp"
 #include "gc/g1/g1BarrierSetRuntime.hpp"
 #include "gc/g1/g1CardTable.hpp"
+#include "gc/g1/g1HeapRegion.hpp"
 #include "gc/g1/g1ThreadLocalData.hpp"
-#include "gc/g1/heapRegion.hpp"
 #include "interpreter/interp_masm.hpp"
 #include "runtime/sharedRuntime.hpp"
 #include "utilities/debug.hpp"
@@ -38,7 +38,10 @@
 #include "c1/c1_LIRAssembler.hpp"
 #include "c1/c1_MacroAssembler.hpp"
 #include "gc/g1/c1/g1BarrierSetC1.hpp"
-#endif
+#endif // COMPILER1
+#ifdef COMPILER2
+#include "gc/g1/c2/g1BarrierSetC2.hpp"
+#endif // COMPILER2
 
 #define __ masm->
 
@@ -67,7 +70,7 @@ void G1BarrierSetAssembler::gen_write_ref_array_pre_barrier(MacroAssembler* masm
 
     __ jcc(Assembler::equal, filtered);
 
-    __ pusha();                      // push registers
+    __ push_call_clobbered_registers(false /* save_fpu */);
 #ifdef _LP64
     if (count == c_rarg0) {
       if (addr == c_rarg1) {
@@ -90,7 +93,7 @@ void G1BarrierSetAssembler::gen_write_ref_array_pre_barrier(MacroAssembler* masm
     __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_array_pre_oop_entry),
                     addr, count);
 #endif
-    __ popa();
+    __ pop_call_clobbered_registers(false /* save_fpu */);
 
     __ bind(filtered);
   }
@@ -98,7 +101,7 @@ void G1BarrierSetAssembler::gen_write_ref_array_pre_barrier(MacroAssembler* masm
 
 void G1BarrierSetAssembler::gen_write_ref_array_post_barrier(MacroAssembler* masm, DecoratorSet decorators,
                                                              Register addr, Register count, Register tmp) {
-  __ pusha();             // push registers (overkill)
+  __ push_call_clobbered_registers(false /* save_fpu */);
 #ifdef _LP64
   if (c_rarg0 == count) { // On win64 c_rarg0 == rcx
     assert_different_registers(c_rarg1, addr);
@@ -114,7 +117,7 @@ void G1BarrierSetAssembler::gen_write_ref_array_post_barrier(MacroAssembler* mas
   __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_array_post_entry),
                   addr, count);
 #endif
-  __ popa();
+  __ pop_call_clobbered_registers(false /* save_fpu */);
 }
 
 void G1BarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorators, BasicType type,
@@ -125,8 +128,24 @@ void G1BarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorator
   bool on_reference = on_weak || on_phantom;
   ModRefBarrierSetAssembler::load_at(masm, decorators, type, dst, src, tmp1, tmp_thread);
   if (on_oop && on_reference) {
-    const Register thread = NOT_LP64(tmp_thread) LP64_ONLY(r15_thread);
-    NOT_LP64(__ get_thread(thread));
+    Register thread = NOT_LP64(tmp_thread) LP64_ONLY(r15_thread);
+
+#ifndef _LP64
+    // Work around the x86_32 bug that only manifests with Loom for some reason.
+    // MacroAssembler::resolve_weak_handle calls this barrier with tmp_thread == noreg.
+    if (thread == noreg) {
+      if (dst != rcx && tmp1 != rcx) {
+        thread = rcx;
+      } else if (dst != rdx && tmp1 != rdx) {
+        thread = rdx;
+      } else if (dst != rdi && tmp1 != rdi) {
+        thread = rdi;
+      }
+    }
+    assert_different_registers(dst, tmp1, thread);
+    __ push(thread);
+    __ get_thread(thread);
+#endif
 
     // Generate the G1 pre-barrier code to log the value of
     // the referent field in an SATB buffer.
@@ -137,9 +156,63 @@ void G1BarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorator
                          tmp1 /* tmp */,
                          true /* tosca_live */,
                          true /* expand_call */);
+
+#ifndef _LP64
+    __ pop(thread);
+#endif
   }
 }
 
+static void generate_queue_insertion(MacroAssembler* masm, ByteSize index_offset, ByteSize buffer_offset, Label& runtime,
+                                     const Register thread, const Register value, const Register temp) {
+  // This code assumes that buffer index is pointer sized.
+  STATIC_ASSERT(in_bytes(SATBMarkQueue::byte_width_of_index()) == sizeof(intptr_t));
+  // Can we store a value in the given thread's buffer?
+  // (The index field is typed as size_t.)
+  __ movptr(temp, Address(thread, in_bytes(index_offset)));   // temp := *(index address)
+  __ testptr(temp, temp);                                     // index == 0?
+  __ jcc(Assembler::zero, runtime);                           // jump to runtime if index == 0 (full buffer)
+  // The buffer is not full, store value into it.
+  __ subptr(temp, wordSize);                                  // temp := next index
+  __ movptr(Address(thread, in_bytes(index_offset)), temp);   // *(index address) := next index
+  __ addptr(temp, Address(thread, in_bytes(buffer_offset)));  // temp := buffer address + next index
+  __ movptr(Address(temp, 0), value);                         // *(buffer address + next index) := value
+}
+
+static void generate_pre_barrier_fast_path(MacroAssembler* masm,
+                                           const Register thread) {
+  Address in_progress(thread, in_bytes(G1ThreadLocalData::satb_mark_queue_active_offset()));
+  // Is marking active?
+  if (in_bytes(SATBMarkQueue::byte_width_of_active()) == 4) {
+    __ cmpl(in_progress, 0);
+  } else {
+    assert(in_bytes(SATBMarkQueue::byte_width_of_active()) == 1, "Assumption");
+    __ cmpb(in_progress, 0);
+  }
+}
+
+static void generate_pre_barrier_slow_path(MacroAssembler* masm,
+                                           const Register obj,
+                                           const Register pre_val,
+                                           const Register thread,
+                                           const Register tmp,
+                                           Label& done,
+                                           Label& runtime) {
+  // Do we need to load the previous value?
+  if (obj != noreg) {
+    __ load_heap_oop(pre_val, Address(obj, 0), noreg, noreg, AS_RAW);
+  }
+  // Is the previous value null?
+  __ cmpptr(pre_val, NULL_WORD);
+  __ jcc(Assembler::equal, done);
+  generate_queue_insertion(masm,
+                           G1ThreadLocalData::satb_mark_queue_index_offset(),
+                           G1ThreadLocalData::satb_mark_queue_buffer_offset(),
+                           runtime,
+                           thread, pre_val, tmp);
+  __ jmp(done);
+}
+
 void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm,
                                                  Register obj,
                                                  Register pre_val,
@@ -165,57 +238,19 @@ void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm,
     assert(pre_val != rax, "check this code");
   }
 
-  Address in_progress(thread, in_bytes(G1ThreadLocalData::satb_mark_queue_active_offset()));
-  Address index(thread, in_bytes(G1ThreadLocalData::satb_mark_queue_index_offset()));
-  Address buffer(thread, in_bytes(G1ThreadLocalData::satb_mark_queue_buffer_offset()));
-
-  // Is marking active?
-  if (in_bytes(SATBMarkQueue::byte_width_of_active()) == 4) {
-    __ cmpl(in_progress, 0);
-  } else {
-    assert(in_bytes(SATBMarkQueue::byte_width_of_active()) == 1, "Assumption");
-    __ cmpb(in_progress, 0);
-  }
+  generate_pre_barrier_fast_path(masm, thread);
+  // If marking is not active (*(mark queue active address) == 0), jump to done
   __ jcc(Assembler::equal, done);
-
-  // Do we need to load the previous value?
-  if (obj != noreg) {
-    __ load_heap_oop(pre_val, Address(obj, 0), noreg, noreg, AS_RAW);
-  }
-
-  // Is the previous value null?
-  __ cmpptr(pre_val, (int32_t) NULL_WORD);
-  __ jcc(Assembler::equal, done);
-
-  // Can we store original value in the thread's buffer?
-  // Is index == 0?
-  // (The index field is typed as size_t.)
-
-  __ movptr(tmp, index);                   // tmp := *index_adr
-  __ cmpptr(tmp, 0);                       // tmp == 0?
-  __ jcc(Assembler::equal, runtime);       // If yes, goto runtime
-
-  __ subptr(tmp, wordSize);                // tmp := tmp - wordSize
-  __ movptr(index, tmp);                   // *index_adr := tmp
-  __ addptr(tmp, buffer);                  // tmp := tmp + *buffer_adr
-
-  // Record the previous value
-  __ movptr(Address(tmp, 0), pre_val);
-  __ jmp(done);
+  generate_pre_barrier_slow_path(masm, obj, pre_val, thread, tmp, done, runtime);
 
   __ bind(runtime);
-  // save the live input values
-  if(tosca_live) __ push(rax);
 
-  if (obj != noreg && obj != rax)
-    __ push(obj);
-
-  if (pre_val != rax)
-    __ push(pre_val);
+  // Determine and save the live input values
+  __ push_call_clobbered_registers();
 
   // Calling the runtime using the regular call_VM_leaf mechanism generates
   // code (generated by InterpreterMacroAssember::call_VM_leaf_base)
-  // that checks that the *(ebp+frame::interpreter_frame_last_sp) == NULL.
+  // that checks that the *(ebp+frame::interpreter_frame_last_sp) == nullptr.
   //
   // If we care generating the pre-barrier without a frame (e.g. in the
   // intrinsified Reference.get() routine) then ebp might be pointing to
@@ -225,8 +260,6 @@ void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm,
   // So when we do not have have a full interpreter frame on the stack
   // expand_call should be passed true.
 
-  NOT_LP64( __ push(thread); )
-
   if (expand_call) {
     LP64_ONLY( assert(pre_val != c_rarg1, "smashed arg"); )
 #ifdef _LP64
@@ -245,112 +278,196 @@ void G1BarrierSetAssembler::g1_write_barrier_pre(MacroAssembler* masm,
     __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_pre_entry), pre_val, thread);
   }
 
-  NOT_LP64( __ pop(thread); )
-
-  // save the live input values
-  if (pre_val != rax)
-    __ pop(pre_val);
-
-  if (obj != noreg && obj != rax)
-    __ pop(obj);
-
-  if(tosca_live) __ pop(rax);
+  __ pop_call_clobbered_registers();
 
   __ bind(done);
 }
 
+static void generate_post_barrier_fast_path(MacroAssembler* masm,
+                                            const Register store_addr,
+                                            const Register new_val,
+                                            const Register tmp,
+                                            const Register tmp2,
+                                            Label& done,
+                                            bool new_val_may_be_null) {
+  CardTableBarrierSet* ct = barrier_set_cast<CardTableBarrierSet>(BarrierSet::barrier_set());
+  // Does store cross heap regions?
+  __ movptr(tmp, store_addr);                                    // tmp := store address
+  __ xorptr(tmp, new_val);                                       // tmp := store address ^ new value
+  __ shrptr(tmp, G1HeapRegion::LogOfHRGrainBytes);               // ((store address ^ new value) >> LogOfHRGrainBytes) == 0?
+  __ jcc(Assembler::equal, done);
+  // Crosses regions, storing null?
+  if (new_val_may_be_null) {
+    __ cmpptr(new_val, NULL_WORD);                               // new value == null?
+    __ jcc(Assembler::equal, done);
+  }
+  // Storing region crossing non-null, is card young?
+  __ movptr(tmp, store_addr);                                    // tmp := store address
+  __ shrptr(tmp, CardTable::card_shift());                       // tmp := card address relative to card table base
+  // Do not use ExternalAddress to load 'byte_map_base', since 'byte_map_base' is NOT
+  // a valid address and therefore is not properly handled by the relocation code.
+  __ movptr(tmp2, (intptr_t)ct->card_table()->byte_map_base());  // tmp2 := card table base address
+  __ addptr(tmp, tmp2);                                          // tmp := card address
+  __ cmpb(Address(tmp, 0), G1CardTable::g1_young_card_val());    // *(card address) == young_card_val?
+}
+
+static void generate_post_barrier_slow_path(MacroAssembler* masm,
+                                            const Register thread,
+                                            const Register tmp,
+                                            const Register tmp2,
+                                            Label& done,
+                                            Label& runtime) {
+  __ membar(Assembler::Membar_mask_bits(Assembler::StoreLoad));  // StoreLoad membar
+  __ cmpb(Address(tmp, 0), G1CardTable::dirty_card_val());       // *(card address) == dirty_card_val?
+  __ jcc(Assembler::equal, done);
+  // Storing a region crossing, non-null oop, card is clean.
+  // Dirty card and log.
+  __ movb(Address(tmp, 0), G1CardTable::dirty_card_val());       // *(card address) := dirty_card_val
+  generate_queue_insertion(masm,
+                           G1ThreadLocalData::dirty_card_queue_index_offset(),
+                           G1ThreadLocalData::dirty_card_queue_buffer_offset(),
+                           runtime,
+                           thread, tmp, tmp2);
+  __ jmp(done);
+}
+
 void G1BarrierSetAssembler::g1_write_barrier_post(MacroAssembler* masm,
                                                   Register store_addr,
                                                   Register new_val,
                                                   Register thread,
                                                   Register tmp,
                                                   Register tmp2) {
-  // Generated code assumes that buffer index is pointer sized.
-  STATIC_ASSERT(in_bytes(SATBMarkQueue::byte_width_of_index()) == sizeof(intptr_t));
 #ifdef _LP64
   assert(thread == r15_thread, "must be");
 #endif // _LP64
 
-  Address queue_index(thread, in_bytes(G1ThreadLocalData::dirty_card_queue_index_offset()));
-  Address buffer(thread, in_bytes(G1ThreadLocalData::dirty_card_queue_buffer_offset()));
-
-  CardTableBarrierSet* ct =
-    barrier_set_cast<CardTableBarrierSet>(BarrierSet::barrier_set());
-
   Label done;
   Label runtime;
 
-  // Does store cross heap regions?
-
-  __ movptr(tmp, store_addr);
-  __ xorptr(tmp, new_val);
-  __ shrptr(tmp, HeapRegion::LogOfHRGrainBytes);
+  generate_post_barrier_fast_path(masm, store_addr, new_val, tmp, tmp2, done, true /* new_val_may_be_null */);
+  // If card is young, jump to done
   __ jcc(Assembler::equal, done);
-
-  // crosses regions, storing NULL?
-
-  __ cmpptr(new_val, (int32_t) NULL_WORD);
-  __ jcc(Assembler::equal, done);
-
-  // storing region crossing non-NULL, is card already dirty?
-
-  const Register card_addr = tmp;
-  const Register cardtable = tmp2;
-
-  __ movptr(card_addr, store_addr);
-  __ shrptr(card_addr, CardTable::card_shift);
-  // Do not use ExternalAddress to load 'byte_map_base', since 'byte_map_base' is NOT
-  // a valid address and therefore is not properly handled by the relocation code.
-  __ movptr(cardtable, (intptr_t)ct->card_table()->byte_map_base());
-  __ addptr(card_addr, cardtable);
-
-  __ cmpb(Address(card_addr, 0), (int)G1CardTable::g1_young_card_val());
-  __ jcc(Assembler::equal, done);
-
-  __ membar(Assembler::Membar_mask_bits(Assembler::StoreLoad));
-  __ cmpb(Address(card_addr, 0), (int)G1CardTable::dirty_card_val());
-  __ jcc(Assembler::equal, done);
-
-
-  // storing a region crossing, non-NULL oop, card is clean.
-  // dirty card and log.
-
-  __ movb(Address(card_addr, 0), (int)G1CardTable::dirty_card_val());
-
-  __ movptr(tmp2, queue_index);
-  __ testptr(tmp2, tmp2);
-  __ jcc(Assembler::zero, runtime);
-  __ subptr(tmp2, wordSize);
-  __ movptr(queue_index, tmp2);
-  __ addptr(tmp2, buffer);
-  __ movptr(Address(tmp2, 0), card_addr);
-  __ jmp(done);
+  generate_post_barrier_slow_path(masm, thread, tmp, tmp2, done, runtime);
 
   __ bind(runtime);
   // save the live input values
-  __ push(store_addr);
-#ifdef _LP64
-  __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_post_entry), card_addr, r15_thread);
-#else
-  __ push(thread);
-  __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_post_entry), card_addr, thread);
-  __ pop(thread);
-#endif
-  __ pop(store_addr);
+  RegSet saved = RegSet::of(store_addr NOT_LP64(COMMA thread));
+  __ push_set(saved);
+  __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_post_entry), tmp, thread);
+  __ pop_set(saved);
 
   __ bind(done);
 }
 
+#if defined(COMPILER2)
+
+static void generate_c2_barrier_runtime_call(MacroAssembler* masm, G1BarrierStubC2* stub, const Register arg, const address runtime_path) {
+#ifdef _LP64
+  SaveLiveRegisters save_registers(masm, stub);
+  if (c_rarg0 != arg) {
+    __ mov(c_rarg0, arg);
+  }
+  __ mov(c_rarg1, r15_thread);
+  // rax is a caller-saved, non-argument-passing register, so it does not
+  // interfere with c_rarg0 or c_rarg1. If it contained any live value before
+  // entering this stub, it is saved at this point, and restored after the
+  // call. If it did not contain any live value, it is free to be used. In
+  // either case, it is safe to use it here as a call scratch register.
+  __ call(RuntimeAddress(runtime_path), rax);
+#else
+  Unimplemented();
+#endif // _LP64
+}
+
+void G1BarrierSetAssembler::g1_write_barrier_pre_c2(MacroAssembler* masm,
+                                                    Register obj,
+                                                    Register pre_val,
+                                                    Register thread,
+                                                    Register tmp,
+                                                    G1PreBarrierStubC2* stub) {
+#ifdef _LP64
+  assert(thread == r15_thread, "must be");
+#endif // _LP64
+  assert(pre_val != noreg, "check this code");
+  if (obj != noreg) {
+    assert_different_registers(obj, pre_val, tmp);
+  }
+
+  stub->initialize_registers(obj, pre_val, thread, tmp);
+
+  generate_pre_barrier_fast_path(masm, thread);
+  // If marking is active (*(mark queue active address) != 0), jump to stub (slow path)
+  __ jcc(Assembler::notEqual, *stub->entry());
+
+  __ bind(*stub->continuation());
+}
+
+void G1BarrierSetAssembler::generate_c2_pre_barrier_stub(MacroAssembler* masm,
+                                                         G1PreBarrierStubC2* stub) const {
+  Assembler::InlineSkippedInstructionsCounter skip_counter(masm);
+  Label runtime;
+  Register obj = stub->obj();
+  Register pre_val = stub->pre_val();
+  Register thread = stub->thread();
+  Register tmp = stub->tmp1();
+  assert(stub->tmp2() == noreg, "not needed in this platform");
+
+  __ bind(*stub->entry());
+  generate_pre_barrier_slow_path(masm, obj, pre_val, thread, tmp, *stub->continuation(), runtime);
+
+  __ bind(runtime);
+  generate_c2_barrier_runtime_call(masm, stub, pre_val, CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_pre_entry));
+  __ jmp(*stub->continuation());
+}
+
+void G1BarrierSetAssembler::g1_write_barrier_post_c2(MacroAssembler* masm,
+                                                     Register store_addr,
+                                                     Register new_val,
+                                                     Register thread,
+                                                     Register tmp,
+                                                     Register tmp2,
+                                                     G1PostBarrierStubC2* stub) {
+#ifdef _LP64
+  assert(thread == r15_thread, "must be");
+#endif // _LP64
+
+  stub->initialize_registers(thread, tmp, tmp2);
+
+  bool new_val_may_be_null = (stub->barrier_data() & G1C2BarrierPostNotNull) == 0;
+  generate_post_barrier_fast_path(masm, store_addr, new_val, tmp, tmp2, *stub->continuation(), new_val_may_be_null);
+  // If card is not young, jump to stub (slow path)
+  __ jcc(Assembler::notEqual, *stub->entry());
+
+  __ bind(*stub->continuation());
+}
+
+void G1BarrierSetAssembler::generate_c2_post_barrier_stub(MacroAssembler* masm,
+                                                          G1PostBarrierStubC2* stub) const {
+  Assembler::InlineSkippedInstructionsCounter skip_counter(masm);
+  Label runtime;
+  Register thread = stub->thread();
+  Register tmp = stub->tmp1(); // tmp holds the card address.
+  Register tmp2 = stub->tmp2();
+  assert(stub->tmp3() == noreg, "not needed in this platform");
+
+  __ bind(*stub->entry());
+  generate_post_barrier_slow_path(masm, thread, tmp, tmp2, *stub->continuation(), runtime);
+
+  __ bind(runtime);
+  generate_c2_barrier_runtime_call(masm, stub, tmp, CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_post_entry));
+  __ jmp(*stub->continuation());
+}
+
+#endif // COMPILER2
+
 void G1BarrierSetAssembler::oop_store_at(MacroAssembler* masm, DecoratorSet decorators, BasicType type,
-                                         Address dst, Register val, Register tmp1, Register tmp2) {
+                                         Address dst, Register val, Register tmp1, Register tmp2, Register tmp3) {
   bool in_heap = (decorators & IN_HEAP) != 0;
   bool as_normal = (decorators & AS_NORMAL) != 0;
-  assert((decorators & IS_DEST_UNINITIALIZED) == 0, "unsupported");
 
   bool needs_pre_barrier = as_normal;
   bool needs_post_barrier = val != noreg && in_heap;
 
-  Register tmp3 = LP64_ONLY(r8) NOT_LP64(rsi);
   Register rthread = LP64_ONLY(r15_thread) NOT_LP64(rcx);
   // flatten object address if needed
   // We do it regardless of precise because we need the registers
@@ -379,7 +496,7 @@ void G1BarrierSetAssembler::oop_store_at(MacroAssembler* masm, DecoratorSet deco
                          false /* expand_call */);
   }
   if (val == noreg) {
-    BarrierSetAssembler::store_at(masm, decorators, type, Address(tmp1, 0), val, noreg, noreg);
+    BarrierSetAssembler::store_at(masm, decorators, type, Address(tmp1, 0), val, noreg, noreg, noreg);
   } else {
     Register new_val = val;
     if (needs_post_barrier) {
@@ -389,7 +506,7 @@ void G1BarrierSetAssembler::oop_store_at(MacroAssembler* masm, DecoratorSet deco
         __ movptr(new_val, val);
       }
     }
-    BarrierSetAssembler::store_at(masm, decorators, type, Address(tmp1, 0), val, noreg, noreg);
+    BarrierSetAssembler::store_at(masm, decorators, type, Address(tmp1, 0), val, noreg, noreg, noreg);
     if (needs_post_barrier) {
       g1_write_barrier_post(masm /*masm*/,
                             tmp1 /* store_adr */,
@@ -423,7 +540,7 @@ void G1BarrierSetAssembler::gen_pre_barrier_stub(LIR_Assembler* ce, G1PreBarrier
     ce->mem2reg(stub->addr(), stub->pre_val(), T_OBJECT, stub->patch_code(), stub->info(), false /*wide*/);
   }
 
-  __ cmpptr(pre_val_reg, (int32_t)NULL_WORD);
+  __ cmpptr(pre_val_reg, NULL_WORD);
   __ jcc(Assembler::equal, *stub->continuation());
   ce->store_parameter(stub->pre_val()->as_register(), 0);
   __ call(RuntimeAddress(bs->pre_barrier_c1_runtime_code_blob()->code_begin()));
@@ -437,7 +554,7 @@ void G1BarrierSetAssembler::gen_post_barrier_stub(LIR_Assembler* ce, G1PostBarri
   assert(stub->addr()->is_register(), "Precondition.");
   assert(stub->new_val()->is_register(), "Precondition.");
   Register new_val_reg = stub->new_val()->as_register();
-  __ cmpptr(new_val_reg, (int32_t) NULL_WORD);
+  __ cmpptr(new_val_reg, NULL_WORD);
   __ jcc(Assembler::equal, *stub->continuation());
   ce->store_parameter(stub->addr()->as_pointer_register(), 0);
   __ call(RuntimeAddress(bs->post_barrier_c1_runtime_code_blob()->code_begin()));
@@ -496,13 +613,13 @@ void G1BarrierSetAssembler::generate_c1_pre_barrier_runtime_stub(StubAssembler*
 
   __ bind(runtime);
 
-  __ save_live_registers_no_oop_map(true);
+  __ push_call_clobbered_registers();
 
   // load the pre-value
   __ load_parameter(0, rcx);
   __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_pre_entry), rcx, thread);
 
-  __ restore_live_registers(true);
+  __ pop_call_clobbered_registers();
 
   __ bind(done);
 
@@ -515,9 +632,6 @@ void G1BarrierSetAssembler::generate_c1_pre_barrier_runtime_stub(StubAssembler*
 void G1BarrierSetAssembler::generate_c1_post_barrier_runtime_stub(StubAssembler* sasm) {
   __ prologue("g1_post_barrier", false);
 
-  // arg0: store_address
-  Address store_addr(rbp, 2*BytesPerWord);
-
   CardTableBarrierSet* ct =
     barrier_set_cast<CardTableBarrierSet>(BarrierSet::barrier_set());
 
@@ -525,7 +639,7 @@ void G1BarrierSetAssembler::generate_c1_post_barrier_runtime_stub(StubAssembler*
   Label enqueued;
   Label runtime;
 
-  // At this point we know new_value is non-NULL and the new_value crosses regions.
+  // At this point we know new_value is non-null and the new_value crosses regions.
   // Must check to see if card is already dirty
 
   const Register thread = NOT_LP64(rax) LP64_ONLY(r15_thread);
@@ -540,7 +654,7 @@ void G1BarrierSetAssembler::generate_c1_post_barrier_runtime_stub(StubAssembler*
   const Register card_addr = rcx;
 
   __ load_parameter(0, card_addr);
-  __ shrptr(card_addr, CardTable::card_shift);
+  __ shrptr(card_addr, CardTable::card_shift());
   // Do not use ExternalAddress to load 'byte_map_base', since 'byte_map_base' is NOT
   // a valid address and therefore is not properly handled by the relocation code.
   __ movptr(cardtable, (intptr_t)ct->card_table()->byte_map_base());
@@ -548,17 +662,17 @@ void G1BarrierSetAssembler::generate_c1_post_barrier_runtime_stub(StubAssembler*
 
   NOT_LP64(__ get_thread(thread);)
 
-  __ cmpb(Address(card_addr, 0), (int)G1CardTable::g1_young_card_val());
+  __ cmpb(Address(card_addr, 0), G1CardTable::g1_young_card_val());
   __ jcc(Assembler::equal, done);
 
   __ membar(Assembler::Membar_mask_bits(Assembler::StoreLoad));
-  __ cmpb(Address(card_addr, 0), (int)CardTable::dirty_card_val());
+  __ cmpb(Address(card_addr, 0), CardTable::dirty_card_val());
   __ jcc(Assembler::equal, done);
 
-  // storing region crossing non-NULL, card is clean.
+  // storing region crossing non-null, card is clean.
   // dirty card and log.
 
-  __ movb(Address(card_addr, 0), (int)CardTable::dirty_card_val());
+  __ movb(Address(card_addr, 0), CardTable::dirty_card_val());
 
   const Register tmp = rdx;
   __ push(rdx);
@@ -573,12 +687,11 @@ void G1BarrierSetAssembler::generate_c1_post_barrier_runtime_stub(StubAssembler*
   __ jmp(enqueued);
 
   __ bind(runtime);
-
-  __ save_live_registers_no_oop_map(true);
+  __ push_call_clobbered_registers();
 
   __ call_VM_leaf(CAST_FROM_FN_PTR(address, G1BarrierSetRuntime::write_ref_field_post_entry), card_addr, thread);
 
-  __ restore_live_registers(true);
+  __ pop_call_clobbered_registers();
 
   __ bind(enqueued);
   __ pop(rdx);
