@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "classfile/classLoaderData.hpp"
 #include "gc/shared/barrierSet.hpp"
 #include "gc/shared/barrierSetAssembler.hpp"
@@ -423,7 +422,7 @@ void BarrierSetAssembler::c2i_entry_barrier(MacroAssembler* masm) {
   __ load_method_holder_cld(tmp1, rbx);
 
    // Is it a strong CLD?
-  __ cmpl(Address(tmp1, ClassLoaderData::keep_alive_offset()), 0);
+  __ cmpl(Address(tmp1, ClassLoaderData::keep_alive_ref_count_offset()), 0);
   __ jcc(Assembler::greater, method_live);
 
    // Is it a weak but alive CLD?
@@ -583,6 +582,25 @@ void SaveLiveRegisters::initialize(BarrierStubC2* stub) {
   caller_saved.Insert(OptoReg::as_OptoReg(r10->as_VMReg()));
   caller_saved.Insert(OptoReg::as_OptoReg(r11->as_VMReg()));
 
+  if (UseAPX) {
+    caller_saved.Insert(OptoReg::as_OptoReg(r16->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r17->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r18->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r19->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r20->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r21->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r22->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r23->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r24->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r25->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r26->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r27->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r28->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r29->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r30->as_VMReg()));
+    caller_saved.Insert(OptoReg::as_OptoReg(r31->as_VMReg()));
+  }
+
   int gp_spill_size = 0;
   int opmask_spill_size = 0;
   int xmm_spill_size = 0;
