@@ -24,7 +24,6 @@
 *
 */
 
-#include "precompiled.hpp"
 #include "asm/assembler.hpp"
 #include "asm/assembler.inline.hpp"
 #include "macroAssembler_x86.hpp"
@@ -176,7 +175,7 @@
 
 // The 32 bit code is at most SSE2 compliant
 
-ATTRIBUTE_ALIGNED(16) juint _static_const_table_cos[] =
+ATTRIBUTE_ALIGNED(16) static const juint _static_const_table_cos[] =
 {
     0x00000000UL, 0x00000000UL, 0x00000000UL, 0x00000000UL, 0x00000000UL,
     0x00000000UL, 0x00000000UL, 0x3ff00000UL, 0x176d6d31UL, 0xbf73b92eUL,
