@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,92 +22,73 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "runtime/deoptimization.hpp"
 #include "runtime/frame.inline.hpp"
+#include "runtime/javaThread.hpp"
 #include "runtime/stubRoutines.hpp"
-#include "runtime/thread.inline.hpp"
 #include "utilities/globalDefinitions.hpp"
 #include "crc32c.h"
 
 // Implementation of the platform-specific part of StubRoutines - for
 // a description of how to extend it, see the stubRoutines.hpp file.
 
-address StubRoutines::x86::_verify_mxcsr_entry = NULL;
-address StubRoutines::x86::_key_shuffle_mask_addr = NULL;
-address StubRoutines::x86::_counter_shuffle_mask_addr = NULL;
-address StubRoutines::x86::_ghash_long_swap_mask_addr = NULL;
-address StubRoutines::x86::_ghash_byte_swap_mask_addr = NULL;
-address StubRoutines::x86::_ghash_poly_addr = NULL;
-address StubRoutines::x86::_ghash_shuffmask_addr = NULL;
-address StubRoutines::x86::_upper_word_mask_addr = NULL;
-address StubRoutines::x86::_shuffle_byte_flip_mask_addr = NULL;
-address StubRoutines::x86::_k256_adr = NULL;
-address StubRoutines::x86::_vector_short_to_byte_mask = NULL;
-address StubRoutines::x86::_vector_int_to_byte_mask = NULL;
-address StubRoutines::x86::_vector_int_to_short_mask = NULL;
-address StubRoutines::x86::_vector_all_bits_set = NULL;
-address StubRoutines::x86::_vector_byte_shuffle_mask = NULL;
-address StubRoutines::x86::_vector_short_shuffle_mask = NULL;
-address StubRoutines::x86::_vector_int_shuffle_mask = NULL;
-address StubRoutines::x86::_vector_long_shuffle_mask = NULL;
-address StubRoutines::x86::_vector_float_sign_mask = NULL;
-address StubRoutines::x86::_vector_float_sign_flip = NULL;
-address StubRoutines::x86::_vector_double_sign_mask = NULL;
-address StubRoutines::x86::_vector_double_sign_flip = NULL;
-address StubRoutines::x86::_vector_byte_perm_mask = NULL;
-address StubRoutines::x86::_vector_long_sign_mask = NULL;
-address StubRoutines::x86::_vector_iota_indices = NULL;
-address StubRoutines::x86::_vector_32_bit_mask = NULL;
-address StubRoutines::x86::_vector_64_bit_mask = NULL;
+address StubRoutines::x86::_verify_mxcsr_entry = nullptr;
+address StubRoutines::x86::_upper_word_mask_addr = nullptr;
+address StubRoutines::x86::_shuffle_byte_flip_mask_addr = nullptr;
+address StubRoutines::x86::_k256_adr = nullptr;
+address StubRoutines::x86::_vector_short_to_byte_mask = nullptr;
+address StubRoutines::x86::_vector_int_to_byte_mask = nullptr;
+address StubRoutines::x86::_vector_int_to_short_mask = nullptr;
+address StubRoutines::x86::_vector_all_bits_set = nullptr;
+address StubRoutines::x86::_vector_byte_shuffle_mask = nullptr;
+address StubRoutines::x86::_vector_int_mask_cmp_bits = nullptr;
+address StubRoutines::x86::_vector_short_shuffle_mask = nullptr;
+address StubRoutines::x86::_vector_int_shuffle_mask = nullptr;
+address StubRoutines::x86::_vector_long_shuffle_mask = nullptr;
+address StubRoutines::x86::_vector_float_sign_mask = nullptr;
+address StubRoutines::x86::_vector_float_sign_flip = nullptr;
+address StubRoutines::x86::_vector_double_sign_mask = nullptr;
+address StubRoutines::x86::_vector_double_sign_flip = nullptr;
+address StubRoutines::x86::_vector_byte_perm_mask = nullptr;
+address StubRoutines::x86::_vector_long_sign_mask = nullptr;
+address StubRoutines::x86::_vector_iota_indices = nullptr;
+address StubRoutines::x86::_vector_reverse_bit_lut = nullptr;
+address StubRoutines::x86::_vector_reverse_byte_perm_mask_long = nullptr;
+address StubRoutines::x86::_vector_reverse_byte_perm_mask_int = nullptr;
+address StubRoutines::x86::_vector_reverse_byte_perm_mask_short = nullptr;
+address StubRoutines::x86::_vector_popcount_lut = nullptr;
+address StubRoutines::x86::_vector_count_leading_zeros_lut = nullptr;
+address StubRoutines::x86::_vector_32_bit_mask = nullptr;
+address StubRoutines::x86::_vector_64_bit_mask = nullptr;
 #ifdef _LP64
-address StubRoutines::x86::_k256_W_adr = NULL;
-address StubRoutines::x86::_k512_W_addr = NULL;
-address StubRoutines::x86::_pshuffle_byte_flip_mask_addr_sha512 = NULL;
+address StubRoutines::x86::_k256_W_adr = nullptr;
+address StubRoutines::x86::_k512_W_addr = nullptr;
+address StubRoutines::x86::_pshuffle_byte_flip_mask_addr_sha512 = nullptr;
 // Base64 masks
-address StubRoutines::x86::_encoding_table_base64 = NULL;
-address StubRoutines::x86::_shuffle_base64 = NULL;
-address StubRoutines::x86::_avx2_shuffle_base64 = NULL;
-address StubRoutines::x86::_avx2_input_mask_base64 = NULL;
-address StubRoutines::x86::_avx2_lut_base64 = NULL;
-address StubRoutines::x86::_counter_mask_addr = NULL;
-address StubRoutines::x86::_lookup_lo_base64 = NULL;
-address StubRoutines::x86::_lookup_hi_base64 = NULL;
-address StubRoutines::x86::_lookup_lo_base64url = NULL;
-address StubRoutines::x86::_lookup_hi_base64url = NULL;
-address StubRoutines::x86::_pack_vec_base64 = NULL;
-address StubRoutines::x86::_join_0_1_base64 = NULL;
-address StubRoutines::x86::_join_1_2_base64 = NULL;
-address StubRoutines::x86::_join_2_3_base64 = NULL;
-address StubRoutines::x86::_decoding_table_base64 = NULL;
-address StubRoutines::x86::_ghash_poly512_addr = NULL;
+address StubRoutines::x86::_encoding_table_base64 = nullptr;
+address StubRoutines::x86::_shuffle_base64 = nullptr;
+address StubRoutines::x86::_avx2_shuffle_base64 = nullptr;
+address StubRoutines::x86::_avx2_input_mask_base64 = nullptr;
+address StubRoutines::x86::_avx2_lut_base64 = nullptr;
+address StubRoutines::x86::_avx2_decode_tables_base64 = nullptr;
+address StubRoutines::x86::_avx2_decode_lut_tables_base64 = nullptr;
+address StubRoutines::x86::_lookup_lo_base64 = nullptr;
+address StubRoutines::x86::_lookup_hi_base64 = nullptr;
+address StubRoutines::x86::_lookup_lo_base64url = nullptr;
+address StubRoutines::x86::_lookup_hi_base64url = nullptr;
+address StubRoutines::x86::_pack_vec_base64 = nullptr;
+address StubRoutines::x86::_join_0_1_base64 = nullptr;
+address StubRoutines::x86::_join_1_2_base64 = nullptr;
+address StubRoutines::x86::_join_2_3_base64 = nullptr;
+address StubRoutines::x86::_decoding_table_base64 = nullptr;
+address StubRoutines::x86::_compress_perm_table32 = nullptr;
+address StubRoutines::x86::_compress_perm_table64 = nullptr;
+address StubRoutines::x86::_expand_perm_table32 = nullptr;
+address StubRoutines::x86::_expand_perm_table64 = nullptr;
 #endif
-address StubRoutines::x86::_pshuffle_byte_flip_mask_addr = NULL;
+address StubRoutines::x86::_pshuffle_byte_flip_mask_addr = nullptr;
 
-//tables common for sin and cos
-address StubRoutines::x86::_ONEHALF_adr = NULL;
-address StubRoutines::x86::_P_2_adr = NULL;
-address StubRoutines::x86::_SC_4_adr = NULL;
-address StubRoutines::x86::_Ctable_adr = NULL;
-address StubRoutines::x86::_SC_2_adr = NULL;
-address StubRoutines::x86::_SC_3_adr = NULL;
-address StubRoutines::x86::_SC_1_adr = NULL;
-address StubRoutines::x86::_PI_INV_TABLE_adr = NULL;
-address StubRoutines::x86::_PI_4_adr = NULL;
-address StubRoutines::x86::_PI32INV_adr = NULL;
-address StubRoutines::x86::_SIGN_MASK_adr = NULL;
-address StubRoutines::x86::_P_1_adr = NULL;
-address StubRoutines::x86::_P_3_adr = NULL;
-address StubRoutines::x86::_NEG_ZERO_adr = NULL;
-
-//tables common for sincos and tancot
-address StubRoutines::x86::_L_2il0floatpacket_0_adr = NULL;
-address StubRoutines::x86::_Pi4Inv_adr = NULL;
-address StubRoutines::x86::_Pi4x3_adr = NULL;
-address StubRoutines::x86::_Pi4x4_adr = NULL;
-address StubRoutines::x86::_ones_adr = NULL;
-
-uint64_t StubRoutines::x86::_crc_by128_masks[] =
+const uint64_t StubRoutines::x86::_crc_by128_masks[] =
 {
   /* The fields in this structure are arranged so that they can be
    * picked up two at a time with 128-bit loads.
@@ -146,7 +127,7 @@ uint64_t StubRoutines::x86::_crc_by128_masks[] =
 /**
  *  crc_table[] from jdk/src/share/native/java/util/zip/zlib-1.2.5/crc32.h
  */
-juint StubRoutines::x86::_crc_table[] =
+const juint StubRoutines::x86::_crc_table[] =
 {
     0x00000000UL, 0x77073096UL, 0xee0e612cUL, 0x990951baUL, 0x076dc419UL,
     0x706af48fUL, 0xe963a535UL, 0x9e6495a3UL, 0x0edb8832UL, 0x79dcb8a4UL,
@@ -203,7 +184,7 @@ juint StubRoutines::x86::_crc_table[] =
 };
 
 #ifdef _LP64
-juint StubRoutines::x86::_crc_table_avx512[] =
+const juint StubRoutines::x86::_crc_table_avx512[] =
 {
     0xe95c1271UL, 0x00000000UL, 0xce3371cbUL, 0x00000000UL,
     0xccaa009eUL, 0x00000000UL, 0x751997d0UL, 0x00000001UL,
@@ -220,39 +201,74 @@ juint StubRoutines::x86::_crc_table_avx512[] =
     0x00000000UL, 0x00000000UL, 0x00000000UL, 0x00000000UL
 };
 
-juint StubRoutines::x86::_crc_by128_masks_avx512[] =
+const juint StubRoutines::x86::_crc32c_table_avx512[] =
+{
+    0xb9e02b86UL, 0x00000000UL, 0xdcb17aa4UL, 0x00000000UL,
+    0x493c7d27UL, 0x00000000UL, 0xc1068c50UL, 0x0000000eUL,
+    0x06e38d70UL, 0x00000002UL, 0x6992cea2UL, 0x00000000UL,
+    0x493c7d27UL, 0x00000000UL, 0xdd45aab8UL, 0x00000000UL,
+    0xdea713f0UL, 0x00000000UL, 0x05ec76f0UL, 0x00000001UL,
+    0x47db8317UL, 0x00000000UL, 0x2ad91c30UL, 0x00000000UL,
+    0x0715ce53UL, 0x00000000UL, 0xc49f4f67UL, 0x00000000UL,
+    0x39d3b296UL, 0x00000000UL, 0x083a6eecUL, 0x00000000UL,
+    0x9e4addf8UL, 0x00000000UL, 0x740eef02UL, 0x00000000UL,
+    0xddc0152bUL, 0x00000000UL, 0x1c291d04UL, 0x00000000UL,
+    0xba4fc28eUL, 0x00000000UL, 0x3da6d0cbUL, 0x00000000UL,
+    0x493c7d27UL, 0x00000000UL, 0xc1068c50UL, 0x0000000eUL,
+    0x00000000UL, 0x00000000UL, 0x00000000UL, 0x00000000UL
+};
+
+const juint StubRoutines::x86::_crc_by128_masks_avx512[] =
 {
     0xffffffffUL, 0xffffffffUL, 0x00000000UL, 0x00000000UL,
     0x00000000UL, 0xffffffffUL, 0xffffffffUL, 0xffffffffUL,
     0x80808080UL, 0x80808080UL, 0x80808080UL, 0x80808080UL
 };
 
-juint StubRoutines::x86::_shuf_table_crc32_avx512[] =
+const juint StubRoutines::x86::_shuf_table_crc32_avx512[] =
 {
     0x83828100UL, 0x87868584UL, 0x8b8a8988UL, 0x8f8e8d8cUL,
     0x03020100UL, 0x07060504UL, 0x0b0a0908UL, 0x000e0d0cUL
 };
-
-juint StubRoutines::x86::_adler32_ascale_table[] =
-{
-    0x00000000UL, 0x00000001UL, 0x00000002UL, 0x00000003UL,
-    0x00000004UL, 0x00000005UL, 0x00000006UL, 0x00000007UL
-};
-
-juint StubRoutines::x86::_adler32_shuf0_table[] =
-{
-    0xFFFFFF00UL, 0xFFFFFF01UL, 0xFFFFFF02UL, 0xFFFFFF03UL,
-    0xFFFFFF04UL, 0xFFFFFF05UL, 0xFFFFFF06UL, 0xFFFFFF07UL
-};
-
-juint StubRoutines::x86::_adler32_shuf1_table[] =
-{
-    0xFFFFFF08UL, 0xFFFFFF09, 0xFFFFFF0AUL, 0xFFFFFF0BUL,
-    0xFFFFFF0CUL, 0xFFFFFF0D, 0xFFFFFF0EUL, 0xFFFFFF0FUL
-};
-
 #endif // _LP64
 
+const jint StubRoutines::x86::_arrays_hashcode_powers_of_31[] =
+{
+     2111290369,
+    -2010103841,
+      350799937,
+       11316127,
+      693101697,
+     -254736545,
+      961614017,
+       31019807,
+    -2077209343,
+      -67006753,
+     1244764481,
+    -2038056289,
+      211350913,
+     -408824225,
+     -844471871,
+     -997072353,
+     1353309697,
+     -510534177,
+     1507551809,
+     -505558625,
+     -293403007,
+      129082719,
+    -1796951359,
+     -196513505,
+    -1807454463,
+     1742810335,
+      887503681,
+       28629151,
+         923521,
+          29791,
+            961,
+             31,
+              1,
+};
+
 #define D 32
 #define P 0x82F63B78 // Reflection of Castagnoli (0x11EDC6F41)
 
@@ -262,7 +278,7 @@ uint32_t _crc32c_pow_2k_table[TILL_CYCLE]; // because _crc32c_pow_2k_table[TILL_
 // A. Kadatch and B. Jenkins / Everything we know about CRC but afraid to forget September 3, 2010 8
 // Listing 1: Multiplication of normalized polynomials
 // "a" and "b" occupy D least significant bits.
-uint32_t crc32c_multiply(uint32_t a, uint32_t b) {
+static uint32_t crc32c_multiply(uint32_t a, uint32_t b) {
   uint32_t product = 0;
   uint32_t b_pow_x_table[D + 1]; // b_pow_x_table[k] = (b * x**k) mod P
   b_pow_x_table[0] = b;
@@ -286,7 +302,7 @@ uint32_t crc32c_multiply(uint32_t a, uint32_t b) {
 #undef P
 
 // A. Kadatch and B. Jenkins / Everything we know about CRC but afraid to forget September 3, 2010 9
-void crc32c_init_pow_2k(void) {
+static void crc32c_init_pow_2k(void) {
   // _crc32c_pow_2k_table(0) =
   // x^(2^k) mod P(x) = x mod P(x) = x
   // Since we are operating on a reflected values
@@ -301,7 +317,7 @@ void crc32c_init_pow_2k(void) {
 }
 
 // x^N mod P(x)
-uint32_t crc32c_f_pow_n(uint32_t n) {
+static uint32_t crc32c_f_pow_n(uint32_t n) {
   //            result = 1 (polynomial)
   uint32_t one, result = 0x80000000, i = 0;
 
@@ -357,7 +373,7 @@ void StubRoutines::x86::generate_CRC32C_table(bool is_pclmulqdq_table_supported)
   }
 }
 
-ATTRIBUTE_ALIGNED(64) juint StubRoutines::x86::_k256[] =
+ATTRIBUTE_ALIGNED(64) const juint StubRoutines::x86::_k256[] =
 {
     0x428a2f98UL, 0x71374491UL, 0xb5c0fbcfUL, 0xe9b5dba5UL,
     0x3956c25bUL, 0x59f111f1UL, 0x923f82a4UL, 0xab1c5ed5UL,
@@ -383,7 +399,7 @@ ATTRIBUTE_ALIGNED(64) juint StubRoutines::x86::_k256[] =
 ATTRIBUTE_ALIGNED(64) juint StubRoutines::x86::_k256_W[2*sizeof(StubRoutines::x86::_k256)];
 
 // used in MacroAssembler::sha512_AVX2
-ATTRIBUTE_ALIGNED(64) julong StubRoutines::x86::_k512_W[] =
+ATTRIBUTE_ALIGNED(64) const julong StubRoutines::x86::_k512_W[] =
 {
     0x428a2f98d728ae22ULL, 0x7137449123ef65cdULL,
     0xb5c0fbcfec4d3b2fULL, 0xe9b5dba58189dbbcULL,
