@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2024, Oracle and/or its affiliates. All rights reserved.
  * Copyright 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -346,9 +346,6 @@ int ZeroInterpreter::native_entry(Method* method, intptr_t UNUSED, TRAPS) {
           success = false;
         }
       }
-      if (success) {
-        THREAD->inc_held_monitor_count();
-      }
     }
     if (!success) {
       CALL_VM_NOCHECK(InterpreterRuntime::monitorenter(thread, monitor));
@@ -485,26 +482,27 @@ int ZeroInterpreter::native_entry(Method* method, intptr_t UNUSED, TRAPS) {
 
   // Unlock if necessary
   if (monitor) {
-    BasicLock *lock = monitor->lock();
-    markWord header = lock->displaced_header();
-    oop rcvr = monitor->obj();
-    monitor->set_obj(nullptr);
-
-    bool dec_monitor_count = true;
-    if (header.to_pointer() != nullptr) {
-      markWord old_header = markWord::encode(lock);
-      if (rcvr->cas_set_mark(header, old_header) != old_header) {
-        monitor->set_obj(rcvr);
-        dec_monitor_count = false;
-        InterpreterRuntime::monitorexit(monitor);
+    bool success = false;
+    if (LockingMode == LM_LEGACY) {
+      BasicLock* lock = monitor->lock();
+      oop rcvr = monitor->obj();
+      monitor->set_obj(nullptr);
+      success = true;
+      markWord header = lock->displaced_header();
+      if (header.to_pointer() != nullptr) { // Check for recursive lock
+        markWord old_header = markWord::encode(lock);
+        if (rcvr->cas_set_mark(header, old_header) != old_header) {
+          monitor->set_obj(rcvr);
+          success = false;
+        }
       }
     }
-    if (dec_monitor_count) {
-      THREAD->dec_held_monitor_count();
+    if (!success) {
+      InterpreterRuntime::monitorexit(monitor);
     }
   }
 
- unwind_and_return:
+  unwind_and_return:
 
   // Unwind the current activation
   thread->pop_zero_frame();
