@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -53,13 +53,13 @@ LIR_Opr LIR_OprFact::value_type(ValueType* type) {
   switch (tag) {
   case metaDataTag : {
     ClassConstant* c = type->as_ClassConstant();
-    if (c != NULL && !c->value()->is_loaded()) {
-      return LIR_OprFact::metadataConst(NULL);
-    } else if (c != NULL) {
+    if (c != nullptr && !c->value()->is_loaded()) {
+      return LIR_OprFact::metadataConst(nullptr);
+    } else if (c != nullptr) {
       return LIR_OprFact::metadataConst(c->value()->constant_encoding());
     } else {
       MethodConstant* m = type->as_MethodConstant();
-      assert (m != NULL, "not a class or a method?");
+      assert (m != nullptr, "not a class or a method?");
       return LIR_OprFact::metadataConst(m->value()->constant_encoding());
     }
   }
@@ -194,16 +194,14 @@ void LIR_Op2::verify() const {
              "can't produce oops from arith");
   }
 
-  if (TwoOperandLIRForm) {
+  if (two_operand_lir_form) {
 
-#ifdef ASSERT
     bool threeOperandForm = false;
 #ifdef S390
     // There are 3 operand shifts on S390 (see LIR_Assembler::shift_op()).
     threeOperandForm =
       code() == lir_shl ||
       ((code() == lir_shr || code() == lir_ushr) && (result_opr()->is_double_cpu() || in_opr1()->type() == T_OBJECT));
-#endif
 #endif
 
     switch (code()) {
@@ -236,32 +234,32 @@ void LIR_Op2::verify() const {
 
 
 LIR_OpBranch::LIR_OpBranch(LIR_Condition cond, BlockBegin* block)
-  : LIR_Op2(lir_branch, cond, LIR_OprFact::illegalOpr, LIR_OprFact::illegalOpr, (CodeEmitInfo*)NULL)
+  : LIR_Op2(lir_branch, cond, LIR_OprFact::illegalOpr, LIR_OprFact::illegalOpr, (CodeEmitInfo*)nullptr)
   , _label(block->label())
   , _block(block)
-  , _ublock(NULL)
-  , _stub(NULL) {
+  , _ublock(nullptr)
+  , _stub(nullptr) {
 }
 
 LIR_OpBranch::LIR_OpBranch(LIR_Condition cond, CodeStub* stub) :
-  LIR_Op2(lir_branch, cond, LIR_OprFact::illegalOpr, LIR_OprFact::illegalOpr, (CodeEmitInfo*)NULL)
+  LIR_Op2(lir_branch, cond, LIR_OprFact::illegalOpr, LIR_OprFact::illegalOpr, (CodeEmitInfo*)nullptr)
   , _label(stub->entry())
-  , _block(NULL)
-  , _ublock(NULL)
+  , _block(nullptr)
+  , _ublock(nullptr)
   , _stub(stub) {
 }
 
 LIR_OpBranch::LIR_OpBranch(LIR_Condition cond, BlockBegin* block, BlockBegin* ublock)
-  : LIR_Op2(lir_cond_float_branch, cond, LIR_OprFact::illegalOpr, LIR_OprFact::illegalOpr, (CodeEmitInfo*)NULL)
+  : LIR_Op2(lir_cond_float_branch, cond, LIR_OprFact::illegalOpr, LIR_OprFact::illegalOpr, (CodeEmitInfo*)nullptr)
   , _label(block->label())
   , _block(block)
   , _ublock(ublock)
-  , _stub(NULL)
+  , _stub(nullptr)
 {
 }
 
 void LIR_OpBranch::change_block(BlockBegin* b) {
-  assert(_block != NULL, "must have old block");
+  assert(_block != nullptr, "must have old block");
   assert(_block->label() == label(), "must be equal");
 
   _block = b;
@@ -269,7 +267,7 @@ void LIR_OpBranch::change_block(BlockBegin* b) {
 }
 
 void LIR_OpBranch::change_ublock(BlockBegin* b) {
-  assert(_ublock != NULL, "must have old block");
+  assert(_ublock != nullptr, "must have old block");
   _ublock = b;
 }
 
@@ -291,25 +289,25 @@ LIR_OpTypeCheck::LIR_OpTypeCheck(LIR_Code code, LIR_Opr result, LIR_Opr object,
                                  bool fast_check, CodeEmitInfo* info_for_exception, CodeEmitInfo* info_for_patch,
                                  CodeStub* stub)
 
-  : LIR_Op(code, result, NULL)
+  : LIR_Op(code, result, nullptr)
   , _object(object)
   , _array(LIR_OprFact::illegalOpr)
   , _klass(klass)
   , _tmp1(tmp1)
   , _tmp2(tmp2)
   , _tmp3(tmp3)
-  , _fast_check(fast_check)
   , _info_for_patch(info_for_patch)
   , _info_for_exception(info_for_exception)
   , _stub(stub)
-  , _profiled_method(NULL)
+  , _profiled_method(nullptr)
   , _profiled_bci(-1)
   , _should_profile(false)
+  , _fast_check(fast_check)
 {
   if (code == lir_checkcast) {
-    assert(info_for_exception != NULL, "checkcast throws exceptions");
+    assert(info_for_exception != nullptr, "checkcast throws exceptions");
   } else if (code == lir_instanceof) {
-    assert(info_for_exception == NULL, "instanceof throws no exceptions");
+    assert(info_for_exception == nullptr, "instanceof throws no exceptions");
   } else {
     ShouldNotReachHere();
   }
@@ -318,24 +316,24 @@ LIR_OpTypeCheck::LIR_OpTypeCheck(LIR_Code code, LIR_Opr result, LIR_Opr object,
 
 
 LIR_OpTypeCheck::LIR_OpTypeCheck(LIR_Code code, LIR_Opr object, LIR_Opr array, LIR_Opr tmp1, LIR_Opr tmp2, LIR_Opr tmp3, CodeEmitInfo* info_for_exception)
-  : LIR_Op(code, LIR_OprFact::illegalOpr, NULL)
+  : LIR_Op(code, LIR_OprFact::illegalOpr, nullptr)
   , _object(object)
   , _array(array)
-  , _klass(NULL)
+  , _klass(nullptr)
   , _tmp1(tmp1)
   , _tmp2(tmp2)
   , _tmp3(tmp3)
-  , _fast_check(false)
-  , _info_for_patch(NULL)
+  , _info_for_patch(nullptr)
   , _info_for_exception(info_for_exception)
-  , _stub(NULL)
-  , _profiled_method(NULL)
+  , _stub(nullptr)
+  , _profiled_method(nullptr)
   , _profiled_bci(-1)
   , _should_profile(false)
+  , _fast_check(false)
 {
   if (code == lir_store_check) {
     _stub = new ArrayStoreExceptionStub(object, info_for_exception);
-    assert(info_for_exception != NULL, "store_check throws exceptions");
+    assert(info_for_exception != nullptr, "store_check throws exceptions");
   } else {
     ShouldNotReachHere();
   }
@@ -353,11 +351,19 @@ LIR_OpArrayCopy::LIR_OpArrayCopy(LIR_Opr src, LIR_Opr src_pos, LIR_Opr dst, LIR_
   , _tmp(tmp)
   , _expected_type(expected_type)
   , _flags(flags) {
+#if defined(X86) || defined(AARCH64) || defined(S390) || defined(RISCV) || defined(PPC64)
+  if (expected_type != nullptr && flags == 0) {
+    _stub = nullptr;
+  } else {
+    _stub = new ArrayCopyStub(this);
+  }
+#else
   _stub = new ArrayCopyStub(this);
+#endif
 }
 
 LIR_OpUpdateCRC32::LIR_OpUpdateCRC32(LIR_Opr crc, LIR_Opr val, LIR_Opr res)
-  : LIR_Op(lir_updatecrc32, res, NULL)
+  : LIR_Op(lir_updatecrc32, res, nullptr)
   , _crc(crc)
   , _val(val) {
 }
@@ -409,8 +415,8 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_membar_storeload:         // result and info always invalid
     case lir_on_spin_wait:
     {
-      assert(op->as_Op0() != NULL, "must be");
-      assert(op->_info == NULL, "info not used by this instruction");
+      assert(op->as_Op0() != nullptr, "must be");
+      assert(op->_info == nullptr, "info not used by this instruction");
       assert(op->_result->is_illegal(), "not used");
       break;
     }
@@ -420,8 +426,8 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_osr_entry:                // may have result, info always invalid
     case lir_get_thread:               // may have result, info always invalid
     {
-      assert(op->as_Op0() != NULL, "must be");
-      if (op->_info != NULL)           do_info(op->_info);
+      assert(op->as_Op0() != nullptr, "must be");
+      if (op->_info != nullptr)           do_info(op->_info);
       if (op->_result->is_valid())     do_output(op->_result);
       break;
     }
@@ -430,8 +436,8 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 // LIR_OpLabel
     case lir_label:                    // result and info always invalid
     {
-      assert(op->as_OpLabel() != NULL, "must be");
-      assert(op->_info == NULL, "info not used by this instruction");
+      assert(op->as_OpLabel() != nullptr, "must be");
+      assert(op->_info == nullptr, "info not used by this instruction");
       assert(op->_result->is_illegal(), "not used");
       break;
     }
@@ -447,7 +453,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_null_check:     // input and info always valid, result always invalid
     case lir_move:           // input and result always valid, may have info
     {
-      assert(op->as_Op1() != NULL, "must be");
+      assert(op->as_Op1() != nullptr, "must be");
       LIR_Op1* op1 = (LIR_Op1*)op;
 
       if (op1->_info)                  do_info(op1->_info);
@@ -459,23 +465,23 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 
     case lir_return:
     {
-      assert(op->as_OpReturn() != NULL, "must be");
+      assert(op->as_OpReturn() != nullptr, "must be");
       LIR_OpReturn* op_ret = (LIR_OpReturn*)op;
 
       if (op_ret->_info)               do_info(op_ret->_info);
       if (op_ret->_opr->is_valid())    do_input(op_ret->_opr);
       if (op_ret->_result->is_valid()) do_output(op_ret->_result);
-      if (op_ret->stub() != NULL)      do_stub(op_ret->stub());
+      if (op_ret->stub() != nullptr)      do_stub(op_ret->stub());
 
       break;
     }
 
     case lir_safepoint:
     {
-      assert(op->as_Op1() != NULL, "must be");
+      assert(op->as_Op1() != nullptr, "must be");
       LIR_Op1* op1 = (LIR_Op1*)op;
 
-      assert(op1->_info != NULL, "");  do_info(op1->_info);
+      assert(op1->_info != nullptr, "");  do_info(op1->_info);
       if (op1->_opr->is_valid())       do_temp(op1->_opr); // safepoints on SPARC need temporary register
       assert(op1->_result->is_illegal(), "safepoint does not produce value");
 
@@ -485,10 +491,10 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 // LIR_OpConvert;
     case lir_convert:        // input and result always valid, info always invalid
     {
-      assert(op->as_OpConvert() != NULL, "must be");
+      assert(op->as_OpConvert() != nullptr, "must be");
       LIR_OpConvert* opConvert = (LIR_OpConvert*)op;
 
-      assert(opConvert->_info == NULL, "must be");
+      assert(opConvert->_info == nullptr, "must be");
       if (opConvert->_opr->is_valid())       do_input(opConvert->_opr);
       if (opConvert->_result->is_valid())    do_output(opConvert->_result);
       do_stub(opConvert->_stub);
@@ -500,7 +506,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_branch:                   // may have info, input and result register always invalid
     case lir_cond_float_branch:        // may have info, input and result register always invalid
     {
-      assert(op->as_OpBranch() != NULL, "must be");
+      assert(op->as_OpBranch() != nullptr, "must be");
       LIR_OpBranch* opBranch = (LIR_OpBranch*)op;
 
       assert(opBranch->_tmp1->is_illegal() && opBranch->_tmp2->is_illegal() &&
@@ -510,9 +516,9 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
       if (opBranch->_opr1->is_valid()) do_input(opBranch->_opr1);
       if (opBranch->_opr2->is_valid()) do_input(opBranch->_opr2);
 
-      if (opBranch->_info != NULL)     do_info(opBranch->_info);
+      if (opBranch->_info != nullptr)  do_info(opBranch->_info);
       assert(opBranch->_result->is_illegal(), "not used");
-      if (opBranch->_stub != NULL)     opBranch->stub()->visit(this);
+      if (opBranch->_stub != nullptr)  opBranch->stub()->visit(this);
 
       break;
     }
@@ -521,7 +527,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 // LIR_OpAllocObj
     case lir_alloc_object:
     {
-      assert(op->as_OpAllocObj() != NULL, "must be");
+      assert(op->as_OpAllocObj() != nullptr, "must be");
       LIR_OpAllocObj* opAllocObj = (LIR_OpAllocObj*)op;
 
       if (opAllocObj->_info)                     do_info(opAllocObj->_info);
@@ -533,17 +539,17 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
       if (opAllocObj->_tmp3->is_valid())         do_temp(opAllocObj->_tmp3);
       if (opAllocObj->_tmp4->is_valid())         do_temp(opAllocObj->_tmp4);
       if (opAllocObj->_result->is_valid())       do_output(opAllocObj->_result);
-                                                 do_stub(opAllocObj->_stub);
+      if (opAllocObj->_stub != nullptr)          do_stub(opAllocObj->_stub);
       break;
     }
 
 
 // LIR_OpRoundFP;
     case lir_roundfp: {
-      assert(op->as_OpRoundFP() != NULL, "must be");
+      assert(op->as_OpRoundFP() != nullptr, "must be");
       LIR_OpRoundFP* opRoundFP = (LIR_OpRoundFP*)op;
 
-      assert(op->_info == NULL, "info not used by this instruction");
+      assert(op->_info == nullptr, "info not used by this instruction");
       assert(opRoundFP->_tmp->is_illegal(), "not used");
       do_input(opRoundFP->_opr);
       do_output(opRoundFP->_result);
@@ -563,6 +569,8 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_sqrt:
     case lir_abs:
     case lir_neg:
+    case lir_f2hf:
+    case lir_hf2f:
     case lir_logic_and:
     case lir_logic_or:
     case lir_logic_xor:
@@ -573,7 +581,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_xchg:
     case lir_assert:
     {
-      assert(op->as_Op2() != NULL, "must be");
+      assert(op->as_Op2() != nullptr, "must be");
       LIR_Op2* op2 = (LIR_Op2*)op;
       assert(op2->_tmp2->is_illegal() && op2->_tmp3->is_illegal() &&
              op2->_tmp4->is_illegal() && op2->_tmp5->is_illegal(), "not used");
@@ -598,10 +606,10 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     // to the result operand, otherwise the backend fails
     case lir_cmove:
     {
-      assert(op->as_Op4() != NULL, "must be");
+      assert(op->as_Op4() != nullptr, "must be");
       LIR_Op4* op4 = (LIR_Op4*)op;
 
-      assert(op4->_info == NULL && op4->_tmp1->is_illegal() && op4->_tmp2->is_illegal() &&
+      assert(op4->_info == nullptr && op4->_tmp1->is_illegal() && op4->_tmp2->is_illegal() &&
              op4->_tmp3->is_illegal() && op4->_tmp4->is_illegal() && op4->_tmp5->is_illegal(), "not used");
       assert(op4->_opr1->is_valid() && op4->_opr2->is_valid() && op4->_result->is_valid(), "used");
 
@@ -621,10 +629,10 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_mul:
     case lir_div:
     {
-      assert(op->as_Op2() != NULL, "must be");
+      assert(op->as_Op2() != nullptr, "must be");
       LIR_Op2* op2 = (LIR_Op2*)op;
 
-      assert(op2->_info == NULL, "not used");
+      assert(op2->_info == nullptr, "not used");
       assert(op2->_opr1->is_valid(), "used");
       assert(op2->_opr2->is_valid(), "used");
       assert(op2->_result->is_valid(), "used");
@@ -640,7 +648,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     }
 
     case lir_throw: {
-      assert(op->as_Op2() != NULL, "must be");
+      assert(op->as_Op2() != nullptr, "must be");
       LIR_Op2* op2 = (LIR_Op2*)op;
 
       if (op2->_info)                     do_info(op2->_info);
@@ -654,10 +662,10 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     }
 
     case lir_unwind: {
-      assert(op->as_Op1() != NULL, "must be");
+      assert(op->as_Op1() != nullptr, "must be");
       LIR_Op1* op1 = (LIR_Op1*)op;
 
-      assert(op1->_info == NULL, "no info");
+      assert(op1->_info == nullptr, "no info");
       assert(op1->_opr->is_valid(), "exception oop");         do_input(op1->_opr);
       assert(op1->_result->is_illegal(), "no result");
 
@@ -667,7 +675,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 // LIR_Op3
     case lir_idiv:
     case lir_irem: {
-      assert(op->as_Op3() != NULL, "must be");
+      assert(op->as_Op3() != nullptr, "must be");
       LIR_Op3* op3= (LIR_Op3*)op;
 
       if (op3->_info)                     do_info(op3->_info);
@@ -686,9 +694,9 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 
     case lir_fmad:
     case lir_fmaf: {
-      assert(op->as_Op3() != NULL, "must be");
+      assert(op->as_Op3() != nullptr, "must be");
       LIR_Op3* op3= (LIR_Op3*)op;
-      assert(op3->_info == NULL, "no info");
+      assert(op3->_info == nullptr, "no info");
       do_input(op3->_opr1);
       do_input(op3->_opr2);
       do_input(op3->_opr3);
@@ -702,7 +710,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_icvirtual_call:
     case lir_dynamic_call: {
       LIR_OpJavaCall* opJavaCall = op->as_OpJavaCall();
-      assert(opJavaCall != NULL, "must be");
+      assert(opJavaCall != nullptr, "must be");
 
       if (opJavaCall->_receiver->is_valid())     do_input(opJavaCall->_receiver);
 
@@ -729,7 +737,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 
 // LIR_OpRTCall
     case lir_rtcall: {
-      assert(op->as_OpRTCall() != NULL, "must be");
+      assert(op->as_OpRTCall() != nullptr, "must be");
       LIR_OpRTCall* opRTCall = (LIR_OpRTCall*)op;
 
       // only visit register parameters
@@ -750,7 +758,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 
 // LIR_OpArrayCopy
     case lir_arraycopy: {
-      assert(op->as_OpArrayCopy() != NULL, "must be");
+      assert(op->as_OpArrayCopy() != nullptr, "must be");
       LIR_OpArrayCopy* opArrayCopy = (LIR_OpArrayCopy*)op;
 
       assert(opArrayCopy->_result->is_illegal(), "unused");
@@ -771,13 +779,13 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 
 // LIR_OpUpdateCRC32
     case lir_updatecrc32: {
-      assert(op->as_OpUpdateCRC32() != NULL, "must be");
+      assert(op->as_OpUpdateCRC32() != nullptr, "must be");
       LIR_OpUpdateCRC32* opUp = (LIR_OpUpdateCRC32*)op;
 
       assert(opUp->_crc->is_valid(), "used");          do_input(opUp->_crc);     do_temp(opUp->_crc);
       assert(opUp->_val->is_valid(), "used");          do_input(opUp->_val);     do_temp(opUp->_val);
       assert(opUp->_result->is_valid(), "used");       do_output(opUp->_result);
-      assert(opUp->_info == NULL, "no info for LIR_OpUpdateCRC32");
+      assert(opUp->_info == nullptr, "no info for LIR_OpUpdateCRC32");
 
       break;
     }
@@ -786,7 +794,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 // LIR_OpLock
     case lir_lock:
     case lir_unlock: {
-      assert(op->as_OpLock() != NULL, "must be");
+      assert(op->as_OpLock() != nullptr, "must be");
       LIR_OpLock* opLock = (LIR_OpLock*)op;
 
       if (opLock->_info)                          do_info(opLock->_info);
@@ -808,7 +816,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 
 // LIR_OpDelay
     case lir_delay_slot: {
-      assert(op->as_OpDelay() != NULL, "must be");
+      assert(op->as_OpDelay() != nullptr, "must be");
       LIR_OpDelay* opDelay = (LIR_OpDelay*)op;
 
       visit(opDelay->delay_op());
@@ -819,7 +827,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_instanceof:
     case lir_checkcast:
     case lir_store_check: {
-      assert(op->as_OpTypeCheck() != NULL, "must be");
+      assert(op->as_OpTypeCheck() != nullptr, "must be");
       LIR_OpTypeCheck* opTypeCheck = (LIR_OpTypeCheck*)op;
 
       if (opTypeCheck->_info_for_exception)       do_info(opTypeCheck->_info_for_exception);
@@ -833,7 +841,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
       if (opTypeCheck->_tmp2->is_valid())         do_temp(opTypeCheck->_tmp2);
       if (opTypeCheck->_tmp3->is_valid())         do_temp(opTypeCheck->_tmp3);
       if (opTypeCheck->_result->is_valid())       do_output(opTypeCheck->_result);
-                                                  do_stub(opTypeCheck->_stub);
+      if (opTypeCheck->_stub != nullptr)          do_stub(opTypeCheck->_stub);
       break;
     }
 
@@ -841,22 +849,19 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_cas_long:
     case lir_cas_obj:
     case lir_cas_int: {
-      assert(op->as_OpCompareAndSwap() != NULL, "must be");
-      LIR_OpCompareAndSwap* opCompareAndSwap = (LIR_OpCompareAndSwap*)op;
+      assert(op->as_OpCompareAndSwap() != nullptr, "must be");
+      LIR_OpCompareAndSwap* opCmpAndSwap = (LIR_OpCompareAndSwap*)op;
 
-      assert(opCompareAndSwap->_addr->is_valid(),      "used");
-      assert(opCompareAndSwap->_cmp_value->is_valid(), "used");
-      assert(opCompareAndSwap->_new_value->is_valid(), "used");
-      if (opCompareAndSwap->_info)                    do_info(opCompareAndSwap->_info);
-                                                      do_input(opCompareAndSwap->_addr);
-                                                      do_temp(opCompareAndSwap->_addr);
-                                                      do_input(opCompareAndSwap->_cmp_value);
-                                                      do_temp(opCompareAndSwap->_cmp_value);
-                                                      do_input(opCompareAndSwap->_new_value);
-                                                      do_temp(opCompareAndSwap->_new_value);
-      if (opCompareAndSwap->_tmp1->is_valid())        do_temp(opCompareAndSwap->_tmp1);
-      if (opCompareAndSwap->_tmp2->is_valid())        do_temp(opCompareAndSwap->_tmp2);
-      if (opCompareAndSwap->_result->is_valid())      do_output(opCompareAndSwap->_result);
+      if (opCmpAndSwap->_info)                              do_info(opCmpAndSwap->_info);
+      assert(opCmpAndSwap->_addr->is_valid(), "used");      do_input(opCmpAndSwap->_addr);
+                                                            do_temp(opCmpAndSwap->_addr);
+      assert(opCmpAndSwap->_cmp_value->is_valid(), "used"); do_input(opCmpAndSwap->_cmp_value);
+                                                            do_temp(opCmpAndSwap->_cmp_value);
+      assert(opCmpAndSwap->_new_value->is_valid(), "used"); do_input(opCmpAndSwap->_new_value);
+                                                            do_temp(opCmpAndSwap->_new_value);
+      if (opCmpAndSwap->_tmp1->is_valid())                  do_temp(opCmpAndSwap->_tmp1);
+      if (opCmpAndSwap->_tmp2->is_valid())                  do_temp(opCmpAndSwap->_tmp2);
+      if (opCmpAndSwap->_result->is_valid())                do_output(opCmpAndSwap->_result);
 
       break;
     }
@@ -864,18 +869,22 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 
 // LIR_OpAllocArray;
     case lir_alloc_array: {
-      assert(op->as_OpAllocArray() != NULL, "must be");
+      assert(op->as_OpAllocArray() != nullptr, "must be");
       LIR_OpAllocArray* opAllocArray = (LIR_OpAllocArray*)op;
 
       if (opAllocArray->_info)                        do_info(opAllocArray->_info);
-      if (opAllocArray->_klass->is_valid())           do_input(opAllocArray->_klass); do_temp(opAllocArray->_klass);
-      if (opAllocArray->_len->is_valid())             do_input(opAllocArray->_len);   do_temp(opAllocArray->_len);
+      if (opAllocArray->_klass->is_valid()) {         do_input(opAllocArray->_klass);
+                                                      do_temp(opAllocArray->_klass);
+                                            }
+      if (opAllocArray->_len->is_valid())   {         do_input(opAllocArray->_len);
+                                                      do_temp(opAllocArray->_len);
+                                            }
       if (opAllocArray->_tmp1->is_valid())            do_temp(opAllocArray->_tmp1);
       if (opAllocArray->_tmp2->is_valid())            do_temp(opAllocArray->_tmp2);
       if (opAllocArray->_tmp3->is_valid())            do_temp(opAllocArray->_tmp3);
       if (opAllocArray->_tmp4->is_valid())            do_temp(opAllocArray->_tmp4);
       if (opAllocArray->_result->is_valid())          do_output(opAllocArray->_result);
-                                                      do_stub(opAllocArray->_stub);
+      if (opAllocArray->_stub != nullptr)             do_stub(opAllocArray->_stub);
       break;
     }
 
@@ -883,7 +892,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
     case lir_load_klass:
     {
       LIR_OpLoadKlass* opLoadKlass = op->as_OpLoadKlass();
-      assert(opLoadKlass != NULL, "must be");
+      assert(opLoadKlass != nullptr, "must be");
 
       do_input(opLoadKlass->_obj);
       do_output(opLoadKlass->_result);
@@ -894,7 +903,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 
 // LIR_OpProfileCall:
     case lir_profile_call: {
-      assert(op->as_OpProfileCall() != NULL, "must be");
+      assert(op->as_OpProfileCall() != nullptr, "must be");
       LIR_OpProfileCall* opProfileCall = (LIR_OpProfileCall*)op;
 
       if (opProfileCall->_recv->is_valid())              do_temp(opProfileCall->_recv);
@@ -905,7 +914,7 @@ void LIR_OpVisitState::visit(LIR_Op* op) {
 
 // LIR_OpProfileType:
     case lir_profile_type: {
-      assert(op->as_OpProfileType() != NULL, "must be");
+      assert(op->as_OpProfileType() != nullptr, "must be");
       LIR_OpProfileType* opProfileType = (LIR_OpProfileType*)op;
 
       do_input(opProfileType->_mdp); do_temp(opProfileType->_mdp);
@@ -923,17 +932,17 @@ void LIR_Op::visit(LIR_OpVisitState* state) {
 }
 
 void LIR_OpVisitState::do_stub(CodeStub* stub) {
-  if (stub != NULL) {
+  if (stub != nullptr) {
     stub->visit(this);
   }
 }
 
 XHandlers* LIR_OpVisitState::all_xhandler() {
-  XHandlers* result = NULL;
+  XHandlers* result = nullptr;
 
   int i;
   for (i = 0; i < info_count(); i++) {
-    if (info_at(i)->exception_handlers() != NULL) {
+    if (info_at(i)->exception_handlers() != nullptr) {
       result = info_at(i)->exception_handlers();
       break;
     }
@@ -941,13 +950,13 @@ XHandlers* LIR_OpVisitState::all_xhandler() {
 
 #ifdef ASSERT
   for (i = 0; i < info_count(); i++) {
-    assert(info_at(i)->exception_handlers() == NULL ||
+    assert(info_at(i)->exception_handlers() == nullptr ||
            info_at(i)->exception_handlers() == result,
            "only one xhandler list allowed per LIR-operation");
   }
 #endif
 
-  if (result != NULL) {
+  if (result != nullptr) {
     return result;
   } else {
     return new XHandlers();
@@ -972,8 +981,8 @@ bool LIR_OpVisitState::no_operands(LIR_Op* op) {
 
 // LIR_OpReturn
 LIR_OpReturn::LIR_OpReturn(LIR_Opr opr) :
-    LIR_Op1(lir_return, opr, (CodeEmitInfo*)NULL /* info */),
-    _stub(NULL) {
+    LIR_Op1(lir_return, opr, (CodeEmitInfo*)nullptr /* info */),
+    _stub(nullptr) {
   if (VM_Version::supports_stack_watermark_barrier()) {
     _stub = new C1SafepointPollStub();
   }
@@ -996,7 +1005,10 @@ void LIR_OpLabel::emit_code(LIR_Assembler* masm) {
 
 void LIR_OpArrayCopy::emit_code(LIR_Assembler* masm) {
   masm->emit_arraycopy(this);
-  masm->append_code_stub(stub());
+  ArrayCopyStub* code_stub = stub();
+  if (code_stub != nullptr) {
+    masm->append_code_stub(code_stub);
+  }
 }
 
 void LIR_OpUpdateCRC32::emit_code(LIR_Assembler* masm) {
@@ -1025,7 +1037,7 @@ void LIR_OpBranch::emit_code(LIR_Assembler* masm) {
 
 void LIR_OpConvert::emit_code(LIR_Assembler* masm) {
   masm->emit_opConvert(this);
-  if (stub() != NULL) {
+  if (stub() != nullptr) {
     masm->append_code_stub(stub());
   }
 }
@@ -1095,7 +1107,7 @@ LIR_List::LIR_List(Compilation* compilation, BlockBegin* block)
   , _block(block)
 #endif
 #ifdef ASSERT
-  , _file(NULL)
+  , _file(nullptr)
   , _line(0)
 #endif
 #ifdef RISCV
@@ -1108,8 +1120,8 @@ LIR_List::LIR_List(Compilation* compilation, BlockBegin* block)
 #ifdef ASSERT
 void LIR_List::set_file_and_line(const char * file, int line) {
   const char * f = strrchr(file, '/');
-  if (f == NULL) f = strrchr(file, '\\');
-  if (f == NULL) {
+  if (f == nullptr) f = strrchr(file, '\\');
+  if (f == nullptr) {
     f = file;
   } else {
     f++;
@@ -1140,8 +1152,14 @@ void LIR_List::set_cmp_oprs(LIR_Op* op) {
       op->as_Op4()->set_in_opr3(_cmp_opr1);
       op->as_Op4()->set_in_opr4(_cmp_opr2);
       break;
+    case lir_cas_long:
+    case lir_cas_obj:
+    case lir_cas_int:
+      _cmp_opr1 = op->as_OpCompareAndSwap()->result_opr();
+      _cmp_opr2 = LIR_OprFact::intConst(0);
+      break;
 #if INCLUDE_ZGC
-    case lir_zloadbarrier_test:
+    case lir_xloadbarrier_test:
       _cmp_opr1 = FrameMap::as_opr(t1);
       _cmp_opr2 = LIR_OprFact::intConst(0);
       break;
@@ -1158,7 +1176,7 @@ void LIR_List::append(LIR_InsertionBuffer* buffer) {
 
   if (buffer->number_of_ops() > 0) {
     // increase size of instructions list
-    _operations.at_grow(n + buffer->number_of_ops() - 1, NULL);
+    _operations.at_grow(n + buffer->number_of_ops() - 1, nullptr);
     // insert ops from buffer into instructions list
     int op_index = buffer->number_of_ops() - 1;
     int ip_index = buffer->number_of_insertion_points() - 1;
@@ -1356,7 +1374,7 @@ void LIR_List::allocate_object(LIR_Opr dst, LIR_Opr t1, LIR_Opr t2, LIR_Opr t3,
                            stub));
 }
 
-void LIR_List::allocate_array(LIR_Opr dst, LIR_Opr len, LIR_Opr t1,LIR_Opr t2, LIR_Opr t3,LIR_Opr t4, BasicType type, LIR_Opr klass, CodeStub* stub) {
+void LIR_List::allocate_array(LIR_Opr dst, LIR_Opr len, LIR_Opr t1,LIR_Opr t2, LIR_Opr t3,LIR_Opr t4, BasicType type, LIR_Opr klass, CodeStub* stub, bool zero_array) {
   append(new LIR_OpAllocArray(
                            klass,
                            len,
@@ -1366,7 +1384,8 @@ void LIR_List::allocate_array(LIR_Opr dst, LIR_Opr len, LIR_Opr t1,LIR_Opr t2, L
                            t3,
                            t4,
                            type,
-                           stub));
+                           stub,
+                           zero_array));
 }
 
 void LIR_List::shift_left(LIR_Opr value, LIR_Opr count, LIR_Opr dst, LIR_Opr tmp) {
@@ -1423,7 +1442,7 @@ void LIR_List::unlock_object(LIR_Opr hdr, LIR_Opr obj, LIR_Opr lock, LIR_Opr scr
                     lock,
                     scratch,
                     stub,
-                    NULL));
+                    nullptr));
 }
 
 
@@ -1440,7 +1459,7 @@ void LIR_List::checkcast (LIR_Opr result, LIR_Opr object, ciKlass* klass,
                           ciMethod* profiled_method, int profiled_bci) {
   LIR_OpTypeCheck* c = new LIR_OpTypeCheck(lir_checkcast, result, object, klass,
                                            tmp1, tmp2, tmp3, fast_check, info_for_exception, info_for_patch, stub);
-  if (profiled_method != NULL) {
+  if (profiled_method != nullptr) {
     c->set_profiled_method(profiled_method);
     c->set_profiled_bci(profiled_bci);
     c->set_should_profile(true);
@@ -1449,8 +1468,8 @@ void LIR_List::checkcast (LIR_Opr result, LIR_Opr object, ciKlass* klass,
 }
 
 void LIR_List::instanceof(LIR_Opr result, LIR_Opr object, ciKlass* klass, LIR_Opr tmp1, LIR_Opr tmp2, LIR_Opr tmp3, bool fast_check, CodeEmitInfo* info_for_patch, ciMethod* profiled_method, int profiled_bci) {
-  LIR_OpTypeCheck* c = new LIR_OpTypeCheck(lir_instanceof, result, object, klass, tmp1, tmp2, tmp3, fast_check, NULL, info_for_patch, NULL);
-  if (profiled_method != NULL) {
+  LIR_OpTypeCheck* c = new LIR_OpTypeCheck(lir_instanceof, result, object, klass, tmp1, tmp2, tmp3, fast_check, nullptr, info_for_patch, nullptr);
+  if (profiled_method != nullptr) {
     c->set_profiled_method(profiled_method);
     c->set_profiled_bci(profiled_bci);
     c->set_should_profile(true);
@@ -1462,7 +1481,7 @@ void LIR_List::instanceof(LIR_Opr result, LIR_Opr object, ciKlass* klass, LIR_Op
 void LIR_List::store_check(LIR_Opr object, LIR_Opr array, LIR_Opr tmp1, LIR_Opr tmp2, LIR_Opr tmp3,
                            CodeEmitInfo* info_for_exception, ciMethod* profiled_method, int profiled_bci) {
   LIR_OpTypeCheck* c = new LIR_OpTypeCheck(lir_store_check, object, array, tmp1, tmp2, tmp3, info_for_exception);
-  if (profiled_method != NULL) {
+  if (profiled_method != nullptr) {
     c->set_profiled_method(profiled_method);
     c->set_profiled_bci(profiled_bci);
     c->set_should_profile(true);
@@ -1474,7 +1493,7 @@ void LIR_List::null_check(LIR_Opr opr, CodeEmitInfo* info, bool deoptimize_on_nu
   if (deoptimize_on_null) {
     // Emit an explicit null check and deoptimize if opr is null
     CodeStub* deopt = new DeoptimizeStub(info, Deoptimization::Reason_null_check, Deoptimization::Action_none);
-    cmp(lir_cond_equal, opr, LIR_OprFact::oopConst(NULL));
+    cmp(lir_cond_equal, opr, LIR_OprFact::oopConst(nullptr));
     branch(lir_cond_equal, deopt);
   } else {
     // Emit an implicit null check
@@ -1615,7 +1634,7 @@ static void print_block(BlockBegin* x) {
   if (x->is_set(BlockBegin::linear_scan_loop_end_flag))    tty->print("le ");
 
   // print block bci range
-  tty->print("[%d, %d] ", x->bci(), (end == NULL ? -1 : end->printable_bci()));
+  tty->print("[%d, %d] ", x->bci(), (end == nullptr ? -1 : end->printable_bci()));
 
   // print predecessors and successors
   if (x->number_of_preds() > 0) {
@@ -1625,7 +1644,7 @@ static void print_block(BlockBegin* x) {
     }
   }
 
-  if (end != NULL && x->number_of_sux() > 0) {
+  if (end != nullptr && x->number_of_sux() > 0) {
     tty->print("sux: ");
     for (int i = 0; i < x->number_of_sux(); i ++) {
       tty->print("B%d ", x->sux_at(i)->block_id());
@@ -1671,16 +1690,16 @@ void LIR_Op::print_on(outputStream* out) const {
   }
   out->print("%s ", name());
   print_instr(out);
-  if (info() != NULL) out->print(" [bci:%d]", info()->stack()->bci());
+  if (info() != nullptr) out->print(" [bci:%d]", info()->stack()->bci());
 #ifdef ASSERT
-  if (Verbose && _file != NULL) {
+  if (Verbose && _file != nullptr) {
     out->print(" (%s:%d)", _file, _line);
   }
 #endif
 }
 
 const char * LIR_Op::name() const {
-  const char* s = NULL;
+  const char* s = nullptr;
   switch(code()) {
      // LIR_Op0
      case lir_membar:                s = "membar";        break;
@@ -1730,6 +1749,8 @@ const char * LIR_Op::name() const {
      case lir_abs:                   s = "abs";           break;
      case lir_neg:                   s = "neg";           break;
      case lir_sqrt:                  s = "sqrt";          break;
+     case lir_f2hf:                  s = "f2hf";          break;
+     case lir_hf2f:                  s = "hf2f";          break;
      case lir_logic_and:             s = "logic_and";     break;
      case lir_logic_or:              s = "logic_or";      break;
      case lir_logic_xor:             s = "logic_xor";     break;
@@ -1881,17 +1902,17 @@ void LIR_OpBranch::print_instr(outputStream* out) const {
   print_condition(out, cond());             out->print(" ");
   in_opr1()->print(out); out->print(" ");
   in_opr2()->print(out); out->print(" ");
-  if (block() != NULL) {
+  if (block() != nullptr) {
     out->print("[B%d] ", block()->block_id());
-  } else if (stub() != NULL) {
+  } else if (stub() != nullptr) {
     out->print("[");
     stub()->print_name(out);
     out->print(": " INTPTR_FORMAT "]", p2i(stub()));
-    if (stub()->info() != NULL) out->print(" [bci:%d]", stub()->info()->stack()->bci());
+    if (stub()->info() != nullptr) out->print(" [bci:%d]", stub()->info()->stack()->bci());
   } else {
     out->print("[label:" INTPTR_FORMAT "] ", p2i(label()));
   }
-  if (ublock() != NULL) {
+  if (ublock() != nullptr) {
     out->print("unordered: [B%d] ", ublock()->block_id());
   }
 }
@@ -2000,7 +2021,7 @@ void LIR_OpTypeCheck::print_instr(outputStream* out) const {
   tmp2()->print(out);                    out->print(" ");
   tmp3()->print(out);                    out->print(" ");
   result_opr()->print(out);              out->print(" ");
-  if (info_for_exception() != NULL) out->print(" [bci:%d]", info_for_exception()->stack()->bci());
+  if (info_for_exception() != nullptr) out->print(" [bci:%d]", info_for_exception()->stack()->bci());
 }
 
 
@@ -2066,7 +2087,7 @@ void LIR_OpProfileCall::print_instr(outputStream* out) const {
 // LIR_OpProfileType
 void LIR_OpProfileType::print_instr(outputStream* out) const {
   out->print("exact = ");
-  if  (exact_klass() == NULL) {
+  if (exact_klass() == nullptr) {
     out->print("unknown");
   } else {
     exact_klass()->print_name_on(out);
