@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "asm/assembler.inline.hpp"
 #include "c1/c1_Compilation.hpp"
 #include "c1/c1_Instruction.hpp"
@@ -30,9 +29,8 @@
 #include "c1/c1_LIRAssembler.hpp"
 #include "c1/c1_MacroAssembler.hpp"
 #include "c1/c1_ValueStack.hpp"
-#include "ci/ciInstance.hpp"
+#include "compiler/compilerDefinitions.inline.hpp"
 #include "compiler/oopMap.hpp"
-#include "gc/shared/barrierSet.hpp"
 #include "runtime/os.hpp"
 #include "runtime/vm_version.hpp"
 
@@ -43,6 +41,7 @@ void LIR_Assembler::patching_epilog(PatchingStub* patch, LIR_PatchCode patch_cod
   while ((intx) _masm->pc() - (intx) patch->pc_start() < NativeGeneralJump::instruction_size) {
     _masm->nop();
   }
+  info->set_force_reexecute();
   patch->install(_masm, patch_code, obj, info);
   append_code_stub(patch);
 
@@ -75,6 +74,7 @@ void LIR_Assembler::patching_epilog(PatchingStub* patch, LIR_PatchCode patch_cod
       case Bytecodes::_getstatic:
       case Bytecodes::_ldc:
       case Bytecodes::_ldc_w:
+      case Bytecodes::_ldc2_w:
         break;
       default:
         ShouldNotReachHere();
@@ -102,12 +102,12 @@ PatchingStub::PatchID LIR_Assembler::patching_id(CodeEmitInfo* info) {
 
 LIR_Assembler::LIR_Assembler(Compilation* c):
    _masm(c->masm())
- , _bs(BarrierSet::barrier_set())
  , _compilation(c)
  , _frame_map(c->frame_map())
- , _current_block(NULL)
- , _pending_non_safepoint(NULL)
+ , _current_block(nullptr)
+ , _pending_non_safepoint(nullptr)
  , _pending_non_safepoint_offset(0)
+ , _immediate_oops_patched(0)
 {
   _slow_case_stubs = new CodeStubList();
 }
@@ -129,6 +129,7 @@ void LIR_Assembler::check_codespace() {
 
 
 void LIR_Assembler::append_code_stub(CodeStub* stub) {
+  _immediate_oops_patched += stub->nr_immediate_oops_patched();
   _slow_case_stubs->append(stub);
 }
 
@@ -144,7 +145,7 @@ void LIR_Assembler::emit_stubs(CodeStubList* stub_list) {
       stringStream st;
       s->print_name(&st);
       st.print(" slow case");
-      _masm->block_comment(st.as_string());
+      _masm->block_comment(st.freeze());
     }
 #endif
     s->emit_code(this);
@@ -192,13 +193,13 @@ void LIR_Assembler::emit_exception_entries(ExceptionInfoList* info_list) {
     for (int j = 0; j < handlers->length(); j++) {
       XHandler* handler = handlers->handler_at(j);
       assert(handler->lir_op_id() != -1, "handler not processed by LinearScan");
-      assert(handler->entry_code() == NULL ||
+      assert(handler->entry_code() == nullptr ||
              handler->entry_code()->instructions_list()->last()->code() == lir_branch ||
              handler->entry_code()->instructions_list()->last()->code() == lir_delay_slot, "last operation must be branch");
 
       if (handler->entry_pco() == -1) {
         // entry code not emitted yet
-        if (handler->entry_code() != NULL && handler->entry_code()->instructions_list()->length() > 1) {
+        if (handler->entry_code() != nullptr && handler->entry_code()->instructions_list()->length() > 1) {
           handler->set_entry_pco(code_offset());
           if (CommentedAssembly) {
             _masm->block_comment("Exception adapter block");
@@ -252,14 +253,14 @@ void LIR_Assembler::emit_block(BlockBegin* block) {
   }
 #endif /* PRODUCT */
 
-  assert(block->lir() != NULL, "must have LIR");
+  assert(block->lir() != nullptr, "must have LIR");
   X86_ONLY(assert(_masm->rsp_offset() == 0, "frame size should be fixed"));
 
 #ifndef PRODUCT
   if (CommentedAssembly) {
     stringStream st;
     st.print_cr(" block B%d [%d, %d]", block->block_id(), block->bci(), block->end()->printable_bci());
-    _masm->block_comment(st.as_string());
+    _masm->block_comment(st.freeze());
   }
 #endif
 
@@ -289,7 +290,7 @@ void LIR_Assembler::emit_lir_list(LIR_List* list) {
           (op->code() == lir_leal && op->as_Op1()->patch_code() != lir_patch_none)) {
         stringStream st;
         op->print_on(&st);
-        _masm->block_comment(st.as_string());
+        _masm->block_comment(st.freeze());
       }
     }
     if (PrintLIRWithAssembly) {
@@ -332,7 +333,7 @@ void LIR_Assembler::add_debug_info_for_branch(CodeEmitInfo* info) {
   int pc_offset = code_offset();
   flush_debug_info(pc_offset);
   info->record_debug_info(compilation()->debug_info_recorder(), pc_offset);
-  if (info->exception_handlers() != NULL) {
+  if (info->exception_handlers() != nullptr) {
     compilation()->add_exception_handlers_for_pco(pc_offset, info->exception_handlers());
   }
 }
@@ -341,28 +342,28 @@ void LIR_Assembler::add_debug_info_for_branch(CodeEmitInfo* info) {
 void LIR_Assembler::add_call_info(int pc_offset, CodeEmitInfo* cinfo) {
   flush_debug_info(pc_offset);
   cinfo->record_debug_info(compilation()->debug_info_recorder(), pc_offset);
-  if (cinfo->exception_handlers() != NULL) {
+  if (cinfo->exception_handlers() != nullptr) {
     compilation()->add_exception_handlers_for_pco(pc_offset, cinfo->exception_handlers());
   }
 }
 
 static ValueStack* debug_info(Instruction* ins) {
   StateSplit* ss = ins->as_StateSplit();
-  if (ss != NULL) return ss->state();
+  if (ss != nullptr) return ss->state();
   return ins->state_before();
 }
 
 void LIR_Assembler::process_debug_info(LIR_Op* op) {
   Instruction* src = op->source();
-  if (src == NULL)  return;
+  if (src == nullptr)  return;
   int pc_offset = code_offset();
   if (_pending_non_safepoint == src) {
     _pending_non_safepoint_offset = pc_offset;
     return;
   }
   ValueStack* vstack = debug_info(src);
-  if (vstack == NULL)  return;
-  if (_pending_non_safepoint != NULL) {
+  if (vstack == nullptr)  return;
+  if (_pending_non_safepoint != nullptr) {
     // Got some old debug info.  Get rid of it.
     if (debug_info(_pending_non_safepoint) == vstack) {
       _pending_non_safepoint_offset = pc_offset;
@@ -371,7 +372,7 @@ void LIR_Assembler::process_debug_info(LIR_Op* op) {
     if (_pending_non_safepoint_offset < pc_offset) {
       record_non_safepoint_debug_info();
     }
-    _pending_non_safepoint = NULL;
+    _pending_non_safepoint = nullptr;
   }
   // Remember the debug info.
   if (pc_offset > compilation()->debug_info_recorder()->last_pc_offset()) {
@@ -381,18 +382,18 @@ void LIR_Assembler::process_debug_info(LIR_Op* op) {
 }
 
 // Index caller states in s, where 0 is the oldest, 1 its callee, etc.
-// Return NULL if n is too large.
+// Return null if n is too large.
 // Returns the caller_bci for the next-younger state, also.
 static ValueStack* nth_oldest(ValueStack* s, int n, int& bci_result) {
   ValueStack* t = s;
   for (int i = 0; i < n; i++) {
-    if (t == NULL)  break;
+    if (t == nullptr)  break;
     t = t->caller_state();
   }
-  if (t == NULL)  return NULL;
+  if (t == nullptr)  return nullptr;
   for (;;) {
     ValueStack* tc = t->caller_state();
-    if (tc == NULL)  return s;
+    if (tc == nullptr)  return s;
     t = tc;
     bci_result = tc->bci();
     s = s->caller_state();
@@ -413,7 +414,7 @@ void LIR_Assembler::record_non_safepoint_debug_info() {
   for (int n = 0; ; n++) {
     int s_bci = bci;
     ValueStack* s = nth_oldest(vstack, n, s_bci);
-    if (s == NULL)  break;
+    if (s == nullptr)  break;
     IRScope* scope = s->scope();
     //Always pass false for reexecute since these ScopeDescs are never used for deopt
     methodHandle null_mh;
@@ -447,15 +448,20 @@ void LIR_Assembler::emit_rtcall(LIR_OpRTCall* op) {
   rt_call(op->result_opr(), op->addr(), op->arguments(), op->tmp(), op->info());
 }
 
-
 void LIR_Assembler::emit_call(LIR_OpJavaCall* op) {
   verify_oop_map(op->info());
 
   // must align calls sites, otherwise they can't be updated atomically
   align_call(op->code());
 
-  // emit the static call stub stuff out of line
-  emit_static_call_stub();
+  if (CodeBuffer::supports_shared_stubs() && op->method()->can_be_statically_bound()) {
+    // Calls of the same statically bound method can share
+    // a stub to the interpreter.
+    CodeBuffer::csize_t call_offset = pc() - _masm->code()->insts_begin();
+    _masm->code()->shared_stub_to_interp_for(op->method(), call_offset);
+  } else {
+    emit_static_call_stub();
+  }
   CHECK_BAILOUT();
 
   switch (op->code()) {
@@ -519,11 +525,22 @@ void LIR_Assembler::emit_op1(LIR_Op1* op) {
       break;
     }
 
+    case lir_abs:
+    case lir_sqrt:
+    case lir_f2hf:
+    case lir_hf2f:
+      intrinsic_op(op->code(), op->in_opr(), op->tmp_opr(), op->result_opr(), op);
+      break;
+
+    case lir_neg:
+      negate(op->in_opr(), op->result_opr(), op->tmp_opr());
+      break;
+
     case lir_return: {
-      assert(op->as_OpReturn() != NULL, "sanity");
+      assert(op->as_OpReturn() != nullptr, "sanity");
       LIR_OpReturn *ret_op = (LIR_OpReturn*)op;
       return_op(ret_op->in_opr(), ret_op->stub());
-      if (ret_op->stub() != NULL) {
+      if (ret_op->stub() != nullptr) {
         append_code_stub(ret_op->stub());
       }
       break;
@@ -590,7 +607,7 @@ void LIR_Assembler::emit_op1(LIR_Op1* op) {
 void LIR_Assembler::emit_op0(LIR_Op0* op) {
   switch (op->code()) {
     case lir_nop:
-      assert(op->info() == NULL, "not supported");
+      assert(op->info() == nullptr, "not supported");
       _masm->nop();
       break;
 
@@ -598,21 +615,23 @@ void LIR_Assembler::emit_op0(LIR_Op0* op) {
       Unimplemented();
       break;
 
-    case lir_std_entry:
+    case lir_std_entry: {
       // init offsets
       offsets()->set_value(CodeOffsets::OSR_Entry, _masm->offset());
-      _masm->align(CodeEntryAlignment);
       if (needs_icache(compilation()->method())) {
-        check_icache();
+        int offset = check_icache();
+        offsets()->set_value(CodeOffsets::Entry, offset);
       }
+      _masm->align(CodeEntryAlignment);
       offsets()->set_value(CodeOffsets::Verified_Entry, _masm->offset());
-      _masm->verified_entry();
+      _masm->verified_entry(compilation()->directive()->BreakAtExecuteOption);
       if (needs_clinit_barrier_on_entry(compilation()->method())) {
         clinit_barrier(compilation()->method());
       }
       build_frame();
       offsets()->set_value(CodeOffsets::Frame_Complete, _masm->offset());
       break;
+    }
 
     case lir_osr_entry:
       offsets()->set_value(CodeOffsets::OSR_Entry, _masm->offset());
@@ -675,7 +694,7 @@ void LIR_Assembler::emit_op0(LIR_Op0* op) {
 void LIR_Assembler::emit_op2(LIR_Op2* op) {
   switch (op->code()) {
     case lir_cmp:
-      if (op->info() != NULL) {
+      if (op->info() != nullptr) {
         assert(op->in_opr1()->is_address() || op->in_opr2()->is_address(),
                "shouldn't be codeemitinfo for non-address operands");
         add_debug_info_for_null_check_here(op->info()); // exception possible
@@ -689,10 +708,6 @@ void LIR_Assembler::emit_op2(LIR_Op2* op) {
       comp_fl2i(op->code(), op->in_opr1(), op->in_opr2(), op->result_opr(), op);
       break;
 
-    case lir_cmove:
-      cmove(op->condition(), op->in_opr1(), op->in_opr2(), op->result_opr(), op->type());
-      break;
-
     case lir_shl:
     case lir_shr:
     case lir_ushr:
@@ -718,17 +733,6 @@ void LIR_Assembler::emit_op2(LIR_Op2* op) {
         op->fpu_pop_count() == 1);
       break;
 
-    case lir_abs:
-    case lir_sqrt:
-    case lir_tan:
-    case lir_log10:
-      intrinsic_op(op->code(), op->in_opr1(), op->in_opr2(), op->result_opr(), op);
-      break;
-
-    case lir_neg:
-      negate(op->in_opr1(), op->result_opr(), op->in_opr2());
-      break;
-
     case lir_logic_and:
     case lir_logic_or:
     case lir_logic_xor:
@@ -754,6 +758,17 @@ void LIR_Assembler::emit_op2(LIR_Op2* op) {
   }
 }
 
+void LIR_Assembler::emit_op4(LIR_Op4* op) {
+  switch(op->code()) {
+    case lir_cmove:
+      cmove(op->condition(), op->in_opr1(), op->in_opr2(), op->result_opr(), op->type(), op->in_opr3(), op->in_opr4());
+      break;
+
+    default:
+      Unimplemented();
+      break;
+  }
+}
 
 void LIR_Assembler::build_frame() {
   _masm->build_frame(initial_frame_size_in_bytes(), bang_size_in_bytes());
@@ -773,10 +788,10 @@ void LIR_Assembler::roundfp_op(LIR_Opr src, LIR_Opr tmp, LIR_Opr dest, bool pop_
 void LIR_Assembler::move_op(LIR_Opr src, LIR_Opr dest, BasicType type, LIR_PatchCode patch_code, CodeEmitInfo* info, bool pop_fpu_stack, bool wide) {
   if (src->is_register()) {
     if (dest->is_register()) {
-      assert(patch_code == lir_patch_none && info == NULL, "no patching and info allowed here");
+      assert(patch_code == lir_patch_none && info == nullptr, "no patching and info allowed here");
       reg2reg(src,  dest);
     } else if (dest->is_stack()) {
-      assert(patch_code == lir_patch_none && info == NULL, "no patching and info allowed here");
+      assert(patch_code == lir_patch_none && info == nullptr, "no patching and info allowed here");
       reg2stack(src, dest, type, pop_fpu_stack);
     } else if (dest->is_address()) {
       reg2mem(src, dest, type, patch_code, info, pop_fpu_stack, wide);
@@ -785,7 +800,7 @@ void LIR_Assembler::move_op(LIR_Opr src, LIR_Opr dest, BasicType type, LIR_Patch
     }
 
   } else if (src->is_stack()) {
-    assert(patch_code == lir_patch_none && info == NULL, "no patching and info allowed here");
+    assert(patch_code == lir_patch_none && info == nullptr, "no patching and info allowed here");
     if (dest->is_register()) {
       stack2reg(src, dest, type);
     } else if (dest->is_stack()) {
@@ -798,7 +813,7 @@ void LIR_Assembler::move_op(LIR_Opr src, LIR_Opr dest, BasicType type, LIR_Patch
     if (dest->is_register()) {
       const2reg(src, dest, patch_code, info); // patching is possible
     } else if (dest->is_stack()) {
-      assert(patch_code == lir_patch_none && info == NULL, "no patching and info allowed here");
+      assert(patch_code == lir_patch_none && info == nullptr, "no patching and info allowed here");
       const2stack(src, dest);
     } else if (dest->is_address()) {
       assert(patch_code == lir_patch_none, "no patching allowed here");
@@ -824,8 +839,6 @@ void LIR_Assembler::verify_oop_map(CodeEmitInfo* info) {
       if (v.is_oop()) {
         VMReg r = v.reg();
         if (!r->is_stack()) {
-          stringStream st;
-          st.print("bad oop %s at %d", r->as_Register()->name(), _masm->offset());
           _masm->verify_oop(r->as_Register());
         } else {
           _masm->verify_stack_oop(r->reg2stack() * VMRegImpl::stack_slot_size);
