@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,7 +32,7 @@
 #endif
 
 
-class ciTypeFlow : public ResourceObj {
+class ciTypeFlow : public ArenaObj {
 private:
   ciEnv*    _env;
   ciMethod* _method;
@@ -66,7 +66,7 @@ public:
   // Represents information about an "active" jsr call.  This
   // class represents a call to the routine at some entry address
   // with some distinct return address.
-  class JsrRecord : public ResourceObj {
+  class JsrRecord : public ArenaObj {
   private:
     int _entry_address;
     int _return_address;
@@ -97,7 +97,7 @@ public:
   //
   // Note that different amounts of effort can be expended determining
   // if paths are compatible.  <DISCUSSION>
-  class JsrSet : public ResourceObj {
+  class JsrSet : public AnyObj {
   private:
     GrowableArray<JsrRecord*> _set;
 
@@ -153,7 +153,7 @@ public:
 
   // A StateVector summarizes the type information at some
   // point in the program
-  class StateVector : public ResourceObj {
+  class StateVector : public AnyObj {
   private:
     ciType**    _types;
     int         _stack_size;
@@ -186,7 +186,7 @@ public:
       switch (t->basic_type()) {
       case T_LONG:    return long2_type();
       case T_DOUBLE:  return double2_type();
-      default:        ShouldNotReachHere(); return NULL;
+      default:        ShouldNotReachHere(); return nullptr;
       }
     }
 
@@ -213,9 +213,12 @@ public:
       return (Cell)(outer()->max_locals() + stack_size());
     }
 
+    Cell local_limit_cell() const { return (Cell) outer()->max_locals(); }
+
     // Cell creation
     Cell      local(int lnum) const {
       assert(lnum < outer()->max_locals(), "index check");
+      assert(Cell_0 <= lnum && lnum <= Cell_max, "out of Cell's range");
       return (Cell)(lnum);
     }
 
@@ -333,16 +336,16 @@ public:
     }
     // pop_objArray and pop_typeArray narrow the tos to ciObjArrayKlass
     // or ciTypeArrayKlass (resp.).  In the rare case that an explicit
-    // null is popped from the stack, we return NULL.  Caller beware.
+    // null is popped from the stack, we return null.  Caller beware.
     ciObjArrayKlass* pop_objArray() {
       ciType* array = pop_value();
-      if (array == null_type())  return NULL;
+      if (array == null_type())  return nullptr;
       assert(array->is_obj_array_klass(), "must be object array type");
       return array->as_obj_array_klass();
     }
     ciTypeArrayKlass* pop_typeArray() {
       ciType* array = pop_value();
-      if (array == null_type())  return NULL;
+      if (array == null_type())  return nullptr;
       assert(array->is_type_array_klass(), "must be prim array type");
       return array->as_type_array_klass();
     }
@@ -368,7 +371,7 @@ public:
 
     void overwrite_local_double_long(int index) {
       // Invalidate the previous local if it contains first half of
-      // a double or long value since it's seconf half is being overwritten.
+      // a double or long value since its second half is being overwritten.
       int prev_index = index - 1;
       if (prev_index >= 0 &&
           (is_double(type_at(local(prev_index))) ||
@@ -501,8 +504,8 @@ public:
     int    _index;
     Block* _succ;
   public:
-    SuccIter()                        : _pred(NULL), _index(-1), _succ(NULL) {}
-    SuccIter(Block* pred)             : _pred(pred), _index(-1), _succ(NULL) { next(); }
+    SuccIter()                        : _pred(nullptr), _index(-1), _succ(nullptr) {}
+    SuccIter(Block* pred)             : _pred(pred), _index(-1), _succ(nullptr) { next(); }
     int    index()     { return _index; }
     Block* pred()      { return _pred; }           // Return predecessor
     bool   done()      { return _index < 0; }      // Finished?
@@ -513,7 +516,7 @@ public:
   };
 
   // A basic block
-  class Block : public ResourceObj {
+  class Block : public ArenaObj {
   private:
     ciBlock*                          _ciblock;
     GrowableArray<Block*>*           _exceptions;
@@ -535,8 +538,11 @@ public:
     // Has this block been cloned for a loop backedge?
     bool                             _backedge_copy;
 
-    // This block is entry to irreducible loop.
-    bool                             _irreducible_entry;
+    // This block is a loop head of an irreducible loop.
+    bool                             _irreducible_loop_head;
+
+    // This block is a secondary entry to an irreducible loop (entry but not head).
+    bool                             _irreducible_loop_secondary_entry;
 
     // This block has monitor entry point.
     bool                             _has_monitorenter;
@@ -590,7 +596,7 @@ public:
       assert(is_loop_head(), "only loop heads");
       // Find outermost loop with same loop head
       Loop* lp = loop();
-      while (lp->parent() != NULL) {
+      while (lp->parent() != nullptr) {
         if (lp->parent()->head() != lp->head()) break;
         lp = lp->parent();
       }
@@ -604,7 +610,7 @@ public:
                                       StateVector* state,
                                       JsrSet* jsrs);
     GrowableArray<Block*>* successors() {
-      assert(_successors != NULL, "must be filled in");
+      assert(_successors != nullptr, "must be filled in");
       return _successors;
     }
 
@@ -615,7 +621,7 @@ public:
 
     // Get the exceptional successors for this Block.
     GrowableArray<Block*>* exceptions() {
-      if (_exceptions == NULL) {
+      if (_exceptions == nullptr) {
         compute_exceptions();
       }
       return _exceptions;
@@ -624,7 +630,7 @@ public:
     // Get the exception klasses corresponding to the
     // exceptional successors for this Block.
     GrowableArray<ciInstanceKlass*>* exc_klasses() {
-      if (_exc_klasses == NULL) {
+      if (_exc_klasses == nullptr) {
         compute_exceptions();
       }
       return _exc_klasses;
@@ -687,8 +693,11 @@ public:
     Loop*  loop() const                  { return _loop; }
     void   set_loop(Loop* lp)            { _loop = lp; }
     bool   is_loop_head() const          { return _loop && _loop->head() == this; }
-    void   set_irreducible_entry(bool c) { _irreducible_entry = c; }
-    bool   is_irreducible_entry() const  { return _irreducible_entry; }
+    bool   is_in_irreducible_loop() const;
+    void   set_irreducible_loop_head()   { _irreducible_loop_head = true; }
+    bool   is_irreducible_loop_head() const { return _irreducible_loop_head; }
+    void   set_irreducible_loop_secondary_entry() { _irreducible_loop_secondary_entry = true; }
+    bool   is_irreducible_loop_secondary_entry() const { return _irreducible_loop_secondary_entry; }
     void   set_has_monitorenter()        { _has_monitorenter = true; }
     bool   has_monitorenter() const      { return _has_monitorenter; }
     bool   is_visited() const            { return has_pre_order(); }
@@ -697,7 +706,7 @@ public:
     Block* looping_succ(Loop* lp);       // Successor inside of loop
     bool   is_single_entry_loop_head() const {
       if (!is_loop_head()) return false;
-      for (Loop* lp = loop(); lp != NULL && lp->head() == this; lp = lp->parent())
+      for (Loop* lp = loop(); lp != nullptr && lp->head() == this; lp = lp->parent())
         if (lp->is_irreducible()) return false;
       return true;
     }
@@ -707,7 +716,7 @@ public:
   };
 
   // Loop
-  class Loop : public ResourceObj {
+  class Loop : public ArenaObj {
   private:
     Loop* _parent;
     Loop* _sibling;  // List of siblings, null terminated
@@ -716,12 +725,16 @@ public:
     Block* _tail;    // Tail of loop
     bool   _irreducible;
     LocalSet _def_locals;
+    int _profiled_count;
+
+    ciTypeFlow* outer() const { return head()->outer(); }
+    bool at_insertion_point(Loop* lp, Loop* current);
 
   public:
     Loop(Block* head, Block* tail) :
-      _parent(NULL), _sibling(NULL), _child(NULL),
+      _parent(nullptr), _sibling(nullptr), _child(nullptr),
       _head(head),   _tail(tail),
-      _irreducible(false), _def_locals() {}
+      _irreducible(false), _def_locals(), _profiled_count(-1) {}
 
     Loop* parent()  const { return _parent; }
     Loop* sibling() const { return _sibling; }
@@ -751,12 +764,15 @@ public:
     // Mark non-single entry to loop
     void set_irreducible(Block* entry) {
       _irreducible = true;
-      entry->set_irreducible_entry(true);
+      head()->set_irreducible_loop_head();
+      entry->set_irreducible_loop_secondary_entry();
     }
     bool is_irreducible() const { return _irreducible; }
 
     bool is_root() const { return _tail->pre_order() == max_jint; }
 
+    int profiled_count();
+
     void print(outputStream* st = tty, int indent = 0) const PRODUCT_RETURN;
   };
 
@@ -767,7 +783,7 @@ public:
     Loop* _current;
   public:
     PreorderLoops(Loop* root) : _root(root), _current(root) {}
-    bool done() { return _current == NULL; }  // Finished iterating?
+    bool done() { return _current == nullptr; }  // Finished iterating?
     void next();                            // Advance to next loop
     Loop* current() { return _current; }      // Return current loop.
   };
@@ -795,7 +811,7 @@ private:
   bool can_trap(ciBytecodeStream& str);
 
   // Clone the loop heads. Returns true if any cloning occurred.
-  bool clone_loop_heads(Loop* lp, StateVector* temp_vector, JsrSet* temp_set);
+  bool clone_loop_heads(StateVector* temp_vector, JsrSet* temp_set);
 
   // Clone lp's head and replace tail's successors with clone.
   Block* clone_loop_head(Loop* lp, StateVector* temp_vector, JsrSet* temp_set);
@@ -812,11 +828,11 @@ public:
   int backedge_copy_count(int ciBlockIndex, JsrSet* jsrs) const;
 
   // Return an existing block containing bci which has a JsrSet compatible
-  // with jsrs, or NULL if there is none.
+  // with jsrs, or null if there is none.
   Block* existing_block_at(int bci, JsrSet* set) { return block_at(bci, set, no_create); }
 
   // Tell whether the flow analysis has encountered an error of some sort.
-  bool failing() { return env()->failing() || _failure_reason != NULL; }
+  bool failing() { return env()->failing() || _failure_reason != nullptr; }
 
   // Reason this compilation is failing, such as "too many basic blocks".
   const char* failure_reason() { return _failure_reason; }
@@ -825,7 +841,7 @@ public:
   void record_failure(const char* reason);
 
   // Return the block of a given pre-order number.
-  int have_block_count() const      { return _block_map != NULL; }
+  int have_block_count() const      { return _block_map != nullptr; }
   int block_count() const           { assert(have_block_count(), "");
                                       return _next_pre_order; }
   Block* pre_order_at(int po) const { assert(0 <= po && po < block_count(), "out of bounds");
@@ -847,7 +863,7 @@ private:
   int _next_pre_order;
 
   // Are there more blocks on the work list?
-  bool work_list_empty() { return _work_list == NULL; }
+  bool work_list_empty() { return _work_list == nullptr; }
 
   // Get the next basic block from our work list.
   Block* work_list_next();
@@ -873,7 +889,7 @@ public:
   JsrRecord* make_jsr_record(int entry_address, int return_address);
 
   void  set_loop_tree_root(Loop* ltr) { _loop_tree_root = ltr; }
-  Loop* loop_tree_root()              { return _loop_tree_root; }
+  Loop* loop_tree_root() const        { return _loop_tree_root; }
 
 private:
   // Get the initial state for start_bci:
@@ -920,6 +936,7 @@ public:
   // Determine if bci is dominated by dom_bci
   bool is_dominated_by(int bci, int dom_bci);
 
+  void print() const PRODUCT_RETURN;
   void print_on(outputStream* st) const PRODUCT_RETURN;
 
   void rpo_print_on(outputStream* st) const PRODUCT_RETURN;
