@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,16 +23,19 @@
  */
 
 #include "precompiled.hpp"
-#include "jni.h"
 #include "cds/archiveBuilder.hpp"
 #include "cds/archiveUtils.hpp"
+#include "cds/cdsConfig.hpp"
 #include "cds/filemap.hpp"
 #include "cds/heapShared.hpp"
 #include "classfile/classLoader.hpp"
 #include "classfile/classLoaderData.inline.hpp"
 #include "classfile/javaClasses.inline.hpp"
 #include "classfile/moduleEntry.hpp"
+#include "classfile/systemDictionary.hpp"
+#include "jni.h"
 #include "logging/log.hpp"
+#include "logging/logStream.hpp"
 #include "memory/resourceArea.hpp"
 #include "memory/universe.hpp"
 #include "oops/oopHandle.inline.hpp"
@@ -45,7 +48,7 @@
 #include "utilities/quickSort.hpp"
 #include "utilities/resourceHash.hpp"
 
-ModuleEntry* ModuleEntryTable::_javabase_module = NULL;
+ModuleEntry* ModuleEntryTable::_javabase_module = nullptr;
 
 oop ModuleEntry::module() const { return _module.resolve(); }
 
@@ -56,9 +59,9 @@ void ModuleEntry::set_location(Symbol* location) {
 
   _location = location;
 
-  if (location != NULL) {
+  if (location != nullptr) {
     location->increment_refcount();
-    CDS_ONLY(if (UseSharedSpaces) {
+    CDS_ONLY(if (CDSConfig::is_using_archive()) {
         _shared_path_index = FileMapInfo::get_module_shared_path_index(location);
       });
   }
@@ -74,9 +77,9 @@ void ModuleEntry::set_location(Symbol* location) {
 //        and its version is the same as module java.base's version
 // The above check is imprecise but should work in almost all cases.
 bool ModuleEntry::should_show_version() {
-  if (version() == NULL || !is_named()) return false;
+  if (version() == nullptr || !is_named()) return false;
 
-  if (location() != NULL) {
+  if (location() != nullptr) {
     ResourceMark rm;
     const char* loc = location()->as_C_string();
     ClassLoaderData* cld = loader_data();
@@ -119,7 +122,7 @@ void ModuleEntry::set_shared_protection_domain(ClassLoaderData *loader_data,
 
 // Returns true if this module can read module m
 bool ModuleEntry::can_read(ModuleEntry* m) const {
-  assert(m != NULL, "No module to lookup in this module's reads list");
+  assert(m != nullptr, "No module to lookup in this module's reads list");
 
   // Unnamed modules read everyone and all modules
   // read java.base.  If either of these conditions
@@ -146,7 +149,7 @@ bool ModuleEntry::can_read(ModuleEntry* m) const {
   if (!has_reads_list()) {
     return false;
   } else {
-    return _reads->contains(m);
+    return reads()->contains(m);
   }
 }
 
@@ -158,12 +161,13 @@ void ModuleEntry::add_read(ModuleEntry* m) {
   }
 
   MutexLocker m1(Module_lock);
-  if (m == NULL) {
+  if (m == nullptr) {
     set_can_read_all_unnamed();
   } else {
-    if (_reads == NULL) {
+    if (reads() == nullptr) {
       // Lazily create a module's reads list
-      _reads = new (ResourceObj::C_HEAP, mtModule) GrowableArray<ModuleEntry*>(MODULE_READS_SIZE, mtModule);
+      GrowableArray<ModuleEntry*>* new_reads = new (mtModule) GrowableArray<ModuleEntry*>(MODULE_READS_SIZE, mtModule);
+      set_reads(new_reads);
     }
 
     // Determine, based on this newly established read edge to module m,
@@ -171,7 +175,7 @@ void ModuleEntry::add_read(ModuleEntry* m) {
     set_read_walk_required(m->loader_data());
 
     // Establish readability to module m
-    _reads->append_if_missing(m);
+    reads()->append_if_missing(m);
   }
 }
 
@@ -190,7 +194,7 @@ void ModuleEntry::set_read_walk_required(ClassLoaderData* m_loader_data) {
     if (log_is_enabled(Trace, module)) {
       ResourceMark rm;
       log_trace(module)("ModuleEntry::set_read_walk_required(): module %s reads list must be walked",
-                        (name() != NULL) ? name()->as_C_string() : UNNAMED_MODULE);
+                        (name() != nullptr) ? name()->as_C_string() : UNNAMED_MODULE);
     }
   }
 }
@@ -205,7 +209,7 @@ void ModuleEntry::set_is_open(bool is_open) {
 // module will return false.
 bool ModuleEntry::has_reads_list() const {
   assert_locked_or_safepoint(Module_lock);
-  return ((_reads != NULL) && !_reads->is_empty());
+  return ((reads() != nullptr) && !reads()->is_empty());
 }
 
 // Purge dead module entries out of reads list.
@@ -220,16 +224,16 @@ void ModuleEntry::purge_reads() {
     if (log_is_enabled(Trace, module)) {
       ResourceMark rm;
       log_trace(module)("ModuleEntry::purge_reads(): module %s reads list being walked",
-                        (name() != NULL) ? name()->as_C_string() : UNNAMED_MODULE);
+                        (name() != nullptr) ? name()->as_C_string() : UNNAMED_MODULE);
     }
 
     // Go backwards because this removes entries that are dead.
-    int len = _reads->length();
+    int len = reads()->length();
     for (int idx = len - 1; idx >= 0; idx--) {
-      ModuleEntry* module_idx = _reads->at(idx);
+      ModuleEntry* module_idx = reads()->at(idx);
       ClassLoaderData* cld_idx = module_idx->loader_data();
       if (cld_idx->is_unloading()) {
-        _reads->delete_at(idx);
+        reads()->delete_at(idx);
       } else {
         // Update the need to walk this module's reads based on live modules
         set_read_walk_required(cld_idx);
@@ -240,19 +244,19 @@ void ModuleEntry::purge_reads() {
 
 void ModuleEntry::module_reads_do(ModuleClosure* f) {
   assert_locked_or_safepoint(Module_lock);
-  assert(f != NULL, "invariant");
+  assert(f != nullptr, "invariant");
 
   if (has_reads_list()) {
-    int reads_len = _reads->length();
-    for (int i = 0; i < reads_len; ++i) {
-      f->do_module(_reads->at(i));
+    int reads_len = reads()->length();
+    for (ModuleEntry* m : *reads()) {
+      f->do_module(m);
     }
   }
 }
 
 void ModuleEntry::delete_reads() {
-  delete _reads;
-  _reads = NULL;
+  delete reads();
+  _reads = nullptr;
 }
 
 ModuleEntry::ModuleEntry(Handle module_handle,
@@ -269,7 +273,8 @@ ModuleEntry::ModuleEntry(Handle module_handle,
     _has_default_read_edges(false),
     _must_walk_reads(false),
     _is_open(is_open),
-    _is_patched(false) {
+    _is_patched(false)
+    DEBUG_ONLY(COMMA _reads_is_archived(false)) {
 
   // Initialize fields specific to a ModuleEntry
   if (_name == nullptr) {
@@ -293,7 +298,7 @@ ModuleEntry::ModuleEntry(Handle module_handle,
     if (log_is_enabled(Trace, module, patch)) {
       ResourceMark rm;
       log_trace(module, patch)("Marked module %s as patched from --patch-module",
-                               name != NULL ? name->as_C_string() : UNNAMED_MODULE);
+                               name != nullptr ? name->as_C_string() : UNNAMED_MODULE);
     }
   }
 
@@ -333,7 +338,7 @@ ModuleEntry* ModuleEntry::create_boot_unnamed_module(ClassLoaderData* cld) {
   // is not known until a call to JVM_SetBootLoaderUnnamedModule is made. At
   // this point initially create the ModuleEntry for the unnamed module.
   ModuleEntry* unnamed_module = new_unnamed_module_entry(Handle(), cld);
-  assert(unnamed_module != NULL, "boot loader unnamed module should not be null");
+  assert(unnamed_module != nullptr, "boot loader unnamed module should not be null");
   return unnamed_module;
 }
 
@@ -383,26 +388,44 @@ typedef ResourceHashtable<
   const ModuleEntry*,
   ModuleEntry*,
   557, // prime number
-  ResourceObj::C_HEAP> ArchivedModuleEntries;
-static ArchivedModuleEntries* _archive_modules_entries = NULL;
+  AnyObj::C_HEAP> ArchivedModuleEntries;
+static ArchivedModuleEntries* _archive_modules_entries = nullptr;
+
+#ifndef PRODUCT
+static int _num_archived_module_entries = 0;
+static int _num_inited_module_entries = 0;
+#endif
 
 ModuleEntry* ModuleEntry::allocate_archived_entry() const {
   assert(is_named(), "unnamed packages/modules are not archived");
   ModuleEntry* archived_entry = (ModuleEntry*)ArchiveBuilder::rw_region_alloc(sizeof(ModuleEntry));
   memcpy((void*)archived_entry, (void*)this, sizeof(ModuleEntry));
+  archived_entry->_archived_module_index = -1;
 
-  if (_archive_modules_entries == NULL) {
-    _archive_modules_entries = new (ResourceObj::C_HEAP, mtClass)ArchivedModuleEntries();
+  if (_archive_modules_entries == nullptr) {
+    _archive_modules_entries = new (mtClass)ArchivedModuleEntries();
   }
-  assert(_archive_modules_entries->get(this) == NULL, "Each ModuleEntry must not be shared across ModuleEntryTables");
+  assert(_archive_modules_entries->get(this) == nullptr, "Each ModuleEntry must not be shared across ModuleEntryTables");
   _archive_modules_entries->put(this, archived_entry);
+  DEBUG_ONLY(_num_archived_module_entries++);
 
+  if (log_is_enabled(Info, cds, module)) {
+    ResourceMark rm;
+    LogStream ls(Log(cds, module)::info());
+    ls.print("Stored in archive: ");
+    archived_entry->print(&ls);
+  }
   return archived_entry;
 }
 
+bool ModuleEntry::has_been_archived() {
+  assert(!ArchiveBuilder::current()->is_in_buffer_space(this), "must be called on original ModuleEntry");
+  return _archive_modules_entries->contains(this);
+}
+
 ModuleEntry* ModuleEntry::get_archived_entry(ModuleEntry* orig_entry) {
   ModuleEntry** ptr = _archive_modules_entries->get(orig_entry);
-  assert(ptr != NULL && *ptr != NULL, "must have been allocated");
+  assert(ptr != nullptr && *ptr != nullptr, "must have been allocated");
   return *ptr;
 }
 
@@ -410,8 +433,8 @@ ModuleEntry* ModuleEntry::get_archived_entry(ModuleEntry* orig_entry) {
 // GrowableArray cannot be directly archived, as it needs to be expandable at runtime.
 // Write it out as an Array, and convert it back to GrowableArray at runtime.
 Array<ModuleEntry*>* ModuleEntry::write_growable_array(GrowableArray<ModuleEntry*>* array) {
-  Array<ModuleEntry*>* archived_array = NULL;
-  int length = (array == NULL) ? 0 : array->length();
+  Array<ModuleEntry*>* archived_array = nullptr;
+  int length = (array == nullptr) ? 0 : array->length();
   if (length > 0) {
     archived_array = ArchiveBuilder::new_ro_array<ModuleEntry*>(length);
     for (int i = 0; i < length; i++) {
@@ -425,10 +448,10 @@ Array<ModuleEntry*>* ModuleEntry::write_growable_array(GrowableArray<ModuleEntry
 }
 
 GrowableArray<ModuleEntry*>* ModuleEntry::restore_growable_array(Array<ModuleEntry*>* archived_array) {
-  GrowableArray<ModuleEntry*>* array = NULL;
-  int length = (archived_array == NULL) ? 0 : archived_array->length();
+  GrowableArray<ModuleEntry*>* array = nullptr;
+  int length = (archived_array == nullptr) ? 0 : archived_array->length();
   if (length > 0) {
-    array = new (ResourceObj::C_HEAP, mtModule)GrowableArray<ModuleEntry*>(length, mtModule);
+    array = new (mtModule) GrowableArray<ModuleEntry*>(length, mtModule);
     for (int i = 0; i < length; i++) {
       ModuleEntry* archived_entry = archived_array->at(i);
       array->append(archived_entry);
@@ -445,49 +468,60 @@ void ModuleEntry::iterate_symbols(MetaspaceClosure* closure) {
 }
 
 void ModuleEntry::init_as_archived_entry() {
-  Array<ModuleEntry*>* archived_reads = write_growable_array(_reads);
+  set_archived_reads(write_growable_array(reads()));
 
-  _loader_data = NULL;  // re-init at runtime
+  _loader_data = nullptr;  // re-init at runtime
   _shared_path_index = FileMapInfo::get_module_shared_path_index(_location);
-  if (name() != NULL) {
+  if (name() != nullptr) {
     _name = ArchiveBuilder::get_buffered_symbol(_name);
     ArchivePtrMarker::mark_pointer((address*)&_name);
   }
-  _reads = (GrowableArray<ModuleEntry*>*)archived_reads;
-  if (_version != NULL) {
+  if (_version != nullptr) {
     _version = ArchiveBuilder::get_buffered_symbol(_version);
   }
-  if (_location != NULL) {
+  if (_location != nullptr) {
     _location = ArchiveBuilder::get_buffered_symbol(_location);
   }
-  JFR_ONLY(set_trace_id(0));// re-init at runtime
+  JFR_ONLY(set_trace_id(0);) // re-init at runtime
 
   ArchivePtrMarker::mark_pointer((address*)&_reads);
   ArchivePtrMarker::mark_pointer((address*)&_version);
   ArchivePtrMarker::mark_pointer((address*)&_location);
 }
 
-void ModuleEntry::init_archived_oops() {
-  assert(DumpSharedSpaces, "static dump only");
-  oop module_obj = module();
-  if (module_obj != NULL) {
-    oop m = HeapShared::find_archived_heap_object(module_obj);
-    assert(m != NULL, "sanity");
-    _archived_module_index = HeapShared::append_root(m);
-  }
-  assert(shared_protection_domain() == NULL, "never set during -Xshare:dump");
+void ModuleEntry::update_oops_in_archived_module(int root_oop_index) {
+  assert(CDSConfig::is_dumping_full_module_graph(), "sanity");
+  assert(_archived_module_index == -1, "must be set exactly once");
+  assert(root_oop_index >= 0, "sanity");
+
+  _archived_module_index = root_oop_index;
+
+  assert(shared_protection_domain() == nullptr, "never set during -Xshare:dump");
   // Clear handles and restore at run time. Handles cannot be archived.
   OopHandle null_handle;
   _module = null_handle;
+
+  // For verify_archived_module_entries()
+  DEBUG_ONLY(_num_inited_module_entries++);
 }
 
+#ifndef PRODUCT
+void ModuleEntry::verify_archived_module_entries() {
+  assert(_num_archived_module_entries == _num_inited_module_entries,
+         "%d ModuleEntries have been archived but %d of them have been properly initialized with archived java.lang.Module objects",
+         _num_archived_module_entries, _num_inited_module_entries);
+}
+#endif // PRODUCT
+
 void ModuleEntry::load_from_archive(ClassLoaderData* loader_data) {
+  assert(CDSConfig::is_using_archive(), "runtime only");
   set_loader_data(loader_data);
-  _reads = restore_growable_array((Array<ModuleEntry*>*)_reads);
+  set_reads(restore_growable_array(archived_reads()));
   JFR_ONLY(INIT_ID(this);)
 }
 
 void ModuleEntry::restore_archived_oops(ClassLoaderData* loader_data) {
+  assert(CDSConfig::is_using_archive(), "runtime only");
   Handle module_handle(Thread::current(), HeapShared::get_root(_archived_module_index, /*clear=*/true));
   assert(module_handle.not_null(), "huh");
   set_module(loader_data->add_handle(module_handle));
@@ -496,12 +530,19 @@ void ModuleEntry::restore_archived_oops(ClassLoaderData* loader_data) {
   // because it may be affected by archive relocation.
   java_lang_Module::set_module_entry(module_handle(), this);
 
-  if (loader_data->class_loader() != NULL) {
-    java_lang_Module::set_loader(module_handle(), loader_data->class_loader());
+  assert(java_lang_Module::loader(module_handle()) == loader_data->class_loader(),
+         "must be set in dump time");
+
+  if (log_is_enabled(Info, cds, module)) {
+    ResourceMark rm;
+    LogStream ls(Log(cds, module)::info());
+    ls.print("Restored from archive: ");
+    print(&ls);
   }
 }
 
 void ModuleEntry::clear_archived_oops() {
+  assert(CDSConfig::is_using_archive(), "runtime only");
   HeapShared::clear_root(_archived_module_index);
 }
 
@@ -527,7 +568,7 @@ Array<ModuleEntry*>* ModuleEntryTable::allocate_archived_entries() {
 
   if (n > 1) {
     // Always allocate in the same order to produce deterministic archive.
-    QuickSort::sort(archived_modules->data(), n, (_sort_Fn)compare_module_by_name, true);
+    QuickSort::sort(archived_modules->data(), n, compare_module_by_name);
   }
   for (int i = 0; i < n; i++) {
     archived_modules->at_put(i, archived_modules->at(i)->allocate_archived_entry());
@@ -537,24 +578,16 @@ Array<ModuleEntry*>* ModuleEntryTable::allocate_archived_entries() {
 }
 
 void ModuleEntryTable::init_archived_entries(Array<ModuleEntry*>* archived_modules) {
-  assert(DumpSharedSpaces, "dump time only");
+  assert(CDSConfig::is_dumping_full_module_graph(), "sanity");
   for (int i = 0; i < archived_modules->length(); i++) {
     ModuleEntry* archived_entry = archived_modules->at(i);
     archived_entry->init_as_archived_entry();
   }
 }
 
-void ModuleEntryTable::init_archived_oops(Array<ModuleEntry*>* archived_modules) {
-  assert(DumpSharedSpaces, "dump time only");
-  for (int i = 0; i < archived_modules->length(); i++) {
-    ModuleEntry* archived_entry = archived_modules->at(i);
-    archived_entry->init_archived_oops();
-  }
-}
-
 void ModuleEntryTable::load_archived_entries(ClassLoaderData* loader_data,
                                              Array<ModuleEntry*>* archived_modules) {
-  assert(UseSharedSpaces, "runtime only");
+  assert(CDSConfig::is_using_archive(), "runtime only");
 
   for (int i = 0; i < archived_modules->length(); i++) {
     ModuleEntry* archived_entry = archived_modules->at(i);
@@ -564,7 +597,7 @@ void ModuleEntryTable::load_archived_entries(ClassLoaderData* loader_data,
 }
 
 void ModuleEntryTable::restore_archived_oops(ClassLoaderData* loader_data, Array<ModuleEntry*>* archived_modules) {
-  assert(UseSharedSpaces, "runtime only");
+  assert(CDSConfig::is_using_archive(), "runtime only");
   for (int i = 0; i < archived_modules->length(); i++) {
     ModuleEntry* archived_entry = archived_modules->at(i);
     archived_entry->restore_archived_oops(loader_data);
@@ -581,9 +614,9 @@ ModuleEntry* ModuleEntryTable::locked_create_entry(Handle module_handle,
                                                    Symbol* module_version,
                                                    Symbol* module_location,
                                                    ClassLoaderData* loader_data) {
-  assert(module_name != NULL, "ModuleEntryTable locked_create_entry should never be called for unnamed module.");
+  assert(module_name != nullptr, "ModuleEntryTable locked_create_entry should never be called for unnamed module.");
   assert(Module_lock->owned_by_self(), "should have the Module_lock");
-  assert(lookup_only(module_name) == NULL, "Module already exists");
+  assert(lookup_only(module_name) == nullptr, "Module already exists");
   ModuleEntry* entry = new ModuleEntry(module_handle, is_open, module_name,
                                        module_version, module_location, loader_data);
   bool created = _table.put(module_name, entry);
@@ -594,7 +627,7 @@ ModuleEntry* ModuleEntryTable::locked_create_entry(Handle module_handle,
 // lookup_only by Symbol* to find a ModuleEntry.
 ModuleEntry* ModuleEntryTable::lookup_only(Symbol* name) {
   assert_locked_or_safepoint(Module_lock);
-  assert(name != NULL, "name cannot be NULL");
+  assert(name != nullptr, "name cannot be nullptr");
   ModuleEntry** entry = _table.get(name);
   return (entry == nullptr) ? nullptr : *entry;
 }
@@ -614,7 +647,7 @@ void ModuleEntryTable::finalize_javabase(Handle module_handle, Symbol* version,
   ClassLoaderData* boot_loader_data = ClassLoaderData::the_null_class_loader_data();
   ModuleEntryTable* module_table = boot_loader_data->modules();
 
-  assert(module_table != NULL, "boot loader's ModuleEntryTable not defined");
+  assert(module_table != nullptr, "boot loader's ModuleEntryTable not defined");
 
   if (module_handle.is_null()) {
     fatal("Unable to finalize module definition for " JAVA_BASE_NAME);
@@ -622,7 +655,7 @@ void ModuleEntryTable::finalize_javabase(Handle module_handle, Symbol* version,
 
   // Set java.lang.Module, version and location for java.base
   ModuleEntry* jb_module = javabase_moduleEntry();
-  assert(jb_module != NULL, JAVA_BASE_NAME " ModuleEntry not defined");
+  assert(jb_module != nullptr, JAVA_BASE_NAME " ModuleEntry not defined");
   jb_module->set_version(version);
   jb_module->set_location(location);
   // Once java.base's ModuleEntry _module field is set with the known
@@ -672,11 +705,11 @@ void ModuleEntryTable::patch_javabase_entries(JavaThread* current, Handle module
     {
       java_lang_Class::fixup_module_field(k, module_handle);
     }
-    k->class_loader_data()->dec_keep_alive();
+    k->class_loader_data()->dec_keep_alive_ref_count();
   }
 
   delete java_lang_Class::fixup_module_field_list();
-  java_lang_Class::set_fixup_module_field_list(NULL);
+  java_lang_Class::set_fixup_module_field_list(nullptr);
 }
 
 void ModuleEntryTable::print(outputStream* st) {
@@ -709,11 +742,11 @@ void ModuleEntryTable::modules_do(ModuleClosure* closure) {
 void ModuleEntry::print(outputStream* st) {
   st->print_cr("entry " PTR_FORMAT " name %s module " PTR_FORMAT " loader %s version %s location %s strict %s",
                p2i(this),
-               name() == NULL ? UNNAMED_MODULE : name()->as_C_string(),
+               name_as_C_string(),
                p2i(module()),
                loader_data()->loader_name_and_id(),
-               version() != NULL ? version()->as_C_string() : "NULL",
-               location() != NULL ? location()->as_C_string() : "NULL",
+               version() != nullptr ? version()->as_C_string() : "nullptr",
+               location() != nullptr ? location()->as_C_string() : "nullptr",
                BOOL_TO_STR(!can_read_all_unnamed()));
 }
 
@@ -726,5 +759,5 @@ void ModuleEntryTable::verify() {
 }
 
 void ModuleEntry::verify() {
-  guarantee(loader_data() != NULL, "A module entry must be associated with a loader.");
+  guarantee(loader_data() != nullptr, "A module entry must be associated with a loader.");
 }
