@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -137,7 +137,6 @@ class SharedClassLoadingMark {
 };
 
 class SystemDictionaryShared: public SystemDictionary {
-  friend class ExcludeDumpTimeSharedClasses;
   friend class CleanupDumpTimeLambdaProxyClassTable;
 
   struct ArchiveInfo {
@@ -145,7 +144,7 @@ class SystemDictionaryShared: public SystemDictionary {
     RunTimeSharedDictionary _unregistered_dictionary;
     LambdaProxyClassDictionary _lambda_proxy_class_dictionary;
 
-    const RunTimeLambdaProxyClassInfo* lookup_lambda_proxy_class(LambdaProxyClassKey* key) {
+    const RunTimeLambdaProxyClassInfo* lookup_lambda_proxy_class(RunTimeLambdaProxyClassKey* key) {
       return _lambda_proxy_class_dictionary.lookup(key, key->hash(), 0);
     }
 
@@ -153,13 +152,6 @@ class SystemDictionaryShared: public SystemDictionary {
     void print_table_statistics(const char* prefix, outputStream* st);
   };
 
-public:
-  enum : char {
-    FROM_FIELD_IS_PROTECTED = 1 << 0,
-    FROM_IS_ARRAY           = 1 << 1,
-    FROM_IS_OBJECT          = 1 << 2
-  };
-
 private:
 
   static DumpTimeSharedClassTable* _dumptime_table;
@@ -194,15 +186,17 @@ private:
   static void cleanup_lambda_proxy_class_dictionary();
   static void reset_registered_lambda_proxy_class(InstanceKlass* ik);
   static bool is_jfr_event_class(InstanceKlass *k);
-  static bool is_registered_lambda_proxy_class(InstanceKlass* ik);
   static bool check_for_exclusion_impl(InstanceKlass* k);
   static void remove_dumptime_info(InstanceKlass* k) NOT_CDS_RETURN;
   static bool has_been_redefined(InstanceKlass* k);
   static InstanceKlass* retrieve_lambda_proxy_class(const RunTimeLambdaProxyClassInfo* info) NOT_CDS_RETURN_(nullptr);
-
   DEBUG_ONLY(static bool _class_loading_may_happen;)
 
+  static void copy_verification_constraints_from_preimage(InstanceKlass* klass);
+  static void copy_linking_constraints_from_preimage(InstanceKlass* klass);
+
 public:
+  static bool is_registered_lambda_proxy_class(InstanceKlass* ik);
   static bool is_hidden_lambda_proxy(InstanceKlass* ik);
   static bool is_early_klass(InstanceKlass* k);   // Was k loaded while JvmtiExport::is_early_phase()==true
   static bool has_archived_enum_objs(InstanceKlass* ik);
@@ -231,6 +225,7 @@ public:
 
   static void initialize() NOT_CDS_RETURN;
   static void init_dumptime_info(InstanceKlass* k) NOT_CDS_RETURN;
+  static void init_dumptime_info_from_preimage(InstanceKlass* k) NOT_CDS_RETURN;
   static void handle_class_unloading(InstanceKlass* k) NOT_CDS_RETURN;
 
   static Dictionary* boot_loader_dictionary() {
@@ -288,7 +283,10 @@ public:
   }
   static bool add_unregistered_class(Thread* current, InstanceKlass* k);
 
-  static void check_excluded_classes();
+  static void finish_exclusion_checks();
+  static DumpTimeSharedClassTable* dumptime_table() { return _dumptime_table; }
+
+  static bool should_be_excluded(Klass* k);
   static bool check_for_exclusion(InstanceKlass* k, DumpTimeClassInfo* info);
   static void validate_before_archiving(InstanceKlass* k);
   static bool is_excluded_class(InstanceKlass* k);
@@ -296,7 +294,6 @@ public:
   static void set_excluded_locked(InstanceKlass* k);
   static bool warn_excluded(InstanceKlass* k, const char* reason);
   static void dumptime_classes_do(class MetaspaceClosure* it);
-  static size_t estimate_size_for_archive();
   static void write_to_archive(bool is_static_archive = true);
   static void adjust_lambda_proxy_class_dictionary();
   static void serialize_dictionary_headers(class SerializeClosure* soc,
