@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -52,8 +52,7 @@ enum class CodeBlobType {
 // CodeBlob - superclass for all entries in the CodeCache.
 //
 // Subtypes are:
-//  CompiledMethod       : Compiled Java methods (include method that calls to native code)
-//   nmethod             : JIT Compiled Java methods
+//  nmethod              : JIT Compiled Java methods
 //  RuntimeBlob          : Non-compiled method code; generated glue code
 //   BufferBlob          : Used for non-relocatable code such as interpreter, stubroutines, etc.
 //    AdapterBlob        : Used to hold C2I/I2C adapters
@@ -62,126 +61,182 @@ enum class CodeBlobType {
 //   RuntimeStub         : Call to VM runtime methods
 //   SingletonBlob       : Super-class for all blobs that exist in only one instance
 //    DeoptimizationBlob : Used for deoptimization
-//    ExceptionBlob      : Used for stack unrolling
 //    SafepointBlob      : Used to handle illegal instruction exceptions
+//    ExceptionBlob      : Used for stack unrolling
 //    UncommonTrapBlob   : Used to handle uncommon traps
 //   UpcallStub  : Used for upcalls from native code
 //
 //
-// Layout : continuous in the CodeCache
+// Layout in the CodeCache:
 //   - header
-//   - relocation
 //   - content space
 //     - instruction space
-//   - data space
+// Outside of the CodeCache:
+//   - mutable_data
+//     - relocation info
+//     - additional data for subclasses
 
+enum class CodeBlobKind : u1 {
+  None,
+  Nmethod,
+  Buffer,
+  Adapter,
+  Vtable,
+  MHAdapter,
+  RuntimeStub,
+  Deoptimization,
+  Safepoint,
+#ifdef COMPILER2
+  Exception,
+  UncommonTrap,
+#endif
+  Upcall,
+  Number_Of_Kinds
+};
 
-class CodeBlobLayout;
-class UpcallStub; // for as_upcall_stub()
-class RuntimeStub; // for as_runtime_stub()
+class UpcallStub;      // for as_upcall_stub()
+class RuntimeStub;     // for as_runtime_stub()
 class JavaFrameAnchor; // for UpcallStub::jfa_for_frame
+class AdapterBlob;
+class ExceptionBlob;
+class DeoptimizationBlob;
+class SafepointBlob;
+class UncommonTrapBlob;
 
 class CodeBlob {
   friend class VMStructs;
   friend class JVMCIVMStructs;
-  friend class CodeCacheDumper;
+
+private:
+  void restore_mutable_data(address reloc_data);
 
 protected:
-
   // order fields from large to small to minimize padding between fields
-  address    _code_begin;
-  address    _code_end;
-  address    _content_begin;                     // address to where content region begins (this includes consts, insts, stubs)
-                                                 // address    _content_end - not required, for all CodeBlobs _code_end == _content_end for now
-  address    _data_end;
-  address    _relocation_begin;
-  address    _relocation_end;
-
-  ImmutableOopMapSet* _oop_maps;                 // OopMap for this CodeBlob
-
+  ImmutableOopMapSet* _oop_maps;   // OopMap for this CodeBlob
   const char*         _name;
-  S390_ONLY(int       _ctable_offset;)
+  address             _mutable_data;
 
-  int        _size;                              // total size of CodeBlob in bytes
-  int        _header_size;                       // size of header (depends on subclass)
-  int        _frame_complete_offset;             // instruction offsets in [0.._frame_complete_offset) have
-                                                 // not finished setting up their frame. Beware of pc's in
-                                                 // that range. There is a similar range(s) on returns
-                                                 // which we don't detect.
-  int        _data_offset;                       // offset to where data region begins
-  int        _frame_size;                        // size of stack frame in words (NOT slots. On x64 these are 64bit words)
+  int      _size;                  // total size of CodeBlob in bytes
+  int      _relocation_size;       // size of relocation (could be bigger than 64Kb)
+  int      _content_offset;        // offset to where content region begins (this includes consts, insts, stubs)
+  int      _code_offset;           // offset to where instructions region begins (this includes insts, stubs)
+  int      _data_offset;           // offset to where data region begins
+  int      _frame_size;            // size of stack frame in words (NOT slots. On x64 these are 64bit words)
+  int      _mutable_data_size;
 
-  bool                _caller_must_gc_arguments;
+  S390_ONLY(int _ctable_offset;)
 
-  bool                _is_compiled;
-  const CompilerType  _type;                     // CompilerType
+  uint16_t _header_size;           // size of header (depends on subclass)
+  int16_t  _frame_complete_offset; // instruction offsets in [0.._frame_complete_offset) have
+                                   // not finished setting up their frame. Beware of pc's in
+                                   // that range. There is a similar range(s) on returns
+                                   // which we don't detect.
+
+  CodeBlobKind _kind;              // Kind of this code blob
+
+  bool _caller_must_gc_arguments;
 
 #ifndef PRODUCT
   AsmRemarks _asm_remarks;
   DbgStrings _dbg_strings;
+#endif
 
- ~CodeBlob() {
-    _asm_remarks.clear();
-    _dbg_strings.clear();
-  }
-#endif // not PRODUCT
+  void print_on_impl(outputStream* st) const;
+  void print_value_on_impl(outputStream* st) const;
+
+  class Vptr {
+   public:
+    virtual void print_on(const CodeBlob* instance, outputStream* st) const = 0;
+    virtual void print_value_on(const CodeBlob* instance, outputStream* st) const = 0;
+    virtual void prepare_for_archiving(CodeBlob* instance) const {
+      instance->prepare_for_archiving_impl();
+    };
+    virtual void post_restore(CodeBlob* instance) const {
+      instance->post_restore_impl();
+    };
+  };
+
+  static const Vptr* vptr(CodeBlobKind kind);
+  const Vptr* vptr() const;
+
+  CodeBlob(const char* name, CodeBlobKind kind, CodeBuffer* cb, int size, uint16_t header_size,
+           int16_t frame_complete_offset, int frame_size, OopMapSet* oop_maps, bool caller_must_gc_arguments,
+           int mutable_data_size);
+
+  // Simple CodeBlob used for simple BufferBlob.
+  CodeBlob(const char* name, CodeBlobKind kind, int size, uint16_t header_size);
+
+
+  void operator delete(void* p) { }
+
+  void prepare_for_archiving_impl();
+  void post_restore_impl();
 
-  CodeBlob(const char* name, CompilerType type, const CodeBlobLayout& layout, int frame_complete_offset,
-           int frame_size, ImmutableOopMapSet* oop_maps,
-           bool caller_must_gc_arguments, bool compiled = false);
-  CodeBlob(const char* name, CompilerType type, const CodeBlobLayout& layout, CodeBuffer* cb, int frame_complete_offset,
-           int frame_size, OopMapSet* oop_maps,
-           bool caller_must_gc_arguments, bool compiled = false);
 public:
-  // Only used by unit test.
-  CodeBlob() : _type(compiler_none) {}
+
+  ~CodeBlob() {
+    assert(_oop_maps == nullptr, "Not flushed");
+  }
 
   // Returns the space needed for CodeBlob
   static unsigned int allocation_size(CodeBuffer* cb, int header_size);
   static unsigned int align_code_offset(int offset);
 
   // Deletion
-  virtual void flush();
+  void purge();
 
   // Typing
-  virtual bool is_buffer_blob() const                 { return false; }
-  virtual bool is_nmethod() const                     { return false; }
-  virtual bool is_runtime_stub() const                { return false; }
-  virtual bool is_deoptimization_stub() const         { return false; }
-  virtual bool is_uncommon_trap_stub() const          { return false; }
-  virtual bool is_exception_stub() const              { return false; }
-  virtual bool is_safepoint_stub() const              { return false; }
-  virtual bool is_adapter_blob() const                { return false; }
-  virtual bool is_vtable_blob() const                 { return false; }
-  virtual bool is_method_handles_adapter_blob() const { return false; }
-  virtual bool is_upcall_stub() const                 { return false; }
-  bool is_compiled() const                            { return _is_compiled; }
-  const bool* is_compiled_addr() const                { return &_is_compiled; }
-
-  inline bool is_compiled_by_c1() const    { return _type == compiler_c1; };
-  inline bool is_compiled_by_c2() const    { return _type == compiler_c2; };
-  inline bool is_compiled_by_jvmci() const { return _type == compiler_jvmci; };
-  const char* compiler_name() const;
-  CompilerType compiler_type() const { return _type; }
+  bool is_nmethod() const                     { return _kind == CodeBlobKind::Nmethod; }
+  bool is_buffer_blob() const                 { return _kind == CodeBlobKind::Buffer; }
+  bool is_runtime_stub() const                { return _kind == CodeBlobKind::RuntimeStub; }
+  bool is_deoptimization_stub() const         { return _kind == CodeBlobKind::Deoptimization; }
+#ifdef COMPILER2
+  bool is_uncommon_trap_stub() const          { return _kind == CodeBlobKind::UncommonTrap; }
+  bool is_exception_stub() const              { return _kind == CodeBlobKind::Exception; }
+#else
+  bool is_uncommon_trap_stub() const          { return false; }
+  bool is_exception_stub() const              { return false; }
+#endif
+  bool is_safepoint_stub() const              { return _kind == CodeBlobKind::Safepoint; }
+  bool is_adapter_blob() const                { return _kind == CodeBlobKind::Adapter; }
+  bool is_vtable_blob() const                 { return _kind == CodeBlobKind::Vtable; }
+  bool is_method_handles_adapter_blob() const { return _kind == CodeBlobKind::MHAdapter; }
+  bool is_upcall_stub() const                 { return _kind == CodeBlobKind::Upcall; }
 
   // Casting
-  nmethod* as_nmethod_or_null()                { return is_nmethod() ? (nmethod*) this : nullptr; }
-  nmethod* as_nmethod()                        { assert(is_nmethod(), "must be nmethod"); return (nmethod*) this; }
-  CompiledMethod* as_compiled_method_or_null() { return is_compiled() ? (CompiledMethod*) this : nullptr; }
-  CompiledMethod* as_compiled_method()         { assert(is_compiled(), "must be compiled"); return (CompiledMethod*) this; }
-  CodeBlob* as_codeblob_or_null() const        { return (CodeBlob*) this; }
-  UpcallStub* as_upcall_stub() const           { assert(is_upcall_stub(), "must be upcall stub"); return (UpcallStub*) this; }
-  RuntimeStub* as_runtime_stub() const         { assert(is_runtime_stub(), "must be runtime blob"); return (RuntimeStub*) this; }
+  nmethod* as_nmethod_or_null() const         { return is_nmethod() ? (nmethod*) this : nullptr; }
+  nmethod* as_nmethod() const                 { assert(is_nmethod(), "must be nmethod"); return (nmethod*) this; }
+  CodeBlob* as_codeblob() const               { return (CodeBlob*) this; }
+  AdapterBlob* as_adapter_blob() const        { assert(is_adapter_blob(), "must be adapter blob"); return (AdapterBlob*) this; }
+  ExceptionBlob* as_exception_blob() const    { assert(is_exception_stub(), "must be exception stub"); return (ExceptionBlob*) this; }
+  DeoptimizationBlob* as_deoptimization_blob() const { assert(is_deoptimization_stub(), "must be deopt stub"); return (DeoptimizationBlob*) this; }
+  SafepointBlob* as_safepoint_blob() const    { assert(is_safepoint_stub(), "must be safepoint stub"); return (SafepointBlob*) this; }
+  UpcallStub* as_upcall_stub() const          { assert(is_upcall_stub(), "must be upcall stub"); return (UpcallStub*) this; }
+  RuntimeStub* as_runtime_stub() const        { assert(is_runtime_stub(), "must be runtime blob"); return (RuntimeStub*) this; }
+  UncommonTrapBlob* as_uncommon_trap_blob() const { assert(is_uncommon_trap_stub(), "must be uncommon trap stub"); return (UncommonTrapBlob*) this; }
 
   // Boundaries
-  address header_begin() const        { return (address) this; }
-  relocInfo* relocation_begin() const { return (relocInfo*) _relocation_begin; };
-  relocInfo* relocation_end() const   { return (relocInfo*) _relocation_end; }
-  address content_begin() const       { return _content_begin; }
-  address content_end() const         { return _code_end; } // _code_end == _content_end is true for all types of blobs for now, it is also checked in the constructor
-  address code_begin() const          { return _code_begin;    }
-  address code_end() const            { return _code_end; }
-  address data_end() const            { return _data_end;      }
+  address    header_begin() const             { return (address)    this; }
+  address    header_end() const               { return ((address)   this) + _header_size; }
+  address    content_begin() const            { return (address)    header_begin() + _content_offset; }
+  address    content_end() const              { return (address)    header_begin() + _data_offset; }
+  address    code_begin() const               { return (address)    header_begin() + _code_offset; }
+  address    code_end() const                 { return (address)    header_begin() + _data_offset; }
+  address    data_begin() const               { return (address)    header_begin() + _data_offset; }
+  address    data_end() const                 { return (address)    header_begin() + _size; }
+  address    blob_end() const                 { return (address)    header_begin() + _size; }
+  // code_end == content_end is true for all types of blobs for now, it is also checked in the constructor
+
+  int mutable_data_size() const               { return _mutable_data_size; }
+  address mutable_data_begin() const          { return _mutable_data; }
+  address mutable_data_end() const            { return _mutable_data + _mutable_data_size; }
+
+  relocInfo* relocation_begin() const         { return (relocInfo*)_mutable_data; }
+  relocInfo* relocation_end() const           { return (relocInfo*)((address)relocation_begin() + _relocation_size); }
+
+  // Offsets
+  int content_offset() const                  { return _content_offset; }
+  int code_offset() const                     { return _code_offset; }
 
   // This field holds the beginning of the const section in the old code buffer.
   // It is needed to fix relocations of pc-relative loads when resizing the
@@ -190,36 +245,33 @@ public:
   void set_ctable_begin(address ctable) { S390_ONLY(_ctable_offset = ctable - header_begin();) }
 
   // Sizes
-  int size() const                               { return _size; }
-  int header_size() const                        { return _header_size; }
-  int relocation_size() const                    { return pointer_delta_as_int((address) relocation_end(), (address) relocation_begin()); }
-  int content_size() const                       { return pointer_delta_as_int(content_end(), content_begin()); }
-  int code_size() const                          { return pointer_delta_as_int(code_end(), code_begin()); }
+  int size() const               { return _size; }
+  int header_size() const        { return _header_size; }
+  int relocation_size() const    { return pointer_delta_as_int((address) relocation_end(), (address) relocation_begin()); }
+  int content_size() const       { return pointer_delta_as_int(content_end(), content_begin()); }
+  int code_size() const          { return pointer_delta_as_int(code_end(), code_begin()); }
+
   // Only used from CodeCache::free_unused_tail() after the Interpreter blob was trimmed
   void adjust_size(size_t used) {
     _size = (int)used;
-    _data_offset = (int)used;
-    _code_end = (address)this + used;
-    _data_end = (address)this + used;
+    _data_offset = _size;
   }
 
   // Containment
-  bool blob_contains(address addr) const         { return header_begin()       <= addr && addr < data_end();       }
+  bool blob_contains(address addr) const         { return header_begin()       <= addr && addr < blob_end();       }
   bool code_contains(address addr) const         { return code_begin()         <= addr && addr < code_end();       }
   bool contains(address addr) const              { return content_begin()      <= addr && addr < content_end();    }
   bool is_frame_complete_at(address addr) const  { return _frame_complete_offset != CodeOffsets::frame_never_safe &&
                                                           code_contains(addr) && addr >= code_begin() + _frame_complete_offset; }
   int frame_complete_offset() const              { return _frame_complete_offset; }
 
-  virtual bool is_not_entrant() const            { return false; }
-
   // OopMap for frame
   ImmutableOopMapSet* oop_maps() const           { return _oop_maps; }
   void set_oop_maps(OopMapSet* p);
+  void set_oop_maps(ImmutableOopMapSet* p)       { _oop_maps = p; }
 
   const ImmutableOopMap* oop_map_for_slot(int slot, address return_address) const;
   const ImmutableOopMap* oop_map_for_return_address(address return_address) const;
-  virtual void preserve_callee_argument_oops(frame fr, const RegisterMap* reg_map, OopClosure* f) = 0;
 
   // Frame support. Sizes are in word units.
   int  frame_size() const                        { return _frame_size; }
@@ -233,21 +285,16 @@ public:
   void set_name(const char* name)                { _name = name; }
 
   // Debugging
-  virtual void verify() = 0;
-  virtual void print() const;
-  virtual void print_on(outputStream* st) const;
-  virtual void print_value_on(outputStream* st) const;
+  void verify();
+  void print() const;
+  void print_on(outputStream* st) const;
+  void print_value_on(outputStream* st) const;
+
   void dump_for_addr(address addr, outputStream* st, bool verbose) const;
-  void print_code();
+  void print_code_on(outputStream* st);
 
   // Print to stream, any comments associated with offset.
-  virtual void print_block_comment(outputStream* stream, address block_begin) const {
-#ifndef PRODUCT
-    ptrdiff_t offset = block_begin - code_begin();
-    assert(offset >= 0, "Expecting non-negative offset!");
-    _asm_remarks.print(uint(offset), stream);
-#endif
-  }
+  void print_block_comment(outputStream* stream, address block_begin) const;
 
 #ifndef PRODUCT
   AsmRemarks &asm_remarks() { return _asm_remarks; }
@@ -256,99 +303,31 @@ public:
   void use_remarks(AsmRemarks &remarks) { _asm_remarks.share(remarks); }
   void use_strings(DbgStrings &strings) { _dbg_strings.share(strings); }
 #endif
+
+  void copy_to(address buffer) {
+    memcpy(buffer, this, this->size());
+  }
+
+  // methods to archive a blob into AOT code cache
+  void prepare_for_archiving();
+  static void archive_blob(CodeBlob* blob, address archive_buffer);
+
+  // methods to restore a blob from AOT code cache into the CodeCache
+  void post_restore();
+  CodeBlob* restore(address code_cache_buffer, const char* name, address archived_reloc_data, ImmutableOopMapSet* archived_oop_maps);
+  static CodeBlob* create(CodeBlob* archived_blob,
+                          const char* name,
+                          address archived_reloc_data,
+                          ImmutableOopMapSet* archived_oop_maps
+#ifndef PRODUCT
+                          , AsmRemarks& archived_asm_remarks
+                          , DbgStrings& archived_dbg_strings
+#endif // PRODUCT
+                         );
 };
 
-class CodeBlobLayout : public StackObj {
-private:
-  int _size;
-  int _header_size;
-  int _relocation_size;
-  int _content_offset;
-  int _code_offset;
-  int _data_offset;
-  address _code_begin;
-  address _code_end;
-  address _content_begin;
-  address _content_end;
-  address _data_end;
-  address _relocation_begin;
-  address _relocation_end;
-
-public:
-  CodeBlobLayout(address code_begin, address code_end, address content_begin, address content_end, address data_end, address relocation_begin, address relocation_end) :
-    _size(0),
-    _header_size(0),
-    _relocation_size(0),
-    _content_offset(0),
-    _code_offset(0),
-    _data_offset(0),
-    _code_begin(code_begin),
-    _code_end(code_end),
-    _content_begin(content_begin),
-    _content_end(content_end),
-    _data_end(data_end),
-    _relocation_begin(relocation_begin),
-    _relocation_end(relocation_end)
-  {
-  }
-
-  CodeBlobLayout(const address start, int size, int header_size, int relocation_size, int data_offset) :
-    _size(size),
-    _header_size(header_size),
-    _relocation_size(relocation_size),
-    _content_offset(CodeBlob::align_code_offset(_header_size + _relocation_size)),
-    _code_offset(_content_offset),
-    _data_offset(data_offset)
-  {
-    assert(is_aligned(_relocation_size, oopSize), "unaligned size");
-
-    _code_begin = (address) start + _code_offset;
-    _code_end = (address) start + _data_offset;
-
-    _content_begin = (address) start + _content_offset;
-    _content_end = (address) start + _data_offset;
-
-    _data_end = (address) start + _size;
-    _relocation_begin = (address) start + _header_size;
-    _relocation_end = _relocation_begin + _relocation_size;
-  }
-
-  CodeBlobLayout(const address start, int size, int header_size, const CodeBuffer* cb) :
-    _size(size),
-    _header_size(header_size),
-    _relocation_size(align_up(cb->total_relocation_size(), oopSize)),
-    _content_offset(CodeBlob::align_code_offset(_header_size + _relocation_size)),
-    _code_offset(_content_offset + cb->total_offset_of(cb->insts())),
-    _data_offset(_content_offset + align_up(cb->total_content_size(), oopSize))
-  {
-    assert(is_aligned(_relocation_size, oopSize), "unaligned size");
-
-    _code_begin = (address) start + _code_offset;
-    _code_end = (address) start + _data_offset;
-
-    _content_begin = (address) start + _content_offset;
-    _content_end = (address) start + _data_offset;
-
-    _data_end = (address) start + _size;
-    _relocation_begin = (address) start + _header_size;
-    _relocation_end = _relocation_begin + _relocation_size;
-  }
-
-  int size() const { return _size; }
-  int header_size() const { return _header_size; }
-  int relocation_size() const { return _relocation_size; }
-  int content_offset() const { return _content_offset; }
-  int code_offset() const { return _code_offset; }
-  int data_offset() const { return _data_offset; }
-  address code_begin() const { return _code_begin; }
-  address code_end() const { return _code_end; }
-  address data_end() const { return _data_end; }
-  address relocation_begin() const { return _relocation_begin; }
-  address relocation_end() const { return _relocation_end; }
-  address content_begin() const { return _content_begin; }
-  address content_end() const { return _content_end; }
-};
-
+//----------------------------------------------------------------------------------------------------
+// RuntimeBlob: used for non-compiled method code (adapters, stubs, blobs)
 
 class RuntimeBlob : public CodeBlob {
   friend class VMStructs;
@@ -356,17 +335,20 @@ class RuntimeBlob : public CodeBlob {
 
   // Creation
   // a) simple CodeBlob
-  // frame_complete is the offset from the beginning of the instructions
-  // to where the frame setup (from stackwalk viewpoint) is complete.
-  RuntimeBlob(const char* name, int header_size, int size, int frame_complete, int locs_size);
+  RuntimeBlob(const char* name, CodeBlobKind kind, int size, uint16_t header_size)
+    : CodeBlob(name, kind, size, header_size)
+  {}
 
   // b) full CodeBlob
+  // frame_complete is the offset from the beginning of the instructions
+  // to where the frame setup (from stackwalk viewpoint) is complete.
   RuntimeBlob(
     const char* name,
+    CodeBlobKind kind,
     CodeBuffer* cb,
-    int         header_size,
     int         size,
-    int         frame_complete,
+    uint16_t    header_size,
+    int16_t     frame_complete,
     int         frame_size,
     OopMapSet*  oop_maps,
     bool        caller_must_gc_arguments = false
@@ -374,17 +356,11 @@ class RuntimeBlob : public CodeBlob {
 
   static void free(RuntimeBlob* blob);
 
-  void verify();
-
-  // OopMap for frame
-  virtual void preserve_callee_argument_oops(frame fr, const RegisterMap* reg_map, OopClosure* f)  { ShouldNotReachHere(); }
-
-  // Debugging
-  virtual void print_on(outputStream* st) const { CodeBlob::print_on(st); }
-  virtual void print_value_on(outputStream* st) const { CodeBlob::print_value_on(st); }
-
   // Deal with Disassembler, VTune, Forte, JvmtiExport, MemoryService.
   static void trace_new_stub(RuntimeBlob* blob, const char* name1, const char* name2 = "");
+
+  class Vptr : public CodeBlob::Vptr {
+  };
 };
 
 class WhiteBox;
@@ -401,31 +377,31 @@ class BufferBlob: public RuntimeBlob {
 
  private:
   // Creation support
-  BufferBlob(const char* name, int size);
-  BufferBlob(const char* name, int size, CodeBuffer* cb);
+  BufferBlob(const char* name, CodeBlobKind kind, int size);
+  BufferBlob(const char* name, CodeBlobKind kind, CodeBuffer* cb, int size);
 
-  // This ordinary operator delete is needed even though not used, so the
-  // below two-argument operator delete will be treated as a placement
-  // delete rather than an ordinary sized delete; see C++14 3.7.4.2/p2.
-  void operator delete(void* p);
   void* operator new(size_t s, unsigned size) throw();
 
  public:
   // Creation
-  static BufferBlob* create(const char* name, int buffer_size);
+  static BufferBlob* create(const char* name, uint buffer_size);
   static BufferBlob* create(const char* name, CodeBuffer* cb);
 
   static void free(BufferBlob* buf);
 
-  // Typing
-  virtual bool is_buffer_blob() const            { return true; }
+  void print_on_impl(outputStream* st) const;
+  void print_value_on_impl(outputStream* st) const;
 
-  // GC/Verification support
-  void preserve_callee_argument_oops(frame fr, const RegisterMap* reg_map, OopClosure* f)  { /* nothing to do */ }
+  class Vptr : public RuntimeBlob::Vptr {
+    void print_on(const CodeBlob* instance, outputStream* st) const override {
+      ((const BufferBlob*)instance)->print_on_impl(st);
+    }
+    void print_value_on(const CodeBlob* instance, outputStream* st) const override {
+      ((const BufferBlob*)instance)->print_value_on_impl(st);
+    }
+  };
 
-  void verify();
-  void print_on(outputStream* st) const;
-  void print_value_on(outputStream* st) const;
+  static const Vptr _vpntr;
 };
 
 
@@ -439,9 +415,6 @@ private:
 public:
   // Creation
   static AdapterBlob* create(CodeBuffer* cb);
-
-  // Typing
-  virtual bool is_adapter_blob() const { return true; }
 };
 
 //---------------------------------------------------------------------------------------------------
@@ -454,9 +427,6 @@ private:
 public:
   // Creation
   static VtableBlob* create(const char* name, int buffer_size);
-
-  // Typing
-  virtual bool is_vtable_blob() const { return true; }
 };
 
 //----------------------------------------------------------------------------------------------------
@@ -464,14 +434,11 @@ public:
 
 class MethodHandlesAdapterBlob: public BufferBlob {
 private:
-  MethodHandlesAdapterBlob(int size): BufferBlob("MethodHandles adapters", size) {}
+  MethodHandlesAdapterBlob(int size): BufferBlob("MethodHandles adapters", CodeBlobKind::MHAdapter, size) {}
 
 public:
   // Creation
   static MethodHandlesAdapterBlob* create(int buffer_size);
-
-  // Typing
-  virtual bool is_method_handles_adapter_blob() const { return true; }
 };
 
 
@@ -486,16 +453,12 @@ class RuntimeStub: public RuntimeBlob {
     const char* name,
     CodeBuffer* cb,
     int         size,
-    int         frame_complete,
+    int16_t     frame_complete,
     int         frame_size,
     OopMapSet*  oop_maps,
     bool        caller_must_gc_arguments
   );
 
-  // This ordinary operator delete is needed even though not used, so the
-  // below two-argument operator delete will be treated as a placement
-  // delete rather than an ordinary sized delete; see C++14 3.7.4.2/p2.
-  void operator delete(void* p);
   void* operator new(size_t s, unsigned size) throw();
 
  public:
@@ -503,25 +466,30 @@ class RuntimeStub: public RuntimeBlob {
   static RuntimeStub* new_runtime_stub(
     const char* stub_name,
     CodeBuffer* cb,
-    int         frame_complete,
+    int16_t     frame_complete,
     int         frame_size,
     OopMapSet*  oop_maps,
-    bool        caller_must_gc_arguments
+    bool        caller_must_gc_arguments,
+    bool        alloc_fail_is_fatal=true
   );
 
   static void free(RuntimeStub* stub) { RuntimeBlob::free(stub); }
 
-  // Typing
-  bool is_runtime_stub() const                   { return true; }
+  address entry_point() const         { return code_begin(); }
 
-  address entry_point() const                    { return code_begin(); }
+  void print_on_impl(outputStream* st) const;
+  void print_value_on_impl(outputStream* st) const;
 
-  // GC/Verification support
-  void preserve_callee_argument_oops(frame fr, const RegisterMap *reg_map, OopClosure* f)  { /* nothing to do */ }
+  class Vptr : public RuntimeBlob::Vptr {
+    void print_on(const CodeBlob* instance, outputStream* st) const override {
+      instance->as_runtime_stub()->print_on_impl(st);
+    }
+    void print_value_on(const CodeBlob* instance, outputStream* st) const override {
+      instance->as_runtime_stub()->print_value_on_impl(st);
+    }
+  };
 
-  void verify();
-  void print_on(outputStream* st) const;
-  void print_value_on(outputStream* st) const;
+  static const Vptr _vpntr;
 };
 
 
@@ -532,31 +500,36 @@ class SingletonBlob: public RuntimeBlob {
   friend class VMStructs;
 
  protected:
-  // This ordinary operator delete is needed even though not used, so the
-  // below two-argument operator delete will be treated as a placement
-  // delete rather than an ordinary sized delete; see C++14 3.7.4.2/p2.
-  void operator delete(void* p);
-  void* operator new(size_t s, unsigned size) throw();
+  void* operator new(size_t s, unsigned size, bool alloc_fail_is_fatal=true) throw();
 
  public:
    SingletonBlob(
-     const char* name,
-     CodeBuffer* cb,
-     int         header_size,
-     int         size,
-     int         frame_size,
-     OopMapSet*  oop_maps
+     const char*  name,
+     CodeBlobKind kind,
+     CodeBuffer*  cb,
+     int          size,
+     uint16_t     header_size,
+     int          frame_size,
+     OopMapSet*   oop_maps
    )
-   : RuntimeBlob(name, cb, header_size, size, CodeOffsets::frame_never_safe, frame_size, oop_maps)
+   : RuntimeBlob(name, kind, cb, size, header_size, CodeOffsets::frame_never_safe, frame_size, oop_maps)
   {};
 
   address entry_point()                          { return code_begin(); }
 
-  // GC/Verification support
-  void preserve_callee_argument_oops(frame fr, const RegisterMap *reg_map, OopClosure* f)  { /* nothing to do */ }
-  void verify(); // does nothing
-  void print_on(outputStream* st) const;
-  void print_value_on(outputStream* st) const;
+  void print_on_impl(outputStream* st) const;
+  void print_value_on_impl(outputStream* st) const;
+
+  class Vptr : public RuntimeBlob::Vptr {
+    void print_on(const CodeBlob* instance, outputStream* st) const override {
+      ((const SingletonBlob*)instance)->print_on_impl(st);
+    }
+    void print_value_on(const CodeBlob* instance, outputStream* st) const override {
+      ((const SingletonBlob*)instance)->print_value_on_impl(st);
+    }
+  };
+
+  static const Vptr _vpntr;
 };
 
 
@@ -601,15 +574,6 @@ class DeoptimizationBlob: public SingletonBlob {
     int         frame_size
   );
 
-  // Typing
-  bool is_deoptimization_stub() const { return true; }
-
-  // GC for args
-  void preserve_callee_argument_oops(frame fr, const RegisterMap *reg_map, OopClosure* f) { /* Nothing to do */ }
-
-  // Printing
-  void print_value_on(outputStream* st) const;
-
   address unpack() const                         { return code_begin() + _unpack_offset;           }
   address unpack_with_exception() const          { return code_begin() + _unpack_with_exception;   }
   address unpack_with_reexecution() const        { return code_begin() + _unpack_with_reexecution; }
@@ -639,6 +603,16 @@ class DeoptimizationBlob: public SingletonBlob {
   }
   address implicit_exception_uncommon_trap() const { return code_begin() + _implicit_exception_uncommon_trap_offset; }
 #endif // INCLUDE_JVMCI
+
+  void print_value_on_impl(outputStream* st) const;
+
+  class Vptr : public SingletonBlob::Vptr {
+    void print_value_on(const CodeBlob* instance, outputStream* st) const override {
+      ((const DeoptimizationBlob*)instance)->print_value_on_impl(st);
+    }
+  };
+
+  static const Vptr _vpntr;
 };
 
 
@@ -665,12 +639,6 @@ class UncommonTrapBlob: public SingletonBlob {
     OopMapSet*  oop_maps,
     int         frame_size
   );
-
-  // GC for args
-  void preserve_callee_argument_oops(frame fr, const RegisterMap *reg_map, OopClosure* f)  { /* nothing to do */ }
-
-  // Typing
-  bool is_uncommon_trap_stub() const             { return true; }
 };
 
 
@@ -696,11 +664,17 @@ class ExceptionBlob: public SingletonBlob {
     int         frame_size
   );
 
-  // GC for args
-  void preserve_callee_argument_oops(frame fr, const RegisterMap* reg_map, OopClosure* f)  { /* nothing to do */ }
+  void post_restore_impl() {
+    trace_new_stub(this, "ExceptionBlob");
+  }
 
-  // Typing
-  bool is_exception_stub() const                 { return true; }
+  class Vptr : public SingletonBlob::Vptr {
+    void post_restore(CodeBlob* instance) const override {
+      ((ExceptionBlob*)instance)->post_restore_impl();
+    }
+  };
+
+  static const Vptr _vpntr;
 };
 #endif // COMPILER2
 
@@ -726,12 +700,6 @@ class SafepointBlob: public SingletonBlob {
     OopMapSet*  oop_maps,
     int         frame_size
   );
-
-  // GC for args
-  void preserve_callee_argument_oops(frame fr, const RegisterMap* reg_map, OopClosure* f)  { /* nothing to do */ }
-
-  // Typing
-  bool is_safepoint_stub() const                 { return true; }
 };
 
 //----------------------------------------------------------------------------------------------------
@@ -740,20 +708,14 @@ class UpcallLinker;
 
 // A (Panama) upcall stub. Not used by JNI.
 class UpcallStub: public RuntimeBlob {
+  friend class VMStructs;
   friend class UpcallLinker;
  private:
-  intptr_t _exception_handler_offset;
   jobject _receiver;
   ByteSize _frame_data_offset;
 
-  UpcallStub(const char* name, CodeBuffer* cb, int size,
-                     intptr_t exception_handler_offset,
-                     jobject receiver, ByteSize frame_data_offset);
+  UpcallStub(const char* name, CodeBuffer* cb, int size, jobject receiver, ByteSize frame_data_offset);
 
-  // This ordinary operator delete is needed even though not used, so the
-  // below two-argument operator delete will be treated as a placement
-  // delete rather than an ordinary sized delete; see C++14 3.7.4.2/p2.
-  void operator delete(void* p);
   void* operator new(size_t s, unsigned size) throw();
 
   struct FrameData {
@@ -767,28 +729,30 @@ class UpcallStub: public RuntimeBlob {
   FrameData* frame_data_for_frame(const frame& frame) const;
  public:
   // Creation
-  static UpcallStub* create(const char* name, CodeBuffer* cb,
-                            intptr_t exception_handler_offset,
-                            jobject receiver, ByteSize frame_data_offset);
+  static UpcallStub* create(const char* name, CodeBuffer* cb, jobject receiver, ByteSize frame_data_offset);
 
   static void free(UpcallStub* blob);
 
-  address exception_handler() { return code_begin() + _exception_handler_offset; }
   jobject receiver() { return _receiver; }
 
   JavaFrameAnchor* jfa_for_frame(const frame& frame) const;
 
-  // Typing
-  virtual bool is_upcall_stub() const override { return true; }
-
-  // GC/Verification support
+  // GC support
   void oops_do(OopClosure* f, const frame& frame);
-  virtual void preserve_callee_argument_oops(frame fr, const RegisterMap* reg_map, OopClosure* f) override;
-  virtual void verify() override;
 
-  // Misc.
-  virtual void print_on(outputStream* st) const override;
-  virtual void print_value_on(outputStream* st) const override;
+  void print_on_impl(outputStream* st) const;
+  void print_value_on_impl(outputStream* st) const;
+
+  class Vptr : public RuntimeBlob::Vptr {
+    void print_on(const CodeBlob* instance, outputStream* st) const override {
+      instance->as_upcall_stub()->print_on_impl(st);
+    }
+    void print_value_on(const CodeBlob* instance, outputStream* st) const override {
+      instance->as_upcall_stub()->print_value_on_impl(st);
+    }
+  };
+
+  static const Vptr _vpntr;
 };
 
 #endif // SHARE_CODE_CODEBLOB_HPP
