@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,17 +22,22 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "code/codeCache.hpp"
 #include "code/compiledIC.hpp"
 #include "code/nmethod.hpp"
 #include "code/relocInfo.hpp"
 #include "memory/resourceArea.hpp"
+#include "memory/universe.hpp"
 #include "oops/compressedOops.inline.hpp"
+#include "oops/oop.inline.hpp"
 #include "runtime/flags/flagSetting.hpp"
 #include "runtime/stubCodeGenerator.hpp"
+#include "utilities/align.hpp"
+#include "utilities/checkedCast.hpp"
 #include "utilities/copy.hpp"
-#include "oops/oop.inline.hpp"
+
+#include <new>
+#include <type_traits>
 
 const RelocationHolder RelocationHolder::none; // its type is relocInfo::none
 
@@ -40,15 +45,18 @@ const RelocationHolder RelocationHolder::none; // its type is relocInfo::none
 // Implementation of relocInfo
 
 #ifdef ASSERT
-relocInfo::relocInfo(relocType t, int off, int f) {
-  assert(t != data_prefix_tag, "cannot build a prefix this way");
-  assert((t & type_mask) == t, "wrong type");
-  assert((f & format_mask) == f, "wrong format");
-  assert(off >= 0 && off < offset_limit(), "offset out off bounds");
-  assert((off & (offset_unit-1)) == 0, "misaligned offset");
-  (*this) = relocInfo(t, RAW_BITS, off, f);
+relocInfo::relocType relocInfo::check_relocType(relocType type) {
+  assert(type != data_prefix_tag, "cannot build a prefix this way");
+  assert((type & type_mask) == type, "wrong type");
+  return type;
 }
-#endif
+
+void relocInfo::check_offset_and_format(int offset, int format) {
+  assert(offset >= 0 && offset < offset_limit(), "offset out off bounds");
+  assert(is_aligned(offset, offset_unit), "misaligned offset");
+  assert((format & format_mask) == format, "wrong format");
+}
+#endif // ASSERT
 
 void relocInfo::initialize(CodeSection* dest, Relocation* reloc) {
   relocInfo* data = this+1;  // here's where the data might go
@@ -68,7 +76,7 @@ relocInfo* relocInfo::finish_prefix(short* prefix_limit) {
   assert(sizeof(relocInfo) == sizeof(short), "change this code");
   short* p = (short*)(this+1);
   assert(prefix_limit >= p, "must be a valid span of data");
-  int plen = prefix_limit - p;
+  int plen = checked_cast<int>(prefix_limit - p);
   if (plen == 0) {
     debug_only(_value = 0xFFFF);
     return this;                         // no data: remove self completely
@@ -78,7 +86,7 @@ relocInfo* relocInfo::finish_prefix(short* prefix_limit) {
     return this+1;
   }
   // cannot compact, so just update the count and return the limit pointer
-  (*this) = prefix_relocInfo(plen);   // write new datalen
+  (*this) = prefix_info(plen);       // write new datalen
   assert(data() + datalen() == prefix_limit, "pointers must line up");
   return (relocInfo*)prefix_limit;
 }
@@ -92,18 +100,6 @@ void relocInfo::set_type(relocType t) {
   assert(format()==old_format, "sanity check");
 }
 
-nmethod* RelocIterator::code_as_nmethod() const {
-  return _code->as_nmethod();
-}
-
-void relocInfo::set_format(int f) {
-  int old_offset = addr_offset();
-  assert((f & format_mask) == f, "wrong format");
-  _value = (_value & ~(format_mask << offset_width)) | (f << offset_width);
-  assert(addr_offset()==old_offset, "sanity check");
-}
-
-
 void relocInfo::change_reloc_info_for_address(RelocIterator *itr, address pc, relocType old_type, relocType new_type) {
   bool found = false;
   while (itr->next() && !found) {
@@ -117,23 +113,18 @@ void relocInfo::change_reloc_info_for_address(RelocIterator *itr, address pc, re
 }
 
 
-void relocInfo::remove_reloc_info_for_address(RelocIterator *itr, address pc, relocType old_type) {
-  change_reloc_info_for_address(itr, pc, old_type, none);
-}
-
-
 // ----------------------------------------------------------------------------------------------------
 // Implementation of RelocIterator
 
-void RelocIterator::initialize(CompiledMethod* nm, address begin, address limit) {
+void RelocIterator::initialize(nmethod* nm, address begin, address limit) {
   initialize_misc();
 
-  if (nm == NULL && begin != NULL) {
+  if (nm == nullptr && begin != nullptr) {
     // allow nmethod to be deduced from beginning address
     CodeBlob* cb = CodeCache::find_blob(begin);
-    nm = (cb != NULL) ? cb->as_compiled_method_or_null() : NULL;
+    nm = (cb != nullptr) ? cb->as_nmethod_or_null() : nullptr;
   }
-  guarantee(nm != NULL, "must be able to deduce nmethod from other arguments");
+  guarantee(nm != nullptr, "must be able to deduce nmethod from other arguments");
 
   _code    = nm;
   _current = nm->relocation_begin() - 1;
@@ -150,19 +141,19 @@ void RelocIterator::initialize(CompiledMethod* nm, address begin, address limit)
   _section_end  [CodeBuffer::SECT_STUBS ] = nm->stub_end()    ;
 
   assert(!has_current(), "just checking");
-  assert(begin == NULL || begin >= nm->code_begin(), "in bounds");
-  assert(limit == NULL || limit <= nm->code_end(),   "in bounds");
+  assert(begin == nullptr || begin >= nm->code_begin(), "in bounds");
+  assert(limit == nullptr || limit <= nm->code_end(),   "in bounds");
   set_limits(begin, limit);
 }
 
 
 RelocIterator::RelocIterator(CodeSection* cs, address begin, address limit) {
   initialize_misc();
-
+  assert(((cs->locs_start() != nullptr) && (cs->locs_end() != nullptr)), "valid start and end pointer");
   _current = cs->locs_start()-1;
   _end     = cs->locs_end();
   _addr    = cs->start();
-  _code    = NULL; // Not cb->blob();
+  _code    = nullptr; // Not cb->blob();
 
   CodeBuffer* cb = cs->outer();
   assert((int) SECT_LIMIT == CodeBuffer::SECT_LIMIT, "my copy must be equal");
@@ -174,19 +165,11 @@ RelocIterator::RelocIterator(CodeSection* cs, address begin, address limit) {
 
   assert(!has_current(), "just checking");
 
-  assert(begin == NULL || begin >= cs->start(), "in bounds");
-  assert(limit == NULL || limit <= cs->end(),   "in bounds");
+  assert(begin == nullptr || begin >= cs->start(), "in bounds");
+  assert(limit == nullptr || limit <= cs->end(),   "in bounds");
   set_limits(begin, limit);
 }
 
-
-enum { indexCardSize = 128 };
-struct RelocIndexEntry {
-  jint addr_offset;          // offset from header_end of an addr()
-  jint reloc_offset;         // offset from header_end of a relocInfo (prefix)
-};
-
-
 bool RelocIterator::addr_in_const() const {
   const int n = CodeBuffer::SECT_CONSTS;
   return section_start(n) <= addr() && addr() < section_end(n);
@@ -197,7 +180,7 @@ void RelocIterator::set_limits(address begin, address limit) {
   _limit = limit;
 
   // the limit affects this next stuff:
-  if (begin != NULL) {
+  if (begin != nullptr) {
     relocInfo* backup;
     address    backup_addr;
     while (true) {
@@ -207,7 +190,7 @@ void RelocIterator::set_limits(address begin, address limit) {
     }
     // At this point, either we are at the first matching record,
     // or else there is no such record, and !has_current().
-    // In either case, revert to the immediatly preceding state.
+    // In either case, revert to the immediately preceding state.
     _current = backup;
     _addr    = backup_addr;
     set_has_current(false);
@@ -215,12 +198,6 @@ void RelocIterator::set_limits(address begin, address limit) {
 }
 
 
-void RelocIterator::set_limit(address limit) {
-  address code_end = (address)code() + code()->size();
-  assert(limit == NULL || limit <= code_end, "in bounds");
-  _limit = limit;
-}
-
 // All the strange bit-encodings are in here.
 // The idea is to encode relocation data which are small integers
 // very efficiently (a single extra halfword).  Larger chunks of
@@ -244,8 +221,8 @@ void RelocIterator::advance_over_prefix() {
 void RelocIterator::initialize_misc() {
   set_has_current(false);
   for (int i = (int) CodeBuffer::SECT_FIRST; i < (int) CodeBuffer::SECT_LIMIT; i++) {
-    _section_start[i] = NULL;  // these will be lazily computed, if needed
-    _section_end  [i] = NULL;
+    _section_start[i] = nullptr;  // these will be lazily computed, if needed
+    _section_end  [i] = nullptr;
   }
 }
 
@@ -261,44 +238,49 @@ Relocation* RelocIterator::reloc() {
   APPLY_TO_RELOCATIONS(EACH_TYPE);
   #undef EACH_TYPE
   assert(t == relocInfo::none, "must be padding");
-  return new(_rh) Relocation();
+  _rh = RelocationHolder::none;
+  return _rh.reloc();
 }
 
+// Verify all the destructors are trivial, so we don't need to worry about
+// destroying old contents of a RelocationHolder being assigned or destroyed.
+#define VERIFY_TRIVIALLY_DESTRUCTIBLE_AUX(Reloc) \
+  static_assert(std::is_trivially_destructible<Reloc>::value, "must be");
+
+#define VERIFY_TRIVIALLY_DESTRUCTIBLE(name) \
+  VERIFY_TRIVIALLY_DESTRUCTIBLE_AUX(PASTE_TOKENS(name, _Relocation));
+
+APPLY_TO_RELOCATIONS(VERIFY_TRIVIALLY_DESTRUCTIBLE)
+VERIFY_TRIVIALLY_DESTRUCTIBLE_AUX(Relocation)
+
+#undef VERIFY_TRIVIALLY_DESTRUCTIBLE_AUX
+#undef VERIFY_TRIVIALLY_DESTRUCTIBLE
+
+// Define all the copy_into functions.  These rely on all Relocation types
+// being trivially destructible (verified above).  So it doesn't matter
+// whether the target holder has been previously initialized or not.  There's
+// no need to consider that distinction and destruct the relocation in an
+// already initialized holder.
+#define DEFINE_COPY_INTO_AUX(Reloc)                             \
+  void Reloc::copy_into(RelocationHolder& holder) const {       \
+    copy_into_helper(*this, holder);                            \
+  }
+
+#define DEFINE_COPY_INTO(name) \
+  DEFINE_COPY_INTO_AUX(PASTE_TOKENS(name, _Relocation))
+
+APPLY_TO_RELOCATIONS(DEFINE_COPY_INTO)
+DEFINE_COPY_INTO_AUX(Relocation)
+
+#undef DEFINE_COPY_INTO_AUX
+#undef DEFINE_COPY_INTO
 
 //////// Methods for flyweight Relocation types
 
-
-RelocationHolder RelocationHolder::plus(int offset) const {
-  if (offset != 0) {
-    switch (type()) {
-    case relocInfo::none:
-      break;
-    case relocInfo::oop_type:
-      {
-        oop_Relocation* r = (oop_Relocation*)reloc();
-        return oop_Relocation::spec(r->oop_index(), r->offset() + offset);
-      }
-    case relocInfo::metadata_type:
-      {
-        metadata_Relocation* r = (metadata_Relocation*)reloc();
-        return metadata_Relocation::spec(r->metadata_index(), r->offset() + offset);
-      }
-    default:
-      ShouldNotReachHere();
-    }
-  }
-  return (*this);
-}
-
-
-void Relocation::guarantee_size() {
-  guarantee(false, "Make _relocbuf bigger!");
-}
-
-    // some relocations can compute their own values
+// some relocations can compute their own values
 address Relocation::value() {
   ShouldNotReachHere();
-  return NULL;
+  return nullptr;
 }
 
 
@@ -309,7 +291,7 @@ void Relocation::set_value(address x) {
 void Relocation::const_set_data_value(address x) {
 #ifdef _LP64
   if (format() == relocInfo::narrow_oop_in_const) {
-    *(narrowOop*)addr() = CompressedOops::encode((oop) x);
+    *(narrowOop*)addr() = CompressedOops::encode(cast_to_oop(x));
   } else {
 #endif
     *(address*)addr() = x;
@@ -321,7 +303,7 @@ void Relocation::const_set_data_value(address x) {
 void Relocation::const_verify_data_value(address x) {
 #ifdef _LP64
   if (format() == relocInfo::narrow_oop_in_const) {
-    guarantee(*(narrowOop*)addr() == CompressedOops::encode((oop) x), "must agree");
+    guarantee(*(narrowOop*)addr() == CompressedOops::encode(cast_to_oop(x)), "must agree");
   } else {
 #endif
     guarantee(*(address*)addr() == x, "must agree");
@@ -355,7 +337,7 @@ address Relocation::new_addr_for(address olda,
   int sect = CodeBuffer::SECT_NONE;
   // Look for olda in the source buffer, and all previous incarnations
   // if the source buffer has been expanded.
-  for (; src != NULL; src = src->before_expand()) {
+  for (; src != nullptr; src = src->before_expand()) {
     sect = src->section_index_of(olda);
     if (sect != CodeBuffer::SECT_NONE)  break;
   }
@@ -367,7 +349,7 @@ address Relocation::new_addr_for(address olda,
 
 void Relocation::normalize_address(address& addr, const CodeSection* dest, bool allow_other_sections) {
   address addr0 = addr;
-  if (addr0 == NULL || dest->allocates2(addr0))  return;
+  if (addr0 == nullptr || dest->allocates2(addr0))  return;
   CodeBuffer* cb = dest->outer();
   addr = new_addr_for(addr0, cb, cb);
   assert(allow_other_sections || dest->contains2(addr),
@@ -390,28 +372,36 @@ void CallRelocation::fix_relocation_after_move(const CodeBuffer* src, CodeBuffer
 }
 
 
+#ifdef USE_TRAMPOLINE_STUB_FIX_OWNER
+void trampoline_stub_Relocation::fix_relocation_after_move(const CodeBuffer* src, CodeBuffer* dest) {
+  // Finalize owner destination only for nmethods
+  if (dest->blob() != nullptr) return;
+  pd_fix_owner_after_move();
+}
+#endif
+
 //// pack/unpack methods
 
 void oop_Relocation::pack_data_to(CodeSection* dest) {
   short* p = (short*) dest->locs_end();
-  p = pack_2_ints_to(p, _oop_index, _offset);
+  p = pack_1_int_to(p, _oop_index);
   dest->set_locs_end((relocInfo*) p);
 }
 
 
 void oop_Relocation::unpack_data() {
-  unpack_2_ints(_oop_index, _offset);
+  _oop_index = unpack_1_int();
 }
 
 void metadata_Relocation::pack_data_to(CodeSection* dest) {
   short* p = (short*) dest->locs_end();
-  p = pack_2_ints_to(p, _metadata_index, _offset);
+  p = pack_1_int_to(p, _metadata_index);
   dest->set_locs_end((relocInfo*) p);
 }
 
 
 void metadata_Relocation::unpack_data() {
-  unpack_2_ints(_metadata_index, _offset);
+  _metadata_index = unpack_1_int();
 }
 
 
@@ -430,7 +420,7 @@ void virtual_call_Relocation::unpack_data() {
   jint x0 = 0;
   unpack_2_ints(x0, _method_index);
   address point = addr();
-  _cached_value = x0==0? NULL: address_from_scaled_offset(x0, point);
+  _cached_value = x0==0? nullptr: address_from_scaled_offset(x0, point);
 }
 
 void runtime_call_w_cp_Relocation::pack_data_to(CodeSection * dest) {
@@ -446,18 +436,14 @@ void static_stub_Relocation::pack_data_to(CodeSection* dest) {
   short* p = (short*) dest->locs_end();
   CodeSection* insts = dest->outer()->insts();
   normalize_address(_static_call, insts);
-  jint is_aot = _is_aot ? 1 : 0;
-  p = pack_2_ints_to(p, scaled_offset(_static_call, insts->start()), is_aot);
+  p = pack_1_int_to(p, scaled_offset(_static_call, insts->start()));
   dest->set_locs_end((relocInfo*) p);
 }
 
 void static_stub_Relocation::unpack_data() {
   address base = binding()->section_start(CodeBuffer::SECT_INSTS);
-  jint offset;
-  jint is_aot;
-  unpack_2_ints(offset, is_aot);
+  jint offset = unpack_1_int();
   _static_call = address_from_scaled_offset(offset, base);
-  _is_aot = (is_aot == 1);
 }
 
 void trampoline_stub_Relocation::pack_data_to(CodeSection* dest ) {
@@ -475,27 +461,15 @@ void trampoline_stub_Relocation::unpack_data() {
 
 void external_word_Relocation::pack_data_to(CodeSection* dest) {
   short* p = (short*) dest->locs_end();
-#ifndef _LP64
-  p = pack_1_int_to(p, (int32_t) (intptr_t)_target);
-#else
-  jlong t = (jlong) _target;
-  int32_t lo = low(t);
-  int32_t hi = high(t);
-  p = pack_2_ints_to(p, lo, hi);
-#endif /* _LP64 */
+  int index = ExternalsRecorder::find_index(_target);
+  p = pack_1_int_to(p, index);
   dest->set_locs_end((relocInfo*) p);
 }
 
 
 void external_word_Relocation::unpack_data() {
-#ifndef _LP64
-  _target = (address) (intptr_t)unpack_1_int();
-#else
-  jint lo, hi;
-  unpack_2_ints(lo, hi);
-  jlong t = jlong_from(hi, lo);;
-  _target = (address) t;
-#endif /* _LP64 */
+  int index = unpack_1_int();
+  _target = ExternalsRecorder::at(index);
 }
 
 
@@ -506,7 +480,7 @@ void internal_word_Relocation::pack_data_to(CodeSection* dest) {
   // Check whether my target address is valid within this section.
   // If not, strengthen the relocation type to point to another section.
   int sindex = _section;
-  if (sindex == CodeBuffer::SECT_NONE && _target != NULL
+  if (sindex == CodeBuffer::SECT_NONE && _target != nullptr
       && (!dest->allocates(_target) || _target == dest->locs_point())) {
     sindex = dest->outer()->section_index_of(_target);
     guarantee(sindex != CodeBuffer::SECT_NONE, "must belong somewhere");
@@ -522,12 +496,12 @@ void internal_word_Relocation::pack_data_to(CodeSection* dest) {
 
   if (sindex == CodeBuffer::SECT_NONE) {
     assert(type() == relocInfo::internal_word_type, "must be base class");
-    guarantee(_target == NULL || dest->allocates2(_target), "must be within the given code section");
+    guarantee(_target == nullptr || dest->allocates2(_target), "must be within the given code section");
     jint x0 = scaled_offset_null_special(_target, dest->locs_point());
-    assert(!(x0 == 0 && _target != NULL), "correct encoding of null target");
+    assert(!(x0 == 0 && _target != nullptr), "correct encoding of null target");
     p = pack_1_int_to(p, x0);
   } else {
-    assert(_target != NULL, "sanity");
+    assert(_target != nullptr, "sanity");
     CodeSection* sect = dest->outer()->code_section(sindex);
     guarantee(sect->allocates2(_target), "must be in correct section");
     address base = sect->start();
@@ -543,7 +517,7 @@ void internal_word_Relocation::pack_data_to(CodeSection* dest) {
 
 void internal_word_Relocation::unpack_data() {
   jint x0 = unpack_1_int();
-  _target = x0==0? NULL: address_from_scaled_offset(x0, addr());
+  _target = x0==0? nullptr: address_from_scaled_offset(x0, addr());
   _section = CodeBuffer::SECT_NONE;
 }
 
@@ -572,10 +546,11 @@ oop* oop_Relocation::oop_addr() {
 
 
 oop oop_Relocation::oop_value() {
-  oop v = *oop_addr();
   // clean inline caches store a special pseudo-null
-  if (v == (oop)Universe::non_oop_word())  v = NULL;
-  return v;
+  if (Universe::contains_non_oop_word(oop_addr())) {
+    return nullptr;
+  }
+  return *oop_addr();
 }
 
 
@@ -610,7 +585,7 @@ Metadata** metadata_Relocation::metadata_addr() {
 Metadata* metadata_Relocation::metadata_value() {
   Metadata* v = *metadata_addr();
   // clean inline caches store a special pseudo-null
-  if (v == (Metadata*)Universe::non_oop_word())  v = NULL;
+  if (v == (Metadata*)Universe::non_oop_word())  v = nullptr;
   return v;
   }
 
@@ -622,31 +597,21 @@ void metadata_Relocation::fix_metadata_relocation() {
   }
 }
 
-
-void metadata_Relocation::verify_metadata_relocation() {
-  if (!metadata_is_immediate()) {
-    // get the metadata from the pool, and re-insert it into the instruction:
-    verify_value(value());
-  }
-}
-
 address virtual_call_Relocation::cached_value() {
-  assert(_cached_value != NULL && _cached_value < addr(), "must precede ic_call");
+  assert(_cached_value != nullptr && _cached_value < addr(), "must precede ic_call");
   return _cached_value;
 }
 
 Method* virtual_call_Relocation::method_value() {
-  CompiledMethod* cm = code();
-  if (cm == NULL) return (Method*)NULL;
-  Metadata* m = cm->metadata_at(_method_index);
-  assert(m != NULL || _method_index == 0, "should be non-null for non-zero index");
-  assert(m == NULL || m->is_method(), "not a method");
+  nmethod* nm = code();
+  if (nm == nullptr) return (Method*)nullptr;
+  Metadata* m = nm->metadata_at(_method_index);
+  assert(m != nullptr || _method_index == 0, "should be non-null for non-zero index");
+  assert(m == nullptr || m->is_method(), "not a method");
   return (Method*)m;
 }
 
 void virtual_call_Relocation::clear_inline_cache() {
-  // No stubs for ICs
-  // Clean IC
   ResourceMark rm;
   CompiledIC* icache = CompiledIC_at(this);
   icache->set_to_clean();
@@ -664,44 +629,41 @@ void opt_virtual_call_Relocation::unpack_data() {
 }
 
 Method* opt_virtual_call_Relocation::method_value() {
-  CompiledMethod* cm = code();
-  if (cm == NULL) return (Method*)NULL;
-  Metadata* m = cm->metadata_at(_method_index);
-  assert(m != NULL || _method_index == 0, "should be non-null for non-zero index");
-  assert(m == NULL || m->is_method(), "not a method");
+  nmethod* nm = code();
+  if (nm == nullptr) return (Method*)nullptr;
+  Metadata* m = nm->metadata_at(_method_index);
+  assert(m != nullptr || _method_index == 0, "should be non-null for non-zero index");
+  assert(m == nullptr || m->is_method(), "not a method");
   return (Method*)m;
 }
 
 void opt_virtual_call_Relocation::clear_inline_cache() {
-  // No stubs for ICs
-  // Clean IC
   ResourceMark rm;
-  CompiledIC* icache = CompiledIC_at(this);
-  icache->set_to_clean();
+  CompiledDirectCall* callsite = CompiledDirectCall::at(this);
+  callsite->set_to_clean();
 }
 
-
-address opt_virtual_call_Relocation::static_stub(bool is_aot) {
+address opt_virtual_call_Relocation::static_stub() {
   // search for the static stub who points back to this static call
   address static_call_addr = addr();
   RelocIterator iter(code());
   while (iter.next()) {
     if (iter.type() == relocInfo::static_stub_type) {
       static_stub_Relocation* stub_reloc = iter.static_stub_reloc();
-      if (stub_reloc->static_call() == static_call_addr && stub_reloc->is_aot() == is_aot) {
+      if (stub_reloc->static_call() == static_call_addr) {
         return iter.addr();
       }
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 Method* static_call_Relocation::method_value() {
-  CompiledMethod* cm = code();
-  if (cm == NULL) return (Method*)NULL;
-  Metadata* m = cm->metadata_at(_method_index);
-  assert(m != NULL || _method_index == 0, "should be non-null for non-zero index");
-  assert(m == NULL || m->is_method(), "not a method");
+  nmethod* nm = code();
+  if (nm == nullptr) return (Method*)nullptr;
+  Metadata* m = nm->metadata_at(_method_index);
+  assert(m != nullptr || _method_index == 0, "should be non-null for non-zero index");
+  assert(m == nullptr || m->is_method(), "not a method");
   return (Method*)m;
 }
 
@@ -716,34 +678,34 @@ void static_call_Relocation::unpack_data() {
 }
 
 void static_call_Relocation::clear_inline_cache() {
-  // Safe call site info
-  CompiledStaticCall* handler = this->code()->compiledStaticCall_at(this);
-  handler->set_to_clean();
+  ResourceMark rm;
+  CompiledDirectCall* callsite = CompiledDirectCall::at(this);
+  callsite->set_to_clean();
 }
 
 
-address static_call_Relocation::static_stub(bool is_aot) {
+address static_call_Relocation::static_stub() {
   // search for the static stub who points back to this static call
   address static_call_addr = addr();
   RelocIterator iter(code());
   while (iter.next()) {
     if (iter.type() == relocInfo::static_stub_type) {
       static_stub_Relocation* stub_reloc = iter.static_stub_reloc();
-      if (stub_reloc->static_call() == static_call_addr && stub_reloc->is_aot() == is_aot) {
+      if (stub_reloc->static_call() == static_call_addr) {
         return iter.addr();
       }
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 // Finds the trampoline address for a call. If no trampoline stub is
-// found NULL is returned which can be handled by the caller.
+// found nullptr is returned which can be handled by the caller.
 address trampoline_stub_Relocation::get_trampoline_for(address call, nmethod* code) {
   // There are no relocations available when the code gets relocated
   // because of CodeBuffer expansion.
   if (code->relocation_size() == 0)
-    return NULL;
+    return nullptr;
 
   RelocIterator iter(code, call);
   while (iter.next()) {
@@ -754,23 +716,23 @@ address trampoline_stub_Relocation::get_trampoline_for(address call, nmethod* co
     }
   }
 
-  return NULL;
+  return nullptr;
 }
 
 void static_stub_Relocation::clear_inline_cache() {
   // Call stub is only used when calling the interpreted code.
   // It does not really need to be cleared, except that we want to clean out the methodoop.
-  CompiledDirectStaticCall::set_stub_to_clean(this);
+  CompiledDirectCall::set_stub_to_clean(this);
 }
 
 
 void external_word_Relocation::fix_relocation_after_move(const CodeBuffer* src, CodeBuffer* dest) {
-  if (_target != NULL) {
+  if (_target != nullptr) {
     // Probably this reference is absolute,  not relative, so the following is
     // probably a no-op.
     set_value(_target);
   }
-  // If target is NULL, this is  an absolute embedded reference to an external
+  // If target is nullptr, this is  an absolute embedded reference to an external
   // location, which means  there is nothing to fix here.  In either case, the
   // resulting target should be an "external" address.
   postcond(src->section_index_of(target()) == CodeBuffer::SECT_NONE);
@@ -780,7 +742,7 @@ void external_word_Relocation::fix_relocation_after_move(const CodeBuffer* src,
 
 address external_word_Relocation::target() {
   address target = _target;
-  if (target == NULL) {
+  if (target == nullptr) {
     target = pd_get_address_from_code();
   }
   return target;
@@ -789,7 +751,7 @@ address external_word_Relocation::target() {
 
 void internal_word_Relocation::fix_relocation_after_move(const CodeBuffer* src, CodeBuffer* dest) {
   address target = _target;
-  if (target == NULL) {
+  if (target == nullptr) {
     target = new_addr_for(this->target(), src, dest);
   }
   set_value(target);
@@ -798,7 +760,7 @@ void internal_word_Relocation::fix_relocation_after_move(const CodeBuffer* src,
 
 address internal_word_Relocation::target() {
   address target = _target;
-  if (target == NULL) {
+  if (target == nullptr) {
     if (addr_in_const()) {
       target = *(address*)addr();
     } else {
@@ -855,19 +817,19 @@ void RelocIterator::print_current() {
   case relocInfo::oop_type:
     {
       oop_Relocation* r = oop_reloc();
-      oop* oop_addr  = NULL;
-      oop  raw_oop   = NULL;
-      oop  oop_value = NULL;
-      if (code() != NULL || r->oop_is_immediate()) {
+      oop* oop_addr  = nullptr;
+      oop  raw_oop   = nullptr;
+      oop  oop_value = nullptr;
+      if (code() != nullptr || r->oop_is_immediate()) {
         oop_addr  = r->oop_addr();
         raw_oop   = *oop_addr;
         oop_value = r->oop_value();
       }
-      tty->print(" | [oop_addr=" INTPTR_FORMAT " *=" INTPTR_FORMAT " offset=%d]",
-                 p2i(oop_addr), p2i(raw_oop), r->offset());
+      tty->print(" | [oop_addr=" INTPTR_FORMAT " *=" INTPTR_FORMAT "]",
+                 p2i(oop_addr), p2i(raw_oop));
       // Do not print the oop by default--we want this routine to
       // work even during GC or other inconvenient times.
-      if (WizardMode && oop_value != NULL) {
+      if (WizardMode && oop_value != nullptr) {
         tty->print("oop_value=" INTPTR_FORMAT ": ", p2i(oop_value));
         if (oopDesc::is_oop(oop_value)) {
           oop_value->print_value_on(tty);
@@ -878,17 +840,17 @@ void RelocIterator::print_current() {
   case relocInfo::metadata_type:
     {
       metadata_Relocation* r = metadata_reloc();
-      Metadata** metadata_addr  = NULL;
-      Metadata*    raw_metadata   = NULL;
-      Metadata*    metadata_value = NULL;
-      if (code() != NULL || r->metadata_is_immediate()) {
+      Metadata** metadata_addr  = nullptr;
+      Metadata*    raw_metadata   = nullptr;
+      Metadata*    metadata_value = nullptr;
+      if (code() != nullptr || r->metadata_is_immediate()) {
         metadata_addr  = r->metadata_addr();
         raw_metadata   = *metadata_addr;
         metadata_value = r->metadata_value();
       }
-      tty->print(" | [metadata_addr=" INTPTR_FORMAT " *=" INTPTR_FORMAT " offset=%d]",
-                 p2i(metadata_addr), p2i(raw_metadata), r->offset());
-      if (metadata_value != NULL) {
+      tty->print(" | [metadata_addr=" INTPTR_FORMAT " *=" INTPTR_FORMAT "]",
+                 p2i(metadata_addr), p2i(raw_metadata));
+      if (metadata_value != nullptr) {
         tty->print("metadata_value=" INTPTR_FORMAT ": ", p2i(metadata_value));
         metadata_value->print_value_on(tty);
       }
