@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2023, 2025, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2017, 2022, Red Hat, Inc. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -36,7 +37,6 @@
 class EpsilonHeap : public CollectedHeap {
   friend class VMStructs;
 private:
-  SoftRefPolicy _soft_ref_policy;
   EpsilonMonitoringSupport* _monitoring_support;
   MemoryPool* _pool;
   GCMemoryManager _memory_manager;
@@ -53,93 +53,88 @@ public:
   static EpsilonHeap* heap();
 
   EpsilonHeap() :
-          _memory_manager("Epsilon Heap", ""),
-          _space(NULL) {};
+          _memory_manager("Epsilon Heap"),
+          _space(nullptr) {};
 
-  virtual Name kind() const {
+  Name kind() const override {
     return CollectedHeap::Epsilon;
   }
 
-  virtual const char* name() const {
+  const char* name() const override {
     return "Epsilon";
   }
 
-  virtual SoftRefPolicy* soft_ref_policy() {
-    return &_soft_ref_policy;
-  }
+  jint initialize() override;
+  void initialize_serviceability() override;
 
-  virtual jint initialize();
-  virtual void post_initialize();
-  virtual void initialize_serviceability();
+  GrowableArray<GCMemoryManager*> memory_managers() override;
+  GrowableArray<MemoryPool*> memory_pools() override;
 
-  virtual GrowableArray<GCMemoryManager*> memory_managers();
-  virtual GrowableArray<MemoryPool*> memory_pools();
+  size_t max_capacity() const override { return _virtual_space.reserved_size();  }
+  size_t capacity()     const override { return _virtual_space.committed_size(); }
+  size_t used()         const override { return _space->used(); }
 
-  virtual size_t max_capacity() const { return _virtual_space.reserved_size();  }
-  virtual size_t capacity()     const { return _virtual_space.committed_size(); }
-  virtual size_t used()         const { return _space->used(); }
-
-  virtual bool is_in(const void* p) const {
+  bool is_in(const void* p) const override {
     return _space->is_in(p);
   }
 
-  virtual bool requires_barriers(stackChunkOop obj) const { return false; }
+  bool requires_barriers(stackChunkOop obj) const override { return false; }
 
-  virtual bool is_maximal_no_gc() const {
+  bool is_maximal_no_gc() const override {
     // No GC is going to happen. Return "we are at max", when we are about to fail.
     return used() == capacity();
   }
 
   // Allocation
   HeapWord* allocate_work(size_t size, bool verbose = true);
-  virtual HeapWord* mem_allocate(size_t size, bool* gc_overhead_limit_was_exceeded);
-  virtual HeapWord* allocate_new_tlab(size_t min_size,
-                                      size_t requested_size,
-                                      size_t* actual_size);
+  HeapWord* mem_allocate(size_t size, bool* gc_overhead_limit_was_exceeded) override;
+  HeapWord* allocate_new_tlab(size_t min_size,
+                              size_t requested_size,
+                              size_t* actual_size) override;
 
   // TLAB allocation
-  virtual size_t tlab_capacity(Thread* thr)         const { return capacity();     }
-  virtual size_t tlab_used(Thread* thr)             const { return used();         }
-  virtual size_t max_tlab_size()                    const { return _max_tlab_size; }
-  virtual size_t unsafe_max_tlab_alloc(Thread* thr) const;
+  size_t tlab_capacity(Thread* thr)         const override { return capacity();     }
+  size_t tlab_used(Thread* thr)             const override { return used();         }
+  size_t max_tlab_size()                    const override { return _max_tlab_size; }
+  size_t unsafe_max_tlab_alloc(Thread* thr) const override;
 
-  virtual void collect(GCCause::Cause cause);
-  virtual void do_full_collection(bool clear_all_soft_refs);
+  void collect(GCCause::Cause cause) override;
+  void do_full_collection(bool clear_all_soft_refs) override;
 
   // Heap walking support
-  virtual void object_iterate(ObjectClosure* cl);
+  void object_iterate(ObjectClosure* cl) override;
 
   // Object pinning support: every object is implicitly pinned
-  virtual bool supports_object_pinning() const           { return true; }
-  virtual oop pin_object(JavaThread* thread, oop obj)    { return obj; }
-  virtual void unpin_object(JavaThread* thread, oop obj) { }
+  void pin_object(JavaThread* thread, oop obj) override { }
+  void unpin_object(JavaThread* thread, oop obj) override { }
 
   // No support for block parsing.
-  HeapWord* block_start(const void* addr) const { return NULL;  }
+  HeapWord* block_start(const void* addr) const { return nullptr;  }
   bool block_is_obj(const HeapWord* addr) const { return false; }
 
   // No GC threads
-  virtual void gc_threads_do(ThreadClosure* tc) const {}
+  void gc_threads_do(ThreadClosure* tc) const override {}
 
   // No nmethod handling
-  virtual void register_nmethod(nmethod* nm) {}
-  virtual void unregister_nmethod(nmethod* nm) {}
-  virtual void verify_nmethod(nmethod* nm) {}
+  void register_nmethod(nmethod* nm) override {}
+  void unregister_nmethod(nmethod* nm) override {}
+  void verify_nmethod(nmethod* nm) override {}
 
   // No heap verification
-  virtual void prepare_for_verify() {}
-  virtual void verify(VerifyOption option) {}
+  void prepare_for_verify() override {}
+  void verify(VerifyOption option) override {}
 
   MemRegion reserved_region() const { return _reserved; }
   bool is_in_reserved(const void* addr) const { return _reserved.contains(addr); }
 
   // Support for loading objects from CDS archive into the heap
-  virtual bool can_load_archived_objects() const { return UseCompressedOops; }
-  virtual HeapWord* allocate_loaded_archive_space(size_t size);
+  bool can_load_archived_objects() const override { return true; }
+  HeapWord* allocate_loaded_archive_space(size_t size) override;
 
-  virtual void print_on(outputStream* st) const;
-  virtual void print_tracing_info() const;
-  virtual bool print_location(outputStream* st, void* addr) const;
+  void print_heap_on(outputStream* st) const override;
+  void print_gc_on(outputStream* st) const override {}
+  void print_tracing_info() const override;
+  bool print_location(outputStream* st, void* addr) const override;
 
 private:
   void print_heap_info(size_t used) const;
