@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,11 +22,10 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "gc/g1/g1Analytics.hpp"
 #include "gc/g1/g1CollectedHeap.inline.hpp"
-#include "gc/g1/g1CollectionSet.hpp"
-#include "gc/g1/g1CollectionSetCandidates.hpp"
+#include "gc/g1/g1CollectionSet.inline.hpp"
+#include "gc/g1/g1CollectionSetCandidates.inline.hpp"
 #include "gc/g1/g1CollectorState.hpp"
 #include "gc/g1/g1HeapRegion.inline.hpp"
 #include "gc/g1/g1HeapRegionRemSet.inline.hpp"
@@ -54,10 +53,13 @@ G1CollectionSet::G1CollectionSet(G1CollectedHeap* g1h, G1Policy* policy) :
   _collection_set_regions(nullptr),
   _collection_set_cur_length(0),
   _collection_set_max_length(0),
+  _collection_set_groups(),
+  _selected_groups_cur_length(0),
+  _selected_groups_inc_part_start(0),
   _eden_region_length(0),
   _survivor_region_length(0),
   _initial_old_region_length(0),
-  _optional_old_regions(),
+  _optional_groups(),
   _inc_build_state(Inactive),
   _inc_part_start(0) {
 }
@@ -78,7 +80,8 @@ void G1CollectionSet::init_region_lengths(uint eden_cset_region_length,
          "Young region length %u should match collection set length %u", young_region_length(), _collection_set_cur_length);
 
   _initial_old_region_length = 0;
-  _optional_old_regions.clear();
+  assert(_optional_groups.length() == 0, "Should not have any optional groups yet");
+  _optional_groups.clear();
 }
 
 void G1CollectionSet::initialize(uint max_region_length) {
@@ -92,7 +95,10 @@ void G1CollectionSet::initialize(uint max_region_length) {
 void G1CollectionSet::abandon_all_candidates() {
   _candidates.clear();
   _initial_old_region_length = 0;
-  _optional_old_regions.clear();
+}
+
+void G1CollectionSet::prepare_groups_for_scan () {
+  collection_set_groups()->prepare_for_scan();
 }
 
 void G1CollectionSet::add_old_region(G1HeapRegion* hr) {
@@ -102,6 +108,8 @@ void G1CollectionSet::add_old_region(G1HeapRegion* hr) {
          "Precondition, actively building cset or adding optional later on");
   assert(hr->is_old(), "the region should be old");
 
+  assert(!hr->rem_set()->is_added_to_cset_group(), "Should have already uninstalled group remset");
+
   assert(!hr->in_collection_set(), "should not already be in the collection set");
   _g1h->register_old_region_with_region_attr(hr);
 
@@ -127,6 +135,7 @@ void G1CollectionSet::finalize_incremental_building() {
 void G1CollectionSet::clear() {
   assert_at_safepoint_on_vm_thread();
   _collection_set_cur_length = 0;
+  _collection_set_groups.clear();
 }
 
 void G1CollectionSet::iterate(G1HeapRegionClosure* cl) const {
@@ -152,10 +161,10 @@ void G1CollectionSet::par_iterate(G1HeapRegionClosure* cl,
 void G1CollectionSet::iterate_optional(G1HeapRegionClosure* cl) const {
   assert_at_safepoint();
 
-  for (G1HeapRegion* r : _optional_old_regions) {
+  _optional_groups.iterate([&] (G1HeapRegion* r) {
     bool result = cl->do_heap_region(r);
     guarantee(!result, "Must not cancel iteration");
-  }
+  });
 }
 
 void G1CollectionSet::iterate_incremental_part_from(G1HeapRegionClosure* cl,
@@ -284,7 +293,7 @@ double G1CollectionSet::finalize_young_part(double target_pause_time_ms, G1Survi
 
   size_t pending_cards = _policy->pending_cards_at_gc_start();
 
-  log_trace(gc, ergo, cset)("Start choosing CSet. Pending cards: " SIZE_FORMAT " target pause time: %1.2fms",
+  log_trace(gc, ergo, cset)("Start choosing CSet. Pending cards: %zu target pause time: %1.2fms",
                             pending_cards, target_pause_time_ms);
 
   // The young list is laid with the survivor regions from the previous
@@ -321,63 +330,40 @@ static int compare_region_idx(const uint a, const uint b) {
   return static_cast<int>(a-b);
 }
 
-// The current mechanism skips evacuation of pinned old regions like g1 does for
-// young regions:
-// * evacuating pinned marking collection set candidate regions (available during mixed
-//   gc) like young regions would not result in any memory gain but only take additional
-//   time away from processing regions that would actually result in memory being freed.
-//   To advance mixed gc progress (we committed to evacuate all marking collection set
-//   candidate regions within the maximum number of mixed gcs in the phase), move them
-//   to the optional collection set candidates to reclaim them asap as time permits.
-// * evacuating out retained collection set candidates would also just take up time with
-//   no actual space freed in old gen. Better to concentrate on others.
-//   Retained collection set candidates are aged out, ie. made to regular old regions
-//   without remembered sets after a few attempts to save computation costs of keeping
-//   them candidates for very long living pinned regions.
+// The current mechanism for evacuating pinned old regions is as below:
+// * pinned regions in the marking collection set candidate list (available during mixed gc) are evacuated like
+//   pinned young regions to avoid the complexity of dealing with pinned regions that are part of a
+//   collection group sharing a single cardset. These regions will be partially evacuated and added to the
+//   retained collection set by the evacuation failure handling mechanism.
+// * evacuating pinned regions out of retained collection set candidates would also just take up time
+//   with no actual space freed in old gen. Better to concentrate on others. So we skip over pinned
+//   regions in retained collection set candidates. Retained collection set candidates are aged out, ie.
+//   made to regular old regions without remembered sets after a few attempts to save computation costs
+//   of keeping them candidates for very long living pinned regions.
 void G1CollectionSet::finalize_old_part(double time_remaining_ms) {
   double non_young_start_time_sec = os::elapsedTime();
 
+  _selected_groups_cur_length = 0;
+  _selected_groups_inc_part_start = 0;
+
   if (!candidates()->is_empty()) {
     candidates()->verify();
 
-    G1CollectionCandidateRegionList initial_old_regions;
-    assert(_optional_old_regions.length() == 0, "must be");
-    G1CollectionCandidateRegionList pinned_marking_regions;
-    G1CollectionCandidateRegionList pinned_retained_regions;
-
     if (collector_state()->in_mixed_phase()) {
-      time_remaining_ms = _policy->select_candidates_from_marking(&candidates()->marking_regions(),
-                                                                  time_remaining_ms,
-                                                                  &initial_old_regions,
-                                                                  &_optional_old_regions,
-                                                                  &pinned_marking_regions);
+      time_remaining_ms = select_candidates_from_marking(time_remaining_ms);
     } else {
       log_debug(gc, ergo, cset)("Do not add marking candidates to collection set due to pause type.");
     }
 
-    _policy->select_candidates_from_retained(&candidates()->retained_regions(),
-                                             time_remaining_ms,
-                                             &initial_old_regions,
-                                             &_optional_old_regions,
-                                             &pinned_retained_regions);
-
-    // Move initially selected old regions to collection set directly.
-    move_candidates_to_collection_set(&initial_old_regions);
-    // Only prepare selected optional regions for now.
-    prepare_optional_regions(&_optional_old_regions);
-    // Move pinned marking regions we came across to retained candidates so that
-    // there is progress in the mixed gc phase.
-    move_pinned_marking_to_retained(&pinned_marking_regions);
-    // Drop pinned retained regions to make progress with retained regions. Regions
-    // in that list must have been pinned for at least G1NumCollectionsKeepPinned
-    // GCs and hence are considered "long lived".
-    drop_pinned_retained_regions(&pinned_retained_regions);
-
+    if (candidates()->retained_groups().num_regions() > 0) {
+      select_candidates_from_retained(time_remaining_ms);
+    }
     candidates()->verify();
   } else {
     log_debug(gc, ergo, cset)("No candidates to reclaim.");
   }
 
+  _selected_groups_cur_length = collection_set_groups()->length();
   stop_incremental_building();
 
   double non_young_end_time_sec = os::elapsedTime();
@@ -386,50 +372,295 @@ void G1CollectionSet::finalize_old_part(double time_remaining_ms) {
   QuickSort::sort(_collection_set_regions, _collection_set_cur_length, compare_region_idx);
 }
 
-void G1CollectionSet::move_candidates_to_collection_set(G1CollectionCandidateRegionList* regions) {
-  for (G1HeapRegion* r : *regions) {
-    _g1h->clear_region_attr(r);
-    add_old_region(r);
-  }
-  candidates()->remove(regions);
+static void print_finish_message(const char* reason, bool from_marking) {
+  log_debug(gc, ergo, cset)("Finish adding %s candidates to collection set (%s).",
+                            from_marking ? "marking" : "retained", reason);
 }
 
-void G1CollectionSet::prepare_optional_regions(G1CollectionCandidateRegionList* regions){
-  uint cur_index = 0;
-  for (G1HeapRegion* r : *regions) {
+double G1CollectionSet::select_candidates_from_marking(double time_remaining_ms) {
+  uint num_expensive_regions = 0;
+  uint num_inital_regions = 0;
+  uint num_initial_groups = 0;
+  uint num_optional_regions = 0;
+
+  double predicted_initial_time_ms = 0.0;
+  double predicted_optional_time_ms = 0.0;
+
+  double optional_threshold_ms = time_remaining_ms * _policy->optional_prediction_fraction();
+
+  uint min_old_cset_length = _policy->calc_min_old_cset_length(candidates()->last_marking_candidates_length());
+  uint max_old_cset_length = MAX2(min_old_cset_length, _policy->calc_max_old_cset_length());
+  bool check_time_remaining = _policy->use_adaptive_young_list_length();
+
+  G1CSetCandidateGroupList* from_marking_groups = &candidates()->from_marking_groups();
+
+  log_debug(gc, ergo, cset)("Start adding marking candidates to collection set. "
+                            "Min %u regions, max %u regions, available %u regions (%u groups), "
+                            "time remaining %1.2fms, optional threshold %1.2fms",
+                            min_old_cset_length, max_old_cset_length, from_marking_groups->num_regions(), from_marking_groups->length(),
+                            time_remaining_ms, optional_threshold_ms);
+
+  G1CSetCandidateGroupList selected_groups;
+
+  for (G1CSetCandidateGroup* group : *from_marking_groups) {
+    if (num_inital_regions + num_optional_regions >= max_old_cset_length) {
+      // Added maximum number of old regions to the CSet.
+      print_finish_message("Maximum number of regions reached", true);
+      break;
+    }
+
+    double predicted_time_ms = group->predict_group_total_time_ms();
+
+    time_remaining_ms = MAX2(time_remaining_ms - predicted_time_ms, 0.0);
+    // Add regions to old set until we reach the minimum amount
+    if (num_inital_regions < min_old_cset_length) {
+
+      num_initial_groups++;
+
+      add_group_to_collection_set(group);
+      selected_groups.append(group);
+
+      num_inital_regions += group->length();
+
+      predicted_initial_time_ms += predicted_time_ms;
+      // Record the number of regions added with no time remaining
+      if (time_remaining_ms == 0.0) {
+        num_expensive_regions += group->length();
+      }
+    } else if (!check_time_remaining) {
+      // In the non-auto-tuning case, we'll finish adding regions
+      // to the CSet if we reach the minimum.
+      print_finish_message("Region amount reached min", true);
+      break;
+    } else {
+      // Keep adding regions to old set until we reach the optional threshold
+      if (time_remaining_ms > optional_threshold_ms) {
+        num_initial_groups++;
+
+        add_group_to_collection_set(group);
+        selected_groups.append(group);
+
+        num_inital_regions += group->length();
+
+        predicted_initial_time_ms += predicted_time_ms;
+
+      } else if (time_remaining_ms > 0) {
+        // Keep adding optional regions until time is up.
+        _optional_groups.append(group);
+        prepare_optional_group(group, num_optional_regions);
+        num_optional_regions += group->length();
+        predicted_optional_time_ms += predicted_time_ms;
+      } else {
+        print_finish_message("Predicted time too high", true);
+        break;
+      }
+    }
+  }
+
+  // Remove selected groups from list of candidate groups.
+  if (num_initial_groups > 0) {
+    candidates()->remove(&selected_groups);
+  }
+
+  if (from_marking_groups->length() == 0) {
+    log_debug(gc, ergo, cset)("Marking candidates exhausted.");
+  }
+
+  if (num_expensive_regions > 0) {
+    log_debug(gc, ergo, cset)("Added %u marking candidates to collection set although the predicted time was too high.",
+                              num_expensive_regions);
+  }
+
+  log_debug(gc, ergo, cset)("Finish adding marking candidates to collection set. Initial: %u regions (%u groups), optional: %u regions (%u groups), "
+                            "predicted initial time: %1.2fms, predicted optional time: %1.2fms, time remaining: %1.2fms",
+                            selected_groups.num_regions(), selected_groups.length(), _optional_groups.num_regions(), _optional_groups.length(),
+                            predicted_initial_time_ms, predicted_optional_time_ms, time_remaining_ms);
+
+  assert(selected_groups.num_regions() == num_inital_regions, "must be");
+  assert(_optional_groups.num_regions() == num_optional_regions, "must be");
+  return time_remaining_ms;
+}
+
+void G1CollectionSet::select_candidates_from_retained(double time_remaining_ms) {
+  uint num_initial_regions = 0;
+  uint num_optional_regions = 0;
+  uint num_expensive_regions = 0;
+  uint num_pinned_regions = 0;
+
+  double predicted_initial_time_ms = 0.0;
+  double predicted_optional_time_ms = 0.0;
+
+  uint const min_regions = _policy->min_retained_old_cset_length();
+  // We want to make sure that on the one hand we process the retained regions asap,
+  // but on the other hand do not take too many of them as optional regions.
+  // So we split the time budget into budget we will unconditionally take into the
+  // initial old regions, and budget for taking optional regions from the retained
+  // list.
+  double optional_time_remaining_ms = _policy->max_time_for_retaining();
+  time_remaining_ms = MIN2(time_remaining_ms, optional_time_remaining_ms);
+
+  G1CSetCandidateGroupList* retained_groups = &candidates()->retained_groups();
+
+  log_debug(gc, ergo, cset)("Start adding retained candidates to collection set. "
+                            "Min %u regions, available %u regions (%u groups), "
+                            "time remaining %1.2fms, optional remaining %1.2fms",
+                            min_regions, retained_groups->num_regions(), retained_groups->length(),
+                            time_remaining_ms, optional_time_remaining_ms);
+
+  G1CSetCandidateGroupList remove_from_retained;
+  G1CSetCandidateGroupList groups_to_abandon;
+
+  for (G1CSetCandidateGroup* group : *retained_groups) {
+    assert(group->length() == 1, "Retained groups should have only 1 region");
+
+    double predicted_time_ms = group->predict_group_total_time_ms();
+
+    bool fits_in_remaining_time = predicted_time_ms <= time_remaining_ms;
+
+    G1CollectionSetCandidateInfo* ci = group->at(0); // We only have one region in the group.
+    G1HeapRegion* r = ci->_r;
+
+    // If we can't reclaim that region ignore it for now.
+    if (r->has_pinned_objects()) {
+      num_pinned_regions++;
+      if (ci->update_num_unreclaimed()) {
+        log_trace(gc, ergo, cset)("Retained candidate %u can not be reclaimed currently. Skipping.", r->hrm_index());
+      } else {
+        log_trace(gc, ergo, cset)("Retained candidate %u can not be reclaimed currently. Dropping.", r->hrm_index());
+        // Drop pinned retained regions to make progress with retained regions. Regions
+        // in that list must have been pinned for at least G1NumCollectionsKeepPinned
+        // GCs and hence are considered "long lived".
+        _g1h->clear_region_attr(r);
+        groups_to_abandon.append(group);
+        remove_from_retained.append(group);
+      }
+      continue;
+    }
+
+    if (fits_in_remaining_time || (num_expensive_regions < min_regions)) {
+      predicted_initial_time_ms += predicted_time_ms;
+      if (!fits_in_remaining_time) {
+        num_expensive_regions++;
+      }
+
+      add_group_to_collection_set(group);
+      remove_from_retained.append(group);
+
+      num_initial_regions += group->length();
+    } else if (predicted_time_ms <= optional_time_remaining_ms) {
+      // Prepare optional collection region.
+      _optional_groups.append(group);
+      prepare_optional_group(group, num_optional_regions);
+      num_optional_regions += group->length();
+      predicted_optional_time_ms += predicted_time_ms;
+    } else {
+      // Fits neither initial nor optional time limit. Exit.
+      break;
+    }
+    time_remaining_ms = MAX2(0.0, time_remaining_ms - predicted_time_ms);
+    optional_time_remaining_ms = MAX2(0.0, optional_time_remaining_ms - predicted_time_ms);
+  }
+
+  if (num_initial_regions == retained_groups->num_regions()) {
+    log_debug(gc, ergo, cset)("Retained candidates exhausted.");
+  }
+
+  if (num_expensive_regions > 0) {
+    log_debug(gc, ergo, cset)("Added %u retained candidates to collection set although the predicted time was too high.",
+                              num_expensive_regions);
+  }
+
+  // Remove groups from retained and also do some bookkeeping on CandidateOrigin
+  // for the regions in these groups.
+  candidates()->remove(&remove_from_retained);
+
+  groups_to_abandon.clear(true /* uninstall_group_cardset */);
+
+  log_debug(gc, ergo, cset)("Finish adding retained candidates to collection set. Initial: %u, optional: %u, pinned: %u, "
+                            "predicted initial time: %1.2fms, predicted optional time: %1.2fms, "
+                            "time remaining: %1.2fms optional time remaining %1.2fms",
+                            num_initial_regions, num_optional_regions, num_pinned_regions,
+                            predicted_initial_time_ms, predicted_optional_time_ms, time_remaining_ms, optional_time_remaining_ms);
+}
+
+double G1CollectionSet::select_candidates_from_optional_groups(double time_remaining_ms, uint& num_regions_selected) {
+
+  assert(_optional_groups.num_regions() > 0,
+         "Should only be called when there are optional regions");
+
+  uint num_groups_selected = 0;
+  double total_prediction_ms = 0.0;
+  G1CSetCandidateGroupList selected;
+  for (G1CSetCandidateGroup* group : _optional_groups) {
+    double predicted_time_ms = group->predict_group_total_time_ms();
+
+    if (predicted_time_ms > time_remaining_ms) {
+      log_debug(gc, ergo, cset)("Prediction %.3fms for group with %u regions does not fit remaining time: %.3fms.",
+                                predicted_time_ms, group->length(), time_remaining_ms);
+      break;
+    }
+
+    total_prediction_ms += predicted_time_ms;
+    time_remaining_ms -= predicted_time_ms;
+
+    num_regions_selected += group->length();
+    num_groups_selected++;
+
+    add_group_to_collection_set(group);
+    selected.append(group);
+  }
+
+  log_debug(gc, ergo, cset) ("Completed with groups, selected %u", num_regions_selected);
+  // Remove selected groups from candidate list.
+  if (num_groups_selected > 0) {
+    _optional_groups.remove(&selected);
+    candidates()->remove(&selected);
+  }
+  return total_prediction_ms;
+}
+
+uint G1CollectionSet::select_optional_collection_set_regions(double time_remaining_ms) {
+  uint optional_regions_count = num_optional_regions();
+  assert(optional_regions_count > 0,
+         "Should only be called when there are optional regions");
+
+  uint num_regions_selected = 0;
+
+  double total_prediction_ms = select_candidates_from_optional_groups(time_remaining_ms, num_regions_selected);
+
+  time_remaining_ms -= total_prediction_ms;
+
+  log_debug(gc, ergo, cset)("Prepared %u regions out of %u for optional evacuation. Total predicted time: %.3fms",
+                            num_regions_selected, optional_regions_count, total_prediction_ms);
+  return num_regions_selected;
+}
+
+void G1CollectionSet::prepare_optional_group(G1CSetCandidateGroup* gr, uint cur_index) {
+  for (G1CollectionSetCandidateInfo ci : *gr) {
+    G1HeapRegion* r = ci._r;
+
     assert(r->is_old(), "the region should be old");
     assert(!r->in_collection_set(), "should not already be in the CSet");
 
     _g1h->register_optional_region_with_region_attr(r);
-
     r->set_index_in_opt_cset(cur_index++);
   }
 }
 
-void G1CollectionSet::move_pinned_marking_to_retained(G1CollectionCandidateRegionList* regions) {
-  if (regions->length() == 0) {
-    return;
+void G1CollectionSet::add_group_to_collection_set(G1CSetCandidateGroup* gr) {
+  for (G1CollectionSetCandidateInfo ci : *gr) {
+    G1HeapRegion* r = ci._r;
+    r->uninstall_cset_group();
+    assert(r->rem_set()->is_complete(), "must be");
+    add_region_to_collection_set(r);
   }
-  candidates()->remove(regions);
-
-  for (G1HeapRegion* r : *regions) {
-    assert(r->has_pinned_objects(), "must be pinned");
-    assert(r->rem_set()->is_complete(), "must be complete");
-    candidates()->add_retained_region_unsorted(r);
-  }
-  candidates()->sort_by_efficiency();
+  _collection_set_groups.append(gr);
 }
 
-void G1CollectionSet::drop_pinned_retained_regions(G1CollectionCandidateRegionList* regions) {
-  if (regions->length() == 0) {
-    return;
-  }
-  candidates()->remove(regions);
-
-  // We can now drop these region's remembered sets.
-  for (G1HeapRegion* r : *regions) {
-    r->rem_set()->clear(true /* only_cardset */);
-  }
+void G1CollectionSet::add_region_to_collection_set(G1HeapRegion* r) {
+  _g1h->clear_region_attr(r);
+  assert(r->rem_set()->is_complete(), "Remset for region %u complete", r->hrm_index());
+  add_old_region(r);
 }
 
 void G1CollectionSet::finalize_initial_collection_set(double target_pause_time_ms, G1SurvivorRegions* survivor) {
@@ -440,32 +671,31 @@ void G1CollectionSet::finalize_initial_collection_set(double target_pause_time_m
 bool G1CollectionSet::finalize_optional_for_evacuation(double remaining_pause_time) {
   update_incremental_marker();
 
-  G1CollectionCandidateRegionList selected_regions;
-  _policy->calculate_optional_collection_set_regions(&_optional_old_regions,
-                                                     remaining_pause_time,
-                                                     &selected_regions);
-
-  move_candidates_to_collection_set(&selected_regions);
-
-  _optional_old_regions.remove_prefix(&selected_regions);
+  uint num_regions_selected = select_optional_collection_set_regions(remaining_pause_time);
 
+  _selected_groups_cur_length = collection_set_groups()->length();
   stop_incremental_building();
 
   _g1h->verify_region_attr_remset_is_tracked();
 
-  return selected_regions.length() > 0;
+  return num_regions_selected > 0;
 }
 
 void G1CollectionSet::abandon_optional_collection_set(G1ParScanThreadStateSet* pss) {
-  for (G1HeapRegion* r : _optional_old_regions) {
-    pss->record_unused_optional_region(r);
-    // Clear collection set marker and make sure that the remembered set information
-    // is correct as we still need it later.
-    _g1h->clear_region_attr(r);
-    _g1h->register_region_with_region_attr(r);
-    r->clear_index_in_opt_cset();
+  if (_optional_groups.length() > 0) {
+    auto reset = [&] (G1HeapRegion* r) {
+      pss->record_unused_optional_region(r);
+      // Clear collection set marker and make sure that the remembered set information
+      // is correct as we still need it later.
+      _g1h->clear_region_attr(r);
+      _g1h->register_region_with_region_attr(r);
+      r->clear_index_in_opt_cset();
+    };
+
+    _optional_groups.iterate(reset);
+    // Remove groups from list without deleting the groups or clearing the associated cardsets.
+    _optional_groups.remove_selected(_optional_groups.length(), _optional_groups.num_regions());
   }
-  _optional_old_regions.clear();
 
   _g1h->verify_region_attr_remset_is_tracked();
 }
