@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -247,15 +247,14 @@ void G1MonitoringSupport::recalculate_sizes() {
   _old_gen_used = _overall_used - MIN2(_overall_used, _eden_space_used + _survivor_space_used);
 
   uint survivor_list_length = _g1h->survivor_regions_count();
-  // Max length includes any potential extensions to the young gen
-  // we'll do when the GC locker is active.
-  uint young_list_max_length = _g1h->policy()->young_list_max_length();
-  assert(young_list_max_length >= survivor_list_length, "invariant");
-  uint eden_list_max_length = young_list_max_length - survivor_list_length;
+
+  uint young_list_target_length = _g1h->policy()->young_list_target_length();
+  assert(young_list_target_length >= survivor_list_length, "invariant");
+  uint eden_list_max_length = young_list_target_length - survivor_list_length;
 
   // First calculate the committed sizes that can be calculated independently.
-  _survivor_space_committed = survivor_list_length * HeapRegion::GrainBytes;
-  _old_gen_committed = HeapRegion::align_up_to_region_byte_size(_old_gen_used);
+  _survivor_space_committed = survivor_list_length * G1HeapRegion::GrainBytes;
+  _old_gen_committed = G1HeapRegion::align_up_to_region_byte_size(_old_gen_used);
 
   // Next, start with the overall committed size.
   _overall_committed = _g1h->capacity();
@@ -267,7 +266,7 @@ void G1MonitoringSupport::recalculate_sizes() {
   committed -= _survivor_space_committed + _old_gen_committed;
 
   // Next, calculate and remove the committed size for the eden.
-  _eden_space_committed = (size_t) eden_list_max_length * HeapRegion::GrainBytes;
+  _eden_space_committed = (size_t) eden_list_max_length * G1HeapRegion::GrainBytes;
   // Somewhat defensive: be robust in case there are inaccuracies in
   // the calculations
   _eden_space_committed = MIN2(_eden_space_committed, committed);
