@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2006, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "gc/parallel/mutableNUMASpace.hpp"
 #include "gc/shared/collectedHeap.hpp"
 #include "gc/shared/gc_globals.hpp"
@@ -55,7 +54,7 @@ MutableNUMASpace::MutableNUMASpace(size_t alignment) : MutableSpace(alignment),
 
   size_t lgrp_limit = os::numa_get_groups_num();
   uint *lgrp_ids = NEW_C_HEAP_ARRAY(uint, lgrp_limit, mtGC);
-  size_t lgrp_num = os::numa_get_leaf_groups(reinterpret_cast<int*>(lgrp_ids), lgrp_limit);
+  size_t lgrp_num = os::numa_get_leaf_groups(lgrp_ids, lgrp_limit);
   assert(lgrp_num > 0, "There should be at least one locality group");
 
   lgrp_spaces()->reserve(checked_cast<int>(lgrp_num));
@@ -79,25 +78,11 @@ void MutableNUMASpace::mangle_unused_area() {
   // This method should do nothing.
   // It can be called on a numa space during a full compaction.
 }
-void MutableNUMASpace::mangle_unused_area_complete() {
-  // This method should do nothing.
-  // It can be called on a numa space during a full compaction.
-}
+
 void MutableNUMASpace::mangle_region(MemRegion mr) {
   // This method should do nothing because numa spaces are not mangled.
 }
-void MutableNUMASpace::set_top_for_allocations(HeapWord* v) {
-  assert(false, "Do not mangle MutableNUMASpace's");
-}
-void MutableNUMASpace::set_top_for_allocations() {
-  // This method should do nothing.
-}
-void MutableNUMASpace::check_mangled_unused_area(HeapWord* limit) {
-  // This method should do nothing.
-}
-void MutableNUMASpace::check_mangled_unused_area_complete() {
-  // This method should do nothing.
-}
+
 #endif  // NOT_PRODUCT
 
 // There may be unallocated holes in the middle chunks
@@ -113,7 +98,7 @@ void MutableNUMASpace::ensure_parsability() {
         while (words_left_to_fill > 0) {
           size_t words_to_fill = MIN2(words_left_to_fill, CollectedHeap::filler_array_max_size());
           assert(words_to_fill >= CollectedHeap::min_fill_size(),
-                 "Remaining size (" SIZE_FORMAT ") is too small to fill (based on " SIZE_FORMAT " and " SIZE_FORMAT ")",
+                 "Remaining size (%zu) is too small to fill (based on %zu and %zu)",
                  words_to_fill, words_left_to_fill, CollectedHeap::filler_array_max_size());
           CollectedHeap::fill_with_object(cur_top, words_to_fill);
           cur_top += words_to_fill;
@@ -142,6 +127,11 @@ size_t MutableNUMASpace::free_in_words() const {
   return s;
 }
 
+int MutableNUMASpace::lgrp_space_index(int lgrp_id) const {
+  return lgrp_spaces()->find_if([&](LGRPSpace* space) {
+    return space->lgrp_id() == checked_cast<uint>(lgrp_id);
+  });
+}
 
 size_t MutableNUMASpace::tlab_capacity(Thread *thr) const {
   guarantee(thr != nullptr, "No thread");
@@ -160,7 +150,7 @@ size_t MutableNUMASpace::tlab_capacity(Thread *thr) const {
     }
   }
   // That's the normal case, where we know the locality group of the thread.
-  int i = lgrp_spaces()->find(&lgrp_id, LGRPSpace::equals);
+  int i = lgrp_space_index(lgrp_id);
   if (i == -1) {
     return 0;
   }
@@ -179,7 +169,7 @@ size_t MutableNUMASpace::tlab_used(Thread *thr) const {
       return 0;
     }
   }
-  int i = lgrp_spaces()->find(&lgrp_id, LGRPSpace::equals);
+  int i = lgrp_space_index(lgrp_id);
   if (i == -1) {
     return 0;
   }
@@ -199,7 +189,7 @@ size_t MutableNUMASpace::unsafe_max_tlab_alloc(Thread *thr) const {
       return 0;
     }
   }
-  int i = lgrp_spaces()->find(&lgrp_id, LGRPSpace::equals);
+  int i = lgrp_space_index(lgrp_id);
   if (i == -1) {
     return 0;
   }
@@ -222,7 +212,7 @@ void MutableNUMASpace::bias_region(MemRegion mr, uint lgrp_id) {
     // Then we uncommit the pages in the range.
     // The alignment_hint argument must be less than or equal to the small page
     // size if not using large pages or else this function does nothing.
-    os::free_memory((char*)aligned_region.start(), aligned_region.byte_size(), os_align);
+    os::disclaim_memory((char*)aligned_region.start(), aligned_region.byte_size());
     // And make them local/first-touch biased.
     os::numa_make_local((char*)aligned_region.start(), aligned_region.byte_size(), checked_cast<int>(lgrp_id));
   }
@@ -236,20 +226,6 @@ void MutableNUMASpace::update() {
                SpaceDecorator::Clear,
                SpaceDecorator::DontMangle);
   }
-
-  scan_pages(NUMAPageScanRate);
-}
-
-// Scan pages. Free pages that have smaller size or wrong placement.
-void MutableNUMASpace::scan_pages(size_t page_count)
-{
-  size_t pages_per_chunk = page_count / lgrp_spaces()->length();
-  if (pages_per_chunk > 0) {
-    for (int i = 0; i < lgrp_spaces()->length(); i++) {
-      LGRPSpace *ls = lgrp_spaces()->at(i);
-      ls->scan_pages(page_size(), pages_per_chunk);
-    }
-  }
 }
 
 // Accumulate statistics about the allocation rate of each lgrp.
@@ -583,7 +559,7 @@ HeapWord* MutableNUMASpace::cas_allocate(size_t size) {
     thr->set_lgrp_id(lgrp_id);
   }
 
-  int i = lgrp_spaces()->find(&lgrp_id, LGRPSpace::equals);
+  int i = lgrp_space_index(lgrp_id);
   // It is possible that a new CPU has been hotplugged and
   // we haven't reshaped the space accordingly.
   if (i == -1) {
@@ -642,8 +618,8 @@ void MutableNUMASpace::print_on(outputStream* st) const {
       for (int i = 0; i < lgrp_spaces()->length(); i++) {
         lgrp_spaces()->at(i)->accumulate_statistics(page_size());
       }
-      st->print("    local/remote/unbiased/uncommitted: " SIZE_FORMAT "K/"
-                SIZE_FORMAT "K/" SIZE_FORMAT "K/" SIZE_FORMAT "K\n",
+      st->print("    local/remote/unbiased/uncommitted: %zuK/"
+                "%zuK/%zuK/%zuK\n",
                 ls->space_stats()->_local_space / K,
                 ls->space_stats()->_remote_space / K,
                 ls->space_stats()->_unbiased_space / K,
@@ -691,43 +667,3 @@ void MutableNUMASpace::LGRPSpace::accumulate_statistics(size_t page_size) {
                                    pointer_delta(space()->end(), end, sizeof(char));
 
 }
-
-// Scan page_count pages and verify if they have the right size and right placement.
-// If invalid pages are found they are freed in hope that subsequent reallocation
-// will be more successful.
-void MutableNUMASpace::LGRPSpace::scan_pages(size_t page_size, size_t page_count)
-{
-  char* range_start = (char*)align_up(space()->bottom(), page_size);
-  char* range_end = (char*)align_down(space()->end(), page_size);
-
-  if (range_start > last_page_scanned() || last_page_scanned() >= range_end) {
-    set_last_page_scanned(range_start);
-  }
-
-  char *scan_start = last_page_scanned();
-  char* scan_end = MIN2(scan_start + page_size * page_count, range_end);
-
-  os::page_info page_expected, page_found;
-  page_expected.size = page_size;
-  page_expected.lgrp_id = checked_cast<uint>(lgrp_id());
-
-  char *s = scan_start;
-  while (s < scan_end) {
-    char *e = os::scan_pages(s, (char*)scan_end, &page_expected, &page_found);
-    if (e == nullptr) {
-      break;
-    }
-    if (e != scan_end) {
-      assert(e < scan_end, "e: " PTR_FORMAT " scan_end: " PTR_FORMAT, p2i(e), p2i(scan_end));
-
-      if ((page_expected.size != page_size || checked_cast<uint>(page_expected.lgrp_id) != lgrp_id())
-          && page_expected.size != 0) {
-        os::free_memory(s, pointer_delta(e, s, sizeof(char)), page_size);
-      }
-      page_expected = page_found;
-    }
-    s = e;
-  }
-
-  set_last_page_scanned(scan_end);
-}
