@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -31,6 +31,7 @@
 #include "classfile/javaClasses.inline.hpp"
 #include "gc/parallel/parMarkBitMap.hpp"
 #include "gc/parallel/psParallelCompact.inline.hpp"
+#include "gc/parallel/psStringDedup.hpp"
 #include "gc/shared/taskqueue.inline.hpp"
 #include "oops/access.inline.hpp"
 #include "oops/arrayOop.hpp"
@@ -40,30 +41,10 @@
 #include "utilities/debug.hpp"
 #include "utilities/globalDefinitions.hpp"
 
-class PCMarkAndPushClosure: public OopClosure {
-private:
-  ParCompactionManager* _compaction_manager;
-public:
-  PCMarkAndPushClosure(ParCompactionManager* cm) : _compaction_manager(cm) { }
-
-  template <typename T> void do_oop_nv(T* p)      { _compaction_manager->mark_and_push(p); }
-  virtual void do_oop(oop* p)                     { do_oop_nv(p); }
-  virtual void do_oop(narrowOop* p)               { do_oop_nv(p); }
-};
-
-class PCIterateMarkAndPushClosure: public MetadataVisitingOopIterateClosure {
-private:
-  ParCompactionManager* _compaction_manager;
-public:
-  PCIterateMarkAndPushClosure(ParCompactionManager* cm, ReferenceProcessor* rp) : MetadataVisitingOopIterateClosure(rp), _compaction_manager(cm) { }
-
-  template <typename T> void do_oop_nv(T* p)      { _compaction_manager->mark_and_push(p); }
-  virtual void do_oop(oop* p)                     { do_oop_nv(p); }
-  virtual void do_oop(narrowOop* p)               { do_oop_nv(p); }
-
-  void do_klass_nv(Klass* k)                      { _compaction_manager->follow_klass(k); }
-  void do_cld_nv(ClassLoaderData* cld)            { _compaction_manager->follow_class_loader(cld); }
-};
+template <typename T>
+inline void PCMarkAndPushClosure::do_oop_work(T* p) {
+  _compaction_manager->mark_and_push(p);
+}
 
 inline bool ParCompactionManager::steal(int queue_num, oop& t) {
   return oop_task_queues()->steal(queue_num, t);
@@ -78,7 +59,7 @@ inline bool ParCompactionManager::steal(int queue_num, size_t& region) {
 }
 
 inline void ParCompactionManager::push(oop obj) {
-  _marking_stack.push(obj);
+  _oop_stack.push(obj);
 }
 
 void ParCompactionManager::push_objarray(oop obj, size_t index)
@@ -106,17 +87,22 @@ inline void ParCompactionManager::mark_and_push(T* p) {
     oop obj = CompressedOops::decode_not_null(heap_oop);
     assert(ParallelScavengeHeap::heap()->is_in(obj), "should be in heap");
 
-    if (mark_bitmap()->is_unmarked(obj) && PSParallelCompact::mark_obj(obj)) {
+    if (mark_bitmap()->mark_obj(obj)) {
+      if (StringDedup::is_enabled() &&
+          java_lang_String::is_instance(obj) &&
+          psStringDedup::is_candidate_from_mark(obj)) {
+        _string_dedup_requests.add(obj);
+      }
+
+      ContinuationGCSupport::transform_stack_chunk(obj);
+
+      assert(_marking_stats_cache != nullptr, "inv");
+      _marking_stats_cache->push(obj, obj->size());
       push(obj);
     }
   }
 }
 
-inline void ParCompactionManager::follow_klass(Klass* klass) {
-  oop holder = klass->class_loader_data()->holder_no_keepalive();
-  mark_and_push(&holder);
-}
-
 inline void ParCompactionManager::FollowStackClosure::do_void() {
   _compaction_manager->follow_marking_stacks();
   if (_terminator != nullptr) {
@@ -140,7 +126,7 @@ inline void follow_array_specialized(objArrayOop obj, int index, ParCompactionMa
     cm->push_objarray(obj, end_index); // Push the continuation.
   }
 
-  // Push the non-NULL elements of the next stride on the marking stack.
+  // Push the non-null elements of the next stride on the marking stack.
   for (T* e = beg; e < end; e++) {
     cm->mark_and_push<T>(e);
   }
@@ -154,28 +140,84 @@ inline void ParCompactionManager::follow_array(objArrayOop obj, int index) {
   }
 }
 
-inline void ParCompactionManager::update_contents(oop obj) {
-  if (!obj->klass()->is_typeArray_klass()) {
-    PCAdjustPointerClosure apc(this);
-    obj->oop_iterate(&apc);
-  }
-}
-
-inline void ParCompactionManager::follow_class_loader(ClassLoaderData* cld) {
-  PCMarkAndPushClosure mark_and_push_closure(this);
-  cld->oops_do(&mark_and_push_closure, true);
-}
-
 inline void ParCompactionManager::follow_contents(oop obj) {
   assert(PSParallelCompact::mark_bitmap()->is_marked(obj), "should be marked");
-  PCIterateMarkAndPushClosure cl(this, PSParallelCompact::ref_processor());
 
   if (obj->is_objArray()) {
-    cl.do_klass(obj->klass());
+    _mark_and_push_closure.do_klass(obj->klass());
     follow_array(objArrayOop(obj), 0);
   } else {
-    obj->oop_iterate(&cl);
+    obj->oop_iterate(&_mark_and_push_closure);
   }
 }
 
+inline void ParCompactionManager::MarkingStatsCache::push(size_t region_id, size_t live_words) {
+  size_t index = (region_id & entry_mask);
+  if (entries[index].region_id == region_id) {
+    // Hit
+    entries[index].live_words += live_words;
+    return;
+  }
+  // Miss
+  if (entries[index].live_words != 0) {
+    evict(index);
+  }
+  entries[index].region_id = region_id;
+  entries[index].live_words = live_words;
+}
+
+inline void ParCompactionManager::MarkingStatsCache::push(oop obj, size_t live_words) {
+  ParallelCompactData& data = PSParallelCompact::summary_data();
+  const size_t region_size = ParallelCompactData::RegionSize;
+
+  HeapWord* addr = cast_from_oop<HeapWord*>(obj);
+  const size_t start_region_id = data.addr_to_region_idx(addr);
+  const size_t end_region_id = data.addr_to_region_idx(addr + live_words - 1);
+  if (start_region_id == end_region_id) {
+    // Completely inside this region
+    push(start_region_id, live_words);
+    return;
+  }
+
+  // First region
+  push(start_region_id, region_size - data.region_offset(addr));
+
+  // Middle regions; bypass cache
+  for (size_t i = start_region_id + 1; i < end_region_id; ++i) {
+    data.region(i)->set_partial_obj_size(region_size);
+    data.region(i)->set_partial_obj_addr(addr);
+  }
+
+  // Last region; bypass cache
+  const size_t end_offset = data.region_offset(addr + live_words - 1);
+  data.region(end_region_id)->set_partial_obj_size(end_offset + 1);
+  data.region(end_region_id)->set_partial_obj_addr(addr);
+}
+
+inline void ParCompactionManager::MarkingStatsCache::evict(size_t index) {
+  ParallelCompactData& data = PSParallelCompact::summary_data();
+  // flush to global data
+  data.region(entries[index].region_id)->add_live_obj(entries[index].live_words);
+}
+
+inline void ParCompactionManager::MarkingStatsCache::evict_all() {
+  for (size_t i = 0; i < num_entries; ++i) {
+    if (entries[i].live_words != 0) {
+      evict(i);
+      entries[i].live_words = 0;
+    }
+  }
+}
+
+inline void ParCompactionManager::create_marking_stats_cache() {
+  assert(_marking_stats_cache == nullptr, "precondition");
+  _marking_stats_cache = new MarkingStatsCache();
+}
+
+inline void ParCompactionManager::flush_and_destroy_marking_stats_cache() {
+  _marking_stats_cache->evict_all();
+  delete _marking_stats_cache;
+  _marking_stats_cache = nullptr;
+}
+
 #endif // SHARE_GC_PARALLEL_PSCOMPACTIONMANAGER_INLINE_HPP
