@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -95,15 +95,19 @@ private:
   GCMemoryManager* _young_manager;
   GCMemoryManager* _old_manager;
 
+  // Indicate whether heap is almost or approaching full.
+  // Usually, there is some memory headroom for application/gc to run properly.
+  // However, in extreme cases, e.g. young-gen is non-empty after a full gc, we
+  // will attempt some uncommon measures, e.g. alllocating small objs in
+  // old-gen.
+  bool _is_heap_almost_full;
+
   // Helper functions for allocation
   HeapWord* attempt_allocation(size_t size,
                                bool   is_tlab,
                                bool   first_only);
 
   void do_full_collection(bool clear_all_soft_refs) override;
-  void do_full_collection_no_gc_locker(bool clear_all_soft_refs);
-
-  void collect_at_safepoint(bool full);
 
   // Does the "cause" of GC indicate that
   // we absolutely __must__ clear soft refs?
@@ -111,6 +115,9 @@ private:
 
   bool is_young_gc_safe() const;
 
+  void gc_prologue();
+  void gc_epilogue(bool full);
+
 public:
   // Returns JNI_OK on success
   jint initialize() override;
@@ -137,7 +144,7 @@ public:
   HeapWord* satisfy_failed_allocation(size_t size, bool is_tlab);
 
   // Callback from VM_SerialGCCollect.
-  void try_collect_at_safepoint(bool full);
+  void collect_at_safepoint(bool full);
 
   // Perform a full collection of the heap; intended for use in implementing
   // "System.gc". This implies as full a collection as the CollectedHeap
@@ -199,7 +206,8 @@ public:
   void prepare_for_verify() override;
   void verify(VerifyOption option) override;
 
-  void print_on(outputStream* st) const override;
+  void print_heap_on(outputStream* st) const override;
+  void print_gc_on(outputStream* st) const override;
   void gc_threads_do(ThreadClosure* tc) const override;
   void print_tracing_info() const override;
 
@@ -226,10 +234,6 @@ public:
     SO_ScavengeCodeCache   = 0x10
   };
 
- protected:
-  virtual void gc_prologue(bool full);
-  virtual void gc_epilogue(bool full);
-
  public:
   // Apply closures on various roots in Young GC or marking/adjust phases of Full GC.
   void process_roots(ScanningOption so,
@@ -251,8 +255,7 @@ private:
   // Try to allocate space by expanding the heap.
   HeapWord* expand_heap_and_allocate(size_t size, bool is_tlab);
 
-  HeapWord* mem_allocate_work(size_t size,
-                              bool is_tlab);
+  HeapWord* mem_allocate_work(size_t size, bool is_tlab);
 
   MemoryPool* _eden_pool;
   MemoryPool* _survivor_pool;
@@ -291,7 +294,7 @@ public:
   void safepoint_synchronize_end() override;
 
   // Support for loading objects from CDS archive into the heap
-  bool can_load_archived_objects() const override { return UseCompressedOops; }
+  bool can_load_archived_objects() const override { return true; }
   HeapWord* allocate_loaded_archive_space(size_t size) override;
   void complete_loaded_archive_space(MemRegion archive_space) override;
 
