@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "classfile/altHashing.hpp"
 #include "classfile/javaClasses.inline.hpp"
 #include "classfile/stringTable.hpp"
@@ -56,10 +55,11 @@
 // the vectors.  The size of the table is the size of either vector.
 //
 // The capacity of the vectors is explicitly controlled, based on the size.
-// Given N > 0 and 2^N <= size < 2^(N+1), then capacity = 2^N + k * 2^(N-1)
+// Given N > 0 and 2^N <= size < 2^(N+1), then capacity <= 2^N + k * 2^(N-1)
 // for the smallest integer k in [0,2] such that size <= capacity.  That is,
 // use a power of 2 or the midpoint between consecutive powers of 2 that is
-// minimally at least size.
+// minimally at least size.  When adding an entry and the capacity has been
+// reached, capacity is increased to the next of those values.
 //
 // The main benefit of this representation is that it uses less space than a
 // more traditional linked-list of entry nodes representation.  Such a
@@ -89,12 +89,11 @@ class StringDedup::Table::Bucket {
   GrowableArrayCHeap<uint, mtStringDedup> _hashes;
   GrowableArrayCHeap<TableValue, mtStringDedup> _values;
 
-  void adjust_capacity(int new_capacity);
   void expand_if_full();
 
 public:
   // precondition: reserve == 0 or is the result of needed_capacity.
-  Bucket(int reserve = 0);
+  explicit Bucket(int reserve = 0);
 
   ~Bucket() {
     while (!_values.is_empty()) {
@@ -107,7 +106,7 @@ public:
   const GrowableArrayView<uint>& hashes() const { return _hashes; }
   const GrowableArrayView<TableValue>& values() const { return _values; }
 
-  bool is_empty() const { return _hashes.length() == 0; }
+  bool is_empty() const { return _hashes.is_empty(); }
   int length() const { return _hashes.length(); }
 
   void add(uint hash_code, TableValue value) {
@@ -150,33 +149,17 @@ int StringDedup::Table::Bucket::needed_capacity(int needed) {
   return (needed <= low) ? low : high;
 }
 
-void StringDedup::Table::Bucket::adjust_capacity(int new_capacity) {
-  GrowableArrayCHeap<uint, mtStringDedup> new_hashes{new_capacity};
-  GrowableArrayCHeap<TableValue, mtStringDedup> new_values{new_capacity};
-  while (!_hashes.is_empty()) {
-    new_hashes.push(_hashes.pop());
-    new_values.push(_values.pop());
-  }
-  _hashes.swap(&new_hashes);
-  _values.swap(&new_values);
-}
-
 void StringDedup::Table::Bucket::expand_if_full() {
-  if (_hashes.length() == _hashes.max_length()) {
-    adjust_capacity(needed_capacity(_hashes.max_length() + 1));
+  if (_hashes.is_full()) {
+    int needed = needed_capacity(_hashes.capacity() + 1);
+    _hashes.reserve(needed);
+    _values.reserve(needed);
   }
 }
 
 void StringDedup::Table::Bucket::shrink() {
-  if (_hashes.is_empty()) {
-    _hashes.clear_and_deallocate();
-    _values.clear_and_deallocate();
-  } else {
-    int target = needed_capacity(_hashes.length());
-    if (target < _hashes.max_length()) {
-      adjust_capacity(target);
-    }
-  }
+  _hashes.shrink_to_fit();
+  _values.shrink_to_fit();
 }
 
 StringDedup::Table::TableValue
@@ -228,7 +211,7 @@ void StringDedup::Table::Bucket::verify(size_t bucket_index,
 // If a dead count report is received while cleaning, further cleaning may
 // reduce the number of dead entries.  With STW reference processing one
 // could maintain an accurate dead count by deducting cleaned entries.  But
-// that doesn't work for concurrent reference processsing.  In that case the
+// that doesn't work for concurrent reference processing.  In that case the
 // dead count being reported may include entries that have already been
 // removed by concurrent cleaning.
 //
@@ -299,7 +282,6 @@ public:
   virtual bool step() = 0;
   virtual TableValue find(typeArrayOop obj, uint hash_code) const = 0;
   virtual void report_end() const = 0;
-  virtual Stat::Phase phase() const = 0;
   virtual void verify() const = 0;
 };
 
@@ -337,10 +319,6 @@ public:
     _cur_stat.report_resize_table_end();
   }
 
-  virtual Stat::Phase phase() const {
-    return Stat::Phase::resize_table;
-  }
-
   virtual void verify() const;
 };
 
@@ -405,10 +383,6 @@ public:
     _cur_stat.report_cleanup_table_end();
   }
 
-  virtual Stat::Phase phase() const {
-    return Stat::Phase::cleanup_table;
-  }
-
   virtual void verify() const {} // Nothing to do here.
 };
 
@@ -734,11 +708,6 @@ void StringDedup::Table::cleanup_end() {
   Atomic::store(&_dead_state, DeadState::wait2);
 }
 
-StringDedup::Stat::Phase StringDedup::Table::cleanup_phase() {
-  assert(_cleanup_state != nullptr, "precondition");
-  return _cleanup_state->phase();
-}
-
 void StringDedup::Table::verify() {
   size_t total_count = 0;
   for (size_t i = 0; i < _number_of_buckets; ++i) {
