@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -21,7 +21,6 @@
  * questions.
  */
 
-#include "precompiled.hpp"
 #include "gc/shared/gc_globals.hpp"
 #include "gc/shared/suspendibleThreadSet.hpp"
 #include "gc/z/zAbort.inline.hpp"
@@ -411,7 +410,7 @@ static void retire_target_page(ZGeneration* generation, ZPage* page) {
   // relocate the remaining objects, leaving the target page empty when
   // relocation completed.
   if (page->used() == 0) {
-    ZHeap::heap()->free_page(page);
+    ZHeap::heap()->free_page(page, true /* allow_defragment */);
   }
 }
 
@@ -670,9 +669,9 @@ private:
     // moved them over to the current bitmap.
     //
     // If the young generation runs multiple cycles while the old generation is
-    // relocating, then the first cycle will have consume the the old remset,
+    // relocating, then the first cycle will have consumed the old remset,
     // bits and moved associated objects to a new old page. The old relocation
-    // could find either the the two bitmaps. So, either it will find the original
+    // could find either of the two bitmaps. So, either it will find the original
     // remset bits for the page, or it will find an empty bitmap for the page. It
     // doesn't matter for correctness, because the young generation marking has
     // already taken care of the bits.
@@ -848,7 +847,7 @@ private:
     to_page->reset(to_age);
     to_page->reset_top_for_allocation();
     if (promotion) {
-      to_page->remset_initialize();
+      to_page->remset_alloc();
     }
 
     // Verify that the inactive remset is clear when resetting the page for
@@ -866,7 +865,7 @@ private:
     start_in_place_relocation_prepare_remset(from_page);
 
     if (promotion) {
-      // Register the the promotion
+      // Register the promotion
       ZGeneration::young()->in_place_relocate_promote(from_page, to_page);
       ZGeneration::young()->register_in_place_relocate_promoted(from_page);
     }
@@ -943,35 +942,15 @@ public:
     return ZGeneration::old()->active_remset_is_current();
   }
 
-  void clear_remset_before_reuse(ZPage* page, bool in_place) {
+  void clear_remset_before_in_place_reuse(ZPage* page) {
     if (_forwarding->from_age() != ZPageAge::old) {
       // No remset bits
       return;
     }
 
-    if (in_place) {
-      // Clear 'previous' remset bits. For in-place relocated pages, the previous
-      // remset bits are always used, even when active_remset_is_current().
-      page->clear_remset_previous();
-
-      return;
-    }
-
-    // Normal relocate
-
-    // Clear active remset bits
-    if (active_remset_is_current()) {
-      page->clear_remset_current();
-    } else {
-      page->clear_remset_previous();
-    }
-
-    // Verify that inactive remset bits are all cleared
-    if (active_remset_is_current()) {
-      page->verify_remset_cleared_previous();
-    } else {
-      page->verify_remset_cleared_current();
-    }
+    // Clear 'previous' remset bits. For in-place relocated pages, the previous
+    // remset bits are always used, even when active_remset_is_current().
+    page->clear_remset_previous();
   }
 
   void finish_in_place_relocation() {
@@ -1017,7 +996,7 @@ public:
       ZPage* const page = _forwarding->detach_page();
 
       // Ensure that previous remset bits are cleared
-      clear_remset_before_reuse(page, true /* in_place */);
+      clear_remset_before_in_place_reuse(page);
 
       page->log_msg(" (relocate page done in-place)");
 
@@ -1029,15 +1008,10 @@ public:
       // Wait for all other threads to call release_page
       ZPage* const page = _forwarding->detach_page();
 
-      // Ensure that all remset bits are cleared
-      // Note: cleared after detach_page, when we know that
-      // the young generation isn't scanning the remset.
-      clear_remset_before_reuse(page, false /* in_place */);
-
       page->log_msg(" (relocate page done normal)");
 
       // Free page
-      ZHeap::heap()->free_page(page);
+      ZHeap::heap()->free_page(page, true /* allow_defragment */);
     }
   }
 };
@@ -1292,7 +1266,7 @@ public:
       new_page->reset(to_age);
       new_page->reset_livemap();
       if (promotion) {
-        new_page->remset_initialize();
+        new_page->remset_alloc();
       }
 
       if (promotion) {
