@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,46 +24,51 @@
 #ifndef SHARE_GC_Z_ZVIRTUALMEMORY_HPP
 #define SHARE_GC_Z_ZVIRTUALMEMORY_HPP
 
+#include "gc/z/zAddress.hpp"
 #include "gc/z/zMemory.hpp"
 
 class ZVirtualMemory {
   friend class VMStructs;
 
 private:
-  uintptr_t _start;
-  uintptr_t _end;
+  zoffset     _start;
+  zoffset_end _end;
 
 public:
   ZVirtualMemory();
-  ZVirtualMemory(uintptr_t start, size_t size);
+  ZVirtualMemory(zoffset start, size_t size);
 
   bool is_null() const;
-  uintptr_t start() const;
-  uintptr_t end() const;
+  zoffset start() const;
+  zoffset_end end() const;
   size_t size() const;
 
   ZVirtualMemory split(size_t size);
 };
 
 class ZVirtualMemoryManager {
+  friend class ZMapperTest;
+
 private:
+  static size_t calculate_min_range(size_t size);
+
   ZMemoryManager _manager;
-  uintptr_t      _reserved;
+  size_t         _reserved;
   bool           _initialized;
 
   // Platform specific implementation
   void pd_initialize_before_reserve();
   void pd_initialize_after_reserve();
-  bool pd_reserve(uintptr_t addr, size_t size);
-  void pd_unreserve(uintptr_t addr, size_t size);
+  bool pd_reserve(zaddress_unsafe addr, size_t size);
+  void pd_unreserve(zaddress_unsafe addr, size_t size);
 
-  bool reserve_contiguous(uintptr_t start, size_t size);
+  bool reserve_contiguous(zoffset start, size_t size);
   bool reserve_contiguous(size_t size);
-  size_t reserve_discontiguous(uintptr_t start, size_t size, size_t min_range);
+  size_t reserve_discontiguous(zoffset start, size_t size, size_t min_range);
   size_t reserve_discontiguous(size_t size);
   bool reserve(size_t max_capacity);
 
-  void nmt_reserve(uintptr_t start, size_t size);
+  DEBUG_ONLY(size_t force_reserve_discontiguous(size_t size);)
 
 public:
   ZVirtualMemoryManager(size_t max_capacity);
@@ -71,7 +76,7 @@ public:
   bool is_initialized() const;
 
   size_t reserved() const;
-  uintptr_t lowest_available_address() const;
+  zoffset lowest_available_address() const;
 
   ZVirtualMemory alloc(size_t size, bool force_low_address);
   void free(const ZVirtualMemory& vmem);
