@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "classfile/classPrinter.hpp"
 #include "classfile/javaClasses.inline.hpp"
 #include "interpreter/bytecodeHistogram.hpp"
@@ -37,6 +36,7 @@
 #include "oops/method.hpp"
 #include "oops/resolvedFieldEntry.hpp"
 #include "oops/resolvedIndyEntry.hpp"
+#include "oops/resolvedMethodEntry.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/mutexLocker.hpp"
 #include "runtime/osThread.hpp"
@@ -95,7 +95,8 @@ class BytecodePrinter {
   // the adjustments that BytecodeStream performs applies.
   void trace(const methodHandle& method, address bcp, uintptr_t tos, uintptr_t tos2, outputStream* st) {
     ResourceMark rm;
-    if (_current_method != method()) {
+    bool method_changed = _current_method != method();
+    if (method_changed) {
       // Note 1: This code will not work as expected with true MT/MP.
       //         Need an explicit lock or a different solution.
       // It is possible for this block to be skipped, if a garbage
@@ -103,7 +104,7 @@ class BytecodePrinter {
       // the incoming method.  We could lose a line of trace output.
       // This is acceptable in a debug-only feature.
       st->cr();
-      st->print("[%ld] ", (long) Thread::current()->osthread()->thread_id());
+      st->print("[%zu] ", Thread::current()->osthread()->thread_id_for_printing());
       method->print_name(st);
       st->cr();
       _current_method = method();
@@ -118,17 +119,24 @@ class BytecodePrinter {
       code = Bytecodes::code_at(method(), bcp);
     }
     _code = code;
-     int bci = (int)(bcp - method->code_base());
-    st->print("[%ld] ", (long) Thread::current()->osthread()->thread_id());
-    if (Verbose) {
-      st->print("%8d  %4d  " INTPTR_FORMAT " " INTPTR_FORMAT " %s",
-           BytecodeCounter::counter_value(), bci, tos, tos2, Bytecodes::name(code));
-    } else {
-      st->print("%8d  %4d  %s",
-           BytecodeCounter::counter_value(), bci, Bytecodes::name(code));
-    }
     _next_pc = is_wide() ? bcp+2 : bcp+1;
-    print_attributes(bci, st);
+    // Trace each bytecode unless we're truncating the tracing output, then only print the first
+    // bytecode in every method as well as returns/throws that pop control flow
+    if (!TraceBytecodesTruncated || method_changed ||
+        code == Bytecodes::_athrow ||
+        code == Bytecodes::_return_register_finalizer ||
+        (code >= Bytecodes::_ireturn && code <= Bytecodes::_return)) {
+      int bci = (int)(bcp - method->code_base());
+      st->print("[%zu] ", Thread::current()->osthread()->thread_id_for_printing());
+      if (Verbose) {
+        st->print("%8d  %4d  " INTPTR_FORMAT " " INTPTR_FORMAT " %s",
+            BytecodeCounter::counter_value(), bci, tos, tos2, Bytecodes::name(code));
+      } else {
+        st->print("%8d  %4d  %s",
+            BytecodeCounter::counter_value(), bci, Bytecodes::name(code));
+      }
+      print_attributes(bci, st);
+    }
     // Set is_wide for the next one, since the caller of this doesn't skip
     // the next bytecode.
     _is_wide = (code == Bytecodes::_wide);
@@ -191,10 +199,13 @@ void BytecodeTracer::print_method_codes(const methodHandle& method, int from, in
   BytecodeStream s(method);
   s.set_interval(from, to);
 
-  ttyLocker ttyl;  // keep the following output coherent
+  // Keep output to st coherent: collect all lines and print at once.
+  ResourceMark rm;
+  stringStream ss;
   while (s.next() >= 0) {
-    method_printer.trace(method, s.bcp(), st);
+    method_printer.trace(method, s.bcp(), &ss);
   }
+  st->print("%s", ss.as_string());
 }
 
 void BytecodePrinter::print_constant(int cp_index, outputStream* st) {
@@ -326,20 +337,14 @@ void BytecodePrinter::print_bsm(int cp_index, outputStream* st) {
   st->print_cr("  }");
 }
 
-void BytecodePrinter::print_cpcache_entry(int cpc_index, outputStream* st) {
-  ConstantPool* constants = method()->constants();
-  ConstantPoolCacheEntry* cpce = constants->cache()->entry_at(cpc_index);
-  st->print("  ConstantPoolCacheEntry: ");
-  cpce->print(st, cpc_index, constants->cache());
-}
-
 void BytecodePrinter::print_attributes(int bci, outputStream* st) {
   // Show attributes of pre-rewritten codes
   Bytecodes::Code code = Bytecodes::java_code(raw_code());
   // If the code doesn't have any fields there's nothing to print.
   // note this is ==1 because the tableswitch and lookupswitch are
   // zero size (for some reason) and we want to print stuff out for them.
-  if (Bytecodes::length_for(code) == 1) {
+  // Also skip this if we're truncating bytecode output
+  if (TraceBytecodesTruncated || Bytecodes::length_for(code) == 1) {
     st->cr();
     return;
   }
@@ -516,20 +521,24 @@ void BytecodePrinter::print_attributes(int bci, outputStream* st) {
     case Bytecodes::_invokestatic:
       {
         int cp_index;
-        int cpcache_index;
         if (is_linked()) {
-          cpcache_index = get_native_index_u2();
-          cp_index = cpcache()->entry_at(cpcache_index)->constant_pool_index();
+          int method_index = get_native_index_u2();
+          ResolvedMethodEntry* method_entry = cpcache()->resolved_method_entry_at(method_index);
+          cp_index = method_entry->constant_pool_index();
+          print_field_or_method(cp_index, st);
+
+          if (raw_code() == Bytecodes::_invokehandle &&
+              ClassPrinter::has_mode(_flags, ClassPrinter::PRINT_METHOD_HANDLE)) {
+            assert(is_linked(), "invokehandle is only in rewritten methods");
+            method_entry->print_on(st);
+            if (method_entry->has_appendix()) {
+              st->print("  appendix: ");
+              constants()->resolved_reference_from_method(method_index)->print_on(st);
+            }
+          }
         } else {
-          cpcache_index = -1;
           cp_index = get_Java_index_u2();
-        }
-        print_field_or_method(cp_index, st);
-        if (raw_code() == Bytecodes::_invokehandle &&
-            ClassPrinter::has_mode(_flags, ClassPrinter::PRINT_METHOD_HANDLE)) {
-          assert(is_linked(), "invokehandle is only in rewritten methods");
-          assert(cpcache_index >= 0, "must be");
-          print_cpcache_entry(cpcache_index, st);
+          print_field_or_method(cp_index, st);
         }
       }
       break;
@@ -538,8 +547,8 @@ void BytecodePrinter::print_attributes(int bci, outputStream* st) {
       {
         int cp_index;
         if (is_linked()) {
-          int cpcache_index = get_native_index_u2();
-          cp_index = cpcache()->entry_at(cpcache_index)->constant_pool_index();
+          int method_index = get_native_index_u2();
+          cp_index = cpcache()->resolved_method_entry_at(method_index)->constant_pool_index();
         } else {
           cp_index = get_Java_index_u2();
         }
@@ -554,8 +563,7 @@ void BytecodePrinter::print_attributes(int bci, outputStream* st) {
         int indy_index;
         int cp_index;
         if (is_linked()) {
-          int i = get_native_index_u4();
-          indy_index = ConstantPool::decode_invokedynamic_index(i);
+          indy_index = get_native_index_u4();
           cp_index = constants()->resolved_indy_entry_at(indy_index)->constant_pool_index();
         } else {
           indy_index = -1;
@@ -591,6 +599,10 @@ void BytecodePrinter::print_attributes(int bci, outputStream* st) {
 void BytecodePrinter::bytecode_epilog(int bci, outputStream* st) {
   MethodData* mdo = method()->method_data();
   if (mdo != nullptr) {
+
+    // Lock to read ProfileData, and ensure lock is not broken by a safepoint
+    MutexLocker ml(mdo->extra_data_lock(), Mutex::_no_safepoint_check_flag);
+
     ProfileData* data = mdo->bci_to_data(bci);
     if (data != nullptr) {
       st->print("  %d ", mdo->dp_to_di(data->dp()));
