@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,17 +25,19 @@
 #ifndef SHARE_JFR_JFR_HPP
 #define SHARE_JFR_JFR_HPP
 
-#include "jni.h"
 #include "memory/allStatic.hpp"
-#include "utilities/exceptions.hpp"
 #include "oops/oopsHierarchy.hpp"
+#include "utilities/exceptions.hpp"
 #include "utilities/globalDefinitions.hpp"
 
 class CallInfo;
 class ciKlass;
 class ciMethod;
+class ClassFileParser;
 class GraphBuilder;
+class InstanceKlass;
 class JavaThread;
+struct JavaVMOption;
 class Klass;
 class outputStream;
 class Parse;
@@ -58,6 +60,8 @@ class Jfr : AllStatic {
   static bool is_excluded(Thread* thread);
   static void include_thread(Thread* thread);
   static void exclude_thread(Thread* thread);
+  static void on_klass_creation(InstanceKlass*& ik, ClassFileParser& parser, TRAPS);
+  static void on_klass_redefinition(const InstanceKlass* ik, Thread* thread);
   static void on_thread_start(Thread* thread);
   static void on_thread_exit(Thread* thread);
   static void on_resolution(const CallInfo& info, TRAPS);
@@ -70,6 +74,10 @@ class Jfr : AllStatic {
   static void on_vm_error_report(outputStream* st);
   static bool on_flight_recorder_option(const JavaVMOption** option, char* delimiter);
   static bool on_start_flight_recording_option(const JavaVMOption** option, char* delimiter);
+  static void on_backpatching(const Method* callee_method, JavaThread* jt);
+  static void initialize_main_thread(JavaThread* jt);
+  static bool has_sample_request(JavaThread* jt);
+  static void check_and_process_sample_request(JavaThread* jt);
 };
 
 #endif // SHARE_JFR_JFR_HPP
