@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "jfrfiles/jfrEventClasses.hpp"
 #include "jfr/jni/jfrJavaSupport.hpp"
 #include "jfr/leakprofiler/leakProfiler.hpp"
@@ -40,17 +39,19 @@
 #include "jfr/recorder/storage/jfrStorage.hpp"
 #include "jfr/recorder/storage/jfrStorageControl.hpp"
 #include "jfr/recorder/stringpool/jfrStringPool.hpp"
+#include "jfr/support/jfrDeprecationManager.hpp"
 #include "jfr/utilities/jfrAllocation.hpp"
+#include "jfr/utilities/jfrThreadIterator.hpp"
 #include "jfr/utilities/jfrTime.hpp"
 #include "jfr/writers/jfrJavaEventWriter.hpp"
 #include "jfr/utilities/jfrTypes.hpp"
 #include "logging/log.hpp"
 #include "runtime/atomic.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
+#include "runtime/javaThread.hpp"
 #include "runtime/mutexLocker.hpp"
 #include "runtime/os.hpp"
 #include "runtime/safepoint.hpp"
-#include "runtime/thread.inline.hpp"
 #include "runtime/vmOperations.hpp"
 #include "runtime/vmThread.hpp"
 
@@ -67,7 +68,7 @@ class JfrRotationLock : public StackObj {
 
   static bool acquire(Thread* thread) {
     if (Atomic::cmpxchg(&_lock, 0, 1) == 0) {
-      assert(_owner_thread == NULL, "invariant");
+      assert(_owner_thread == nullptr, "invariant");
       _owner_thread = thread;
       return true;
     }
@@ -86,7 +87,7 @@ class JfrRotationLock : public StackObj {
 
  public:
   JfrRotationLock() : _thread(Thread::current()), _recursive(false) {
-    assert(_thread != NULL, "invariant");
+    assert(_thread != nullptr, "invariant");
     if (_thread == _owner_thread) {
       // Recursive case is not supported.
       _recursive = true;
@@ -103,7 +104,7 @@ class JfrRotationLock : public StackObj {
     if (_recursive) {
       return;
     }
-    _owner_thread = NULL;
+    _owner_thread = nullptr;
     OrderAccess::storestore();
     _lock = 0;
   }
@@ -117,10 +118,34 @@ class JfrRotationLock : public StackObj {
   }
 };
 
-const Thread* JfrRotationLock::_owner_thread = NULL;
+const Thread* JfrRotationLock::_owner_thread = nullptr;
 const int JfrRotationLock::retry_wait_millis = 10;
 volatile int JfrRotationLock::_lock = 0;
 
+// Reset thread local state used for object allocation sampling.
+class ClearObjectAllocationSampling : public ThreadClosure {
+ public:
+  void do_thread(Thread* t) {
+    assert(t != nullptr, "invariant");
+    t->jfr_thread_local()->clear_last_allocated_bytes();
+  }
+};
+
+template <typename Iterator>
+static inline void iterate(Iterator& it, ClearObjectAllocationSampling& coas) {
+  while (it.has_next()) {
+    coas.do_thread(it.next());
+  }
+}
+
+static void clear_object_allocation_sampling() {
+  ClearObjectAllocationSampling coas;
+  JfrJavaThreadIterator jit;
+  iterate(jit, coas);
+  JfrNonJavaThreadIterator njit;
+  iterate(njit, coas);
+}
+
 template <typename Instance, size_t(Instance::*func)()>
 class Content {
  private:
@@ -367,14 +392,22 @@ static u4 write_metadata(JfrChunkWriter& chunkwriter) {
   return invoke(wm);
 }
 
-template <typename Instance, void(Instance::*func)()>
-class JfrVMOperation : public VM_Operation {
+class JfrSafepointClearVMOperation : public VM_Operation {
  private:
-  Instance& _instance;
+  JfrRecorderService& _instance;
  public:
-  JfrVMOperation(Instance& instance) : _instance(instance) {}
-  void doit() { (_instance.*func)(); }
-  VMOp_Type type() const { return VMOp_JFRCheckpoint; }
+  JfrSafepointClearVMOperation(JfrRecorderService& instance) : _instance(instance) {}
+  void doit() { _instance.safepoint_clear(); }
+  VMOp_Type type() const { return VMOp_JFRSafepointClear; }
+};
+
+class JfrSafepointWriteVMOperation : public VM_Operation {
+ private:
+  JfrRecorderService& _instance;
+ public:
+  JfrSafepointWriteVMOperation(JfrRecorderService& instance) : _instance(instance) {}
+  void doit() { _instance.safepoint_write(); }
+  VMOp_Type type() const { return VMOp_JFRSafepointWrite; }
 };
 
 JfrRecorderService::JfrRecorderService() :
@@ -406,6 +439,7 @@ static void start_recorder() {
 
 static void stop_recorder() {
   assert(JfrRotationLock::is_owner(), "invariant");
+  JfrDeprecationManager::on_recorder_stop();
   set_recorder_state(RUNNING, STOPPED);
   log_debug(jfr, system)("Recording service STOPPED");
 }
@@ -437,23 +471,25 @@ void JfrRecorderService::clear() {
 }
 
 void JfrRecorderService::pre_safepoint_clear() {
+  clear_object_allocation_sampling();
   _storage.clear();
   JfrStackTraceRepository::clear();
 }
 
 void JfrRecorderService::invoke_safepoint_clear() {
-  JfrVMOperation<JfrRecorderService, &JfrRecorderService::safepoint_clear> safepoint_task(*this);
+  JfrSafepointClearVMOperation op(*this);
   ThreadInVMfromNative transition(JavaThread::current());
-  VMThread::execute(&safepoint_task);
+  VMThread::execute(&op);
 }
 
 void JfrRecorderService::safepoint_clear() {
   assert(SafepointSynchronize::is_at_safepoint(), "invariant");
-  _checkpoint_manager.begin_epoch_shift();
   _storage.clear();
+  _checkpoint_manager.notify_threads();
   _chunkwriter.set_time_stamp();
+  JfrDeprecationManager::on_safepoint_clear();
   JfrStackTraceRepository::clear();
-  _checkpoint_manager.end_epoch_shift();
+  _checkpoint_manager.shift_epoch();
 }
 
 void JfrRecorderService::post_safepoint_clear() {
@@ -480,6 +516,7 @@ void JfrRecorderService::vm_error_rotation() {
     Thread* const thread = Thread::current();
     _storage.flush_regular_buffer(thread->jfr_thread_local()->native_buffer(), thread);
     _chunkwriter.mark_chunk_final();
+    JfrDeprecationManager::write_edges(_chunkwriter, thread, true);
     invoke_flush();
     _chunkwriter.set_time_stamp();
     _repository.close_chunk();
@@ -547,21 +584,21 @@ void JfrRecorderService::pre_safepoint_write() {
 }
 
 void JfrRecorderService::invoke_safepoint_write() {
-  JfrVMOperation<JfrRecorderService, &JfrRecorderService::safepoint_write> safepoint_task(*this);
+  JfrSafepointWriteVMOperation op(*this);
   // can safepoint here
   ThreadInVMfromNative transition(JavaThread::current());
-  VMThread::execute(&safepoint_task);
+  VMThread::execute(&op);
 }
 
 void JfrRecorderService::safepoint_write() {
   assert(SafepointSynchronize::is_at_safepoint(), "invariant");
-  _checkpoint_manager.begin_epoch_shift();
   JfrStackTraceRepository::clear_leak_profiler();
   _checkpoint_manager.on_rotation();
   _storage.write_at_safepoint();
   _chunkwriter.set_time_stamp();
+  JfrDeprecationManager::on_safepoint_write();
   write_stacktrace(_stack_trace_repository, _chunkwriter, true);
-  _checkpoint_manager.end_epoch_shift();
+  _checkpoint_manager.shift_epoch();
 }
 
 void JfrRecorderService::post_safepoint_write() {
@@ -583,13 +620,13 @@ void JfrRecorderService::post_safepoint_write() {
 }
 
 static JfrBuffer* thread_local_buffer(Thread* t) {
-  assert(t != NULL, "invariant");
+  assert(t != nullptr, "invariant");
   return t->jfr_thread_local()->native_buffer();
 }
 
 static void reset_buffer(JfrBuffer* buffer, Thread* t) {
-  assert(buffer != NULL, "invariant");
-  assert(t != NULL, "invariant");
+  assert(buffer != nullptr, "invariant");
+  assert(t != nullptr, "invariant");
   assert(buffer == thread_local_buffer(t), "invariant");
   buffer->set_pos(const_cast<u1*>(buffer->top()));
 }
@@ -600,7 +637,7 @@ static void reset_thread_local_buffer(Thread* t) {
 
 static void write_thread_local_buffer(JfrChunkWriter& chunkwriter, Thread* t) {
   JfrBuffer * const buffer = thread_local_buffer(t);
-  assert(buffer != NULL, "invariant");
+  assert(buffer != nullptr, "invariant");
   if (!buffer->empty()) {
     chunkwriter.write_unbuffered(buffer->top(), buffer->pos() - buffer->top());
   }
@@ -608,17 +645,11 @@ static void write_thread_local_buffer(JfrChunkWriter& chunkwriter, Thread* t) {
 
 size_t JfrRecorderService::flush() {
   size_t total_elements = flush_metadata(_chunkwriter);
-  const size_t storage_elements = flush_storage(_storage, _chunkwriter);
-  if (0 == storage_elements) {
-    return total_elements;
-  }
-  total_elements += storage_elements;
+  total_elements = flush_storage(_storage, _chunkwriter);
   if (_string_pool.is_modified()) {
     total_elements += flush_stringpool(_string_pool, _chunkwriter);
   }
-  if (_stack_trace_repository.is_modified()) {
-    total_elements += flush_stacktrace(_stack_trace_repository, _chunkwriter);
-  }
+  total_elements += flush_stacktrace(_stack_trace_repository, _chunkwriter);
   return flush_typeset(_checkpoint_manager, _chunkwriter) + total_elements;
 }
 
@@ -658,3 +689,18 @@ void JfrRecorderService::evaluate_chunk_size_for_rotation() {
   DEBUG_ONLY(JfrJavaSupport::check_java_thread_in_native(JavaThread::current()));
   JfrChunkRotation::evaluate(_chunkwriter);
 }
+
+void JfrRecorderService::emit_leakprofiler_events(int64_t cutoff_ticks, bool emit_all, bool skip_bfs) {
+  DEBUG_ONLY(JfrJavaSupport::check_java_thread_in_native(JavaThread::current()));
+  // Take the rotation lock to exclude flush() during event emits. This is because event emit
+  // also creates a number checkpoint events. Those checkpoint events require a future typeset checkpoint
+  // event for completeness, i.e. to be generated before being flushed to a segment.
+  // The upcoming flush() or rotation() after event emit completes this typeset checkpoint
+  // and serializes all event emit checkpoint events to the same segment.
+  JfrRotationLock lock;
+  // Take the rotation lock before the transition.
+  JavaThread* current_thread = JavaThread::current();
+  MACOS_AARCH64_ONLY(ThreadWXEnable __wx(WXWrite, current_thread));
+  ThreadInVMfromNative transition(current_thread);
+  LeakProfiler::emit_events(cutoff_ticks, emit_all, skip_bfs);
+}
