@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,12 +22,12 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "jfr/recorder/storage/jfrVirtualMemory.hpp"
+#include "memory/memoryReserver.hpp"
 #include "memory/virtualspace.hpp"
+#include "nmt/memTracker.hpp"
 #include "runtime/globals.hpp"
 #include "runtime/os.hpp"
-#include "services/memTracker.hpp"
 #include "utilities/globalDefinitions.hpp"
 
 /*
@@ -97,14 +97,16 @@ JfrVirtualMemorySegment::JfrVirtualMemorySegment() :
 
 JfrVirtualMemorySegment::~JfrVirtualMemorySegment() {
   decommit();
-  _rs.release();
+  if (_rs.is_reserved()) {
+    MemoryReserver::release(_rs);
+  }
 }
 
 bool JfrVirtualMemorySegment::initialize(size_t reservation_size_request_bytes) {
   assert(is_aligned(reservation_size_request_bytes, os::vm_allocation_granularity()), "invariant");
-  _rs = ReservedSpace(reservation_size_request_bytes,
-                      os::vm_allocation_granularity(),
-                      os::vm_page_size());
+  _rs = MemoryReserver::reserve(reservation_size_request_bytes,
+                                os::vm_allocation_granularity(),
+                                os::vm_page_size());
   if (!_rs.is_reserved()) {
     return false;
   }
@@ -117,7 +119,7 @@ bool JfrVirtualMemorySegment::initialize(size_t reservation_size_request_bytes)
                               _rs.base(),
                               _rs.size(),
                               os::vm_page_size());
-  MemTracker::record_virtual_memory_type((address)_rs.base(), mtTracing);
+  MemTracker::record_virtual_memory_tag((address)_rs.base(), mtTracing);
   assert(is_aligned(_rs.base(), os::vm_page_size()), "invariant");
   assert(is_aligned(_rs.size(), os::vm_page_size()), "invariant");
 
@@ -420,11 +422,11 @@ void* JfrVirtualMemory::initialize(size_t reservation_size_request_bytes,
   _aligned_datum_size_bytes = align_up(datum_size_bytes, BytesPerWord);
   assert(is_aligned(_aligned_datum_size_bytes, BytesPerWord), "invariant");
 
-  reservation_size_request_bytes = ReservedSpace::allocation_align_size_up(reservation_size_request_bytes);
+  reservation_size_request_bytes = os::align_up_vm_allocation_granularity(reservation_size_request_bytes);
   assert(is_aligned(reservation_size_request_bytes, os::vm_allocation_granularity()), "invariant");
   assert(is_aligned(reservation_size_request_bytes, _aligned_datum_size_bytes), "invariant");
   block_size_request_bytes = MAX2(block_size_request_bytes, (size_t)os::vm_allocation_granularity());
-  block_size_request_bytes = ReservedSpace::allocation_align_size_up(block_size_request_bytes);
+  block_size_request_bytes = os::align_up_vm_allocation_granularity(block_size_request_bytes);
   assert(is_aligned(block_size_request_bytes, os::vm_allocation_granularity()), "invariant");
   assert(is_aligned(block_size_request_bytes, _aligned_datum_size_bytes), "invariant");
   // adjustment to valid ratio in units of vm_allocation_granularity
