@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -21,7 +21,6 @@
  * questions.
  */
 
-#include "precompiled.hpp"
 #include "classfile/symbolTable.hpp"
 #include "classfile/systemDictionary.hpp"
 #include "classfile/vmClasses.hpp"
@@ -67,7 +66,7 @@ void HotSpotJVMCI::compute_offset(int &dest_offset, Klass* klass, const char* na
   InstanceKlass* ik = InstanceKlass::cast(klass);
   Symbol* name_symbol = SymbolTable::probe(name, (int)strlen(name));
   Symbol* signature_symbol = SymbolTable::probe(signature, (int)strlen(signature));
-  if (name_symbol == NULL || signature_symbol == NULL) {
+  if (name_symbol == nullptr || signature_symbol == nullptr) {
 #ifndef PRODUCT
     ik->print_on(tty);
 #endif
@@ -80,20 +79,25 @@ void HotSpotJVMCI::compute_offset(int &dest_offset, Klass* klass, const char* na
     fatal("Could not find field %s.%s with signature %s", ik->external_name(), name, signature);
   }
   guarantee(fd.is_static() == static_field, "static/instance mismatch");
-  dest_offset = fd.offset();
-  assert(dest_offset != 0, "must be valid offset");
-  if (static_field) {
-    // Must ensure classes for static fields are initialized as the
-    // accessor itself does not include a class initialization check.
-    ik->initialize(CHECK);
+  assert(fd.offset() != 0, "must be valid offset");
+  if (dest_offset != fd.offset()) {
+    if (dest_offset != 0) {
+      fatal("offset for %s %s.%s re-initialized: %d -> %d", signature, ik->external_name(), name, dest_offset, fd.offset());
+    }
+    dest_offset = fd.offset();
+    if (static_field) {
+      // Must ensure classes for static fields are initialized as the
+      // accessor itself does not include a class initialization check.
+      ik->initialize(CHECK);
+    }
+    JVMCI_event_2("   field offset for %s %s.%s = %d", signature, ik->external_name(), name, dest_offset);
   }
-  JVMCI_event_2("   field offset for %s %s.%s = %d", signature, ik->external_name(), name, dest_offset);
 }
 
 #ifndef PRODUCT
 static void check_resolve_method(const char* call_type, Klass* resolved_klass, Symbol* method_name, Symbol* method_signature, TRAPS) {
-  Method* method = NULL;
-  LinkInfo link_info(resolved_klass, method_name, method_signature, NULL, LinkInfo::AccessCheck::skip, LinkInfo::LoaderConstraintCheck::skip);
+  Method* method = nullptr;
+  LinkInfo link_info(resolved_klass, method_name, method_signature, nullptr, LinkInfo::AccessCheck::skip, LinkInfo::LoaderConstraintCheck::skip);
   if (strcmp(call_type, "call_static") == 0) {
     method = LinkResolver::resolve_static_call_or_null(link_info);
   } else if (strcmp(call_type, "call_virtual") == 0) {
@@ -103,7 +107,7 @@ static void check_resolve_method(const char* call_type, Klass* resolved_klass, S
   } else {
     fatal("Unknown or unsupported call type: %s", call_type);
   }
-  if (method == NULL) {
+  if (method == nullptr) {
     fatal("Could not resolve %s.%s%s", resolved_klass->external_name(), method_name->as_C_string(), method_signature->as_C_string());
   }
 }
@@ -120,11 +124,18 @@ jmethodID JNIJVMCI::_HotSpotConstantPool_fromMetaspace_method;
 jmethodID JNIJVMCI::_HotSpotResolvedObjectTypeImpl_fromMetaspace_method;
 jmethodID JNIJVMCI::_HotSpotResolvedPrimitiveType_fromMetaspace_method;
 
-#define START_CLASS(className, fullClassName)                          { \
+#define START_CLASS(className, fullClassName)                          {                 \
   Klass* k = SystemDictionary::resolve_or_fail(vmSymbols::fullClassName(), true, CHECK); \
-  className::_klass = InstanceKlass::cast(k);                                     \
-  JVMCI_event_2(" klass for %s = " PTR_FORMAT, k->external_name(), p2i(k));       \
-  className::_klass->initialize(CHECK);
+  InstanceKlass* current = className::_klass;                                            \
+  if (current != InstanceKlass::cast(k)) {                                               \
+    if (current != nullptr) {                                                            \
+      fatal("klass for %s re-initialized: " PTR_FORMAT " -> " PTR_FORMAT,                \
+          k->external_name(), p2i(current), p2i(k));                                     \
+    }                                                                                    \
+    JVMCI_event_2(" klass for %s = " PTR_FORMAT, k->external_name(), p2i(k));            \
+    className::_klass = InstanceKlass::cast(k);                                          \
+    className::_klass->initialize(CHECK);                                                \
+  }
 
 #define END_CLASS }
 
@@ -139,10 +150,10 @@ jmethodID JNIJVMCI::_HotSpotResolvedPrimitiveType_fromMetaspace_method;
 #define STATIC_INT_FIELD(className, name) FIELD(className, name, "I", true)
 #define STATIC_BOOLEAN_FIELD(className, name) FIELD(className, name, "Z", true)
 #ifdef PRODUCT
-#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName, args)
+#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName)
 #define CONSTRUCTOR(className, signature)
 #else
-#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName, args) \
+#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName) \
   check_resolve_method(#hsCallType, k, vmSymbols::methodName##_name(), vmSymbols::signatureSymbolName(), CHECK);
 #define CONSTRUCTOR(className, signature) { \
   TempNewSymbol sig = SymbolTable::new_symbol(signature); \
@@ -187,11 +198,11 @@ void HotSpotJVMCI::compute_offsets(TRAPS) {
     return resolve(object)->is_a(className::klass());                                   \
   }                                                                                     \
   void HotSpotJVMCI::className::check(oop obj, const char* field_name, int offset) {    \
-    assert(obj != NULL, "NULL field access of %s.%s", #className, field_name); \
+    assert(obj != nullptr, "null field access of %s.%s", #className, field_name);       \
     assert(obj->is_a(className::klass()), "wrong class, " #className " expected, found %s", obj->klass()->external_name()); \
     assert(offset != 0, "must be valid offset");                                        \
   }                                                                                     \
-  InstanceKlass* HotSpotJVMCI::className::_klass = NULL;
+  InstanceKlass* HotSpotJVMCI::className::_klass = nullptr;
 
 #define END_CLASS
 
@@ -213,28 +224,28 @@ void HotSpotJVMCI::compute_offsets(TRAPS) {
 #define STATIC_OBJECTARRAY_FIELD(className, name, signature) STATIC_OOPISH_FIELD(className, name, objArrayOop)
 #define STATIC_OOPISH_FIELD(className, name, type)                                                                        \
     type HotSpotJVMCI::className::name(JVMCIEnv* env) {                                                                   \
-      assert(className::klass() != NULL && className::klass()->is_linked(), "Class not yet linked: " #className);         \
+      assert(className::klass() != nullptr && className::klass()->is_linked(), "Class not yet linked: " #className);      \
       InstanceKlass* ik = className::klass();                                                                             \
       oop base = ik->static_field_base_raw();                                                                             \
       oop result = HeapAccess<>::oop_load_at(base, className::_##name##_offset);                                          \
       return type(result);                                                                                                \
     }                                                                                                                     \
     void HotSpotJVMCI::className::set_##name(JVMCIEnv* env, type x) {                                                     \
-      assert(className::klass() != NULL && className::klass()->is_linked(), "Class not yet linked: " #className);         \
-      assert(className::klass() != NULL, "Class not yet loaded: " #className);                                            \
+      assert(className::klass() != nullptr && className::klass()->is_linked(), "Class not yet linked: " #className);      \
+      assert(className::klass() != nullptr, "Class not yet loaded: " #className);                                         \
       InstanceKlass* ik = className::klass();                                                                             \
       oop base = ik->static_field_base_raw();                                                                             \
       HeapAccess<>::oop_store_at(base, className::_##name##_offset, x);                                                   \
     }
 #define STATIC_PRIMITIVE_FIELD(className, name, jtypename)                                                                \
     jtypename HotSpotJVMCI::className::get_##name(JVMCIEnv* env) {                                                        \
-      assert(className::klass() != NULL && className::klass()->is_linked(), "Class not yet linked: " #className);         \
+      assert(className::klass() != nullptr && className::klass()->is_linked(), "Class not yet linked: " #className);      \
       InstanceKlass* ik = className::klass();                                                                             \
       oop base = ik->static_field_base_raw();                                                                             \
       return *base->field_addr<jtypename>(className::_##name##_offset);                                                   \
     }                                                                                                                     \
     void HotSpotJVMCI::className::set_##name(JVMCIEnv* env, jtypename x) {                                                \
-      assert(className::klass() != NULL && className::klass()->is_linked(), "Class not yet linked: " #className);         \
+      assert(className::klass() != nullptr && className::klass()->is_linked(), "Class not yet linked: " #className);      \
       InstanceKlass* ik = className::klass();                                                                             \
       oop base = ik->static_field_base_raw();                                                                             \
       *base->field_addr<jtypename>(className::_##name##_offset) = x;                                                      \
@@ -242,7 +253,7 @@ void HotSpotJVMCI::compute_offsets(TRAPS) {
 
 #define STATIC_INT_FIELD(className, name) STATIC_PRIMITIVE_FIELD(className, name, jint)
 #define STATIC_BOOLEAN_FIELD(className, name) STATIC_PRIMITIVE_FIELD(className, name, jboolean)
-#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName, args)
+#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName)
 #define CONSTRUCTOR(className, signature)
 
 /**
@@ -254,7 +265,7 @@ void HotSpotJVMCI::compute_offsets(TRAPS) {
  *  oop HotSpotJVMCI::Architecture::wordKind(JVMCIEnv* env, oop obj) { ... }
  * void HotSpotJVMCI::Architecture::set_wordKind(JVMCIEnv* env, oop obj, oop x) { ... }
  *
- * InstanceKlass *HotSpotJVMCI::Architecture::_klass = NULL;
+ * InstanceKlass *HotSpotJVMCI::Architecture::_klass = nullptr;
  */
 JVMCI_CLASSES_DO(START_CLASS, END_CLASS, CHAR_FIELD, INT_FIELD, BOOLEAN_FIELD, LONG_FIELD, FLOAT_FIELD, OBJECT_FIELD, PRIMARRAY_FIELD, OBJECTARRAY_FIELD, STATIC_OBJECT_FIELD, STATIC_OBJECTARRAY_FIELD, STATIC_INT_FIELD, STATIC_BOOLEAN_FIELD, METHOD, CONSTRUCTOR)
 
@@ -284,7 +295,7 @@ JVMCI_CLASSES_DO(START_CLASS, END_CLASS, CHAR_FIELD, INT_FIELD, BOOLEAN_FIELD, L
  * this ensures the declaring class is initialized.
  */
 void JNIJVMCI::initialize_field_id(JNIEnv* env, jfieldID &fieldid, jclass clazz, const char* class_name, const char* name, const char* signature, bool static_field) {
-  if (JVMCILibDumpJNIConfig != NULL) {
+  if (JVMCILibDumpJNIConfig != nullptr) {
     fileStream* st = JVMCIGlobals::get_jni_config_file();
     st->print_cr("field %s %s %s", class_name, name, signature);
     return;
@@ -300,7 +311,16 @@ void JNIJVMCI::initialize_field_id(JNIEnv* env, jfieldID &fieldid, jclass clazz,
     // Class initialization barrier
     fieldid = env->GetFieldID(clazz, name, signature);
   }
-  JVMCI_event_2("   jfieldID for %s %s.%s = " PTR_FORMAT, signature, class_name, name, p2i(fieldid));
+  // SVM guarantees that jfieldIDs for fields in the native image are also
+  // in the image and thus always have the same address.
+  if (current != fieldid) {
+    if (current != nullptr) {
+      fatal("jfieldID for %s %s.%s re-initialized: " PTR_FORMAT " -> " PTR_FORMAT,
+         signature, class_name, name, p2i(current), p2i(fieldid));
+    }
+    JVMCI_event_2("   jfieldID for %s %s.%s = " PTR_FORMAT, signature, class_name, name, p2i(fieldid));
+  }
+
 
   if (env->ExceptionCheck()) {
     env->ExceptionDescribe();
@@ -312,19 +332,29 @@ void JNIJVMCI::initialize_field_id(JNIEnv* env, jfieldID &fieldid, jclass clazz,
 
 #define START_CLASS(className, fullClassName) {                                             \
   current_class_name = vmSymbols::fullClassName()->as_C_string();                           \
-  if (JVMCILibDumpJNIConfig != NULL) {                                                      \
+  if (JVMCILibDumpJNIConfig != nullptr) {                                                   \
     fileStream* st = JVMCIGlobals::get_jni_config_file();                                   \
     st->print_cr("class %s", current_class_name);                                           \
   } else {                                                                                  \
     jclass k = env->FindClass(current_class_name);                                          \
     JVMCI_EXCEPTION_CHECK(env, "FindClass(%s)", current_class_name);                        \
-    assert(k != NULL, #fullClassName " not initialized");                                   \
+    assert(k != nullptr, #fullClassName " not initialized");                                \
     k = (jclass) env->NewGlobalRef(k);                                                      \
-    JVMCI_event_2(" jclass for %s = " PTR_FORMAT, current_class_name, p2i(k));              \
-    className::_class = k;                                                                  \
+    jclass current = className::_class;                                                     \
+    if (current != k) {                                                                     \
+      JVMCI_event_2(" jclass for %s = " PTR_FORMAT, current_class_name, p2i(k));            \
+      /* SVM guarantees that jclass handles to classes in a native image are also */        \
+      /* in the image. Further calling NewGlobalRef on such a handle returns a stable */    \
+      /* value across all JavaVMs executing on the same native image. */                    \
+      if (current != nullptr) {                                                             \
+           fatal("jclass for %s re-initialized: " PTR_FORMAT " -> " PTR_FORMAT,             \
+           current_class_name, p2i(current), p2i(k));                                       \
+      }                                                                                     \
+      className::_class = k;                                                                \
+    }                                                                                       \
   }
 
-#define END_CLASS current_class_name = NULL; }
+#define END_CLASS current_class_name = nullptr; }
 
 #define FIELD(className, name, signature, static_field) initialize_field_id(env, className::_##name##_field_id, className::_class, current_class_name, #name, signature, static_field);
 #define CHAR_FIELD(className, name) FIELD(className, name, "C", false)
@@ -338,23 +368,31 @@ void JNIJVMCI::initialize_field_id(JNIEnv* env, jfieldID &fieldid, jclass clazz,
 #define STATIC_BOOLEAN_FIELD(className, name) FIELD(className, name, "Z", true)
 
 #define GET_JNI_METHOD(jniGetMethod, dst, clazz, methodName, signature)                        \
-    if (JVMCILibDumpJNIConfig != NULL) {                                                       \
+    if (JVMCILibDumpJNIConfig != nullptr) {                                                    \
       fileStream* st = JVMCIGlobals::get_jni_config_file();                                    \
       st->print_cr("method %s %s %s", current_class_name, methodName, signature);              \
     } else {                                                                                   \
       jmethodID current = dst;                                                                 \
       dst = env->jniGetMethod(clazz, methodName, signature);                                   \
-      JVMCI_EXCEPTION_CHECK(env, #jniGetMethod "(%s.%s%s)",                                    \
-                  current_class_name, methodName, signature);                                  \
-      assert(dst != NULL, "uninitialized");                                                    \
-      JVMCI_event_2("   jmethodID for %s.%s%s = " PTR_FORMAT,                                  \
-                  current_class_name, methodName, signature, p2i(dst));                        \
+      assert(dst != nullptr, "uninitialized");                                                 \
+      if (current != dst) {                                                                    \
+        JVMCI_event_2("   jmethodID for %s.%s%s = " PTR_FORMAT,                                \
+                    current_class_name, methodName, signature, p2i(dst));                      \
+        /* SVM guarantees that jmethodIDs for methods in the native image are also */          \
+        /* in the image and thus always have the same address. */                              \
+        if (current != nullptr) {                                                              \
+          fatal("jmethod for %s.%s%s re-initialized: " PTR_FORMAT " -> " PTR_FORMAT,           \
+                        current_class_name, methodName, signature, p2i(current), p2i(dst));    \
+        }                                                                                      \
+        JVMCI_EXCEPTION_CHECK(env, #jniGetMethod "(%s.%s%s)",                                  \
+                            current_class_name, methodName, signature);                        \
+      }                                                                                        \
     }
 
 #define GET_JNI_CONSTRUCTOR(clazz, signature) \
   GET_JNI_METHOD(GetMethodID, JNIJVMCI::clazz::_constructor, clazz::_class, "<init>", signature) \
 
-#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName, args) \
+#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName) \
      GET_JNI_METHOD(jniGetMethod,                                        \
                     className::_##methodName##_method,                   \
                     className::clazz(),                                  \
@@ -367,6 +405,7 @@ void JNIJVMCI::initialize_field_id(JNIEnv* env, jfieldID &fieldid, jclass clazz,
 extern "C" {
   void     JNICALL JVM_RegisterJVMCINatives(JNIEnv *env, jclass compilerToVMClass);
   jobject  JNICALL JVM_GetJVMCIRuntime(JNIEnv *env, jclass c);
+  jlong    JNICALL JVM_ReadSystemPropertiesInfo(JNIEnv *env, jclass c, jintArray offsets_handle);
 }
 
 // Dumps symbols for public <init>() and <init>(String) methods of
@@ -380,21 +419,21 @@ class ThrowableInitDumper : public SymbolClosure {
   void do_symbol(Symbol** p) {
     JavaThread* THREAD = JavaThread::current(); // For exception macros.
     Symbol* name = *p;
-    if (name == NULL) {
+    if (name == nullptr) {
       return;
     }
     Klass* k = SystemDictionary::resolve_or_null(name, CHECK_EXIT);
-    if (k != NULL && k->is_instance_klass()) {
+    if (k != nullptr && k->is_instance_klass()) {
       InstanceKlass* iklass = InstanceKlass::cast(k);
       if (iklass->is_subclass_of(vmClasses::Throwable_klass()) && iklass->is_public() && !iklass->is_abstract()) {
-        const char* class_name = NULL;
+        const char* class_name = nullptr;
         Array<Method*>* methods = iklass->methods();
         for (int i = 0; i < methods->length(); i++) {
           Method* m = methods->at(i);
           if (m->name() == vmSymbols::object_initializer_name() &&
               m->is_public() &&
               (m->signature() == vmSymbols::void_method_signature() || m->signature() == vmSymbols::string_void_signature())) {
-            if (class_name == NULL) {
+            if (class_name == nullptr) {
               class_name = name->as_C_string();
               _st->print_cr("class %s", class_name);
             }
@@ -412,7 +451,7 @@ class ThrowableInitDumper : public SymbolClosure {
  */
 void JNIJVMCI::initialize_ids(JNIEnv* env) {
   ResourceMark rm;
-  const char* current_class_name = NULL;
+  const char* current_class_name = nullptr;
   JVMCI_CLASSES_DO(START_CLASS, END_CLASS, CHAR_FIELD, INT_FIELD, BOOLEAN_FIELD, LONG_FIELD, FLOAT_FIELD, OBJECT_FIELD, OBJECT_FIELD, OBJECT_FIELD, STATIC_OBJECT_FIELD, STATIC_OBJECT_FIELD, STATIC_INT_FIELD, STATIC_BOOLEAN_FIELD, METHOD, CONSTRUCTOR)
 
   IN_CLASS(java_lang_Class);
@@ -447,11 +486,11 @@ void JNIJVMCI::initialize_ids(JNIEnv* env) {
 
 #define DO_BOX_CLASS(klass, basicType, type) \
   current_class_name = "java/lang/" #klass;                                                                       \
-  if (JVMCILibDumpJNIConfig == NULL) {                                                                            \
+  if (JVMCILibDumpJNIConfig == nullptr) {                                                                         \
     _box_classes[basicType] = env->FindClass("java/lang/" #klass);                                                \
     JVMCI_EXCEPTION_CHECK(env, "FindClass(%s)", #klass);                                                          \
     _box_classes[basicType] = (jclass) env->NewGlobalRef(_box_classes[basicType]);                                \
-    assert(_box_classes[basicType] != NULL, "uninitialized");                                                     \
+    assert(_box_classes[basicType] != nullptr, "uninitialized");                                                  \
     _box_fields[basicType] = env->GetFieldID(_box_classes[basicType], "value", #type);                            \
     JVMCI_EXCEPTION_CHECK(env, "GetFieldID(%s, value, %s)", #klass, #type);                                       \
     GET_JNI_METHOD(GetMethodID, _box_constructors[basicType], _box_classes[basicType], "<init>", "(" #type ")V"); \
@@ -463,11 +502,11 @@ void JNIJVMCI::initialize_ids(JNIEnv* env) {
 
   BOX_CLASSES(DO_BOX_CLASS);
 
-  if (JVMCILibDumpJNIConfig == NULL) {
+  if (JVMCILibDumpJNIConfig == nullptr) {
     _byte_array = env->FindClass("[B");
     JVMCI_EXCEPTION_CHECK(env, "FindClass([B)");
     _byte_array = (jclass) env->NewGlobalRef(_byte_array);
-    assert(_byte_array != NULL, "uninitialized");
+    assert(_byte_array != nullptr, "uninitialized");
   } else {
     fileStream* st = JVMCIGlobals::get_jni_config_file();
     st->print_cr("class [B");
@@ -486,7 +525,7 @@ void JNIJVMCI::initialize_ids(JNIEnv* env) {
   }                                                                                                                 \
 } while(0)
 
-  if (JVMCILibDumpJNIConfig != NULL) {
+  if (JVMCILibDumpJNIConfig != nullptr) {
     JavaThread* THREAD = JavaThread::current(); // For exception macros.
     fileStream* st = JVMCIGlobals::get_jni_config_file();
 
@@ -509,7 +548,7 @@ void JNIJVMCI::initialize_ids(JNIEnv* env) {
 }
 
 static void register_natives_for_class(JNIEnv* env, jclass clazz, const char* name, const JNINativeMethod *methods, jint nMethods) {
-  if (clazz == NULL) {
+  if (clazz == nullptr) {
     clazz = env->FindClass(name);
     if (env->ExceptionCheck()) {
       env->ExceptionDescribe();
@@ -526,10 +565,12 @@ static void register_natives_for_class(JNIEnv* env, jclass clazz, const char* na
 void JNIJVMCI::register_natives(JNIEnv* env) {
   if (env != JavaThread::current()->jni_environment()) {
     JNINativeMethod CompilerToVM_nmethods[] = {{ CC"registerNatives", CC"()V", FN_PTR(JVM_RegisterJVMCINatives) }};
-    JNINativeMethod JVMCI_nmethods[] = {{ CC"initializeRuntime",   CC"()Ljdk/vm/ci/runtime/JVMCIRuntime;", FN_PTR(JVM_GetJVMCIRuntime) }};
+    JNINativeMethod JVMCI_nmethods[] = {{ CC"initializeRuntime", CC"()Ljdk/vm/ci/runtime/JVMCIRuntime;", FN_PTR(JVM_GetJVMCIRuntime) }};
+    JNINativeMethod Services_nmethods[] = {{ CC"readSystemPropertiesInfo", CC"([I)J", FN_PTR(JVM_ReadSystemPropertiesInfo) }};
 
-    register_natives_for_class(env, NULL, "jdk/vm/ci/hotspot/CompilerToVM", CompilerToVM_nmethods, 1);
+    register_natives_for_class(env, nullptr, "jdk/vm/ci/hotspot/CompilerToVM", CompilerToVM_nmethods, 1);
     register_natives_for_class(env, JVMCI::clazz(), "jdk/vm/ci/runtime/JVMCI", JVMCI_nmethods, 1);
+    register_natives_for_class(env, Services::clazz(), "jdk/vm/ci/services/Services", Services_nmethods, 1);
   }
 }
 
@@ -542,9 +583,9 @@ void JNIJVMCI::register_natives(JNIEnv* env) {
 #define EMPTY2(x,y)
 #define FIELD3(className, name, sig) FIELD2(className, name)
 #define FIELD2(className, name) \
-  jfieldID JNIJVMCI::className::_##name##_field_id = 0; \
+  jfieldID JNIJVMCI::className::_##name##_field_id = nullptr; \
   int HotSpotJVMCI::className::_##name##_offset = 0;
-#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName, args)
+#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName)
 #define CONSTRUCTOR(className, signature)
 
 // Generates the definitions of static fields used by the accessors. For example:
@@ -582,11 +623,11 @@ JVMCI_CLASSES_DO(EMPTY2, EMPTY0, FIELD2, FIELD2, FIELD2, FIELD2, FIELD2, FIELD3,
     return jni()->IsInstanceOf(object.as_jobject(), className::clazz()) != 0;                                                     \
   }                                                                                                                               \
   void JNIJVMCI::className::check(JVMCIEnv* jvmciEnv, JVMCIObject obj, const char* field_name, jfieldID offset) {                 \
-    assert(obj.is_non_null(), "NULL field access of %s.%s", #className, field_name);                                     \
-    assert(jvmciEnv->isa_##className(obj), "wrong class, " #className " expected, found %s", jvmciEnv->klass_name(obj)); \
-    assert(offset != 0, "must be valid offset");                                                                                  \
+    assert(obj.is_non_null(), "null field access of %s.%s", #className, field_name);                                              \
+    assert(jvmciEnv->isa_##className(obj), "wrong class, " #className " expected, found %s", jvmciEnv->klass_name(obj));          \
+    assert(offset != nullptr, "must be valid offset");                                                                            \
   }                                                                                                                               \
-  jclass JNIJVMCI::className::_class = NULL;
+  jclass JNIJVMCI::className::_class = nullptr;
 
 #define END_CLASS
 
@@ -650,7 +691,7 @@ JVMCI_CLASSES_DO(EMPTY2, EMPTY0, FIELD2, FIELD2, FIELD2, FIELD2, FIELD2, FIELD3,
 
 #define STATIC_INT_FIELD(className, name) STATIC_PRIMITIVE_FIELD(className, name, jint, Int, EMPTY_CAST)
 #define STATIC_BOOLEAN_FIELD(className, name) STATIC_PRIMITIVE_FIELD(className, name, jboolean, Boolean, EMPTY_CAST)
-#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName, args) \
+#define METHOD(jniCallType, jniGetMethod, hsCallType, returnType, className, methodName, signatureSymbolName) \
   jmethodID JNIJVMCI::className::_##methodName##_method;
 
 #define CONSTRUCTOR(className, signature) \
