@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,9 +24,9 @@
 #ifndef SHARE_JVMCI_JVMCIRUNTIME_HPP
 #define SHARE_JVMCI_JVMCIRUNTIME_HPP
 
-#include "jvm_io.h"
 #include "code/nmethod.hpp"
 #include "gc/shared/collectedHeap.hpp"
+#include "jvm_io.h"
 #include "jvmci/jvmci.hpp"
 #include "jvmci/jvmciExceptions.hpp"
 #include "jvmci/jvmciObject.hpp"
@@ -35,24 +35,31 @@
 #include "gc/g1/g1CardTable.hpp"
 #endif // INCLUDE_G1GC
 
+#define JVMCI_NOT_ENABLED_ERROR_MESSAGE "JVMCI is not enabled. Must specify '-XX:+EnableJVMCI' or '--add-modules=jdk.internal.vm.ci' to the java launcher."
+
 class JVMCIEnv;
 class JVMCICompiler;
 class JVMCICompileState;
 class MetadataHandles;
 
-// Encapsulates the JVMCI metadata for an nmethod.
-// JVMCINMethodData objects are inlined into nmethods
-// at nmethod::_jvmci_data_offset.
-class JVMCINMethodData {
+// Encapsulates the JVMCI metadata for an nmethod.  JVMCINMethodData objects are normally inlined
+// into nmethods at nmethod::_jvmci_data_offset but during construction of the nmethod they are
+// resource allocated so they can be passed into the nmethod constructor.
+class JVMCINMethodData : public ResourceObj {
   friend class JVMCIVMStructs;
-  // Index for the HotSpotNmethod mirror in the nmethod's oops table.
-  // This is -1 if there is no mirror in the oops table.
-  int _nmethod_mirror_index;
 
   // Is HotSpotNmethod.name non-null? If so, the value is
   // embedded in the end of this object.
   bool _has_name;
 
+  // Index for the HotSpotNmethod mirror in the nmethod's oops table.
+  // This is -1 if there is no mirror in the oops table.
+  int _nmethod_mirror_index;
+
+  // This is the offset of the patchable part of the nmethod entry barrier sequence. The meaning is
+  // somewhat platform dependent as the way patching is done varies by architecture.
+  int _nmethod_entry_patch_offset;
+
   // Address of the failed speculations list to which a speculation
   // is appended when it causes a deoptimization.
   FailedSpeculation** _failed_speculations;
@@ -65,25 +72,52 @@ class JVMCINMethodData {
     SPECULATION_LENGTH_MASK = (1 << SPECULATION_LENGTH_BITS) - 1
   };
 
+  // Allocate a temporary data object for use during installation
+  void initialize(int nmethod_mirror_index,
+                   int nmethod_entry_patch_offset,
+                   const char* nmethod_mirror_name,
+                   FailedSpeculation** failed_speculations);
+
+  void* operator new(size_t size, const char* nmethod_mirror_name) {
+    assert(size == sizeof(JVMCINMethodData), "must agree");
+    size_t total_size = compute_size(nmethod_mirror_name);
+    return (address)resource_allocate_bytes(total_size);
+  }
+
 public:
+  static JVMCINMethodData* create(int nmethod_mirror_index,
+                                  int nmethod_entry_patch_offset,
+                                  const char* nmethod_mirror_name,
+                                  FailedSpeculation** failed_speculations) {
+    JVMCINMethodData* result = new (nmethod_mirror_name) JVMCINMethodData();
+    result->initialize(nmethod_mirror_index,
+                       nmethod_entry_patch_offset,
+                       nmethod_mirror_name,
+                       failed_speculations);
+    return result;
+  }
+
   // Computes the size of a JVMCINMethodData object
   static int compute_size(const char* nmethod_mirror_name) {
     int size = sizeof(JVMCINMethodData);
-    if (nmethod_mirror_name != NULL) {
+    if (nmethod_mirror_name != nullptr) {
       size += (int) strlen(nmethod_mirror_name) + 1;
     }
     return size;
   }
 
-  void initialize(int nmethod_mirror_index,
-             const char* name,
-             FailedSpeculation** failed_speculations);
+  int size() {
+    return compute_size(name());
+  }
+
+  // Copy the contents of this object into data which is normally the storage allocated in the nmethod.
+  void copy(JVMCINMethodData* data);
 
   // Adds `speculation` to the failed speculations list.
   void add_failed_speculation(nmethod* nm, jlong speculation);
 
-  // Gets the JVMCI name of the nmethod (which may be NULL).
-  const char* name() { return _has_name ? (char*)(((address) this) + sizeof(JVMCINMethodData)) : NULL; }
+  // Gets the JVMCI name of the nmethod (which may be null).
+  const char* name() { return _has_name ? (char*)(((address) this) + sizeof(JVMCINMethodData)) : nullptr; }
 
   // Clears the HotSpotNmethod.address field in the  mirror. If nm
   // is dead, the HotSpotNmethod.entryPoint field is also cleared.
@@ -94,6 +128,10 @@ public:
 
   // Sets the mirror in nm's oops table.
   void set_nmethod_mirror(nmethod* nm, oop mirror);
+
+  int nmethod_entry_patch_offset() {
+    return _nmethod_entry_patch_offset;
+  }
 };
 
 // A top level class that represents an initialized JVMCI runtime.
@@ -147,7 +185,7 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
   JavaVM* _shared_library_javavm;
 
   // Id for _shared_library_javavm.
-  int _shared_library_javavm_id;
+  jlong _shared_library_javavm_id;
 
   // Position and link in global list of JVMCI shared library runtimes.
   // The HotSpot heap based runtime will have an id of -1 and the
@@ -188,8 +226,6 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
                                           int klass_index,
                                           bool& is_accessible,
                                           Klass* loading_klass);
-  static void   get_field_by_index_impl(InstanceKlass* loading_klass, fieldDescriptor& fd,
-                                        int field_index);
   static Method*  get_method_by_index_impl(const constantPoolHandle& cpool,
                                            int method_index, Bytecodes::Code bc,
                                            InstanceKlass* loading_klass);
@@ -210,7 +246,7 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
   // If such a runtime exists, its _num_attached_threads is incremented
   // and the caller must subsequently attach `thread` to it.
   // JVMCI_lock must be held by current thread.
-  // If nullptr is returned, then `*count` contains the number of JVMCIRuntimes
+  // If null is returned, then `*count` contains the number of JVMCIRuntimes
   // currently allocated.
   static JVMCIRuntime* select_runtime(JavaThread* thread, JVMCIRuntime* skip, int* count);
 
@@ -235,14 +271,18 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
 
   // Ensures that a JVMCI shared library JavaVM exists for this runtime.
   // If the JavaVM was created by this call, then the thread-local JNI
-  // interface pointer for the JavaVM is returned otherwise nullptr is returned.
-  JNIEnv* init_shared_library_javavm();
+  // interface pointer for the JavaVM is returned otherwise null is returned.
+  // If this method tried to create the JavaVM but failed, the error code returned
+  // by JNI_CreateJavaVM is returned in create_JavaVM_err and, if available, an
+  // error message is malloc'ed and assigned to err_msg. The caller is responsible
+  // for freeing err_msg.
+  JNIEnv* init_shared_library_javavm(int* create_JavaVM_err, const char** err_msg);
 
   // Determines if the JVMCI shared library JavaVM exists for this runtime.
   bool has_shared_library_javavm() { return _shared_library_javavm != nullptr; }
 
   // Gets an ID for the JVMCI shared library JavaVM associated with this runtime.
-  int get_shared_library_javavm_id() { return _shared_library_javavm_id; }
+  jlong get_shared_library_javavm_id() { return _shared_library_javavm_id; }
 
   // Copies info about the JVMCI shared library JavaVM associated with this
   // runtime into `info` as follows:
@@ -272,11 +312,12 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
   // used when creating an IndirectHotSpotObjectConstantImpl in the
   // shared library JavaVM.
   jlong make_oop_handle(const Handle& obj);
+#ifdef ASSERT
+  static bool is_oop_handle(jlong handle);
+#endif
 
   // Releases all the non-null entries in _oop_handles whose referent is null.
   // Returns the number of handles released by this call.
-  // The method also resets _last_found_oop_handle_index to -1
-  // and _null_oop_handles to 0.
   int release_cleared_oop_handles();
 
   // Allocation and management of metadata handles.
@@ -334,11 +375,9 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
   // Explicitly initialize HotSpotJVMCIRuntime itself
   void initialize_HotSpotJVMCIRuntime(JVMCI_TRAPS);
 
-  void call_getCompiler(TRAPS);
-
   // Shuts down this runtime by calling HotSpotJVMCIRuntime.shutdown().
   // If this is the last thread attached to this runtime, then
-  // `_HotSpotJVMCIRuntime_instance` is set to nullptr and `_init_state`
+  // `_HotSpotJVMCIRuntime_instance` is set to null and `_init_state`
   // to uninitialized.
   void shutdown();
 
@@ -349,7 +388,7 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
   void bootstrap_finished(TRAPS);
 
   // Look up a klass by name from a particular class loader (the accessor's).
-  // If require_local, result must be defined in that class loader, or NULL.
+  // If require_local, result must be defined in that class loader, or null.
   // If !require_local, a result from remote class loader may be reported,
   // if sufficient class loader constraints exist such that initiating
   // a class loading request from the given loader is bound to return
@@ -369,8 +408,6 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
                                      int klass_index,
                                      bool& is_accessible,
                                      Klass* loading_klass);
-  static void   get_field_by_index(InstanceKlass* loading_klass, fieldDescriptor& fd,
-                                   int field_index);
   static Method*  get_method_by_index(const constantPoolHandle& cpool,
                                       int method_index, Bytecodes::Code bc,
                                       InstanceKlass* loading_klass);
@@ -383,7 +420,10 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
 
   // Helper routine for determining the validity of a compilation
   // with respect to concurrent class loading.
-  static JVMCI::CodeInstallResult validate_compile_task_dependencies(Dependencies* target, JVMCICompileState* task, char** failure_detail);
+  static JVMCI::CodeInstallResult validate_compile_task_dependencies(Dependencies* target,
+                                                                     JVMCICompileState* task,
+                                                                     char** failure_detail,
+                                                                     bool& failing_dep_is_call_site);
 
   // Compiles `target` with the JVMCI compiler.
   void compile_method(JVMCIEnv* JVMCIENV, JVMCICompiler* compiler, const methodHandle& target, int entry_bci);
@@ -391,6 +431,9 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
   // Determines if the GC identified by `name` is supported by the JVMCI compiler.
   bool is_gc_supported(JVMCIEnv* JVMCIENV, CollectedHeap::Name name);
 
+  // Determines if the intrinsic identified by `id` is supported by the JVMCI compiler.
+  bool is_intrinsic_supported(JVMCIEnv* JVMCIENV, jint id);
+
   // Register the result of a compilation.
   JVMCI::CodeInstallResult register_method(JVMCIEnv* JVMCIENV,
                                            const methodHandle&       target,
@@ -409,12 +452,14 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
                                            int                       compile_id,
                                            bool                      has_monitors,
                                            bool                      has_unsafe_access,
+                                           bool                      has_scoped_access,
                                            bool                      has_wide_vector,
                                            JVMCIObject               compiled_code,
                                            JVMCIObject               nmethod_mirror,
                                            FailedSpeculation**       failed_speculations,
                                            char*                     speculations,
-                                           int                       speculations_len);
+                                           int                       speculations_len,
+                                           int                       nmethod_entry_patch_offset);
 
   // Detach `thread` from this runtime and destroy this runtime's JavaVM
   // if using one JavaVM per JVMCI compilation .
@@ -423,13 +468,13 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
   // Reports an unexpected exception and exits the VM with a fatal error.
   static void fatal_exception(JVMCIEnv* JVMCIENV, const char* message);
 
-  static void describe_pending_hotspot_exception(JavaThread* THREAD, bool clear);
+  static void describe_pending_hotspot_exception(JavaThread* THREAD);
 
 #define CHECK_EXIT THREAD); \
   if (HAS_PENDING_EXCEPTION) { \
     char buf[256]; \
     jio_snprintf(buf, 256, "Uncaught exception at %s:%d", __FILE__, __LINE__); \
-    JVMCIRuntime::fatal_exception(NULL, buf); \
+    JVMCIRuntime::fatal_exception(nullptr, buf); \
     return; \
   } \
   (void)(0
@@ -438,7 +483,7 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
   if (HAS_PENDING_EXCEPTION) { \
     char buf[256]; \
     jio_snprintf(buf, 256, "Uncaught exception at %s:%d", __FILE__, __LINE__); \
-    JVMCIRuntime::fatal_exception(NULL, buf); \
+    JVMCIRuntime::fatal_exception(nullptr, buf); \
     return v; \
   } \
   (void)(0
@@ -463,34 +508,16 @@ class JVMCIRuntime: public CHeapObj<mtJVMCI> {
 
   static BasicType kindToBasicType(const Handle& kind, TRAPS);
 
-  static void new_instance_common(JavaThread* current, Klass* klass, bool null_on_fail);
-  static void new_array_common(JavaThread* current, Klass* klass, jint length, bool null_on_fail);
-  static void new_multi_array_common(JavaThread* current, Klass* klass, int rank, jint* dims, bool null_on_fail);
-  static void dynamic_new_array_common(JavaThread* current, oopDesc* element_mirror, jint length, bool null_on_fail);
-  static void dynamic_new_instance_common(JavaThread* current, oopDesc* type_mirror, bool null_on_fail);
-
   // The following routines are called from compiled JVMCI code
 
-  // When allocation fails, these stubs:
-  // 1. Exercise -XX:+HeapDumpOnOutOfMemoryError and -XX:OnOutOfMemoryError handling and also
-  //    post a JVMTI_EVENT_RESOURCE_EXHAUSTED event if the failure is an OutOfMemroyError
-  // 2. Return NULL with a pending exception.
-  // Compiled code must ensure these stubs are not called twice for the same allocation
-  // site due to the non-repeatable side effects in the case of OOME.
-  static void new_instance(JavaThread* current, Klass* klass) { new_instance_common(current, klass, false); }
-  static void new_array(JavaThread* current, Klass* klass, jint length) { new_array_common(current, klass, length, false); }
-  static void new_multi_array(JavaThread* current, Klass* klass, int rank, jint* dims) { new_multi_array_common(current, klass, rank, dims, false); }
-  static void dynamic_new_array(JavaThread* current, oopDesc* element_mirror, jint length) { dynamic_new_array_common(current, element_mirror, length, false); }
-  static void dynamic_new_instance(JavaThread* current, oopDesc* type_mirror) { dynamic_new_instance_common(current, type_mirror, false); }
-
-  // When allocation fails, these stubs return NULL and have no pending exception. Compiled code
-  // can use these stubs if a failed allocation will be retried (e.g., by deoptimizing and
-  // re-executing in the interpreter).
-  static void new_instance_or_null(JavaThread* thread, Klass* klass) { new_instance_common(thread, klass, true); }
-  static void new_array_or_null(JavaThread* thread, Klass* klass, jint length) { new_array_common(thread, klass, length, true); }
-  static void new_multi_array_or_null(JavaThread* thread, Klass* klass, int rank, jint* dims) { new_multi_array_common(thread, klass, rank, dims, true); }
-  static void dynamic_new_array_or_null(JavaThread* thread, oopDesc* element_mirror, jint length) { dynamic_new_array_common(thread, element_mirror, length, true); }
-  static void dynamic_new_instance_or_null(JavaThread* thread, oopDesc* type_mirror) { dynamic_new_instance_common(thread, type_mirror, true); }
+  // When allocation fails, these stubs return null and have no pending OutOfMemoryError exception.
+  // Compiled code can use these stubs if a failed allocation will be retried (e.g., by deoptimizing
+  // and re-executing in the interpreter).
+  static void new_instance_or_null(JavaThread* thread, Klass* klass);
+  static void new_array_or_null(JavaThread* thread, Klass* klass, jint length);
+  static void new_multi_array_or_null(JavaThread* thread, Klass* klass, int rank, jint* dims);
+  static void dynamic_new_array_or_null(JavaThread* thread, oopDesc* element_mirror, jint length);
+  static void dynamic_new_instance_or_null(JavaThread* thread, oopDesc* type_mirror);
 
   static void vm_message(jboolean vmError, jlong format, jlong v1, jlong v2, jlong v3);
   static jint identity_hash_code(JavaThread* current, oopDesc* obj);
