@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -44,6 +44,9 @@ class Chunk {
  private:
   Chunk*       _next;     // Next Chunk in list
   const size_t _len;      // Size of this Chunk
+  // Used for Compilation Memory Statistic
+  uint64_t _stamp;
+
 public:
   NONCOPYABLE(Chunk);
 
@@ -67,7 +70,7 @@ public:
     init_size  =  1*K  - slack, // Size of first chunk (normal aka small)
     medium_size= 10*K  - slack, // Size of medium-sized chunk
     size       = 32*K  - slack, // Default size of an Arena chunk (following the first)
-    non_pool_size = init_size + 32 // An initial size which is not one of above
+    max_default_size = size     // Largest default size
   };
 
   static void chop(Chunk* chunk);                  // Chop this chunk
@@ -82,24 +85,58 @@ public:
   char* bottom() const          { return ((char*) this) + aligned_overhead_size();  }
   char* top()    const          { return bottom() + _len; }
   bool contains(char* p) const  { return bottom() <= p && p <= top(); }
+
+  void set_stamp(uint64_t v) { _stamp = v; }
+  uint64_t stamp() const     { return _stamp; }
 };
 
+// Arena types (for Compilation Memory Statistic)
+#define DO_ARENA_TAG(FN) \
+  FN(ra,          Resource areas) \
+  FN(node,        C2 Node arena) \
+  FN(comp,        C2 Compile arena) \
+  FN(type,        C2 Type arena) \
+  FN(states,      C2 Matcher States Arena) \
+  FN(reglive,     C2 Register Allocation Live Arenas) \
+  FN(regsplit,    C2 Register Allocation Split Arena) \
+  FN(superword,   C2 SuperWord Arenas) \
+  FN(cienv,       CI Env Arena) \
+  FN(ha,          Handle area) \
+  FN(other,       Other arenas) \
+
 // Fast allocation of memory
 class Arena : public CHeapObjBase {
+public:
+  enum class Tag: uint8_t {
+#define ARENA_TAG_ENUM(name, desc) tag_##name,
+    DO_ARENA_TAG(ARENA_TAG_ENUM)
+#undef ARENA_TAG_ENUM
+    tag_count
+  };
+
+  constexpr static int tag_count() {
+    return static_cast<int>(Tag::tag_count);
+  }
+
+  static const char* tag_name[static_cast<int>(Arena::Tag::tag_count)];
+  static const char* tag_desc[static_cast<int>(Arena::Tag::tag_count)];
+
+private:
+  const MemTag _mem_tag;        // Native Memory Tracking tag
+  const Tag _tag;
+  size_t _size_in_bytes;        // Size of arena (used for native memory tracking)
+
 protected:
   friend class HandleMark;
   friend class NoHandleMark;
-  friend class VMStructs;
-
-  MEMFLAGS    _flags;           // Memory tracking flags
 
   Chunk* _first;                // First chunk
   Chunk* _chunk;                // current chunk
   char* _hwm;                   // High water mark
   char* _max;                   // and max in current chunk
+
   // Get a new Chunk of at least size x
   void* grow(size_t x, AllocFailType alloc_failmode = AllocFailStrategy::EXIT_OOM);
-  size_t _size_in_bytes;        // Size of arena (used for native memory tracking)
 
   void* internal_amalloc(size_t x, AllocFailType alloc_failmode = AllocFailStrategy::EXIT_OOM)  {
     assert(is_aligned(x, BytesPerWord), "misaligned size");
@@ -115,9 +152,7 @@ protected:
  public:
   // Start the chunk_pool cleaner task
   static void start_chunk_pool_cleaner_task();
-
-  Arena(MEMFLAGS memflag);
-  Arena(MEMFLAGS memflag, size_t init_size);
+  Arena(MemTag mem_tag, Tag tag = Tag::tag_other, size_t init_size = Chunk::init_size);
   ~Arena();
   void  destruct_contents();
   char* hwm() const             { return _hwm; }
@@ -171,6 +206,9 @@ protected:
   size_t size_in_bytes() const         {  return _size_in_bytes; };
   void set_size_in_bytes(size_t size);
 
+  MemTag get_mem_tag() const { return _mem_tag; }
+  Tag get_tag() const { return _tag; }
+
 private:
   // Reset this Arena to empty, access will trigger grow if necessary
   void reset(void) {
