@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,10 +33,10 @@
 #include "memory/heapInspection.hpp"
 #include "memory/resourceArea.hpp"
 #include "memory/universe.hpp"
+#include "nmt/memTracker.hpp"
 #include "oops/oop.inline.hpp"
 #include "runtime/atomic.hpp"
 #include "runtime/os.hpp"
-#include "services/memTracker.hpp"
 #include "utilities/globalDefinitions.hpp"
 #include "utilities/macros.hpp"
 #include "utilities/stack.inline.hpp"
@@ -44,14 +44,14 @@
 // HeapInspection
 
 inline KlassInfoEntry::~KlassInfoEntry() {
-  if (_subclasses != NULL) {
+  if (_subclasses != nullptr) {
     delete _subclasses;
   }
 }
 
 inline void KlassInfoEntry::add_subclass(KlassInfoEntry* cie) {
-  if (_subclasses == NULL) {
-    _subclasses = new  (ResourceObj::C_HEAP, mtServiceability) GrowableArray<KlassInfoEntry*>(4, mtServiceability);
+  if (_subclasses == nullptr) {
+    _subclasses = new (mtServiceability) GrowableArray<KlassInfoEntry*>(4, mtServiceability);
   }
   _subclasses->append(cie);
 }
@@ -80,17 +80,17 @@ int KlassInfoEntry::compare(KlassInfoEntry* e1, KlassInfoEntry* e2) {
 
 const char* KlassInfoEntry::name() const {
   const char* name;
-  if (_klass->name() != NULL) {
+  if (_klass->name() != nullptr) {
     name = _klass->external_name();
   } else {
-    if (_klass == Universe::boolArrayKlassObj())         name = "<boolArrayKlass>";         else
-    if (_klass == Universe::charArrayKlassObj())         name = "<charArrayKlass>";         else
-    if (_klass == Universe::floatArrayKlassObj())        name = "<floatArrayKlass>";        else
-    if (_klass == Universe::doubleArrayKlassObj())       name = "<doubleArrayKlass>";       else
-    if (_klass == Universe::byteArrayKlassObj())         name = "<byteArrayKlass>";         else
-    if (_klass == Universe::shortArrayKlassObj())        name = "<shortArrayKlass>";        else
-    if (_klass == Universe::intArrayKlassObj())          name = "<intArrayKlass>";          else
-    if (_klass == Universe::longArrayKlassObj())         name = "<longArrayKlass>";         else
+    if (_klass == Universe::boolArrayKlass())         name = "<boolArrayKlass>";         else
+    if (_klass == Universe::charArrayKlass())         name = "<charArrayKlass>";         else
+    if (_klass == Universe::floatArrayKlass())        name = "<floatArrayKlass>";        else
+    if (_klass == Universe::doubleArrayKlass())       name = "<doubleArrayKlass>";       else
+    if (_klass == Universe::byteArrayKlass())         name = "<byteArrayKlass>";         else
+    if (_klass == Universe::shortArrayKlass())        name = "<shortArrayKlass>";        else
+    if (_klass == Universe::intArrayKlass())          name = "<intArrayKlass>";          else
+    if (_klass == Universe::longArrayKlass())         name = "<longArrayKlass>";         else
       name = "<no name>";
   }
   return name;
@@ -107,8 +107,8 @@ void KlassInfoEntry::print_on(outputStream* st) const {
                  (uint64_t)_instance_words * HeapWordSize,
                  name(),
                  module->name()->as_C_string(),
-                 module->version() != NULL ? "@" : "",
-                 module->version() != NULL ? module->version()->as_C_string() : "");
+                 module->version() != nullptr ? "@" : "",
+                 module->version() != nullptr ? module->version()->as_C_string() : "");
   } else {
     st->print_cr(INT64_FORMAT_W(13) "  " UINT64_FORMAT_W(13) "  %s",
                  (int64_t)_instance_count,
@@ -119,12 +119,12 @@ void KlassInfoEntry::print_on(outputStream* st) const {
 
 KlassInfoEntry* KlassInfoBucket::lookup(Klass* const k) {
   // Can happen if k is an archived class that we haven't loaded yet.
-  if (k->java_mirror_no_keepalive() == NULL) {
-    return NULL;
+  if (k->java_mirror_no_keepalive() == nullptr) {
+    return nullptr;
   }
 
   KlassInfoEntry* elt = _list;
-  while (elt != NULL) {
+  while (elt != nullptr) {
     if (elt->is_equal(k)) {
       return elt;
     }
@@ -132,7 +132,7 @@ KlassInfoEntry* KlassInfoBucket::lookup(Klass* const k) {
   }
   elt = new (std::nothrow) KlassInfoEntry(k, list());
   // We may be out of space to allocate the new entry.
-  if (elt != NULL) {
+  if (elt != nullptr) {
     set_list(elt);
   }
   return elt;
@@ -140,7 +140,7 @@ KlassInfoEntry* KlassInfoBucket::lookup(Klass* const k) {
 
 void KlassInfoBucket::iterate(KlassInfoClosure* cic) {
   KlassInfoEntry* elt = _list;
-  while (elt != NULL) {
+  while (elt != nullptr) {
     cic->do_cinfo(elt);
     elt = elt->next();
   }
@@ -148,8 +148,8 @@ void KlassInfoBucket::iterate(KlassInfoClosure* cic) {
 
 void KlassInfoBucket::empty() {
   KlassInfoEntry* elt = _list;
-  _list = NULL;
-  while (elt != NULL) {
+  _list = nullptr;
+  while (elt != nullptr) {
     KlassInfoEntry* next = elt->next();
     delete elt;
     elt = next;
@@ -170,11 +170,11 @@ public:
 
 KlassInfoTable::KlassInfoTable(bool add_all_classes) {
   _size_of_instances_in_words = 0;
-  _ref = (HeapWord*) Universe::boolArrayKlassObj();
+  _ref = (uintptr_t) Universe::boolArrayKlass();
   _buckets =
     (KlassInfoBucket*)  AllocateHeap(sizeof(KlassInfoBucket) * _num_buckets,
        mtInternal, CURRENT_PC, AllocFailStrategy::RETURN_NULL);
-  if (_buckets != NULL) {
+  if (_buckets != nullptr) {
     for (int index = 0; index < _num_buckets; index++) {
       _buckets[index].initialize();
     }
@@ -186,27 +186,27 @@ KlassInfoTable::KlassInfoTable(bool add_all_classes) {
 }
 
 KlassInfoTable::~KlassInfoTable() {
-  if (_buckets != NULL) {
+  if (_buckets != nullptr) {
     for (int index = 0; index < _num_buckets; index++) {
       _buckets[index].empty();
     }
     FREE_C_HEAP_ARRAY(KlassInfoBucket, _buckets);
-    _buckets = NULL;
+    _buckets = nullptr;
   }
 }
 
 uint KlassInfoTable::hash(const Klass* p) {
-  return (uint)(((uintptr_t)p - (uintptr_t)_ref) >> 2);
+  return (uint)(((uintptr_t)p - _ref) >> 2);
 }
 
 KlassInfoEntry* KlassInfoTable::lookup(Klass* k) {
   uint         idx = hash(k) % _num_buckets;
-  assert(_buckets != NULL, "Allocation failure should have been caught");
+  assert(_buckets != nullptr, "Allocation failure should have been caught");
   KlassInfoEntry*  e   = _buckets[idx].lookup(k);
   // Lookup may fail if this is a new klass for which we
   // could not allocate space for an new entry, or if it's
   // an archived class that we haven't loaded yet.
-  assert(e == NULL || k == e->klass(), "must be equal");
+  assert(e == nullptr || k == e->klass(), "must be equal");
   return e;
 }
 
@@ -215,9 +215,9 @@ KlassInfoEntry* KlassInfoTable::lookup(Klass* k) {
 bool KlassInfoTable::record_instance(const oop obj) {
   Klass*        k = obj->klass();
   KlassInfoEntry* elt = lookup(k);
-  // elt may be NULL if it's a new klass for which we
+  // elt may be null if it's a new klass for which we
   // could not allocate space for a new entry in the hashtable.
-  if (elt != NULL) {
+  if (elt != nullptr) {
     elt->set_count(elt->count() + 1);
     elt->set_words(elt->words() + obj->size());
     _size_of_instances_in_words += obj->size();
@@ -228,7 +228,7 @@ bool KlassInfoTable::record_instance(const oop obj) {
 }
 
 void KlassInfoTable::iterate(KlassInfoClosure* cic) {
-  assert(_buckets != NULL, "Allocation failure should have been caught");
+  assert(_buckets != nullptr, "Allocation failure should have been caught");
   for (int index = 0; index < _num_buckets; index++) {
     _buckets[index].iterate(cic);
   }
@@ -243,9 +243,9 @@ size_t KlassInfoTable::size_of_instances_in_words() const {
 bool KlassInfoTable::merge_entry(const KlassInfoEntry* cie) {
   Klass*          k = cie->klass();
   KlassInfoEntry* elt = lookup(k);
-  // elt may be NULL if it's a new klass for which we
+  // elt may be null if it's a new klass for which we
   // could not allocate space for a new entry in the hashtable.
-  if (elt != NULL) {
+  if (elt != nullptr) {
     elt->set_count(elt->count() + cie->count());
     elt->set_words(elt->words() + cie->words());
     _size_of_instances_in_words += cie->words();
@@ -279,7 +279,7 @@ int KlassInfoHisto::sort_helper(KlassInfoEntry** e1, KlassInfoEntry** e2) {
 
 KlassInfoHisto::KlassInfoHisto(KlassInfoTable* cit) :
   _cit(cit) {
-  _elements = new (ResourceObj::C_HEAP, mtServiceability) GrowableArray<KlassInfoEntry*>(_histo_initial_size, mtServiceability);
+  _elements = new (mtServiceability) GrowableArray<KlassInfoEntry*>(_histo_initial_size, mtServiceability);
 }
 
 KlassInfoHisto::~KlassInfoHisto() {
@@ -349,9 +349,9 @@ void KlassHierarchy::print_class_hierarchy(outputStream* st, bool print_interfac
     cie->set_index(i + 1);
 
     // Add the class to the subclass array of its superclass.
-    if (super != NULL) {
+    if (super != nullptr) {
       KlassInfoEntry* super_cie = cit.lookup(super);
-      assert(super_cie != NULL, "could not lookup superclass");
+      assert(super_cie != nullptr, "could not lookup superclass");
       super_cie->add_subclass(cie);
     }
   }
@@ -359,7 +359,7 @@ void KlassHierarchy::print_class_hierarchy(outputStream* st, bool print_interfac
   // Set the do_print flag for each class that should be printed.
   for(int i = 0; i < elements.length(); i++) {
     KlassInfoEntry* cie = elements.at(i);
-    if (classname == NULL) {
+    if (classname == nullptr) {
       // We are printing all classes.
       cie->set_do_print(true);
     } else {
@@ -374,7 +374,7 @@ void KlassHierarchy::print_class_hierarchy(outputStream* st, bool print_interfac
   // maintain the list of classes we still need to process. Start things off
   // by priming it with java.lang.Object.
   KlassInfoEntry* jlo_cie = cit.lookup(vmClasses::Object_klass());
-  assert(jlo_cie != NULL, "could not lookup java.lang.Object");
+  assert(jlo_cie != nullptr, "could not lookup java.lang.Object");
   class_stack.push(jlo_cie);
 
   // Repeatedly pop the top item off the stack, print its class info,
@@ -384,7 +384,7 @@ void KlassHierarchy::print_class_hierarchy(outputStream* st, bool print_interfac
     KlassInfoEntry* curr_cie = class_stack.pop();
     if (curr_cie->do_print()) {
       print_class(st, curr_cie, print_interfaces);
-      if (curr_cie->subclasses() != NULL) {
+      if (curr_cie->subclasses() != nullptr) {
         // Current class has subclasses, so push all of them onto the stack.
         for (int i = 0; i < curr_cie->subclasses()->length(); i++) {
           KlassInfoEntry* cie = curr_cie->subclasses()->at(i);
@@ -404,7 +404,7 @@ void KlassHierarchy::set_do_print_for_class_hierarchy(KlassInfoEntry* cie, Klass
                                                       bool print_subclasses) {
   // Set do_print for all superclasses of this class.
   Klass* super = ((InstanceKlass*)cie->klass())->java_super();
-  while (super != NULL) {
+  while (super != nullptr) {
     KlassInfoEntry* super_cie = cit->lookup(super);
     super_cie->set_do_print(true);
     super = super->super();
@@ -416,7 +416,7 @@ void KlassHierarchy::set_do_print_for_class_hierarchy(KlassInfoEntry* cie, Klass
   while (!class_stack.is_empty()) {
     KlassInfoEntry* curr_cie = class_stack.pop();
     curr_cie->set_do_print(true);
-    if (print_subclasses && curr_cie->subclasses() != NULL) {
+    if (print_subclasses && curr_cie->subclasses() != nullptr) {
       // Current class has subclasses, so push all of them onto the stack.
       for (int i = 0; i < curr_cie->subclasses()->length(); i++) {
         KlassInfoEntry* cie = curr_cie->subclasses()->at(i);
@@ -440,7 +440,7 @@ static void print_indent(outputStream* st, int indent) {
 static void print_classname(outputStream* st, Klass* klass) {
   oop loader_oop = klass->class_loader_data()->class_loader();
   st->print("%s/", klass->external_name());
-  if (loader_oop == NULL) {
+  if (loader_oop == nullptr) {
     st->print("null");
   } else {
     st->print(PTR_FORMAT, p2i(klass->class_loader_data()));
@@ -461,7 +461,7 @@ void KlassHierarchy::print_class(outputStream* st, KlassInfoEntry* cie, bool pri
 
   // Print indentation with proper indicators of superclass.
   Klass* super = klass->super();
-  while (super != NULL) {
+  while (super != nullptr) {
     super = super->super();
     indent++;
   }
@@ -530,7 +530,7 @@ class RecordInstanceClosure : public ObjectClosure {
 
  private:
   bool should_visit(oop obj) {
-    return _filter == NULL || _filter->do_object_b(obj);
+    return _filter == nullptr || _filter->do_object_b(obj);
   }
 };
 
@@ -564,27 +564,16 @@ void ParHeapInspectTask::work(uint worker_id) {
   }
 }
 
-uintx HeapInspection::populate_table(KlassInfoTable* cit, BoolObjectClosure *filter, uint parallel_thread_num) {
-
+uintx HeapInspection::populate_table(KlassInfoTable* cit, BoolObjectClosure *filter, WorkerThreads* workers) {
   // Try parallel first.
-  if (parallel_thread_num > 1) {
+  if (workers != nullptr) {
     ResourceMark rm;
-
-    WorkerThreads* workers = Universe::heap()->safepoint_workers();
-    if (workers != NULL) {
-      // The GC provided a WorkerThreads to be used during a safepoint.
-
-      // Can't run with more threads than provided by the WorkerThreads.
-      const uint capped_parallel_thread_num = MIN2(parallel_thread_num, workers->max_workers());
-      WithActiveWorkers with_active_workers(workers, capped_parallel_thread_num);
-
-      ParallelObjectIterator poi(workers->active_workers());
-      ParHeapInspectTask task(&poi, cit, filter);
-      // Run task with the active workers.
-      workers->run_task(&task);
-      if (task.success()) {
-        return task.missed_count();
-      }
+    ParallelObjectIterator poi(workers->active_workers());
+    ParHeapInspectTask task(&poi, cit, filter);
+    // Run task with the active workers.
+    workers->run_task(&task);
+    if (task.success()) {
+      return task.missed_count();
     }
   }
 
@@ -595,13 +584,13 @@ uintx HeapInspection::populate_table(KlassInfoTable* cit, BoolObjectClosure *fil
   return ric.missed_count();
 }
 
-void HeapInspection::heap_inspection(outputStream* st, uint parallel_thread_num) {
+void HeapInspection::heap_inspection(outputStream* st, WorkerThreads* workers) {
   ResourceMark rm;
 
   KlassInfoTable cit(false);
   if (!cit.allocation_failed()) {
     // populate table with object allocation info
-    uintx missed_count = populate_table(&cit, NULL, parallel_thread_num);
+    uintx missed_count = populate_table(&cit, nullptr, workers);
     if (missed_count != 0) {
       log_info(gc, classhisto)("WARNING: Ran out of C-heap; undercounted " UINTX_FORMAT
                                " total instances in data below",
