@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -36,6 +36,7 @@ macro(AddF)
 macro(AddI)
 macro(AddL)
 macro(AddP)
+macro(AddHF)
 macro(Allocate)
 macro(AllocateArray)
 macro(AndI)
@@ -64,6 +65,7 @@ macro(CallLeafVector)
 macro(CallRuntime)
 macro(CallStaticJava)
 macro(CastDD)
+macro(CastHH)
 macro(CastFF)
 macro(CastII)
 macro(CastLL)
@@ -132,6 +134,7 @@ macro(Con)
 macro(ConN)
 macro(ConNKlass)
 macro(ConD)
+macro(ConH)
 macro(ConF)
 macro(ConI)
 macro(ConL)
@@ -166,6 +169,7 @@ macro(CountTrailingZerosV)
 macro(CreateEx)
 macro(DecodeN)
 macro(DecodeNKlass)
+macro(DivHF)
 macro(DivD)
 macro(DivF)
 macro(DivI)
@@ -184,6 +188,7 @@ macro(FastLock)
 macro(FastUnlock)
 macro(FmaD)
 macro(FmaF)
+macro(FmaHF)
 macro(ForwardException)
 macro(Goto)
 macro(Halt)
@@ -222,6 +227,7 @@ macro(MachProj)
 macro(MulAddS2I)
 macro(MaxI)
 macro(MaxL)
+macro(MaxHF)
 macro(MaxD)
 macro(MaxF)
 macro(MemBarAcquire)
@@ -237,6 +243,7 @@ macro(MemBarStoreStore)
 macro(MergeMem)
 macro(MinI)
 macro(MinL)
+macro(MinHF)
 macro(MinF)
 macro(MinD)
 macro(ModD)
@@ -253,6 +260,7 @@ macro(IsInfiniteF)
 macro(IsFiniteF)
 macro(IsInfiniteD)
 macro(IsFiniteD)
+macro(MulHF)
 macro(MulD)
 macro(MulF)
 macro(MulHiL)
@@ -270,8 +278,9 @@ macro(Opaque1)
 macro(OpaqueLoopInit)
 macro(OpaqueLoopStride)
 macro(OpaqueZeroTripGuard)
-macro(Opaque4)
+macro(OpaqueNotNull)
 macro(OpaqueInitializedAssertionPredicate)
+macro(OpaqueTemplateAssertionPredicate)
 macro(ProfileBoolean)
 macro(OrI)
 macro(OrL)
@@ -329,19 +338,21 @@ shmacro(ShenandoahLoadReferenceBarrier)
 macro(SCMemProj)
 macro(CopySignD)
 macro(CopySignF)
+macro(SaturatingAddV)
+macro(SaturatingSubV)
 macro(SignumD)
 macro(SignumF)
 macro(SignumVF)
 macro(SignumVD)
 macro(SqrtD)
 macro(SqrtF)
+macro(SqrtHF)
 macro(RoundF)
 macro(RoundD)
 macro(Start)
 macro(StartOSR)
 macro(StoreB)
 macro(StoreC)
-macro(StoreCM)
 macro(StoreD)
 macro(StoreF)
 macro(StoreI)
@@ -355,6 +366,7 @@ macro(StrEquals)
 macro(StrIndexOf)
 macro(StrIndexOfChar)
 macro(StrInflatedCopy)
+macro(SubHF)
 macro(SubD)
 macro(SubF)
 macro(SubI)
@@ -436,6 +448,8 @@ macro(XorV)
 macro(XorReductionV)
 macro(MinV)
 macro(MaxV)
+macro(UMinV)
+macro(UMaxV)
 macro(MinReductionV)
 macro(MaxReductionV)
 macro(CompressV)
@@ -481,8 +495,11 @@ macro(ExtractF)
 macro(ExtractD)
 macro(Digit)
 macro(LowerCase)
+macro(ReinterpretS2HF)
+macro(ReinterpretHF2S)
 macro(UpperCase)
 macro(Whitespace)
+macro(SelectFromTwoVector)
 macro(VectorBox)
 macro(VectorBoxAllocate)
 macro(VectorUnbox)
