@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -46,6 +46,7 @@
 #include "runtime/vmThread.hpp"
 #include "utilities/ticks.hpp"
 #include "utilities/vmEnums.hpp"
+#include "opto/printinlining.hpp"
 
 class AbstractLockNode;
 class AddPNode;
@@ -70,6 +71,7 @@ class Node_List;
 class Node_Notes;
 class NodeHash;
 class NodeCloneInfo;
+class OpaqueTemplateAssertionPredicateNode;
 class OptoReg;
 class ParsePredicateNode;
 class PhaseCFG;
@@ -174,7 +176,6 @@ class CloneMap {
 
 class Options {
   friend class Compile;
-  friend class VMStructs;
  private:
   const bool _subsume_loads;         // Load can be matched as part of a larger op.
   const bool _do_escape_analysis;    // Do escape analysis.
@@ -221,7 +222,6 @@ class Options {
 // This class defines a top-level Compiler invocation.
 
 class Compile : public Phase {
-  friend class VMStructs;
 
  public:
   // Fixed alias indexes.  (See also MergeMemNode.)
@@ -237,13 +237,14 @@ class Compile : public Phase {
   // (The time collection itself is always conditionalized on CITime.)
   class TracePhase : public TraceTime {
    private:
-    Compile*    _compile;
+    Compile* const _compile;
     CompileLog* _log;
-    const char* _phase_name;
-    bool _dolog;
+    const bool _dolog;
    public:
-    TracePhase(const char* name, elapsedTimer* accumulator);
+    TracePhase(PhaseTraceId phaseTraceId);
+    TracePhase(const char* name, PhaseTraceId phaseTraceId);
     ~TracePhase();
+    const char* phase_name() const { return title(); }
   };
 
   // Information per category of alias (memory slice)
@@ -318,6 +319,7 @@ class Compile : public Phase {
   uintx                 _max_node_limit;        // Max unique node count during a single compilation.
 
   bool                  _post_loop_opts_phase;  // Loop opts are finished.
+  bool                  _merge_stores_phase;    // Phase for merging stores, after post loop opts phase.
   bool                  _allow_macro_nodes;     // True if we allow creation of macro nodes.
 
   int                   _major_progress;        // Count of something big happening
@@ -370,9 +372,12 @@ class Compile : public Phase {
   GrowableArray<CallGenerator*> _intrinsics;    // List of intrinsics.
   GrowableArray<Node*>  _macro_nodes;           // List of nodes which need to be expanded before matching.
   GrowableArray<ParsePredicateNode*> _parse_predicates; // List of Parse Predicates.
-  GrowableArray<Node*>  _template_assertion_predicate_opaqs; // List of Opaque4 nodes for Template Assertion Predicates.
+  // List of OpaqueTemplateAssertionPredicateNode nodes for Template Assertion Predicates which can be seen as list
+  // of Template Assertion Predicates themselves.
+  GrowableArray<OpaqueTemplateAssertionPredicateNode*>  _template_assertion_predicate_opaques;
   GrowableArray<Node*>  _expensive_nodes;       // List of nodes that are expensive to compute and that we'd better not let the GVN freely common
   GrowableArray<Node*>  _for_post_loop_igvn;    // List of nodes for IGVN after loop opts are over
+  GrowableArray<Node*>  _for_merge_stores_igvn; // List of nodes for IGVN merge stores
   GrowableArray<UnstableIfTrap*> _unstable_if_traps;        // List of ifnodes after IGVN
   GrowableArray<Node_List*> _coarsened_locks;   // List of coarsened Lock and Unlock nodes
   ConnectionGraph*      _congraph;
@@ -391,6 +396,8 @@ class Compile : public Phase {
   DEBUG_ONLY(Unique_Node_List* _modified_nodes;)   // List of nodes which inputs were modified
   DEBUG_ONLY(bool       _phase_optimize_finished;) // Used for live node verification while creating new nodes
 
+  DEBUG_ONLY(bool       _phase_verify_ideal_loop;) // Are we in PhaseIdealLoop verification?
+
   // Arenas for new-space and old-space nodes.
   // Swapped between using _node_arena.
   // The lifetime of the old-space nodes is during xform.
@@ -468,29 +475,6 @@ private:
   // "MemLimit" directive was specified and the memory limit was hit during compilation
   bool                          _oom;
 
-  // Inlining may not happen in parse order which would make
-  // PrintInlining output confusing. Keep track of PrintInlining
-  // pieces in order.
-  class PrintInliningBuffer : public CHeapObj<mtCompiler> {
-   private:
-    CallGenerator* _cg;
-    stringStream   _ss;
-    static const size_t default_stream_buffer_size = 128;
-
-   public:
-    PrintInliningBuffer()
-      : _cg(nullptr), _ss(default_stream_buffer_size) {}
-
-    stringStream* ss()             { return &_ss; }
-    CallGenerator* cg()            { return _cg; }
-    void set_cg(CallGenerator* cg) { _cg = cg; }
-  };
-
-  stringStream* _print_inlining_stream;
-  GrowableArray<PrintInliningBuffer*>* _print_inlining_list;
-  int _print_inlining_idx;
-  char* _print_inlining_output;
-
   // Only keep nodes in the expensive node list that need to be optimized
   void cleanup_expensive_nodes(PhaseIterGVN &igvn);
   // Use for sorting expensive nodes to bring similar nodes together
@@ -502,37 +486,17 @@ private:
 
   void* _replay_inline_data; // Pointer to data loaded from file
 
-  void print_inlining_init();
-  void print_inlining_reinit();
-  void print_inlining_commit();
-  void print_inlining_push();
-  PrintInliningBuffer* print_inlining_current();
-
   void log_late_inline_failure(CallGenerator* cg, const char* msg);
   DEBUG_ONLY(bool _exception_backedge;)
 
   void record_method_not_compilable_oom();
 
- public:
+  InlinePrinter _inline_printer;
 
+public:
   void* barrier_set_state() const { return _barrier_set_state; }
 
-  stringStream* print_inlining_stream() {
-    assert(print_inlining() || print_intrinsics(), "PrintInlining off?");
-    return _print_inlining_stream;
-  }
-
-  void print_inlining_update(CallGenerator* cg);
-  void print_inlining_update_delayed(CallGenerator* cg);
-  void print_inlining_move_to(CallGenerator* cg);
-  void print_inlining_assert_ready();
-  void print_inlining_reset();
-
-  void print_inlining(ciMethod* method, int inline_level, int bci, InliningResult result, const char* msg = nullptr) {
-    stringStream ss;
-    CompileTask::print_inlining_inner(&ss, method, inline_level, bci, result, msg);
-    print_inlining_stream()->print("%s", ss.freeze());
-  }
+  InlinePrinter* inline_printer() { return &_inline_printer; }
 
 #ifndef PRODUCT
   IdealGraphPrinter* igv_printer() { return _igv_printer; }
@@ -707,30 +671,35 @@ private:
   void print_method(CompilerPhaseType cpt, int level, Node* n = nullptr);
 
 #ifndef PRODUCT
+  void init_igv();
   void dump_igv(const char* graph_name, int level = 3) {
     if (should_print_igv(level)) {
-      _igv_printer->print_method(graph_name, level);
+      _igv_printer->print_graph(graph_name);
     }
   }
 
   void igv_print_method_to_file(const char* phase_name = "Debug", bool append = false);
   void igv_print_method_to_network(const char* phase_name = "Debug");
+  void igv_print_graph_to_network(const char* name, Node* node, GrowableArray<const Node*>& visible_nodes);
   static IdealGraphPrinter* debug_file_printer() { return _debug_file_printer; }
   static IdealGraphPrinter* debug_network_printer() { return _debug_network_printer; }
 #endif
 
+  const GrowableArray<ParsePredicateNode*>& parse_predicates() const {
+    return _parse_predicates;
+  }
+
+  const GrowableArray<OpaqueTemplateAssertionPredicateNode*>& template_assertion_predicate_opaques() const {
+    return _template_assertion_predicate_opaques;
+  }
+
   int           macro_count()             const { return _macro_nodes.length(); }
   int           parse_predicate_count()   const { return _parse_predicates.length(); }
-  int           template_assertion_predicate_count() const { return _template_assertion_predicate_opaqs.length(); }
+  int           template_assertion_predicate_count() const { return _template_assertion_predicate_opaques.length(); }
   int           expensive_count()         const { return _expensive_nodes.length(); }
   int           coarsened_count()         const { return _coarsened_locks.length(); }
 
   Node*         macro_node(int idx)       const { return _macro_nodes.at(idx); }
-  ParsePredicateNode* parse_predicate(int idx) const { return _parse_predicates.at(idx); }
-
-  Node* template_assertion_predicate_opaq_node(int idx) const {
-    return _template_assertion_predicate_opaqs.at(idx);
-  }
 
   Node*         expensive_node(int idx)   const { return _expensive_nodes.at(idx); }
 
@@ -766,15 +735,15 @@ private:
     }
   }
 
-  void add_template_assertion_predicate_opaq(Node* n) {
-    assert(!_template_assertion_predicate_opaqs.contains(n),
-           "duplicate entry in template assertion predicate opaque4 list");
-    _template_assertion_predicate_opaqs.append(n);
+  void add_template_assertion_predicate_opaque(OpaqueTemplateAssertionPredicateNode* n) {
+    assert(!_template_assertion_predicate_opaques.contains(n),
+           "Duplicate entry in Template Assertion Predicate OpaqueTemplateAssertionPredicate list");
+    _template_assertion_predicate_opaques.append(n);
   }
 
-  void remove_template_assertion_predicate_opaq(Node* n) {
+  void remove_template_assertion_predicate_opaque(OpaqueTemplateAssertionPredicateNode* n) {
     if (template_assertion_predicate_count() > 0) {
-      _template_assertion_predicate_opaqs.remove_if_existing(n);
+      _template_assertion_predicate_opaques.remove_if_existing(n);
     }
   }
   void add_coarsened_locks(GrowableArray<AbstractLockNode*>& locks);
@@ -786,6 +755,12 @@ private:
   void   set_post_loop_opts_phase() { _post_loop_opts_phase = true;  }
   void reset_post_loop_opts_phase() { _post_loop_opts_phase = false; }
 
+#ifdef ASSERT
+  bool       phase_verify_ideal_loop() const { return _phase_verify_ideal_loop; }
+  void   set_phase_verify_ideal_loop() { _phase_verify_ideal_loop = true; }
+  void reset_phase_verify_ideal_loop() { _phase_verify_ideal_loop = false; }
+#endif
+
   bool       allow_macro_nodes() { return _allow_macro_nodes;  }
   void reset_allow_macro_nodes() { _allow_macro_nodes = false;  }
 
@@ -798,6 +773,12 @@ private:
   void remove_useless_unstable_if_traps(Unique_Node_List &useful);
   void process_for_unstable_if_traps(PhaseIterGVN& igvn);
 
+  bool     merge_stores_phase() { return _merge_stores_phase;  }
+  void set_merge_stores_phase() { _merge_stores_phase = true;  }
+  void record_for_merge_stores_igvn(Node* n);
+  void remove_from_merge_stores_igvn(Node* n);
+  void process_for_merge_stores_igvn(PhaseIterGVN& igvn);
+
   void shuffle_macro_nodes();
   void sort_macro_nodes();
 
@@ -815,7 +796,7 @@ private:
   ciEnv*      env() const            { return _env; }
   CompileLog* log() const            { return _log; }
 
-  bool        failing() const        {
+  bool        failing_internal() const {
     return _env->failing() ||
            _failure_reason.get() != nullptr;
   }
@@ -827,6 +808,27 @@ private:
 
   const CompilationFailureInfo* first_failure_details() const { return _first_failure_details; }
 
+  bool failing() {
+    if (failing_internal()) {
+      return true;
+    }
+#ifdef ASSERT
+    // Disable stress code for PhaseIdealLoop verification (would have cascading effects).
+    if (phase_verify_ideal_loop()) {
+      return false;
+    }
+    if (StressBailout) {
+      return fail_randomly();
+    }
+#endif
+    return false;
+  }
+
+#ifdef ASSERT
+  bool fail_randomly();
+  bool failure_is_artificial();
+#endif
+
   bool failure_reason_is(const char* r) const {
     return (r == _failure_reason.get()) ||
            (r != nullptr &&
@@ -834,11 +836,11 @@ private:
             strcmp(r, _failure_reason.get()) == 0);
   }
 
-  void record_failure(const char* reason);
-  void record_method_not_compilable(const char* reason) {
+  void record_failure(const char* reason DEBUG_ONLY(COMMA bool allow_multiple_failures = false));
+  void record_method_not_compilable(const char* reason DEBUG_ONLY(COMMA bool allow_multiple_failures = false)) {
     env()->record_method_not_compilable(reason);
     // Record failure reason.
-    record_failure(reason);
+    record_failure(reason DEBUG_ONLY(COMMA allow_multiple_failures));
   }
   bool check_node_count(uint margin, const char* reason) {
     if (oom()) {
@@ -913,11 +915,9 @@ private:
   Node_Notes* default_node_notes() const        { return _default_node_notes; }
   void    set_default_node_notes(Node_Notes* n) { _default_node_notes = n; }
 
-  Node_Notes*       node_notes_at(int idx) {
-    return locate_node_notes(_node_note_array, idx, false);
-  }
-  inline bool   set_node_notes_at(int idx, Node_Notes* value);
+  Node_Notes*       node_notes_at(int idx);
 
+  inline bool   set_node_notes_at(int idx, Node_Notes* value);
   // Copy notes from source to dest, if they exist.
   // Overwrite dest only if source provides something.
   // Return true if information was moved.
@@ -1069,7 +1069,6 @@ private:
 
   void remove_useless_coarsened_locks(Unique_Node_List& useful);
 
-  void process_print_inlining();
   void dump_print_inlining();
 
   bool over_inlining_cutoff() const {
@@ -1215,7 +1214,6 @@ private:
   void final_graph_reshaping_impl(Node *n, Final_Reshape_Counts& frc, Unique_Node_List& dead_nodes);
   void final_graph_reshaping_main_switch(Node* n, Final_Reshape_Counts& frc, uint nop, Unique_Node_List& dead_nodes);
   void final_graph_reshaping_walk(Node_Stack& nstack, Node* root, Final_Reshape_Counts& frc, Unique_Node_List& dead_nodes);
-  void eliminate_redundant_card_marks(Node* n);
   void handle_div_mod_op(Node* n, BasicType bt, bool is_unsigned);
 
   // Logic cone optimization.
