@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "memory/allocation.inline.hpp"
 #include "opto/addnode.hpp"
 #include "opto/connode.hpp"
@@ -35,6 +34,7 @@
 #include "opto/phaseX.hpp"
 #include "opto/subnode.hpp"
 #include "utilities/powerOfTwo.hpp"
+#include "opto/runtime.hpp"
 
 // Portions of code courtesy of Clifford Click
 
@@ -42,6 +42,23 @@
 
 #include <math.h>
 
+ModFloatingNode::ModFloatingNode(Compile* C, const TypeFunc* tf, const char* name) : CallLeafNode(tf, nullptr, name, TypeRawPtr::BOTTOM) {
+  add_flag(Flag_is_macro);
+  C->add_macro_node(this);
+}
+
+ModDNode::ModDNode(Compile* C, Node* a, Node* b) : ModFloatingNode(C, OptoRuntime::Math_DD_D_Type(), "drem") {
+  init_req(TypeFunc::Parms + 0, a);
+  init_req(TypeFunc::Parms + 1, C->top());
+  init_req(TypeFunc::Parms + 2, b);
+  init_req(TypeFunc::Parms + 3, C->top());
+}
+
+ModFNode::ModFNode(Compile* C, Node* a, Node* b) : ModFloatingNode(C, OptoRuntime::modf_Type(), "frem") {
+  init_req(TypeFunc::Parms + 0, a);
+  init_req(TypeFunc::Parms + 1, b);
+}
+
 //----------------------magic_int_divide_constants-----------------------------
 // Compute magic multiplier and shift constant for converting a 32 bit divide
 // by constant into a multiply/shift/add series. Return false if calculations
@@ -89,7 +106,7 @@ static bool magic_int_divide_constants(jint d, jint &M, jint &s) {
 
 //--------------------------transform_int_divide-------------------------------
 // Convert a division by constant divisor into an alternate Ideal graph.
-// Return NULL if no transformation occurs.
+// Return null if no transformation occurs.
 static Node *transform_int_divide( PhaseGVN *phase, Node *dividend, jint divisor ) {
 
   // Check for invalid divisors
@@ -101,7 +118,7 @@ static Node *transform_int_divide( PhaseGVN *phase, Node *dividend, jint divisor
   const int N = 32;
 
   // Result
-  Node *q = NULL;
+  Node *q = nullptr;
 
   if (d == 1) {
     // division by +/- 1
@@ -266,7 +283,6 @@ static Node* long_by_long_mulhi(PhaseGVN* phase, Node* dividend, jlong magic_con
   }
 
   // Taken from Hacker's Delight, Fig. 8-2. Multiply high signed.
-  // (http://www.hackersdelight.org/HDcode/mulhs.c)
   //
   // int mulhs(int u, int v) {
   //    unsigned u0, v0, w0;
@@ -334,7 +350,7 @@ static Node* long_by_long_mulhi(PhaseGVN* phase, Node* dividend, jlong magic_con
 
 //--------------------------transform_long_divide------------------------------
 // Convert a division by constant divisor into an alternate Ideal graph.
-// Return NULL if no transformation occurs.
+// Return null if no transformation occurs.
 static Node *transform_long_divide( PhaseGVN *phase, Node *dividend, jlong divisor ) {
   // Check for invalid divisors
   assert( divisor != 0L && divisor != min_jlong,
@@ -345,7 +361,7 @@ static Node *transform_long_divide( PhaseGVN *phase, Node *dividend, jlong divis
   const int N = 64;
 
   // Result
-  Node *q = NULL;
+  Node *q = nullptr;
 
   if (d == 1) {
     // division by +/- 1
@@ -448,6 +464,47 @@ static Node *transform_long_divide( PhaseGVN *phase, Node *dividend, jlong divis
   return q;
 }
 
+template <typename TypeClass, typename Unsigned>
+Node* unsigned_div_ideal(PhaseGVN* phase, bool can_reshape, Node* div) {
+  // Check for dead control input
+  if (div->in(0) != nullptr && div->remove_dead_region(phase, can_reshape)) {
+    return div;
+  }
+  // Don't bother trying to transform a dead node
+  if (div->in(0) != nullptr && div->in(0)->is_top()) {
+    return nullptr;
+  }
+
+  const Type* t = phase->type(div->in(2));
+  if (t == Type::TOP) {
+    return nullptr;
+  }
+  const TypeClass* type_divisor = t->cast<TypeClass>();
+
+  // Check for useless control input
+  // Check for excluding div-zero case
+  if (div->in(0) != nullptr && (type_divisor->_hi < 0 || type_divisor->_lo > 0)) {
+    div->set_req(0, nullptr); // Yank control input
+    return div;
+  }
+
+  if (!type_divisor->is_con()) {
+    return nullptr;
+  }
+  Unsigned divisor = static_cast<Unsigned>(type_divisor->get_con()); // Get divisor
+
+  if (divisor == 0 || divisor == 1) {
+    return nullptr; // Dividing by zero constant does not idealize
+  }
+
+  if (is_power_of_2(divisor)) {
+    return make_urshift<TypeClass>(div->in(1), phase->intcon(log2i_graceful(divisor)));
+  }
+
+  return nullptr;
+}
+
+
 //=============================================================================
 //------------------------------Identity---------------------------------------
 // If the divisor is 1, we are an identity on the dividend.
@@ -460,29 +517,29 @@ Node* DivINode::Identity(PhaseGVN* phase) {
 Node *DivINode::Ideal(PhaseGVN *phase, bool can_reshape) {
   if (in(0) && remove_dead_region(phase, can_reshape))  return this;
   // Don't bother trying to transform a dead node
-  if( in(0) && in(0)->is_top() )  return NULL;
+  if( in(0) && in(0)->is_top() )  return nullptr;
 
   const Type *t = phase->type( in(2) );
-  if( t == TypeInt::ONE )       // Identity?
-    return NULL;                // Skip it
+  if( t == TypeInt::ONE )      // Identity?
+    return nullptr;            // Skip it
 
   const TypeInt *ti = t->isa_int();
-  if( !ti ) return NULL;
+  if( !ti ) return nullptr;
 
   // Check for useless control input
   // Check for excluding div-zero case
   if (in(0) && (ti->_hi < 0 || ti->_lo > 0)) {
-    set_req(0, NULL);           // Yank control input
+    set_req(0, nullptr);           // Yank control input
     return this;
   }
 
-  if( !ti->is_con() ) return NULL;
+  if( !ti->is_con() ) return nullptr;
   jint i = ti->get_con();       // Get divisor
 
-  if (i == 0) return NULL;      // Dividing by zero constant does not idealize
+  if (i == 0) return nullptr;   // Dividing by zero constant does not idealize
 
   // Dividing by MININT does not optimize as a power-of-2 shift.
-  if( i == min_jint ) return NULL;
+  if( i == min_jint ) return nullptr;
 
   return transform_int_divide( phase, in(1), i );
 }
@@ -566,29 +623,29 @@ Node* DivLNode::Identity(PhaseGVN* phase) {
 Node *DivLNode::Ideal( PhaseGVN *phase, bool can_reshape) {
   if (in(0) && remove_dead_region(phase, can_reshape))  return this;
   // Don't bother trying to transform a dead node
-  if( in(0) && in(0)->is_top() )  return NULL;
+  if( in(0) && in(0)->is_top() )  return nullptr;
 
   const Type *t = phase->type( in(2) );
   if( t == TypeLong::ONE )      // Identity?
-    return NULL;                // Skip it
+    return nullptr;             // Skip it
 
   const TypeLong *tl = t->isa_long();
-  if( !tl ) return NULL;
+  if( !tl ) return nullptr;
 
   // Check for useless control input
   // Check for excluding div-zero case
   if (in(0) && (tl->_hi < 0 || tl->_lo > 0)) {
-    set_req(0, NULL);           // Yank control input
+    set_req(0, nullptr);         // Yank control input
     return this;
   }
 
-  if( !tl->is_con() ) return NULL;
+  if( !tl->is_con() ) return nullptr;
   jlong l = tl->get_con();      // Get divisor
 
-  if (l == 0) return NULL;      // Dividing by zero constant does not idealize
+  if (l == 0) return nullptr;   // Dividing by zero constant does not idealize
 
   // Dividing by MINLONG does not optimize as a power-of-2 shift.
-  if( l == min_jlong ) return NULL;
+  if( l == min_jlong ) return nullptr;
 
   return transform_long_divide( phase, in(1), l );
 }
@@ -717,28 +774,28 @@ Node* DivFNode::Identity(PhaseGVN* phase) {
 Node *DivFNode::Ideal(PhaseGVN *phase, bool can_reshape) {
   if (in(0) && remove_dead_region(phase, can_reshape))  return this;
   // Don't bother trying to transform a dead node
-  if( in(0) && in(0)->is_top() )  return NULL;
+  if( in(0) && in(0)->is_top() )  return nullptr;
 
   const Type *t2 = phase->type( in(2) );
   if( t2 == TypeF::ONE )         // Identity?
-    return NULL;                // Skip it
+    return nullptr;              // Skip it
 
   const TypeF *tf = t2->isa_float_constant();
-  if( !tf ) return NULL;
-  if( tf->base() != Type::FloatCon ) return NULL;
+  if( !tf ) return nullptr;
+  if( tf->base() != Type::FloatCon ) return nullptr;
 
   // Check for out of range values
-  if( tf->is_nan() || !tf->is_finite() ) return NULL;
+  if( tf->is_nan() || !tf->is_finite() ) return nullptr;
 
   // Get the value
   float f = tf->getf();
   int exp;
 
   // Only for special case of dividing by a power of 2
-  if( frexp((double)f, &exp) != 0.5 ) return NULL;
+  if( frexp((double)f, &exp) != 0.5 ) return nullptr;
 
   // Limit the range of acceptable exponents
-  if( exp < -126 || exp > 126 ) return NULL;
+  if( exp < -126 || exp > 126 ) return nullptr;
 
   // Compute the reciprocal
   float reciprocal = ((float)1.0) / f;
@@ -809,28 +866,28 @@ Node* DivDNode::Identity(PhaseGVN* phase) {
 Node *DivDNode::Ideal(PhaseGVN *phase, bool can_reshape) {
   if (in(0) && remove_dead_region(phase, can_reshape))  return this;
   // Don't bother trying to transform a dead node
-  if( in(0) && in(0)->is_top() )  return NULL;
+  if( in(0) && in(0)->is_top() )  return nullptr;
 
   const Type *t2 = phase->type( in(2) );
   if( t2 == TypeD::ONE )         // Identity?
-    return NULL;                // Skip it
+    return nullptr;              // Skip it
 
   const TypeD *td = t2->isa_double_constant();
-  if( !td ) return NULL;
-  if( td->base() != Type::DoubleCon ) return NULL;
+  if( !td ) return nullptr;
+  if( td->base() != Type::DoubleCon ) return nullptr;
 
   // Check for out of range values
-  if( td->is_nan() || !td->is_finite() ) return NULL;
+  if( td->is_nan() || !td->is_finite() ) return nullptr;
 
   // Get the value
   double d = td->getd();
   int exp;
 
   // Only for special case of dividing by a power of 2
-  if( frexp(d, &exp) != 0.5 ) return NULL;
+  if( frexp(d, &exp) != 0.5 ) return nullptr;
 
   // Limit the range of acceptable exponents
-  if( exp < -1021 || exp > 1022 ) return NULL;
+  if( exp < -1021 || exp > 1022 ) return nullptr;
 
   // Compute the reciprocal
   double reciprocal = 1.0 / d;
@@ -841,28 +898,100 @@ Node *DivDNode::Ideal(PhaseGVN *phase, bool can_reshape) {
   return (new MulDNode(in(1), phase->makecon(TypeD::make(reciprocal))));
 }
 
+//=============================================================================
+//------------------------------Identity---------------------------------------
+// If the divisor is 1, we are an identity on the dividend.
+Node* UDivINode::Identity(PhaseGVN* phase) {
+  return (phase->type( in(2) )->higher_equal(TypeInt::ONE)) ? in(1) : this;
+}
+//------------------------------Value------------------------------------------
+// A UDivINode divides its inputs.  The third input is a Control input, used to
+// prevent hoisting the divide above an unsafe test.
+const Type* UDivINode::Value(PhaseGVN* phase) const {
+  // Either input is TOP ==> the result is TOP
+  const Type *t1 = phase->type( in(1) );
+  const Type *t2 = phase->type( in(2) );
+  if( t1 == Type::TOP ) return Type::TOP;
+  if( t2 == Type::TOP ) return Type::TOP;
+
+  // x/x == 1 since we always generate the dynamic divisor check for 0.
+  if (in(1) == in(2)) {
+    return TypeInt::ONE;
+  }
+
+  // Either input is BOTTOM ==> the result is the local BOTTOM
+  const Type *bot = bottom_type();
+  if( (t1 == bot) || (t2 == bot) ||
+      (t1 == Type::BOTTOM) || (t2 == Type::BOTTOM) )
+    return bot;
+
+  // Otherwise we give up all hope
+  return TypeInt::INT;
+}
+
+//------------------------------Idealize---------------------------------------
+Node *UDivINode::Ideal(PhaseGVN *phase, bool can_reshape) {
+  return unsigned_div_ideal<TypeInt, juint>(phase, can_reshape, this);
+}
+
+//=============================================================================
+//------------------------------Identity---------------------------------------
+// If the divisor is 1, we are an identity on the dividend.
+Node* UDivLNode::Identity(PhaseGVN* phase) {
+  return (phase->type( in(2) )->higher_equal(TypeLong::ONE)) ? in(1) : this;
+}
+//------------------------------Value------------------------------------------
+// A UDivLNode divides its inputs.  The third input is a Control input, used to
+// prevent hoisting the divide above an unsafe test.
+const Type* UDivLNode::Value(PhaseGVN* phase) const {
+  // Either input is TOP ==> the result is TOP
+  const Type *t1 = phase->type( in(1) );
+  const Type *t2 = phase->type( in(2) );
+  if( t1 == Type::TOP ) return Type::TOP;
+  if( t2 == Type::TOP ) return Type::TOP;
+
+  // x/x == 1 since we always generate the dynamic divisor check for 0.
+  if (in(1) == in(2)) {
+    return TypeLong::ONE;
+  }
+
+  // Either input is BOTTOM ==> the result is the local BOTTOM
+  const Type *bot = bottom_type();
+  if( (t1 == bot) || (t2 == bot) ||
+      (t1 == Type::BOTTOM) || (t2 == Type::BOTTOM) )
+    return bot;
+
+  // Otherwise we give up all hope
+  return TypeLong::LONG;
+}
+
+//------------------------------Idealize---------------------------------------
+Node *UDivLNode::Ideal(PhaseGVN *phase, bool can_reshape) {
+  return unsigned_div_ideal<TypeLong, julong>(phase, can_reshape, this);
+}
+
 //=============================================================================
 //------------------------------Idealize---------------------------------------
 Node *ModINode::Ideal(PhaseGVN *phase, bool can_reshape) {
   // Check for dead control input
   if( in(0) && remove_dead_region(phase, can_reshape) )  return this;
   // Don't bother trying to transform a dead node
-  if( in(0) && in(0)->is_top() )  return NULL;
+  if( in(0) && in(0)->is_top() )  return nullptr;
 
   // Get the modulus
   const Type *t = phase->type( in(2) );
-  if( t == Type::TOP ) return NULL;
+  if( t == Type::TOP ) return nullptr;
   const TypeInt *ti = t->is_int();
 
   // Check for useless control input
   // Check for excluding mod-zero case
   if (in(0) && (ti->_hi < 0 || ti->_lo > 0)) {
-    set_req(0, NULL);        // Yank control input
+    set_req(0, nullptr);        // Yank control input
     return this;
   }
 
   // See if we are MOD'ing by 2^k or 2^k-1.
-  if( !ti->is_con() ) return NULL;
+  if( !ti->is_con() ) return nullptr;
   jint con = ti->get_con();
 
   Node *hook = new Node(1);
@@ -915,7 +1044,7 @@ Node *ModINode::Ideal(PhaseGVN *phase, bool can_reshape) {
   // into a long multiply/int multiply/subtract case
 
   // Cannot handle mod 0, and min_jint isn't handled by the transform
-  if( con == 0 || con == min_jint ) return NULL;
+  if( con == 0 || con == min_jint ) return nullptr;
 
   // Get the absolute value of the constant; at this point, we can use this
   jint pos_con = (con >= 0) ? con : -con;
@@ -942,11 +1071,11 @@ Node *ModINode::Ideal(PhaseGVN *phase, bool can_reshape) {
 
   // Divide using the transform from DivI to MulL
   Node *result = transform_int_divide( phase, in(1), pos_con );
-  if (result != NULL) {
+  if (result != nullptr) {
     Node *divide = phase->transform(result);
 
     // Re-multiply, using a shift if this is a power of two
-    Node *mult = NULL;
+    Node *mult = nullptr;
 
     if( log2_con >= 0 )
       mult = phase->transform( new LShiftINode( divide, phase->intcon( log2_con ) ) );
@@ -1005,6 +1134,99 @@ const Type* ModINode::Value(PhaseGVN* phase) const {
   return TypeInt::make( i1->get_con() % i2->get_con() );
 }
 
+//=============================================================================
+//------------------------------Idealize---------------------------------------
+
+template <typename TypeClass, typename Unsigned>
+static Node* unsigned_mod_ideal(PhaseGVN* phase, bool can_reshape, Node* mod) {
+  // Check for dead control input
+  if (mod->in(0) != nullptr && mod->remove_dead_region(phase, can_reshape)) {
+    return mod;
+  }
+  // Don't bother trying to transform a dead node
+  if (mod->in(0) != nullptr && mod->in(0)->is_top()) {
+    return nullptr;
+  }
+
+  // Get the modulus
+  const Type* t = phase->type(mod->in(2));
+  if (t == Type::TOP) {
+    return nullptr;
+  }
+  const TypeClass* type_divisor = t->cast<TypeClass>();
+
+  // Check for useless control input
+  // Check for excluding mod-zero case
+  if (mod->in(0) != nullptr && (type_divisor->_hi < 0 || type_divisor->_lo > 0)) {
+    mod->set_req(0, nullptr); // Yank control input
+    return mod;
+  }
+
+  if (!type_divisor->is_con()) {
+    return nullptr;
+  }
+  Unsigned divisor = static_cast<Unsigned>(type_divisor->get_con());
+
+  if (divisor == 0) {
+    return nullptr;
+  }
+
+  if (is_power_of_2(divisor)) {
+    return make_and<TypeClass>(mod->in(1), phase->makecon(TypeClass::make(divisor - 1)));
+  }
+
+  return nullptr;
+}
+
+template <typename TypeClass, typename Unsigned, typename Signed>
+static const Type* unsigned_mod_value(PhaseGVN* phase, const Node* mod) {
+  const Type* t1 = phase->type(mod->in(1));
+  const Type* t2 = phase->type(mod->in(2));
+  if (t1 == Type::TOP) {
+    return Type::TOP;
+  }
+  if (t2 == Type::TOP) {
+    return Type::TOP;
+  }
+
+  // 0 MOD X is 0
+  if (t1 == TypeClass::ZERO) {
+    return TypeClass::ZERO;
+  }
+  // X MOD X is 0
+  if (mod->in(1) == mod->in(2)) {
+    return TypeClass::ZERO;
+  }
+
+  // Either input is BOTTOM ==> the result is the local BOTTOM
+  const Type* bot = mod->bottom_type();
+  if ((t1 == bot) || (t2 == bot) ||
+      (t1 == Type::BOTTOM) || (t2 == Type::BOTTOM)) {
+    return bot;
+  }
+
+  const TypeClass* type_divisor = t2->cast<TypeClass>();
+  if (type_divisor->is_con() && type_divisor->get_con() == 1) {
+    return TypeClass::ZERO;
+  }
+
+  const TypeClass* type_dividend = t1->cast<TypeClass>();
+  if (type_dividend->is_con() && type_divisor->is_con()) {
+    Unsigned dividend = static_cast<Unsigned>(type_dividend->get_con());
+    Unsigned divisor = static_cast<Unsigned>(type_divisor->get_con());
+    return TypeClass::make(static_cast<Signed>(dividend % divisor));
+  }
+
+  return bot;
+}
+
+Node* UModINode::Ideal(PhaseGVN* phase, bool can_reshape) {
+  return unsigned_mod_ideal<TypeInt, juint>(phase, can_reshape, this);
+}
+
+const Type* UModINode::Value(PhaseGVN* phase) const {
+  return unsigned_mod_value<TypeInt, juint, jint>(phase, this);
+}
 
 //=============================================================================
 //------------------------------Idealize---------------------------------------
@@ -1012,22 +1234,22 @@ Node *ModLNode::Ideal(PhaseGVN *phase, bool can_reshape) {
   // Check for dead control input
   if( in(0) && remove_dead_region(phase, can_reshape) )  return this;
   // Don't bother trying to transform a dead node
-  if( in(0) && in(0)->is_top() )  return NULL;
+  if( in(0) && in(0)->is_top() )  return nullptr;
 
   // Get the modulus
   const Type *t = phase->type( in(2) );
-  if( t == Type::TOP ) return NULL;
+  if( t == Type::TOP ) return nullptr;
   const TypeLong *tl = t->is_long();
 
   // Check for useless control input
   // Check for excluding mod-zero case
   if (in(0) && (tl->_hi < 0 || tl->_lo > 0)) {
-    set_req(0, NULL);        // Yank control input
+    set_req(0, nullptr);        // Yank control input
     return this;
   }
 
   // See if we are MOD'ing by 2^k or 2^k-1.
-  if( !tl->is_con() ) return NULL;
+  if( !tl->is_con() ) return nullptr;
   jlong con = tl->get_con();
 
   Node *hook = new Node(1);
@@ -1082,7 +1304,7 @@ Node *ModLNode::Ideal(PhaseGVN *phase, bool can_reshape) {
   // into a long multiply/int multiply/subtract case
 
   // Cannot handle mod 0, and min_jlong isn't handled by the transform
-  if( con == 0 || con == min_jlong ) return NULL;
+  if( con == 0 || con == min_jlong ) return nullptr;
 
   // Get the absolute value of the constant; at this point, we can use this
   jlong pos_con = (con >= 0) ? con : -con;
@@ -1109,11 +1331,11 @@ Node *ModLNode::Ideal(PhaseGVN *phase, bool can_reshape) {
 
   // Divide using the transform from DivL to MulL
   Node *result = transform_long_divide( phase, in(1), pos_con );
-  if (result != NULL) {
+  if (result != nullptr) {
     Node *divide = phase->transform(result);
 
     // Re-multiply, using a shift if this is a power of two
-    Node *mult = NULL;
+    Node *mult = nullptr;
 
     if( log2_con >= 0 )
       mult = phase->transform( new LShiftLNode( divide, phase->intcon( log2_con ) ) );
@@ -1172,39 +1394,48 @@ const Type* ModLNode::Value(PhaseGVN* phase) const {
   return TypeLong::make( i1->get_con() % i2->get_con() );
 }
 
+Node *UModLNode::Ideal(PhaseGVN *phase, bool can_reshape) {
+  return unsigned_mod_ideal<TypeLong, julong>(phase, can_reshape, this);
+}
+
+const Type* UModLNode::Value(PhaseGVN* phase) const {
+  return unsigned_mod_value<TypeLong, julong, jlong>(phase, this);
+}
+
+Node* ModFNode::Ideal(PhaseGVN* phase, bool can_reshape) {
+  if (!can_reshape) {
+    return nullptr;
+  }
 
-//=============================================================================
-//------------------------------Value------------------------------------------
-const Type* ModFNode::Value(PhaseGVN* phase) const {
   // Either input is TOP ==> the result is TOP
-  const Type *t1 = phase->type( in(1) );
-  const Type *t2 = phase->type( in(2) );
-  if( t1 == Type::TOP ) return Type::TOP;
-  if( t2 == Type::TOP ) return Type::TOP;
-
-  // Either input is BOTTOM ==> the result is the local BOTTOM
-  const Type *bot = bottom_type();
-  if( (t1 == bot) || (t2 == bot) ||
-      (t1 == Type::BOTTOM) || (t2 == Type::BOTTOM) )
-    return bot;
+  const Type* t1 = phase->type(dividend());
+  const Type* t2 = phase->type(divisor());
+  if (t1 == Type::TOP || t2 == Type::TOP) {
+    return phase->C->top();
+  }
 
   // If either number is not a constant, we know nothing.
   if ((t1->base() != Type::FloatCon) || (t2->base() != Type::FloatCon)) {
-    return Type::FLOAT;         // note: x%x can be either NaN or 0
+    return nullptr; // note: x%x can be either NaN or 0
   }
 
   float f1 = t1->getf();
   float f2 = t2->getf();
-  jint  x1 = jint_cast(f1);     // note:  *(int*)&f1, not just (int)f1
-  jint  x2 = jint_cast(f2);
+  jint x1 = jint_cast(f1); // note:  *(int*)&f1, not just (int)f1
+  jint x2 = jint_cast(f2);
 
   // If either is a NaN, return an input NaN
-  if (g_isnan(f1))    return t1;
-  if (g_isnan(f2))    return t2;
+  if (g_isnan(f1)) {
+    return replace_with_con(phase, t1);
+  }
+  if (g_isnan(f2)) {
+    return replace_with_con(phase, t2);
+  }
 
   // If an operand is infinity or the divisor is +/- zero, punt.
-  if (!g_isfinite(f1) || !g_isfinite(f2) || x2 == 0 || x2 == min_jint)
-    return Type::FLOAT;
+  if (!g_isfinite(f1) || !g_isfinite(f2) || x2 == 0 || x2 == min_jint) {
+    return nullptr;
+  }
 
   // We must be modulo'ing 2 float constants.
   // Make sure that the sign of the fmod is equal to the sign of the dividend
@@ -1213,42 +1444,43 @@ const Type* ModFNode::Value(PhaseGVN* phase) const {
     xr ^= min_jint;
   }
 
-  return TypeF::make(jfloat_cast(xr));
+  return replace_with_con(phase, TypeF::make(jfloat_cast(xr)));
 }
 
+Node* ModDNode::Ideal(PhaseGVN* phase, bool can_reshape) {
+  if (!can_reshape) {
+    return nullptr;
+  }
 
-//=============================================================================
-//------------------------------Value------------------------------------------
-const Type* ModDNode::Value(PhaseGVN* phase) const {
   // Either input is TOP ==> the result is TOP
-  const Type *t1 = phase->type( in(1) );
-  const Type *t2 = phase->type( in(2) );
-  if( t1 == Type::TOP ) return Type::TOP;
-  if( t2 == Type::TOP ) return Type::TOP;
-
-  // Either input is BOTTOM ==> the result is the local BOTTOM
-  const Type *bot = bottom_type();
-  if( (t1 == bot) || (t2 == bot) ||
-      (t1 == Type::BOTTOM) || (t2 == Type::BOTTOM) )
-    return bot;
+  const Type* t1 = phase->type(dividend());
+  const Type* t2 = phase->type(divisor());
+  if (t1 == Type::TOP || t2 == Type::TOP) {
+    return nullptr;
+  }
 
   // If either number is not a constant, we know nothing.
   if ((t1->base() != Type::DoubleCon) || (t2->base() != Type::DoubleCon)) {
-    return Type::DOUBLE;        // note: x%x can be either NaN or 0
+    return nullptr; // note: x%x can be either NaN or 0
   }
 
   double f1 = t1->getd();
   double f2 = t2->getd();
-  jlong  x1 = jlong_cast(f1);   // note:  *(long*)&f1, not just (long)f1
-  jlong  x2 = jlong_cast(f2);
+  jlong x1 = jlong_cast(f1); // note:  *(long*)&f1, not just (long)f1
+  jlong x2 = jlong_cast(f2);
 
   // If either is a NaN, return an input NaN
-  if (g_isnan(f1))    return t1;
-  if (g_isnan(f2))    return t2;
+  if (g_isnan(f1)) {
+    return replace_with_con(phase, t1);
+  }
+  if (g_isnan(f2)) {
+    return replace_with_con(phase, t2);
+  }
 
   // If an operand is infinity or the divisor is +/- zero, punt.
-  if (!g_isfinite(f1) || !g_isfinite(f2) || x2 == 0 || x2 == min_jlong)
-    return Type::DOUBLE;
+  if (!g_isfinite(f1) || !g_isfinite(f2) || x2 == 0 || x2 == min_jlong) {
+    return nullptr;
+  }
 
   // We must be modulo'ing 2 double constants.
   // Make sure that the sign of the fmod is equal to the sign of the dividend
@@ -1257,7 +1489,36 @@ const Type* ModDNode::Value(PhaseGVN* phase) const {
     xr ^= min_jlong;
   }
 
-  return TypeD::make(jdouble_cast(xr));
+  return replace_with_con(phase, TypeD::make(jdouble_cast(xr)));
+}
+
+Node* ModFloatingNode::replace_with_con(PhaseGVN* phase, const Type* con) {
+  Compile* C = phase->C;
+  Node* con_node = phase->makecon(con);
+  CallProjections projs;
+  extract_projections(&projs, false, false);
+  C->gvn_replace_by(projs.fallthrough_proj, in(TypeFunc::Control));
+  if (projs.fallthrough_catchproj != nullptr) {
+    C->gvn_replace_by(projs.fallthrough_catchproj, in(TypeFunc::Control));
+  }
+  if (projs.fallthrough_memproj != nullptr) {
+    C->gvn_replace_by(projs.fallthrough_memproj, in(TypeFunc::Memory));
+  }
+  if (projs.catchall_memproj != nullptr) {
+    C->gvn_replace_by(projs.catchall_memproj, C->top());
+  }
+  if (projs.fallthrough_ioproj != nullptr) {
+    C->gvn_replace_by(projs.fallthrough_ioproj, in(TypeFunc::I_O));
+  }
+  assert(projs.catchall_ioproj == nullptr, "no exceptions from floating mod");
+  assert(projs.catchall_catchproj == nullptr, "no exceptions from floating mod");
+  if (projs.resproj != nullptr) {
+    C->gvn_replace_by(projs.resproj, con_node);
+  }
+  C->gvn_replace_by(this, C->top());
+  C->remove_macro_node(this);
+  disconnect_inputs(C);
+  return nullptr;
 }
 
 //=============================================================================
@@ -1268,6 +1529,24 @@ DivModNode::DivModNode( Node *c, Node *dividend, Node *divisor ) : MultiNode(3)
   init_req(2, divisor);
 }
 
+DivModNode* DivModNode::make(Node* div_or_mod, BasicType bt, bool is_unsigned) {
+  assert(bt == T_INT || bt == T_LONG, "only int or long input pattern accepted");
+
+  if (bt == T_INT) {
+    if (is_unsigned) {
+      return UDivModINode::make(div_or_mod);
+    } else {
+      return DivModINode::make(div_or_mod);
+    }
+  } else {
+    if (is_unsigned) {
+      return UDivModLNode::make(div_or_mod);
+    } else {
+      return DivModLNode::make(div_or_mod);
+    }
+  }
+}
+
 //------------------------------make------------------------------------------
 DivModINode* DivModINode::make(Node* div_or_mod) {
   Node* n = div_or_mod;
@@ -1320,3 +1599,56 @@ Node *DivModLNode::match( const ProjNode *proj, const Matcher *match ) {
   }
   return new MachProjNode(this, proj->_con, rm, ideal_reg);
 }
+
+//------------------------------make------------------------------------------
+UDivModINode* UDivModINode::make(Node* div_or_mod) {
+  Node* n = div_or_mod;
+  assert(n->Opcode() == Op_UDivI || n->Opcode() == Op_UModI,
+         "only div or mod input pattern accepted");
+
+  UDivModINode* divmod = new UDivModINode(n->in(0), n->in(1), n->in(2));
+  Node*        dproj  = new ProjNode(divmod, DivModNode::div_proj_num);
+  Node*        mproj  = new ProjNode(divmod, DivModNode::mod_proj_num);
+  return divmod;
+}
+
+//------------------------------make------------------------------------------
+UDivModLNode* UDivModLNode::make(Node* div_or_mod) {
+  Node* n = div_or_mod;
+  assert(n->Opcode() == Op_UDivL || n->Opcode() == Op_UModL,
+         "only div or mod input pattern accepted");
+
+  UDivModLNode* divmod = new UDivModLNode(n->in(0), n->in(1), n->in(2));
+  Node*        dproj  = new ProjNode(divmod, DivModNode::div_proj_num);
+  Node*        mproj  = new ProjNode(divmod, DivModNode::mod_proj_num);
+  return divmod;
+}
+
+//------------------------------match------------------------------------------
+// return result(s) along with their RegMask info
+Node* UDivModINode::match( const ProjNode *proj, const Matcher *match ) {
+  uint ideal_reg = proj->ideal_reg();
+  RegMask rm;
+  if (proj->_con == div_proj_num) {
+    rm = match->divI_proj_mask();
+  } else {
+    assert(proj->_con == mod_proj_num, "must be div or mod projection");
+    rm = match->modI_proj_mask();
+  }
+  return new MachProjNode(this, proj->_con, rm, ideal_reg);
+}
+
+
+//------------------------------match------------------------------------------
+// return result(s) along with their RegMask info
+Node* UDivModLNode::match( const ProjNode *proj, const Matcher *match ) {
+  uint ideal_reg = proj->ideal_reg();
+  RegMask rm;
+  if (proj->_con == div_proj_num) {
+    rm = match->divL_proj_mask();
+  } else {
+    assert(proj->_con == mod_proj_num, "must be div or mod projection");
+    rm = match->modL_proj_mask();
+  }
+  return new MachProjNode(this, proj->_con, rm, ideal_reg);
+}
