@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "memory/allocation.inline.hpp"
 #include "opto/chaitin.hpp"
 #include "opto/compile.hpp"
@@ -47,7 +46,7 @@ julong IndexSet::_total_used_blocks = 0;
 julong IndexSet::_total_unused_blocks = 0;
 
 // Per set, or all sets operation tracing
-int IndexSet::_serial_count = 1;
+uint IndexSet::_serial_count = 1;
 #endif
 
 //---------------------------- IndexSet::populate_free_list() -----------------------------
@@ -122,7 +121,7 @@ IndexSet::BitBlock *IndexSet::alloc_block_containing(uint element) {
 // Add a BitBlock to the free list.
 
 void IndexSet::free_block(uint i) {
-  debug_only(check_watch("free block", i));
+  DEBUG_ONLY(check_watch("free block", i));
   assert(i < _max_blocks, "block index too large");
   BitBlock *block = _blocks[i];
   assert(block != &_empty_block, "cannot free the empty block");
@@ -229,6 +228,11 @@ IndexSet::IndexSet (IndexSet *set) {
   _count = set->_count;
   _current_block_limit = set->_current_block_limit;
   _max_blocks = set->_max_blocks;
+  if (set->_blocks == nullptr) {
+    _blocks = nullptr;
+    _arena = arena();
+    return;
+  }
   if (_max_blocks <= preallocated_block_list_size) {
     _blocks = _preallocated_block_list;
   } else {
@@ -259,15 +263,8 @@ void IndexSet::initialize(uint max_elements) {
   _count = 0;
   _current_block_limit = 0;
   _max_blocks = (max_elements + bits_per_block - 1) / bits_per_block;
-
-  if (_max_blocks <= preallocated_block_list_size) {
-    _blocks = _preallocated_block_list;
-  } else {
-    _blocks = (IndexSet::BitBlock**) arena()->AmallocWords(sizeof(IndexSet::BitBlock*) * _max_blocks);
-  }
-  for (uint i = 0; i < _max_blocks; i++) {
-    set_block(i, &_empty_block);
-  }
+  _arena = arena();
+  _blocks = nullptr;
 }
 
 //---------------------------- IndexSet::initialize()------------------------------
@@ -284,15 +281,8 @@ void IndexSet::initialize(uint max_elements, Arena *arena) {
   _count = 0;
   _current_block_limit = 0;
   _max_blocks = (max_elements + bits_per_block - 1) / bits_per_block;
-
-  if (_max_blocks <= preallocated_block_list_size) {
-    _blocks = _preallocated_block_list;
-  } else {
-    _blocks = (IndexSet::BitBlock**) arena->AmallocWords(sizeof(IndexSet::BitBlock*) * _max_blocks);
-  }
-  for (uint i = 0; i < _max_blocks; i++) {
-    set_block(i, &_empty_block);
-  }
+  _arena = arena;
+  _blocks = nullptr;
 }
 
 //---------------------------- IndexSet::swap() -----------------------------
@@ -305,6 +295,12 @@ void IndexSet::swap(IndexSet *set) {
   set->check_watch("swap", _serial_number);
 #endif
 
+  if (_blocks == nullptr && set->_blocks == nullptr) {
+    return;
+  }
+  initialize_if_needed();
+  set->initialize_if_needed();
+
   uint max = MAX2(_current_block_limit, set->_current_block_limit);
   for (uint i = 0; i < max; i++) {
     BitBlock *temp = _blocks[i];
@@ -345,7 +341,7 @@ void IndexSet::tally_iteration_statistics() const {
   inc_stat_counter(&_total_bits, count());
 
   for (uint i = 0; i < _max_blocks; i++) {
-    if (_blocks[i] != &_empty_block) {
+    if (_blocks != nullptr && _blocks[i] != &_empty_block) {
       inc_stat_counter(&_total_used_blocks, 1);
     } else {
       inc_stat_counter(&_total_unused_blocks, 1);
