@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -331,7 +331,7 @@ public:
 
   // Identify extra cases that we might want to vectorize automatically
   // And exclude cases which are not profitable to auto-vectorize.
-  static bool match_rule_supported_superword(int opcode, int vlen, BasicType bt);
+  static bool match_rule_supported_auto_vectorization(int opcode, int vlen, BasicType bt);
 
   // identify extra cases that we might want to provide match rules for
   // e.g. Op_ vector nodes and other intrinsics while guarding with vlen
@@ -341,8 +341,9 @@ public:
 
   static bool vector_needs_partial_operations(Node* node, const TypeVect* vt);
 
+  static bool vector_rearrange_requires_load_shuffle(BasicType elem_bt, int vlen);
+
   static const RegMask* predicate_reg_mask(void);
-  static const TypeVectMask* predicate_reg_type(const Type* elemTy, int length);
 
   // Vector width in bytes
   static int vector_width_in_bytes(BasicType bt);
@@ -355,7 +356,7 @@ public:
             Matcher::min_vector_size(bt) <= size);
   }
   // Limits on max vector size (number of elements) for auto-vectorization.
-  static int superword_max_vector_size(const BasicType bt);
+  static int max_vector_size_auto_vectorization(const BasicType bt);
 
   // Actual max scalable vector register length.
   static int scalable_vector_reg_size(const BasicType bt);
@@ -377,11 +378,16 @@ public:
   static BasicType vector_element_basic_type(const Node* n);
   static BasicType vector_element_basic_type(const MachNode* use, const MachOper* opnd);
 
+  // Vector element basic type is non double word integral type.
+  static bool is_non_long_integral_vector(const Node* n);
+
   // Check if given booltest condition is unsigned or not
   static inline bool is_unsigned_booltest_pred(int bt) {
     return ((bt & BoolTest::unsigned_compare) == BoolTest::unsigned_compare);
   }
 
+  static bool is_encode_and_store_pattern(const Node* n, const Node* m);
+
   // These calls are all generated by the ADLC
 
   // Java-Java calling convention
