@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -35,6 +35,7 @@
 #include "runtime/javaThread.inline.hpp"
 #include "runtime/vframe.inline.hpp"
 #include "runtime/vframeArray.hpp"
+#include "utilities/checkedCast.hpp"
 
 // call frame copied from old .h file and renamed
 typedef struct {
@@ -79,7 +80,7 @@ class vframeStreamForte : public vframeStreamCommon {
 };
 
 
-static bool is_decipherable_compiled_frame(JavaThread* thread, frame* fr, CompiledMethod* nm);
+static bool is_decipherable_compiled_frame(JavaThread* thread, frame* fr, nmethod* nm);
 static bool is_decipherable_interpreted_frame(JavaThread* thread,
                                               frame* fr,
                                               Method** method_p,
@@ -91,10 +92,10 @@ static bool is_decipherable_interpreted_frame(JavaThread* thread,
 vframeStreamForte::vframeStreamForte(JavaThread *jt,
                                      frame fr,
                                      bool stop_at_java_call_stub)
-    : vframeStreamCommon(RegisterMap(jt,
-                                     RegisterMap::UpdateMap::skip,
-                                     RegisterMap::ProcessFrames::skip,
-                                     RegisterMap::WalkContinuation::skip)) {
+    : vframeStreamCommon(jt,
+                         RegisterMap::UpdateMap::skip,
+                         RegisterMap::ProcessFrames::skip,
+                         RegisterMap::WalkContinuation::skip) {
   _reg_map.set_async(true);
   _stop_at_java_call_stub = stop_at_java_call_stub;
   _frame = fr;
@@ -149,7 +150,7 @@ void vframeStreamForte::forte_next() {
 // Determine if 'fr' is a decipherable compiled frame. We are already
 // assured that fr is for a java compiled method.
 
-static bool is_decipherable_compiled_frame(JavaThread* thread, frame* fr, CompiledMethod* nm) {
+static bool is_decipherable_compiled_frame(JavaThread* thread, frame* fr, nmethod* nm) {
   assert(nm->is_java_method(), "invariant");
 
   if (thread->has_last_Java_frame() && thread->last_Java_pc() == fr->pc()) {
@@ -158,7 +159,7 @@ static bool is_decipherable_compiled_frame(JavaThread* thread, frame* fr, Compil
     PcDesc* pc_desc = nm->pc_desc_at(fr->pc());
 
     // Did we find a useful PcDesc?
-    if (pc_desc != NULL &&
+    if (pc_desc != nullptr &&
         pc_desc->scope_decode_offset() != DebugInformationRecorder::serialized_null) {
       return true;
     }
@@ -172,7 +173,7 @@ static bool is_decipherable_compiled_frame(JavaThread* thread, frame* fr, Compil
   PcDesc* pc_desc = nm->pc_desc_near(fr->pc() + 1);
 
   // Now do we have a useful PcDesc?
-  if (pc_desc == NULL ||
+  if (pc_desc == nullptr ||
       pc_desc->scope_decode_offset() == DebugInformationRecorder::serialized_null) {
     // No debug information is available for this PC.
     //
@@ -207,7 +208,7 @@ static bool is_decipherable_compiled_frame(JavaThread* thread, frame* fr, Compil
 
 // Determine if 'fr' is a walkable interpreted frame. Returns false
 // if it is not. *method_p, and *bci_p are not set when false is
-// returned. *method_p is non-NULL if frame was executing a Java
+// returned. *method_p is non-null if frame was executing a Java
 // method. *bci_p is != -1 if a valid BCI in the Java method could
 // be found.
 // Note: this method returns true when a valid Java method is found
@@ -273,13 +274,13 @@ static bool is_decipherable_interpreted_frame(JavaThread* thread,
 // Check the return value of find_initial_Java_frame and the value of
 // 'method_p' to decide on how use the results returned by this method.
 //
-// If 'method_p' is not NULL, an initial Java frame has been found and
+// If 'method_p' is not null, an initial Java frame has been found and
 // the stack can be walked starting from that initial frame. In this case,
 // 'method_p' points to the Method that the initial frame belongs to and
 // the initial Java frame is returned in initial_frame_p.
 //
 // find_initial_Java_frame() returns true if a Method has been found (i.e.,
-// 'method_p' is not NULL) and the initial frame that belongs to that Method
+// 'method_p' is not null) and the initial frame that belongs to that Method
 // is decipherable.
 //
 // A frame is considered to be decipherable:
@@ -292,7 +293,7 @@ static bool is_decipherable_interpreted_frame(JavaThread* thread,
 // Note that find_initial_Java_frame() can return false even if an initial
 // Java method was found (e.g., there is no PCDesc available for the method).
 //
-// If 'method_p' is NULL, it was not possible to find a Java frame when
+// If 'method_p' is null, it was not possible to find a Java frame when
 // walking the stack starting from 'fr'. In this case find_initial_Java_frame
 // returns false.
 
@@ -305,10 +306,10 @@ static bool find_initial_Java_frame(JavaThread* thread,
   // It is possible that for a frame containing a compiled method
   // we can capture the method but no bci. If we get no
   // bci the frame isn't walkable but the method is usable.
-  // Therefore we init the returned Method* to NULL so the
+  // Therefore we init the returned Method* to null so the
   // caller can make the distinction.
 
-  *method_p = NULL;
+  *method_p = nullptr;
 
   // On the initial call to this method the frame we get may not be
   // recognizable to us. This should only happen if we are in a JRT_LEAF
@@ -333,11 +334,11 @@ static bool find_initial_Java_frame(JavaThread* thread,
       }
 
       if (candidate.is_entry_frame()) {
-        // jcw is NULL if the java call wrapper could not be found
+        // jcw is null if the java call wrapper could not be found
         JavaCallWrapper* jcw = candidate.entry_frame_call_wrapper_if_safe(thread);
         // If initial frame is frame from StubGenerator and there is no
         // previous anchor, there are no java frames associated with a method
-        if (jcw == NULL || jcw->is_first_frame()) {
+        if (jcw == nullptr || jcw->is_first_frame()) {
           return false;
         }
       }
@@ -363,7 +364,7 @@ static bool find_initial_Java_frame(JavaThread* thread,
   // it see if we can find such a frame because only frames with codeBlobs
   // are possible Java frames.
 
-  if (fr->cb() == NULL) {
+  if (fr->cb() == nullptr) {
 
     // See if we can find a useful frame
     int loop_count;
@@ -376,9 +377,9 @@ static bool find_initial_Java_frame(JavaThread* thread,
     for (loop_count = 0; loop_max == 0 || loop_count < loop_max; loop_count++) {
       if (!candidate.safe_for_sender(thread)) return false;
       candidate = candidate.sender(&map);
-      if (candidate.cb() != NULL) break;
+      if (candidate.cb() != nullptr) break;
     }
-    if (candidate.cb() == NULL) return false;
+    if (candidate.cb() == nullptr) return false;
   }
 
   // We have a frame known to be in the codeCache
@@ -393,11 +394,11 @@ static bool find_initial_Java_frame(JavaThread* thread,
   for (loop_count = 0; loop_max == 0 || loop_count < loop_max; loop_count++) {
 
     if (candidate.is_entry_frame()) {
-      // jcw is NULL if the java call wrapper couldn't be found
+      // jcw is null if the java call wrapper couldn't be found
       JavaCallWrapper *jcw = candidate.entry_frame_call_wrapper_if_safe(thread);
       // If initial frame is frame from StubGenerator and there is no
       // previous anchor, there are no java frames associated with a method
-      if (jcw == NULL || jcw->is_first_frame()) {
+      if (jcw == nullptr || jcw->is_first_frame()) {
         return false;
       }
     }
@@ -412,9 +413,9 @@ static bool find_initial_Java_frame(JavaThread* thread,
       return false;
     }
 
-    if (candidate.cb()->is_compiled()) {
+    if (candidate.cb()->is_nmethod()) {
 
-      CompiledMethod* nm = candidate.cb()->as_compiled_method();
+      nmethod* nm = candidate.cb()->as_nmethod();
       *method_p = nm->method();
 
       // If the frame is not decipherable, then the value of -1
@@ -443,7 +444,7 @@ static bool find_initial_Java_frame(JavaThread* thread,
       // is_decipherable_compiled_frame may modify candidate's pc
       *initial_frame_p = candidate;
 
-      assert(nm->pc_desc_at(candidate.pc()) != NULL, "debug information must be available if the frame is decipherable");
+      assert(nm->pc_desc_at(candidate.pc()) != nullptr, "debug information must be available if the frame is decipherable");
 
       return true;
     }
@@ -457,7 +458,7 @@ static bool find_initial_Java_frame(JavaThread* thread,
     // since once we find a frame in the code cache they
     // all should be there.
 
-    if (candidate.cb() == NULL) return false;
+    if (candidate.cb() == nullptr) return false;
 
   }
 
@@ -478,13 +479,13 @@ static void forte_fill_call_trace_given_top(JavaThread* thd,
   int count;
 
   count = 0;
-  assert(trace->frames != NULL, "trace->frames must be non-NULL");
+  assert(trace->frames != nullptr, "trace->frames must be non-null");
 
   // Walk the stack starting from 'top_frame' and search for an initial Java frame.
   find_initial_Java_frame(thd, &top_frame, &initial_Java_frame, &method, &bci);
 
   // Check if a Java Method has been found.
-  if (method == NULL) return;
+  if (method == nullptr) return;
 
   if (!Method::is_valid_method(method)) {
     trace->num_frames = ticks_GC_active; // -2
@@ -579,7 +580,7 @@ void AsyncGetCallTrace(ASGCT_CallTrace *trace, jint depth, void* ucontext) {
   Thread* raw_thread = Thread::current_or_null_safe();
   JavaThread* thread;
 
-  if (trace->env_id == NULL || raw_thread == NULL || !raw_thread->is_Java_thread() ||
+  if (trace->env_id == nullptr || raw_thread == nullptr || !raw_thread->is_Java_thread() ||
       (thread = JavaThread::cast(raw_thread))->is_exiting()) {
     // bad env_id, thread has exited or thread is exiting
     trace->num_frames = ticks_thread_exit; // -8
@@ -601,13 +602,13 @@ void AsyncGetCallTrace(ASGCT_CallTrace *trace, jint depth, void* ucontext) {
     return;
   }
 
-  if (Universe::heap()->is_gc_active()) {
+  if (Universe::heap()->is_stw_gc_active()) {
     trace->num_frames = ticks_GC_active; // -2
     return;
   }
 
-  // !important! make sure all to call thread->set_in_asgct(false) before every return
-  thread->set_in_asgct(true);
+  // signify to other code in the VM that we're in ASGCT
+  ThreadInAsgct tia(thread);
 
   switch (thread->thread_state()) {
   case _thread_new:
@@ -666,7 +667,6 @@ void AsyncGetCallTrace(ASGCT_CallTrace *trace, jint depth, void* ucontext) {
     trace->num_frames = ticks_unknown_state; // -7
     break;
   }
-  thread->set_in_asgct(false);
 }
 
 
@@ -711,11 +711,12 @@ bool Forte::is_enabled() {
 
 void Forte::register_stub(const char* name, address start, address end) {
 #if !defined(_WINDOWS)
-  assert(pointer_delta(end, start, sizeof(jbyte)) < INT_MAX,
+  size_t code_size = pointer_delta(end, start, sizeof(jbyte));
+  assert(code_size < INT_MAX,
          "Code size exceeds maximum range");
 
-  collector_func_load((char*)name, NULL, NULL, start,
-    pointer_delta(end, start, sizeof(jbyte)), 0, NULL);
+  collector_func_load((char*)name, nullptr, nullptr, start,
+                      checked_cast<int>(code_size), 0, nullptr);
 #endif // !_WINDOWS
 }
 
