@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "classfile/javaClasses.inline.hpp"
 #include "classfile/vmClasses.hpp"
 #include "classfile/vmSymbols.hpp"
@@ -351,33 +350,24 @@ check_is_obj_array(JavaThread* thr, jarray jArray) {
   }
 }
 
-// Arbitrary (but well-known) tag for GetStringChars
-const void* STRING_TAG = (void*)0x47114711;
-
-// Arbitrary (but well-known) tag for GetStringUTFChars
-const void* STRING_UTF_TAG = (void*) 0x48124812;
-
-// Arbitrary (but well-known) tag for GetPrimitiveArrayCritical
-const void* CRITICAL_TAG = (void*)0x49134913;
-
 /*
  * Copy and wrap array elements for bounds checking.
  * Remember the original elements (GuardedMemory::get_tag())
  */
 static void* check_jni_wrap_copy_array(JavaThread* thr, jarray array,
-                                       void* orig_elements, jboolean is_critical = JNI_FALSE) {
+    void* orig_elements) {
   void* result;
   IN_VM(
     oop a = JNIHandles::resolve_non_null(array);
     size_t len = arrayOop(a)->length() <<
         TypeArrayKlass::cast(a->klass())->log2_element_size();
-    result = GuardedMemory::wrap_copy(orig_elements, len, orig_elements, is_critical ? CRITICAL_TAG : nullptr);
+    result = GuardedMemory::wrap_copy(orig_elements, len, orig_elements);
   )
   return result;
 }
 
 static void* check_wrapped_array(JavaThread* thr, const char* fn_name,
-                                 void* obj, void* carray, size_t* rsz, jboolean is_critical) {
+    void* obj, void* carray, size_t* rsz) {
   if (carray == nullptr) {
     tty->print_cr("%s: elements vector null" PTR_FORMAT, fn_name, p2i(obj));
     NativeReportJNIFatalError(thr, "Elements vector null");
@@ -396,29 +386,6 @@ static void* check_wrapped_array(JavaThread* thr, const char* fn_name,
     DEBUG_ONLY(guarded.print_on(tty);) // This may crash.
     NativeReportJNIFatalError(thr, err_msg("%s: unrecognized elements", fn_name));
   }
-  if (orig_result == STRING_TAG || orig_result == STRING_UTF_TAG) {
-    bool was_utf = orig_result == STRING_UTF_TAG;
-    tty->print_cr("%s: called on something allocated by %s",
-                  fn_name, was_utf ? "GetStringUTFChars" : "GetStringChars");
-    DEBUG_ONLY(guarded.print_on(tty);) // This may crash.
-    NativeReportJNIFatalError(thr, err_msg("%s called on something allocated by %s",
-                                           fn_name, was_utf ? "GetStringUTFChars" : "GetStringChars"));
-  }
-
-  if (is_critical && (guarded.get_tag2() != CRITICAL_TAG)) {
-    tty->print_cr("%s: called on something not allocated by GetPrimitiveArrayCritical", fn_name);
-    DEBUG_ONLY(guarded.print_on(tty);) // This may crash.
-    NativeReportJNIFatalError(thr, err_msg("%s called on something not allocated by GetPrimitiveArrayCritical",
-                                           fn_name));
-  }
-
-  if (!is_critical && (guarded.get_tag2() == CRITICAL_TAG)) {
-    tty->print_cr("%s: called on something allocated by GetPrimitiveArrayCritical", fn_name);
-    DEBUG_ONLY(guarded.print_on(tty);) // This may crash.
-    NativeReportJNIFatalError(thr, err_msg("%s called on something allocated by GetPrimitiveArrayCritical",
-                                           fn_name));
-  }
-
   if (rsz != nullptr) {
     *rsz = guarded.get_user_size();
   }
@@ -428,7 +395,7 @@ static void* check_wrapped_array(JavaThread* thr, const char* fn_name,
 static void* check_wrapped_array_release(JavaThread* thr, const char* fn_name,
                                          void* obj, void* carray, jint mode, jboolean is_critical) {
   size_t sz;
-  void* orig_result = check_wrapped_array(thr, fn_name, obj, carray, &sz, is_critical);
+  void* orig_result = check_wrapped_array(thr, fn_name, obj, carray, &sz);
   switch (mode) {
   case 0:
     memcpy(orig_result, carray, sz);
@@ -507,7 +474,7 @@ void jniCheck::validate_class_descriptor(JavaThread* thr, const char* name) {
   }
 
   // Verify that the class name given is a valid utf8 string
-  if (!UTF8::is_legal_utf8((const unsigned char*)name, (int)strlen(name), false)) {
+  if (!UTF8::is_legal_utf8((const unsigned char*)name, strlen(name), false)) {
     char msg[JVM_MAXPATHLEN];
     jio_snprintf(msg, JVM_MAXPATHLEN, "%s%s%s", fatal_non_utf8_class_name1, name, fatal_non_utf8_class_name2);
     ReportJNIFatalError(thr, msg);
@@ -1463,6 +1430,9 @@ JNI_ENTRY_CHECKED(jsize,
     return result;
 JNI_END
 
+// Arbitrary (but well-known) tag
+const void* STRING_TAG = (void*)0x47114711;
+
 JNI_ENTRY_CHECKED(const jchar *,
   checked_jni_GetStringChars(JNIEnv *env,
                              jstring str,
@@ -1540,10 +1510,34 @@ JNI_ENTRY_CHECKED(jsize,
       checkString(thr, str);
     )
     jsize result = UNCHECKED()->GetStringUTFLength(env,str);
+    jlong full_length =  UNCHECKED()->GetStringUTFLengthAsLong(env,str);
+    if (full_length > result) {
+      ResourceMark rm(thr);
+      stringStream ss;
+      ss.print("WARNING: large String with modified UTF-8 length " JLONG_FORMAT
+                " is reporting a reduced length of %d - use GetStringUTFLengthAsLong instead",
+                full_length, result);
+      NativeReportJNIWarning(thr, ss.as_string());
+    }
     functionExit(thr);
     return result;
 JNI_END
 
+JNI_ENTRY_CHECKED(jlong,
+  checked_jni_GetStringUTFLengthAsLong(JNIEnv *env,
+                                       jstring str))
+    functionEnter(thr);
+    IN_VM(
+      checkString(thr, str);
+    )
+    jlong result = UNCHECKED()->GetStringUTFLengthAsLong(env,str);
+    functionExit(thr);
+    return result;
+JNI_END
+
+// Arbitrary (but well-known) tag - different than GetStringChars
+const void* STRING_UTF_TAG = (void*) 0x48124812;
+
 JNI_ENTRY_CHECKED(const char *,
   checked_jni_GetStringUTFChars(JNIEnv *env,
                                 jstring str,
@@ -1865,7 +1859,7 @@ JNI_ENTRY_CHECKED(void *,
     )
     void *result = UNCHECKED()->GetPrimitiveArrayCritical(env, array, isCopy);
     if (result != nullptr) {
-      result = check_jni_wrap_copy_array(thr, array, result, JNI_TRUE);
+      result = check_jni_wrap_copy_array(thr, array, result);
     }
     functionExit(thr);
     return result;
@@ -2309,7 +2303,12 @@ struct JNINativeInterface_  checked_jni_NativeInterface = {
 
     // Virtual threads
 
-    checked_jni_IsVirtualThread
+    checked_jni_IsVirtualThread,
+
+    // Large UTF8 support
+
+    checked_jni_GetStringUTFLengthAsLong
+
 };
 
 
@@ -2321,7 +2320,7 @@ struct JNINativeInterface_* jni_functions_check() {
   // make sure the last pointer in the checked table is not null, indicating
   // an addition to the JNINativeInterface_ structure without initializing
   // it in the checked table.
-  debug_only(intptr_t *lastPtr = (intptr_t *)((char *)&checked_jni_NativeInterface + \
+  DEBUG_ONLY(intptr_t *lastPtr = (intptr_t *)((char *)&checked_jni_NativeInterface + \
              sizeof(*unchecked_jni_NativeInterface) - sizeof(char *));)
   assert(*lastPtr != 0,
          "Mismatched JNINativeInterface tables, check for new entries");
