@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -54,12 +54,17 @@ class Abstract_VM_Version: AllStatic {
   static const char*  _s_vm_release;
   static const char*  _s_internal_vm_info_string;
 
-  // CPU feature flags.
+  // CPU feature flags, can be affected by VM settings.
   static uint64_t _features;
   static const char* _features_string;
 
+  // Original CPU feature flags, not affected by VM settings.
+  static uint64_t _cpu_features;
+
   // These are set by machine-dependent initializations
+#ifndef SUPPORTS_NATIVE_CX8
   static bool         _supports_cx8;
+#endif
   static bool         _supports_atomic_getset4;
   static bool         _supports_atomic_getset8;
   static bool         _supports_atomic_getadd4;
@@ -105,6 +110,7 @@ class Abstract_VM_Version: AllStatic {
   static const char* vm_info_string();
   static const char* vm_release();
   static const char* vm_platform_string();
+  static const char* vm_variant();
 
   static int vm_major_version()               { return _vm_major_version; }
   static int vm_minor_version()               { return _vm_minor_version; }
@@ -133,6 +139,8 @@ class Abstract_VM_Version: AllStatic {
   static void print_platform_virtualization_info(outputStream*) { }
 
   // does HW support an 8-byte compare-exchange operation?
+  // Required to be true but still dynamically checked at runtime
+  // for platforms that don't set SUPPORTS_NATIVE_CX8
   static bool supports_cx8()  {
 #ifdef SUPPORTS_NATIVE_CX8
     return true;
@@ -180,6 +188,12 @@ class Abstract_VM_Version: AllStatic {
   // Does platform support stack watermark barriers for concurrent stack processing?
   constexpr static bool supports_stack_watermark_barrier() { return false; }
 
+  // Is recursive lightweight locking implemented for this platform?
+  constexpr static bool supports_recursive_lightweight_locking() { return false; }
+
+  // Does platform support secondary supers table lookup?
+  constexpr static bool supports_secondary_supers_table() { return false; }
+
   // Does platform support float16 instructions?
   static bool supports_float16() { return false; }
 
