@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,14 +22,16 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "logging/log.hpp"
+#include "memory/resourceArea.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
+#include "runtime/javaThread.inline.hpp"
 #include "runtime/mutex.hpp"
 #include "runtime/os.inline.hpp"
 #include "runtime/osThread.hpp"
 #include "runtime/safepointMechanism.inline.hpp"
-#include "runtime/thread.inline.hpp"
+#include "runtime/semaphore.inline.hpp"
+#include "runtime/threadCrashProtection.hpp"
 #include "utilities/events.hpp"
 #include "utilities/macros.hpp"
 
@@ -38,13 +40,13 @@ class InFlightMutexRelease {
   Mutex* _in_flight_mutex;
  public:
   InFlightMutexRelease(Mutex* in_flight_mutex) : _in_flight_mutex(in_flight_mutex) {
-    assert(in_flight_mutex != NULL, "must be");
+    assert(in_flight_mutex != nullptr, "must be");
   }
   void operator()(JavaThread* current) {
     _in_flight_mutex->release_for_safepoint();
-    _in_flight_mutex = NULL;
+    _in_flight_mutex = nullptr;
   }
-  bool not_released() { return _in_flight_mutex != NULL; }
+  bool not_released() { return _in_flight_mutex != nullptr; }
 };
 
 #ifdef ASSERT
@@ -55,31 +57,28 @@ void Mutex::check_block_state(Thread* thread) {
     fatal("VM thread could block on lock that may be held by a JavaThread during safepoint: %s", name());
   }
 
-  assert(!os::ThreadCrashProtection::is_crash_protected(thread),
+  assert(!ThreadCrashProtection::is_crash_protected(thread),
          "locking not allowed when crash protection is set");
 }
 
 void Mutex::check_safepoint_state(Thread* thread) {
   check_block_state(thread);
 
-  // If the JavaThread checks for safepoint, verify that the lock wasn't created with safepoint_check_never.
-  if (thread->is_active_Java_thread()) {
-    assert(_safepoint_check_required != _safepoint_check_never,
-           "This lock should never have a safepoint check for Java threads: %s",
-           name());
+  // If the lock acquisition checks for safepoint, verify that the lock was created with rank that
+  // has safepoint checks. Technically this doesn't affect NonJavaThreads since they won't actually
+  // check for safepoint, but let's make the rule unconditional unless there's a good reason not to.
+  assert(_rank > nosafepoint,
+         "This lock should not be taken with a safepoint check: %s", name());
 
+  if (thread->is_active_Java_thread()) {
     // Also check NoSafepointVerifier, and thread state is _thread_in_vm
     JavaThread::cast(thread)->check_for_valid_safepoint_state();
-  } else {
-    // If initialized with safepoint_check_never, a NonJavaThread should never ask to safepoint check either.
-    assert(_safepoint_check_required != _safepoint_check_never,
-           "NonJavaThread should not check for safepoint");
   }
 }
 
 void Mutex::check_no_safepoint_state(Thread* thread) {
   check_block_state(thread);
-  assert(!thread->is_active_Java_thread() || _safepoint_check_required != _safepoint_check_always,
+  assert(!thread->is_active_Java_thread() || _rank <= nosafepoint,
          "This lock should always have a safepoint check for Java threads: %s",
          name());
 }
@@ -98,7 +97,7 @@ void Mutex::lock_contended(Thread* self) {
     // Is it a JavaThread participating in the safepoint protocol.
     if (is_active_Java_thread) {
       InFlightMutexRelease ifmr(this);
-      assert(rank() > Mutex::special, "Potential deadlock with special or lesser rank mutex");
+      assert(rank() > Mutex::nosafepoint, "Potential deadlock with nosafepoint or lesser rank mutex");
       {
         ThreadBlockInVMPreprocess<InFlightMutexRelease> tbivmdc(JavaThread::cast(self), ifmr);
         _lock.lock();
@@ -125,7 +124,7 @@ void Mutex::lock(Thread* self) {
     lock_contended(self);
   }
 
-  assert_owner(NULL);
+  assert_owner(nullptr);
   set_owner(self);
 }
 
@@ -146,7 +145,7 @@ void Mutex::lock_without_safepoint_check(Thread * self) {
   check_rank(self);
 
   _lock.lock();
-  assert_owner(NULL);
+  assert_owner(nullptr);
   set_owner(self);
 }
 
@@ -167,12 +166,12 @@ bool Mutex::try_lock_inner(bool do_rank_checks) {
   if (do_rank_checks) {
     check_rank(self);
   }
-  // Some safepoint_check_always locks use try_lock, so cannot check
+  // Some safepoint checking locks use try_lock, so cannot check
   // safepoint state, but can check blocking state.
   check_block_state(self);
 
   if (_lock.try_lock()) {
-    assert_owner(NULL);
+    assert_owner(nullptr);
     set_owner(self);
     return true;
   }
@@ -190,13 +189,13 @@ bool Mutex::try_lock_without_rank_check() {
 }
 
 void Mutex::release_for_safepoint() {
-  assert_owner(NULL);
+  assert_owner(nullptr);
   _lock.unlock();
 }
 
 void Mutex::unlock() {
   DEBUG_ONLY(assert_owner(Thread::current()));
-  set_owner(NULL);
+  set_owner(nullptr);
   _lock.unlock();
 }
 
@@ -210,17 +209,16 @@ void Monitor::notify_all() {
   _lock.notify_all();
 }
 
-bool Monitor::wait_without_safepoint_check(int64_t timeout) {
+// timeout is in milliseconds - with zero meaning never timeout
+bool Monitor::wait_without_safepoint_check(uint64_t timeout) {
   Thread* const self = Thread::current();
 
-  // timeout is in milliseconds - with zero meaning never timeout
-  assert(timeout >= 0, "negative timeout");
   assert_owner(self);
   check_rank(self);
 
-  // conceptually set the owner to NULL in anticipation of
+  // conceptually set the owner to null in anticipation of
   // abdicating the lock in wait
-  set_owner(NULL);
+  set_owner(nullptr);
 
   // Check safepoint state after resetting owner and possible NSV.
   check_no_safepoint_state(self);
@@ -230,19 +228,18 @@ bool Monitor::wait_without_safepoint_check(int64_t timeout) {
   return wait_status != 0;          // return true IFF timeout
 }
 
-bool Monitor::wait(int64_t timeout) {
+// timeout is in milliseconds - with zero meaning never timeout
+bool Monitor::wait(uint64_t timeout) {
   JavaThread* const self = JavaThread::current();
   // Safepoint checking logically implies an active JavaThread.
   assert(self->is_active_Java_thread(), "invariant");
 
-  // timeout is in milliseconds - with zero meaning never timeout
-  assert(timeout >= 0, "negative timeout");
   assert_owner(self);
   check_rank(self);
 
-  // conceptually set the owner to NULL in anticipation of
+  // conceptually set the owner to null in anticipation of
   // abdicating the lock in wait
-  set_owner(NULL);
+  set_owner(nullptr);
 
   // Check safepoint state after resetting owner and possible NSV.
   check_safepoint_state(self);
@@ -259,7 +256,7 @@ bool Monitor::wait(int64_t timeout) {
 
   if (ifmr.not_released()) {
     // Not unlocked by ~ThreadBlockInVMPreprocess
-    assert_owner(NULL);
+    assert_owner(nullptr);
     // Conceptually reestablish ownership of the lock.
     set_owner(self);
   } else {
@@ -269,34 +266,36 @@ bool Monitor::wait(int64_t timeout) {
   return wait_status != 0;          // return true IFF timeout
 }
 
+static const int MAX_NUM_MUTEX = 1204;
+static Mutex* _internal_mutex_arr[MAX_NUM_MUTEX];
+Mutex** Mutex::_mutex_array = _internal_mutex_arr;
+int Mutex::_num_mutex = 0;
+
+void Mutex::add_mutex(Mutex* var) {
+  assert(Mutex::_num_mutex < MAX_NUM_MUTEX, "increase MAX_NUM_MUTEX");
+  Mutex::_mutex_array[_num_mutex++] = var;
+}
+
 Mutex::~Mutex() {
-  assert_owner(NULL);
+  assert_owner(nullptr);
   os::free(const_cast<char*>(_name));
 }
 
-Mutex::Mutex(int Rank, const char * name, SafepointCheckRequired safepoint_check_required,
-             bool allow_vm_block) : _owner(NULL) {
+Mutex::Mutex(Rank rank, const char * name, bool allow_vm_block) : _owner(nullptr) {
   assert(os::mutex_init_done(), "Too early!");
-  assert(name != NULL, "Mutex requires a name");
+  assert(name != nullptr, "Mutex requires a name");
   _name = os::strdup(name, mtInternal);
 #ifdef ASSERT
   _allow_vm_block  = allow_vm_block;
-  _rank            = Rank;
-  _safepoint_check_required = safepoint_check_required;
+  _rank            = rank;
   _skip_rank_check = false;
 
-  assert(_rank < nonleaf || _safepoint_check_required == _safepoint_check_always,
-         "higher than nonleaf should safepoint %s", name);
-
-  assert(_rank > special || _safepoint_check_required == _safepoint_check_never,
-         "Special locks or below should never safepoint");
+  assert(_rank >= static_cast<Rank>(0) && _rank <= safepoint, "Bad lock rank %s: %s", rank_name(), name);
 
   // The allow_vm_block also includes allowing other non-Java threads to block or
   // allowing Java threads to block in native.
-  assert(_safepoint_check_required == _safepoint_check_always || _allow_vm_block,
-         "Safepoint check never locks should always allow the vm to block");
-
-  assert(_rank >= 0, "Bad lock rank");
+  assert(_rank > nosafepoint || _allow_vm_block,
+         "Locks that don't check for safepoint should always allow the vm to block: %s", name);
 #endif
 }
 
@@ -312,31 +311,69 @@ void Mutex::print_on_error(outputStream* st) const {
 
 // ----------------------------------------------------------------------------------
 // Non-product code
+//
+#ifdef ASSERT
+static Mutex::Rank _ranks[] = { Mutex::event, Mutex::service, Mutex::stackwatermark, Mutex::tty, Mutex::oopstorage,
+                                Mutex::nosafepoint, Mutex::safepoint };
 
-#ifndef PRODUCT
-const char* print_safepoint_check(Mutex::SafepointCheckRequired safepoint_check) {
-  switch (safepoint_check) {
-  case Mutex::_safepoint_check_never:     return "safepoint_check_never";
-  case Mutex::_safepoint_check_always:    return "safepoint_check_always";
-  default: return "";
+static const char* _rank_names[] = { "event", "service", "stackwatermark", "tty", "oopstorage",
+                                     "nosafepoint", "safepoint" };
+
+static const int _num_ranks = 7;
+
+static const char* rank_name_internal(Mutex::Rank r) {
+  // Find closest rank and print out the name
+  stringStream st;
+  for (int i = 0; i < _num_ranks; i++) {
+    if (r == _ranks[i]) {
+      return _rank_names[i];
+    } else if (r  > _ranks[i] && (i < _num_ranks-1 && r < _ranks[i+1])) {
+      int delta = static_cast<int>(_ranks[i+1]) - static_cast<int>(r);
+      st.print("%s-%d", _rank_names[i+1], delta);
+      return st.as_string();
+    }
   }
+  return "fail";
 }
 
+const char* Mutex::rank_name() const {
+  return rank_name_internal(_rank);
+}
+
+
+void Mutex::assert_no_overlap(Rank orig, Rank adjusted, int adjust) {
+  int i = 0;
+  while (_ranks[i] < orig) i++;
+  // underflow is caught in constructor
+  if (i != 0 && adjusted > event && adjusted <= _ranks[i-1]) {
+    ResourceMark rm;
+    assert(adjusted > _ranks[i-1],
+           "Rank %s-%d overlaps with %s",
+           rank_name_internal(orig), adjust, rank_name_internal(adjusted));
+  }
+}
+#endif // ASSERT
+
+#ifndef PRODUCT
 void Mutex::print_on(outputStream* st) const {
   st->print("Mutex: [" PTR_FORMAT "] %s - owner: " PTR_FORMAT,
             p2i(this), _name, p2i(owner()));
   if (_allow_vm_block) {
     st->print("%s", " allow_vm_block");
   }
-  st->print(" %s", print_safepoint_check(_safepoint_check_required));
+  DEBUG_ONLY(st->print(" %s", rank_name()));
   st->cr();
 }
+
+void Mutex::print() const {
+  print_on(::tty);
+}
 #endif // PRODUCT
 
 #ifdef ASSERT
 void Mutex::assert_owner(Thread * expected) {
   const char* msg = "invalid owner";
-  if (expected == NULL) {
+  if (expected == nullptr) {
     msg = "should be un-owned";
   }
   else if (expected == Thread::current()) {
@@ -349,7 +386,7 @@ void Mutex::assert_owner(Thread * expected) {
 
 Mutex* Mutex::get_least_ranked_lock(Mutex* locks) {
   Mutex *res, *tmp;
-  for (res = tmp = locks; tmp != NULL; tmp = tmp->next()) {
+  for (res = tmp = locks; tmp != nullptr; tmp = tmp->next()) {
     if (tmp->rank() < res->rank()) {
       res = tmp;
     }
@@ -359,8 +396,8 @@ Mutex* Mutex::get_least_ranked_lock(Mutex* locks) {
 
 Mutex* Mutex::get_least_ranked_lock_besides_this(Mutex* locks) {
   Mutex *res, *tmp;
-  for (res = NULL, tmp = locks; tmp != NULL; tmp = tmp->next()) {
-    if (tmp != this && (res == NULL || tmp->rank() < res->rank())) {
+  for (res = nullptr, tmp = locks; tmp != nullptr; tmp = tmp->next()) {
+    if (tmp != this && (res == nullptr || tmp->rank() < res->rank())) {
       res = tmp;
     }
   }
@@ -372,31 +409,36 @@ Mutex* Mutex::get_least_ranked_lock_besides_this(Mutex* locks) {
 void Mutex::check_rank(Thread* thread) {
   Mutex* locks_owned = thread->owned_locks();
 
-  if (!SafepointSynchronize::is_at_safepoint()) {
-    // We expect the locks already acquired to be in increasing rank order,
-    // modulo locks acquired in try_lock_without_rank_check()
-    for (Mutex* tmp = locks_owned; tmp != NULL; tmp = tmp->next()) {
-      if (tmp->next() != NULL) {
-        assert(tmp->rank() < tmp->next()->rank()
-               || tmp->skip_rank_check(), "mutex rank anomaly?");
-      }
+  // We expect the locks already acquired to be in increasing rank order,
+  // modulo locks acquired in try_lock_without_rank_check()
+  for (Mutex* tmp = locks_owned; tmp != nullptr; tmp = tmp->next()) {
+    if (tmp->next() != nullptr) {
+      assert(tmp->rank() < tmp->next()->rank()
+             || tmp->skip_rank_check(), "mutex rank anomaly?");
     }
   }
 
-  bool check_can_be_skipped = SafepointSynchronize::is_at_safepoint();
   if (owned_by_self()) {
     // wait() case
     Mutex* least = get_least_ranked_lock_besides_this(locks_owned);
-    // We enforce not holding locks of rank special or lower while waiting.
+    // For JavaThreads, we enforce not holding locks of rank nosafepoint or lower while waiting
+    // because the held lock has a NoSafepointVerifier so waiting on a lower ranked lock will not be
+    // able to check for safepoints first with a TBIVM.
+    // For all threads, we enforce not holding the tty lock or below, since this could block progress also.
     // Also "this" should be the monitor with lowest rank owned by this thread.
-    if (least != NULL && (least->rank() <= special ||
-        (least->rank() <= this->rank() && !check_can_be_skipped))) {
-      assert(false, "Attempting to wait on monitor %s/%d while holding lock %s/%d -- "
-             "possible deadlock. %s", name(), rank(), least->name(), least->rank(),
-             least->rank() <= this->rank() ? "Should wait on the least ranked monitor from "
-             "all owned locks." : "Should not block(wait) while holding a lock of rank special.");
+    if (least != nullptr && ((least->rank() <= Mutex::nosafepoint && thread->is_Java_thread()) ||
+                           least->rank() <= Mutex::tty ||
+                           least->rank() <= this->rank())) {
+      ResourceMark rm(thread);
+      assert(false, "Attempting to wait on monitor %s/%s while holding lock %s/%s -- "
+             "possible deadlock. %s", name(), rank_name(), least->name(), least->rank_name(),
+             least->rank() <= this->rank() ?
+              "Should wait on the least ranked monitor from all owned locks." :
+             thread->is_Java_thread() ?
+              "Should not block(wait) while holding a lock of rank nosafepoint or below." :
+              "Should not block(wait) while holding a lock of rank tty or below.");
     }
-  } else if (!check_can_be_skipped) {
+  } else {
     // lock()/lock_without_safepoint_check()/try_lock() case
     Mutex* least = get_least_ranked_lock(locks_owned);
     // Deadlock prevention rules require us to acquire Mutexes only in
@@ -404,28 +446,20 @@ void Mutex::check_rank(Thread* thread) {
     // that the thread holds and m2 is the mutex the thread is trying
     // to acquire, then deadlock prevention rules require that the rank
     // of m2 be less than the rank of m1. This prevents circular waits.
-    if (least != NULL && least->rank() <= this->rank()) {
+    if (least != nullptr && least->rank() <= this->rank()) {
+      ResourceMark rm(thread);
       if (least->rank() > Mutex::tty) {
         // Printing owned locks acquires tty lock. If the least rank was below or equal
         // tty, then deadlock detection code would circle back here, until we run
         // out of stack and crash hard. Print locks only when it is safe.
         thread->print_owned_locks();
       }
-      assert(false, "Attempting to acquire lock %s/%d out of order with lock %s/%d -- "
-             "possible deadlock", this->name(), this->rank(), least->name(), least->rank());
+      assert(false, "Attempting to acquire lock %s/%s out of order with lock %s/%s -- "
+             "possible deadlock", this->name(), this->rank_name(), least->name(), least->rank_name());
     }
   }
 }
 
-bool Mutex::contains(Mutex* locks, Mutex* lock) {
-  for (; locks != NULL; locks = locks->next()) {
-    if (locks == lock) {
-      return true;
-    }
-  }
-  return false;
-}
-
 // Called immediately after lock acquisition or release as a diagnostic
 // to track the lock-set of the thread.
 // Rather like an EventListener for _owner (:>).
@@ -437,15 +471,15 @@ void Mutex::set_owner_implementation(Thread *new_owner) {
   // It uses the Mutex::_owner, Mutex::_next, and
   // Thread::_owned_locks fields, and no other function
   // changes those fields.
-  // It is illegal to set the mutex from one non-NULL
-  // owner to another--it must be owned by NULL as an
+  // It is illegal to set the mutex from one non-null
+  // owner to another--it must be owned by null as an
   // intermediate state.
 
-  if (new_owner != NULL) {
+  if (new_owner != nullptr) {
     // the thread is acquiring this lock
 
     assert(new_owner == Thread::current(), "Should I be doing this?");
-    assert(owner() == NULL, "setting the owner thread of an already owned mutex");
+    assert(owner() == nullptr, "setting the owner thread of an already owned mutex");
     raw_set_owner(new_owner); // set the owner
 
     // link "this" into the owned locks list
@@ -466,30 +500,30 @@ void Mutex::set_owner_implementation(Thread *new_owner) {
     _last_owner = old_owner;
     _skip_rank_check = false;
 
-    assert(old_owner != NULL, "removing the owner thread of an unowned mutex");
+    assert(old_owner != nullptr, "removing the owner thread of an unowned mutex");
     assert(old_owner == Thread::current(), "removing the owner thread of an unowned mutex");
 
-    raw_set_owner(NULL); // set the owner
+    raw_set_owner(nullptr); // set the owner
 
     Mutex* locks = old_owner->owned_locks();
 
     // remove "this" from the owned locks list
 
-    Mutex* prev = NULL;
+    Mutex* prev = nullptr;
     bool found = false;
-    for (; locks != NULL; prev = locks, locks = locks->next()) {
+    for (; locks != nullptr; prev = locks, locks = locks->next()) {
       if (locks == this) {
         found = true;
         break;
       }
     }
     assert(found, "Removing a lock not owned");
-    if (prev == NULL) {
+    if (prev == nullptr) {
       old_owner->_owned_locks = _next;
     } else {
       prev->_next = _next;
     }
-    _next = NULL;
+    _next = nullptr;
 
     // ~NSV implied with locking allow_vm_block flag.
     if (old_owner->is_Java_thread() && _allow_vm_block && this != tty_lock) {
@@ -498,3 +532,88 @@ void Mutex::set_owner_implementation(Thread *new_owner) {
   }
 }
 #endif // ASSERT
+
+// Print all mutexes/monitors that are currently owned by a thread; called
+// by fatal error handler.
+void Mutex::print_owned_locks_on_error(outputStream* st) {
+  st->print("VM Mutex/Monitor currently owned by a thread: ");
+  bool none = true;
+  for (int i = 0; i < _num_mutex; i++) {
+    // see if it has an owner
+    if (_mutex_array[i]->owner() != nullptr) {
+      if (none) {
+        // print format used by Mutex::print_on_error()
+        st->print_cr(" ([mutex/lock_event])");
+        none = false;
+      }
+      _mutex_array[i]->print_on_error(st);
+      st->cr();
+    }
+  }
+  if (none) st->print_cr("None");
+}
+
+void Mutex::print_lock_ranks(outputStream* st) {
+  st->print_cr("VM Mutex/Monitor ranks: ");
+
+#ifdef ASSERT
+  // Be extra defensive and figure out the bounds on
+  // ranks right here. This also saves a bit of time
+  // in the #ranks*#mutexes loop below.
+  int min_rank = INT_MAX;
+  int max_rank = INT_MIN;
+  for (int i = 0; i < _num_mutex; i++) {
+    Mutex* m = _mutex_array[i];
+    int r = (int) m->rank();
+    if (min_rank > r) min_rank = r;
+    if (max_rank < r) max_rank = r;
+  }
+
+  // Print the listings rank by rank
+  for (int r = min_rank; r <= max_rank; r++) {
+    bool first = true;
+    for (int i = 0; i < _num_mutex; i++) {
+      Mutex* m = _mutex_array[i];
+      if (r != (int) m->rank()) continue;
+
+      if (first) {
+        st->cr();
+        st->print_cr("Rank \"%s\":", m->rank_name());
+        first = false;
+      }
+      st->print_cr("  %s", m->name());
+    }
+  }
+#else
+  st->print_cr("  Only known in debug builds.");
+#endif // ASSERT
+}
+
+RecursiveMutex::RecursiveMutex() : _sem(1), _owner(nullptr), _recursions(0) {}
+
+void RecursiveMutex::lock(Thread* current) {
+  assert(current == Thread::current(), "must be current thread");
+  if (current == _owner) {
+    _recursions++;
+  } else {
+    // can be called by jvmti by VMThread.
+    if (current->is_Java_thread()) {
+      _sem.wait_with_safepoint_check(JavaThread::cast(current));
+    } else {
+      _sem.wait();
+    }
+    _recursions++;
+    assert(_recursions == 1, "should be");
+    _owner = current;
+  }
+}
+
+void RecursiveMutex::unlock(Thread* current) {
+  assert(current == Thread::current(), "must be current thread");
+  assert(current == _owner, "must be owner");
+  _recursions--;
+  if (_recursions == 0) {
+    _owner = nullptr;
+    _sem.signal();
+  }
+}
