@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "asm/macroAssembler.hpp"
 #include "asm/macroAssembler.inline.hpp"
 #include "code/codeCache.hpp"
@@ -35,28 +34,27 @@
 
 // Implementation of StubCodeDesc
 
-StubCodeDesc* StubCodeDesc::_list = NULL;
+StubCodeDesc* StubCodeDesc::_list = nullptr;
 bool          StubCodeDesc::_frozen = false;
 
 StubCodeDesc* StubCodeDesc::desc_for(address pc) {
   StubCodeDesc* p = _list;
-  while (p != NULL && !p->contains(pc)) {
+  while (p != nullptr && !p->contains(pc)) {
     p = p->_next;
   }
   return p;
 }
 
-const char* StubCodeDesc::name_for(address pc) {
-  StubCodeDesc* p = desc_for(pc);
-  return p == NULL ? NULL : p->name();
-}
-
-
 void StubCodeDesc::freeze() {
   assert(!_frozen, "repeated freeze operation");
   _frozen = true;
 }
 
+void StubCodeDesc::unfreeze() {
+  assert(_frozen, "repeated unfreeze operation");
+  _frozen = false;
+}
+
 void StubCodeDesc::print_on(outputStream* st) const {
   st->print("%s", group());
   st->print("::");
@@ -77,7 +75,7 @@ StubCodeGenerator::~StubCodeGenerator() {
 #ifndef PRODUCT
   CodeBuffer* cbuf = _masm->code();
   CodeBlob*   blob = CodeCache::find_blob(cbuf->insts()->start());
-  if (blob != NULL) {
+  if (blob != nullptr) {
     blob->use_remarks(cbuf->asm_remarks());
     blob->use_strings(cbuf->dbg_strings());
   }
@@ -89,6 +87,13 @@ void StubCodeGenerator::stub_prolog(StubCodeDesc* cdesc) {
 }
 
 void StubCodeGenerator::stub_epilog(StubCodeDesc* cdesc) {
+  LogTarget(Debug, stubs) lt;
+  if (lt.is_enabled()) {
+    LogStream ls(lt);
+    cdesc->print_on(&ls);
+    ls.cr();
+  }
+
   if (_print_code) {
 #ifndef PRODUCT
     // Find the assembly code remarks in the outer CodeBuffer.
