@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,6 +24,8 @@
 
 #include "precompiled.hpp"
 #include "classfile/classLoaderDataGraph.hpp"
+#include "classfile/stringTable.hpp"
+#include "classfile/symbolTable.hpp"
 #include "classfile/vmSymbols.hpp"
 #include "code/codeCache.hpp"
 #include "compiler/compileBroker.hpp"
@@ -103,6 +105,14 @@ void VM_CleanClassLoaderDataMetaspaces::doit() {
   ClassLoaderDataGraph::walk_metadata_and_clean_metaspaces();
 }
 
+void VM_RehashStringTable::doit() {
+  StringTable::rehash_table();
+}
+
+void VM_RehashSymbolTable::doit() {
+  SymbolTable::rehash_table();
+}
+
 VM_DeoptimizeFrame::VM_DeoptimizeFrame(JavaThread* thread, intptr_t* id, int reason) {
   _thread = thread;
   _id     = id;
@@ -265,13 +275,13 @@ void VM_ThreadDump::doit_epilogue() {
   }
 }
 
-// Hash table of void* to a list of ObjectMonitor* owned by the JavaThread.
+// Hash table of int64_t to a list of ObjectMonitor* owned by the JavaThread.
 // The JavaThread's owner key is either a JavaThread* or a stack lock
-// address in the JavaThread so we use "void*".
+// address in the JavaThread so we use "int64_t".
 //
 class ObjectMonitorsDump : public MonitorClosure, public ObjectMonitorsView {
  private:
-  static unsigned int ptr_hash(void* const& s1) {
+  static unsigned int ptr_hash(int64_t const& s1) {
     // 2654435761 = 2^32 * Phi (golden ratio)
     return (unsigned int)(((uint32_t)(uintptr_t)s1) * 2654435761u);
   }
@@ -284,24 +294,24 @@ class ObjectMonitorsDump : public MonitorClosure, public ObjectMonitorsView {
 
   // ResourceHashtable SIZE is specified at compile time so we
   // use 1031 which is the first prime after 1024.
-  typedef ResourceHashtable<void*, ObjectMonitorLinkedList*, 1031, AnyObj::C_HEAP, mtThread,
+  typedef ResourceHashtable<int64_t, ObjectMonitorLinkedList*, 1031, AnyObj::C_HEAP, mtThread,
                             &ObjectMonitorsDump::ptr_hash> PtrTable;
   PtrTable* _ptrs;
   size_t _key_count;
   size_t _om_count;
 
-  void add_list(void* key, ObjectMonitorLinkedList* list) {
+  void add_list(int64_t key, ObjectMonitorLinkedList* list) {
     _ptrs->put(key, list);
     _key_count++;
   }
 
-  ObjectMonitorLinkedList* get_list(void* key) {
+  ObjectMonitorLinkedList* get_list(int64_t key) {
     ObjectMonitorLinkedList** listpp = _ptrs->get(key);
     return (listpp == nullptr) ? nullptr : *listpp;
   }
 
   void add(ObjectMonitor* monitor) {
-    void* key = monitor->owner();
+    int64_t key = monitor->owner();
 
     ObjectMonitorLinkedList* list = get_list(key);
     if (list == nullptr) {
@@ -325,7 +335,7 @@ class ObjectMonitorsDump : public MonitorClosure, public ObjectMonitorsView {
   ~ObjectMonitorsDump() {
     class CleanupObjectMonitorsDump: StackObj {
      public:
-      bool do_entry(void*& key, ObjectMonitorLinkedList*& list) {
+      bool do_entry(int64_t& key, ObjectMonitorLinkedList*& list) {
         list->clear();  // clear the LinkListNodes
         delete list;    // then delete the LinkedList
         return true;
@@ -340,7 +350,7 @@ class ObjectMonitorsDump : public MonitorClosure, public ObjectMonitorsView {
   void do_monitor(ObjectMonitor* monitor) override {
     assert(monitor->has_owner(), "Expects only owned monitors");
 
-    if (monitor->is_owner_anonymous()) {
+    if (monitor->has_anonymous_owner()) {
       // There's no need to collect anonymous owned monitors
       // because the caller of this code is only interested
       // in JNI owned monitors.
@@ -358,7 +368,8 @@ class ObjectMonitorsDump : public MonitorClosure, public ObjectMonitorsView {
 
   // Implements the ObjectMonitorsView interface
   void visit(MonitorClosure* closure, JavaThread* thread) override {
-    ObjectMonitorLinkedList* list = get_list(thread);
+    int64_t key = ObjectMonitor::owner_id_from(thread);
+    ObjectMonitorLinkedList* list = get_list(key);
     LinkedListIterator<ObjectMonitor*> iter(list != nullptr ? list->head() : nullptr);
     while (!iter.is_empty()) {
       ObjectMonitor* monitor = *iter.next();
@@ -500,10 +511,9 @@ int VM_Exit::wait_for_threads_in_native_to_block() {
   // don't have to wait for user threads to be quiescent, but it's always
   // better to terminate VM when current thread is the only active thread, so
   // wait for user threads too. Numbers are in 10 milliseconds.
-  int max_wait_user_thread = 30;                  // at least 300 milliseconds
-  int max_wait_compiler_thread = 1000;            // at least 10 seconds
-
-  int max_wait = max_wait_compiler_thread;
+  int wait_time_per_attempt = 10;               // in milliseconds
+  int max_wait_attempts_user_thread = UserThreadWaitAttemptsAtExit;
+  int max_wait_attempts_compiler_thread = 1000; // at least 10 seconds
 
   int attempts = 0;
   JavaThreadIteratorWithHandle jtiwh;
@@ -536,16 +546,17 @@ int VM_Exit::wait_for_threads_in_native_to_block() {
 
     if (num_active == 0) {
        return 0;
-    } else if (attempts > max_wait) {
+    } else if (attempts >= max_wait_attempts_compiler_thread) {
        return num_active;
-    } else if (num_active_compiler_thread == 0 && attempts > max_wait_user_thread) {
+    } else if (num_active_compiler_thread == 0 &&
+               attempts >= max_wait_attempts_user_thread) {
        return num_active;
     }
 
     attempts++;
 
     MonitorLocker ml(&timer, Mutex::_no_safepoint_check_flag);
-    ml.wait(10);
+    ml.wait(wait_time_per_attempt);
   }
 }
 
@@ -600,12 +611,16 @@ void VM_Exit::doit() {
 
 
 void VM_Exit::wait_if_vm_exited() {
-  if (_vm_exited &&
-      Thread::current_or_null() != _shutdown_thread) {
-    // _vm_exited is set at safepoint, and the Threads_lock is never released
-    // so we will block here until the process dies.
-    Threads_lock->lock();
-    ShouldNotReachHere();
+  if (_vm_exited) {
+    // Need to check for an unattached thread as only attached threads
+    // can acquire the lock.
+    Thread* current = Thread::current_or_null();
+    if (current != nullptr && current != _shutdown_thread) {
+      // _vm_exited is set at safepoint, and the Threads_lock is never released
+      // so we will block here until the process dies.
+      Threads_lock->lock();
+      ShouldNotReachHere();
+    }
   }
 }
 
