@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "compiler/compileBroker.hpp"
 #include "gc/shared/collectedHeap.hpp"
 #include "jfr/jfrEvents.hpp"
@@ -35,6 +34,7 @@
 #include "oops/oop.inline.hpp"
 #include "oops/verifyOopClosure.hpp"
 #include "runtime/atomic.hpp"
+#include "runtime/cpuTimeCounters.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
 #include "runtime/javaThread.inline.hpp"
@@ -60,7 +60,7 @@ void VMOperationTimeoutTask::task() {
   if (is_armed()) {
     jlong delay = nanos_to_millis(os::javaTimeNanos() - _arm_time);
     if (delay > AbortVMOnVMOperationTimeoutDelay) {
-      fatal("%s VM operation took too long: " JLONG_FORMAT " ms elapsed since VM-op start (timeout: " INTX_FORMAT " ms)",
+      fatal("%s VM operation took too long: " JLONG_FORMAT " ms elapsed since VM-op start (timeout: %zd ms)",
             _vm_op_name, delay, AbortVMOnVMOperationTimeoutDelay);
     }
   }
@@ -87,7 +87,7 @@ void VMOperationTimeoutTask::disarm() {
   // VMOperationTimeoutTask might miss the arm-disarm window depending on
   // the scheduling.
   if (vm_op_duration > AbortVMOnVMOperationTimeoutDelay) {
-    fatal("%s VM operation took too long: completed in " JLONG_FORMAT " ms (timeout: " INTX_FORMAT " ms)",
+    fatal("%s VM operation took too long: completed in " JLONG_FORMAT " ms (timeout: %zd ms)",
           _vm_op_name, vm_op_duration, AbortVMOnVMOperationTimeoutDelay);
   }
   _vm_op_name = nullptr;
@@ -97,20 +97,20 @@ void VMOperationTimeoutTask::disarm() {
 // Implementation of VMThread stuff
 
 static VM_SafepointALot safepointALot_op;
-static VM_Cleanup       cleanup_op;
+static VM_ForceSafepoint no_op;
 
 bool              VMThread::_should_terminate   = false;
 bool              VMThread::_terminated         = false;
-Monitor*          VMThread::_terminate_lock     = NULL;
-VMThread*         VMThread::_vm_thread          = NULL;
-VM_Operation*     VMThread::_cur_vm_operation   = NULL;
-VM_Operation*     VMThread::_next_vm_operation  = &cleanup_op; // Prevent any thread from setting an operation until VM thread is ready.
-PerfCounter*      VMThread::_perf_accumulated_vm_operation_time = NULL;
-VMOperationTimeoutTask* VMThread::_timeout_task = NULL;
+Monitor*          VMThread::_terminate_lock     = nullptr;
+VMThread*         VMThread::_vm_thread          = nullptr;
+VM_Operation*     VMThread::_cur_vm_operation   = nullptr;
+VM_Operation*     VMThread::_next_vm_operation  = &no_op; // Prevent any thread from setting an operation until VM thread is ready.
+PerfCounter*      VMThread::_perf_accumulated_vm_operation_time = nullptr;
+VMOperationTimeoutTask* VMThread::_timeout_task = nullptr;
 
 
 void VMThread::create() {
-  assert(vm_thread() == NULL, "we can only allocate one VMThread");
+  assert(vm_thread() == nullptr, "we can only allocate one VMThread");
   _vm_thread = new VMThread();
 
   if (AbortVMOnVMOperationTimeout) {
@@ -125,7 +125,7 @@ void VMThread::create() {
     _timeout_task = new VMOperationTimeoutTask(interval);
     _timeout_task->enroll();
   } else {
-    assert(_timeout_task == NULL, "sanity");
+    assert(_timeout_task == nullptr, "sanity");
   }
 
   _terminate_lock = new Monitor(Mutex::nosafepoint, "VMThreadTerminate_lock");
@@ -136,6 +136,7 @@ void VMThread::create() {
     _perf_accumulated_vm_operation_time =
                  PerfDataManager::create_counter(SUN_THREADS, "vmOperationTime",
                                                  PerfData::U_Ticks, CHECK);
+    CPUTimeCounters::create_counter(CPUTimeGroups::CPUTimeType::vm);
   }
 }
 
@@ -144,7 +145,7 @@ VMThread::VMThread() : NamedThread(), _is_running(false) {
 }
 
 void VMThread::destroy() {
-  _vm_thread = NULL;      // VM thread is gone
+  _vm_thread = nullptr;      // VM thread is gone
 }
 
 static VM_Halt halt_op;
@@ -177,7 +178,7 @@ void VMThread::run() {
   // Note the intention to exit before safepointing.
   // 6295565  This has the effect of waiting for any large tty
   // outputs to finish.
-  if (xtty != NULL) {
+  if (xtty != nullptr) {
     ttyLocker ttyl;
     xtty->begin_elem("destroy_vm");
     xtty->stamp();
@@ -212,7 +213,7 @@ void VMThread::run() {
   // signal other threads that VM process is gone
   {
     // Note: we must have the _no_safepoint_check_flag. Mutex::lock() allows
-    // VM thread to enter any lock at Safepoint as long as its _owner is NULL.
+    // VM thread to enter any lock at Safepoint as long as its _owner is null.
     // If that happens after _terminate_lock->wait() has unset _owner
     // but before it actually drops the lock and waits, the notification below
     // may get lost and we will have a hang. To avoid this, we need to use
@@ -257,9 +258,8 @@ void VMThread::wait_for_vm_thread_exit() {
 }
 
 static void post_vm_operation_event(EventExecuteVMOperation* event, VM_Operation* op) {
-  assert(event != NULL, "invariant");
-  assert(event->should_commit(), "invariant");
-  assert(op != NULL, "invariant");
+  assert(event != nullptr, "invariant");
+  assert(op != nullptr, "invariant");
   const bool evaluate_at_safepoint = op->evaluate_at_safepoint();
   event->set_operation(op->type());
   event->set_safepoint(evaluate_at_safepoint);
@@ -289,6 +289,12 @@ void VMThread::evaluate_operation(VM_Operation* op) {
                      op->evaluate_at_safepoint() ? 0 : 1);
   }
 
+  if (UsePerfData && os::is_thread_cpu_time_supported()) {
+    assert(Thread::current() == this, "Must be called from VM thread");
+    // Update vm_thread_cpu_time after each VM operation.
+    ThreadTotalCPUTimeClosure tttc(CPUTimeGroups::CPUTimeType::vm);
+    tttc.do_thread(this);
+  }
 }
 
 class HandshakeALotClosure : public HandshakeClosure {
@@ -301,44 +307,28 @@ class HandshakeALotClosure : public HandshakeClosure {
   }
 };
 
-bool VMThread::handshake_alot() {
-  assert(_cur_vm_operation == NULL, "should not have an op yet");
-  assert(_next_vm_operation == NULL, "should not have an op yet");
-  if (!HandshakeALot) {
+bool VMThread::handshake_or_safepoint_alot() {
+  assert(_cur_vm_operation == nullptr, "should not have an op yet");
+  assert(_next_vm_operation == nullptr, "should not have an op yet");
+  if (!HandshakeALot && !SafepointALot) {
     return false;
   }
-  static jlong last_halot_ms = 0;
+  static jlong last_alot_ms = 0;
   jlong now_ms = nanos_to_millis(os::javaTimeNanos());
-  // If only HandshakeALot is set, but GuaranteedSafepointInterval is 0,
-  // we emit a handshake if it's been more than a second since the last one.
+  // If HandshakeALot or SafepointALot are set, but GuaranteedSafepointInterval is explicitly
+  // set to 0 on the command line, we emit the operation if it's been more than a second
+  // since the last one.
   jlong interval = GuaranteedSafepointInterval != 0 ? GuaranteedSafepointInterval : 1000;
-  jlong deadline_ms = interval + last_halot_ms;
+  jlong deadline_ms = interval + last_alot_ms;
   if (now_ms > deadline_ms) {
-    last_halot_ms = now_ms;
+    last_alot_ms = now_ms;
     return true;
   }
   return false;
 }
 
-void VMThread::setup_periodic_safepoint_if_needed() {
-  assert(_cur_vm_operation  == NULL, "Already have an op");
-  assert(_next_vm_operation == NULL, "Already have an op");
-  // Check for a cleanup before SafepointALot to keep stats correct.
-  long interval_ms = SafepointTracing::time_since_last_safepoint_ms();
-  bool max_time_exceeded = GuaranteedSafepointInterval != 0 &&
-                           (interval_ms >= GuaranteedSafepointInterval);
-  if (!max_time_exceeded) {
-    return;
-  }
-  if (SafepointSynchronize::is_cleanup_needed()) {
-    _next_vm_operation = &cleanup_op;
-  } else if (SafepointALot) {
-    _next_vm_operation = &safepointALot_op;
-  }
-}
-
 bool VMThread::set_next_operation(VM_Operation *op) {
-  if (_next_vm_operation != NULL) {
+  if (_next_vm_operation != nullptr) {
     return false;
   }
   log_debug(vmthread)("Adding VM operation: %s", op->name());
@@ -392,8 +382,8 @@ static void self_destruct_if_needed() {
 void VMThread::inner_execute(VM_Operation* op) {
   assert(Thread::current()->is_VM_thread(), "Must be the VM thread");
 
-  VM_Operation* prev_vm_operation = NULL;
-  if (_cur_vm_operation != NULL) {
+  VM_Operation* prev_vm_operation = nullptr;
+  if (_cur_vm_operation != nullptr) {
     // Check that the VM operation allows nested VM operation.
     // This is normally not the case, e.g., the compiler
     // does not allow nested scavenges or compiles.
@@ -408,12 +398,19 @@ void VMThread::inner_execute(VM_Operation* op) {
   _cur_vm_operation = op;
 
   HandleMark hm(VMThread::vm_thread());
-  EventMarkVMOperation em("Executing %sVM operation: %s", prev_vm_operation != NULL ? "nested " : "", op->name());
 
-  log_debug(vmthread)("Evaluating %s %s VM operation: %s",
-                       prev_vm_operation != NULL ? "nested" : "",
-                      _cur_vm_operation->evaluate_at_safepoint() ? "safepoint" : "non-safepoint",
-                      _cur_vm_operation->name());
+  const char* const cause = op->cause();
+  stringStream ss;
+  ss.print("Executing%s%s VM operation: %s",
+           prev_vm_operation != nullptr ? " nested" : "",
+           op->evaluate_at_safepoint() ? " safepoint" : " non-safepoint",
+           op->name());
+  if (cause != nullptr) {
+    ss.print(" (%s)", cause);
+  }
+
+  EventMarkVMOperation em("%s", ss.freeze());
+  log_debug(vmthread)("%s", ss.freeze());
 
   bool end_safepoint = false;
   bool has_timeout_task = (_timeout_task != nullptr);
@@ -444,33 +441,32 @@ void VMThread::wait_for_operation() {
 
   // Clear previous operation.
   // On first call this clears a dummy place-holder.
-  _next_vm_operation = NULL;
+  _next_vm_operation = nullptr;
   // Notify operation is done and notify a next operation can be installed.
   ml_op_lock.notify_all();
 
   while (!should_terminate()) {
     self_destruct_if_needed();
-    if (_next_vm_operation != NULL) {
+    if (_next_vm_operation != nullptr) {
       return;
     }
-    if (handshake_alot()) {
-      {
+    if (handshake_or_safepoint_alot()) {
+      if (HandshakeALot) {
         MutexUnlocker mul(VMOperation_lock);
         HandshakeALotClosure hal_cl;
         Handshake::execute(&hal_cl);
       }
       // When we unlocked above someone might have setup a new op.
-      if (_next_vm_operation != NULL) {
+      if (_next_vm_operation != nullptr) {
+        return;
+      }
+      if (SafepointALot) {
+        _next_vm_operation = &safepointALot_op;
         return;
       }
     }
-    assert(_next_vm_operation == NULL, "Must be");
-    assert(_cur_vm_operation  == NULL, "Must be");
-
-    setup_periodic_safepoint_if_needed();
-    if (_next_vm_operation != NULL) {
-      return;
-    }
+    assert(_next_vm_operation == nullptr, "Must be");
+    assert(_cur_vm_operation  == nullptr, "Must be");
 
     // We didn't find anything to execute, notify any waiter so they can install an op.
     ml_op_lock.notify_all();
@@ -479,20 +475,20 @@ void VMThread::wait_for_operation() {
 }
 
 void VMThread::loop() {
-  assert(_cur_vm_operation == NULL, "no current one should be executing");
+  assert(_cur_vm_operation == nullptr, "no current one should be executing");
 
   SafepointSynchronize::init(_vm_thread);
 
   // Need to set a calling thread for ops not passed
   // via the normal way.
-  cleanup_op.set_calling_thread(_vm_thread);
+  no_op.set_calling_thread(_vm_thread);
   safepointALot_op.set_calling_thread(_vm_thread);
 
   while (true) {
     if (should_terminate()) break;
     wait_for_operation();
     if (should_terminate()) break;
-    assert(_next_vm_operation != NULL, "Must have one");
+    assert(_next_vm_operation != nullptr, "Must have one");
     inner_execute(_next_vm_operation);
   }
 }
@@ -552,5 +548,5 @@ void VMThread::execute(VM_Operation* op) {
 }
 
 void VMThread::verify() {
-  oops_do(&VerifyOopClosure::verify_oop, NULL);
+  oops_do(&VerifyOopClosure::verify_oop, nullptr);
 }
