@@ -64,11 +64,6 @@ public:
     return "Print JVM version information.";
   }
   static const char* impact() { return "Low"; }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.util.PropertyPermission",
-                        "java.vm.version", "read"};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -80,11 +75,6 @@ public:
     return "Print the command line used to start this VM instance.";
   }
   static const char* impact() { return "Low"; }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS) {
     Arguments::print_on(_output);
   }
@@ -101,11 +91,6 @@ public:
     static const char* impact() {
       return "Low";
     }
-    static const JavaPermission permission() {
-      JavaPermission p = {"java.util.PropertyPermission",
-                          "*", "read"};
-      return p;
-    }
     virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -123,11 +108,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -146,11 +126,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "control", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -164,11 +139,6 @@ public:
   static const char* impact() {
     return "High";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -186,11 +156,6 @@ public:
     return "Load JVMTI native agent.";
   }
   static const char* impact() { return "Low"; }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "control", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 #endif // INCLUDE_JVMTI
@@ -208,11 +173,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -240,11 +200,6 @@ public:
     return "Print information about JVM environment and status.";
   }
   static const char* impact() { return "Low"; }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -284,11 +239,6 @@ public:
   static const char* impact() {
     return "Medium";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-      "monitor", nullptr};
-      return p;
-  }
 
   virtual void execute(DCmdSource source, TRAPS);
 };
@@ -303,11 +253,6 @@ public:
   static const char* impact() {
     return "Medium";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-      "monitor", nullptr};
-      return p;
-  }
 
   virtual void execute(DCmdSource source, TRAPS);
 };
@@ -334,11 +279,6 @@ public:
     return "High: Depends on Java heap size and content. "
            "Request a full GC unless the '-all' option is specified.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 #endif // INCLUDE_SERVICES
@@ -360,11 +300,6 @@ public:
   static const char* impact() {
     return "High: Depends on Java heap size and content.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -387,11 +322,6 @@ public:
   static const char* impact() {
       return "Medium: Depends on number of loaded classes.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -412,11 +342,6 @@ public:
   static const char* impact() {
     return "Medium: Pause time depends on number of loaded classes";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 #endif // INCLUDE_CDS
@@ -436,11 +361,6 @@ public:
   static const char* impact() {
     return "Medium: Depends on the number of threads.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -546,12 +466,6 @@ public:
     return "Print the management agent status.";
   }
 
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
-
   virtual void execute(DCmdSource source, TRAPS);
 
 };
@@ -568,11 +482,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -592,11 +501,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 #endif // LINUX
@@ -613,11 +517,6 @@ public:
   static const char* impact() {
     return "Medium";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -633,11 +532,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -659,11 +553,6 @@ public:
     return "Low: Depends on code heap size and content. "
            "Holds CodeCache_lock during analysis step, usually sub-second duration.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 //---<  END  >--- CodeHeap State Analytics.
@@ -680,11 +569,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -700,11 +584,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "control", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -723,11 +602,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "control", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -743,11 +617,6 @@ public:
   static const char* impact() {
     return "Low";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "control", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -809,11 +678,6 @@ public:
   static const char* impact() {
     return "Medium: Depends on Java content.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -832,11 +696,6 @@ public:
   static const char* impact() {
     return "Medium: Depends on Java content.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -855,11 +714,6 @@ public:
   static const char* impact() {
       return "Medium: Depends on Java content.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -878,39 +732,13 @@ public:
   static const char* impact() {
       return "Medium: Depends on number of loaded classes.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
-#if INCLUDE_JVMTI
-class DebugOnCmdStartDCmd : public DCmd {
-public:
-  DebugOnCmdStartDCmd(outputStream* output, bool heap) : DCmd(output, heap) {}
-  static const char* name() {
-    return "VM.start_java_debugging";
-  }
-  static const char* description() {
-    return "Starts up the Java debugging if the jdwp agentlib was enabled with the option onjcmd=y.";
-  }
-  static const char* impact() {
-    return "High: Switches the VM into Java debug mode.";
-  }
-  static const JavaPermission permission() {
-    JavaPermission p = { "java.lang.management.ManagementPermission", "control", nullptr };
-    return p;
-  }
-  virtual void execute(DCmdSource source, TRAPS);
-};
-#endif // INCLUDE_JVMTI
-
 class EventLogDCmd : public DCmdWithParser {
 protected:
   DCmdArgument<char*> _log;
-  DCmdArgument<char*> _max;
+  DCmdArgument<jlong> _max;
 public:
   static int num_arguments() { return 2; }
   EventLogDCmd(outputStream* output, bool heap);
@@ -923,11 +751,6 @@ public:
   static const char* impact() {
     return "Low: Depends on event log size. ";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -950,10 +773,33 @@ public:
   static const char* impact() {
     return "Medium: Depends on the number of threads.";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission", "monitor", nullptr};
-    return p;
+  virtual void execute(DCmdSource source, TRAPS);
+};
+
+class VThreadSchedulerDCmd : public DCmd {
+public:
+  VThreadSchedulerDCmd(outputStream* output, bool heap) : DCmd(output, heap) { }
+  static const char* name() {
+    return "Thread.vthread_scheduler";
   }
+  static const char* description() {
+    return "Print the virtual thread scheduler, and the delayed task schedulers that support "
+           "virtual threads doing timed operations.";
+  }
+  static const char* impact() { return "Low"; }
+  virtual void execute(DCmdSource source, TRAPS);
+};
+
+class VThreadPollersDCmd : public DCmd {
+public:
+  VThreadPollersDCmd(outputStream* output, bool heap) : DCmd(output, heap) { }
+  static const char* name() {
+    return "Thread.vthread_pollers";
+  }
+  static const char* description() {
+    return "Print the I/O pollers that support virtual threads doing blocking network I/O operations.";
+  }
+  static const char* impact() { return "Low"; }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -973,29 +819,19 @@ public:
   static const char* impact() {
     return "Medium: Pause time depends on number of compiled methods";
   }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "monitor", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
-#if defined(LINUX) || defined(_WIN64)
+#if defined(LINUX) || defined(_WIN64) || defined(__APPLE__)
 
 class SystemMapDCmd : public DCmd {
 public:
   SystemMapDCmd(outputStream* output, bool heap);
   static const char* name() { return "System.map"; }
   static const char* description() {
-    return "Prints an annotated process memory map of the VM process (linux and Windows only).";
+    return "Prints an annotated process memory map of the VM process (linux, Windows and MacOS only).";
   }
   static const char* impact() { return "Medium; can be high for very large java heaps."; }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "control", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
@@ -1006,17 +842,12 @@ public:
   SystemDumpMapDCmd(outputStream* output, bool heap);
   static const char* name() { return "System.dump_map"; }
   static const char* description() {
-    return "Dumps an annotated process memory map to an output file (linux and Windows only).";
+    return "Dumps an annotated process memory map to an output file (linux, Windows and MacOS only).";
   }
   static const char* impact() { return "Medium; can be high for very large java heaps."; }
-  static const JavaPermission permission() {
-    JavaPermission p = {"java.lang.management.ManagementPermission",
-                        "control", nullptr};
-    return p;
-  }
   virtual void execute(DCmdSource source, TRAPS);
 };
 
-#endif // LINUX or WINDOWS
+#endif // LINUX, WINDOWS or MACOS
 
 #endif // SHARE_SERVICES_DIAGNOSTICCOMMAND_HPP
