@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "jvm.h"
 #include "memory/oopFactory.hpp"
 #include "memory/resourceArea.hpp"
@@ -424,7 +423,7 @@ bool DCmd::reorder_help_cmd(CmdLine line, stringStream &updated_line) {
   args.print("%s", line.args_addr());
   char* rest = args.as_string();
   char* token = strtok_r(rest, " ", &rest);
-  while (token != NULL) {
+  while (token != nullptr) {
     if (strcmp(token, "-h") == 0 || strcmp(token, "--help") == 0 ||
         strcmp(token, "-help") == 0) {
       updated_line.print("%s", "help ");
@@ -587,7 +586,7 @@ GrowableArray<DCmdInfo*>* DCmdFactory::DCmdInfo_list(DCmdSource source ) {
     if (!factory->is_hidden() && (factory->export_flags() & source)) {
       array->append(new DCmdInfo(factory->name(),
                     factory->description(), factory->impact(),
-                    factory->permission(), factory->num_arguments(),
+                    factory->num_arguments(),
                     factory->is_enabled()));
     }
     factory = factory->next();
