@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "memory/allocation.inline.hpp"
 #include "memory/resourceArea.hpp"
 #include "runtime/javaThread.hpp"
@@ -61,6 +60,10 @@ GrowableArrayNestingCheck::GrowableArrayNestingCheck(bool on_resource_area) :
     _nesting(on_resource_area ? Thread::current()->resource_area()->nesting() : 0) {
 }
 
+GrowableArrayNestingCheck::GrowableArrayNestingCheck(Arena* arena) :
+    _nesting((arena->get_tag() == Arena::Tag::tag_ra) ? static_cast<ResourceArea*>(arena)->nesting() : 0) {
+}
+
 void GrowableArrayNestingCheck::on_resource_area_alloc() const {
   // Check for insidious allocation bug: if a GrowableArray overflows, the
   // grown array must be allocated under the same ResourceMark as the original.
@@ -70,6 +73,11 @@ void GrowableArrayNestingCheck::on_resource_area_alloc() const {
   }
 }
 
+void GrowableArrayNestingCheck::on_arena_alloc(Arena* arena) const {
+  if ((arena->get_tag() == Arena::Tag::tag_ra) && (_nesting != static_cast<ResourceArea*>(arena)->nesting())) {
+    fatal("allocation bug: GrowableArray is growing within nested ResourceMark");
+  }
+}
 void GrowableArrayMetadata::init_checks(const GrowableArrayBase* array) const {
   // Stack allocated arrays support all three element allocation locations
   if (array->allocated_on_stack_or_embedded()) {
@@ -89,4 +97,8 @@ void GrowableArrayMetadata::on_resource_area_alloc_check() const {
   _nesting_check.on_resource_area_alloc();
 }
 
+void GrowableArrayMetadata::on_arena_alloc_check() const {
+  _nesting_check.on_arena_alloc(arena());
+}
+
 #endif // ASSERT
