@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2003, 2023, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2017, 2020 SAP SE. All rights reserved.
- * Copyright (c) 2023, Red Hat, Inc. and/or its affiliates.
+ * Copyright (c) 2003, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2024 SAP SE. All rights reserved.
+ * Copyright (c) 2023, 2025, Red Hat, Inc. and/or its affiliates.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,7 +24,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "cds/metaspaceShared.hpp"
 #include "code/codeCache.hpp"
 #include "compiler/compilationFailureInfo.hpp"
@@ -67,6 +66,7 @@
 #include "utilities/events.hpp"
 #include "utilities/globalDefinitions.hpp"
 #include "utilities/macros.hpp"
+#include "utilities/nativeStackPrinter.hpp"
 #include "utilities/ostream.hpp"
 #include "utilities/vmError.hpp"
 #if INCLUDE_JFR
@@ -95,9 +95,9 @@ const char*       VMError::_message;
 char              VMError::_detail_msg[1024];
 Thread*           VMError::_thread;
 address           VMError::_pc;
-void*             VMError::_siginfo;
-void*             VMError::_context;
-bool              VMError::_print_native_stack_used = false;
+const void*       VMError::_siginfo;
+const void*       VMError::_context;
+bool              VMError::_print_stack_from_frame_used = false;
 const char*       VMError::_filename;
 int               VMError::_lineno;
 size_t            VMError::_size;
@@ -132,7 +132,7 @@ static const char* env_list[] = {
   // defined on Windows
   "OS", "PROCESSOR_IDENTIFIER", "_ALT_JAVA_HOME_DIR", "TMP", "TEMP",
 
-  (const char *)0
+  nullptr                       // End marker.
 };
 
 // A simple parser for -XX:OnError, usage:
@@ -215,8 +215,8 @@ void VMError::reattempt_test_hit_stack_limit(outputStream* st) {
     const size_t available_headroom   = stack_pointer - unguarded_stack_end;
     const size_t allocation_size      = available_headroom - _reattempt_required_stack_headroom + K;
 
-    st->print_cr("Current Stack Pointer: " PTR_FORMAT " alloca " SIZE_FORMAT
-                 " of " SIZE_FORMAT " bytes available unguarded stack space",
+    st->print_cr("Current Stack Pointer: " PTR_FORMAT " alloca %zu"
+                 " of %zu bytes available unguarded stack space",
                  p2i(stack_pointer), allocation_size, available_headroom);
 
     // Allocate byte blob on the stack. Make pointer volatile to avoid having
@@ -262,13 +262,13 @@ char* VMError::error_string(char* buf, int buflen) {
 
   if (signame) {
     jio_snprintf(buf, buflen,
-                 "%s (0x%x) at pc=" PTR_FORMAT ", pid=%d, tid=" UINTX_FORMAT,
+                 "%s (0x%x) at pc=" PTR_FORMAT ", pid=%d, tid=%zu",
                  signame, _id, _pc,
                  os::current_process_id(), os::current_thread_id());
   } else if (_filename != nullptr && _lineno > 0) {
     // skip directory names
     int n = jio_snprintf(buf, buflen,
-                         "Internal Error at %s:%d, pid=%d, tid=" UINTX_FORMAT,
+                         "Internal Error at %s:%d, pid=%d, tid=%zu",
                          get_filename_only(), _lineno,
                          os::current_process_id(), os::current_thread_id());
     if (n >= 0 && n < buflen && _message) {
@@ -282,7 +282,7 @@ char* VMError::error_string(char* buf, int buflen) {
     }
   } else {
     jio_snprintf(buf, buflen,
-                 "Internal Error (0x%x), pid=%d, tid=" UINTX_FORMAT,
+                 "Internal Error (0x%x), pid=%d, tid=%zu",
                  _id, os::current_process_id(), os::current_thread_id());
   }
 
@@ -418,75 +418,6 @@ static const char* find_code_name(address pc) {
   return nullptr;
 }
 
-/**
- * Gets the caller frame of `fr`.
- *
- * @returns an invalid frame (i.e. fr.pc() === 0) if the caller cannot be obtained
- */
-static frame next_frame(frame fr, Thread* t) {
-  // Compiled code may use EBP register on x86 so it looks like
-  // non-walkable C frame. Use frame.sender() for java frames.
-  frame invalid;
-  if (t != nullptr && t->is_Java_thread()) {
-    // Catch very first native frame by using stack address.
-    // For JavaThread stack_base and stack_size should be set.
-    if (!t->is_in_full_stack((address)(fr.real_fp() + 1))) {
-      return invalid;
-    }
-    if (fr.is_interpreted_frame() || (fr.cb() != nullptr && fr.cb()->frame_size() > 0)) {
-      RegisterMap map(JavaThread::cast(t),
-                      RegisterMap::UpdateMap::skip,
-                      RegisterMap::ProcessFrames::include,
-                      RegisterMap::WalkContinuation::skip); // No update
-      return fr.sender(&map);
-    } else {
-      // is_first_C_frame() does only simple checks for frame pointer,
-      // it will pass if java compiled code has a pointer in EBP.
-      if (os::is_first_C_frame(&fr)) return invalid;
-      return os::get_sender_for_C_frame(&fr);
-    }
-  } else {
-    if (os::is_first_C_frame(&fr)) return invalid;
-    return os::get_sender_for_C_frame(&fr);
-  }
-}
-
-void VMError::print_native_stack(outputStream* st, frame fr, Thread* t, bool print_source_info, int max_frames, char* buf, int buf_size) {
-
-  // see if it's a valid frame
-  if (fr.pc()) {
-    st->print_cr("Native frames: (J=compiled Java code, j=interpreted, Vv=VM code, C=native code)");
-    const int limit = max_frames == -1 ? StackPrintLimit : MIN2(max_frames, StackPrintLimit);
-    int count = 0;
-    while (count++ < limit) {
-      fr.print_on_error(st, buf, buf_size);
-      if (fr.pc()) { // print source file and line, if available
-        char filename[128];
-        int line_no;
-        if (count == 1 && _lineno != 0) {
-          // We have source information of the first frame for internal errors. There is no need to parse it from the symbols.
-          st->print("  (%s:%d)", get_filename_only(), _lineno);
-        } else if (print_source_info &&
-                   Decoder::get_source_info(fr.pc(), filename, sizeof(filename), &line_no, count != 1)) {
-          st->print("  (%s:%d)", filename, line_no);
-        }
-      }
-      st->cr();
-      fr = next_frame(fr, t);
-      if (fr.pc() == nullptr) {
-        break;
-      }
-    }
-
-    if (count > limit) {
-      st->print_cr("...<more frames>...");
-    }
-
-  } else {
-    st->print_cr("Native frames: <unavailable>");
-  }
-}
-
 static void print_oom_reasons(outputStream* st) {
   st->print_cr("# Possible reasons:");
   st->print_cr("#   The system is out of physical RAM or swap space");
@@ -532,7 +463,7 @@ static void print_oom_reasons(outputStream* st) {
   st->print_cr("# This output file may be truncated or incomplete.");
 }
 
-static void print_stack_location(outputStream* st, void* context, int& continuation) {
+static void print_stack_location(outputStream* st, const void* context, int& continuation) {
   const int number_of_stack_slots = 8;
 
   int i = continuation;
@@ -717,6 +648,12 @@ void VMError::report(outputStream* st, bool _verbose) {
   address lastpc = nullptr;
 
   BEGIN
+  if (MemTracker::enabled() &&
+      NmtVirtualMemory_lock != nullptr &&
+      NmtVirtualMemory_lock->owned_by_self()) {
+    // Manually unlock to avoid reentrancy due to mallocs in detailed mode.
+    NmtVirtualMemory_lock->unlock();
+  }
 
   STEP("printing fatal error message")
     st->print_cr("#");
@@ -836,7 +773,7 @@ void VMError::report(outputStream* st, bool _verbose) {
           st->print((_id == (int)OOM_MALLOC_ERROR) ? "(malloc) failed to allocate " :
                     (_id == (int)OOM_MMAP_ERROR)   ? "(mmap) failed to map " :
                                                     "(mprotect) failed to protect ");
-          jio_snprintf(buf, sizeof(buf), SIZE_FORMAT, _size);
+          jio_snprintf(buf, sizeof(buf), "%zu", _size);
           st->print("%s", buf);
           st->print(" bytes.");
           if (strlen(_detail_msg) > 0) {
@@ -895,7 +832,7 @@ void VMError::report(outputStream* st, bool _verbose) {
   STEP("printing current thread and pid")
     // process id, thread id
     st->print(", pid=%d", os::current_process_id());
-    st->print(", tid=" UINTX_FORMAT, os::current_thread_id());
+    st->print(", tid=%zu", os::current_thread_id());
     st->cr();
 
   STEP_IF("printing error message", should_report_bug(_id)) // already printed the message.
@@ -1002,13 +939,16 @@ void VMError::report(outputStream* st, bool _verbose) {
     if (fr.sp()) {
       st->print(",  sp=" PTR_FORMAT, p2i(fr.sp()));
       size_t free_stack_size = pointer_delta(fr.sp(), stack_bottom, 1024);
-      st->print(",  free space=" SIZE_FORMAT "k", free_stack_size);
+      st->print(",  free space=%zuk", free_stack_size);
     }
 
     st->cr();
 
   STEP_IF("printing native stack (with source info)", _verbose)
-    if (os::platform_print_native_stack(st, _context, buf, sizeof(buf), lastpc)) {
+
+    NativeStackPrinter nsp(_thread, _context, _filename != nullptr ? get_filename_only() : nullptr, _lineno);
+    if (nsp.print_stack(st, buf, sizeof(buf), lastpc,
+                        true /*print_source_info */, -1 /* max stack */)) {
       // We have printed the native stack in platform-specific code
       // Windows/x64 needs special handling.
       // Stack walking may get stuck. Try to find the calling code.
@@ -1019,19 +959,16 @@ void VMError::report(outputStream* st, bool _verbose) {
         }
       }
     } else {
-      frame fr = _context ? os::fetch_frame_from_context(_context)
-                          : os::current_frame();
-
-      print_native_stack(st, fr, _thread, true, -1, buf, sizeof(buf));
-      _print_native_stack_used = true;
+      _print_stack_from_frame_used = true; // frame-based native stack walk done
     }
 
   REATTEMPT_STEP_IF("retry printing native stack (no source info)", _verbose)
     st->cr();
     st->print_cr("Retrying call stack printing without source information...");
-    frame fr = _context ? os::fetch_frame_from_context(_context) : os::current_frame();
-    print_native_stack(st, fr, _thread, false, -1, buf, sizeof(buf));
-    _print_native_stack_used = true;
+    NativeStackPrinter nsp(_thread, _context, get_filename_only(), _lineno);
+    nsp.print_stack_from_frame(st, buf, sizeof(buf),
+                               false /*print_source_info */, -1 /* max stack */);
+    _print_stack_from_frame_used = true;
 
   STEP_IF("printing Java stack", _verbose && _thread && _thread->is_Java_thread())
     if (_verbose && _thread && _thread->is_Java_thread()) {
@@ -1141,7 +1078,7 @@ void VMError::report(outputStream* st, bool _verbose) {
       }
 
       // Scan the native stack
-      if (!_print_native_stack_used) {
+      if (!_print_stack_from_frame_used) {
         // Only try to print code of the crashing frame since
         // the native stack cannot be walked with next_frame.
         if (print_code(st, _thread, _pc, true, printed, printed_capacity)) {
@@ -1154,7 +1091,7 @@ void VMError::report(outputStream* st, bool _verbose) {
           if (print_code(st, _thread, fr.pc(), fr.pc() == _pc, printed, printed_capacity)) {
             printed_len++;
           }
-          fr = next_frame(fr, _thread);
+          fr = frame::next_frame(fr, _thread);
         }
       }
 
@@ -1223,7 +1160,7 @@ void VMError::report(outputStream* st, bool _verbose) {
 
   STEP_IF("printing owned locks on error", _verbose)
     // mutexes/monitors that currently have an owner
-    print_owned_locks_on_error(st);
+    Mutex::print_owned_locks_on_error(st);
     st->cr();
 
   STEP_IF("printing number of OutOfMemoryError and StackOverflow exceptions",
@@ -1275,6 +1212,12 @@ void VMError::report(outputStream* st, bool _verbose) {
     os::print_dll_info(st);
     st->cr();
 
+#if INCLUDE_JVMTI
+  STEP_IF("printing jvmti agent info", _verbose)
+    os::print_jvmti_agent_info(st);
+    st->cr();
+#endif
+
   STEP_IF("printing native decoder state", _verbose)
     Decoder::print_state_on(st);
     st->cr();
@@ -1453,6 +1396,11 @@ void VMError::print_vm_info(outputStream* st) {
   os::print_dll_info(st);
   st->cr();
 
+#if INCLUDE_JVMTI
+  os::print_jvmti_agent_info(st);
+  st->cr();
+#endif
+
   // STEP("printing VM options")
 
   // VM options
@@ -1583,8 +1531,8 @@ int VMError::prepare_log_file(const char* pattern, const char* default_pattern,
   return fd;
 }
 
-void VMError::report_and_die(Thread* thread, unsigned int sig, address pc, void* siginfo,
-                             void* context, const char* detail_fmt, ...)
+void VMError::report_and_die(Thread* thread, unsigned int sig, address pc, const void* siginfo,
+                             const void* context, const char* detail_fmt, ...)
 {
   va_list detail_args;
   va_start(detail_args, detail_fmt);
@@ -1592,7 +1540,7 @@ void VMError::report_and_die(Thread* thread, unsigned int sig, address pc, void*
   va_end(detail_args);
 }
 
-void VMError::report_and_die(Thread* thread, void* context, const char* filename, int lineno, const char* message,
+void VMError::report_and_die(Thread* thread, const void* context, const char* filename, int lineno, const char* message,
                              const char* detail_fmt, ...) {
   va_list detail_args;
   va_start(detail_args, detail_fmt);
@@ -1600,12 +1548,18 @@ void VMError::report_and_die(Thread* thread, void* context, const char* filename
   va_end(detail_args);
 }
 
-void VMError::report_and_die(Thread* thread, unsigned int sig, address pc, void* siginfo, void* context)
+void VMError::report_and_die(Thread* thread, unsigned int sig, address pc, const void* siginfo, const void* context)
 {
+  if (ExecutingUnitTests) {
+    // See TEST_VM_CRASH_SIGNAL gtest macro
+    char tmp[64];
+    fprintf(stderr, "signaled: %s", os::exception_name(sig, tmp, sizeof(tmp)));
+  }
+
   report_and_die(thread, sig, pc, siginfo, context, "%s", "");
 }
 
-void VMError::report_and_die(Thread* thread, void* context, const char* filename, int lineno, const char* message,
+void VMError::report_and_die(Thread* thread, const void* context, const char* filename, int lineno, const char* message,
                              const char* detail_fmt, va_list detail_args)
 {
   report_and_die(INTERNAL_ERROR, message, detail_fmt, detail_args, thread, nullptr, nullptr, context, filename, lineno, 0);
@@ -1617,7 +1571,7 @@ void VMError::report_and_die(Thread* thread, const char* filename, int lineno, s
 }
 
 void VMError::report_and_die(int id, const char* message, const char* detail_fmt, va_list detail_args,
-                             Thread* thread, address pc, void* siginfo, void* context, const char* filename,
+                             Thread* thread, address pc, const void* siginfo, const void* context, const char* filename,
                              int lineno, size_t size)
 {
   // A single scratch buffer to be used from here on.
@@ -1696,7 +1650,7 @@ void VMError::report_and_die(int id, const char* message, const char* detail_fmt
       ShowMessageBoxOnError = false;
     }
 
-    os::check_dump_limit(buffer, sizeof(buffer));
+    os::check_core_dump_prerequisites(buffer, sizeof(buffer));
 
     // reset signal handlers or exception filter; make sure recursive crashes
     // are handled properly.
@@ -1708,7 +1662,7 @@ void VMError::report_and_die(int id, const char* message, const char* detail_fmt
       if (!SuppressFatalErrorMessage) {
         char msgbuf[64];
         jio_snprintf(msgbuf, sizeof(msgbuf),
-                     "[thread " INTX_FORMAT " also had an error]",
+                     "[thread %zd also had an error]",
                      mytid);
         out.print_raw_cr(msgbuf);
       }
@@ -1786,12 +1740,12 @@ void VMError::report_and_die(int id, const char* message, const char* detail_fmt
               st->print_cr("]");
             }
             st->print("[stack: ");
-            frame fr = context ? os::fetch_frame_from_context(context) : os::current_frame();
+            NativeStackPrinter nsp(_thread, context, _filename != nullptr ? get_filename_only() : nullptr, _lineno);
             // Subsequent secondary errors build up stack; to avoid flooding the hs-err file with irrelevant
             // call stacks, limit the stack we print here (we are only interested in what happened before the
             // last assert/fault).
             const int max_stack_size = 15;
-            print_native_stack(st, fr, _thread, true, max_stack_size, tmp, sizeof(tmp));
+            nsp.print_stack_from_frame(st, tmp, sizeof(tmp), true /* print_source_info */, max_stack_size);
             st->print_cr("]");
           } // !recursed
           recursed = false; // Note: reset outside !recursed
@@ -1862,7 +1816,7 @@ void VMError::report_and_die(int id, const char* message, const char* detail_fmt
   if (DumpReplayDataOnError && _thread && _thread->is_Compiler_thread() && !skip_replay) {
     skip_replay = true;
     ciEnv* env = ciEnv::current();
-    if (env != nullptr) {
+    if (env != nullptr && env->task() != nullptr) {
       const bool overwrite = false; // We do not overwrite an existing replay file.
       int fd = prepare_log_file(ReplayDataFile, "replay_pid%p.log", overwrite, buffer, sizeof(buffer));
       if (fd != -1) {
