@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -50,18 +50,6 @@ import java.util.Date;
  * <a href="doc-files/net-properties.html#Proxies">Proxy settings</a> as well as
  * <a href="doc-files/net-properties.html#MiscHTTP"> various other settings</a>.
  * </P>
- * <p>
- * <b>Security permissions</b>
- * <p>
- * If a security manager is installed, and if a method is called which results in an
- * attempt to open a connection, the caller must possess either:
- * <ul><li>a "connect" {@link SocketPermission} to the host/port combination of the
- * destination URL or</li>
- * <li>a {@link URLPermission} that permits this request.</li>
- * </ul><p>
- * If automatic redirection is enabled, and this request is redirected to another
- * destination, then the caller must also have permission to connect to the
- * redirected host/URL.
  *
  * @see     java.net.HttpURLConnection#disconnect()
  * @since 1.1
@@ -277,9 +265,11 @@ public abstract class HttpURLConnection extends URLConnection {
      * <p>
      * This method must be called before the URLConnection is connected.
      *
-     * @param   chunklen The number of bytes to write in each chunk.
-     *          If chunklen is less than or equal to zero, a default
-     *          value will be used.
+     * @param   chunklen The number of bytes to be written in each chunk,
+     *          including a chunk size header as a hexadecimal string
+     *          (minimum of 1 byte), two CRLF's (4 bytes) and a minimum
+     *          payload length of 1 byte. If chunklen is less than or equal
+     *          to 5, a higher default value will be used.
      *
      * @throws  IllegalStateException if URLConnection is already connected
      *          or if a different streaming mode is already enabled.
@@ -371,29 +361,13 @@ public abstract class HttpURLConnection extends URLConnection {
 
     /**
      * Sets whether HTTP redirects  (requests with response code 3xx) should
-     * be automatically followed by this class.  True by default.  Applets
-     * cannot change this variable.
-     * <p>
-     * If there is a security manager, this method first calls
-     * the security manager's {@code checkSetFactory} method
-     * to ensure the operation is allowed.
-     * This could result in a SecurityException.
+     * be automatically followed by this class.  True by default.
      *
      * @param set a {@code boolean} indicating whether or not
      * to follow HTTP redirects.
-     * @throws     SecurityException  if a security manager exists and its
-     *             {@code checkSetFactory} method doesn't
-     *             allow the operation.
-     * @see        SecurityManager#checkSetFactory
      * @see #getFollowRedirects()
      */
     public static void setFollowRedirects(boolean set) {
-        @SuppressWarnings("removal")
-        SecurityManager sec = System.getSecurityManager();
-        if (sec != null) {
-            // seems to be the best check here...
-            sec.checkSetFactory();
-        }
         followRedirects = set;
     }
 
@@ -459,9 +433,6 @@ public abstract class HttpURLConnection extends URLConnection {
      * @param method the HTTP method
      * @throws    ProtocolException if the method cannot be reset or if
      *              the requested method isn't valid for HTTP.
-     * @throws    SecurityException if a security manager is set and the
-     *              method is "TRACE", but the "allowHttpTrace"
-     *              NetPermission is not granted.
      * @see #getRequestMethod()
      */
     public void setRequestMethod(String method) throws ProtocolException {
@@ -475,13 +446,6 @@ public abstract class HttpURLConnection extends URLConnection {
 
         for (int i = 0; i < methods.length; i++) {
             if (methods[i].equals(method)) {
-                if (method.equals("TRACE")) {
-                    @SuppressWarnings("removal")
-                    SecurityManager s = System.getSecurityManager();
-                    if (s != null) {
-                        s.checkPermission(new NetPermission("allowHttpTrace"));
-                    }
-                }
                 this.method = method;
                 return;
             }
@@ -597,16 +561,18 @@ public abstract class HttpURLConnection extends URLConnection {
     }
 
     @SuppressWarnings("deprecation")
-    public long getHeaderFieldDate(String name, long Default) {
+    public long getHeaderFieldDate(String name, long defaultValue) {
         String dateString = getHeaderField(name);
-        try {
-            if (dateString.indexOf("GMT") == -1) {
-                dateString = dateString+" GMT";
+        if (dateString != null) {
+            if (!dateString.contains("GMT")) {
+                dateString = dateString + " GMT";
+            }
+            try {
+                return Date.parse(dateString);
+            } catch (Exception e) {
             }
-            return Date.parse(dateString);
-        } catch (Exception e) {
         }
-        return Default;
+        return defaultValue;
     }
 
 
@@ -671,7 +637,7 @@ public abstract class HttpURLConnection extends URLConnection {
         return null;
     }
 
-    /**
+    /*
      * The response codes for HTTP, as of version 1.1.
      */
 
