@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,8 +32,6 @@ import java.io.FileReader;
 import java.io.InputStream;
 import java.io.IOException;
 import java.io.Serializable;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.util.concurrent.ConcurrentHashMap;
@@ -103,8 +101,8 @@ import sun.util.logging.PlatformLogger;
  * and/or minor unit are encountered, those entries are ignored and the remainder
  * of entries in file are processed.
  *
- * <p>
- * It is recommended to use {@link java.math.BigDecimal} class while dealing
+ * @apiNote
+ * It is recommended to use the {@link java.math.BigDecimal} class while dealing
  * with {@code Currency} or monetary values as it provides better handling of floating
  * point numbers and their operations.
  *
@@ -213,63 +211,57 @@ public final class Currency implements Serializable {
         initStatic();
     }
 
-    @SuppressWarnings("removal")
     private static void initStatic() {
-        AccessController.doPrivileged(new PrivilegedAction<>() {
-            @Override
-            public Void run() {
-                try {
-                    try (InputStream in = getClass().getResourceAsStream("/java/util/currency.data")) {
-                        if (in == null) {
-                            throw new InternalError("Currency data not found");
-                        }
-                        DataInputStream dis = new DataInputStream(new BufferedInputStream(in));
-                        if (dis.readInt() != MAGIC_NUMBER) {
-                            throw new InternalError("Currency data is possibly corrupted");
-                        }
-                        formatVersion = dis.readInt();
-                        if (formatVersion != VALID_FORMAT_VERSION) {
-                            throw new InternalError("Currency data format is incorrect");
-                        }
-                        dataVersion = dis.readInt();
-                        mainTable = readIntArray(dis, A_TO_Z * A_TO_Z);
-                        int scCount = dis.readInt();
-                        specialCasesList = readSpecialCases(dis, scCount);
-                        int ocCount = dis.readInt();
-                        otherCurrenciesList = readOtherCurrencies(dis, ocCount);
-                    }
-                } catch (IOException e) {
-                    throw new InternalError(e);
-                }
 
-                // look for the properties file for overrides
-                String propsFile = System.getProperty("java.util.currency.data");
-                if (propsFile == null) {
-                    propsFile = StaticProperty.javaHome() + File.separator + "lib" +
-                        File.separator + "currency.properties";
+        try {
+            try (InputStream in = Currency.class.getResourceAsStream("/java/util/currency.data")) {
+                if (in == null) {
+                    throw new InternalError("Currency data not found");
                 }
-                try {
-                    File propFile = new File(propsFile);
-                    if (propFile.exists()) {
-                        Properties props = new Properties();
-                        try (FileReader fr = new FileReader(propFile)) {
-                            props.load(fr);
-                        }
-                        Pattern propertiesPattern =
-                                Pattern.compile("([A-Z]{3})\\s*,\\s*(\\d{3})\\s*,\\s*" +
-                                        "(\\d+)\\s*,?\\s*(\\d{4}-\\d{2}-\\d{2}T\\d{2}:" +
-                                        "\\d{2}:\\d{2})?");
-                        List<CurrencyProperty> currencyEntries
-                                = getValidCurrencyData(props, propertiesPattern);
-                        currencyEntries.forEach(Currency::replaceCurrencyData);
-                    }
-                } catch (IOException e) {
-                    CurrencyProperty.info("currency.properties is ignored"
-                            + " because of an IOException", e);
+                DataInputStream dis = new DataInputStream(new BufferedInputStream(in));
+                if (dis.readInt() != MAGIC_NUMBER) {
+                    throw new InternalError("Currency data is possibly corrupted");
                 }
-                return null;
+                formatVersion = dis.readInt();
+                if (formatVersion != VALID_FORMAT_VERSION) {
+                    throw new InternalError("Currency data format is incorrect");
+                }
+                dataVersion = dis.readInt();
+                mainTable = readIntArray(dis, A_TO_Z * A_TO_Z);
+                int scCount = dis.readInt();
+                specialCasesList = readSpecialCases(dis, scCount);
+                int ocCount = dis.readInt();
+                otherCurrenciesList = readOtherCurrencies(dis, ocCount);
             }
-        });
+        } catch (IOException e) {
+            throw new InternalError(e);
+        }
+
+        // look for the properties file for overrides
+        String propsFile = System.getProperty("java.util.currency.data");
+        if (propsFile == null) {
+            propsFile = StaticProperty.javaHome() + File.separator + "lib" +
+                File.separator + "currency.properties";
+        }
+        try {
+            File propFile = new File(propsFile);
+            if (propFile.exists()) {
+                Properties props = new Properties();
+                try (FileReader fr = new FileReader(propFile)) {
+                    props.load(fr);
+                }
+                Pattern propertiesPattern =
+                        Pattern.compile("([A-Z]{3})\\s*,\\s*(\\d{3})\\s*,\\s*" +
+                                "(\\d+)\\s*,?\\s*(\\d{4}-\\d{2}-\\d{2}T\\d{2}:" +
+                                "\\d{2}:\\d{2})?");
+                List<CurrencyProperty> currencyEntries
+                        = getValidCurrencyData(props, propertiesPattern);
+                currencyEntries.forEach(Currency::replaceCurrencyData);
+            }
+        } catch (IOException e) {
+            CurrencyProperty.info("currency.properties is ignored"
+                    + " because of an IOException", e);
+        }
     }
 
     /**
@@ -319,8 +311,8 @@ public final class Currency implements Serializable {
             // or in the list of other currencies.
             boolean found = false;
             if (currencyCode.length() != 3) {
-                throw new IllegalArgumentException(
-                        "The input currency code: \"%s\" must have a length of 3 characters".formatted(currencyCode));
+                throw new IllegalArgumentException("The input currency code must " +
+                        "have a length of 3 characters");
             }
             char char1 = currencyCode.charAt(0);
             char char2 = currencyCode.charAt(1);
@@ -343,8 +335,8 @@ public final class Currency implements Serializable {
             if (!found) {
                 OtherCurrencyEntry ocEntry = OtherCurrencyEntry.findEntry(currencyCode);
                 if (ocEntry == null) {
-                    throw new IllegalArgumentException(
-                            "The input currency code: \"%s\" is not a valid ISO 4217 code".formatted(currencyCode));
+                    throw new IllegalArgumentException("The input currency code" +
+                            " is not a valid ISO 4217 code");
                 }
                 defaultFractionDigits = ocEntry.fraction;
                 numericCode = ocEntry.numericCode;
@@ -399,8 +391,8 @@ public final class Currency implements Serializable {
         String country = CalendarDataUtility.findRegionOverride(locale).getCountry();
 
         if (country == null || !country.matches("^[a-zA-Z]{2}$")) {
-            throw new IllegalArgumentException(
-                    "The country of the input locale: \"%s\" is not a valid ISO 3166 country code".formatted(locale));
+            throw new IllegalArgumentException("The country of the input locale" +
+                    " is not a valid ISO 3166 country code");
         }
 
         char char1 = country.charAt(0);
@@ -417,8 +409,8 @@ public final class Currency implements Serializable {
         } else {
             // special cases
             if (tableEntry == INVALID_COUNTRY_ENTRY) {
-                throw new IllegalArgumentException(
-                        "The country of the input locale: \"%s\" is not a valid ISO 3166 country code".formatted(locale));
+                throw new IllegalArgumentException("The country of the input locale" +
+                        " is not a valid ISO 3166 country code");
             }
             if (tableEntry == COUNTRY_WITHOUT_CURRENCY_ENTRY) {
                 return null;
@@ -452,8 +444,8 @@ public final class Currency implements Serializable {
     public static Set<Currency> getAvailableCurrencies() {
         synchronized(Currency.class) {
             if (available == null) {
+                var sysTime = System.currentTimeMillis();
                 available = new HashSet<>(256);
-
                 // Add simple currencies first
                 for (char c1 = 'A'; c1 <= 'Z'; c1 ++) {
                     for (char c2 = 'A'; c2 <= 'Z'; c2 ++) {
@@ -475,7 +467,7 @@ public final class Currency implements Serializable {
                             SpecialCaseEntry scEntry = specialCasesList.get(index);
 
                             if (scEntry.cutOverTime == Long.MAX_VALUE
-                                    || System.currentTimeMillis() < scEntry.cutOverTime) {
+                                    || sysTime < scEntry.cutOverTime) {
                                 available.add(getInstance(scEntry.oldCurrency,
                                         scEntry.oldCurrencyFraction,
                                         scEntry.oldCurrencyNumericCode));
@@ -494,10 +486,7 @@ public final class Currency implements Serializable {
                 }
             }
         }
-
-        @SuppressWarnings("unchecked")
-        Set<Currency> result = (Set<Currency>) available.clone();
-        return result;
+        return new HashSet<>(available);
     }
 
     /**
@@ -521,7 +510,8 @@ public final class Currency implements Serializable {
      * {@linkplain Locale##def_locale_extension Unicode extensions},
      * the symbol returned from this method reflects
      * the value specified with that extension.
-     * <p>
+     *
+     * @implSpec
      * This is equivalent to calling
      * {@link #getSymbol(Locale)
      *     getSymbol(Locale.getDefault(Locale.Category.DISPLAY))}.
@@ -621,7 +611,8 @@ public final class Currency implements Serializable {
      * the default {@link Locale.Category#DISPLAY DISPLAY} locale.
      * If there is no suitable display name found
      * for the default locale, the ISO 4217 currency code is returned.
-     * <p>
+     *
+     * @implSpec
      * This is equivalent to calling
      * {@link #getDisplayName(Locale)
      *     getDisplayName(Locale.getDefault(Locale.Category.DISPLAY))}.
@@ -683,8 +674,8 @@ public final class Currency implements Serializable {
      */
     private static int getMainTableEntry(char char1, char char2) {
         if (char1 < 'A' || char1 > 'Z' || char2 < 'A' || char2 > 'Z') {
-            throw new IllegalArgumentException(
-                    "The country code: \"%c%c\" is not a valid ISO 3166 code".formatted(char1, char2));
+            throw new IllegalArgumentException("The country code is not a " +
+                    "valid ISO 3166 code");
         }
         return mainTable[(char1 - 'A') * A_TO_Z + (char2 - 'A')];
     }
@@ -695,8 +686,8 @@ public final class Currency implements Serializable {
      */
     private static void setMainTableEntry(char char1, char char2, int entry) {
         if (char1 < 'A' || char1 > 'Z' || char2 < 'A' || char2 > 'Z') {
-            throw new IllegalArgumentException(
-                    "The country code: \"%c%c\" is not a valid ISO 3166 code".formatted(char1, char2));
+            throw new IllegalArgumentException("The country code is not a " +
+                    "valid ISO 3166 code");
         }
         mainTable[(char1 - 'A') * A_TO_Z + (char2 - 'A')] = entry;
     }
