@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -34,32 +34,15 @@ import java.io.IOException;
  * This class implements an output stream filter for compressing data in
  * the "deflate" compression format. It is also used as the basis for other
  * types of compression filters, such as GZIPOutputStream.
+ * <p> Unless otherwise noted, passing a {@code null} argument to a constructor
+ * or method in this class will cause a {@link NullPointerException} to be
+ * thrown.
  *
  * @see         Deflater
  * @author      David Connelly
  * @since 1.1
  */
 public class DeflaterOutputStream extends FilterOutputStream {
-
-    /*
-     * The default size of the output buffer
-     */
-    static final int DEFAULT_BUF_SIZE = 512;
-
-    /*
-     * When calling Deflater.deflate() with Deflater.SYNC_FLUSH or Deflater.FULL_FLUSH,
-     * the callers are expected to ensure that the size of the buffer is greater than 6.
-     * This expectation comes from the underlying zlib library which in its zlib.h
-     * states:
-     * "If deflate returns with avail_out == 0, this function must be called again
-     * with the same value of the flush parameter and more output space (updated
-     * avail_out), until the flush is complete (deflate returns with non-zero
-     * avail_out). In the case of a Z_FULL_FLUSH or Z_SYNC_FLUSH, make sure that
-     * avail_out is greater than six when the flush marker begins, in order to avoid
-     * repeated flush markers upon calling deflate() again when avail_out == 0."
-     */
-    private static final int SYNC_FLUSH_MIN_BUF_SIZE = 7;
-
     /**
      * Compressor for this stream.
      */
@@ -143,7 +126,7 @@ public class DeflaterOutputStream extends FilterOutputStream {
     public DeflaterOutputStream(OutputStream out,
                                 Deflater def,
                                 boolean syncFlush) {
-        this(out, def, DEFAULT_BUF_SIZE, syncFlush);
+        this(out, def, 512, syncFlush);
     }
 
 
@@ -158,7 +141,7 @@ public class DeflaterOutputStream extends FilterOutputStream {
      * @param def the compressor ("deflater")
      */
     public DeflaterOutputStream(OutputStream out, Deflater def) {
-        this(out, def, DEFAULT_BUF_SIZE, false);
+        this(out, def, 512, false);
     }
 
     boolean usesDefaultDeflater = false;
@@ -178,7 +161,7 @@ public class DeflaterOutputStream extends FilterOutputStream {
      * @since 1.7
      */
     public DeflaterOutputStream(OutputStream out, boolean syncFlush) {
-        this(out, out != null ? new Deflater() : null, DEFAULT_BUF_SIZE, syncFlush);
+        this(out, out != null ? new Deflater() : null, 512, syncFlush);
         usesDefaultDeflater = true;
     }
 
@@ -201,7 +184,6 @@ public class DeflaterOutputStream extends FilterOutputStream {
      * @param b the byte to be written
      * @throws    IOException if an I/O error has occurred
      */
-    @Override
     public void write(int b) throws IOException {
         byte[] buf = new byte[1];
         buf[0] = (byte)(b & 0xff);
@@ -216,7 +198,6 @@ public class DeflaterOutputStream extends FilterOutputStream {
      * @param len the length of the data
      * @throws    IOException if an I/O error has occurred
      */
-    @Override
     public void write(byte[] b, int off, int len) throws IOException {
         if (def.finished()) {
             throw new IOException("write beyond end of stream");
@@ -260,17 +241,32 @@ public class DeflaterOutputStream extends FilterOutputStream {
      * underlying stream.
      * @throws    IOException if an I/O error has occurred
      */
-    @Override
     public void close() throws IOException {
         if (!closed) {
+            closed = true;
+            IOException finishException = null;
             try {
                 finish();
+            } catch (IOException ioe){
+                finishException = ioe;
+                throw ioe;
             } finally {
-                if (usesDefaultDeflater)
+                if (usesDefaultDeflater) {
                     def.end();
+                }
+                if (finishException == null) {
+                    out.close();
+                } else {
+                    try {
+                        out.close();
+                    } catch (IOException ioe) {
+                        if (finishException != ioe) {
+                            ioe.addSuppressed(finishException);
+                        }
+                        throw ioe;
+                    }
+                }
             }
-            out.close();
-            closed = true;
         }
     }
 
@@ -300,20 +296,13 @@ public class DeflaterOutputStream extends FilterOutputStream {
      *
      * @since 1.7
      */
-    @Override
     public void flush() throws IOException {
         if (syncFlush && !def.finished()) {
             int len = 0;
-            // For SYNC_FLUSH, the Deflater.deflate() expects the callers
-            // to use a buffer whose length is greater than 6 to avoid
-            // flush marker (5 bytes) being repeatedly output to the output buffer
-            // every time it is invoked.
-            final byte[] flushBuf = buf.length < SYNC_FLUSH_MIN_BUF_SIZE
-                    ? new byte[DEFAULT_BUF_SIZE]
-                    : buf;
-            while ((len = def.deflate(flushBuf, 0, flushBuf.length, Deflater.SYNC_FLUSH)) > 0) {
-                out.write(flushBuf, 0, len);
-                if (len < flushBuf.length)
+            while ((len = def.deflate(buf, 0, buf.length, Deflater.SYNC_FLUSH)) > 0)
+            {
+                out.write(buf, 0, len);
+                if (len < buf.length)
                     break;
             }
         }
