@@ -24,16 +24,22 @@
  */
 package jdk.internal.classfile.impl;
 
-import java.lang.classfile.*;
-import java.lang.classfile.constantpool.*;
-
+import java.lang.classfile.Annotation;
+import java.lang.classfile.AnnotationElement;
+import java.lang.classfile.AnnotationValue;
+import java.lang.classfile.constantpool.DoubleEntry;
+import java.lang.classfile.constantpool.FloatEntry;
+import java.lang.classfile.constantpool.IntegerEntry;
+import java.lang.classfile.constantpool.LongEntry;
+import java.lang.classfile.constantpool.Utf8Entry;
 import java.util.List;
 
-import static java.lang.classfile.ClassFile.*;
+import static java.util.Objects.requireNonNull;
 
 public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elements)
         implements Annotation {
     public AnnotationImpl {
+        requireNonNull(className);
         elements = List.copyOf(elements);
     }
 
@@ -52,6 +58,11 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record AnnotationElementImpl(Utf8Entry name,
                                         AnnotationValue value)
             implements AnnotationElement {
+        public AnnotationElementImpl {
+            requireNonNull(name);
+            requireNonNull(value);
+        }
+
         @Override
         public String toString() {
             return name + "=" + value;
@@ -61,8 +72,8 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record OfStringImpl(Utf8Entry constant)
             implements AnnotationValue.OfString {
         @Override
-        public char tag() {
-            return AEV_STRING;
+        public int tag() {
+            return TAG_STRING;
         }
 
         @Override
@@ -74,8 +85,8 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record OfDoubleImpl(DoubleEntry constant)
             implements AnnotationValue.OfDouble {
         @Override
-        public char tag() {
-            return AEV_DOUBLE;
+        public int tag() {
+            return TAG_DOUBLE;
         }
 
         @Override
@@ -87,8 +98,8 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record OfFloatImpl(FloatEntry constant)
             implements AnnotationValue.OfFloat {
         @Override
-        public char tag() {
-            return AEV_FLOAT;
+        public int tag() {
+            return TAG_FLOAT;
         }
 
         @Override
@@ -100,8 +111,8 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record OfLongImpl(LongEntry constant)
             implements AnnotationValue.OfLong {
         @Override
-        public char tag() {
-            return AEV_LONG;
+        public int tag() {
+            return TAG_LONG;
         }
 
         @Override
@@ -113,8 +124,8 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record OfIntImpl(IntegerEntry constant)
             implements AnnotationValue.OfInt {
         @Override
-        public char tag() {
-            return AEV_INT;
+        public int tag() {
+            return TAG_INT;
         }
 
         @Override
@@ -126,8 +137,8 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record OfShortImpl(IntegerEntry constant)
             implements AnnotationValue.OfShort {
         @Override
-        public char tag() {
-            return AEV_SHORT;
+        public int tag() {
+            return TAG_SHORT;
         }
 
         @Override
@@ -139,8 +150,8 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record OfCharImpl(IntegerEntry constant)
             implements AnnotationValue.OfChar {
         @Override
-        public char tag() {
-            return AEV_CHAR;
+        public int tag() {
+            return TAG_CHAR;
         }
 
         @Override
@@ -152,8 +163,8 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record OfByteImpl(IntegerEntry constant)
             implements AnnotationValue.OfByte {
         @Override
-        public char tag() {
-            return AEV_BYTE;
+        public int tag() {
+            return TAG_BYTE;
         }
 
         @Override
@@ -165,8 +176,8 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
     public record OfBooleanImpl(IntegerEntry constant)
             implements AnnotationValue.OfBoolean {
         @Override
-        public char tag() {
-            return AEV_BOOLEAN;
+        public int tag() {
+            return TAG_BOOLEAN;
         }
 
         @Override
@@ -182,32 +193,32 @@ public record AnnotationImpl(Utf8Entry className, List<AnnotationElement> elemen
         }
 
         @Override
-        public char tag() {
-            return AEV_ARRAY;
+        public int tag() {
+            return TAG_ARRAY;
         }
     }
 
     public record OfEnumImpl(Utf8Entry className, Utf8Entry constantName)
             implements AnnotationValue.OfEnum {
         @Override
-        public char tag() {
-            return AEV_ENUM;
+        public int tag() {
+            return TAG_ENUM;
         }
     }
 
     public record OfAnnotationImpl(Annotation annotation)
             implements AnnotationValue.OfAnnotation {
         @Override
-        public char tag() {
-            return AEV_ANNOTATION;
+        public int tag() {
+            return TAG_ANNOTATION;
         }
     }
 
     public record OfClassImpl(Utf8Entry className)
             implements AnnotationValue.OfClass {
         @Override
-        public char tag() {
-            return AEV_CLASS;
+        public int tag() {
+            return TAG_CLASS;
         }
     }
 }
