@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,13 +30,10 @@ import jdk.internal.util.StaticProperty;
 
 import java.io.File;
 import java.io.IOException;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.util.ArrayDeque;
 import java.util.Deque;
 import java.util.function.BiFunction;
 import java.util.function.Function;
-import java.util.Objects;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
@@ -62,8 +59,6 @@ public final class NativeLibraries {
     // unless specified
     private final Class<?> caller;      // may be null
     private final boolean searchJavaLibraryPath;
-    // loading JNI native libraries
-    private final boolean isJNI;
 
     /**
      * Creates a NativeLibraries instance for loading JNI native libraries
@@ -81,48 +76,14 @@ public final class NativeLibraries {
      * @see <a href="${docroot}/specs/jni/invocation.html##library-and-version-management">
      *     JNI Specification: Library and Version Management</a>
      */
-    public static NativeLibraries jniNativeLibraries(ClassLoader loader) {
-        return new NativeLibraries(loader);
+    public static NativeLibraries newInstance(ClassLoader loader) {
+        return new NativeLibraries(loader, loader != null ? null : NativeLibraries.class, loader != null);
     }
 
-    /**
-     * Creates a raw NativeLibraries instance that has the following properties:
-     * 1. Native libraries loaded in this raw NativeLibraries instance are
-     *    not JNI native libraries.  Hence JNI_OnLoad and JNI_OnUnload will
-     *    be ignored.  No support for linking of native method.
-     * 2. Native libraries not auto-unloaded.  They may be explicitly unloaded
-     *    via NativeLibraries::unload.
-     * 3. No relationship with class loaders.
-     *
-     * This static factory method is restricted for JDK trusted class use.
-     */
-    public static NativeLibraries rawNativeLibraries(Class<?> trustedCaller,
-                                                     boolean searchJavaLibraryPath) {
-        return new NativeLibraries(trustedCaller, searchJavaLibraryPath);
-    }
-
-    private NativeLibraries(ClassLoader loader) {
-        // for null loader, default the caller to this class and
-        // do not search java.library.path
+    private NativeLibraries(ClassLoader loader, Class<?> caller, boolean searchJavaLibraryPath) {
         this.loader = loader;
-        this.caller = loader != null ? null : NativeLibraries.class;
-        this.searchJavaLibraryPath = loader != null ? true : false;
-        this.isJNI = true;
-    }
-
-    /*
-     * Constructs a NativeLibraries instance of no relationship with class loaders
-     * and disabled auto unloading.
-     */
-    private NativeLibraries(Class<?> caller, boolean searchJavaLibraryPath) {
-        Objects.requireNonNull(caller);
-        if (!VM.isSystemDomainLoader(caller.getClassLoader())) {
-            throw new IllegalArgumentException("must be JDK trusted class");
-        }
-        this.loader = caller.getClassLoader();
         this.caller = caller;
         this.searchJavaLibraryPath = searchJavaLibraryPath;
-        this.isJNI = false;
     }
 
     /*
@@ -151,25 +112,17 @@ public final class NativeLibraries {
      * @param file the path of the native library
      * @throws UnsatisfiedLinkError if any error in loading the native library
      */
-    @SuppressWarnings("removal")
     public NativeLibrary loadLibrary(Class<?> fromClass, File file) {
         // Check to see if we're attempting to access a static library
         String name = findBuiltinLib(file.getName());
         boolean isBuiltin = (name != null);
         if (!isBuiltin) {
-            name = AccessController.doPrivileged(new PrivilegedAction<>() {
-                    public String run() {
-                        try {
-                            if (loadLibraryOnlyIfPresent && !file.exists()) {
-                                return null;
-                            }
-                            return file.getCanonicalPath();
-                        } catch (IOException e) {
-                            return null;
-                        }
-                    }
-                });
-            if (name == null) {
+            try {
+                if (loadLibraryOnlyIfPresent && !file.exists()) {
+                    return null;
+                }
+                name = file.getCanonicalPath();
+            } catch (IOException e) {
                 return null;
             }
         }
@@ -227,7 +180,7 @@ public final class NativeLibraries {
                 }
             }
 
-            NativeLibraryImpl lib = new NativeLibraryImpl(fromClass, name, isBuiltin, isJNI);
+            NativeLibraryImpl lib = new NativeLibraryImpl(fromClass, name, isBuiltin);
             // load the native library
             NativeLibraryContext.push(lib);
             try {
@@ -237,8 +190,7 @@ public final class NativeLibraries {
                 // auto unloading is only supported for JNI native libraries
                 // loaded by custom class loaders that can be unloaded.
                 // built-in class loaders are never unloaded.
-                boolean autoUnload = isJNI && !VM.isSystemDomainLoader(loader)
-                        && loader != ClassLoaders.appClassLoader();
+                boolean autoUnload = !VM.isSystemDomainLoader(loader) && loader != ClassLoaders.appClassLoader();
                 if (autoUnload) {
                     // register the loaded native library for auto unloading
                     // when the class loader is reclaimed, all native libraries
@@ -269,8 +221,6 @@ public final class NativeLibraries {
      */
     public NativeLibrary loadLibrary(String name) {
         assert name.indexOf(File.separatorChar) < 0;
-        assert caller != null;
-
         return loadLibrary(caller, name);
     }
 
@@ -293,29 +243,6 @@ public final class NativeLibraries {
         return lib;
     }
 
-    /**
-     * Unloads the given native library
-     *
-     * @param lib native library
-     */
-    public void unload(NativeLibrary lib) {
-        if (isJNI) {
-            throw new UnsupportedOperationException("explicit unloading cannot be used with auto unloading");
-        }
-        Objects.requireNonNull(lib);
-        acquireNativeLibraryLock(lib.name());
-        try {
-            NativeLibraryImpl nl = libraries.remove(lib.name());
-            if (nl != lib) {
-                throw new IllegalArgumentException(lib.name() + " not loaded by this NativeLibraries instance");
-            }
-            // unload the native library and also remove from the global name registry
-            nl.unloader().run();
-        } finally {
-            releaseNativeLibraryLock(lib.name());
-        }
-    }
-
     private NativeLibrary findFromPaths(String[] paths, Class<?> fromClass, String name) {
         for (String path : paths) {
             File libfile = new File(path, System.mapLibraryName(name));
@@ -344,7 +271,7 @@ public final class NativeLibraries {
      * the VM when it loads the library, and used by the VM to pass the correct
      * version of JNI to the native methods.
      */
-    static class NativeLibraryImpl implements NativeLibrary {
+    static class NativeLibraryImpl extends NativeLibrary {
         // the class from which the library is loaded, also indicates
         // the loader this native library belongs.
         final Class<?> fromClass;
@@ -353,21 +280,16 @@ public final class NativeLibraries {
         final String name;
         // Indicates if the native library is linked into the VM
         final boolean isBuiltin;
-        // Indicate if this is JNI native library
-        final boolean isJNI;
 
         // opaque handle to native library, used in native code.
         long handle;
         // the version of JNI environment the native library requires.
         int jniVersion;
 
-        NativeLibraryImpl(Class<?> fromClass, String name, boolean isBuiltin, boolean isJNI) {
-            assert !isBuiltin || isJNI : "a builtin native library must be JNI library";
-
+        NativeLibraryImpl(Class<?> fromClass, String name, boolean isBuiltin) {
             this.fromClass = fromClass;
             this.name = name;
             this.isBuiltin = isBuiltin;
-            this.isJNI = isJNI;
         }
 
         @Override
@@ -377,11 +299,15 @@ public final class NativeLibraries {
 
         @Override
         public long find(String name) {
-            return findEntry0(this, name);
+            return findEntry0(handle, name);
         }
 
-        Runnable unloader() {
-            return new Unloader(name, handle, isBuiltin, isJNI);
+        /*
+         * Unloader::run method is invoked to unload the native library
+         * when this class loader becomes phantom reachable.
+         */
+        private Runnable unloader() {
+            return new Unloader(name, handle, isBuiltin);
         }
 
         /*
@@ -392,7 +318,23 @@ public final class NativeLibraries {
                 throw new InternalError("Native library " + name + " has been loaded");
             }
 
-            return load(this, name, isBuiltin, isJNI, loadLibraryOnlyIfPresent);
+            return load(this, name, isBuiltin, throwExceptionIfFail());
+        }
+
+        private boolean throwExceptionIfFail() {
+            if (loadLibraryOnlyIfPresent) return true;
+
+            // If the file exists but fails to load, UnsatisfiedLinkException thrown by the VM
+            // will include the error message from dlopen to provide diagnostic information
+            File file = new File(name);
+            return file.exists();
+        }
+
+        /*
+         * Close this native library.
+         */
+        void close() {
+            unload(name, isBuiltin, handle);
         }
     }
 
@@ -404,15 +346,13 @@ public final class NativeLibraries {
         // This represents the context when a native library is unloaded
         // and getFromClass() will return null,
         static final NativeLibraryImpl UNLOADER =
-                new NativeLibraryImpl(null, "dummy", false, false);
+                new NativeLibraryImpl(null, "dummy", false);
 
         final String name;
         final long handle;
         final boolean isBuiltin;
-        final boolean isJNI;
 
-        Unloader(String name, long handle, boolean isBuiltin, boolean isJNI) {
-            assert !isBuiltin || isJNI : "a builtin native library must be JNI library";
+        Unloader(String name, long handle, boolean isBuiltin) {
             if (handle == 0) {
                 throw new IllegalArgumentException(
                         "Invalid handle for native library " + name);
@@ -421,7 +361,6 @@ public final class NativeLibraries {
             this.name = name;
             this.handle = handle;
             this.isBuiltin = isBuiltin;
-            this.isJNI = isJNI;
         }
 
         @Override
@@ -434,7 +373,7 @@ public final class NativeLibraries {
                 }
                 NativeLibraryContext.push(UNLOADER);
                 try {
-                    unload(name, isBuiltin, isJNI, handle);
+                    unload(name, isBuiltin, handle);
                 } finally {
                     NativeLibraryContext.pop();
                 }
@@ -576,12 +515,22 @@ public final class NativeLibraries {
         return NativeLibraryContext.peek().fromClass;
     }
 
-    // JNI FindClass expects the caller class if invoked from JNI_OnLoad
-    // and JNI_OnUnload is NativeLibrary class
+    /*
+     * Return true if the given library is successfully loaded.
+     * If the given library cannot be loaded for any reason,
+     * if throwExceptionIfFail is false, then this method returns false;
+     * otherwise, UnsatisfiedLinkError will be thrown.
+     *
+     * JNI FindClass expects the caller class if invoked from JNI_OnLoad
+     * and JNI_OnUnload is NativeLibrary class.
+     */
     private static native boolean load(NativeLibraryImpl impl, String name,
-                                       boolean isBuiltin, boolean isJNI,
+                                       boolean isBuiltin,
                                        boolean throwExceptionIfFail);
-    private static native void unload(String name, boolean isBuiltin, boolean isJNI, long handle);
+    /*
+     * Unload the named library.  JNI_OnUnload, if present, will be invoked
+     * before the native library is unloaded.
+     */
+    private static native void unload(String name, boolean isBuiltin, long handle);
     private static native String findBuiltinLib(String name);
-    private static native long findEntry0(NativeLibraryImpl lib, String name);
 }
