@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,7 +30,6 @@ import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.io.PrintStream;
 import java.io.BufferedOutputStream;
-import java.net.InetAddress;
 import java.net.Socket;
 import java.net.SocketException;
 import java.net.URL;
@@ -46,9 +45,8 @@ import java.util.StringTokenizer;
 
 import javax.net.ssl.*;
 import sun.net.www.http.HttpClient;
-import sun.net.www.protocol.http.AuthenticatorKeys;
+import sun.net.www.protocol.http.AuthCacheImpl;
 import sun.net.www.protocol.http.HttpURLConnection;
-import sun.security.action.*;
 
 import sun.security.util.HostnameChecker;
 import sun.security.ssl.SSLSocketImpl;
@@ -138,10 +136,8 @@ final class HttpsClient extends HttpClient
         //
         // If ciphers are assigned, sort them into an array.
         //
-        String ciphers [];
-        String cipherString =
-                GetPropertyAction.privilegedGetProperty("https.cipherSuites");
-
+        String[] ciphers;
+        String cipherString = System.getProperty("https.cipherSuites");
         if (cipherString == null || cipherString.isEmpty()) {
             ciphers = null;
         } else {
@@ -162,10 +158,8 @@ final class HttpsClient extends HttpClient
         //
         // If protocols are assigned, sort them into an array.
         //
-        String protocols [];
-        String protocolString =
-                GetPropertyAction.privilegedGetProperty("https.protocols");
-
+        String[] protocols;
+        String protocolString = System.getProperty("https.protocols");
         if (protocolString == null || protocolString.isEmpty()) {
             protocols = null;
         } else {
@@ -183,65 +177,12 @@ final class HttpsClient extends HttpClient
         return protocols;
     }
 
-    private String getUserAgent() {
-        String userAgent =
-                GetPropertyAction.privilegedGetProperty("https.agent");
-        if (userAgent == null || userAgent.isEmpty()) {
-            userAgent = "JSSE";
-        }
-        return userAgent;
-    }
-
     // CONSTRUCTOR, FACTORY
 
-
     /**
-     * Create an HTTPS client URL.  Traffic will be tunneled through any
-     * intermediate nodes rather than proxied, so that confidentiality
-     * of data exchanged can be preserved.  However, note that all the
-     * anonymous SSL flavors are subject to "person-in-the-middle"
-     * attacks against confidentiality.  If you enable use of those
-     * flavors, you may be giving up the protection you get through
-     * SSL tunneling.
-     *
-     * Use New to get new HttpsClient. This constructor is meant to be
-     * used only by New method. New properly checks for URL spoofing.
-     *
-     * @param url https URL with which a connection must be established
-     */
-    private HttpsClient(SSLSocketFactory sf, URL url)
-    throws IOException
-    {
-        // HttpClient-level proxying is always disabled,
-        // because we override doConnect to do tunneling instead.
-        this(sf, url, (String)null, -1);
-    }
-
-    /**
-     *  Create an HTTPS client URL.  Traffic will be tunneled through
-     * the specified proxy server.
-     */
-    HttpsClient(SSLSocketFactory sf, URL url, String proxyHost, int proxyPort)
-        throws IOException {
-        this(sf, url, proxyHost, proxyPort, -1);
-    }
-
-    /**
-     *  Create an HTTPS client URL.  Traffic will be tunneled through
+     * Create an HTTPS client URL. Traffic will be tunneled through
      * the specified proxy server, with a connect timeout
      */
-    HttpsClient(SSLSocketFactory sf, URL url, String proxyHost, int proxyPort,
-                int connectTimeout)
-        throws IOException {
-        this(sf, url,
-             (proxyHost == null? null:
-                HttpClient.newHttpProxy(proxyHost, proxyPort, "https")),
-                connectTimeout);
-    }
-
-    /**
-     *  Same as previous constructor except using a Proxy
-     */
     HttpsClient(SSLSocketFactory sf, URL url, Proxy proxy,
                 int connectTimeout)
         throws IOException {
@@ -268,37 +209,6 @@ final class HttpsClient extends HttpClient
     // This code largely ripped off from HttpClient.New, and
     // it uses the same keepalive cache.
 
-    static HttpClient New(SSLSocketFactory sf, URL url, HostnameVerifier hv,
-                          HttpURLConnection httpuc)
-            throws IOException {
-        return HttpsClient.New(sf, url, hv, true, httpuc);
-    }
-
-    /** See HttpClient for the model for this method. */
-    static HttpClient New(SSLSocketFactory sf, URL url,
-            HostnameVerifier hv, boolean useCache,
-            HttpURLConnection httpuc) throws IOException {
-        return HttpsClient.New(sf, url, hv, (String)null, -1, useCache, httpuc);
-    }
-
-    /**
-     * Get a HTTPS client to the URL.  Traffic will be tunneled through
-     * the specified proxy server.
-     */
-    static HttpClient New(SSLSocketFactory sf, URL url, HostnameVerifier hv,
-                           String proxyHost, int proxyPort,
-                           HttpURLConnection httpuc) throws IOException {
-        return HttpsClient.New(sf, url, hv, proxyHost, proxyPort, true, httpuc);
-    }
-
-    static HttpClient New(SSLSocketFactory sf, URL url, HostnameVerifier hv,
-                           String proxyHost, int proxyPort, boolean useCache,
-                           HttpURLConnection httpuc)
-        throws IOException {
-        return HttpsClient.New(sf, url, hv, proxyHost, proxyPort, useCache, -1,
-                               httpuc);
-    }
-
     static HttpClient New(SSLSocketFactory sf, URL url, HostnameVerifier hv,
                           String proxyHost, int proxyPort, boolean useCache,
                           int connectTimeout, HttpURLConnection httpuc)
@@ -329,17 +239,16 @@ final class HttpsClient extends HttpClient
             ret = (HttpsClient) kac.get(url, sf);
             if (ret != null && httpuc != null &&
                 httpuc.streaming() &&
-                httpuc.getRequestMethod() == "POST") {
+                "POST".equals(httpuc.getRequestMethod())) {
                 if (!ret.available())
                     ret = null;
             }
 
             if (ret != null) {
-                String ak = httpuc == null ? AuthenticatorKeys.DEFAULT
-                     : httpuc.getAuthenticatorKey();
+                AuthCacheImpl ak = httpuc == null ? null : httpuc.getAuthCache();
                 boolean compatible = ((ret.proxy != null && ret.proxy.equals(p)) ||
                     (ret.proxy == null && p == Proxy.NO_PROXY))
-                     && Objects.equals(ret.getAuthenticatorKey(), ak);
+                     && Objects.equals(ret.getAuthCache(), ak);
 
                 if (compatible) {
                     ret.lock();
@@ -377,18 +286,9 @@ final class HttpsClient extends HttpClient
         if (ret == null) {
             ret = new HttpsClient(sf, url, p, connectTimeout);
             if (httpuc != null) {
-                ret.authenticatorKey = httpuc.getAuthenticatorKey();
+                ret.authcache = httpuc.getAuthCache();
             }
         } else {
-            @SuppressWarnings("removal")
-            SecurityManager security = System.getSecurityManager();
-            if (security != null) {
-                if (ret.proxy == Proxy.NO_PROXY || ret.proxy == null) {
-                    security.checkConnect(InetAddress.getByName(url.getHost()).getHostAddress(), url.getPort());
-                } else {
-                    security.checkConnect(url.getHost(), url.getPort());
-                }
-            }
             ret.url = url;
         }
         ret.setHostnameVerifier(hv);
@@ -396,22 +296,17 @@ final class HttpsClient extends HttpClient
         return ret;
     }
 
-    // METHODS
-    void setHostnameVerifier(HostnameVerifier hv) {
+    private void setHostnameVerifier(HostnameVerifier hv) {
         this.hv = hv;
     }
 
-    void setSSLSocketFactory(SSLSocketFactory sf) {
+    private void setSSLSocketFactory(SSLSocketFactory sf) {
         sslSocketFactory = sf;
     }
 
-    SSLSocketFactory getSSLSocketFactory() {
-        return sslSocketFactory;
-    }
-
     /**
      * The following method, createSocket, is defined in NetworkClient
-     * and overridden here so that the socket facroty is used to create
+     * and overridden here so that the socket factory is used to create
      * new sockets.
      */
     @Override
@@ -434,6 +329,15 @@ final class HttpsClient extends HttpClient
         }
     }
 
+    @Override
+    public void closeServer() {
+        try {
+            // SSLSocket.close may block up to timeout. Make sure it's short.
+            serverSocket.setSoTimeout(1);
+        } catch (Exception e) {}
+        super.closeServer();
+    }
+
 
     @Override
     public boolean needsTunneling() {
@@ -563,13 +467,13 @@ final class HttpsClient extends HttpClient
                 if (isDefaultHostnameVerifier) {
                     // If the HNV is the default from HttpsURLConnection, we
                     // will do the spoof checks in SSLSocket.
-                    SSLParameters paramaters = s.getSSLParameters();
-                    paramaters.setEndpointIdentificationAlgorithm("HTTPS");
+                    SSLParameters parameters = s.getSSLParameters();
+                    parameters.setEndpointIdentificationAlgorithm("HTTPS");
                     // host has been set previously for SSLSocketImpl
                     if (!(s instanceof SSLSocketImpl)) {
-                        paramaters.setServerNames(List.of(new SNIHostName(host)));
+                        parameters.setServerNames(List.of(new SNIHostName(host)));
                     }
-                    s.setSSLParameters(paramaters);
+                    s.setSSLParameters(parameters);
 
                     needToCheckSpoofing = false;
                 }
@@ -645,7 +549,7 @@ final class HttpsClient extends HttpClient
             // ignore
         }
 
-        if ((cipher != null) && (cipher.indexOf("_anon_") != -1)) {
+        if ((cipher != null) && (cipher.contains("_anon_"))) {
             return;
         } else if ((hostnameVerifier != null) &&
                    (hostnameVerifier.verify(host, session))) {
@@ -661,12 +565,17 @@ final class HttpsClient extends HttpClient
 
     @Override
     protected void putInKeepAliveCache() {
-        if (inCache) {
-            assert false : "Duplicate put to keep alive cache";
-            return;
+        lock();
+        try {
+            if (inCache) {
+                assert false : "Duplicate put to keep alive cache";
+                return;
+            }
+            inCache = true;
+            kac.put(url, sslSocketFactory, this);
+        } finally {
+            unlock();
         }
-        inCache = true;
-        kac.put(url, sslSocketFactory, this);
     }
 
     /*
