@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,7 +28,6 @@
 #include <netinet/tcp.h> // defines TCP_NODELAY
 #include <stdlib.h>
 #include <string.h>
-#include <sys/ioctl.h>
 #include <sys/time.h>
 
 #if defined(__linux__)
@@ -51,16 +50,13 @@
 #define IPV6_FLOWINFO_SEND      33
 #endif
 
-#define RESTARTABLE(_cmd, _result) do { \
-    do { \
-        _result = _cmd; \
-    } while((_result == -1) && (errno == EINTR)); \
-} while(0)
-
-int NET_SocketAvailable(int s, int *pbytes) {
-    int result;
-    RESTARTABLE(ioctl(s, FIONREAD, pbytes), result);
-    return result;
+/* Perform platform specific initialization.
+ * Returns 0 on success, non-0 on failure */
+int
+NET_PlatformInit()
+{
+    // Not needed on unix
+    return 0;
 }
 
 void
@@ -123,18 +119,7 @@ jint  IPv6_supported()
          */
         return JNI_FALSE;
     }
-
-    /*
-     * If fd 0 is a socket it means we may have been launched from inetd or
-     * xinetd. If it's a socket then check the family - if it's an
-     * IPv4 socket then we need to disable IPv6.
-     */
-    if (getsockname(0, &sa.sa, &sa_len) == 0) {
-        if (sa.sa.sa_family == AF_INET) {
-            close(fd);
-            return JNI_FALSE;
-        }
-    }
+    close(fd);
 
     /**
      * Linux - check if any interface has an IPv6 address.
@@ -147,13 +132,11 @@ jint  IPv6_supported()
         char *bufP;
 
         if (fP == NULL) {
-            close(fd);
             return JNI_FALSE;
         }
         bufP = fgets(buf, sizeof(buf), fP);
         fclose(fP);
         if (bufP == NULL) {
-            close(fd);
             return JNI_FALSE;
         }
     }
@@ -164,7 +147,6 @@ jint  IPv6_supported()
      *  we should also check if the APIs are available.
      */
     ipv6_fn = JVM_FindLibraryEntry(RTLD_DEFAULT, "inet_pton");
-    close(fd);
     if (ipv6_fn == NULL ) {
         return JNI_FALSE;
     } else {
@@ -211,7 +193,7 @@ void NET_ThrowUnknownHostExceptionWithGaiError(JNIEnv *env,
     buf = (char *) malloc(size);
     if (buf) {
         jstring s;
-        sprintf(buf, format, hostname, error_string);
+        snprintf(buf, size, format, hostname, error_string);
         s = JNU_NewStringPlatform(env, buf);
         if (s != NULL) {
             jobject x = JNU_NewObjectByName(env,
@@ -224,21 +206,6 @@ void NET_ThrowUnknownHostExceptionWithGaiError(JNIEnv *env,
     }
 }
 
-#if defined(_AIX)
-
-/* Initialize stubs for blocking I/O workarounds (see src/solaris/native/java/net/linux_close.c) */
-extern void aix_close_init();
-
-void platformInit () {
-    aix_close_init();
-}
-
-#else
-
-void platformInit () {}
-
-#endif
-
 JNIEXPORT jint JNICALL
 NET_EnableFastTcpLoopback(int fd) {
     return 0;
@@ -356,72 +323,6 @@ int NET_IsZeroAddr(jbyte* caddr) {
     return 1;
 }
 
-/*
- * Map the Java level socket option to the platform specific
- * level and option name.
- */
-int
-NET_MapSocketOption(jint cmd, int *level, int *optname) {
-    static struct {
-        jint cmd;
-        int level;
-        int optname;
-    } const opts[] = {
-        { java_net_SocketOptions_TCP_NODELAY,           IPPROTO_TCP,    TCP_NODELAY },
-        { java_net_SocketOptions_SO_OOBINLINE,          SOL_SOCKET,     SO_OOBINLINE },
-        { java_net_SocketOptions_SO_LINGER,             SOL_SOCKET,     SO_LINGER },
-        { java_net_SocketOptions_SO_SNDBUF,             SOL_SOCKET,     SO_SNDBUF },
-        { java_net_SocketOptions_SO_RCVBUF,             SOL_SOCKET,     SO_RCVBUF },
-        { java_net_SocketOptions_SO_KEEPALIVE,          SOL_SOCKET,     SO_KEEPALIVE },
-        { java_net_SocketOptions_SO_REUSEADDR,          SOL_SOCKET,     SO_REUSEADDR },
-        { java_net_SocketOptions_SO_REUSEPORT,          SOL_SOCKET,     SO_REUSEPORT },
-        { java_net_SocketOptions_SO_BROADCAST,          SOL_SOCKET,     SO_BROADCAST },
-        { java_net_SocketOptions_IP_TOS,                IPPROTO_IP,     IP_TOS },
-        { java_net_SocketOptions_IP_MULTICAST_IF,       IPPROTO_IP,     IP_MULTICAST_IF },
-        { java_net_SocketOptions_IP_MULTICAST_IF2,      IPPROTO_IP,     IP_MULTICAST_IF },
-        { java_net_SocketOptions_IP_MULTICAST_LOOP,     IPPROTO_IP,     IP_MULTICAST_LOOP },
-    };
-
-    int i;
-
-    if (ipv6_available()) {
-        switch (cmd) {
-            // Different multicast options if IPv6 is enabled
-            case java_net_SocketOptions_IP_MULTICAST_IF:
-            case java_net_SocketOptions_IP_MULTICAST_IF2:
-                *level = IPPROTO_IPV6;
-                *optname = IPV6_MULTICAST_IF;
-                return 0;
-
-            case java_net_SocketOptions_IP_MULTICAST_LOOP:
-                *level = IPPROTO_IPV6;
-                *optname = IPV6_MULTICAST_LOOP;
-                return 0;
-#if defined(MACOSX)
-            // Map IP_TOS request to IPV6_TCLASS
-            case java_net_SocketOptions_IP_TOS:
-                *level = IPPROTO_IPV6;
-                *optname = IPV6_TCLASS;
-                return 0;
-#endif
-        }
-    }
-
-    /*
-     * Map the Java level option to the native level
-     */
-    for (i=0; i<(int)(sizeof(opts) / sizeof(opts[0])); i++) {
-        if (cmd == opts[i].cmd) {
-            *level = opts[i].level;
-            *optname = opts[i].optname;
-            return 0;
-        }
-    }
-
-    /* not found */
-    return -1;
-}
-
 /*
  * Wrapper for getsockopt system routine - does any necessary
  * pre/post processing to deal with OS specific oddities :-
@@ -644,7 +545,9 @@ NET_SetSockOpt(int fd, int level, int  opt, const void *arg,
         }
 
         if (sotype == SOCK_DGRAM) {
-            setsockopt(fd, level, SO_REUSEPORT, arg, len);
+            if (setsockopt(fd, level, SO_REUSEPORT, arg, len) < 0) {
+                return -1;
+            }
         }
     }
 #endif
@@ -715,7 +618,7 @@ NET_Wait(JNIEnv *env, jint fd, jint flags, jint timeout)
           pfd.events |= POLLOUT;
 
         errno = 0;
-        read_rv = NET_Poll(&pfd, 1, nanoTimeout / NET_NSEC_PER_MSEC);
+        read_rv = poll(&pfd, 1, nanoTimeout / NET_NSEC_PER_MSEC);
 
         newNanoTime = JVM_NanoTime(env, 0);
         nanoTimeout -= (newNanoTime - prevNanoTime);
