@@ -184,6 +184,7 @@ static jfieldID attrs_st_birthtime_sec;
 #if defined(__linux__) // Linux has nsec granularity if supported
 static jfieldID attrs_st_birthtime_nsec;
 #endif
+static jfieldID attrs_birthtime_available;
 
 static jfieldID attrs_f_frsize;
 static jfieldID attrs_f_blocks;
@@ -203,9 +204,6 @@ typedef int openat_func(int, const char *, int, ...);
 typedef int fstatat_func(int, const char *, struct stat *, int);
 typedef int unlinkat_func(int, const char*, int);
 typedef int renameat_func(int, const char*, int, const char*);
-typedef int futimesat_func(int, const char *, const struct timeval *);
-typedef int futimens_func(int, const struct timespec *);
-typedef int lutimes_func(const char *, const struct timeval *);
 typedef DIR* fdopendir_func(int);
 #if defined(__linux__)
 typedef int statx_func(int dirfd, const char *restrict pathname, int flags,
@@ -216,9 +214,6 @@ static openat_func* my_openat_func = NULL;
 static fstatat_func* my_fstatat_func = NULL;
 static unlinkat_func* my_unlinkat_func = NULL;
 static renameat_func* my_renameat_func = NULL;
-static futimesat_func* my_futimesat_func = NULL;
-static futimens_func* my_futimens_func = NULL;
-static lutimes_func* my_lutimes_func = NULL;
 static fdopendir_func* my_fdopendir_func = NULL;
 #if defined(__linux__)
 static statx_func* my_statx_func = NULL;
@@ -255,23 +250,6 @@ static int statx_wrapper(int dirfd, const char *restrict pathname, int flags,
 }
 #endif
 
-#if defined(__linux__) && defined(__arm__)
-/**
- * Lookup functions with time_t parameter. Try to use 64 bit symbol
- * if sizeof(time_t) exceeds 32 bit.
- */
-static void* lookup_time_t_function(const char* symbol, const char* symbol64) {
-    void *func_ptr = NULL;
-    if (sizeof(time_t) > 4) {
-        func_ptr = dlsym(RTLD_DEFAULT, symbol64);
-    }
-    if (func_ptr == NULL) {
-        return dlsym(RTLD_DEFAULT, symbol);
-    }
-    return func_ptr;
-}
-#endif
-
 /**
  * Call this to throw an internal UnixException when a system/library
  * call fails
@@ -332,6 +310,8 @@ Java_sun_nio_fs_UnixNativeDispatcher_init(JNIEnv* env, jclass this)
     attrs_st_birthtime_nsec = (*env)->GetFieldID(env, clazz, "st_birthtime_nsec", "J");
     CHECK_NULL_RETURN(attrs_st_birthtime_nsec, 0);
 #endif
+    attrs_birthtime_available = (*env)->GetFieldID(env, clazz, "birthtime_available", "Z");
+    CHECK_NULL_RETURN(attrs_birthtime_available, 0);
 
     clazz = (*env)->FindClass(env, "sun/nio/fs/UnixFileStoreAttributes");
     CHECK_NULL_RETURN(clazz, 0);
@@ -360,8 +340,8 @@ Java_sun_nio_fs_UnixNativeDispatcher_init(JNIEnv* env, jclass this)
     /* system calls that might not be available at run time */
 
 #if defined(_ALLBSD_SOURCE)
-    my_openat_func = (openat_func*)dlsym(RTLD_DEFAULT, "openat");
-    my_fstatat_func = (fstatat_func*)dlsym(RTLD_DEFAULT, "fstatat");
+    my_openat_func = (openat_func*) openat;
+    my_fstatat_func = (fstatat_func*) fstatat;
 #else
     // Make sure we link to the 64-bit version of the functions
     my_openat_func = (openat_func*) dlsym(RTLD_DEFAULT, "openat64");
@@ -369,23 +349,11 @@ Java_sun_nio_fs_UnixNativeDispatcher_init(JNIEnv* env, jclass this)
 #endif
     my_unlinkat_func = (unlinkat_func*) dlsym(RTLD_DEFAULT, "unlinkat");
     my_renameat_func = (renameat_func*) dlsym(RTLD_DEFAULT, "renameat");
-#if defined(__linux__) && defined(__arm__)
-    my_futimesat_func = (futimesat_func*) lookup_time_t_function("futimesat",
-        "__futimesat64");
-    my_lutimes_func = (lutimes_func*) lookup_time_t_function("lutimes",
-        "__lutimes64");
-    my_futimens_func = (futimens_func*) lookup_time_t_function("futimens",
-        "__futimens64");
-#else
-#ifndef _ALLBSD_SOURCE
-    my_futimesat_func = (futimesat_func*) dlsym(RTLD_DEFAULT, "futimesat");
-    my_lutimes_func = (lutimes_func*) dlsym(RTLD_DEFAULT, "lutimes");
-#endif
-    my_futimens_func = (futimens_func*) dlsym(RTLD_DEFAULT, "futimens");
-#endif
 #if defined(_AIX)
     // Make sure we link to the 64-bit version of the function
     my_fdopendir_func = (fdopendir_func*) dlsym(RTLD_DEFAULT, "fdopendir64");
+#elif defined(_ALLBSD_SOURCE)
+    my_fdopendir_func = (fdopendir_func*) fdopendir;
 #else
     my_fdopendir_func = (fdopendir_func*) dlsym(RTLD_DEFAULT, "fdopendir");
 #endif
@@ -396,25 +364,11 @@ Java_sun_nio_fs_UnixNativeDispatcher_init(JNIEnv* env, jclass this)
         my_fstatat_func = (fstatat_func*)&fstatat_wrapper;
 #endif
 
-    /* supports futimes or futimesat, futimens, and/or lutimes */
-
-#ifdef _ALLBSD_SOURCE
-    capabilities |= sun_nio_fs_UnixNativeDispatcher_SUPPORTS_FUTIMES;
-    capabilities |= sun_nio_fs_UnixNativeDispatcher_SUPPORTS_LUTIMES;
-#else
-    if (my_futimesat_func != NULL)
-        capabilities |= sun_nio_fs_UnixNativeDispatcher_SUPPORTS_FUTIMES;
-    if (my_lutimes_func != NULL)
-        capabilities |= sun_nio_fs_UnixNativeDispatcher_SUPPORTS_LUTIMES;
-#endif
-    if (my_futimens_func != NULL)
-        capabilities |= sun_nio_fs_UnixNativeDispatcher_SUPPORTS_FUTIMENS;
-
     /* supports openat, etc. */
 
     if (my_openat_func != NULL &&  my_fstatat_func != NULL &&
         my_unlinkat_func != NULL && my_renameat_func != NULL &&
-        my_futimesat_func != NULL && my_fdopendir_func != NULL)
+        my_fdopendir_func != NULL)
     {
         capabilities |= sun_nio_fs_UnixNativeDispatcher_SUPPORTS_OPENAT;
     }
@@ -620,19 +574,19 @@ static void copy_statx_attributes(JNIEnv* env, struct my_statx* buf, jobject att
     (*env)->SetLongField(env, attrs, attrs_st_atime_sec, (jlong)buf->stx_atime.tv_sec);
     (*env)->SetLongField(env, attrs, attrs_st_mtime_sec, (jlong)buf->stx_mtime.tv_sec);
     (*env)->SetLongField(env, attrs, attrs_st_ctime_sec, (jlong)buf->stx_ctime.tv_sec);
+
+    // Check mask for birth time and set flag accordingly. The birth time is
+    // filled in if and only if the STATX_BTIME bit is set in the mask.
+    // Although the statx system call might be supported by the operating
+    // system, the birth time is not necessarily supported by the file system.
     if ((buf->stx_mask & STATX_BTIME) != 0) {
-        //  Birth time was filled in so use it
-        (*env)->SetLongField(env, attrs, attrs_st_birthtime_sec,
-                             (jlong)buf->stx_btime.tv_sec);
-        (*env)->SetLongField(env, attrs, attrs_st_birthtime_nsec,
-                             (jlong)buf->stx_btime.tv_nsec);
+        (*env)->SetBooleanField(env, attrs, attrs_birthtime_available, (jboolean)JNI_TRUE);
+        (*env)->SetLongField(env, attrs, attrs_st_birthtime_sec, (jlong)buf->stx_btime.tv_sec);
+        (*env)->SetLongField(env, attrs, attrs_st_birthtime_nsec, (jlong)buf->stx_btime.tv_nsec);
     } else {
-        //  Birth time was not filled in: fall back to last modification time
-        (*env)->SetLongField(env, attrs, attrs_st_birthtime_sec,
-                             (jlong)buf->stx_mtime.tv_sec);
-        (*env)->SetLongField(env, attrs, attrs_st_birthtime_nsec,
-                             (jlong)buf->stx_mtime.tv_nsec);
+        (*env)->SetBooleanField(env, attrs, attrs_birthtime_available, (jboolean)JNI_FALSE);
     }
+
     (*env)->SetLongField(env, attrs, attrs_st_atime_nsec, (jlong)buf->stx_atime.tv_nsec);
     (*env)->SetLongField(env, attrs, attrs_st_mtime_nsec, (jlong)buf->stx_mtime.tv_nsec);
     (*env)->SetLongField(env, attrs, attrs_st_ctime_nsec, (jlong)buf->stx_ctime.tv_nsec);
@@ -661,7 +615,9 @@ static void copy_stat_attributes(JNIEnv* env, struct stat* buf, jobject attrs) {
     (*env)->SetLongField(env, attrs, attrs_st_ctime_sec, (jlong)buf->st_ctime);
 
 #ifdef _DARWIN_FEATURE_64_BIT_INODE
+    // birthtime_available defaults to 'false'; on Darwin, it is always true
     (*env)->SetLongField(env, attrs, attrs_st_birthtime_sec, (jlong)buf->st_birthtime);
+    (*env)->SetBooleanField(env, attrs, attrs_birthtime_available, (jboolean)JNI_TRUE);
     // rely on default value of 0 for st_birthtime_nsec field on Darwin
 #endif
 
@@ -893,33 +849,6 @@ Java_sun_nio_fs_UnixNativeDispatcher_utimes0(JNIEnv* env, jclass this,
     }
 }
 
-JNIEXPORT void JNICALL
-Java_sun_nio_fs_UnixNativeDispatcher_futimes0(JNIEnv* env, jclass this, jint filedes,
-    jlong accessTime, jlong modificationTime)
-{
-    struct timeval times[2];
-    int err = 0;
-
-    times[0].tv_sec = accessTime / 1000000;
-    times[0].tv_usec = accessTime % 1000000;
-
-    times[1].tv_sec = modificationTime / 1000000;
-    times[1].tv_usec = modificationTime % 1000000;
-
-#ifdef _ALLBSD_SOURCE
-    RESTARTABLE(futimes(filedes, &times[0]), err);
-#else
-    if (my_futimesat_func == NULL) {
-        JNU_ThrowInternalError(env, "my_futimesat_func is NULL");
-        return;
-    }
-    RESTARTABLE((*my_futimesat_func)(filedes, NULL, &times[0]), err);
-#endif
-    if (err == -1) {
-        throwUnixException(env, errno);
-    }
-}
-
 JNIEXPORT void JNICALL
 Java_sun_nio_fs_UnixNativeDispatcher_futimens0(JNIEnv* env, jclass this, jint filedes,
     jlong accessTime, jlong modificationTime)
@@ -933,39 +862,27 @@ Java_sun_nio_fs_UnixNativeDispatcher_futimens0(JNIEnv* env, jclass this, jint fi
     times[1].tv_sec = modificationTime / 1000000000;
     times[1].tv_nsec = modificationTime % 1000000000;
 
-    if (my_futimens_func == NULL) {
-        JNU_ThrowInternalError(env, "my_futimens_func is NULL");
-        return;
-    }
-    RESTARTABLE((*my_futimens_func)(filedes, &times[0]), err);
+    RESTARTABLE(futimens(filedes, &times[0]), err);
     if (err == -1) {
         throwUnixException(env, errno);
     }
 }
 
 JNIEXPORT void JNICALL
-Java_sun_nio_fs_UnixNativeDispatcher_lutimes0(JNIEnv* env, jclass this,
-    jlong pathAddress, jlong accessTime, jlong modificationTime)
-{
+Java_sun_nio_fs_UnixNativeDispatcher_utimensat0(JNIEnv* env, jclass this,
+    jint fd, jlong pathAddress, jlong accessTime, jlong modificationTime, jint flags) {
     int err;
-    struct timeval times[2];
+    struct timespec times[2];
     const char* path = (const char*)jlong_to_ptr(pathAddress);
 
-    times[0].tv_sec = accessTime / 1000000;
-    times[0].tv_usec = accessTime % 1000000;
+    times[0].tv_sec = accessTime / 1000000000;
+    times[0].tv_nsec = accessTime % 1000000000;
 
-    times[1].tv_sec = modificationTime / 1000000;
-    times[1].tv_usec = modificationTime % 1000000;
+    times[1].tv_sec = modificationTime / 1000000000;
+    times[1].tv_nsec = modificationTime % 1000000000;
+
+    RESTARTABLE(utimensat(fd, path, &times[0], flags), err);
 
-#ifdef _ALLBSD_SOURCE
-    RESTARTABLE(lutimes(path, &times[0]), err);
-#else
-    if (my_lutimes_func == NULL) {
-        JNU_ThrowInternalError(env, "my_lutimes_func is NULL");
-        return;
-    }
-    RESTARTABLE((*my_lutimes_func)(path, &times[0]), err);
-#endif
     if (err == -1) {
         throwUnixException(env, errno);
     }
