@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -376,15 +376,6 @@ class JFIFMarkerSegment extends MarkerSegment {
             }
             thumbWidth = Math.min(thumbWidth, MAX_THUMB_WIDTH);
             thumbHeight = Math.min(thumbHeight, MAX_THUMB_HEIGHT);
-
-            int maxArea = (0xffff - DATA_SIZE - LENGTH_SIZE) / thumb.getSampleModel().getNumBands();
-            if (thumbWidth * thumbHeight > maxArea) {
-                writer.warningOccurred(JPEGImageWriter.WARNING_THUMB_CLIPPED);
-                double scale = Math.sqrt( ((double)maxArea) / (double)(thumbWidth * thumbHeight) );
-                thumbWidth = (int) (scale * thumbWidth);
-                thumbHeight = (int) (scale * thumbHeight);
-            }
-
             thumbData = thumb.getRaster().getPixels(0, 0,
                                                     thumbWidth, thumbHeight,
                                                     (int []) null);
@@ -1243,7 +1234,7 @@ class JFIFMarkerSegment extends MarkerSegment {
             return retval;
         }
 
-        private static class ThumbnailReadListener
+        static class ThumbnailReadListener
             implements IIOReadProgressListener {
             JPEGImageReader reader = null;
             ThumbnailReadListener (JPEGImageReader reader) {
