@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,13 +28,11 @@ package javax.imageio;
 import java.awt.image.BufferedImage;
 import java.awt.image.RenderedImage;
 import java.io.File;
-import java.io.FilePermission;
 import java.io.InputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.lang.reflect.Method;
 import java.net.URL;
-import java.security.AccessController;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashSet;
@@ -50,7 +48,6 @@ import javax.imageio.spi.ServiceRegistry;
 import javax.imageio.stream.ImageInputStream;
 import javax.imageio.stream.ImageOutputStream;
 import sun.awt.AppContext;
-import sun.security.action.GetPropertyAction;
 
 /**
  * A class containing static convenience methods for locating
@@ -166,56 +163,19 @@ public final class ImageIO {
      * java.io.tmpdir system property.
      */
     private static String getTempDir() {
-        GetPropertyAction a = new GetPropertyAction("java.io.tmpdir");
-        return AccessController.doPrivileged(a);
+        return System.getProperty("java.io.tmpdir");
     }
 
     /**
      * Determines whether the caller has write access to the cache
      * directory, stores the result in the {@code CacheInfo} object,
-     * and returns the decision.  This method helps to prevent mysterious
-     * SecurityExceptions to be thrown when this convenience class is used
-     * in an applet, for example.
+     * and returns the decision.
      */
     private static boolean hasCachePermission() {
         Boolean hasPermission = getCacheInfo().getHasPermission();
-
         if (hasPermission != null) {
             return hasPermission.booleanValue();
         } else {
-            try {
-                SecurityManager security = System.getSecurityManager();
-                if (security != null) {
-                    File cachedir = getCacheDirectory();
-                    String cachepath;
-
-                    if (cachedir != null) {
-                        cachepath = cachedir.getPath();
-                    } else {
-                        cachepath = getTempDir();
-
-                        if (cachepath == null || cachepath.isEmpty()) {
-                            getCacheInfo().setHasPermission(Boolean.FALSE);
-                            return false;
-                        }
-                    }
-
-                    // we have to check whether we can read, write,
-                    // and delete cache files.
-                    // So, compose cache file path and check it.
-                    String filepath = cachepath;
-                    if (!filepath.endsWith(File.separator)) {
-                        filepath += File.separator;
-                    }
-                    filepath += "*";
-
-                    security.checkPermission(new FilePermission(filepath, "read, write, delete"));
-                }
-            } catch (SecurityException e) {
-                getCacheInfo().setHasPermission(Boolean.FALSE);
-                return false;
-            }
-
             getCacheInfo().setHasPermission(Boolean.TRUE);
             return true;
         }
@@ -275,9 +235,7 @@ public final class ImageIO {
      *
      * @see File#createTempFile(String, String, File)
      *
-     * @exception SecurityException if the security manager denies
-     * access to the directory.
-     * @exception IllegalArgumentException if {@code cacheDir} is
+     * @throws IllegalArgumentException if {@code cacheDir} is
      * non-{@code null} but is not a directory.
      *
      * @see #getCacheDirectory
@@ -324,9 +282,9 @@ public final class ImageIO {
      *
      * @return an {@code ImageInputStream}, or {@code null}.
      *
-     * @exception IllegalArgumentException if {@code input}
+     * @throws IllegalArgumentException if {@code input}
      * is {@code null}.
-     * @exception IOException if a cache file is needed but cannot be
+     * @throws IOException if a cache file is needed but cannot be
      * created.
      *
      * @see javax.imageio.spi.ImageInputStreamSpi
@@ -386,9 +344,9 @@ public final class ImageIO {
      * @return an {@code ImageOutputStream}, or
      * {@code null}.
      *
-     * @exception IllegalArgumentException if {@code output} is
+     * @throws IllegalArgumentException if {@code output} is
      * {@code null}.
-     * @exception IOException if a cache file is needed but cannot be
+     * @throws IOException if a cache file is needed but cannot be
      * created.
      *
      * @see javax.imageio.spi.ImageOutputStreamSpi
@@ -635,7 +593,7 @@ public final class ImageIO {
      *
      * @return an {@code Iterator} containing {@code ImageReader}s.
      *
-     * @exception IllegalArgumentException if {@code input} is
+     * @throws IllegalArgumentException if {@code input} is
      * {@code null}.
      *
      * @see javax.imageio.spi.ImageReaderSpi#canDecodeInput
@@ -695,7 +653,7 @@ public final class ImageIO {
      * @return an {@code Iterator} containing
      * {@code ImageReader}s.
      *
-     * @exception IllegalArgumentException if {@code formatName}
+     * @throws IllegalArgumentException if {@code formatName}
      * is {@code null}.
      *
      * @see javax.imageio.spi.ImageReaderSpi#getFormatNames
@@ -730,7 +688,7 @@ public final class ImageIO {
      * @return an {@code Iterator} containing
      * {@code ImageReader}s.
      *
-     * @exception IllegalArgumentException if {@code fileSuffix}
+     * @throws IllegalArgumentException if {@code fileSuffix}
      * is {@code null}.
      *
      * @see javax.imageio.spi.ImageReaderSpi#getFileSuffixes
@@ -765,7 +723,7 @@ public final class ImageIO {
      * @return an {@code Iterator} containing
      * {@code ImageReader}s.
      *
-     * @exception IllegalArgumentException if {@code MIMEType} is
+     * @throws IllegalArgumentException if {@code MIMEType} is
      * {@code null}.
      *
      * @see javax.imageio.spi.ImageReaderSpi#getMIMETypes
@@ -878,7 +836,7 @@ public final class ImageIO {
      * @return an {@code Iterator} containing
      * {@code ImageWriter}s.
      *
-     * @exception IllegalArgumentException if {@code formatName} is
+     * @throws IllegalArgumentException if {@code formatName} is
      * {@code null}.
      *
      * @see javax.imageio.spi.ImageWriterSpi#getFormatNames
@@ -912,7 +870,7 @@ public final class ImageIO {
      *
      * @return an {@code Iterator} containing {@code ImageWriter}s.
      *
-     * @exception IllegalArgumentException if {@code fileSuffix} is
+     * @throws IllegalArgumentException if {@code fileSuffix} is
      * {@code null}.
      *
      * @see javax.imageio.spi.ImageWriterSpi#getFileSuffixes
@@ -946,7 +904,7 @@ public final class ImageIO {
      *
      * @return an {@code Iterator} containing {@code ImageWriter}s.
      *
-     * @exception IllegalArgumentException if {@code MIMEType} is
+     * @throws IllegalArgumentException if {@code MIMEType} is
      * {@code null}.
      *
      * @see javax.imageio.spi.ImageWriterSpi#getMIMETypes
@@ -993,7 +951,7 @@ public final class ImageIO {
      *
      * @return an {@code ImageWriter}, or null.
      *
-     * @exception IllegalArgumentException if {@code reader} is
+     * @throws IllegalArgumentException if {@code reader} is
      * {@code null}.
      *
      * @see #getImageReader(ImageWriter)
@@ -1073,7 +1031,7 @@ public final class ImageIO {
      *
      * @return an {@code ImageReader}, or null.
      *
-     * @exception IllegalArgumentException if {@code writer} is
+     * @throws IllegalArgumentException if {@code writer} is
      * {@code null}.
      *
      * @see #getImageWriter(ImageReader)
@@ -1149,7 +1107,7 @@ public final class ImageIO {
      *
      * @return an {@code Iterator} containing {@code ImageWriter}s.
      *
-     * @exception IllegalArgumentException if any parameter is
+     * @throws IllegalArgumentException if any parameter is
      * {@code null}.
      *
      * @see javax.imageio.spi.ImageWriterSpi#canEncodeImage(ImageTypeSpecifier)
@@ -1236,7 +1194,7 @@ public final class ImageIO {
      * @return an {@code Iterator} containing
      * {@code ImageTranscoder}s.
      *
-     * @exception IllegalArgumentException if {@code reader} or
+     * @throws IllegalArgumentException if {@code reader} or
      * {@code writer} is {@code null}.
      */
     public static Iterator<ImageTranscoder>
@@ -1293,9 +1251,9 @@ public final class ImageIO {
      * @return a {@code BufferedImage} containing the decoded
      * contents of the input, or {@code null}.
      *
-     * @exception IllegalArgumentException if {@code input} is
+     * @throws IllegalArgumentException if {@code input} is
      * {@code null}.
-     * @exception IOException if an error occurs during reading or when not
+     * @throws IOException if an error occurs during reading or when not
      * able to create required ImageInputStream.
      */
     public static BufferedImage read(File input) throws IOException {
@@ -1344,9 +1302,9 @@ public final class ImageIO {
      * @return a {@code BufferedImage} containing the decoded
      * contents of the input, or {@code null}.
      *
-     * @exception IllegalArgumentException if {@code input} is
+     * @throws IllegalArgumentException if {@code input} is
      * {@code null}.
-     * @exception IOException if an error occurs during reading or when not
+     * @throws IOException if an error occurs during reading or when not
      * able to create required ImageInputStream.
      */
     public static BufferedImage read(InputStream input) throws IOException {
@@ -1388,9 +1346,9 @@ public final class ImageIO {
      * @return a {@code BufferedImage} containing the decoded
      * contents of the input, or {@code null}.
      *
-     * @exception IllegalArgumentException if {@code input} is
+     * @throws IllegalArgumentException if {@code input} is
      * {@code null}.
-     * @exception IOException if an error occurs during reading or when not
+     * @throws IOException if an error occurs during reading or when not
      * able to create required ImageInputStream.
      */
     public static BufferedImage read(URL input) throws IOException {
@@ -1441,9 +1399,9 @@ public final class ImageIO {
      * @return a {@code BufferedImage} containing the decoded
      * contents of the input, or {@code null}.
      *
-     * @exception IllegalArgumentException if {@code stream} is
+     * @throws IllegalArgumentException if {@code stream} is
      * {@code null}.
-     * @exception IOException if an error occurs during reading.
+     * @throws IOException if an error occurs during reading.
      */
     public static BufferedImage read(ImageInputStream stream)
         throws IOException {
@@ -1462,6 +1420,8 @@ public final class ImageIO {
         BufferedImage bi;
         try (stream) {
             bi = reader.read(0, param);
+        } catch (RuntimeException e) {
+            throw new IIOException(e.toString(), e);
         } finally {
             reader.dispose();
         }
@@ -1469,7 +1429,7 @@ public final class ImageIO {
     }
 
     /**
-     * Writes an image using the an arbitrary {@code ImageWriter}
+     * Writes an image using an arbitrary {@code ImageWriter}
      * that supports the given format to an
      * {@code ImageOutputStream}.  The image is written to the
      * {@code ImageOutputStream} starting at the current stream
@@ -1487,9 +1447,9 @@ public final class ImageIO {
      *
      * @return {@code false} if no appropriate writer is found.
      *
-     * @exception IllegalArgumentException if any parameter is
+     * @throws IllegalArgumentException if any parameter is
      * {@code null}.
-     * @exception IOException if an error occurs during writing.
+     * @throws IOException if an error occurs during writing.
      */
     public static boolean write(RenderedImage im,
                                 String formatName,
@@ -1520,9 +1480,9 @@ public final class ImageIO {
      *
      * @return {@code false} if no appropriate writer is found.
      *
-     * @exception IllegalArgumentException if any parameter is
+     * @throws IllegalArgumentException if any parameter is
      * {@code null}.
-     * @exception IOException if an error occurs during writing or when not
+     * @throws IOException if an error occurs during writing or when not
      * able to create required ImageOutputStream.
      */
     public static boolean write(RenderedImage im,
@@ -1568,9 +1528,9 @@ public final class ImageIO {
      *
      * @return {@code false} if no appropriate writer is found.
      *
-     * @exception IllegalArgumentException if any parameter is
+     * @throws IllegalArgumentException if any parameter is
      * {@code null}.
-     * @exception IOException if an error occurs during writing or when not
+     * @throws IOException if an error occurs during writing or when not
      * able to create required ImageOutputStream.
      */
     public static boolean write(RenderedImage im,
