@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -202,6 +202,7 @@ public class FileFontStrike extends PhysicalStrike {
             this.disposer = new FontStrikeDisposer(fileFont, desc);
             initGlyphCache();
             pScalerContext = NullFontScaler.getNullScalerContext();
+            SunFontManager.getInstance().deRegisterBadFont(fileFont);
             return;
         }
         /* First, see if native code should be used to create the glyph.
@@ -314,8 +315,7 @@ public class FileFontStrike extends PhysicalStrike {
              * work to the FM case.
              */
             float advance = getGlyphAdvance(glyphCode, false);
-            StrikeCache.unsafe.putFloat(ptr + StrikeCache.xAdvanceOffset,
-                                        advance);
+            StrikeCache.setGlyphXAdvance(ptr, advance);
             return ptr;
         } else {
             if (FontUtilities.isLogging()) {
@@ -664,8 +664,7 @@ public class FileFontStrike extends PhysicalStrike {
                 glyphPtr = getCachedGlyphPtr(glyphCode);
             }
             if (glyphPtr != 0L) {
-                advance = StrikeCache.unsafe.getFloat
-                    (glyphPtr + StrikeCache.xAdvanceOffset);
+                advance = StrikeCache.getGlyphXAdvance(glyphPtr);
 
             } else {
                 advance = fileFont.getGlyphAdvance(pScalerContext, glyphCode);
@@ -725,15 +724,13 @@ public class FileFontStrike extends PhysicalStrike {
             return;
         }
 
-        topLeftX = StrikeCache.unsafe.getFloat(ptr+StrikeCache.topLeftXOffset);
-        topLeftY = StrikeCache.unsafe.getFloat(ptr+StrikeCache.topLeftYOffset);
+        topLeftX = StrikeCache.getGlyphTopLeftX(ptr);
+        topLeftY = StrikeCache.getGlyphTopLeftY(ptr);
 
         result.x = (int)Math.floor(pt.x + topLeftX + 0.5f);
         result.y = (int)Math.floor(pt.y + topLeftY + 0.5f);
-        result.width =
-            StrikeCache.unsafe.getShort(ptr+StrikeCache.widthOffset)  &0x0ffff;
-        result.height =
-            StrikeCache.unsafe.getShort(ptr+StrikeCache.heightOffset) &0x0ffff;
+        result.width = StrikeCache.getGlyphWidth(ptr) & 0x0ffff;
+        result.height = StrikeCache.getGlyphHeight(ptr) & 0x0ffff;
 
         /* HRGB LCD text may have padding that is empty. This is almost always
          * going to be when topLeftX is -2 or less.
@@ -754,25 +751,22 @@ public class FileFontStrike extends PhysicalStrike {
 
     private int getGlyphImageMinX(long ptr, int origMinX) {
 
-        int width = StrikeCache.unsafe.getChar(ptr+StrikeCache.widthOffset);
-        int height = StrikeCache.unsafe.getChar(ptr+StrikeCache.heightOffset);
-        int rowBytes =
-            StrikeCache.unsafe.getChar(ptr+StrikeCache.rowBytesOffset);
+        int width = StrikeCache.getGlyphWidth(ptr);
+        int height = StrikeCache.getGlyphHeight(ptr);
+        int rowBytes = StrikeCache.getGlyphRowBytes(ptr);
 
         if (rowBytes == width) {
             return origMinX;
         }
 
-        long pixelData =
-            StrikeCache.unsafe.getAddress(ptr + StrikeCache.pixelDataOffset);
-
-        if (pixelData == 0L) {
+        if (StrikeCache.getGlyphImagePtr(ptr) == 0L) {
             return origMinX;
         }
 
+        byte[] pixelData = StrikeCache.getGlyphPixelBytes(ptr);
         for (int y=0;y<height;y++) {
             for (int x=0;x<3;x++) {
-                if (StrikeCache.unsafe.getByte(pixelData+y*rowBytes+x) != 0) {
+                if (pixelData[(y*rowBytes)+x] != 0) {
                     return origMinX;
                 }
             }
@@ -818,10 +812,8 @@ public class FileFontStrike extends PhysicalStrike {
         }
         if (glyphPtr != 0L) {
             metrics = new Point2D.Float();
-            metrics.x = StrikeCache.unsafe.getFloat
-                (glyphPtr + StrikeCache.xAdvanceOffset);
-            metrics.y = StrikeCache.unsafe.getFloat
-                (glyphPtr + StrikeCache.yAdvanceOffset);
+            metrics.x = StrikeCache.getGlyphXAdvance(glyphPtr);
+            metrics.y = StrikeCache.getGlyphYAdvance(glyphPtr);
             /* advance is currently in device space, need to convert back
              * into user space.
              * This must not include the translation component. */
