@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -61,6 +61,7 @@ import javax.print.attribute.standard.Copies;
 import javax.print.attribute.standard.Destination;
 import javax.print.attribute.standard.DialogTypeSelection;
 import javax.print.attribute.standard.JobName;
+import javax.print.attribute.standard.OutputBin;
 import javax.print.attribute.standard.Sides;
 
 import java.io.BufferedOutputStream;
@@ -82,6 +83,7 @@ import java.util.Properties;
 
 import sun.awt.CharsetString;
 import sun.awt.FontConfiguration;
+import sun.awt.OSInfo;
 import sun.awt.PlatformFont;
 import sun.awt.SunToolkit;
 import sun.font.FontAccess;
@@ -263,7 +265,7 @@ public class PSPrinterJob extends RasterPrinterJob {
    private AffineTransform mLastTransform;
 
    private double xres = PS_XRES;
-   private double yres = PS_XRES;
+   private double yres = PS_YRES;
 
    /* non-null if printing EPS for Java Plugin */
    private EPSPrinter epsPrinter = null;
@@ -336,19 +338,9 @@ public class PSPrinterJob extends RasterPrinterJob {
         initStatic();
     }
 
-    @SuppressWarnings("removal")
     private static void initStatic() {
-       //enable priviledges so initProps can access system properties,
-        // open the property file, etc.
-        java.security.AccessController.doPrivileged(
-                            new java.security.PrivilegedAction<Object>() {
-            public Object run() {
-                mFontProps = initProps();
-                String osName = System.getProperty("os.name");
-                isMac = osName.startsWith("Mac");
-                return null;
-            }
-        });
+        mFontProps = initProps();
+        isMac = OSInfo.getOSType() == OSInfo.OSType.MACOSX;
     }
 
     /*
@@ -491,14 +483,19 @@ public class PSPrinterJob extends RasterPrinterJob {
         if (attributes == null) {
             return; // now always use attributes, so this shouldn't happen.
         }
+        mOptions = "";
         Attribute attr = attributes.get(Media.class);
         if (attr instanceof CustomMediaTray) {
             CustomMediaTray customTray = (CustomMediaTray)attr;
             String choice = customTray.getChoiceName();
             if (choice != null) {
-                mOptions = " InputSlot="+ choice;
+                mOptions += " InputSlot="+ choice;
             }
         }
+        String outputBin = getOutputBinValue(outputBinAttr);
+        if (outputBin != null) {
+            mOptions += " output-bin=" + outputBin;
+        }
     }
 
     /**
@@ -506,14 +503,13 @@ public class PSPrinterJob extends RasterPrinterJob {
      * this method is called to mark the start of a
      * document.
      */
-    @SuppressWarnings("removal")
     protected void startDoc() throws PrinterException {
 
         // A security check has been performed in the
         // java.awt.print.printerJob.getPrinterJob method.
-        // We use an inner class to execute the privilged open operations.
+        // We use an inner class to execute the privileged open operations.
         // Note that we only open a file if it has been nominated by
-        // the end-user in a dialog that we ouselves put up.
+        // the end-user in a dialog that we ourselves put up.
 
         OutputStream output = null;
 
@@ -545,7 +541,7 @@ public class PSPrinterJob extends RasterPrinterJob {
                     }
                 } else {
                     PrinterOpener po = new PrinterOpener();
-                    java.security.AccessController.doPrivileged(po);
+                    po.run();
                     if (po.pex != null) {
                         throw po.pex;
                     }
@@ -635,22 +631,16 @@ public class PSPrinterJob extends RasterPrinterJob {
                                            paperWidth + " "+ paperHeight+"]");
 
             final PrintService pservice = getPrintService();
-            Boolean isPS = java.security.AccessController.doPrivileged(
-                new java.security.PrivilegedAction<Boolean>() {
-                    public Boolean run() {
-                       try {
-                           Class<?> psClass = Class.forName("sun.print.IPPPrintService");
-                           if (psClass.isInstance(pservice)) {
-                               Method isPSMethod = psClass.getMethod("isPostscript",
-                                                                     (Class[])null);
-                               return (Boolean)isPSMethod.invoke(pservice, (Object[])null);
-                           }
-                       } catch (Throwable t) {
-                       }
-                       return Boolean.TRUE;
-                    }
+            Boolean isPS = Boolean.TRUE;
+            try {
+                Class<?> psClass = Class.forName("sun.print.IPPPrintService");
+                if (psClass.isInstance(pservice)) {
+                    Method isPSMethod = psClass.getMethod("isPostscript",
+                                                          (Class[])null);
+                    isPS = (Boolean)isPSMethod.invoke(pservice, (Object[])null);
                 }
-            );
+            } catch (Throwable t) {
+            }
             if (isPS) {
                 mPSStream.print(" /DeferredMediaSelection true");
             }
@@ -671,9 +661,9 @@ public class PSPrinterJob extends RasterPrinterJob {
         mPSStream.println("%%EndSetup");
     }
 
-    // Inner class to run "privileged" to open the printer output stream.
+    // Inner class to open the printer output stream.
 
-    private class PrinterOpener implements java.security.PrivilegedAction<OutputStream> {
+    private class PrinterOpener {
         PrinterException pex;
         OutputStream result;
 
@@ -698,9 +688,9 @@ public class PSPrinterJob extends RasterPrinterJob {
         }
     }
 
-    // Inner class to run "privileged" to invoke the system print command
+    // Inner class to invoke the system print command
 
-    private class PrinterSpooler implements java.security.PrivilegedAction<Object> {
+    private class PrinterSpooler {
         PrinterException pex;
 
         private void handleProcessFailure(final Process failedProcess,
@@ -761,21 +751,13 @@ public class PSPrinterJob extends RasterPrinterJob {
     /**
      * Invoked if the application cancelled the printjob.
      */
-    @SuppressWarnings("removal")
     protected void abortDoc() {
         if (mPSStream != null && mDestType != RasterPrinterJob.STREAM) {
             mPSStream.close();
         }
-        java.security.AccessController.doPrivileged(
-            new java.security.PrivilegedAction<Object>() {
-
-            public Object run() {
-               if (spoolFile != null && spoolFile.exists()) {
-                   spoolFile.delete();
-               }
-               return null;
-            }
-        });
+        if (spoolFile != null && spoolFile.exists()) {
+            spoolFile.delete();
+        }
     }
 
     /**
@@ -783,7 +765,6 @@ public class PSPrinterJob extends RasterPrinterJob {
      * this method is called after that last page
      * has been imaged.
      */
-    @SuppressWarnings("removal")
     protected void endDoc() throws PrinterException {
         if (mPSStream != null) {
             mPSStream.println(EOF_COMMENT);
@@ -808,7 +789,7 @@ public class PSPrinterJob extends RasterPrinterJob {
                 }
             }
             PrinterSpooler spooler = new PrinterSpooler();
-            java.security.AccessController.doPrivileged(spooler);
+            spooler.run();
             if (spooler.pex != null) {
                 throw spooler.pex;
             }
@@ -853,28 +834,18 @@ public class PSPrinterJob extends RasterPrinterJob {
                             paperWidth + " " + paperHeight + "]");
 
             final PrintService pservice = getPrintService();
-            @SuppressWarnings("removal")
-            Boolean isPS = java.security.AccessController.doPrivileged(
-                new java.security.PrivilegedAction<Boolean>() {
-                    public Boolean run() {
-                        try {
-                            Class<?> psClass =
-                                Class.forName("sun.print.IPPPrintService");
-                            if (psClass.isInstance(pservice)) {
-                                Method isPSMethod =
-                                    psClass.getMethod("isPostscript",
-                                                      (Class[])null);
-                                return (Boolean)
-                                    isPSMethod.invoke(pservice,
+            Boolean isPS = Boolean.TRUE;
+            try {
+                Class<?> psClass = Class.forName("sun.print.IPPPrintService");
+                if (psClass.isInstance(pservice)) {
+                    Method isPSMethod =
+                                psClass.getMethod("isPostscript",
+                                                  (Class[])null);
+                    isPS = (Boolean) isPSMethod.invoke(pservice,
                                                       (Object[])null);
-                            }
-                        } catch (Throwable t) {
-                        }
-                        return Boolean.TRUE;
-                    }
-                    }
-                );
-
+                }
+            } catch (Throwable t) {
+            }
             if (isPS) {
                 mPSStream.print(" /DeferredMediaSelection true");
             }
@@ -885,7 +856,7 @@ public class PSPrinterJob extends RasterPrinterJob {
     }
 
     /**
-     * The RastePrintJob super class calls this method
+     * The RasterPrintJob super class calls this method
      * at the end of each page.
      */
     protected void endPage(PageFormat format, Printable painter,
@@ -1185,7 +1156,7 @@ public class PSPrinterJob extends RasterPrinterJob {
                         Integer.parseInt(mFontProps.getProperty(psName));
 
                 /* If there is no PostScript font for this font name,
-                 * then we want to termintate the loop and the method
+                 * then we want to terminate the loop and the method
                  * indicating our failure. Setting the array to null
                  * is used to indicate these failures.
                  */
@@ -1623,8 +1594,8 @@ public class PSPrinterJob extends RasterPrinterJob {
             ncomps+=1; // for jobsheet
         }
 
-        String osname = System.getProperty("os.name");
-        if (osname.equals("Linux") || osname.contains("OS X")) {
+        if (OSInfo.getOSType() == OSInfo.OSType.LINUX ||
+                OSInfo.getOSType() == OSInfo.OSType.MACOSX) {
             execCmd = new String[ncomps];
             execCmd[n++] = "/usr/bin/lpr";
             if ((pFlags & PRINTER) != 0) {
@@ -1643,7 +1614,9 @@ public class PSPrinterJob extends RasterPrinterJob {
                 execCmd[n++] = "-o job-sheets=standard";
             }
             if ((pFlags & OPTIONS) != 0) {
-                execCmd[n++] = "-o" + options;
+                for (String option : options.trim().split(" ")) {
+                    execCmd[n++] = "-o " + option;
+                }
             }
         } else {
             ncomps+=1; //add 1 arg for lp
@@ -1666,7 +1639,9 @@ public class PSPrinterJob extends RasterPrinterJob {
                 execCmd[n++] = "-o job-sheets=standard";
             }
             if ((pFlags & OPTIONS) != 0) {
-                execCmd[n++] = "-o" + options;
+                for (String option : options.trim().split(" ")) {
+                    execCmd[n++] = "-o " + option;
+                }
             }
         }
         execCmd[n++] = spoolFile;
