@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -112,9 +112,6 @@ abstract class XDecoratedPeer extends XWindowPeer {
 
         content = XContentWindow.createContent(this);
 
-        if (warningWindow != null) {
-            warningWindow.toFront();
-        }
         focusProxy = createFocusProxy();
     }
 
@@ -419,7 +416,7 @@ abstract class XDecoratedPeer extends XWindowPeer {
             insets_corrected = false;
 
             /*
-             * We can be repareted to root for two reasons:
+             * We can be reparented to root for two reasons:
              *   . setVisible(false)
              *   . WM exited
              */
@@ -480,7 +477,7 @@ abstract class XDecoratedPeer extends XWindowPeer {
          */
         Insets correction = difference(correctWM, currentInsets);
         if (insLog.isLoggable(PlatformLogger.Level.FINEST)) {
-            insLog.finest("Corrention {0}", correction);
+            insLog.finest("Correction {0}", correction);
         }
         if (!isNull(correction)) {
             currentInsets = copy(correctWM);
@@ -678,7 +675,7 @@ abstract class XDecoratedPeer extends XWindowPeer {
     }
 
     /**
-     * @param x, y, width, heith - dimensions of the window with insets
+     * @param x, y, width, height - dimensions of the window with insets
      */
     private void reshape(int x, int y, int width, int height, int operation,
                          boolean userReshape)
@@ -783,7 +780,7 @@ abstract class XDecoratedPeer extends XWindowPeer {
 
         /*
          * Some window managers configure before we are reparented and
-         * the send event flag is set! ugh... (Enlighetenment for one,
+         * the send event flag is set! ugh... (Enlightenment for one,
          * possibly MWM as well).  If we haven't been reparented yet
          * this is just the WM shuffling us into position.  Ignore
          * it!!!! or we wind up in a bogus location.
@@ -843,7 +840,6 @@ abstract class XDecoratedPeer extends XWindowPeer {
         reconfigureContentWindow(newDimensions);
         updateChildrenSizes();
 
-        repositionSecurityWarning();
     }
 
     private void checkShellRectSize(Rectangle shellRect) {
