@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -111,8 +111,6 @@ import java.awt.peer.TextFieldPeer;
 import java.awt.peer.TrayIconPeer;
 import java.awt.peer.WindowPeer;
 import java.beans.PropertyChangeListener;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
@@ -130,7 +128,6 @@ import javax.swing.LookAndFeel;
 import javax.swing.UIDefaults;
 
 import sun.awt.AWTAccessor;
-import sun.awt.AWTPermissions;
 import sun.awt.AppContext;
 import sun.awt.DisplayChangedListener;
 import sun.awt.LightweightFrame;
@@ -146,8 +143,6 @@ import sun.awt.util.ThreadGroupUtils;
 import sun.font.FontConfigManager;
 import sun.java2d.SunGraphicsEnvironment;
 import sun.print.PrintJob2D;
-import sun.security.action.GetBooleanAction;
-import sun.security.action.GetPropertyAction;
 import sun.util.logging.PlatformLogger;
 
 import static sun.awt.X11.XlibUtil.scaleDown;
@@ -202,7 +197,6 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
     private static final X11GraphicsDevice device;
     private static final long display;
     static int awt_multiclick_time;
-    static boolean securityWarningEnabled;
 
     /**
      * Dimensions of default virtual screen in pixels. These values are used to
@@ -214,7 +208,6 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
     private static XMouseInfoPeer xPeer;
 
     static {
-        initSecurityWarning();
         if (GraphicsEnvironment.isHeadless()) {
             localEnv = null;
             device = null;
@@ -243,18 +236,6 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
         return Thread.currentThread() == toolkitThread;
     }
 
-    static void initSecurityWarning() {
-        // Enable warning only for internal builds
-        @SuppressWarnings("removal")
-        String runtime = AccessController.doPrivileged(
-                             new GetPropertyAction("java.runtime.version"));
-        securityWarningEnabled = (runtime != null && runtime.contains("internal"));
-    }
-
-    static boolean isSecurityWarningEnabled() {
-        return securityWarningEnabled;
-    }
-
     static native void awt_output_flush();
 
     static void  awtFUnlock() {
@@ -325,7 +306,6 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
         }
     }
 
-    @SuppressWarnings("removal")
     void init() {
         awtLock();
         try {
@@ -338,13 +318,10 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
             arrowCursor = XlibWrapper.XCreateFontCursor(XToolkit.getDisplay(),
                 XCursorFontConstants.XC_arrow);
             final String extraButtons = "sun.awt.enableExtraMouseButtons";
-            AccessController.doPrivileged((PrivilegedAction<Void>) () -> {
-                areExtraMouseButtonsEnabled =
-                    Boolean.parseBoolean(System.getProperty(extraButtons, "true"));
-                //set system property if not yet assigned
-                System.setProperty(extraButtons, ""+areExtraMouseButtonsEnabled);
-                return null;
-            });
+            areExtraMouseButtonsEnabled =
+                Boolean.parseBoolean(System.getProperty(extraButtons, "true"));
+            //set system property if not yet assigned
+            System.setProperty(extraButtons, "" + areExtraMouseButtonsEnabled);
             // Detect display mode changes
             XlibWrapper.XSelectInput(XToolkit.getDisplay(), XToolkit.getDefaultRootWindow(), XConstants.StructureNotifyMask);
             XToolkit.addEventDispatcher(XToolkit.getDefaultRootWindow(), new XEventDispatcher() {
@@ -358,34 +335,35 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
                         } finally {
                             awtLock();
                         }
+                    } else {
+                        final XAtom XA_NET_WORKAREA = XAtom.get("_NET_WORKAREA");
+                        final boolean rootWindowWorkareaResized = (ev.get_type() == XConstants.PropertyNotify
+                                && ev.get_xproperty().get_atom() == XA_NET_WORKAREA.getAtom());
+                        if (rootWindowWorkareaResized) resetScreenInsetsCache();
                     }
                 }
             });
         } finally {
             awtUnlock();
         }
-        PrivilegedAction<Void> a = () -> {
-            Runnable r = () -> {
-                XSystemTrayPeer peer = XSystemTrayPeer.getPeerInstance();
-                if (peer != null) {
-                    peer.dispose();
-                }
-                if (xs != null) {
-                    ((XAWTXSettings)xs).dispose();
-                }
-                freeXKB();
-                if (log.isLoggable(PlatformLogger.Level.FINE)) {
-                    dumpPeers();
-                }
-            };
-            String name = "XToolkt-Shutdown-Thread";
-            Thread shutdownThread = new Thread(
-                    ThreadGroupUtils.getRootThreadGroup(), r, name, 0, false);
-            shutdownThread.setContextClassLoader(null);
-            Runtime.getRuntime().addShutdownHook(shutdownThread);
-            return null;
+        Runnable r = () -> {
+            XSystemTrayPeer peer = XSystemTrayPeer.getPeerInstance();
+            if (peer != null) {
+                peer.dispose();
+            }
+            if (xs != null) {
+                ((XAWTXSettings)xs).dispose();
+            }
+            freeXKB();
+            if (log.isLoggable(PlatformLogger.Level.FINE)) {
+                dumpPeers();
+            }
         };
-        AccessController.doPrivileged(a);
+        String name = "XToolkt-Shutdown-Thread";
+        Thread shutdownThread = new Thread(
+                ThreadGroupUtils.getRootThreadGroup(), r, name, 0, false);
+        shutdownThread.setContextClassLoader(null);
+        Runtime.getRuntime().addShutdownHook(shutdownThread);
     }
 
     static String getCorrectXIDString(String val) {
@@ -403,7 +381,6 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
         return awtAppClassName;
     }
 
-    @SuppressWarnings("removal")
     public XToolkit() {
         super();
         if (PerformanceLogger.loggingEnabled()) {
@@ -426,16 +403,13 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
             init();
             XWM.init();
 
-            toolkitThread = AccessController.doPrivileged((PrivilegedAction<Thread>) () -> {
-                String name = "AWT-XAWT";
-                Thread thread = new Thread(
+            String name = "AWT-XAWT";
+            toolkitThread = new Thread(
                         ThreadGroupUtils.getRootThreadGroup(), this, name,
                         0, false);
-                thread.setContextClassLoader(null);
-                thread.setPriority(Thread.NORM_PRIORITY + 1);
-                thread.setDaemon(true);
-                return thread;
-            });
+            toolkitThread.setContextClassLoader(null);
+            toolkitThread.setPriority(Thread.NORM_PRIORITY + 1);
+            toolkitThread.setDaemon(true);
             toolkitThread.start();
         }
     }
@@ -843,8 +817,7 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
      * When two screens overlap and the first contains a dock(*****), then
      * _NET_WORKAREA may start at point x1,y1 and end at point x2,y2.
      */
-    @Override
-    public Insets getScreenInsets(final GraphicsConfiguration gc) {
+    private Insets getScreenInsetsImpl(final GraphicsConfiguration gc) {
         GraphicsDevice gd = gc.getDevice();
         XNETProtocol np = XWM.getWM().getNETProtocol();
         if (np == null || !(gd instanceof X11GraphicsDevice) || !np.active()) {
@@ -872,6 +845,34 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
         }
     }
 
+    private void resetScreenInsetsCache() {
+        final GraphicsDevice[] devices = ((X11GraphicsEnvironment)GraphicsEnvironment.
+                getLocalGraphicsEnvironment()).getScreenDevices();
+        for (var gd : devices) {
+            ((X11GraphicsDevice)gd).resetInsets();
+        }
+    }
+
+    @Override
+    public Insets getScreenInsets(final GraphicsConfiguration gc) {
+        final GraphicsDevice gd = gc.getDevice();
+        if (gd instanceof X11GraphicsDevice x11Device) {
+            Insets insets = x11Device.getInsets();
+            if (insets == null) {
+                synchronized (x11Device) {
+                    insets = x11Device.getInsets();
+                    if (insets == null) {
+                        insets = getScreenInsetsImpl(gc);
+                        x11Device.setInsets(insets);
+                    }
+                }
+            }
+            return (Insets) insets.clone();
+        } else {
+            return super.getScreenInsets(gc);
+        }
+    }
+
     /*
      * The current implementation of disabling background erasing for
      * canvases is that we don't set any native background color
@@ -1087,11 +1088,9 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
      * Returns the value of "sun.awt.disableGtkFileDialogs" property. Default
      * value is {@code false}.
      */
-    @SuppressWarnings("removal")
     public static synchronized boolean getSunAwtDisableGtkFileDialogs() {
         if (sunAwtDisableGtkFileDialogs == null) {
-            sunAwtDisableGtkFileDialogs = AccessController.doPrivileged(
-                                              new GetBooleanAction("sun.awt.disableGtkFileDialogs"));
+            sunAwtDisableGtkFileDialogs = Boolean.getBoolean("sun.awt.disableGtkFileDialogs");
         }
         return sunAwtDisableGtkFileDialogs.booleanValue();
     }
@@ -1232,11 +1231,6 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
 
     @Override
     public  Clipboard getSystemClipboard() {
-        @SuppressWarnings("removal")
-        SecurityManager security = System.getSecurityManager();
-        if (security != null) {
-            security.checkPermission(AWTPermissions.ACCESS_CLIPBOARD_PERMISSION);
-        }
         synchronized (this) {
             if (clipboard == null) {
                 clipboard = new XClipboard("System", "CLIPBOARD");
@@ -1247,11 +1241,6 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
 
     @Override
     public Clipboard getSystemSelection() {
-        @SuppressWarnings("removal")
-        SecurityManager security = System.getSecurityManager();
-        if (security != null) {
-            security.checkPermission(AWTPermissions.ACCESS_CLIPBOARD_PERMISSION);
-        }
         synchronized (this) {
             if (selection == null) {
                 selection = new XClipboard("Selection", "PRIMARY");
@@ -1488,7 +1477,7 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
             } catch (InterruptedException ie) {
             // Note: the returned timeStamp can be incorrect in this case.
                 if (log.isLoggable(PlatformLogger.Level.FINE)) {
-                    log.fine("Catched exception, timeStamp may not be correct (ie = " + ie + ")");
+                    log.fine("Caught exception, timeStamp may not be correct (ie = " + ie + ")");
                 }
             }
         } finally {
@@ -1906,7 +1895,7 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
      * @param task a Runnable which {@code run} method will be called
      *        on the toolkit thread when {@code interval} milliseconds
      *        elapse
-     * @param interval an interal in milliseconds
+     * @param interval an interval in milliseconds
      *
      * @throws NullPointerException if {@code task} is {@code null}
      * @throws IllegalArgumentException if {@code interval} is not positive
@@ -2106,9 +2095,7 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
     }
 
     private static void setBackingStoreType() {
-        @SuppressWarnings("removal")
-        String prop = AccessController.doPrivileged(
-                new sun.security.action.GetPropertyAction("sun.awt.backingStore"));
+        String prop = System.getProperty("sun.awt.backingStore");
 
         if (prop == null) {
             backingStoreType = XConstants.NotUseful;
@@ -2385,7 +2372,7 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
     private static int oops_position = 0;
 
     /**
-     * @inheritDoc
+     * {@inheritDoc}
      */
     @Override
     protected boolean syncNativeQueue(long timeout) {
@@ -2399,7 +2386,7 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
                     @Override
                     public void dispatchEvent(XEvent e) {
                         if (e.get_type() == XConstants.ConfigureNotify) {
-                            // OOPS ConfigureNotify event catched
+                            // OOPS ConfigureNotify event caught
                             oops_updated = true;
                             awtLockNotifyAll();
                         }
@@ -2532,8 +2519,7 @@ public final class XToolkit extends UNIXToolkit implements Runnable {
      * Returns the value of "sun.awt.disablegrab" property. Default
      * value is {@code false}.
      */
-    @SuppressWarnings("removal")
     public static boolean getSunAwtDisableGrab() {
-        return AccessController.doPrivileged(new GetBooleanAction("sun.awt.disablegrab"));
+        return Boolean.getBoolean("sun.awt.disablegrab");
     }
 }
