@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,8 +30,6 @@ import java.awt.event.*;
 import java.awt.peer.TrayIconPeer;
 import sun.awt.*;
 import java.awt.image.*;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.lang.reflect.InvocationTargetException;
 import sun.util.logging.PlatformLogger;
 
@@ -66,7 +64,6 @@ public class XTrayIconPeer implements TrayIconPeer,
     static final int TRAY_ICON_WIDTH = 24;
     static final int TRAY_ICON_HEIGHT = 24;
 
-    @SuppressWarnings("removal")
     XTrayIconPeer(TrayIcon target)
       throws AWTException
     {
@@ -84,12 +81,7 @@ public class XTrayIconPeer implements TrayIconPeer,
         // Fix for 6317038: as EmbeddedFrame is instance of Frame, it is blocked
         // by modal dialogs, but in the case of TrayIcon it shouldn't. So we
         // set ModalExclusion property on it.
-        AccessController.doPrivileged(new PrivilegedAction<Object>() {
-            public Object run() {
-                eframe.setModalExclusionType(Dialog.ModalExclusionType.TOOLKIT_EXCLUDE);
-                return null;
-            }
-        });
+        eframe.setModalExclusionType(Dialog.ModalExclusionType.TOOLKIT_EXCLUDE);
 
 
         if (XWM.getWMID() != XWM.METACITY_WM) {
@@ -285,8 +277,7 @@ public class XTrayIconPeer implements TrayIconPeer,
                             disposeOnEDT();
                         }
                     });
-            } catch (InterruptedException ie) {
-            } catch (InvocationTargetException ite) {}
+            } catch (InterruptedException | InvocationTargetException e) {}
         }
     }
 
@@ -540,7 +531,7 @@ public class XTrayIconPeer implements TrayIconPeer,
         public void realDispose(){
             super.dispose();
         }
-    };
+    }
 
     // ***************************************
     // Classes for painting an image on canvas
