@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,22 +25,25 @@
 
 package sun.awt;
 
-import java.awt.AWTPermission;
 import java.awt.DisplayMode;
 import java.awt.GraphicsConfiguration;
 import java.awt.GraphicsDevice;
 import java.awt.GraphicsEnvironment;
+import java.awt.Insets;
 import java.awt.Rectangle;
 import java.awt.Window;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Map;
+import java.util.Objects;
 
+import sun.awt.image.SurfaceManager;
 import sun.awt.util.ThreadGroupUtils;
 import sun.java2d.SunGraphicsEnvironment;
 import sun.java2d.loops.SurfaceType;
+import sun.awt.X11.XToolkit;
 import sun.java2d.opengl.GLXGraphicsConfig;
 import sun.java2d.pipe.Region;
 import sun.java2d.xr.XRGraphicsConfig;
@@ -59,19 +62,20 @@ public final class X11GraphicsDevice extends GraphicsDevice
      * therefore methods, which is using this id should be ready to it.
      */
     private volatile int screen;
-    HashMap<SurfaceType, Object> x11ProxyKeyMap = new HashMap<>();
+    Map<SurfaceType, SurfaceManager.ProxyCache> x11ProxyCacheMap = Collections.synchronizedMap(new HashMap<>());
 
-    private static AWTPermission fullScreenExclusivePermission;
     private static Boolean xrandrExtSupported;
-    private final Object configLock = new Object();
     private SunDisplayChanger topLevels = new SunDisplayChanger();
     private DisplayMode origDisplayMode;
+    private volatile Rectangle bounds;
+    private volatile Insets insets;
     private boolean shutdownHookRegistered;
     private int scale;
 
     public X11GraphicsDevice(int screennum) {
         this.screen = screennum;
         this.scale = initScaleFactor();
+        this.bounds = getBoundsImpl();
     }
 
     /**
@@ -81,15 +85,8 @@ public final class X11GraphicsDevice extends GraphicsDevice
         return screen;
     }
 
-    public Object getProxyKeyFor(SurfaceType st) {
-        synchronized (x11ProxyKeyMap) {
-            Object o = x11ProxyKeyMap.get(st);
-            if (o == null) {
-                o = new Object();
-                x11ProxyKeyMap.put(st, o);
-            }
-            return o;
-        }
+    public SurfaceManager.ProxyCache getProxyCacheFor(SurfaceType st) {
+        return x11ProxyCacheMap.computeIfAbsent(st, unused -> new SurfaceManager.ProxyCache());
     }
 
     /**
@@ -118,7 +115,7 @@ public final class X11GraphicsDevice extends GraphicsDevice
         return Region.clipRound(x / (double)getScaleFactor());
     }
 
-    public Rectangle getBounds() {
+    private Rectangle getBoundsImpl() {
         Rectangle rect = pGetBounds(getScreen());
         if (getScaleFactor() != 1) {
             rect.x = scaleDown(rect.x);
@@ -129,6 +126,23 @@ public final class X11GraphicsDevice extends GraphicsDevice
         return rect;
     }
 
+    public Rectangle getBounds() {
+        return bounds.getBounds();
+    }
+
+    public Insets getInsets() {
+        return insets;
+    }
+
+    public void setInsets(Insets newInsets) {
+        Objects.requireNonNull(newInsets);
+        insets = newInsets;
+    }
+
+    public void resetInsets() {
+        insets = null;
+    }
+
     /**
      * Returns the identification string associated with this graphics
      * device.
@@ -150,8 +164,11 @@ public final class X11GraphicsDevice extends GraphicsDevice
     @Override
     public GraphicsConfiguration[] getConfigurations() {
         if (configs == null) {
-            synchronized (configLock) {
+            XToolkit.awtLock();
+            try {
                 makeConfigurations();
+            } finally {
+                XToolkit.awtUnlock();
             }
         }
         return configs.clone();
@@ -209,7 +226,7 @@ public final class X11GraphicsDevice extends GraphicsDevice
     public native int getNumConfigs(int screen);
 
     /*
-     * Returns the visualid for the given index of graphics configurations.
+     * Returns the visualId for the given index of graphics configurations.
      */
     public native int getConfigVisualId (int index, int screen);
     /*
@@ -238,8 +255,11 @@ public final class X11GraphicsDevice extends GraphicsDevice
     @Override
     public GraphicsConfiguration getDefaultConfiguration() {
         if (defaultConfig == null) {
-            synchronized (configLock) {
+            XToolkit.awtLock();
+            try {
                 makeDefaultConfiguration();
+            } finally {
+                XToolkit.awtUnlock();
             }
         }
         return defaultConfig;
@@ -313,23 +333,7 @@ public final class X11GraphicsDevice extends GraphicsDevice
 
     @Override
     public boolean isFullScreenSupported() {
-        boolean fsAvailable = isXrandrExtensionSupported();
-        if (fsAvailable) {
-            @SuppressWarnings("removal")
-            SecurityManager security = System.getSecurityManager();
-            if (security != null) {
-                if (fullScreenExclusivePermission == null) {
-                    fullScreenExclusivePermission =
-                        new AWTPermission("fullScreenExclusive");
-                }
-                try {
-                    security.checkPermission(fullScreenExclusivePermission);
-                } catch (SecurityException e) {
-                    return false;
-                }
-            }
-        }
-        return fsAvailable;
+        return isXrandrExtensionSupported();
     }
 
     @Override
@@ -423,7 +427,6 @@ public final class X11GraphicsDevice extends GraphicsDevice
         return modes.toArray(retArray);
     }
 
-    @SuppressWarnings("removal")
     @Override
     public synchronized void setDisplayMode(DisplayMode dm) {
         if (!isDisplayChangeSupported()) {
@@ -450,24 +453,20 @@ public final class X11GraphicsDevice extends GraphicsDevice
             // is already in the original DisplayMode at that time, this
             // hook will have no effect)
             shutdownHookRegistered = true;
-            PrivilegedAction<Void> a = () -> {
-                Runnable r = () -> {
-                    Window old = getFullScreenWindow();
-                    if (old != null) {
-                        exitFullScreenExclusive(old);
-                        if (isDisplayChangeSupported()) {
-                            setDisplayMode(origDisplayMode);
-                        }
+            Runnable r = () -> {
+                Window old = getFullScreenWindow();
+                if (old != null) {
+                    exitFullScreenExclusive(old);
+                    if (isDisplayChangeSupported()) {
+                        setDisplayMode(origDisplayMode);
                     }
-                };
-                String name = "Display-Change-Shutdown-Thread-" + screen;
-                Thread t = new Thread(
-                      ThreadGroupUtils.getRootThreadGroup(), r, name, 0, false);
-                t.setContextClassLoader(null);
-                Runtime.getRuntime().addShutdownHook(t);
-                return null;
+                }
             };
-            AccessController.doPrivileged(a);
+            String name = "Display-Change-Shutdown-Thread-" + screen;
+            Thread t = new Thread(
+                  ThreadGroupUtils.getRootThreadGroup(), r, name, 0, false);
+            t.setContextClassLoader(null);
+            Runtime.getRuntime().addShutdownHook(t);
         }
 
         // switch to the new DisplayMode
@@ -510,6 +509,8 @@ public final class X11GraphicsDevice extends GraphicsDevice
     @Override
     public synchronized void displayChanged() {
         scale = initScaleFactor();
+        bounds = getBoundsImpl();
+        insets = null;
         // On X11 the visuals do not change, and therefore we don't need
         // to reset the defaultConfig, config, doubleBufferVisuals,
         // neither do we need to reset the native data.
@@ -571,6 +572,8 @@ public final class X11GraphicsDevice extends GraphicsDevice
     }
 
     public void invalidate(X11GraphicsDevice device) {
+        assert XToolkit.isAWTLockHeldByCurrentThread();
+
         screen = device.screen;
     }
 }
