@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -95,11 +95,8 @@ final class WPathGraphics extends PathGraphics {
     private static boolean useGDITextLayout = true;
     private static boolean preferGDITextLayout = false;
     static {
-        @SuppressWarnings("removal")
-        String textLayoutStr =
-            java.security.AccessController.doPrivileged(
-                   new sun.security.action.GetPropertyAction(
-                         "sun.java2d.print.enableGDITextLayout"));
+        String textLayoutStr = System.getProperty(
+                     "sun.java2d.print.enableGDITextLayout");
 
         if (textLayoutStr != null) {
             useGDITextLayout = Boolean.getBoolean(textLayoutStr);
@@ -175,7 +172,7 @@ final class WPathGraphics extends PathGraphics {
             penSize = new Point2D.Float(lineWidth, lineWidth);
 
             /* Compute the line width in device coordinates.
-             * Work on a point in case there is asymetric scaling
+             * Work on a point in case there is asymmetric scaling
              * between user and device space.
              * Take the absolute value in case there is negative
              * scaling in effect.
@@ -317,8 +314,8 @@ final class WPathGraphics extends PathGraphics {
     private static boolean isXP() {
         String osVersion = System.getProperty("os.version");
         if (osVersion != null) {
-            Float version = Float.valueOf(osVersion);
-            return (version.floatValue() >= 5.1f);
+            float version = Float.parseFloat(osVersion);
+            return version >= 5.1f;
         } else {
             return false;
         }
@@ -1122,7 +1119,7 @@ final class WPathGraphics extends PathGraphics {
                     bgcolor = null;
                 }
                 // if src region extends beyond the image, the "opaque" path
-                // may blit b/g colour (including white) where it shoudn't.
+                // may blit b/g colour (including white) where it shouldn't.
                 if ((srcX+srcWidth > img.getWidth(null) ||
                      srcY+srcHeight > img.getHeight(null))
                     && canDoRedraws()) {
@@ -1697,7 +1694,7 @@ final class WPathGraphics extends PathGraphics {
             /* selectStylePen is not supported, must be Win 9X */
             else {
 
-                /* let's see if we can use a a default pen
+                /* let's see if we can use the default pen
                  *  if it's round end (Windows' default style)
                  *  or it's vertical/horizontal
                  *  or stroke is too thin.
