@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -102,17 +102,36 @@ extern void DWMResetCompositionEnabled();
    first loaded */
 JavaVM *jvm = NULL;
 
+/* Return a handle to the module containing this method, either a DLL in case
+ * of a dynamic library build, or the .EXE in case of a static build.
+ */
+static HMODULE GetAwtModuleHandle() {
+    HMODULE hModule = NULL;
+    GetModuleHandleEx(
+        GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT,
+        (LPCTSTR) &GetAwtModuleHandle,
+        &hModule
+    );
+    return hModule;
+}
+
+extern "C" {
+
 JNIEXPORT jint JNICALL
 DEF_JNI_OnLoad(JavaVM *vm, void *reserved)
 {
     TRY;
 
+    AwtToolkit::GetInstance().SetModuleHandle(GetAwtModuleHandle());
+
     jvm = vm;
     return JNI_VERSION_1_2;
 
     CATCH_BAD_ALLOC_RET(0);
 }
 
+}
+
 extern "C" JNIEXPORT jboolean JNICALL AWTIsHeadless() {
     static JNIEnv *env = NULL;
     static jboolean isHeadless;
@@ -246,32 +265,6 @@ BOOL AwtToolkit::activateKeyboardLayout(HKL hkl) {
     return (prev != 0);
 }
 
-/************************************************************************
- * Exported functions
- */
-
-extern "C" BOOL APIENTRY DllMain(HANDLE hInstance, DWORD ul_reason_for_call,
-                                 LPVOID)
-{
-    // Don't use the TRY and CATCH_BAD_ALLOC_RET macros if we're detaching
-    // the library. Doing so causes awt.dll to call back into the VM during
-    // shutdown. This crashes the HotSpot VM.
-    switch (ul_reason_for_call) {
-    case DLL_PROCESS_ATTACH:
-        TRY;
-        AwtToolkit::GetInstance().SetModuleHandle((HMODULE)hInstance);
-        CATCH_BAD_ALLOC_RET(FALSE);
-        break;
-    case DLL_PROCESS_DETACH:
-#ifdef DEBUG
-        DTrace_DisableMutex();
-        DMem_DisableMutex();
-#endif // DEBUG
-        break;
-    }
-    return TRUE;
-}
-
 /************************************************************************
  * AwtToolkit fields
  */
@@ -1438,24 +1431,6 @@ LRESULT CALLBACK AwtToolkit::MouseLowLevelHook(int code,
             }
 
             tk.m_lastWindowUnderMouse = hwnd;
-
-            if (fw) {
-                fw->UpdateSecurityWarningVisibility();
-            }
-            // ... however, because we use GA_ROOT, we may find the warningIcon
-            // which is not a Java windows.
-            if (AwtWindow::IsWarningWindow(hwnd)) {
-                hwnd = ::GetParent(hwnd);
-                if (hwnd) {
-                    tw = (AwtWindow*)AwtComponent::GetComponent(hwnd);
-                }
-                tk.m_lastWindowUnderMouse = hwnd;
-            }
-            if (tw) {
-                tw->UpdateSecurityWarningVisibility();
-            }
-
-
         }
     }
 
@@ -1909,48 +1884,6 @@ HICON AwtToolkit::GetAwtIconSm()
     return defaultIconSm;
 }
 
-// The icon at index 0 must be gray. See AwtWindow::GetSecurityWarningIcon()
-HICON AwtToolkit::GetSecurityWarningIcon(UINT index, UINT w, UINT h)
-{
-    //Note: should not exceed 10 because of the current implementation.
-    static const int securityWarningIconCounter = 3;
-
-    static HICON securityWarningIcon[securityWarningIconCounter]      = {NULL, NULL, NULL};
-    static UINT securityWarningIconWidth[securityWarningIconCounter]  = {0, 0, 0};
-    static UINT securityWarningIconHeight[securityWarningIconCounter] = {0, 0, 0};
-
-    index = AwtToolkit::CalculateWave(index, securityWarningIconCounter);
-
-    if (securityWarningIcon[index] == NULL ||
-            w != securityWarningIconWidth[index] ||
-            h != securityWarningIconHeight[index])
-    {
-        if (securityWarningIcon[index] != NULL)
-        {
-            ::DestroyIcon(securityWarningIcon[index]);
-        }
-
-        static const wchar_t securityWarningIconName[] = L"SECURITY_WARNING_";
-        wchar_t iconResourceName[sizeof(securityWarningIconName) + 2];
-        ::ZeroMemory(iconResourceName, sizeof(iconResourceName));
-        wcscpy(iconResourceName, securityWarningIconName);
-
-        wchar_t strIndex[2];
-        ::ZeroMemory(strIndex, sizeof(strIndex));
-        strIndex[0] = L'0' + index;
-
-        wcscat(iconResourceName, strIndex);
-
-        securityWarningIcon[index] = (HICON)::LoadImage(GetModuleHandle(),
-                iconResourceName,
-                IMAGE_ICON, w, h, LR_DEFAULTCOLOR);
-        securityWarningIconWidth[index] = w;
-        securityWarningIconHeight[index] = h;
-    }
-
-    return securityWarningIcon[index];
-}
-
 void throw_if_shutdown(void)
 {
     AwtToolkit::GetInstance().VerifyActive();
@@ -2195,7 +2128,7 @@ bool AwtToolkit::PreloadThread::Terminate(bool wrongThread)
     return true;
 }
 
-bool AwtToolkit::PreloadThread::InvokeAndTerminate(void(_cdecl *fn)(void *), void *param)
+bool AwtToolkit::PreloadThread::InvokeAndTerminate(void(*fn)(void *), void *param)
 {
     CriticalSection::Lock lock(threadLock);
 
@@ -2225,7 +2158,7 @@ unsigned WINAPI AwtToolkit::PreloadThread::StaticThreadProc(void *param)
 
 unsigned AwtToolkit::PreloadThread::ThreadProc()
 {
-    void(_cdecl *_execFunc)(void *) = NULL;
+    void(*_execFunc)(void *) = NULL;
     void *_execParam = NULL;
     bool _wrongThread = false;
 
