@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates. All rights reserved.
  */
 /*
  * Licensed to the Apache Software Foundation (ASF) under one or more
@@ -23,6 +23,7 @@ package com.sun.org.apache.xerces.internal.impl.xpath.regex;
 import java.text.CharacterIterator;
 import java.util.Locale;
 import java.util.Stack;
+import java.util.concurrent.Semaphore;
 
 import com.sun.org.apache.xerces.internal.util.IntStack;
 
@@ -704,19 +705,17 @@ public class RegularExpression implements java.io.Serializable {
      */
     public boolean matches(char[] target, int start, int end, Match match) {
 
-        if (this.operations == null || this.context == null) {
-            synchronized (this) {
-                if (this.operations == null)
-                    this.prepare();
-                if (this.context == null)
-                    this.context = new Context();
-            }
+        synchronized (this) {
+            if (this.operations == null)
+                this.prepare();
+            if (this.context == null)
+                this.context = new Context();
         }
-        Context con = null;
-        synchronized (this.context) {
-            con = this.context.inuse ? new Context() : this.context;
-            con.reset(target, start, end, this.numberOfClosures);
+        Context con = this.context;
+        if (!con.claim()) {
+            con = new Context();
         }
+        con.reset(target, start, end, this.numberOfClosures);
         if (match != null) {
             match.setNumberOfGroups(this.nofparen);
             match.setSource(target);
@@ -736,7 +735,7 @@ public class RegularExpression implements java.io.Serializable {
                     con.match.setBeginning(0, con.start);
                     con.match.setEnd(0, matchEnd);
                 }
-                con.setInUse(false);
+                con.release();
                 return true;
             }
             return false;
@@ -754,10 +753,10 @@ public class RegularExpression implements java.io.Serializable {
                     con.match.setBeginning(0, o);
                     con.match.setEnd(0, o+this.fixedString.length());
                 }
-                con.setInUse(false);
+                con.release();
                 return true;
             }
-            con.setInUse(false);
+            con.release();
             return false;
         }
 
@@ -770,7 +769,7 @@ public class RegularExpression implements java.io.Serializable {
             int o = this.fixedStringTable.matches(target, con.start, con.limit);
             if (o < 0) {
                 //System.err.println("Non-match in fixed-string search.");
-                con.setInUse(false);
+                con.release();
                 return false;
             }
         }
@@ -841,10 +840,10 @@ public class RegularExpression implements java.io.Serializable {
                 con.match.setBeginning(0, matchStart);
                 con.match.setEnd(0, matchEnd);
             }
-            con.setInUse(false);
+            con.release();
             return true;
         } else {
-            con.setInUse(false);
+            con.release();
             return false;
         }
     }
@@ -891,19 +890,17 @@ public class RegularExpression implements java.io.Serializable {
      */
     public boolean matches(String  target, int start, int end, Match match) {
 
-        if (this.operations == null || this.context == null) {
-            synchronized (this) {
-                if (this.operations == null)
-                    this.prepare();
-                if (this.context == null)
-                    this.context = new Context();
-            }
+        synchronized (this) {
+            if (this.operations == null)
+                this.prepare();
+            if (this.context == null)
+                this.context = new Context();
         }
-        Context con = null;
-        synchronized (this.context) {
-            con = this.context.inuse ? new Context() : this.context;
-            con.reset(target, start, end, this.numberOfClosures);
+        Context con = this.context;
+        if (!con.claim()) {
+            con = new Context();
         }
+        con.reset(target, start, end, this.numberOfClosures);
         if (match != null) {
             match.setNumberOfGroups(this.nofparen);
             match.setSource(target);
@@ -929,7 +926,7 @@ public class RegularExpression implements java.io.Serializable {
                     con.match.setBeginning(0, con.start);
                     con.match.setEnd(0, matchEnd);
                 }
-                con.setInUse(false);
+                con.release();
                 return true;
             }
             return false;
@@ -947,10 +944,10 @@ public class RegularExpression implements java.io.Serializable {
                     con.match.setBeginning(0, o);
                     con.match.setEnd(0, o+this.fixedString.length());
                 }
-                con.setInUse(false);
+                con.release();
                 return true;
             }
-            con.setInUse(false);
+            con.release();
             return false;
         }
 
@@ -963,7 +960,7 @@ public class RegularExpression implements java.io.Serializable {
             int o = this.fixedStringTable.matches(target, con.start, con.limit);
             if (o < 0) {
                 //System.err.println("Non-match in fixed-string search.");
-                con.setInUse(false);
+                con.release();
                 return false;
             }
         }
@@ -1034,10 +1031,10 @@ public class RegularExpression implements java.io.Serializable {
                 con.match.setBeginning(0, matchStart);
                 con.match.setEnd(0, matchEnd);
             }
-            con.setInUse(false);
+            con.release();
             return true;
         } else {
-            con.setInUse(false);
+            con.release();
             return false;
         }
     }
@@ -1573,19 +1570,17 @@ public class RegularExpression implements java.io.Serializable {
 
 
 
-        if (this.operations == null || this.context == null) {
-            synchronized (this) {
-                if (this.operations == null)
-                    this.prepare();
-                if (this.context == null)
-                    this.context = new Context();
-            }
+        synchronized (this) {
+            if (this.operations == null)
+                this.prepare();
+            if (this.context == null)
+                this.context = new Context();
         }
-        Context con = null;
-        synchronized (this.context) {
-            con = this.context.inuse ? new Context() : this.context;
-            con.reset(target, start, end, this.numberOfClosures);
+        Context con = this.context;
+        if (!con.claim()) {
+            con = new Context();
         }
+        con.reset(target, start, end, this.numberOfClosures);
         if (match != null) {
             match.setNumberOfGroups(this.nofparen);
             match.setSource(target);
@@ -1605,7 +1600,7 @@ public class RegularExpression implements java.io.Serializable {
                     con.match.setBeginning(0, con.start);
                     con.match.setEnd(0, matchEnd);
                 }
-                con.setInUse(false);
+                con.release();
                 return true;
             }
             return false;
@@ -1623,10 +1618,10 @@ public class RegularExpression implements java.io.Serializable {
                     con.match.setBeginning(0, o);
                     con.match.setEnd(0, o+this.fixedString.length());
                 }
-                con.setInUse(false);
+                con.release();
                 return true;
             }
-            con.setInUse(false);
+            con.release();
             return false;
         }
 
@@ -1639,7 +1634,7 @@ public class RegularExpression implements java.io.Serializable {
             int o = this.fixedStringTable.matches(target, con.start, con.limit);
             if (o < 0) {
                 //System.err.println("Non-match in fixed-string search.");
-                con.setInUse(false);
+                con.release();
                 return false;
             }
         }
@@ -1710,10 +1705,10 @@ public class RegularExpression implements java.io.Serializable {
                 con.match.setBeginning(0, matchStart);
                 con.match.setEnd(0, matchEnd);
             }
-            con.setInUse(false);
+            con.release();
             return true;
         } else {
-            con.setInUse(false);
+            con.release();
             return false;
         }
     }
@@ -1744,9 +1739,9 @@ public class RegularExpression implements java.io.Serializable {
     boolean hasBackReferences = false;
 
     transient int minlength;
-    transient volatile Op operations = null;
+    transient Op operations = null;
     transient int numberOfClosures;
-    transient volatile Context context = null;
+    transient Context context = null;
     transient RangeToken firstChar = null;
 
     transient String fixedString = null;
@@ -2021,7 +2016,7 @@ public class RegularExpression implements java.io.Serializable {
         int limit;
         int length;
         Match match;
-        boolean inuse = false;
+        private final Semaphore inuse = new Semaphore(1);
         ClosureContext[] closureContexts;
 
         private StringTarget stringTarget;
@@ -2035,7 +2030,6 @@ public class RegularExpression implements java.io.Serializable {
 
         private void resetCommon(int nofclosures) {
             this.length = this.limit-this.start;
-            setInUse(true);
             this.match = null;
             if (this.closureContexts == null || this.closureContexts.length != nofclosures) {
                 this.closureContexts = new ClosureContext[nofclosures];
@@ -2088,8 +2082,13 @@ public class RegularExpression implements java.io.Serializable {
             this.limit = limit;
             this.resetCommon(nofclosures);
         }
-        synchronized void setInUse(boolean inUse) {
-            this.inuse = inUse;
+
+        boolean claim() {
+            return inuse.tryAcquire();
+        }
+
+        void release() {
+            inuse.release();
         }
     }
 
