@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,7 +32,7 @@ import com.sun.tools.javac.code.Kinds.KindSelector;
 import com.sun.tools.javac.code.Scope.WriteableScope;
 import com.sun.tools.javac.code.Source.Feature;
 import com.sun.tools.javac.code.Symbol.*;
-import com.sun.tools.javac.code.TypeMetadata.Entry.Kind;
+import com.sun.tools.javac.code.TypeMetadata.Annotations;
 import com.sun.tools.javac.comp.Check.CheckContext;
 import com.sun.tools.javac.resources.CompilerProperties.Errors;
 import com.sun.tools.javac.resources.CompilerProperties.Fragments;
@@ -101,9 +101,9 @@ public class Annotate {
     private final Types types;
 
     private final Attribute theUnfinishedDefaultValue;
-    private final boolean allowRepeatedAnnos;
     private final String sourceName;
 
+    @SuppressWarnings("this-escape")
     protected Annotate(Context context) {
         context.put(annotateKey, this);
 
@@ -124,7 +124,6 @@ public class Annotate {
         theUnfinishedDefaultValue =  new Attribute.Error(syms.errType);
 
         Source source = Source.instance(context);
-        allowRepeatedAnnos = Feature.REPEATED_ANNOTATIONS.allowedInSource(source);
         sourceName = source.name;
 
         blockCount = 1;
@@ -230,11 +229,11 @@ public class Annotate {
      *
      * @param annotations the list of JCAnnotations to attribute and enter
      * @param localEnv    the enclosing env
-     * @param s           ths Symbol on which to enter the annotations
-     * @param deferPos    report errors here
+     * @param s           the Symbol on which to enter the annotations
+     * @param deferDecl   enclosing declaration for DeferredLintHandler, or null for no deferral
      */
     public void annotateLater(List<JCAnnotation> annotations, Env<AttrContext> localEnv,
-            Symbol s, DiagnosticPosition deferPos)
+            Symbol s, JCTree deferDecl)
     {
         if (annotations.isEmpty()) {
             return;
@@ -252,11 +251,8 @@ public class Annotate {
             // been handled, meaning that the set of annotations pending completion is now empty.
             Assert.check(s.kind == PCK || s.annotationsPendingCompletion());
             JavaFileObject prev = log.useSource(localEnv.toplevel.sourcefile);
-            DiagnosticPosition prevLintPos =
-                    deferPos != null
-                            ? deferredLintHandler.setPos(deferPos)
-                            : deferredLintHandler.immediate();
-            Lint prevLint = deferPos != null ? null : chk.setLint(lint);
+            Assert.check(deferDecl != null);
+            deferredLintHandler.push(deferDecl);
             try {
                 if (s.hasAnnotations() && annotations.nonEmpty())
                     log.error(annotations.head.pos, Errors.AlreadyAnnotated(Kinds.kindName(s), s));
@@ -267,9 +263,7 @@ public class Annotate {
                 // never called for a type parameter
                 annotateNow(s, annotations, localEnv, false, false);
             } finally {
-                if (prevLint != null)
-                    chk.setLint(prevLint);
-                deferredLintHandler.setPos(prevLintPos);
+                deferredLintHandler.pop();
                 log.useSource(prev);
             }
         });
@@ -287,15 +281,15 @@ public class Annotate {
 
     /** Queue processing of an attribute default value. */
     public void annotateDefaultValueLater(JCExpression defaultValue, Env<AttrContext> localEnv,
-            MethodSymbol m, DiagnosticPosition deferPos)
+            MethodSymbol m, JCTree deferDecl)
     {
         normal(() -> {
             JavaFileObject prev = log.useSource(localEnv.toplevel.sourcefile);
-            DiagnosticPosition prevLintPos = deferredLintHandler.setPos(deferPos);
+            deferredLintHandler.push(deferDecl);
             try {
                 enterDefaultValue(defaultValue, localEnv, m);
             } finally {
-                deferredLintHandler.setPos(prevLintPos);
+                deferredLintHandler.pop();
                 log.useSource(prev);
             }
         });
@@ -347,9 +341,6 @@ public class Annotate {
 
             if (a.type.isErroneous() || a.type.tsym.isAnnotationType()) {
                 if (annotated.containsKey(a.type.tsym)) {
-                    if (!allowRepeatedAnnos) {
-                        log.error(DiagnosticFlag.SOURCE_LEVEL, a.pos(), Feature.REPEATED_ANNOTATIONS.error(sourceName));
-                    }
                     ListBuffer<T> l = annotated.get(a.type.tsym);
                     l = l.append(c);
                     annotated.put(a.type.tsym, l);
@@ -383,6 +374,11 @@ public class Annotate {
                     && types.isSameType(c.type, syms.valueBasedType)) {
                 toAnnotate.flags_field |= Flags.VALUE_BASED;
             }
+
+            if (!c.type.isErroneous()
+                    && types.isSameType(c.type, syms.restrictedType)) {
+                toAnnotate.flags_field |= Flags.RESTRICTED;
+            }
         }
 
         List<T> buf = List.nil();
@@ -442,7 +438,7 @@ public class Annotate {
      *
      * @param a the tree representing an annotation
      * @param expectedAnnotationType the expected (super)type of the annotation
-     * @param env the the current env in where the annotation instance is found
+     * @param env the current env in where the annotation instance is found
      */
     public Attribute.TypeCompound attributeTypeAnnotation(JCAnnotation a, Type expectedAnnotationType,
                                                           Env<AttrContext> env)
@@ -526,6 +522,7 @@ public class Annotate {
                 left.name, List.nil(), null);
         left.sym = method;
         left.type = method.type;
+        chk.checkDeprecated(left, env.info.scope.owner, method);
         if (method.owner != thisAnnotationType.tsym && !badAnnotation)
             log.error(left.pos(), Errors.NoAnnotationMember(left.name, thisAnnotationType));
         Type resultType = method.type.getReturnType();
@@ -554,7 +551,6 @@ public class Annotate {
 
         if (expectedElementType.hasTag(ARRAY)) {
             return getAnnotationArrayValue(expectedElementType, tree, env);
-
         }
 
         //error recovery
@@ -675,7 +671,7 @@ public class Annotate {
 
         // Scan the annotation element value and then attribute nested annotations if present
         if (tree.type != null && tree.type.tsym != null) {
-            queueScanTreeAndTypeAnnotate(tree, env, tree.type.tsym, tree.pos());
+            queueScanTreeAndTypeAnnotate(tree, env, tree.type.tsym, null);
         }
 
         result = cfolder.coerce(result, expectedElementType);
@@ -706,11 +702,15 @@ public class Annotate {
         }
 
         JCNewArray na = (JCNewArray)tree;
+        List<JCExpression> elems = na.elems;
         if (na.elemtype != null) {
             log.error(na.elemtype.pos(), Errors.NewNotAllowedInAnnotation);
+            if (elems == null) {
+                elems = List.nil();
+            }
         }
         ListBuffer<Attribute> buf = new ListBuffer<>();
-        for (List<JCExpression> l = na.elems; l.nonEmpty(); l=l.tail) {
+        for (List<JCExpression> l = elems; l.nonEmpty(); l = l.tail) {
             buf.append(attributeAnnotationValue(types.elemtype(expectedElementType),
                     l.head,
                     env));
@@ -940,7 +940,13 @@ public class Annotate {
         boolean fatalError = false;
 
         // Validate that there is a (and only 1) value method
-        Scope scope = targetContainerType.tsym.members();
+        Scope scope = null;
+        try {
+            scope = targetContainerType.tsym.members();
+        } catch (CompletionFailure ex) {
+            chk.completionError(pos, ex);
+            return null;
+        }
         int nr_value_elems = 0;
         boolean error = false;
         for(Symbol elm : scope.getSymbolsByName(names.value)) {
@@ -1010,7 +1016,7 @@ public class Annotate {
         return validRepeated;
     }
 
-    /********************
+    /* ******************
      * Type annotations *
      ********************/
 
@@ -1018,20 +1024,19 @@ public class Annotate {
      * Attribute the list of annotations and enter them onto s.
      */
     public void enterTypeAnnotations(List<JCAnnotation> annotations, Env<AttrContext> env,
-            Symbol s, DiagnosticPosition deferPos, boolean isTypeParam)
+            Symbol s, JCTree deferDecl, boolean isTypeParam)
     {
         Assert.checkNonNull(s, "Symbol argument to actualEnterTypeAnnotations is nul/");
         JavaFileObject prev = log.useSource(env.toplevel.sourcefile);
-        DiagnosticPosition prevLintPos = null;
 
-        if (deferPos != null) {
-            prevLintPos = deferredLintHandler.setPos(deferPos);
+        if (deferDecl != null) {
+            deferredLintHandler.push(deferDecl);
         }
         try {
             annotateNow(s, annotations, env, true, isTypeParam);
         } finally {
-            if (prevLintPos != null)
-                deferredLintHandler.setPos(prevLintPos);
+            if (deferDecl != null)
+                deferredLintHandler.pop();
             log.useSource(prev);
         }
     }
@@ -1039,11 +1044,10 @@ public class Annotate {
     /**
      * Enqueue tree for scanning of type annotations, attaching to the Symbol sym.
      */
-    public void queueScanTreeAndTypeAnnotate(JCTree tree, Env<AttrContext> env, Symbol sym,
-            DiagnosticPosition deferPos)
+    public void queueScanTreeAndTypeAnnotate(JCTree tree, Env<AttrContext> env, Symbol sym, JCTree deferDecl)
     {
         Assert.checkNonNull(sym);
-        normal(() -> tree.accept(new TypeAnnotate(env, sym, deferPos)));
+        normal(() -> tree.accept(new TypeAnnotate(env, sym, deferDecl)));
     }
 
     /**
@@ -1053,7 +1057,11 @@ public class Annotate {
         typeAnnotation(() -> {
             List<Attribute.TypeCompound> compounds = fromAnnotations(annotations);
             Assert.check(annotations.size() == compounds.size());
-            storeAt.getMetadataOfKind(Kind.ANNOTATIONS).combine(new TypeMetadata.Annotations(compounds));
+            // the type already has annotation metadata, but it's empty
+            Annotations metadata = storeAt.getMetadata(Annotations.class);
+            Assert.checkNonNull(metadata);
+            Assert.check(metadata.annotationBuffer().isEmpty());
+            metadata.annotationBuffer().appendList(compounds);
         });
     }
 
@@ -1074,32 +1082,32 @@ public class Annotate {
     private class TypeAnnotate extends TreeScanner {
         private final Env<AttrContext> env;
         private final Symbol sym;
-        private DiagnosticPosition deferPos;
+        private JCTree deferDecl;
 
-        public TypeAnnotate(Env<AttrContext> env, Symbol sym, DiagnosticPosition deferPos) {
+        public TypeAnnotate(Env<AttrContext> env, Symbol sym, JCTree deferDecl) {
 
             this.env = env;
             this.sym = sym;
-            this.deferPos = deferPos;
+            this.deferDecl = deferDecl;
         }
 
         @Override
         public void visitAnnotatedType(JCAnnotatedType tree) {
-            enterTypeAnnotations(tree.annotations, env, sym, deferPos, false);
+            enterTypeAnnotations(tree.annotations, env, sym, deferDecl, false);
             scan(tree.underlyingType);
         }
 
         @Override
         public void visitTypeParameter(JCTypeParameter tree) {
-            enterTypeAnnotations(tree.annotations, env, sym, deferPos, true);
+            enterTypeAnnotations(tree.annotations, env, sym, deferDecl, true);
             scan(tree.bounds);
         }
 
         @Override
         public void visitNewArray(JCNewArray tree) {
-            enterTypeAnnotations(tree.annotations, env, sym, deferPos, false);
+            enterTypeAnnotations(tree.annotations, env, sym, deferDecl, false);
             for (List<JCAnnotation> dimAnnos : tree.dimAnnotations)
-                enterTypeAnnotations(dimAnnos, env, sym, deferPos, false);
+                enterTypeAnnotations(dimAnnos, env, sym, deferDecl, false);
             scan(tree.elemtype);
             scan(tree.elems);
         }
@@ -1118,8 +1126,8 @@ public class Annotate {
 
         @Override
         public void visitVarDef(JCVariableDecl tree) {
-            DiagnosticPosition prevPos = deferPos;
-            deferPos = tree.pos();
+            JCTree prevDecl = deferDecl;
+            deferDecl = tree;
             try {
                 if (sym != null && sym.kind == VAR) {
                     // Don't visit a parameter once when the sym is the method
@@ -1129,7 +1137,7 @@ public class Annotate {
                 }
                 scan(tree.init);
             } finally {
-                deferPos = prevPos;
+                deferDecl = prevDecl;
             }
         }
 
@@ -1157,9 +1165,18 @@ public class Annotate {
             scan(tree.args);
             // the anonymous class instantiation if any will be visited separately.
         }
+
+        @Override
+        public void visitErroneous(JCErroneous tree) {
+            if (tree.errs != null) {
+                for (JCTree err : tree.errs) {
+                    scan(err);
+                }
+            }
+        }
     }
 
-    /*********************
+    /* *******************
      * Completer support *
      *********************/
 
