@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -114,6 +114,7 @@ public class Enter extends JCTree.Visitor {
         return instance;
     }
 
+    @SuppressWarnings("this-escape")
     protected Enter(Context context) {
         context.put(enterKey, this);
 
@@ -170,6 +171,11 @@ public class Enter extends JCTree.Visitor {
      */
     ListBuffer<ClassSymbol> uncompleted;
 
+    /** The queue of classes that should have typeEnter completer installed after
+     *  all the classes are discovered.
+     */
+    List<ClassSymbol> pendingCompleter = null;
+
     /** The queue of modules whose imports still need to be checked. */
     ListBuffer<JCCompilationUnit> unfinishedModules = new ListBuffer<>();
 
@@ -200,10 +206,10 @@ public class Enter extends JCTree.Visitor {
             env.dup(tree, env.info.dup(WriteableScope.create(tree.sym)));
         localEnv.enclClass = tree;
         localEnv.outer = env;
-        localEnv.info.isSelfCall = false;
         localEnv.info.lint = null; // leave this to be filled in by Attr,
                                    // when annotations have been processed
         localEnv.info.isAnonymousDiamond = TreeInfo.isDiamond(env.tree);
+        localEnv.info.ctorPrologue = false;
         return localEnv;
     }
 
@@ -217,6 +223,7 @@ public class Enter extends JCTree.Visitor {
         tree.toplevelScope = WriteableScope.create(tree.packge);
         tree.namedImportScope = new NamedImportScope(tree.packge);
         tree.starImportScope = new StarImportScope(tree.packge);
+        tree.moduleImportScope = new StarImportScope(tree.packge);
         localEnv.info.scope = tree.toplevelScope;
         localEnv.info.lint = lint;
         return localEnv;
@@ -253,7 +260,6 @@ public class Enter extends JCTree.Visitor {
             env.dup(tree, env.info.dup(WriteableScope.create(tree.sym)));
         localEnv.enclClass = predefClassDef;
         localEnv.outer = env;
-        localEnv.info.isSelfCall = false;
         localEnv.info.lint = null; // leave this to be filled in by Attr,
                                    // when annotations have been processed
         return localEnv;
@@ -307,8 +313,6 @@ public class Enter extends JCTree.Visitor {
 
     @Override
     public void visitTopLevel(JCCompilationUnit tree) {
-//        Assert.checkNonNull(tree.modle, tree.sourcefile.toString());
-
         JavaFileObject prev = log.useSource(tree.sourcefile);
         boolean addEnv = false;
         boolean isPkgInfo = tree.sourcefile.isNameCompatible("package-info",
@@ -356,10 +360,12 @@ public class Enter extends JCTree.Visitor {
             tree.packge.complete(); // Find all classes in package.
 
             Env<AttrContext> topEnv = topLevelEnv(tree);
-            Env<AttrContext> packageEnv = isPkgInfo ? topEnv.dup(pd) : null;
+            Env<AttrContext> packageEnv = null;
 
             // Save environment of package-info.java file.
             if (isPkgInfo) {
+                packageEnv = topEnv.dup(pd != null ? pd : tree);
+
                 Env<AttrContext> env0 = typeEnvs.get(tree.packge);
                 if (env0 != null) {
                     JCCompilationUnit tree0 = env0.toplevel;
@@ -438,6 +444,9 @@ public class Enter extends JCTree.Visitor {
                 log.error(tree.pos(),
                           Errors.ClassPublicShouldBeInFile(topElement, tree.name));
             }
+            if ((tree.mods.flags & IMPLICIT_CLASS) != 0) {
+                syms.removeClass(env.toplevel.modle, tree.name);
+            }
         } else {
             if (!tree.name.isEmpty() &&
                 !chk.checkUniqueClassName(tree.pos(), tree.name, enclScope)) {
@@ -520,8 +529,12 @@ public class Enter extends JCTree.Visitor {
         ct.typarams_field = classEnter(tree.typarams, localEnv);
         ct.allparams_field = null;
 
-        // install further completer for this type.
-        c.completer = typeEnter;
+        // schedule installation of further completer for this type.
+        if (pendingCompleter != null) {
+            pendingCompleter = pendingCompleter.prepend(c);
+        } else {
+            c.completer = typeEnter;
+        }
 
         // Add non-local class to uncompleted, to make sure it will be
         // completed later.
@@ -600,8 +613,18 @@ public class Enter extends JCTree.Visitor {
         if (typeEnter.completionEnabled) uncompleted = new ListBuffer<>();
 
         try {
-            // enter all classes, and construct uncompleted list
-            classEnter(trees, null);
+            List<ClassSymbol> prevPendingCompleter = pendingCompleter;
+            try {
+                pendingCompleter = List.nil();
+                // enter all classes, and construct uncompleted list
+                classEnter(trees, null);
+                // install further completer for classes recognized by the above task:
+                for (ClassSymbol sym : pendingCompleter) {
+                    sym.completer = typeEnter;
+                }
+            } finally {
+                pendingCompleter = prevPendingCompleter;
+            }
 
             // complete all uncompleted classes in memberEnter
             if (typeEnter.completionEnabled) {
