@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -52,7 +52,6 @@ import javax.tools.JavaFileObject.Kind;
 import javax.tools.StandardLocation;
 
 import com.sun.source.tree.ModuleTree.ModuleKind;
-import com.sun.tools.javac.code.ClassFinder;
 import com.sun.tools.javac.code.DeferredLintHandler;
 import com.sun.tools.javac.code.Directive;
 import com.sun.tools.javac.code.Directive.ExportsDirective;
@@ -64,8 +63,11 @@ import com.sun.tools.javac.code.Directive.RequiresFlag;
 import com.sun.tools.javac.code.Directive.UsesDirective;
 import com.sun.tools.javac.code.Flags;
 import com.sun.tools.javac.code.Flags.Flag;
+import com.sun.tools.javac.code.Kinds;
+import com.sun.tools.javac.code.Lint;
 import com.sun.tools.javac.code.Lint.LintCategory;
 import com.sun.tools.javac.code.ModuleFinder;
+import com.sun.tools.javac.code.Preview;
 import com.sun.tools.javac.code.Source;
 import com.sun.tools.javac.code.Source.Feature;
 import com.sun.tools.javac.code.Symbol;
@@ -74,6 +76,7 @@ import com.sun.tools.javac.code.Symbol.Completer;
 import com.sun.tools.javac.code.Symbol.CompletionFailure;
 import com.sun.tools.javac.code.Symbol.MethodSymbol;
 import com.sun.tools.javac.code.Symbol.ModuleFlags;
+import com.sun.tools.javac.code.Symbol.ModuleResolutionFlags;
 import com.sun.tools.javac.code.Symbol.ModuleSymbol;
 import com.sun.tools.javac.code.Symbol.PackageSymbol;
 import com.sun.tools.javac.code.Symtab;
@@ -84,6 +87,7 @@ import com.sun.tools.javac.jvm.JNIWriter;
 import com.sun.tools.javac.jvm.Target;
 import com.sun.tools.javac.main.Option;
 import com.sun.tools.javac.resources.CompilerProperties.Errors;
+import com.sun.tools.javac.resources.CompilerProperties.LintWarnings;
 import com.sun.tools.javac.resources.CompilerProperties.Warnings;
 import com.sun.tools.javac.tree.JCTree;
 import com.sun.tools.javac.tree.JCTree.JCCompilationUnit;
@@ -112,14 +116,10 @@ import static com.sun.tools.javac.code.Flags.ENUM;
 import static com.sun.tools.javac.code.Flags.PUBLIC;
 import static com.sun.tools.javac.code.Flags.UNATTRIBUTED;
 
-import com.sun.tools.javac.code.Kinds;
-
 import static com.sun.tools.javac.code.Kinds.Kind.ERR;
 import static com.sun.tools.javac.code.Kinds.Kind.MDL;
 import static com.sun.tools.javac.code.Kinds.Kind.MTH;
 
-import com.sun.tools.javac.code.Symbol.ModuleResolutionFlags;
-
 import static com.sun.tools.javac.code.TypeTag.CLASS;
 
 /**
@@ -134,11 +134,13 @@ public class Modules extends JCTree.Visitor {
     private static final String ALL_SYSTEM = "ALL-SYSTEM";
     private static final String ALL_MODULE_PATH = "ALL-MODULE-PATH";
 
+    private final Lint lint;
     private final Log log;
     private final Names names;
     private final Symtab syms;
     private final Attr attr;
     private final Check chk;
+    private final Preview preview;
     private final DeferredLintHandler deferredLintHandler;
     private final TypeEnvs typeEnvs;
     private final Types types;
@@ -181,13 +183,16 @@ public class Modules extends JCTree.Visitor {
         return instance;
     }
 
+    @SuppressWarnings("this-escape")
     protected Modules(Context context) {
         context.put(Modules.class, this);
         log = Log.instance(context);
+        lint = Lint.instance(context);
         names = Names.instance(context);
         syms = Symtab.instance(context);
         attr = Attr.instance(context);
         chk = Check.instance(context);
+        preview = Preview.instance(context);
         deferredLintHandler = DeferredLintHandler.instance(context);
         typeEnvs = TypeEnvs.instance(context);
         moduleFinder = ModuleFinder.instance(context);
@@ -199,6 +204,7 @@ public class Modules extends JCTree.Visitor {
         Options options = Options.instance(context);
 
         allowAccessIntoSystem = options.isUnset(Option.RELEASE);
+
         lintOptions = options.isUnset(Option.XLINT_CUSTOM, "-" + LintCategory.OPTIONS.option);
 
         multiModuleMode = fileManager.hasLocation(StandardLocation.MODULE_SOURCE_PATH);
@@ -245,7 +251,12 @@ public class Modules extends JCTree.Visitor {
 
     public boolean enter(List<JCCompilationUnit> trees, ClassSymbol c) {
         Assert.check(rootModules != null || inInitModules || !allowModules);
-        return enter(trees, modules -> {}, c);
+        return enter(trees, modules -> {
+            //make sure java.base is completed in all cases before continuing.
+            //the next steps may query if the current module participates in preview,
+            //and that requires a completed java.base:
+            syms.java_base.complete();
+        }, c);
     }
 
     private boolean enter(List<JCCompilationUnit> trees, Consumer<Set<ModuleSymbol>> init, ClassSymbol c) {
@@ -326,6 +337,7 @@ public class Modules extends JCTree.Visitor {
             } else {
                 sym = syms.enterModule(name);
                 if (sym.module_info.sourcefile != null && sym.module_info.sourcefile != toplevel.sourcefile) {
+                    decl.sym = syms.errModule;
                     log.error(decl.pos(), Errors.DuplicateModule(sym));
                     return;
                 }
@@ -734,7 +746,7 @@ public class Modules extends JCTree.Visitor {
                 ModuleVisitor v = new ModuleVisitor();
                 JavaFileObject prev = log.useSource(tree.sourcefile);
                 JCModuleDecl moduleDecl = tree.getModuleDecl();
-                DiagnosticPosition prevLintPos = deferredLintHandler.setPos(moduleDecl.pos());
+                deferredLintHandler.push(moduleDecl);
 
                 try {
                     moduleDecl.accept(v);
@@ -742,7 +754,7 @@ public class Modules extends JCTree.Visitor {
                     checkCyclicDependencies(moduleDecl);
                 } finally {
                     log.useSource(prev);
-                    deferredLintHandler.setPos(prevLintPos);
+                    deferredLintHandler.pop();
                     msym.flags_field &= ~UNATTRIBUTED;
                 }
             }
@@ -802,11 +814,14 @@ public class Modules extends JCTree.Visitor {
                 allRequires.add(msym);
                 Set<RequiresFlag> flags = EnumSet.noneOf(RequiresFlag.class);
                 if (tree.isTransitive) {
-                    if (msym == syms.java_base && source.compareTo(Source.JDK10) >= 0) {
-                        log.error(tree.pos(), Errors.ModifierNotAllowedHere(names.transitive));
-                    } else {
-                        flags.add(RequiresFlag.TRANSITIVE);
+                    if (msym == syms.java_base &&
+                        !preview.participatesInPreview(syms, sym)) {
+                        if (source.compareTo(Source.JDK10) >= 0) {
+                            preview.checkSourceLevel(tree.pos(),
+                                                     Feature.JAVA_BASE_TRANSITIVE);
+                        }
                     }
+                    flags.add(RequiresFlag.TRANSITIVE);
                 }
                 if (tree.isStaticPhase) {
                     if (msym == syms.java_base && source.compareTo(Source.JDK10) >= 0) {
@@ -976,13 +991,13 @@ public class Modules extends JCTree.Visitor {
             UsesProvidesVisitor v = new UsesProvidesVisitor(msym, env);
             JavaFileObject prev = log.useSource(env.toplevel.sourcefile);
             JCModuleDecl decl = env.toplevel.getModuleDecl();
-            DiagnosticPosition prevLintPos = deferredLintHandler.setPos(decl.pos());
+            deferredLintHandler.push(decl);
 
             try {
                 decl.accept(v);
             } finally {
                 log.useSource(prev);
-                deferredLintHandler.setPos(prevLintPos);
+                deferredLintHandler.pop();
             }
         };
     }
@@ -999,7 +1014,7 @@ public class Modules extends JCTree.Visitor {
             this.env = env;
         }
 
-        @Override @SuppressWarnings("unchecked")
+        @Override
         public void visitModuleDef(JCModuleDecl tree) {
             msym.directives = List.nil();
             msym.provides = List.nil();
@@ -1093,6 +1108,9 @@ public class Modules extends JCTree.Visitor {
                 } finally {
                     env.info.visitingServiceImplementation = prevVisitingServiceImplementation;
                 }
+                if (!it.hasTag(CLASS)) {
+                    continue;
+                }
                 ClassSymbol impl = (ClassSymbol) it.tsym;
                 if ((impl.flags_field & PUBLIC) == 0) {
                     log.error(implName.pos(), Errors.NotDefPublic(impl, impl.location()));
@@ -1223,6 +1241,10 @@ public class Modules extends JCTree.Visitor {
         Assert.checkNonNull(rootModules);
         Assert.checkNull(allModules);
 
+        //java.base may not be completed yet and computeTransitiveClosure
+        //may not complete it either, make sure it is completed:
+        syms.java_base.complete();
+
         Set<ModuleSymbol> observable;
 
         if (limitModsOpt == null && extraLimitMods.isEmpty()) {
@@ -1244,8 +1266,8 @@ public class Modules extends JCTree.Visitor {
             if (lintOptions) {
                 for (ModuleSymbol msym : limitMods) {
                     if (!observable.contains(msym)) {
-                        log.warning(LintCategory.OPTIONS,
-                                Warnings.ModuleForOptionNotFound(Option.LIMIT_MODULES, msym));
+                        log.warning(
+                                LintWarnings.ModuleForOptionNotFound(Option.LIMIT_MODULES, msym));
                     }
                 }
             }
@@ -1343,13 +1365,15 @@ public class Modules extends JCTree.Visitor {
                 .forEach(result::add);
         }
 
-        String incubatingModules = filterAlreadyWarnedIncubatorModules(result.stream()
-                .filter(msym -> msym.resolutionFlags.contains(ModuleResolutionFlags.WARN_INCUBATING))
-                .map(msym -> msym.name.toString()))
-                .collect(Collectors.joining(","));
+        if (lint.isEnabled(LintCategory.INCUBATING)) {
+            String incubatingModules = filterAlreadyWarnedIncubatorModules(result.stream()
+                    .filter(msym -> msym.resolutionFlags.contains(ModuleResolutionFlags.WARN_INCUBATING))
+                    .map(msym -> msym.name.toString()))
+                    .collect(Collectors.joining(","));
 
-        if (!incubatingModules.isEmpty()) {
-            log.warning(Warnings.IncubatingModules(incubatingModules));
+            if (!incubatingModules.isEmpty()) {
+                log.warning(LintWarnings.IncubatingModules(incubatingModules));
+            }
         }
 
         allModules = result;
@@ -1585,6 +1609,9 @@ public class Modules extends JCTree.Visitor {
                 addVisiblePackages(msym, seen, exportsFrom, exports);
             }
         });
+
+        //module readability is reflexive:
+        msym.readModules.add(msym);
     }
 
     private void addVisiblePackages(ModuleSymbol msym,
@@ -1695,8 +1722,8 @@ public class Modules extends JCTree.Visitor {
 
         if (!unknownModules.contains(msym)) {
             if (lintOptions) {
-                log.warning(LintCategory.OPTIONS,
-                        Warnings.ModuleForOptionNotFound(Option.ADD_EXPORTS, msym));
+                log.warning(
+                        LintWarnings.ModuleForOptionNotFound(Option.ADD_EXPORTS, msym));
             }
             unknownModules.add(msym);
         }
@@ -1734,7 +1761,7 @@ public class Modules extends JCTree.Visitor {
             ModuleSymbol msym = syms.enterModule(names.fromString(sourceName));
             if (!allModules.contains(msym)) {
                 if (lintOptions) {
-                    log.warning(Warnings.ModuleForOptionNotFound(Option.ADD_READS, msym));
+                    log.warning(LintWarnings.ModuleForOptionNotFound(Option.ADD_READS, msym));
                 }
                 continue;
             }
@@ -1754,7 +1781,7 @@ public class Modules extends JCTree.Visitor {
                     targetModule = syms.enterModule(names.fromString(targetName));
                     if (!allModules.contains(targetModule)) {
                         if (lintOptions) {
-                            log.warning(LintCategory.OPTIONS, Warnings.ModuleForOptionNotFound(Option.ADD_READS, targetModule));
+                            log.warning(LintWarnings.ModuleForOptionNotFound(Option.ADD_READS, targetModule));
                         }
                         continue;
                     }
