@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -55,7 +55,7 @@ import com.sun.tools.javac.util.StringUtils;
  * deletion without notice.</b>
  */
 @SupportedAnnotationTypes("*")
-@SupportedSourceVersion(SourceVersion.RELEASE_22)
+@SupportedSourceVersion(SourceVersion.RELEASE_25)
 public class PrintingProcessor extends AbstractProcessor {
     PrintWriter writer;
 
@@ -118,7 +118,6 @@ public class PrintingProcessor extends AbstractProcessor {
         }
 
         @Override @DefinedBy(Api.LANGUAGE_MODEL)
-        @SuppressWarnings("preview") // isUnnamed
         public PrintingElementVisitor visitExecutable(ExecutableElement e, Boolean p) {
             ElementKind kind = e.getKind();
 
@@ -126,7 +125,7 @@ public class PrintingProcessor extends AbstractProcessor {
                 kind != INSTANCE_INIT) {
                 Element enclosing = e.getEnclosingElement();
 
-                // Don't print out the constructor of an anonymous or unnamed class
+                // Don't print out the constructor of an anonymous class
                 if (kind == CONSTRUCTOR &&
                     enclosing != null &&
                     (NestingKind.ANONYMOUS ==
@@ -136,13 +135,6 @@ public class PrintingProcessor extends AbstractProcessor {
                         public NestingKind visitType(TypeElement e, Void p) {
                             return e.getNestingKind();
                         }
-                    }).visit(enclosing)
-                    || // Don't print the constructor of an unnamed class
-                    (new SimpleElementVisitor14<Boolean, Void>(false) {
-                        @Override @DefinedBy(Api.LANGUAGE_MODEL)
-                        public Boolean visitType(TypeElement e, Void p) {
-                            return e.isUnnamed();
-                        }
                     }).visit(enclosing)) ) {
                     return this;
                 }
@@ -163,14 +155,25 @@ public class PrintingProcessor extends AbstractProcessor {
                     break;
                 }
 
-                writer.print("(");
-                printParameters(e);
-                writer.print(")");
-                AnnotationValue defaultValue = e.getDefaultValue();
-                if (defaultValue != null)
-                    writer.print(" default " + defaultValue);
+                if (elementUtils.isCompactConstructor(e)) {
+                    // A record's compact constructor by definition
+                    // lacks source-explicit parameters and lacks a
+                    // throws clause.
+                    writer.print(" {} /* compact constructor */ ");
+                } else {
+                    writer.print("(");
+                    printParameters(e);
+                    writer.print(")");
+
+                    // Display any default values for an annotation
+                    // interface element
+                    AnnotationValue defaultValue = e.getDefaultValue();
+                    if (defaultValue != null)
+                        writer.print(" default " + defaultValue);
+
+                    printThrows(e);
+                }
 
-                printThrows(e);
                 writer.println(";");
             }
             return this;
@@ -178,7 +181,6 @@ public class PrintingProcessor extends AbstractProcessor {
 
 
         @Override @DefinedBy(Api.LANGUAGE_MODEL)
-        @SuppressWarnings("preview") // isUnnamed
         public PrintingElementVisitor visitType(TypeElement e, Boolean p) {
             ElementKind kind = e.getKind();
             NestingKind nestingKind = e.getNestingKind();
@@ -222,14 +224,6 @@ public class PrintingProcessor extends AbstractProcessor {
                         printParameters(constructors.get(0));
                 }
                 writer.print(")");
-            } else if (e.isUnnamed()) {
-                writer.println("// Unnamed class in file whose name starts with " + e.getSimpleName());
-
-                for(Element element : e.getEnclosedElements()) {
-                    this.visit(element);
-                }
-
-                return this;
             } else {
                 if (nestingKind == TOP_LEVEL) {
                     PackageElement pkg = elementUtils.getPackageOf(e);
