@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -37,6 +37,7 @@ import sun.net.httpserver.HttpConnection.State;
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLEngine;
 import java.io.BufferedInputStream;
+import java.io.BufferedOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -52,8 +53,6 @@ import java.nio.channels.SelectionKey;
 import java.nio.channels.Selector;
 import java.nio.channels.ServerSocketChannel;
 import java.nio.channels.SocketChannel;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashSet;
@@ -61,9 +60,7 @@ import java.util.List;
 import java.util.Set;
 import java.util.Timer;
 import java.util.TimerTask;
-import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.Executor;
-import java.util.concurrent.TimeUnit;
 
 import static java.nio.charset.StandardCharsets.ISO_8859_1;
 import static sun.net.httpserver.Utils.isValidName;
@@ -96,7 +93,7 @@ class ServerImpl {
     private final Set<HttpConnection> rspConnections;
     private List<Event> events;
     private final Object lolock = new Object();
-    private final CountDownLatch finishedLatch = new CountDownLatch(1);
+    private volatile boolean finished = false;
     private volatile boolean terminating = false;
     private boolean bound = false;
     private boolean started = false;
@@ -182,7 +179,7 @@ class ServerImpl {
     }
 
     public void start () {
-        if (!bound || started || finished()) {
+        if (!bound || started || finished) {
             throw new IllegalStateException ("server in wrong state");
         }
         if (executor == null) {
@@ -225,75 +222,45 @@ class ServerImpl {
         return httpsConfig;
     }
 
-    private final boolean finished(){
-        // if the latch is 0, the server is finished
-        return finishedLatch.getCount() == 0;
-    }
-
     public final boolean isFinishing() {
-        return finished();
+        return finished;
     }
 
-    /**
-     * This method stops the server by adding a stop request event and
-     * waiting for the server until the event is triggered or until the maximum delay is triggered.
-     * <p>
-     * This ensures that the server is stopped immediately after all exchanges are complete. HttpConnections will be forcefully closed if active exchanges do not
-     * complete within the imparted delay.
-     *
-     * @param delay maximum delay to wait for exchanges completion, in seconds
-     */
     public void stop (int delay) {
         if (delay < 0) {
             throw new IllegalArgumentException ("negative delay parameter");
         }
-
-        logger.log(Level.TRACE, "stopping");
-        // posting a stop event, which will flip finished flag if it finishes
-        // before the timeout in this method
         terminating = true;
-
-        addEvent(new Event.StopRequested());
-
         try { schan.close(); } catch (IOException e) {}
         selector.wakeup();
-
-        try {
-            // waiting for the duration of the delay, unless released before
-            finishedLatch.await(delay, TimeUnit.SECONDS);
-
-        } catch (InterruptedException e) {
-            logger.log(Level.TRACE, "Error in awaiting the delay");
-
-        } finally {
-
-            logger.log(Level.TRACE, "closing connections");
-            finishedLatch.countDown();
-            selector.wakeup();
-            synchronized (allConnections) {
-                for (HttpConnection c : allConnections) {
-                    c.close();
-                }
+        long latest = System.currentTimeMillis() + delay * 1000;
+        while (System.currentTimeMillis() < latest) {
+            delay();
+            if (finished) {
+                break;
             }
-            allConnections.clear();
-            idleConnections.clear();
-            newlyAcceptedConnections.clear();
-            timer.cancel();
-            if (reqRspTimeoutEnabled) {
-                timer1.cancel();
+        }
+        finished = true;
+        selector.wakeup();
+        synchronized (allConnections) {
+            for (HttpConnection c : allConnections) {
+                c.close();
             }
-            logger.log(Level.TRACE, "connections closed");
-
-            if (dispatcherThread != null && dispatcherThread != Thread.currentThread()) {
-                logger.log(Level.TRACE, "waiting for dispatcher thread");
-                try {
-                    dispatcherThread.join();
-                } catch (InterruptedException e) {
-                    Thread.currentThread().interrupt();
-                    logger.log(Level.TRACE, "ServerImpl.stop: ", e);
-                }
+        }
+        allConnections.clear();
+        idleConnections.clear();
+        newlyAcceptedConnections.clear();
+        timer.cancel();
+        if (reqRspTimeoutEnabled) {
+            timer1.cancel();
+        }
+        if (dispatcherThread != null && dispatcherThread != Thread.currentThread()) {
+            try {
+                dispatcherThread.join();
+            } catch (InterruptedException e) {
+                Thread.currentThread().interrupt();
+                logger.log (Level.TRACE, "ServerImpl.stop: ", e);
             }
-            logger.log(Level.TRACE, "server stopped");
         }
     }
 
@@ -335,16 +302,8 @@ class ServerImpl {
         logger.log (Level.DEBUG, "context removed: " + context.getPath());
     }
 
-    @SuppressWarnings("removal")
     public InetSocketAddress getAddress() {
-        return AccessController.doPrivileged(
-                new PrivilegedAction<InetSocketAddress>() {
-                    public InetSocketAddress run() {
-                        return
-                            (InetSocketAddress)schan.socket()
-                                .getLocalSocketAddress();
-                    }
-                });
+        return (InetSocketAddress) schan.socket().getLocalSocketAddress();
     }
 
     void addEvent (Event r) {
@@ -423,34 +382,15 @@ class ServerImpl {
     class Dispatcher implements Runnable {
 
         private void handleEvent (Event r) {
-
-            // Stopping marking the state as finished if stop is requested,
-            // termination is in progress and exchange count is 0
-            if (r instanceof Event.StopRequested) {
-                logger.log(Level.TRACE, "Handling Stop Requested Event");
-
-                // checking if terminating is set to true
-                final boolean terminatingCopy = terminating;
-                assert terminatingCopy;
-
-                if (getExchangeCount() == 0 && reqConnections.isEmpty()) {
-                    finishedLatch.countDown();
-                } else {
-                    logger.log(Level.TRACE, "Some requests are still pending");
-                }
-                return;
-            }
-
             ExchangeImpl t = r.exchange;
             HttpConnection c = t.getConnection();
-
             try {
-                if (r instanceof Event.WriteFinished) {
+                if (r instanceof WriteFinishedEvent) {
 
                     logger.log(Level.TRACE, "Write Finished");
                     int exchanges = endExchange();
-                    if (terminating && exchanges == 0 && reqConnections.isEmpty()) {
-                        finishedLatch.countDown();
+                    if (terminating && exchanges == 0) {
+                        finished = true;
                     }
                     LeftOverInputStream is = t.getOriginalInputStream();
                     if (!is.isEOF()) {
@@ -500,12 +440,11 @@ class ServerImpl {
         }
 
         public void run() {
-            // finished() will be true when there are no active exchange after terminating
-             while (!finished()) {
+            while (!finished) {
                 try {
                     List<Event> list = null;
                     synchronized (lolock) {
-                        if (!events.isEmpty()) {
+                        if (events.size() > 0) {
                             list = events;
                             events = new ArrayList<>();
                         }
@@ -652,18 +591,18 @@ class ServerImpl {
         conn.close();
         allConnections.remove(conn);
         switch (conn.getState()) {
-            case REQUEST:
-                reqConnections.remove(conn);
-                break;
-            case RESPONSE:
-                rspConnections.remove(conn);
-                break;
-            case IDLE:
-                idleConnections.remove(conn);
-                break;
-            case NEWLY_ACCEPTED:
-                newlyAcceptedConnections.remove(conn);
-                break;
+        case REQUEST:
+            reqConnections.remove(conn);
+            break;
+        case RESPONSE:
+            rspConnections.remove(conn);
+            break;
+        case IDLE:
+            idleConnections.remove(conn);
+            break;
+        case NEWLY_ACCEPTED:
+            newlyAcceptedConnections.remove(conn);
+            break;
         }
         assert !reqConnections.remove(conn);
         assert !rspConnections.remove(conn);
@@ -738,6 +677,7 @@ class ServerImpl {
                             ServerImpl.this, chan
                         );
                     }
+                    rawout = new BufferedOutputStream(rawout);
                     connection.raw = rawin;
                     connection.rawout = rawout;
                 }
@@ -985,16 +925,19 @@ class ServerImpl {
         logger.log (Level.DEBUG, message);
     }
 
+    void delay () {
+        Thread.yield();
+        try {
+            Thread.sleep (200);
+        } catch (InterruptedException e) {}
+    }
+
     private int exchangeCount = 0;
 
     synchronized void startExchange () {
         exchangeCount ++;
     }
 
-    synchronized int getExchangeCount() {
-        return exchangeCount;
-    }
-
     synchronized int endExchange () {
         exchangeCount --;
         assert exchangeCount >= 0;
