@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -31,11 +31,12 @@ import java.util.Objects;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import jdk.javadoc.internal.doclets.formats.html.markup.ContentBuilder;
-import jdk.javadoc.internal.doclets.formats.html.markup.Entity;
-import jdk.javadoc.internal.doclets.formats.html.markup.Text;
 import jdk.javadoc.internal.doclets.toolkit.Resources;
 import jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberTable;
+import jdk.javadoc.internal.html.Content;
+import jdk.javadoc.internal.html.ContentBuilder;
+import jdk.javadoc.internal.html.Entity;
+import jdk.javadoc.internal.html.Text;
 
 
 /**
@@ -104,6 +105,7 @@ public class Contents {
     public final Content functionalInterfaceMessage;
     public final Content helpLabel;
     public final Content helpSubNavLabel;
+    public final Content hideSidebar;
     public final Content hierarchyForAllPackages;
     public final Content implementation;
     public final Content implementingClassesLabel;
@@ -157,6 +159,9 @@ public class Contents {
     public final Content previewLabel;
     public final Content previewMark;
     public final Content previewPhrase;
+    public final Content restrictedMark;
+    public final Content restrictedMethods;
+    public final Content restrictedPhrase;
     public final Content properties;
     public final Content propertyLabel;
     public final Content propertyDetailsLabel;
@@ -166,9 +171,12 @@ public class Contents {
     public final Content referencedIn;
     public final Content relatedPackages;
     public final Content returns;
+    public final Content searchLabel;
+    public final Content searchTagsLabel;
     public final Content seeAlso;
     public final Content serializedForm;
     public final Content servicesLabel;
+    public final Content showSidebar;
     public final Content specificationLabel;
     public final Content specifiedByLabel;
     public final Content subclassesLabel;
@@ -176,6 +184,7 @@ public class Contents {
     public final Content summaryLabel;
     public final Content systemPropertiesLabel;
     public final Content systemPropertiesSummaryLabel;
+    public final Content tableOfContentsLabel;
     public final Content throws_;
     public final Content treeLabel;
     public final Content typeLabel;
@@ -250,6 +259,7 @@ public class Contents {
         functionalInterfaceMessage = getContent("doclet.Functional_Interface_Message");
         helpLabel = getContent("doclet.Help");
         helpSubNavLabel = getContent("doclet.Help_Sub_Nav");
+        hideSidebar = getContent("doclet.hide_sidebar");
         hierarchyForAllPackages = getContent("doclet.Hierarchy_For_All_Packages");
         implementation = getContent("doclet.Implementation");
         implementingClassesLabel = getContent("doclet.Implementing_Classes");
@@ -303,6 +313,9 @@ public class Contents {
         previewLabel = getContent("doclet.Preview_Label");
         previewMark = getContent("doclet.Preview_Mark");
         previewPhrase = getContent("doclet.Preview");
+        restrictedMark = getContent("doclet.Restricted_Mark");
+        restrictedMethods = getContent("doclet.Restricted_Methods");
+        restrictedPhrase = getContent("doclet.Restricted");
         properties = getContent("doclet.Properties");
         propertyLabel = getContent("doclet.Property");
         propertyDetailsLabel = getContent("doclet.Property_Detail");
@@ -312,9 +325,12 @@ public class Contents {
         referencedIn = getContent("doclet.ReferencedIn");
         relatedPackages = getContent("doclet.Related_Packages");
         returns = getContent("doclet.Returns");
+        searchLabel = getContent("doclet.search");
+        searchTagsLabel = getContent("doclet.searchTags");
         seeAlso = getContent("doclet.See_Also");
         serializedForm = getContent("doclet.Serialized_Form");
         servicesLabel = getContent("doclet.Services");
+        showSidebar = getContent("doclet.show_sidebar");
         specificationLabel = getContent("doclet.Specification");
         specifiedByLabel = getContent("doclet.Specified_By");
         subclassesLabel = getContent("doclet.Subclasses");
@@ -322,6 +338,7 @@ public class Contents {
         summaryLabel = getContent("doclet.Summary");
         systemPropertiesLabel = getContent("doclet.systemProperties");
         systemPropertiesSummaryLabel = getContent("doclet.systemPropertiesSummary");
+        tableOfContentsLabel = getContent("doclet.table_of_contents");
         throws_ = getContent("doclet.Throws");
         treeLabel = getContent("doclet.Tree");
         typeLabel = getContent("doclet.Type");
@@ -428,7 +445,7 @@ public class Contents {
      * @param items     the items
      * @return the composition
      */
-    public Content join(Content separator, Collection<Content> items) {
+    public Content join(Content separator, Collection<? extends Content> items) {
         Content result = new ContentBuilder();
         boolean first = true;
         for (Content c : items) {
