@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,11 +30,11 @@ import java.io.Writer;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.SortedSet;
+
 import javax.lang.model.element.Element;
 import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.TypeElement;
 
-import jdk.javadoc.internal.doclets.formats.html.markup.HtmlTree;
 import jdk.javadoc.internal.doclets.toolkit.Resources;
 import jdk.javadoc.internal.doclets.toolkit.util.DocFile;
 import jdk.javadoc.internal.doclets.toolkit.util.DocFileIOException;
@@ -42,7 +42,7 @@ import jdk.javadoc.internal.doclets.toolkit.util.DocPath;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPaths;
 import jdk.javadoc.internal.doclets.toolkit.util.IndexBuilder;
 import jdk.javadoc.internal.doclets.toolkit.util.IndexItem;
-import jdk.javadoc.internal.doclets.toolkit.util.Utils;
+import jdk.javadoc.internal.html.HtmlTree;
 
 /**
  * Extensions to {@code IndexBuilder} to fill in remaining fields
@@ -51,10 +51,8 @@ import jdk.javadoc.internal.doclets.toolkit.util.Utils;
  * JavaScript files.
  */
 public class HtmlIndexBuilder extends IndexBuilder {
-    private final HtmlConfiguration configuration;
 
     private final Resources resources;
-    private final Utils utils;
     private final HtmlIds htmlIds;
 
     /**
@@ -63,11 +61,9 @@ public class HtmlIndexBuilder extends IndexBuilder {
      * @param configuration the current configuration of the doclet
      */
     HtmlIndexBuilder(HtmlConfiguration configuration) {
-        super(configuration, configuration.getOptions().noDeprecated());
-        this.configuration = configuration;
-        resources = configuration.docResources;
-        utils = configuration.utils;
-        htmlIds = configuration.htmlIds;
+        super(configuration);
+        this.resources = configuration.docResources;
+        this.htmlIds = configuration.htmlIds;
     }
 
     /**
@@ -79,15 +75,12 @@ public class HtmlIndexBuilder extends IndexBuilder {
     @Override
     public void addElements() {
         super.addElements();
-        if (classesOnly) {
-            return;
-        }
 
         Map<String,Integer> duplicateLabelCheck = new HashMap<>();
         for (Character ch : getFirstCharacters()) {
             for (IndexItem item : getItems(ch)) {
                 duplicateLabelCheck.compute(item.getFullyQualifiedLabel(utils),
-                                            (k, v) -> v == null ? 1 : v + 1);
+                        (k, v) -> v == null ? 1 : v + 1);
             }
         }
 
@@ -136,7 +129,7 @@ public class HtmlIndexBuilder extends IndexBuilder {
                     item.setContainingModule(utils.getFullyQualifiedName(utils.containingModule(element)));
                 }
                 if (utils.isExecutableElement(element)) {
-                    String url = HtmlTree.encodeURL(htmlIds.forMember((ExecutableElement) element).name());
+                    String url = HtmlTree.encodeURL(htmlIds.forMember((ExecutableElement) element).getFirst().name());
                     if (!url.equals(item.getLabel())) {
                         item.setUrl(url);
                     }
