@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,26 +27,27 @@ package jdk.javadoc.internal.doclets.formats.html;
 
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Set;
+import java.util.Objects;
 
 import javax.lang.model.element.Element;
+import javax.lang.model.element.ElementKind;
 import javax.lang.model.element.ModuleElement;
 import javax.lang.model.element.PackageElement;
 import javax.lang.model.element.TypeElement;
 
-import jdk.javadoc.internal.doclets.formats.html.markup.ContentBuilder;
-import jdk.javadoc.internal.doclets.formats.html.markup.Entity;
-import jdk.javadoc.internal.doclets.formats.html.markup.HtmlAttr;
-import jdk.javadoc.internal.doclets.formats.html.markup.HtmlStyle;
-import jdk.javadoc.internal.doclets.formats.html.markup.TagName;
-import jdk.javadoc.internal.doclets.formats.html.markup.HtmlTree;
+import jdk.javadoc.internal.doclets.formats.html.markup.HtmlStyles;
 import jdk.javadoc.internal.doclets.formats.html.markup.Links;
-import jdk.javadoc.internal.doclets.formats.html.markup.Text;
 import jdk.javadoc.internal.doclets.toolkit.util.DocFile;
 import jdk.javadoc.internal.doclets.toolkit.util.DocLink;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPath;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPaths;
-import jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberTable;
+import jdk.javadoc.internal.html.Content;
+import jdk.javadoc.internal.html.ContentBuilder;
+import jdk.javadoc.internal.html.Entity;
+import jdk.javadoc.internal.html.HtmlAttr;
+import jdk.javadoc.internal.html.HtmlTag;
+import jdk.javadoc.internal.html.HtmlTree;
+import jdk.javadoc.internal.html.Text;
 
 /**
  * Factory for navigation bar.
@@ -67,14 +68,8 @@ public class Navigation {
     private final DocPath pathToRoot;
     private final Links links;
     private final PageMode documentedPage;
-    private Content navLinkModule;
-    private Content navLinkPackage;
-    private Content navLinkClass;
     private Content userHeader;
     private final String rowListTitle;
-    private final Content searchLabel;
-    private final String searchPlaceholder;
-    private SubNavLinks subNavLinks;
 
     public enum PageMode {
         ALL_CLASSES,
@@ -91,25 +86,15 @@ public class Navigation {
         OVERVIEW,
         PACKAGE,
         PREVIEW,
-        SERIALIZED_FORM,
+        RESTRICTED,
         SEARCH,
+        SEARCH_TAGS,
+        SERIALIZED_FORM,
         SYSTEM_PROPERTIES,
         TREE,
         USE
     }
 
-    /**
-     * An interface to provide links for the subnavigation area.
-     */
-    public interface SubNavLinks {
-        /**
-         * {@return a list of links to display in the subnavigation area}
-         * Links should be wrapped in {@code HtmlTree.LI} elements as they are
-         * displayed within an unordered list.
-         */
-        List<Content> getSubNavLinks();
-    }
-
     /**
      * Creates a {@code Navigation} object for a specific file, to be written in a specific HTML
      * version.
@@ -130,23 +115,6 @@ public class Navigation {
         this.pathToRoot = path.parent().invert();
         this.links = new Links(path);
         this.rowListTitle = configuration.getDocResources().getText("doclet.Navigation");
-        this.searchLabel = contents.getContent("doclet.search");
-        this.searchPlaceholder = configuration.getDocResources().getText("doclet.search_placeholder");
-    }
-
-    public Navigation setNavLinkModule(Content navLinkModule) {
-        this.navLinkModule = navLinkModule;
-        return this;
-    }
-
-    public Navigation setNavLinkPackage(Content navLinkPackage) {
-        this.navLinkPackage = navLinkPackage;
-        return this;
-    }
-
-    public Navigation setNavLinkClass(Content navLinkClass) {
-        this.navLinkClass = navLinkClass;
-        return this;
     }
 
     public Navigation setUserHeader(Content userHeader) {
@@ -154,11 +122,6 @@ public class Navigation {
         return this;
     }
 
-    public Navigation setSubNavLinks(SubNavLinks subNavLinks) {
-        this.subNavLinks = subNavLinks;
-        return this;
-    }
-
     /**
      * Adds the links for the main navigation.
      *
@@ -168,35 +131,28 @@ public class Navigation {
         switch (documentedPage) {
             case OVERVIEW:
                 addActivePageLink(target, contents.overviewLabel, options.createOverview());
-                addModuleLink(target);
-                addPackageLink(target);
-                addPageLabel(target, contents.classLabel, true);
-                addPageLabel(target, contents.useLabel, options.classUse());
                 addTreeLink(target);
                 addPreviewLink(target);
                 addNewLink(target);
                 addDeprecatedLink(target);
                 addIndexLink(target);
+                addSearchLink(target);
                 addHelpLink(target);
                 break;
             case MODULE:
                 addOverviewLink(target);
                 addActivePageLink(target, contents.moduleLabel, configuration.showModules);
-                addPackageLink(target);
-                addPageLabel(target, contents.classLabel, true);
-                addPageLabel(target, contents.useLabel, options.classUse());
                 addTreeLink(target);
                 addPreviewLink(target);
                 addNewLink(target);
                 addDeprecatedLink(target);
                 addIndexLink(target);
+                addSearchLink(target);
                 addHelpLink(target);
                 break;
             case PACKAGE:
                 addOverviewLink(target);
-                addModuleOfElementLink(target);
                 addActivePageLink(target, contents.packageLabel, true);
-                addPageLabel(target, contents.classLabel, true);
                 if (options.classUse()) {
                     addItemToList(target, links.createLink(DocPaths.PACKAGE_USE,
                             contents.useLabel, ""));
@@ -209,12 +165,11 @@ public class Navigation {
                 addNewLink(target);
                 addDeprecatedLink(target);
                 addIndexLink(target);
+                addSearchLink(target);
                 addHelpLink(target);
                 break;
             case CLASS:
                 addOverviewLink(target);
-                addModuleOfElementLink(target);
-                addPackageSummaryLink(target);
                 addActivePageLink(target, contents.classLabel, true);
                 if (options.classUse()) {
                     addItemToList(target, links.createLink(DocPaths.CLASS_USE.resolve(path.basename()),
@@ -228,48 +183,49 @@ public class Navigation {
                 addNewLink(target);
                 addDeprecatedLink(target);
                 addIndexLink(target);
+                addSearchLink(target);
                 addHelpLink(target);
                 break;
             case USE:
                 addOverviewLink(target);
-                addModuleOfElementLink(target);
-                if (element instanceof PackageElement) {
-                    addPackageSummaryLink(target);
-                    addPageLabel(target, contents.classLabel, true);
-                } else {
-                    addPackageOfElementLink(target);
-                    addItemToList(target, navLinkClass);
+                // Class-use page is still generated for deprecated classes with
+                // -nodeprecated option, make sure not to link to non-existent page.
+                if (!options.noDeprecated() || !configuration.utils.isDeprecated(element)) {
+                    addPageElementLink(target);
                 }
                 addActivePageLink(target, contents.useLabel, options.classUse());
-                if (element instanceof PackageElement) {
-                    addItemToList(target, links.createLink(DocPaths.PACKAGE_TREE, contents.treeLabel));
-                } else {
-                    addItemToList(target, configuration.utils.isEnclosingPackageIncluded((TypeElement) element)
-                            ? links.createLink(DocPath.parent.resolve(DocPaths.PACKAGE_TREE), contents.treeLabel)
-                            : links.createLink(pathToRoot.resolve(DocPaths.OVERVIEW_TREE), contents.treeLabel));
+                if (options.createTree()) {
+                    if (configuration.utils.isPackage(element)) {
+                        addItemToList(target, links.createLink(DocPaths.PACKAGE_TREE, contents.treeLabel));
+                    } else {
+                        addItemToList(target, configuration.utils.isEnclosingPackageIncluded((TypeElement) element)
+                                ? links.createLink(DocPath.parent.resolve(DocPaths.PACKAGE_TREE), contents.treeLabel)
+                                : links.createLink(pathToRoot.resolve(DocPaths.OVERVIEW_TREE), contents.treeLabel));
+                    }
                 }
                 addPreviewLink(target);
                 addNewLink(target);
                 addDeprecatedLink(target);
                 addIndexLink(target);
+                addSearchLink(target);
                 addHelpLink(target);
                 break;
             case TREE:
                 addOverviewLink(target);
-                if (element == null) {
-                    addPageLabel(target, contents.moduleLabel, configuration.showModules);
-                    addPageLabel(target, contents.packageLabel, true);
-                } else {
-                    addModuleOfElementLink(target);
-                    addPackageSummaryLink(target);
+                if (element != null && !configuration.utils.isModule(element)) {
+                    addPageElementLink(target);
+                    if (options.classUse()) {
+                        if (configuration.utils.isPackage(element) || configuration.utils.isTypeElement(element)) {
+                            addItemToList(target, links.createLink(DocPaths.PACKAGE_USE, contents.useLabel));
+                        }
+                    }
                 }
-                addPageLabel(target, contents.classLabel, true);
-                addPageLabel(target, contents.useLabel, options.classUse());
                 addActivePageLink(target, contents.treeLabel, options.createTree());
                 addPreviewLink(target);
                 addNewLink(target);
                 addDeprecatedLink(target);
                 addIndexLink(target);
+                addSearchLink(target);
                 addHelpLink(target);
                 break;
             case DEPRECATED:
@@ -277,11 +233,8 @@ public class Navigation {
             case HELP:
             case PREVIEW:
             case NEW:
+            case SEARCH:
                 addOverviewLink(target);
-                addModuleLink(target);
-                addPackageLink(target);
-                addPageLabel(target, contents.classLabel, true);
-                addPageLabel(target, contents.useLabel, options.classUse());
                 addTreeLink(target);
                 if (documentedPage == PageMode.PREVIEW) {
                     addActivePageLink(target, contents.previewLabel,
@@ -306,6 +259,11 @@ public class Navigation {
                 } else {
                     addIndexLink(target);
                 }
+                if (documentedPage == PageMode.SEARCH) {
+                    addActivePageLink(target, contents.searchLabel, options.createIndex());
+                } else {
+                    addSearchLink(target);
+                }
                 if (documentedPage == PageMode.HELP) {
                     addActivePageLink(target, contents.helpLabel, !options.noHelp());
                 } else {
@@ -316,32 +274,43 @@ public class Navigation {
             case ALL_PACKAGES:
             case CONSTANT_VALUES:
             case EXTERNAL_SPECS:
+            case RESTRICTED:
+            case SEARCH_TAGS:
             case SERIALIZED_FORM:
-            case SEARCH:
             case SYSTEM_PROPERTIES:
                 addOverviewLink(target);
-                addModuleLink(target);
-                addPackageLink(target);
-                addPageLabel(target, contents.classLabel, true);
-                addPageLabel(target, contents.useLabel, options.classUse());
                 addTreeLink(target);
                 addPreviewLink(target);
                 addNewLink(target);
                 addDeprecatedLink(target);
                 addIndexLink(target);
+                addSearchLink(target);
                 addHelpLink(target);
                 break;
             case DOC_FILE:
                 addOverviewLink(target);
-                addModuleOfElementLink(target);
-                addItemToList(target, navLinkPackage);
-                addPageLabel(target, contents.classLabel, true);
-                addPageLabel(target, contents.useLabel, options.classUse());
-                addTreeLink(target);
+                if (element != null) {
+                    addPageElementLink(target);
+                    if (options.classUse()) {
+                        if (configuration.utils.isPackage(element)) {
+                            addItemToList(target, links.createLink(pathToRoot.resolve(
+                                    configuration.docPaths.forPackage((PackageElement) element)
+                                            .resolve(DocPaths.PACKAGE_USE)), contents.useLabel));
+                        }
+                    }
+                    if (options.createTree() && configuration.utils.isPackage(element)) {
+                        addItemToList(target, links.createLink(pathToRoot.resolve(
+                                configuration.docPaths.forPackage((PackageElement) element)
+                                        .resolve(DocPaths.PACKAGE_TREE)), contents.treeLabel));
+                    } else {
+                        addTreeLink(target);
+                    }
+                }
                 addPreviewLink(target);
                 addNewLink(target);
                 addDeprecatedLink(target);
                 addIndexLink(target);
+                addSearchLink(target);
                 addHelpLink(target);
                 break;
             default:
@@ -349,86 +318,6 @@ public class Navigation {
         }
     }
 
-    /**
-     * Adds the summary links to the subnavigation.
-     *
-     * @param target the content to which the subnavigation will be added
-     * @param nested whether to create a flat or nested list
-     */
-    private void addSummaryLinks(Content target, boolean nested) {
-        switch (documentedPage) {
-            case MODULE, PACKAGE, CLASS, HELP -> {
-                List<? extends Content> listContents = subNavLinks.getSubNavLinks()
-                        .stream().map(HtmlTree::LI).toList();
-                if (!listContents.isEmpty()) {
-                Content label = switch (documentedPage) {
-                        case MODULE -> contents.moduleSubNavLabel;
-                        case PACKAGE -> contents.packageSubNavLabel;
-                        case CLASS -> contents.summaryLabel;
-                        case HELP -> contents.helpSubNavLabel;
-                        default -> Text.EMPTY;
-                    };
-                    if (nested) {
-                        target.add(HtmlTree.LI(HtmlTree.P(label))
-                                .add(new HtmlTree(TagName.UL).add(listContents)));
-                    } else {
-                        target.add(HtmlTree.LI(label).add(Entity.NO_BREAK_SPACE));
-                        addListToNav(listContents, target);
-                    }
-                }
-            }
-        }
-    }
-
-    /**
-     * Adds the detail links to subnavigation.
-     *
-     * @param target the content to which the links will be added
-     * @param nested whether to create a flat or nested list
-     */
-    private void addDetailLinks(Content target, boolean nested) {
-        if (documentedPage == PageMode.CLASS) {
-            List<Content> listContents = new ArrayList<>();
-            VisibleMemberTable vmt = configuration.getVisibleMemberTable((TypeElement) element);
-            Set<VisibleMemberTable.Kind> detailSet = VisibleMemberTable.Kind.forDetailsOf(element.getKind());
-            for (VisibleMemberTable.Kind kind : detailSet) {
-                addTypeDetailLink(kind, !vmt.getVisibleMembers(kind).isEmpty(), listContents);
-            }
-            if (!listContents.isEmpty()) {
-                if (nested) {
-                    var li = HtmlTree.LI(HtmlTree.P(contents.detailLabel));
-                    li.add(new HtmlTree(TagName.UL).add(listContents));
-                    target.add(li);
-                } else {
-                    var li = HtmlTree.LI(contents.detailLabel);
-                    li.add(Entity.NO_BREAK_SPACE);
-                    target.add(li);
-                    addListToNav(listContents, target);
-                }
-            }
-        }
-    }
-
-    /**
-     * Adds the navigation Type detail link.
-     *
-     * @param kind the kind of member being documented
-     * @param link true if the members are listed and need to be linked
-     * @param listContents the list of contents to which the detail will be added.
-     */
-    protected void addTypeDetailLink(VisibleMemberTable.Kind kind, boolean link, List<Content> listContents) {
-        addContentToList(listContents, switch (kind) {
-            case CONSTRUCTORS -> links.createLink(HtmlIds.CONSTRUCTOR_DETAIL, contents.navConstructor, link);
-            case ENUM_CONSTANTS -> links.createLink(HtmlIds.ENUM_CONSTANT_DETAIL, contents.navEnum, link);
-            case FIELDS -> links.createLink(HtmlIds.FIELD_DETAIL, contents.navField, link);
-            case METHODS -> links.createLink(HtmlIds.METHOD_DETAIL, contents.navMethod, link);
-            case PROPERTIES -> links.createLink(HtmlIds.PROPERTY_DETAIL, contents.navProperty, link);
-            case ANNOTATION_TYPE_MEMBER -> links.createLink(HtmlIds.ANNOTATION_TYPE_ELEMENT_DETAIL,
-                    contents.navAnnotationTypeMember, link);
-            default -> Text.EMPTY;
-        });
-    }
-
     private void addContentToList(List<Content> listContents, Content source) {
         listContents.add(HtmlTree.LI(source));
     }
@@ -437,60 +326,31 @@ public class Navigation {
         list.add(HtmlTree.LI(item));
     }
 
-    private void addListToNav(List<? extends Content> listContents, Content target) {
-        int count = 0;
-        for (Content liContent : listContents) {
-            if (count < listContents.size() - 1) {
-                liContent.add(Entity.NO_BREAK_SPACE);
-                liContent.add("|");
-                liContent.add(Entity.NO_BREAK_SPACE);
-            }
-            target.add(liContent);
-            count++;
-        }
-    }
-
     private void addActivePageLink(Content target, Content label, boolean display) {
         if (display) {
-            target.add(HtmlTree.LI(HtmlStyle.navBarCell1Rev, label));
-        }
-    }
-
-    private void addPageLabel(Content target, Content label, boolean display) {
-        if (display) {
-            target.add(HtmlTree.LI(label));
+            target.add(HtmlTree.LI(HtmlStyles.navBarCell1Rev, label));
         }
     }
 
+    /**
+     * Adds a link to the overview page if indicated by the configuration.
+     * Otherwise a link to the first module or package is added.
+     *
+     * @param target content to add the link to
+     */
     private void addOverviewLink(Content target) {
         if (options.createOverview()) {
             target.add(HtmlTree.LI(links.createLink(pathToRoot.resolve(DocPaths.INDEX),
                     contents.overviewLabel, "")));
-        }
-    }
-
-    private void addModuleLink(Content target) {
-        if (configuration.showModules) {
-            if (configuration.modules.size() == 1) {
-                ModuleElement mdle = configuration.modules.first();
+        } else if (configuration.showModules && configuration.modules.size() > 0) {
+            ModuleElement mdle = configuration.modules.first();
+            if (!mdle.equals(element)) {
                 boolean included = configuration.utils.isIncluded(mdle);
                 target.add(HtmlTree.LI((included)
                         ? links.createLink(pathToRoot.resolve(configuration.docPaths.moduleSummary(mdle)), contents.moduleLabel, "")
                         : contents.moduleLabel));
-            } else if (!configuration.modules.isEmpty()) {
-                addPageLabel(target, contents.moduleLabel, true);
             }
-        }
-    }
-
-    private void addModuleOfElementLink(Content target) {
-        if (configuration.showModules) {
-            target.add(HtmlTree.LI(navLinkModule));
-        }
-    }
-
-    private void addPackageLink(Content target) {
-        if (configuration.packages.size() == 1) {
+        } else if (configuration.packages.size() > 0 && !(element instanceof PackageElement)) {
             PackageElement packageElement = configuration.packages.first();
             boolean included = packageElement != null && configuration.utils.isIncluded(packageElement);
             if (!included) {
@@ -510,22 +370,63 @@ public class Navigation {
                         packageElement, pathToRoot, DocPaths.PACKAGE_SUMMARY.getPath());
                 if (crossPkgLink != null) {
                     target.add(HtmlTree.LI(links.createLink(crossPkgLink, contents.packageLabel)));
-                } else {
-                    target.add(HtmlTree.LI(contents.packageLabel));
                 }
             }
-        } else if (!configuration.packages.isEmpty()) {
-            addPageLabel(target, contents.packageLabel, true);
         }
     }
 
-    private void addPackageOfElementLink(Content target) {
-        target.add(HtmlTree.LI(links.createLink(DocPath.parent.resolve(DocPaths.PACKAGE_SUMMARY),
-                contents.packageLabel)));
+    /**
+     * Adds breadcrumb navigation links for {@code element} and its containing elements
+     * to {@code contents}. Only module, package and type elements are supported in
+     * breadcrumb navigation.
+     *
+     * @param elem a module, package or type element
+     * @param contents the list to which links are added
+     * @param selected {@code true} if elem is the current page element
+     */
+    protected void addBreadcrumbLinks(Element elem, List<Content> contents, boolean selected) {
+        if (elem == null) {
+            return;
+        } else if (elem.getKind() != ElementKind.MODULE) {
+            addBreadcrumbLinks(elem.getEnclosingElement(), contents, false);
+        } else if (!configuration.showModules) {
+            return;
+        }
+        var docPaths = configuration.docPaths;
+        HtmlTree link = switch (elem) {
+            case ModuleElement mdle -> links.createLink(pathToRoot.resolve(
+                    docPaths.moduleSummary(mdle)),
+                    Text.of(mdle.getQualifiedName()));
+            case PackageElement pkg -> links.createLink(pathToRoot.resolve(
+                    docPaths.forPackage(pkg).resolve(DocPaths.PACKAGE_SUMMARY)),
+                    pkg.isUnnamed()
+                            ? configuration.contents.defaultPackageLabel
+                            : Text.of(pkg.getQualifiedName()));
+            // Breadcrumb navigation displays nested classes as separate links.
+            // Enclosing classes may be undocumented, in which case we just display the class name.
+            case TypeElement type -> (configuration.isGeneratedDoc(type) && !configuration.utils.hasHiddenTag(type))
+                    ? links.createLink(pathToRoot.resolve(
+                            docPaths.forClass(type)), type.getSimpleName().toString())
+                    : HtmlTree.SPAN(Text.of(type.getSimpleName().toString()));
+            default -> throw new IllegalArgumentException(Objects.toString(elem));
+        };
+        if (selected) {
+            link.setStyle(HtmlStyles.currentSelection);
+        }
+        contents.add(link);
     }
 
-    private void addPackageSummaryLink(Content target) {
-        target.add(HtmlTree.LI(links.createLink(DocPaths.PACKAGE_SUMMARY, contents.packageLabel)));
+    private void addPageElementLink(Content list) {
+        Content link = switch (element) {
+            case ModuleElement mdle -> links.createLink(pathToRoot.resolve(
+                    configuration.docPaths.moduleSummary(mdle)), contents.moduleLabel);
+            case PackageElement pkg -> links.createLink(pathToRoot.resolve(
+                    configuration.docPaths.forPackage(pkg).resolve(DocPaths.PACKAGE_SUMMARY)), contents.packageLabel);
+            case TypeElement type -> links.createLink(pathToRoot.resolve(
+                    configuration.docPaths.forClass(type)), contents.classLabel);
+            default -> throw new RuntimeException();
+        };
+        list.add(HtmlTree.LI(link));
     }
 
     private void addTreeLink(Content target) {
@@ -569,6 +470,13 @@ public class Navigation {
         }
     }
 
+    private void addSearchLink(Content target) {
+        if (options.createIndex()) {
+            target.add(HtmlTree.LI(links.createLink(
+                    pathToRoot.resolve(DocPaths.SEARCH_PAGE), contents.searchLabel, "")));
+        }
+    }
+
     private void addHelpLink(Content target) {
         if (!options.noHelp()) {
             String helpfile = options.helpFile();
@@ -586,16 +494,17 @@ public class Navigation {
     }
 
     private void addSearch(Content target) {
-        String reset = "reset";
-        var inputText = HtmlTree.INPUT("text", HtmlIds.SEARCH_INPUT)
-                .put(HtmlAttr.PLACEHOLDER, searchPlaceholder);
-        var inputReset = HtmlTree.INPUT(reset, HtmlIds.RESET_BUTTON)
-                .put(HtmlAttr.VALUE, reset);
-        var searchDiv = HtmlTree.DIV(HtmlStyle.navListSearch,
-                links.createLink(pathToRoot.resolve(DocPaths.SEARCH_PAGE),
-                        searchLabel, ""));
-        searchDiv.add(inputText);
-        searchDiv.add(inputReset);
+        var resources = configuration.getDocResources();
+        var inputText = HtmlTree.INPUT(HtmlAttr.InputType.TEXT, HtmlIds.SEARCH_INPUT)
+                .put(HtmlAttr.PLACEHOLDER, resources.getText("doclet.search_placeholder"))
+                .put(HtmlAttr.ARIA_LABEL, resources.getText("doclet.search_in_documentation"))
+                .put(HtmlAttr.AUTOCOMPLETE, "off")
+                .put(HtmlAttr.SPELLCHECK, "false");
+        var inputReset = HtmlTree.INPUT(HtmlAttr.InputType.RESET, HtmlIds.RESET_SEARCH)
+                .put(HtmlAttr.VALUE, resources.getText("doclet.search_reset"));
+        var searchDiv = HtmlTree.DIV(HtmlStyles.navListSearch)
+                .add(inputText)
+                .add(inputReset);
         target.add(searchDiv);
     }
 
@@ -610,61 +519,52 @@ public class Navigation {
         }
         var navigationBar = HtmlTree.NAV();
 
-        var navDiv = new HtmlTree(TagName.DIV);
+        var navContent = HtmlTree.DIV(HtmlStyles.navContent);
         Content skipNavLinks = contents.getContent("doclet.Skip_navigation_links");
         String toggleNavLinks = configuration.getDocResources().getText("doclet.Toggle_navigation_links");
         navigationBar.add(MarkerComments.START_OF_TOP_NAVBAR);
         // The mobile menu button uses three empty spans to produce its animated icon
-        HtmlTree iconSpan = HtmlTree.SPAN(HtmlStyle.navBarToggleIcon).add(Entity.NO_BREAK_SPACE);
-        navDiv.setStyle(HtmlStyle.topNav)
-                .setId(HtmlIds.NAVBAR_TOP)
-                .add(new HtmlTree(TagName.BUTTON).setId(HtmlIds.NAVBAR_TOGGLE_BUTTON)
-                        .put(HtmlAttr.ARIA_CONTROLS, HtmlIds.NAVBAR_TOP.name())
-                        .put(HtmlAttr.ARIA_EXPANDED, String.valueOf(false))
-                        .put(HtmlAttr.ARIA_LABEL, toggleNavLinks)
-                        .add(iconSpan)
-                        .add(iconSpan)
-                        .add(iconSpan))
-                .add(HtmlTree.DIV(HtmlStyle.skipNav,
+        HtmlTree iconSpan = HtmlTree.SPAN(HtmlStyles.navBarToggleIcon).add(Entity.NO_BREAK_SPACE);
+        navContent.add(HtmlTree.DIV(HtmlStyles.navMenuButton,
+                        HtmlTree.BUTTON(HtmlIds.NAVBAR_TOGGLE_BUTTON)
+                                .put(HtmlAttr.ARIA_CONTROLS, HtmlIds.NAVBAR_TOP.name())
+                                .put(HtmlAttr.ARIA_EXPANDED, String.valueOf(false))
+                                .put(HtmlAttr.ARIA_LABEL, toggleNavLinks)
+                                .add(iconSpan)
+                                .add(iconSpan)
+                                .add(iconSpan)))
+                .add(HtmlTree.DIV(HtmlStyles.skipNav,
                         links.createLink(HtmlIds.SKIP_NAVBAR_TOP, skipNavLinks,
                                 skipNavLinks.toString())));
         Content aboutContent = userHeader;
-        boolean addSearch = options.createIndex() && documentedPage != PageMode.SEARCH;
 
-        var aboutDiv = HtmlTree.DIV(HtmlStyle.aboutLanguage, aboutContent);
-        navDiv.add(aboutDiv);
-        var navList = new HtmlTree(TagName.UL)
-                .setId(HtmlIds.NAVBAR_TOP_FIRSTROW)
-                .setStyle(HtmlStyle.navList)
+        var navList = HtmlTree.UL(HtmlIds.NAVBAR_TOP_FIRSTROW, HtmlStyles.navList)
                 .put(HtmlAttr.TITLE, rowListTitle);
         addMainNavLinks(navList);
-        navDiv.add(navList);
-        var ulNavSummaryRight = HtmlTree.UL(HtmlStyle.subNavListSmall);
-        addSummaryLinks(ulNavSummaryRight, true);
-        addDetailLinks(ulNavSummaryRight, true);
-        navDiv.add(ulNavSummaryRight);
-        navigationBar.add(navDiv);
+        navContent.add(navList);
+        var aboutDiv = HtmlTree.DIV(HtmlStyles.aboutLanguage, aboutContent);
+        navContent.add(aboutDiv);
+        navigationBar.add(HtmlTree.DIV(HtmlStyles.topNav, navContent).setId(HtmlIds.NAVBAR_TOP));
 
-        var subDiv = HtmlTree.DIV(HtmlStyle.subNav);
-
-        var div = new HtmlTree(TagName.DIV).setId(HtmlIds.NAVBAR_SUB_LIST);
-        // Add the summary links if present.
-        var ulNavSummary = HtmlTree.UL(HtmlStyle.subNavList);
-        addSummaryLinks(ulNavSummary, false);
-        div.add(ulNavSummary);
-        // Add the detail links if present.
-        var ulNavDetail = HtmlTree.UL(HtmlStyle.subNavList);
-        addDetailLinks(ulNavDetail, false);
-        div.add(ulNavDetail);
-        subDiv.add(div);
-
-        if (addSearch) {
-            addSearch(subDiv);
+        var subNavContent = HtmlTree.DIV(HtmlStyles.navContent);
+        List<Content> subNavLinks = new ArrayList<>();
+        switch (documentedPage) {
+            case MODULE, PACKAGE, CLASS, USE, DOC_FILE, TREE -> {
+                addBreadcrumbLinks(element, subNavLinks, true);
+            }
         }
-        navigationBar.add(subDiv);
+        // Add the breadcrumb navigation links if present.
+        var breadcrumbNav = HtmlTree.OL(HtmlStyles.subNavList);
+        breadcrumbNav.addAll(subNavLinks, HtmlTree::LI);
+        subNavContent.addUnchecked(breadcrumbNav);
+
+        if (options.createIndex() && documentedPage != PageMode.SEARCH) {
+            addSearch(subNavContent);
+        }
+        navigationBar.add(HtmlTree.DIV(HtmlStyles.subNav, subNavContent));
 
         navigationBar.add(MarkerComments.END_OF_TOP_NAVBAR);
-        navigationBar.add(HtmlTree.SPAN(HtmlStyle.skipNav)
+        navigationBar.add(HtmlTree.SPAN(HtmlStyles.skipNav)
                 .addUnchecked(Text.EMPTY)
                 .setId(HtmlIds.SKIP_NAVBAR_TOP));
 
