@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,21 +30,22 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.UnsupportedEncodingException;
+import java.net.URI;
+import java.net.URISyntaxException;
+import java.time.ZonedDateTime;
+import java.time.format.DateTimeFormatter;
+import java.time.format.DateTimeParseException;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collections;
 import java.util.HashSet;
-import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Locale;
 import java.util.MissingResourceException;
 import java.util.Set;
-import java.util.StringTokenizer;
 import java.util.TreeSet;
 
 import jdk.javadoc.doclet.Doclet;
 import jdk.javadoc.doclet.Reporter;
-import jdk.javadoc.internal.doclets.toolkit.util.DocletConstants;
 import jdk.javadoc.internal.doclets.toolkit.util.Utils;
 
 import static javax.tools.Diagnostic.Kind.ERROR;
@@ -78,9 +79,10 @@ public abstract class BaseOptions {
     private boolean copyDocfileSubdirs = false;
 
     /**
-     * Arguments for command-line option {@code -tag} and {@code -taglet}.
+     * Argument for command-line option {@code --date}.
+     * {@code null} if option not given.
      */
-    private final LinkedHashSet<List<String>> customTagStrs = new LinkedHashSet<>();
+    private ZonedDateTime date;
 
     /**
      * Argument for command-line option {@code -d}.
@@ -206,6 +208,13 @@ public abstract class BaseOptions {
      */
     private boolean noDeprecated = false;
 
+    /**
+     * Argument for command-line option {@code --no-fonts}.
+     * True if command-line option {@code --no-fonts} is used and font files
+     * should not be included in generated documentation. Default value is false.
+     */
+    private boolean noFonts = false;
+
     /**
      * Argument for command-line option {@code --no-platform-links}.
      * True if command-line option "--no-platform-links" is used. Default value is
@@ -249,12 +258,6 @@ public abstract class BaseOptions {
      */
     private boolean showAuthor = false;
 
-    /**
-     * Argument for command-line option {@code --show-taglets}.
-     * Show taglets (internal debug switch)
-     */
-    private boolean showTaglets = false;
-
     /**
      * Argument for command-line option {@code -version}.
      * Generate version specific information for the all the classes
@@ -282,37 +285,36 @@ public abstract class BaseOptions {
      */
     private int sourceTabSize;
 
+    /**
+     * Argument for command-line option {@code --spec-base-url}.
+     * The base URL for relative URLs in {@code @spec} tags.
+     */
+    private URI specBaseURI;
+
     /**
      * Value for command-line option {@code --override-methods summary}
-     * or  {@code --override-methods detail}.
-     * Specifies whether those methods that override a super-type's method
+     * or {@code --override-methods detail}.
+     * Specifies whether those methods that override a supertype's method
      * with no changes to the API contract should be summarized in the
      * footnote section.
      */
     private boolean summarizeOverriddenMethods = false;
 
-    /**
-     * Argument for command-line option {@code -tagletpath}.
-     * The path to Taglets
-     */
-    private String tagletPath = null;
-
-    /**
-     * Argument for command-line option {@code --snippet-path}.
-     * The path for external snippets.
-     */
-    private String snippetPath = null;
-
     //</editor-fold>
 
     private final BaseConfiguration config;
 
+    /**
+     * The default amount of space between tab stops.
+     */
+    public static final int DEFAULT_TAB_STOP_LENGTH = 8;
+
     protected BaseOptions(BaseConfiguration config) {
         this.config = config;
 
         excludedDocFileDirs = new HashSet<>();
         excludedQualifiers = new HashSet<>();
-        sourceTabSize = DocletConstants.DEFAULT_TAB_STOP_LENGTH;
+        sourceTabSize = DEFAULT_TAB_STOP_LENGTH;
         groupPairs = new ArrayList<>(0);
     }
 
@@ -338,6 +340,33 @@ public abstract class BaseOptions {
                     }
                 },
 
+                new XOption(resources, "--date", 1) {
+                    // Valid --date range: within ten years of now
+                    private static final ZonedDateTime now = ZonedDateTime.now();
+                    static final ZonedDateTime DATE_MIN = now.minusYears(10);
+                    static final ZonedDateTime DATE_MAX = now.plusYears(10);
+
+                    @Override
+                    public boolean process(String opt,  List<String> args) {
+                        if (noTimestamp) {
+                            messages.error("doclet.Option_conflict", "--date", "-notimestamp");
+                            return false;
+                        }
+                        String arg = args.get(0);
+                        try {
+                            date = ZonedDateTime.parse(arg, DateTimeFormatter.ISO_ZONED_DATE_TIME);
+                            if (date.isBefore(DATE_MIN) || date.isAfter(DATE_MAX)) {
+                                messages.error("doclet.Option_date_out_of_range", arg);
+                                return false;
+                            }
+                            return true;
+                        } catch (DateTimeParseException x) {
+                            messages.error("doclet.Option_date_not_valid", arg);
+                            return false;
+                        }
+                    }
+                },
+
                 new Option(resources, "-docencoding", 1) {
                     @Override
                     public boolean process(String opt, List<String> args) {
@@ -365,7 +394,7 @@ public abstract class BaseOptions {
                 new Option(resources, "-excludedocfilessubdir", 1) {
                     @Override
                     public boolean process(String opt, List<String> args) {
-                        addToSet(excludedDocFileDirs, args.get(0));
+                        excludedDocFileDirs.addAll(List.of(args.get(0).split("[,:]")));
                         return true;
                     }
                 },
@@ -459,6 +488,14 @@ public abstract class BaseOptions {
                     }
                 },
 
+                new Option(resources, "--no-fonts") {
+                    @Override
+                    public boolean process(String opt, List<String> args) {
+                        noFonts = true;
+                        return true;
+                    }
+                },
+
                 new Option(resources, "-nosince") {
                     @Override
                     public boolean process(String opt, List<String> args) {
@@ -471,6 +508,10 @@ public abstract class BaseOptions {
                     @Override
                     public boolean process(String opt, List<String> args) {
                         noTimestamp = true;
+                        if (date != null) {
+                            messages.error("doclet.Option_conflict", "--date", "-notimestamp");
+                            return false;
+                        }
                         return true;
                     }
                 },
@@ -478,7 +519,7 @@ public abstract class BaseOptions {
                 new Option(resources, "-noqualifier", 1) {
                     @Override
                     public boolean process(String opt, List<String> args) {
-                        addToSet(excludedQualifiers, args.get(0));
+                        excludedQualifiers.addAll(List.of(args.get(0).split("[,:]")));
                         return true;
                     }
                 },
@@ -553,50 +594,12 @@ public abstract class BaseOptions {
                         }
                         if (sourceTabSize <= 0) {
                             messages.warning("doclet.sourcetab_warning");
-                            sourceTabSize = DocletConstants.DEFAULT_TAB_STOP_LENGTH;
+                            sourceTabSize = DEFAULT_TAB_STOP_LENGTH;
                         }
                         return true;
                     }
                 },
 
-                new Option(resources, "-tag", 1) {
-                    @Override
-                    public boolean process(String opt, List<String> args) {
-                        ArrayList<String> list = new ArrayList<>();
-                        list.add(opt);
-                        list.add(args.get(0));
-                        customTagStrs.add(list);
-                        return true;
-                    }
-                },
-
-                new Option(resources, "-taglet", 1) {
-                    @Override
-                    public boolean process(String opt, List<String> args) {
-                        ArrayList<String> list = new ArrayList<>();
-                        list.add(opt);
-                        list.add(args.get(0));
-                        customTagStrs.add(list);
-                        return true;
-                    }
-                },
-
-                new Option(resources, "-tagletpath", 1) {
-                    @Override
-                    public boolean process(String opt, List<String> args) {
-                        tagletPath = args.get(0);
-                        return true;
-                    }
-                },
-
-                new Option(resources, "--snippet-path", 1) {
-                    @Override
-                    public boolean process(String opt, List<String> args) {
-                        snippetPath = args.get(0);
-                        return true;
-                    }
-                },
-
                 new Option(resources, "-version") {
                     @Override
                     public boolean process(String opt, List<String> args) {
@@ -621,20 +624,32 @@ public abstract class BaseOptions {
                     }
                 },
 
+                new Option(resources, "--spec-base-url", 1) {
+                    @Override
+                    public boolean process(String opt, List<String> args) {
+                        String arg = args.get(0);
+                        try {
+                            if (!arg.endsWith("/")) {
+                                // to ensure that URI.resolve works as expected
+                                arg += "/";
+                            }
+                            specBaseURI = new URI(arg);
+                            return true;
+                        } catch (URISyntaxException e) {
+                            config.reporter.print(ERROR,
+                                    config.getDocResources().getText("doclet.Invalid_URL",
+                                            e.getMessage()));
+                            return false;
+                        }
+                    }
+                },
+
                 new Hidden(resources, "--disable-javafx-strict-checks") {
                     @Override
                     public boolean process(String opt, List<String> args) {
                         disableJavaFxStrictChecks = true;
                         return true;
                     }
-                },
-
-                new Hidden(resources, "--show-taglets") {
-                    @Override
-                    public boolean process(String opt, List<String> args) {
-                        showTaglets = true;
-                        return true;
-                    }
                 }
         );
         return new TreeSet<>(options);
@@ -686,15 +701,6 @@ public abstract class BaseOptions {
         return true;
     }
 
-    private void addToSet(Set<String> s, String str) {
-        StringTokenizer st = new StringTokenizer(str, ":");
-        String current;
-        while (st.hasMoreTokens()) {
-            current = st.nextToken();
-            s.add(current);
-        }
-    }
-
     /**
      * Add a trailing file separator, if not found. Remove superfluous
      * file separators if any. Preserve the front double file separator for
@@ -733,10 +739,10 @@ public abstract class BaseOptions {
     }
 
     /**
-     * Arguments for command-line option {@code -tag} and {@code -taglet}.
+     * Argument for command-line option {@code --date}.
      */
-    LinkedHashSet<List<String>> customTagStrs() {
-        return customTagStrs;
+    public ZonedDateTime date() {
+        return date;
     }
 
     /**
@@ -898,6 +904,15 @@ public abstract class BaseOptions {
         return noDeprecated;
     }
 
+    /**
+     * Argument for command-line option {@code --no-fonts}.
+     * True if command-line option {@code --no-fonts"} is used.
+     * Default value is false.
+     */
+    public boolean noFonts() {
+        return noFonts;
+    }
+
     /**
      * Argument for command-line option {@code --no-platform-links}.
      * True if command-line option {@code --no-platform-links"} is used.
@@ -947,20 +962,12 @@ public abstract class BaseOptions {
      * Generate author specific information for all the classes if @author
      * tag is used in the doc comment and if -author option is used.
      * <code>showauthor</code> is set to true if -author option is used.
-     * Default is don't show author information.
+     * Default is to not show author information.
      */
     public boolean showAuthor() {
         return showAuthor;
     }
 
-    /**
-     * Argument for command-line option {@code --show-taglets}.
-     * Show taglets (internal debug switch)
-     */
-    public boolean showTaglets() {
-        return showTaglets;
-    }
-
     /**
      * Argument for command-line option {@code -version}.
      * Generate version specific information for the all the classes
@@ -976,7 +983,7 @@ public abstract class BaseOptions {
      * Arguments for command line option {@code --since}.
      */
     public List<String> since() {
-        return Collections.unmodifiableList(since);
+        return List.copyOf(since);
     }
 
     /**
@@ -994,10 +1001,18 @@ public abstract class BaseOptions {
         return sourceTabSize;
     }
 
+    /**
+     * Argument for command-line option {@code --spec-base-url}.
+     * The base URL for relative URLs in {@code @spec} tags.
+     */
+    public URI specBaseURI() {
+        return specBaseURI;
+    }
+
     /**
      * Value for command-line option {@code --override-methods summary}
-     * or  {@code --override-methods detail}.
-     * Specifies whether those methods that override a super-type's method
+     * or {@code --override-methods detail}.
+     * Specifies whether those methods that override a supertype's method
      * with no changes to the API contract should be summarized in the
      * footnote section.
      */
@@ -1005,22 +1020,6 @@ public abstract class BaseOptions {
         return summarizeOverriddenMethods;
     }
 
-    /**
-     * Argument for command-line option {@code -tagletpath}.
-     * The path to Taglets
-     */
-    public String tagletPath() {
-        return tagletPath;
-    }
-
-    /**
-     * Argument for command-line option {@code --snippet-path}.
-     * The path for external snippets.
-     */
-    public String snippetPath() {
-        return snippetPath;
-    }
-
     protected abstract static class Option implements Doclet.Option, Comparable<Option> {
         private final String[] names;
         private final String parameters;
