@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -34,6 +34,7 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 
+import jdk.internal.module.Checks;
 import jdk.internal.module.Modules;
 import jdk.jfr.AnnotationElement;
 import jdk.jfr.Enabled;
@@ -44,8 +45,10 @@ import jdk.jfr.SettingDefinition;
 import jdk.jfr.StackTrace;
 import jdk.jfr.Threshold;
 import jdk.jfr.events.ActiveSettingEvent;
+import jdk.jfr.events.StackFilter;
 import jdk.jfr.internal.settings.CutoffSetting;
 import jdk.jfr.internal.settings.EnabledSetting;
+import jdk.jfr.internal.settings.LevelSetting;
 import jdk.jfr.internal.settings.PeriodSetting;
 import jdk.jfr.internal.settings.StackTraceSetting;
 import jdk.jfr.internal.settings.ThresholdSetting;
@@ -65,6 +68,8 @@ public final class EventControl {
     private static final Type TYPE_PERIOD = TypeLibrary.createType(PeriodSetting.class);
     private static final Type TYPE_CUTOFF = TypeLibrary.createType(CutoffSetting.class);
     private static final Type TYPE_THROTTLE = TypeLibrary.createType(ThrottleSetting.class);
+    private static final long STACK_FILTER_ID = Type.getTypeId(StackFilter.class);
+    private static final Type TYPE_LEVEL = TypeLibrary.createType(LevelSetting.class);
 
     private final ArrayList<SettingControl> settingControls = new ArrayList<>();
     private final ArrayList<NamedControl> namedControls = new ArrayList<>(5);
@@ -72,7 +77,7 @@ public final class EventControl {
     private final String idName;
 
     EventControl(PlatformEventType eventType) {
-        if (eventType.hasDuration()) {
+        if (eventType.hasThreshold()) {
             addControl(Threshold.NAME, defineThreshold(eventType));
         }
         if (eventType.hasStackTrace()) {
@@ -87,8 +92,12 @@ public final class EventControl {
         if (eventType.hasThrottle()) {
             addControl(Throttle.NAME, defineThrottle(eventType));
         }
+        if (eventType.hasLevel()) {
+            addControl(Level.NAME, defineLevel(eventType));
+        }
         addControl(Enabled.NAME, defineEnabled(eventType));
 
+        addStackFilters(eventType);
         List<AnnotationElement> aes = new ArrayList<>(eventType.getAnnotationElements());
         remove(eventType, aes, Threshold.class);
         remove(eventType, aes, Period.class);
@@ -96,11 +105,84 @@ public final class EventControl {
         remove(eventType, aes, StackTrace.class);
         remove(eventType, aes, Cutoff.class);
         remove(eventType, aes, Throttle.class);
+        remove(eventType, aes, StackFilter.class);
         eventType.setAnnotations(aes);
         this.type = eventType;
         this.idName = String.valueOf(eventType.getId());
     }
 
+    private void addStackFilters(PlatformEventType eventType) {
+        String[] filter = getStackFilter(eventType);
+        if (filter != null) {
+            int size = filter.length;
+            List<String> types = new ArrayList<>(size);
+            List<String> methods = new ArrayList<>(size);
+            for (String frame : filter) {
+                int index = frame.indexOf("::");
+                String clazz = null;
+                String method = null;
+                boolean valid = false;
+                if (index != -1) {
+                    clazz = frame.substring(0, index);
+                    method = frame.substring(index + 2);
+                    if (clazz.isEmpty()) {
+                        clazz = null;
+                        valid = isValidMethod(method);
+                    } else {
+                        valid = isValidType(clazz) && isValidMethod(method);
+                    }
+                } else {
+                    clazz = frame;
+                    valid = isValidType(frame);
+                }
+                if (valid) {
+                    if (clazz == null) {
+                        types.add(null);
+                    } else {
+                        types.add(clazz.replace(".", "/"));
+                    }
+                    // If unqualified class name equals method name, it's a constructor
+                    String className = clazz.substring(clazz.lastIndexOf(".") + 1);
+                    if (className.equals(method)) {
+                        method = "<init>";
+                    }
+                    methods.add(method);
+                } else {
+                    Logger.log(LogTag.JFR, LogLevel.WARN, "@StackFrameFilter element ignored, not a valid Java identifier.");
+                }
+            }
+            if (!types.isEmpty()) {
+                String[] typeArray = types.toArray(new String[0]);
+                String[] methodArray = methods.toArray(new String[0]);
+                long id = MetadataRepository.getInstance().registerStackFilter(typeArray, methodArray);
+                eventType.setStackFilterId(id);
+            }
+        }
+    }
+
+    private String[] getStackFilter(PlatformEventType eventType) {
+        for (var a : eventType.getAnnotationElements()) {
+            if (a.getTypeId() == STACK_FILTER_ID) {
+                return (String[])a.getValue("value");
+            }
+        }
+        return null;
+    }
+
+    private boolean isValidType(String className) {
+        if (className.length() < 1 || className.length() > 65535) {
+            return false;
+        }
+        return Checks.isClassName(className);
+    }
+
+    private boolean isValidMethod(String method) {
+        if (method.length() < 1 || method.length() > 65535) {
+            return false;
+        }
+        return Checks.isJavaIdentifier(method);
+    }
+
     private boolean hasControl(String name) {
         for (NamedControl nc : namedControls) {
             if (name.equals(nc.name)) {
@@ -198,7 +280,7 @@ public final class EventControl {
         } catch (Exception e) {
             throw (Error) new InternalError("Could not get constructor for " + settingControlClass.getName()).initCause(e);
         }
-        SecuritySupport.setAccessible(cc);
+        cc.setAccessible(true);
         try {
             return (SettingControl) cc.newInstance();
         } catch (IllegalArgumentException | InvocationTargetException e) {
@@ -207,64 +289,48 @@ public final class EventControl {
     }
 
     private static Control defineEnabled(PlatformEventType type) {
-        Enabled enabled = type.getAnnotation(Enabled.class);
         // Java events are enabled by default,
         // JVM events are not, maybe they should be? Would lower learning curve
         // there too.
-        String def = type.isJVM() ? "false" : "true";
-        if (enabled != null) {
-            def = Boolean.toString(enabled.value());
-        }
+        Boolean defaultValue = Boolean.valueOf(!type.isJVM());
+        String def = type.getAnnotationValue(Enabled.class, defaultValue).toString();
         type.add(PrivateAccess.getInstance().newSettingDescriptor(TYPE_ENABLED, Enabled.NAME, def, Collections.emptyList()));
         return new Control(new EnabledSetting(type, def), def);
     }
 
     private static Control defineThreshold(PlatformEventType type) {
-        Threshold threshold = type.getAnnotation(Threshold.class);
-        String def = "0 ns";
-        if (threshold != null) {
-            def = threshold.value();
-        }
+        String def = type.getAnnotationValue(Threshold.class, ThresholdSetting.DEFAULT_VALUE);
         type.add(PrivateAccess.getInstance().newSettingDescriptor(TYPE_THRESHOLD, Threshold.NAME, def, Collections.emptyList()));
         return new Control(new ThresholdSetting(type), def);
     }
 
     private static Control defineStackTrace(PlatformEventType type) {
-        StackTrace stackTrace = type.getAnnotation(StackTrace.class);
-        String def = "true";
-        if (stackTrace != null) {
-            def = Boolean.toString(stackTrace.value());
-        }
+        String def = type.getAnnotationValue(StackTrace.class, Boolean.TRUE).toString();
         type.add(PrivateAccess.getInstance().newSettingDescriptor(TYPE_STACK_TRACE, StackTrace.NAME, def, Collections.emptyList()));
         return new Control(new StackTraceSetting(type, def), def);
     }
 
     private static Control defineCutoff(PlatformEventType type) {
-        Cutoff cutoff = type.getAnnotation(Cutoff.class);
-        String def = Cutoff.INFINITY;
-        if (cutoff != null) {
-            def = cutoff.value();
-        }
+        String def = type.getAnnotationValue(Cutoff.class, CutoffSetting.DEFAULT_VALUE);
         type.add(PrivateAccess.getInstance().newSettingDescriptor(TYPE_CUTOFF, Cutoff.NAME, def, Collections.emptyList()));
         return new Control(new CutoffSetting(type), def);
     }
 
     private static Control defineThrottle(PlatformEventType type) {
-        Throttle throttle = type.getAnnotation(Throttle.class);
-        String def = Throttle.DEFAULT;
-        if (throttle != null) {
-            def = throttle.value();
-        }
+        String def = type.getAnnotationValue(Throttle.class, ThrottleSetting.DEFAULT_VALUE);
         type.add(PrivateAccess.getInstance().newSettingDescriptor(TYPE_THROTTLE, Throttle.NAME, def, Collections.emptyList()));
         return new Control(new ThrottleSetting(type), def);
     }
 
+    private static Control defineLevel(PlatformEventType type) {
+        String[] levels = type.getAnnotationValue(Level.class, new String[0]);
+        String def = levels[0]; // Level value always exists
+        type.add(PrivateAccess.getInstance().newSettingDescriptor(TYPE_LEVEL, Level.NAME, def, Collections.emptyList()));
+        return new Control(new LevelSetting(type, levels), def);
+    }
+
     private static Control definePeriod(PlatformEventType type) {
-        Period period = type.getAnnotation(Period.class);
-        String def = "everyChunk";
-        if (period != null) {
-            def = period.value();
-        }
+        String def = type.getAnnotationValue(Period.class, PeriodSetting.DEFAULT_VALUE);
         type.add(PrivateAccess.getInstance().newSettingDescriptor(TYPE_PERIOD, PeriodSetting.NAME, def, Collections.emptyList()));
         return new Control(new PeriodSetting(type), def);
     }
@@ -289,7 +355,7 @@ public final class EventControl {
                     value = nc.control.getDefaultValue();
                 }
                 if (ActiveSettingEvent.enabled()) {
-                    ActiveSettingEvent.commit(timestamp, 0L, type.getId(), nc.name(), value);
+                    ActiveSettingEvent.commit(timestamp, type.getId(), nc.name(), value);
                 }
             }
         }
@@ -307,13 +373,6 @@ public final class EventControl {
         return idName;
     }
 
-    /**
-     * A malicious user must never be able to run a callback in the wrong
-     * context. Methods on SettingControl must therefore never be invoked directly
-     * by JFR, instead use jdk.jfr.internal.Control.
-     *
-     * The returned list is only to be used inside EventConfiguration
-     */
     public List<SettingControl> getSettingControls() {
         return settingControls;
     }
