@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,7 +28,9 @@ package jdk.jpackage.internal;
 import jdk.internal.util.Architecture;
 import jdk.internal.util.OSVersion;
 
+import java.io.ByteArrayOutputStream;
 import java.io.IOException;
+import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.net.URI;
 import java.net.URISyntaxException;
@@ -52,13 +54,17 @@ import static jdk.jpackage.internal.StandardBundlerParam.APP_NAME;
 import static jdk.jpackage.internal.StandardBundlerParam.LICENSE_FILE;
 import static jdk.jpackage.internal.StandardBundlerParam.VERSION;
 import static jdk.jpackage.internal.StandardBundlerParam.SIGN_BUNDLE;
-import static jdk.jpackage.internal.MacBaseInstallerBundler.SIGNING_KEYCHAIN;
-import static jdk.jpackage.internal.MacBaseInstallerBundler.SIGNING_KEY_USER;
+import static jdk.jpackage.internal.MacAppBundler.APP_IMAGE_SIGN_IDENTITY;
 import static jdk.jpackage.internal.StandardBundlerParam.APP_STORE;
 import static jdk.jpackage.internal.MacAppImageBuilder.MAC_CF_BUNDLE_IDENTIFIER;
 import static jdk.jpackage.internal.OverridableResource.createResource;
 import static jdk.jpackage.internal.StandardBundlerParam.RESOURCE_DIR;
 
+import jdk.jpackage.internal.model.ConfigException;
+import jdk.jpackage.internal.model.PackagerException;
+import jdk.jpackage.internal.util.FileUtils;
+import jdk.jpackage.internal.util.XmlUtils;
+
 public class MacPkgBundler extends MacBaseInstallerBundler {
 
     private static final ResourceBundle I18N = ResourceBundle.getBundle(
@@ -121,7 +127,7 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
                     }
 
                     if (result != null) {
-                        MacCertificate certificate = new MacCertificate(result);
+                        MacCertificate certificate = new MacCertificate(result, keychain);
 
                         if (!certificate.isValid()) {
                             Log.error(MessageFormat.format(
@@ -134,13 +140,6 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
                 },
             (s, p) -> s);
 
-    public static final BundlerParamInfo<String> INSTALLER_SUFFIX =
-            new StandardBundlerParam<> (
-            "mac.pkg.installerName.suffix",
-            String.class,
-            params -> "",
-            (s, p) -> s);
-
     public Path bundle(Map<String, ? super Object> params,
             Path outdir) throws PackagerException {
         Log.verbose(MessageFormat.format(I18N.getString("message.building-pkg"),
@@ -152,12 +151,23 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
             Path appImageDir = prepareAppBundle(params);
 
             if (appImageDir != null && prepareConfigFiles(params)) {
+                prepareCPLFile(params, appImageDir);
 
-                Path configScript = getConfig_Script(params);
-                if (IOUtils.exists(configScript)) {
-                    IOUtils.run("bash", configScript);
+                if (withServicesPkg(params)) {
+                    prepareServicesPkg(params);
                 }
 
+                if (!APP_STORE.fetchFrom(params)) {
+                    preparePackageScripts(params);
+                }
+
+                new ScriptRunner()
+                        .setDirectory(appImageDir)
+                        .setResourceCategoryId("resource.post-app-image-script")
+                        .setScriptNameSuffix("post-image")
+                        .setEnvironmentVariable("JpAppImageDir", appImageDir.toAbsolutePath().toString())
+                        .run(params);
+
                 return createPKG(params, outdir, appImageDir);
             }
             return null;
@@ -263,7 +273,7 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
         Log.verbose(MessageFormat.format(I18N.getString(
                 "message.preparing-distribution-dist"), f.toAbsolutePath().toString()));
 
-        IOUtils.createXml(f, xml -> {
+        XmlUtils.createXml(f, xml -> {
             xml.writeStartElement("installer-gui-script");
             xml.writeAttribute("minSpecVersion", "1");
 
@@ -358,19 +368,9 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
 
         prepareDistributionXMLFile(params);
 
-        createResource(null, params)
-                .setCategory(I18N.getString("resource.post-install-script"))
-                .saveToFile(getConfig_Script(params));
-
         return true;
     }
 
-    // name of post-image script
-    private Path getConfig_Script(Map<String, ? super Object> params) {
-        return CONFIG_ROOT.fetchFrom(params).resolve(
-                APP_NAME.fetchFrom(params) + "-post-image.sh");
-    }
-
     private void patchCPLFile(Path cpl) throws IOException {
         String cplData = Files.readString(cpl);
         String[] lines = cplData.split("\n");
@@ -448,7 +448,7 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
             source = appLocation;
             dest = newRoot.resolve(appLocation.getFileName());
         }
-        IOUtils.copyRecursive(source, dest);
+        FileUtils.copyRecursive(source, dest);
 
         return newRoot.toString();
     }
@@ -462,8 +462,7 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
         }
     }
 
-    private void createServicesPkg(Map<String, Object> params) throws
-            IOException {
+    private void prepareServicesPkg(Map<String, Object> params) throws IOException {
         Path root = TEMP_ROOT.fetchFrom(params).resolve("services");
 
         Path srcRoot = root.resolve("src");
@@ -480,22 +479,10 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
                 .setSubstitutionData(data)
                 .saveInFolder(scriptsDir);
 
-        var pb = new ProcessBuilder("/usr/bin/pkgbuild",
-                "--root",
-                srcRoot.toString(),
-                "--install-location",
-                "/",
-                "--scripts",
-                scriptsDir.toString(),
-                "--identifier",
-                getServicesIdentifier(params),
-                getPackages_ServicesPackage(params).toAbsolutePath().toString());
-        IOUtils.exec(pb, false, null, true, Executor.INFINITE_TIMEOUT);
-
-        createSupportPkg(params, data);
+        prepareSupportPkg(params, data);
     }
 
-    private void createSupportPkg(Map<String, Object> params,
+    private void prepareSupportPkg(Map<String, Object> params,
             Map<String, String> servicesSubstitutionData) throws IOException {
         Path root = TEMP_ROOT.fetchFrom(params).resolve("support");
 
@@ -516,6 +503,35 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
                         .setPublicName("uninstaller")
                         .setSubstitutionData(data))
                 .saveInFolder(srcRoot.resolve(APP_NAME.fetchFrom(params)));
+    }
+
+    private void createServicesPkg(Map<String, Object> params) throws
+            IOException {
+        Path root = TEMP_ROOT.fetchFrom(params).resolve("services");
+
+        Path srcRoot = root.resolve("src");
+
+        Path scriptsDir = root.resolve("scripts");
+
+        var pb = new ProcessBuilder("/usr/bin/pkgbuild",
+                "--root",
+                srcRoot.toString(),
+                "--install-location",
+                "/",
+                "--scripts",
+                scriptsDir.toString(),
+                "--identifier",
+                getServicesIdentifier(params),
+                getPackages_ServicesPackage(params).toAbsolutePath().toString());
+        IOUtils.exec(pb, false, null, true, Executor.INFINITE_TIMEOUT);
+
+        createSupportPkg(params);
+    }
+
+    private void createSupportPkg(Map<String, Object> params) throws IOException {
+        Path root = TEMP_ROOT.fetchFrom(params).resolve("support");
+
+        Path srcRoot = root.resolve("src");
 
         var pb = new ProcessBuilder("/usr/bin/pkgbuild",
                 "--root",
@@ -528,6 +544,23 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
         IOUtils.exec(pb, false, null, true, Executor.INFINITE_TIMEOUT);
     }
 
+    private void prepareCPLFile(Map<String, ? super Object> params, Path appLocation) throws IOException {
+        String root = getRoot(params, appLocation);
+        // Generate default CPL file
+        Path cpl = CONFIG_ROOT.fetchFrom(params).resolve("cpl.plist");
+        ProcessBuilder pb = new ProcessBuilder("/usr/bin/pkgbuild",
+                "--root",
+                root,
+                "--install-location",
+                getInstallDir(params, false),
+                "--analyze",
+                cpl.toAbsolutePath().toString());
+
+        IOUtils.exec(pb, false, null, true, Executor.INFINITE_TIMEOUT);
+
+        patchCPLFile(cpl);
+    }
+
     private Path createPKG(Map<String, ? super Object> params,
             Path outdir, Path appLocation) {
         // generic find attempt
@@ -540,19 +573,9 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
                 createServicesPkg(params);
             }
 
-            // Generate default CPL file
+            ProcessBuilder pb;
+
             Path cpl = CONFIG_ROOT.fetchFrom(params).resolve("cpl.plist");
-            ProcessBuilder pb = new ProcessBuilder("/usr/bin/pkgbuild",
-                    "--root",
-                    root,
-                    "--install-location",
-                    getInstallDir(params, false),
-                    "--analyze",
-                    cpl.toAbsolutePath().toString());
-
-            IOUtils.exec(pb, false, null, true, Executor.INFINITE_TIMEOUT);
-
-            patchCPLFile(cpl);
 
             // build application package
             if (APP_STORE.fetchFrom(params)) {
@@ -568,7 +591,6 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
                         appPKG.toAbsolutePath().toString());
                 IOUtils.exec(pb, false, null, true, Executor.INFINITE_TIMEOUT);
             } else {
-                preparePackageScripts(params);
                 pb = new ProcessBuilder("/usr/bin/pkgbuild",
                         "--root",
                         root,
@@ -587,7 +609,6 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
 
             // build final package
             Path finalPKG = outdir.resolve(MAC_INSTALLER_NAME.fetchFrom(params)
-                    + INSTALLER_SUFFIX.fetchFrom(params)
                     + ".pkg");
             Files.createDirectories(outdir);
 
@@ -605,8 +626,19 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
                     Log.verbose(I18N.getString("message.signing.pkg"));
                 }
 
-                String signingIdentity =
-                        DEVELOPER_ID_INSTALLER_SIGNING_KEY.fetchFrom(params);
+                String signingIdentity = null;
+                // --mac-installer-sign-identity
+                if (!INSTALLER_SIGN_IDENTITY.getIsDefaultValue(params)) {
+                    signingIdentity = INSTALLER_SIGN_IDENTITY.fetchFrom(params);
+                } else {
+                    // Use --mac-signing-key-user-name if user did not request
+                    // to sign just app image using --mac-app-image-sign-identity
+                    if (APP_IMAGE_SIGN_IDENTITY.getIsDefaultValue(params)) {
+                        // --mac-signing-key-user-name
+                        signingIdentity = DEVELOPER_ID_INSTALLER_SIGNING_KEY.fetchFrom(params);
+                    }
+                }
+
                 if (signingIdentity != null) {
                     commandLine.add("--sign");
                     commandLine.add(signingIdentity);
@@ -638,7 +670,21 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
             commandLine.add(finalPKG.toAbsolutePath().toString());
 
             pb = new ProcessBuilder(commandLine);
-            IOUtils.exec(pb, false, null, true, Executor.INFINITE_TIMEOUT);
+
+            try (ByteArrayOutputStream baos = new ByteArrayOutputStream();
+                 PrintStream ps = new PrintStream(baos)) {
+                try {
+                    IOUtils.exec(pb, false, ps, true, Executor.INFINITE_TIMEOUT);
+                } catch (IOException ioe) {
+                    // Log output of "productbuild" in case of
+                    // error. It should help user to diagnose
+                    // issue when using --mac-installer-sign-identity
+                    Log.info(MessageFormat.format(I18N.getString(
+                             "error.tool.failed.with.output"), "productbuild"));
+                    Log.info(baos.toString().strip());
+                    throw ioe;
+                }
+            }
 
             return finalPKG;
         } catch (Exception ignored) {
@@ -662,6 +708,7 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
     }
 
     private static boolean isValidBundleIdentifier(String id) {
+        Objects.requireNonNull(id);
         for (int i = 0; i < id.length(); i++) {
             char a = id.charAt(i);
             // We check for ASCII codes first which we accept. If check fails,
@@ -686,12 +733,6 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
             validateAppImageAndBundeler(params);
 
             String identifier = MAC_CF_BUNDLE_IDENTIFIER.fetchFrom(params);
-            if (identifier == null) {
-                throw new ConfigException(
-                        I18N.getString("message.app-image-requires-identifier"),
-                        I18N.getString(
-                            "message.app-image-requires-identifier.advice"));
-            }
             if (!isValidBundleIdentifier(identifier)) {
                 throw new ConfigException(
                         MessageFormat.format(I18N.getString(
@@ -702,14 +743,19 @@ public class MacPkgBundler extends MacBaseInstallerBundler {
             // reject explicitly set sign to true and no valid signature key
             if (Optional.ofNullable(
                     SIGN_BUNDLE.fetchFrom(params)).orElse(Boolean.FALSE)) {
-                String signingIdentity =
-                        DEVELOPER_ID_INSTALLER_SIGNING_KEY.fetchFrom(params);
-                if (signingIdentity == null) {
-                    throw new ConfigException(
-                            I18N.getString("error.explicit-sign-no-cert"),
-                            I18N.getString(
-                            "error.explicit-sign-no-cert.advice"));
+                if (!SIGNING_KEY_USER.getIsDefaultValue(params)) {
+                    String signingIdentity =
+                            DEVELOPER_ID_INSTALLER_SIGNING_KEY.fetchFrom(params);
+                    if (signingIdentity == null) {
+                        throw new ConfigException(
+                                I18N.getString("error.explicit-sign-no-cert"),
+                                I18N.getString(
+                                "error.explicit-sign-no-cert.advice"));
+                    }
                 }
+
+                // No need to validate --mac-installer-sign-identity, since it is
+                // pass through option.
             }
 
             // hdiutil is always available so there's no need
