@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -36,12 +36,12 @@ import java.nio.file.Path;
 import java.nio.file.PathMatcher;
 import java.text.MessageFormat;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
 import java.util.UUID;
@@ -55,6 +55,9 @@ import javax.xml.xpath.XPathConstants;
 import javax.xml.xpath.XPathExpressionException;
 import javax.xml.xpath.XPathFactory;
 import jdk.jpackage.internal.AppImageFile.LauncherInfo;
+import jdk.jpackage.internal.model.ConfigException;
+import jdk.jpackage.internal.model.MsiVersion;
+import jdk.jpackage.internal.model.PackagerException;
 
 import static jdk.jpackage.internal.OverridableResource.createResource;
 import static jdk.jpackage.internal.StandardBundlerParam.ABOUT_URL;
@@ -67,7 +70,7 @@ import static jdk.jpackage.internal.StandardBundlerParam.RESOURCE_DIR;
 import static jdk.jpackage.internal.StandardBundlerParam.TEMP_ROOT;
 import static jdk.jpackage.internal.StandardBundlerParam.VENDOR;
 import static jdk.jpackage.internal.StandardBundlerParam.VERSION;
-import jdk.jpackage.internal.WixToolset.WixToolsetType;
+import jdk.jpackage.internal.util.FileUtils;
 import org.w3c.dom.Document;
 import org.w3c.dom.NodeList;
 import org.xml.sax.SAXException;
@@ -228,7 +231,8 @@ public class WinMsiBundler  extends AbstractBundler {
         appImageBundler = new WinAppBundler().setDependentTask(true);
         wixFragments = Stream.of(
                 Map.entry("bundle.wxf", new WixAppImageFragmentBuilder()),
-                Map.entry("ui.wxf", new WixUiFragmentBuilder())
+                Map.entry("ui.wxf", new WixUiFragmentBuilder()),
+                Map.entry("os-condition.wxf", OSVersionCondition.createWixFragmentBuilder())
         ).<WixFragmentBuilder>map(e -> {
             e.getValue().setOutputFileName(e.getKey());
             return e.getValue();
@@ -334,14 +338,7 @@ public class WinMsiBundler  extends AbstractBundler {
 
             FileAssociation.verify(FileAssociation.fetchFrom(params));
 
-            var serviceInstallerResource = initServiceInstallerResource(params);
-            if (serviceInstallerResource != null) {
-                if (!Files.exists(serviceInstallerResource.getExternalPath())) {
-                    throw new ConfigException(I18N.getString(
-                            "error.missing-service-installer"), I18N.getString(
-                                    "error.missing-service-installer.advice"));
-                }
-            }
+            initServiceInstallerResource(params);
 
             return true;
         } catch (RuntimeException re) {
@@ -356,18 +353,14 @@ public class WinMsiBundler  extends AbstractBundler {
     private void prepareProto(Map<String, ? super Object> params)
                 throws PackagerException, IOException {
         Path appImage = StandardBundlerParam.getPredefinedAppImage(params);
-        String appName = APP_NAME.fetchFrom(params);
+        String appName = Objects.requireNonNull(APP_NAME.fetchFrom(params));
         Path appDir;
-        if (appName == null) {
-            // Can happen when no name is given, and using a foreign app-image
-            throw new PackagerException("error.no.name");
-        }
 
         // we either have an application image or need to build one
         if (appImage != null) {
             appDir = MSI_IMAGE_DIR.fetchFrom(params).resolve(appName);
             // copy everything from appImage dir into appDir/name
-            IOUtils.copyRecursive(appImage, appDir);
+            FileUtils.copyRecursive(appImage, appDir);
         } else {
             appDir = appImageBundler.execute(params, MSI_IMAGE_DIR.fetchFrom(
                     params));
@@ -408,11 +401,15 @@ public class WinMsiBundler  extends AbstractBundler {
             ensureByMutationFileIsRTF(destFile);
         }
 
-        var serviceInstallerResource = initServiceInstallerResource(params);
-        if (serviceInstallerResource != null) {
-            var serviceInstallerPath = serviceInstallerResource.getExternalPath();
-            params.put(SERVICE_INSTALLER.getID(), new InstallableFile(
-                    serviceInstallerPath, serviceInstallerPath.getFileName()));
+        try {
+            var serviceInstallerResource = initServiceInstallerResource(params);
+            if (serviceInstallerResource != null) {
+                var serviceInstallerPath = serviceInstallerResource.getExternalPath();
+                params.put(SERVICE_INSTALLER.getID(), new InstallableFile(
+                        serviceInstallerPath, serviceInstallerPath.getFileName()));
+            }
+        } catch (ConfigException ex) {
+            throw new PackagerException(ex);
         }
     }
 
@@ -434,14 +431,6 @@ public class WinMsiBundler  extends AbstractBundler {
             }
 
             Map<String, String> wixVars = prepareMainProjectFile(params);
-
-            new ScriptRunner()
-            .setDirectory(imageDir)
-            .setResourceCategoryId("resource.post-app-image-script")
-            .setScriptNameSuffix("post-image")
-            .setEnvironmentVariable("JpAppImageDir", imageDir.toAbsolutePath().toString())
-            .run(params);
-
             return buildMSI(params, wixVars, outputParentDir);
         } catch (IOException ex) {
             Log.verbose(ex);
@@ -527,9 +516,10 @@ public class WinMsiBundler  extends AbstractBundler {
                 "message.preparing-msi-config"), msiOut.toAbsolutePath()
                         .toString()));
 
-        WixPipeline wixPipeline = new WixPipeline()
-                .setToolset(wixToolset)
-                .setWixObjDir(TEMP_ROOT.fetchFrom(params).resolve("wixobj"))
+        var wixObjDir = TEMP_ROOT.fetchFrom(params).resolve("wixobj");
+
+        var wixPipeline = WixPipeline.build()
+                .setWixObjDir(wixObjDir)
                 .setWorkDir(WIN_APP_IMAGE.fetchFrom(params))
                 .addSource(CONFIG_ROOT.fetchFrom(params).resolve("main.wxs"),
                         wixVars);
@@ -602,17 +592,26 @@ public class WinMsiBundler  extends AbstractBundler {
         // Save all WiX resources into config dir.
         wixResources.saveResources();
 
+        final var imageDir = WIN_APP_IMAGE.fetchFrom(params);
+
+        new ScriptRunner()
+                .setDirectory(imageDir)
+                .setResourceCategoryId("resource.post-app-image-script")
+                .setScriptNameSuffix("post-image")
+                .setEnvironmentVariable("JpAppImageDir", imageDir.toAbsolutePath().toString())
+                .run(params);
+
         // All l10n files are supplied to WiX with "-loc", but only
         // Cultures from custom files and a single primary Culture are
         // included into "-cultures" list
         for (var wxl : primaryWxlFiles) {
-            wixPipeline.addLightOptions("-loc", wxl.toAbsolutePath().normalize().toString());
+            wixPipeline.addLightOptions("-loc", wxl.toString());
         }
 
         List<String> cultures = new ArrayList<>();
         for (var wxl : customWxlFiles) {
             wxl = configDir.resolve(wxl.getFileName());
-            wixPipeline.addLightOptions("-loc", wxl.toAbsolutePath().normalize().toString());
+            wixPipeline.addLightOptions("-loc", wxl.toString());
             cultures.add(getCultureFromWxlFile(wxl));
         }
 
@@ -639,7 +638,8 @@ public class WinMsiBundler  extends AbstractBundler {
             }
         }
 
-        wixPipeline.buildMsi(msiOut.toAbsolutePath());
+        Files.createDirectories(wixObjDir);
+        wixPipeline.create(wixToolset).buildMsi(msiOut.toAbsolutePath());
 
         return msiOut;
     }
@@ -679,14 +679,14 @@ public class WinMsiBundler  extends AbstractBundler {
             if (nodes.getLength() != 1) {
                 throw new IOException(MessageFormat.format(I18N.getString(
                         "error.extract-culture-from-wix-l10n-file"),
-                        wxlPath.toAbsolutePath()));
+                        wxlPath.toAbsolutePath().normalize()));
             }
 
             return nodes.item(0).getNodeValue();
         } catch (XPathExpressionException | ParserConfigurationException
                 | SAXException ex) {
             throw new IOException(MessageFormat.format(I18N.getString(
-                    "error.read-wix-l10n-file"), wxlPath.toAbsolutePath()), ex);
+                    "error.read-wix-l10n-file"), wxlPath.toAbsolutePath().normalize()), ex);
         }
     }
 
@@ -762,7 +762,7 @@ public class WinMsiBundler  extends AbstractBundler {
     }
 
     private static OverridableResource initServiceInstallerResource(
-            Map<String, ? super Object> params) {
+            Map<String, ? super Object> params) throws ConfigException {
         if (StandardBundlerParam.isRuntimeInstaller(params)) {
             // Runtime installer doesn't install launchers,
             // service installer not needed
@@ -780,12 +780,16 @@ public class WinMsiBundler  extends AbstractBundler {
         var result = createResource(null, params)
                 .setPublicName("service-installer.exe")
                 .setSourceOrder(OverridableResource.Source.External);
-        if (result.getResourceDir() == null) {
-            return null;
+        if (result.getResourceDir() != null) {
+            result.setExternal(result.getResourceDir().resolve(result.getPublicName()));
+
+            if (Files.exists(result.getExternalPath())) {
+                return result;
+            }
         }
 
-        return result.setExternal(result.getResourceDir().resolve(
-                result.getPublicName()));
+        throw new ConfigException(I18N.getString("error.missing-service-installer"),
+                I18N.getString("error.missing-service-installer.advice"));
     }
 
     private Path installerIcon;
