@@ -50,8 +50,6 @@
 #include <process.h>
 #pragma warning(pop)
 
-#include <sysinfoapi.h>
-
 typedef unsigned __int32 juint;
 typedef unsigned __int64 julong;
 
@@ -217,10 +215,10 @@ static PdhLookupPerfNameByIndexFunc PdhLookupPerfNameByIndex_i;
  */
 typedef struct {
     HQUERY      query;
-    uint64_t    lastUpdate; // Last time query was updated (millis)
+    uint64_t    lastUpdate; // Last time query was updated (ticks)
 } UpdateQueryS, *UpdateQueryP;
 
-// Min time between query updates (millis)
+// Min time between query updates (ticks)
 static const int MIN_UPDATE_INTERVAL = 500;
 
 /*
@@ -293,7 +291,7 @@ static const DWORD PDH_PROCESSOR_TIME_IDX = 6;
 static const DWORD PDH_PROCESS_IDX = 230;
 static const DWORD PDH_ID_PROCESS_IDX = 784;
 
-/* useful pdh fmt's */
+/* PDH format patterns, and lengths of their constant component. */
 static const char* const OBJECT_COUNTER_FMT = "\\%s\\%s";
 static const size_t OBJECT_COUNTER_FMT_LEN = 2;
 static const char* const OBJECT_WITH_INSTANCES_COUNTER_FMT = "\\%s(%s)\\%s";
@@ -407,8 +405,8 @@ makeFullCounterPath(const char* const objectName,
     assert(objectName);
     assert(counterName);
 
-    fullCounterPathLen = strlen(objectName);
-    fullCounterPathLen += strlen(counterName);
+    // Always include space for null terminator:
+    fullCounterPathLen = strlen(objectName) + strlen(counterName) + 1;
 
     if (imageName) {
         /*
@@ -431,20 +429,19 @@ makeFullCounterPath(const char* const objectName,
         assert(instance);
 
         fullCounterPathLen += strlen(instance);
-
-        fullCounterPath = malloc(fullCounterPathLen + 1);
+        fullCounterPath = malloc(fullCounterPathLen);
 
         if (!fullCounterPath) {
             return NULL;
         }
 
-        _snprintf(fullCounterPath,
-                  fullCounterPathLen,
-                  PROCESS_OBJECT_INSTANCE_COUNTER_FMT,
-                  objectName,
-                  imageName,
-                  instance,
-                  counterName);
+        snprintf(fullCounterPath,
+                 fullCounterPathLen,
+                 PROCESS_OBJECT_INSTANCE_COUNTER_FMT,
+                 objectName,
+                 imageName,
+                 instance,
+                 counterName);
     } else {
         if (instance) {
             /*
@@ -467,30 +464,28 @@ makeFullCounterPath(const char* const objectName,
             fullCounterPathLen += OBJECT_COUNTER_FMT_LEN;
         }
 
-        fullCounterPath = malloc(fullCounterPathLen + 1);
+        fullCounterPath = malloc(fullCounterPathLen);
 
         if (!fullCounterPath) {
             return NULL;
         }
 
         if (instance) {
-            _snprintf(fullCounterPath,
-                      fullCounterPathLen,
-                      OBJECT_WITH_INSTANCES_COUNTER_FMT,
-                      objectName,
-                      instance,
-                      counterName);
+            snprintf(fullCounterPath,
+                     fullCounterPathLen,
+                     OBJECT_WITH_INSTANCES_COUNTER_FMT,
+                     objectName,
+                     instance,
+                     counterName);
         } else {
-            _snprintf(fullCounterPath,
-                      fullCounterPathLen,
-                      OBJECT_COUNTER_FMT,
-                      objectName,
-                      counterName);
+            snprintf(fullCounterPath,
+                     fullCounterPathLen,
+                     OBJECT_COUNTER_FMT,
+                     objectName,
+                     counterName);
         }
     }
 
-    fullCounterPath[fullCounterPathLen] = '\0';
-
     return fullCounterPath;
 }
 
@@ -700,6 +695,8 @@ getProcessID() {
  * (in order to keep this index valid when the list resets from underneath,
  * ensure to call getCurrentQueryIndexForProcess() before every query involving
  * Process object instance data).
+ *
+ * Returns -1 on failure.
  */
 static int
 currentQueryIndexForProcess(void) {
@@ -719,10 +716,10 @@ currentQueryIndexForProcess(void) {
             PDH_FMT_COUNTERVALUE counterValue;
             PDH_STATUS res;
 
-            _snprintf(fullIDProcessCounterPath,
-                      MAX_PATH,
-                      pdhIDProcessCounterFmt,
-                      index);
+            snprintf(fullIDProcessCounterPath,
+                     MAX_PATH,
+                     pdhIDProcessCounterFmt,
+                     index);
 
             if (addCounter(tmpQuery, fullIDProcessCounterPath, &handleCounter) != 0) {
                 break;
@@ -777,8 +774,7 @@ static int
 getCurrentQueryIndexForProcess() {
     int currentQueryIndex = currentQueryIndexForProcess();
 
-    assert(currentQueryIndex >= 0 &&
-           currentQueryIndex < numberOfJavaProcessesAtInitialization);
+    assert(currentQueryIndex < numberOfJavaProcessesAtInitialization);
 
     return currentQueryIndex;
 }
@@ -995,7 +991,7 @@ bindPdhFunctionPointers(HMODULE h) {
  */
 static int
 getPerformanceData(UpdateQueryP query, HCOUNTER c, PDH_FMT_COUNTERVALUE* value, DWORD format) {
-    uint64_t now = GetTickCount64();
+    clock_t now = clock();
 
     /*
      * Need to limit how often we update the query
@@ -1051,24 +1047,22 @@ allocateAndInitializePdhConstants() {
     pdhIDProcessCounterFmtLen += strlen(pdhLocalizedProcessObject);
     pdhIDProcessCounterFmtLen += strlen(pdhLocalizedIDProcessCounter);
     pdhIDProcessCounterFmtLen += PROCESS_OBJECT_INSTANCE_COUNTER_FMT_LEN;
-    pdhIDProcessCounterFmtLen += 2; // "%d"
+    pdhIDProcessCounterFmtLen += 3; // "%d" and '\0'
 
     assert(pdhIDProcessCounterFmtLen < MAX_PATH);
-    pdhIDProcessCounterFmt = malloc(pdhIDProcessCounterFmtLen + 1);
+    pdhIDProcessCounterFmt = malloc(pdhIDProcessCounterFmtLen);
     if (!pdhIDProcessCounterFmt) {
         goto end;
     }
 
     /* "\Process(java#%d)\ID Process" */
-    _snprintf(pdhIDProcessCounterFmt,
-              pdhIDProcessCounterFmtLen,
-              PROCESS_OBJECT_INSTANCE_COUNTER_FMT,
-              pdhLocalizedProcessObject,
-              pdhProcessImageName,
-              "%d",
-              pdhLocalizedIDProcessCounter);
-
-    pdhIDProcessCounterFmt[pdhIDProcessCounterFmtLen] = '\0';
+    snprintf(pdhIDProcessCounterFmt,
+             pdhIDProcessCounterFmtLen,
+             PROCESS_OBJECT_INSTANCE_COUNTER_FMT,
+             pdhLocalizedProcessObject,
+             pdhProcessImageName,
+             "%d",
+             pdhLocalizedIDProcessCounter);
 
     assert(0 == numberOfJavaProcessesAtInitialization);
     currentQueryIndex = currentQueryIndexForProcess();
@@ -1312,11 +1306,10 @@ perfGetProcessCPULoad() {
     }
 
     currentQueryIndex = getCurrentQueryIndexForProcess();
-
-    if (getPerformanceData(&processTotalCPULoad[currentQueryIndex].query,
-                           processTotalCPULoad[currentQueryIndex].counter,
-                           &cv,
-                           PDH_FMT_DOUBLE | PDH_FMT_NOCAP100) == 0) {
+    if (currentQueryIndex >= 0 && getPerformanceData(&processTotalCPULoad[currentQueryIndex].query,
+                                                     processTotalCPULoad[currentQueryIndex].counter,
+                                                     &cv,
+                                                     PDH_FMT_DOUBLE | PDH_FMT_NOCAP100) == 0) {
         double d = cv.doubleValue / cpuFactor;
         d = min(1, d);
         d = max(0, d);
