@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,14 +26,15 @@
 package com.sun.jndi.rmi.registry;
 
 
+import com.sun.naming.internal.NamingManagerHelper;
+import com.sun.naming.internal.ObjectFactoriesFilter;
+
 import java.util.Hashtable;
 import java.util.Properties;
 import java.rmi.*;
 import java.rmi.server.*;
 import java.rmi.registry.Registry;
 import java.rmi.registry.LocateRegistry;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 
 import javax.naming.*;
 import javax.naming.spi.NamingManager;
@@ -54,27 +55,9 @@ public class RegistryContext implements Context, Referenceable {
     private int port;
     private static final NameParser nameParser = new AtomicNameParser();
     private static final String SOCKET_FACTORY = "com.sun.jndi.rmi.factory.socket";
-    /**
-     * Determines whether classes may be loaded from an arbitrary URL code base.
-     */
-    static final boolean trustURLCodebase;
-    static {
-        // System property to control whether classes may be loaded from an
-        // arbitrary URL codebase
-        PrivilegedAction<String> act = () -> System.getProperty(
-            "com.sun.jndi.rmi.object.trustURLCodebase", "false");
-        @SuppressWarnings("removal")
-        String trust = AccessController.doPrivileged(act);
-        trustURLCodebase = "true".equalsIgnoreCase(trust);
-    }
 
     Reference reference = null; // ref used to create this context, if any
 
-    // Environment property that, if set, indicates that a security
-    // manager should be installed (if none is already in place).
-    public static final String SECURITY_MGR =
-            "java.naming.rmi.security.manager";
-
     /**
      * Returns a context for the registry at a given host and port.
      * If "host" is null, uses default host.
@@ -89,9 +72,6 @@ public class RegistryContext implements Context, Referenceable {
         environment = (env == null)
                       ? new Hashtable<String, Object>(5)
                       : (Hashtable<String, Object>) env;
-        if (environment.get(SECURITY_MGR) != null) {
-            installSecurityMgr();
-        }
 
         // chop off '[' and ']' in an IPv6 literal address
         if ((host != null) && (host.charAt(0) == '[')) {
@@ -120,11 +100,6 @@ public class RegistryContext implements Context, Referenceable {
         reference = ctx.reference;
     }
 
-    @SuppressWarnings("removal")
-    protected void finalize() {
-        close();
-    }
-
     public Object lookup(Name name) throws NamingException {
         if (name.isEmpty()) {
             return (new RegistryContext(this));
@@ -307,9 +282,6 @@ public class RegistryContext implements Context, Referenceable {
     public Object addToEnvironment(String propName, Object propVal)
             throws NamingException
     {
-        if (propName.equals(SECURITY_MGR)) {
-            installSecurityMgr();
-        }
         return environment.put(propName, propVal);
     }
 
@@ -424,19 +396,6 @@ public class RegistryContext implements Context, Referenceable {
         }
     }
 
-    /**
-     * Attempts to install a security manager if none is currently in
-     * place.
-     */
-    @SuppressWarnings("removal")
-    private static void installSecurityMgr() {
-
-        try {
-            System.setSecurityManager(new SecurityManager());
-        } catch (Exception e) {
-        }
-    }
-
     /**
      * Encodes an object prior to binding it in the registry.  First,
      * NamingManager.getStateToBind() is invoked.  If the resulting
@@ -478,12 +437,6 @@ public class RegistryContext implements Context, Referenceable {
                         ? ((RemoteReference)r).getReference()
                         : (Object)r;
 
-            /*
-             * Classes may only be loaded from an arbitrary URL codebase when
-             * the system property com.sun.jndi.rmi.object.trustURLCodebase
-             * has been set to "true".
-             */
-
             // Use reference if possible
             Reference ref = null;
             if (obj instanceof Reference) {
@@ -492,14 +445,17 @@ public class RegistryContext implements Context, Referenceable {
                 ref = ((Referenceable)(obj)).getReference();
             }
 
-            if (ref != null && ref.getFactoryClassLocation() != null &&
-                !trustURLCodebase) {
+            /*
+             * Downloading a factory class from a location specified in the reference
+             * can be supported by a custom implementation of "ObjectFactoryBuilder".
+             */
+            if (NamingManagerHelper.getObjectFactoryBuilder() == null
+                && ref != null && ref.getFactoryClassLocation() != null) {
                 throw new ConfigurationException(
-                    "The object factory is untrusted. Set the system property" +
-                    " 'com.sun.jndi.rmi.object.trustURLCodebase' to 'true'.");
+                    "Remote object factories are not supported");
             }
-            return NamingManager.getObjectInstance(obj, name, this,
-                                                   environment);
+            return NamingManagerHelper.getObjectInstance(obj, name, this,
+                    environment, ObjectFactoriesFilter::checkRmiFilter);
         } catch (NamingException e) {
             throw e;
         } catch (RemoteException e) {
@@ -596,11 +552,6 @@ class BindingEnumeration implements NamingEnumeration<Binding> {
         nextName = 0;
     }
 
-    @SuppressWarnings("removal")
-    protected void finalize() {
-        ctx.close();
-    }
-
     public boolean hasMore() {
         if (nextName >= names.length) {
             ctx.close();
@@ -637,8 +588,7 @@ class BindingEnumeration implements NamingEnumeration<Binding> {
         }
     }
 
-    @SuppressWarnings("deprecation")
     public void close () {
-        finalize();
+        ctx.close();
     }
 }
