@@ -29,8 +29,9 @@
 #include "gc/g1/g1HeapRegionSet.hpp"
 #include "gc/g1/g1RegionToSpaceMapper.hpp"
 #include "memory/allocation.hpp"
+#include "memory/memoryReserver.hpp"
 #include "memory/memRegion.hpp"
-#include "memory/virtualspace.hpp"
+
 #include "unittest.hpp"
 
 // @requires UseG1GC
@@ -50,7 +51,7 @@ TEST_OTHER_VM(G1FreeRegionList, length) {
   // the BOT.
   size_t bot_size = G1BlockOffsetTable::compute_size(heap.word_size());
   HeapWord* bot_data = NEW_C_HEAP_ARRAY(HeapWord, bot_size, mtGC);
-  ReservedSpace bot_rs(G1BlockOffsetTable::compute_size(heap.word_size()));
+  ReservedSpace bot_rs = MemoryReserver::reserve(G1BlockOffsetTable::compute_size(heap.word_size()), mtGC);
   G1RegionToSpaceMapper* bot_storage =
     G1RegionToSpaceMapper::create_mapper(bot_rs,
                                          bot_rs.size(),
