@@ -1,6 +1,6 @@
 /*
  * Copyright Amazon.com Inc. or its affiliates. All Rights Reserved.
- * Copyright (c) 2022, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2022, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  * questions.
  *
  */
-#include "precompiled.hpp"
 #include "jvm.h"
 #include "logging/log.hpp"
 #include "logging/logAsyncWriter.hpp"
@@ -255,7 +254,10 @@ TEST_VM_F(AsyncLogTest, droppingMessage) {
 
 TEST_VM_F(AsyncLogTest, stdoutOutput) {
   testing::internal::CaptureStdout();
-  set_log_config("stdout", "logging=debug");
+
+  if (!set_log_config("stdout", "logging=debug")) {
+    return;
+  }
 
   bool async = AsyncLogWriter::instance() != nullptr;
   if (async) {
@@ -266,20 +268,25 @@ TEST_VM_F(AsyncLogTest, stdoutOutput) {
   }
 
   fflush(nullptr);
-  if (write_to_file(testing::internal::GetCapturedStdout())) {
-    EXPECT_TRUE(file_contains_substring(TestLogFileName, "LogStreamWithAsyncLogImpl"));
-    EXPECT_TRUE(file_contains_substring(TestLogFileName, "logStream msg1-msg2-msg3"));
-    EXPECT_TRUE(file_contains_substring(TestLogFileName, "logStream newline"));
+  if (!write_to_file(testing::internal::GetCapturedStdout())) {
+    return;
+  }
 
-    if (async) {
-      EXPECT_TRUE(file_contains_substring(TestLogFileName, "messages dropped due to async logging"));
-    }
+  EXPECT_TRUE(file_contains_substring(TestLogFileName, "LogStreamWithAsyncLogImpl"));
+  EXPECT_TRUE(file_contains_substring(TestLogFileName, "logStream msg1-msg2-msg3"));
+  EXPECT_TRUE(file_contains_substring(TestLogFileName, "logStream newline"));
+
+  if (async) {
+    EXPECT_TRUE(file_contains_substring(TestLogFileName, "messages dropped due to async logging"));
   }
 }
 
 TEST_VM_F(AsyncLogTest, stderrOutput) {
   testing::internal::CaptureStderr();
-  set_log_config("stderr", "logging=debug");
+
+  if (!set_log_config("stderr", "logging=debug")) {
+    return;
+  }
 
   bool async = AsyncLogWriter::instance() != nullptr;
   if (async) {
@@ -290,13 +297,15 @@ TEST_VM_F(AsyncLogTest, stderrOutput) {
   }
 
   fflush(nullptr);
-  if (write_to_file(testing::internal::GetCapturedStderr())) {
-    EXPECT_TRUE(file_contains_substring(TestLogFileName, "LogStreamWithAsyncLogImpl"));
-    EXPECT_TRUE(file_contains_substring(TestLogFileName, "logStream msg1-msg2-msg3"));
-    EXPECT_TRUE(file_contains_substring(TestLogFileName, "logStream newline"));
+  if (!write_to_file(testing::internal::GetCapturedStderr())) {
+    return;
+  }
 
-    if (async) {
-      EXPECT_TRUE(file_contains_substring(TestLogFileName, "messages dropped due to async logging"));
-    }
+  EXPECT_TRUE(file_contains_substring(TestLogFileName, "LogStreamWithAsyncLogImpl"));
+  EXPECT_TRUE(file_contains_substring(TestLogFileName, "logStream msg1-msg2-msg3"));
+  EXPECT_TRUE(file_contains_substring(TestLogFileName, "logStream newline"));
+
+  if (async) {
+    EXPECT_TRUE(file_contains_substring(TestLogFileName, "messages dropped due to async logging"));
   }
 }
