@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -21,7 +21,6 @@
  * questions.
  */
 
-#include "precompiled.hpp"
 #include "concurrentTestRunner.inline.hpp"
 #include "jvm.h"
 #include "logTestFixture.hpp"
@@ -73,8 +72,8 @@ TEST_VM_F(LogConfigurationTest, describe) {
   const char* description = ss.as_string();
 
   // Verify that stdout and stderr are listed by default
-  EXPECT_THAT(description, HasSubstr(StdoutLog->name()));
-  EXPECT_THAT(description, HasSubstr(StderrLog->name()));
+  EXPECT_THAT(description, HasSubstr(LogConfiguration::StdoutLog->name()));
+  EXPECT_THAT(description, HasSubstr(LogConfiguration::StderrLog->name()));
 
   // Verify that each tag, level and decorator is listed
   for (size_t i = 0; i < LogTag::Count; i++) {
@@ -131,7 +130,7 @@ TEST_VM_F(LogConfigurationTest, update_output) {
     EXPECT_TRUE(is_described("all=info"));
 
     // Verify by iterating over tagsets
-    LogOutput* o = StdoutLog;
+    LogOutput* o = LogConfiguration::StdoutLog;
     for (LogTagSet* ts = LogTagSet::first(); ts != nullptr; ts = ts->next()) {
       EXPECT_TRUE(ts->has_output(o));
       EXPECT_TRUE(ts->is_level(LogLevel::Info));
@@ -183,8 +182,8 @@ TEST_VM_F(LogConfigurationTest, disable_logging) {
 
   // Verify that no tagset has logging enabled
   for (LogTagSet* ts = LogTagSet::first(); ts != nullptr; ts = ts->next()) {
-    EXPECT_FALSE(ts->has_output(StdoutLog));
-    EXPECT_FALSE(ts->has_output(StderrLog));
+    EXPECT_FALSE(ts->has_output(LogConfiguration::StdoutLog));
+    EXPECT_FALSE(ts->has_output(LogConfiguration::StderrLog));
     EXPECT_FALSE(ts->is_level(LogLevel::Error));
   }
 }
@@ -198,7 +197,7 @@ TEST_VM_F(LogConfigurationTest, disable_output) {
   EXPECT_TRUE(is_described("#0: stdout all=off"));
 
   // Verify by iterating over tagsets
-  LogOutput* o = StdoutLog;
+  LogOutput* o = LogConfiguration::StdoutLog;
   for (LogTagSet* ts = LogTagSet::first(); ts != nullptr; ts = ts->next()) {
     EXPECT_FALSE(ts->has_output(o));
     EXPECT_FALSE(ts->is_level(LogLevel::Error));
@@ -338,7 +337,7 @@ TEST_VM_F(LogConfigurationTest, parse_empty_command_line_arguments) {
     bool ret = LogConfiguration::parse_command_line_arguments(cmdline);
     EXPECT_TRUE(ret) << "Error parsing command line arguments '" << cmdline << "'";
     for (LogTagSet* ts = LogTagSet::first(); ts != nullptr; ts = ts->next()) {
-      EXPECT_EQ(LogLevel::Unspecified, ts->level_for(StdoutLog));
+      EXPECT_EQ(LogLevel::Unspecified, ts->level_for(LogConfiguration::StdoutLog));
     }
   }
 }
@@ -420,7 +419,7 @@ TEST_VM_F(LogConfigurationTest, configure_stdout) {
   EXPECT_FALSE(log_is_enabled(Debug, logging));
   EXPECT_FALSE(log_is_enabled(Info, gc));
   LogTagSet* logging_ts = &LogTagSetMapping<LOG_TAGS(logging)>::tagset();
-  EXPECT_EQ(LogLevel::Info, logging_ts->level_for(StdoutLog));
+  EXPECT_EQ(LogLevel::Info, logging_ts->level_for(LogConfiguration::StdoutLog));
 
   // Enable 'gc=debug' (no wildcard), verifying no other tags are enabled
   LogConfiguration::configure_stdout(LogLevel::Debug, true, LOG_TAGS(gc));
@@ -430,9 +429,9 @@ TEST_VM_F(LogConfigurationTest, configure_stdout) {
   for (LogTagSet* ts = LogTagSet::first(); ts != nullptr; ts = ts->next()) {
     if (ts->contains(PREFIX_LOG_TAG(gc))) {
       if (ts->ntags() == 1) {
-        EXPECT_EQ(LogLevel::Debug, ts->level_for(StdoutLog));
+        EXPECT_EQ(LogLevel::Debug, ts->level_for(LogConfiguration::StdoutLog));
       } else {
-        EXPECT_EQ(LogLevel::Off, ts->level_for(StdoutLog));
+        EXPECT_EQ(LogLevel::Off, ts->level_for(LogConfiguration::StdoutLog));
       }
     }
   }
@@ -443,12 +442,12 @@ TEST_VM_F(LogConfigurationTest, configure_stdout) {
   EXPECT_TRUE(log_is_enabled(Trace, gc, heap));
   for (LogTagSet* ts = LogTagSet::first(); ts != nullptr; ts = ts->next()) {
     if (ts->contains(PREFIX_LOG_TAG(gc))) {
-      EXPECT_EQ(LogLevel::Trace, ts->level_for(StdoutLog));
+      EXPECT_EQ(LogLevel::Trace, ts->level_for(LogConfiguration::StdoutLog));
     } else if (ts == logging_ts) {
       // Previous setting for 'logging' should remain
-      EXPECT_EQ(LogLevel::Info, ts->level_for(StdoutLog));
+      EXPECT_EQ(LogLevel::Info, ts->level_for(LogConfiguration::StdoutLog));
     } else {
-      EXPECT_EQ(LogLevel::Off, ts->level_for(StdoutLog));
+      EXPECT_EQ(LogLevel::Off, ts->level_for(LogConfiguration::StdoutLog));
     }
   }
 
@@ -459,7 +458,7 @@ TEST_VM_F(LogConfigurationTest, configure_stdout) {
   EXPECT_FALSE(log_is_enabled(Error, gc));
   EXPECT_FALSE(log_is_enabled(Error, gc, heap));
   for (LogTagSet* ts = LogTagSet::first(); ts != nullptr; ts = ts->next()) {
-    EXPECT_EQ(LogLevel::Off, ts->level_for(StdoutLog));
+    EXPECT_EQ(LogLevel::Off, ts->level_for(LogConfiguration::StdoutLog));
   }
 }
 
