@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -21,8 +21,7 @@
  * questions.
  */
 
-#include "precompiled.hpp"
-#include "memory/virtualspace.hpp"
+#include "memory/memoryReserver.hpp"
 #include "oops/oop.hpp"
 #include "runtime/os.hpp"
 #include "utilities/align.hpp"
@@ -35,11 +34,7 @@ namespace {
    public:
     MemoryReleaser(ReservedSpace* rs) : _rs(rs) { }
     ~MemoryReleaser() {
-      if (_rs->special()) {
-        EXPECT_TRUE(os::release_memory_special(_rs->base(), _rs->size()));
-      } else {
-        EXPECT_TRUE(os::release_memory(_rs->base(), _rs->size()));
-      }
+      EXPECT_TRUE(MemoryReserver::release(*_rs));
     }
   };
 
@@ -64,7 +59,7 @@ namespace {
   static void test_reserved_size(size_t size) {
     ASSERT_PRED2(is_size_aligned, size, os::vm_allocation_granularity());
 
-    ReservedSpace rs(size);
+    ReservedSpace rs = MemoryReserver::reserve(size, mtTest);
     MemoryReleaser releaser(&rs);
 
     EXPECT_TRUE(rs.base() != nullptr) << "rs.special: " << rs.special();
@@ -78,7 +73,7 @@ namespace {
   static void test_reserved_size_alignment(size_t size, size_t alignment) {
     ASSERT_PRED2(is_size_aligned, size, alignment) << "Incorrect input parameters";
     size_t page_size = UseLargePages ? os::large_page_size() : os::vm_page_size();
-    ReservedSpace rs(size, alignment, page_size, (char *) nullptr);
+    ReservedSpace rs = MemoryReserver::reserve(size, alignment, page_size, mtTest);
 
     ASSERT_TRUE(rs.base() != nullptr) << "rs.special = " << rs.special();
     ASSERT_EQ(size, rs.size()) << "rs.special = " << rs.special();
@@ -106,7 +101,7 @@ namespace {
     bool large = maybe_large && UseLargePages && size >= os::large_page_size();
     size_t page_size = large ? os::large_page_size() : os::vm_page_size();
 
-    ReservedSpace rs(size, alignment, page_size);
+    ReservedSpace rs = MemoryReserver::reserve(size, alignment, page_size, mtTest);
     MemoryReleaser releaser(&rs);
 
     EXPECT_TRUE(rs.base() != nullptr) << "rs.special: " << rs.special();
@@ -206,7 +201,9 @@ namespace {
    public:
     ReservedSpaceReleaser(ReservedSpace* rs) : _rs(rs) { }
     ~ReservedSpaceReleaser() {
-      _rs->release();
+      if (_rs->is_reserved()) {
+        MemoryReserver::release(*_rs);
+      }
     }
   };
 
@@ -215,12 +212,13 @@ namespace {
       default:
       case Default:
       case Reserve:
-        return ReservedSpace(reserve_size_aligned);
+        return MemoryReserver::reserve(reserve_size_aligned, mtTest);
       case Disable:
       case Commit:
-        return ReservedSpace(reserve_size_aligned,
-                             os::vm_allocation_granularity(),
-                             os::vm_page_size());
+        return MemoryReserver::reserve(reserve_size_aligned,
+                                       os::vm_allocation_granularity(),
+                                       os::vm_page_size(),
+                                       mtTest);
     }
   }
 
@@ -299,7 +297,7 @@ TEST_VM(VirtualSpace, actual_committed_space_one_large_page) {
 
   size_t large_page_size = os::large_page_size();
 
-  ReservedSpace reserved(large_page_size, large_page_size, large_page_size);
+  ReservedSpace reserved = MemoryReserver::reserve(large_page_size, large_page_size, large_page_size, mtTest);
   ReservedSpaceReleaser releaser(&reserved);
   ASSERT_TRUE(reserved.is_reserved());
 
@@ -366,10 +364,10 @@ class TestReservedSpace : AllStatic {
   static void test_reserved_space1(size_t size, size_t alignment) {
     ASSERT_TRUE(is_aligned(size, alignment)) << "Incorrect input parameters";
     size_t page_size = UseLargePages ? os::large_page_size() : os::vm_page_size();
-    ReservedSpace rs(size,          // size
-                     alignment,     // alignment
-                     page_size, // page size
-                     (char *)nullptr); // requested_address
+    ReservedSpace rs = MemoryReserver::reserve(size,
+                                               alignment,
+                                               page_size,
+                                               mtTest);
 
     EXPECT_TRUE(rs.base() != nullptr);
     EXPECT_EQ(rs.size(), size) <<  "rs.size: " << rs.size();
@@ -387,7 +385,7 @@ class TestReservedSpace : AllStatic {
   static void test_reserved_space2(size_t size) {
     ASSERT_TRUE(is_aligned(size, os::vm_allocation_granularity())) << "Must be at least AG aligned";
 
-    ReservedSpace rs(size);
+    ReservedSpace rs = MemoryReserver::reserve(size, mtTest);
 
     EXPECT_TRUE(rs.base() != nullptr);
     EXPECT_EQ(rs.size(), size) <<  "rs.size: " << rs.size();
@@ -412,7 +410,10 @@ class TestReservedSpace : AllStatic {
     bool large = maybe_large && UseLargePages && size >= os::large_page_size();
     size_t page_size = large ? os::large_page_size() : os::vm_page_size();
 
-    ReservedSpace rs(size, alignment, page_size);
+    ReservedSpace rs = MemoryReserver::reserve(size,
+                                               alignment,
+                                               page_size,
+                                               mtTest);
 
     EXPECT_TRUE(rs.base() != nullptr);
     EXPECT_EQ(rs.size(), size) <<  "rs.size: " << rs.size();
@@ -516,12 +517,14 @@ class TestVirtualSpace : AllStatic {
     default:
     case Default:
     case Reserve:
-      return ReservedSpace(reserve_size_aligned);
+      return MemoryReserver::reserve(reserve_size_aligned,
+                                     mtTest);
     case Disable:
     case Commit:
-      return ReservedSpace(reserve_size_aligned,
-                           os::vm_allocation_granularity(),
-                           os::vm_page_size());
+      return MemoryReserver::reserve(reserve_size_aligned,
+                                     os::vm_allocation_granularity(),
+                                     os::vm_page_size(),
+                                     mtTest);
     }
   }
 
@@ -546,7 +549,7 @@ class TestVirtualSpace : AllStatic {
 
     ReservedSpace reserved = reserve_memory(reserve_size_aligned, mode);
 
-    EXPECT_TRUE(reserved.is_reserved());
+    ASSERT_TRUE(reserved.is_reserved());
 
     VirtualSpace vs;
     bool initialized = initialize_virtual_space(vs, reserved, mode);
@@ -566,7 +569,7 @@ class TestVirtualSpace : AllStatic {
       EXPECT_LT(vs.actual_committed_size(), commit_size + commit_granularity);
     }
 
-    reserved.release();
+    MemoryReserver::release(reserved);
   }
 
   static void test_virtual_space_actual_committed_space_one_large_page() {
@@ -576,9 +579,12 @@ class TestVirtualSpace : AllStatic {
 
     size_t large_page_size = os::large_page_size();
 
-    ReservedSpace reserved(large_page_size, large_page_size, large_page_size);
+    ReservedSpace reserved = MemoryReserver::reserve(large_page_size,
+                                                     large_page_size,
+                                                     large_page_size,
+                                                     mtTest);
 
-    EXPECT_TRUE(reserved.is_reserved());
+    ASSERT_TRUE(reserved.is_reserved());
 
     VirtualSpace vs;
     bool initialized = vs.initialize(reserved, 0);
@@ -588,7 +594,7 @@ class TestVirtualSpace : AllStatic {
 
     EXPECT_EQ(vs.actual_committed_size(), large_page_size);
 
-    reserved.release();
+    MemoryReserver::release(reserved);
   }
 
   static void test_virtual_space_actual_committed_space() {
