@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,6 +30,9 @@ import jdk.test.lib.Utils;
 import jdk.test.lib.process.OutputAnalyzer;
 import jdk.test.lib.process.ProcessTools;
 import jdk.test.lib.util.CoreUtils;
+import jdk.test.whitebox.WhiteBox;
+
+import jtreg.SkippedException;
 
 import java.io.BufferedReader;
 import java.io.File;
@@ -126,6 +129,11 @@ public abstract class CiReplayBase {
     }
 
     public void runTest(boolean needCoreDump, String... args) {
+        // The CiReplay tests don't work properly when CDS is disabled
+        boolean cdsEnabled = WhiteBox.getWhiteBox().isSharingEnabled();
+        if (!cdsEnabled) {
+            throw new SkippedException("CDS is not available for this JDK.");
+        }
         cleanup();
         if (generateReplay(needCoreDump, args)) {
             testAction();
