@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,13 +25,16 @@
  * @test
  * @requires vm.jvmci
  * @summary Test CONSTANT_Dynamic resolution by HotSpotConstantPool.
- * @modules java.base/jdk.internal.org.objectweb.asm
- *          jdk.internal.vm.ci/jdk.vm.ci.hotspot:+open
+ * @library /testlibrary/asm
+ * @modules jdk.internal.vm.ci/jdk.vm.ci.hotspot:+open
  *          jdk.internal.vm.ci/jdk.vm.ci.runtime
  *          jdk.internal.vm.ci/jdk.vm.ci.meta
  * @run testng/othervm
  *      -XX:+UnlockExperimentalVMOptions -XX:+EnableJVMCI -XX:-UseJVMCICompiler
  *      jdk.vm.ci.hotspot.test.TestDynamicConstant
+ * @run testng/othervm
+ *      -XX:+UnlockExperimentalVMOptions -XX:+EnableJVMCI -XX:-UseJVMCICompiler -XX:+UnlockDiagnosticVMOptions -XX:UseBootstrapCallInfo=3
+ *      jdk.vm.ci.hotspot.test.TestDynamicConstant
  */
 
 package jdk.vm.ci.hotspot.test;
@@ -51,16 +54,17 @@ import java.util.Set;
 import org.testng.Assert;
 import org.testng.annotations.Test;
 
-import jdk.internal.org.objectweb.asm.ClassWriter;
-import jdk.internal.org.objectweb.asm.ConstantDynamic;
-import jdk.internal.org.objectweb.asm.Handle;
-import jdk.internal.org.objectweb.asm.MethodVisitor;
-import jdk.internal.org.objectweb.asm.Opcodes;
-import jdk.internal.org.objectweb.asm.Type;
+import org.objectweb.asm.ClassWriter;
+import org.objectweb.asm.ConstantDynamic;
+import org.objectweb.asm.Handle;
+import org.objectweb.asm.MethodVisitor;
+import org.objectweb.asm.Opcodes;
+import org.objectweb.asm.Type;
 import jdk.vm.ci.hotspot.HotSpotObjectConstant;
 import jdk.vm.ci.hotspot.HotSpotConstantPool;
 import jdk.vm.ci.meta.ConstantPool;
 import jdk.vm.ci.meta.ConstantPool.BootstrapMethodInvocation;
+import jdk.vm.ci.meta.JavaConstant;
 import jdk.vm.ci.meta.MetaAccessProvider;
 import jdk.vm.ci.meta.PrimitiveConstant;
 import jdk.vm.ci.meta.ResolvedJavaMethod;
@@ -86,6 +90,12 @@ public class TestDynamicConstant implements Opcodes {
          */
         CALL_DIRECT_BSM,
 
+        /**
+         * Condy whose bootstrap method is one of the {@code TestDynamicConstant.get<type>BSM(<type> constant, int i)}
+         * methods with one condy arg and one int arg.
+         */
+        CALL_DIRECT_WITH_ARGS_BSM,
+
         /**
          * Condy whose bootstrap method is {@link ConstantBootstraps#invoke} that invokes one of the
          * {@code TestDynamicConstant.get<type>()} methods.
@@ -164,6 +174,24 @@ public class TestDynamicConstant implements Opcodes {
                 run.visitInsn(type.getOpcode(IRETURN));
                 run.visitMaxs(0, 0);
                 run.visitEnd();
+             } else if (condyType == CondyType.CALL_DIRECT_WITH_ARGS_BSM) {
+                // Example: int TestDynamicConstant.getIntBSM(MethodHandles.Lookup l, String name,
+                // Class<?> type, int constant, int i)
+                String sig1 = "(Ljava/lang/invoke/MethodHandles$Lookup;Ljava/lang/String;Ljava/lang/Class;)" + desc;
+                String sig2 = "(Ljava/lang/invoke/MethodHandles$Lookup;Ljava/lang/String;Ljava/lang/Class;" + desc + "I)" + desc;
+
+                Handle handle1 = new Handle(H_INVOKESTATIC, testClassInternalName, getter + "BSM", sig1, false);
+                Handle handle2 = new Handle(H_INVOKESTATIC, testClassInternalName, getter + "BSM", sig2, false);
+
+                ConstantDynamic condy1 = new ConstantDynamic("const1", desc, handle1);
+                ConstantDynamic condy2 = new ConstantDynamic("const2", desc, handle2, condy1, Integer.MAX_VALUE);
+
+                condy = condy2;
+                MethodVisitor run = cw.visitMethod(PUBLIC_STATIC, "run", "()" + desc, null, null);
+                run.visitLdcInsn(condy);
+                run.visitInsn(type.getOpcode(IRETURN));
+                run.visitMaxs(0, 0);
+                run.visitEnd();
             } else if (condyType == CondyType.CALL_INDIRECT_BSM) {
                 // Example: int TestDynamicConstant.getInt()
                 Handle handle = new Handle(H_INVOKESTATIC, testClassInternalName, getter, "()" + desc, false);
@@ -309,6 +337,12 @@ public class TestDynamicConstant implements Opcodes {
                 assertNoEagerConstantResolution(testClass, cp, getTagAt);
                 assertLookupBMIDoesNotInvokeBM(metaAccess, testClass);
 
+                if (type != Object.class) {
+                    testLookupBootstrapMethodInvocation(condyType, metaAccess, testClass, getTagAt);
+                } else {
+                    // StringConcatFactoryStringConcatFactory cannot accept null constants
+                }
+
                 Object lastConstant = null;
                 for (int cpi = 1; cpi < cp.length(); cpi++) {
                     String tag = String.valueOf(getTagAt.invoke(cp, cpi));
@@ -330,12 +364,6 @@ public class TestDynamicConstant implements Opcodes {
                     actual = ((HotSpotObjectConstant) lastConstant).asObject(type);
                 }
                 Assert.assertEquals(actual, expect, m + ":");
-
-                if (type != Object.class) {
-                    testLookupBootstrapMethodInvocation(condyType, metaAccess, testClass, getTagAt);
-                } else {
-                    // StringConcatFactoryStringConcatFactory cannot accept null constants
-                }
             }
         }
     }
@@ -364,10 +392,29 @@ public class TestDynamicConstant implements Opcodes {
                     Assert.assertTrue(expectedBSMs.contains(bsm), expectedBSMs.toString());
                 } else {
                     Assert.assertFalse(bsmi.isInvokeDynamic());
-                    if (condyType == CondyType.CALL_DIRECT_BSM) {
-                        Assert.assertTrue(bsm.startsWith("jdk.vm.ci.hotspot.test.TestDynamicConstant.get") && bsm.endsWith("BSM"), bsm);
-                    } else {
-                        Assert.assertEquals(bsm, "java.lang.invoke.ConstantBootstraps.invoke");
+                    checkBsmName(condyType, bsm);
+                    List<JavaConstant> staticArguments = bsmi.getStaticArguments();
+                    for (int i = 0; i < staticArguments.size(); ++i) {
+                        JavaConstant constant = staticArguments.get(i);
+                        if (constant instanceof PrimitiveConstant) {
+                            String innerTag = String.valueOf(getTagAt.invoke(cp, constant.asInt()));
+                            if (condyType == CondyType.CALL_DIRECT_WITH_ARGS_BSM) {
+                                Assert.assertEquals(i, 0);
+                                Assert.assertEquals(innerTag, "Dynamic");
+                            }
+                            if (innerTag.equals("Dynamic")) {
+                                BootstrapMethodInvocation innerBsmi = cp.lookupBootstrapMethodInvocation(constant.asInt(), -1);
+                                String innerBsm = innerBsmi.getMethod().format("%H.%n");
+                                checkBsmName(condyType, innerBsm);
+                            } else {
+                                Assert.assertEquals(innerTag, "MethodHandle");
+                            }
+                        } else {
+                            if (condyType == CondyType.CALL_DIRECT_WITH_ARGS_BSM) {
+                                Assert.assertEquals(i, 1);
+                            }
+                            Assert.assertTrue(staticArguments.get(i) instanceof HotSpotObjectConstant);
+                        }
                     }
                 }
             } else {
@@ -378,6 +425,14 @@ public class TestDynamicConstant implements Opcodes {
         testLoadReferencedType(concat, cp);
     }
 
+    private static void checkBsmName(CondyType condyType, String bsm) {
+        if (condyType == CondyType.CALL_DIRECT_BSM || condyType == CondyType.CALL_DIRECT_WITH_ARGS_BSM) {
+            Assert.assertTrue(bsm.startsWith("jdk.vm.ci.hotspot.test.TestDynamicConstant.get") && bsm.endsWith("BSM"), bsm);
+        } else {
+            Assert.assertEquals(bsm, "java.lang.invoke.ConstantBootstraps.invoke");
+        }
+    }
+
     private static int beS4(byte[] data, int bci) {
         return (data[bci] << 24) | ((data[bci + 1] & 0xff) << 16) | ((data[bci + 2] & 0xff) << 8) | (data[bci + 3] & 0xff);
     }
@@ -425,6 +480,18 @@ public class TestDynamicConstant implements Opcodes {
     @SuppressWarnings("unused") public static Object  getObjectBSM (MethodHandles.Lookup l, String name, Class<?> type) { return null; }
     @SuppressWarnings("unused") public static List<?> getListBSM   (MethodHandles.Lookup l, String name, Class<?> type) { return List.of("element"); }
 
+    @SuppressWarnings("unused") public static boolean getBooleanBSM(MethodHandles.Lookup l, String name, Class<?> type, boolean constant, int i) { return true; }
+    @SuppressWarnings("unused") public static char    getCharBSM   (MethodHandles.Lookup l, String name, Class<?> type, char constant, int i) { return '*'; }
+    @SuppressWarnings("unused") public static short   getShortBSM  (MethodHandles.Lookup l, String name, Class<?> type, short constant, int i) { return Short.MAX_VALUE; }
+    @SuppressWarnings("unused") public static byte    getByteBSM   (MethodHandles.Lookup l, String name, Class<?> type, byte constant, int i) { return Byte.MAX_VALUE; }
+    @SuppressWarnings("unused") public static int     getIntBSM    (MethodHandles.Lookup l, String name, Class<?> type, int constant, int i) { return Integer.MAX_VALUE; }
+    @SuppressWarnings("unused") public static float   getFloatBSM  (MethodHandles.Lookup l, String name, Class<?> type, float constant, int i) { return Float.MAX_VALUE; }
+    @SuppressWarnings("unused") public static long    getLongBSM   (MethodHandles.Lookup l, String name, Class<?> type, long constant, int i) { return Long.MAX_VALUE; }
+    @SuppressWarnings("unused") public static double  getDoubleBSM (MethodHandles.Lookup l, String name, Class<?> type, double constant, int i) { return Double.MAX_VALUE; }
+    @SuppressWarnings("unused") public static String  getStringBSM (MethodHandles.Lookup l, String name, Class<?> type, String constant, int i) { return "a string"; }
+    @SuppressWarnings("unused") public static Object  getObjectBSM (MethodHandles.Lookup l, String name, Class<?> type, Object constant, int i) { return null; }
+    @SuppressWarnings("unused") public static List<?> getListBSM   (MethodHandles.Lookup l, String name, Class<?> type, List<?> constant, int i) { return List.of("element"); }
+
 
     public static boolean getBoolean() { return true; }
     public static char    getChar   () { return '*'; }
