@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -65,11 +65,12 @@ public class TestVMProcess {
     private String irEncoding;
 
     public TestVMProcess(List<String> additionalFlags, Class<?> testClass, Set<Class<?>> helperClasses, int defaultWarmup,
-                         boolean testClassesOnBootClassPath) {
+                         boolean allowNotCompilable, boolean testClassesOnBootClassPath) {
         this.cmds = new ArrayList<>();
         TestFrameworkSocket socket = new TestFrameworkSocket();
         try (socket) {
-            prepareTestVMFlags(additionalFlags, socket, testClass, helperClasses, defaultWarmup, testClassesOnBootClassPath);
+            prepareTestVMFlags(additionalFlags, socket, testClass, helperClasses, defaultWarmup,
+                               allowNotCompilable, testClassesOnBootClassPath);
             start();
         }
         processSocketOutput(socket);
@@ -93,7 +94,8 @@ public class TestVMProcess {
     }
 
     private void prepareTestVMFlags(List<String> additionalFlags, TestFrameworkSocket socket, Class<?> testClass,
-                                    Set<Class<?>> helperClasses, int defaultWarmup, boolean testClassesOnBootClassPath) {
+                                    Set<Class<?>> helperClasses, int defaultWarmup, boolean allowNotCompilable,
+                                    boolean testClassesOnBootClassPath) {
         // Set java.library.path so JNI tests which rely on jtreg nativepath setting work
         cmds.add("-Djava.library.path=" + Utils.TEST_NATIVE_PATH);
         // Need White Box access in test VM.
@@ -128,6 +130,10 @@ public class TestVMProcess {
             cmds.add("-DWarmup=" + defaultWarmup);
         }
 
+        if (allowNotCompilable) {
+            cmds.add("-DAllowNotCompilable=true");
+        }
+
         cmds.add(TestVM.class.getName());
         cmds.add(testClass.getName());
         if (helperClasses != null) {
@@ -253,9 +259,7 @@ public class TestVMProcess {
         int exitCode = oa.getExitValue();
         String stdErr = oa.getStderr();
         String stdOut = "";
-        boolean osIsWindows = Platform.isWindows();
-        boolean JVMHadError = (!osIsWindows && exitCode == 134) || (osIsWindows && exitCode == -1);
-        if (JVMHadError) {
+        if (exitCode == 134) {
             // Also dump the stdout if we experience a JVM error (e.g. to show hit assertions etc.).
             stdOut = System.lineSeparator() + System.lineSeparator() + "Standard Output" + System.lineSeparator()
                      + "---------------" + System.lineSeparator() + oa.getOutput();
