@@ -42,6 +42,7 @@
 package compiler.vectorization.runner;
 
 import compiler.lib.ir_framework.*;
+import java.util.Random;
 
 public class BasicFloatOpTest extends VectorizationTestRunner {
 
@@ -50,11 +51,72 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
     private float[] a;
     private float[] b;
     private float[] c;
+    private float[] d;
+    private float[] e;
 
     public BasicFloatOpTest() {
+        // Positive test values                       sign |   exponent | mantisa
+        float smallPositive   = Float.intBitsToFloat(0<<31 | 0x3f << 23 | 0x30000f);
+        float positive        = Float.intBitsToFloat(0<<31 | 0x7f << 23 | 0x30000f);
+        float bigPositive     = Float.intBitsToFloat(0<<31 | 0x7f << 23 | 0x30100f);
+        float biggerPositive  = Float.intBitsToFloat(0<<31 | 0xfe << 23 | 0x30000f);
+        float maxPositive     = Float.MAX_VALUE;
+
+        // Special positive
+        float nan1  = Float.intBitsToFloat(0<<31 | 0xff << 23 | 0x7fffff);
+        float nan2  = Float.intBitsToFloat(0<<31 | 0xff << 23 | 0x30000f);
+        float inf   = Float.intBitsToFloat(0<<31 | 0xff << 23);
+        float zero  = 0.0f;
+
+        // Negative test values                       sign |   exponent | mantisa
+        float smallNegative   = Float.intBitsToFloat(1<<31 | 0x03 << 23 | 0x30000f);
+        float negative        = Float.intBitsToFloat(1<<31 | 0x83 << 23 | 0x30100f);
+        float bigNegative     = Float.intBitsToFloat(1<<31 | 0x83 << 23 | 0x30000f);
+        float biggerNegative  = Float.intBitsToFloat(1<<31 | 0x86 << 23 | 0x30000f);
+        float maxNegative     = Float.intBitsToFloat(1<<31 | 0xfe << 23 | 0x7fffff);
+
+        // Special negative
+        float nNan1  = Float.intBitsToFloat(1<<31 | 0xff << 23 | 0x7fffff);
+        float nNan2  = Float.intBitsToFloat(1<<31 | 0xff << 23 | 0x30000f);
+        float nInf   = Float.intBitsToFloat(1<<31 | 0xff << 23);
+        float nZero  = -0.0f;
+
+        float[] orderedList = new float[] {
+            nInf, maxNegative, biggerNegative, bigNegative, negative, smallNegative, nZero,
+            zero, smallPositive, positive, bigPositive, biggerPositive, maxPositive, inf
+        };
+
+        float[] NaNs = new float[] {
+            nan1, nan2, nNan1, nNan2
+        };
+
+        float[] numberList = new float[] {
+            nInf, maxNegative, biggerNegative, bigNegative, negative, smallNegative, nZero,
+            zero, smallPositive, positive, bigPositive, biggerPositive, maxPositive, inf,
+            nan1, nan2, nNan1, nNan2
+        };
+
+        Random rnd = new Random(11);
         a = new float[SIZE];
         b = new float[SIZE];
         c = new float[SIZE];
+        d = new float[SIZE];
+        e = new float[SIZE];
+
+        for (int i = 0; i < SIZE;) {
+            for (int j = 0; j < numberList.length && i < SIZE; j++, i++) {
+                for (int k = j; k < numberList.length && i < SIZE; k++, i++) {
+                    if (rnd.nextBoolean()) {
+                        d[i] = numberList[j];
+                        e[i] = numberList[k];
+                    } else {
+                        d[i] = numberList[k];
+                        e[i] = numberList[j];
+                    }
+                }
+            }
+        }
+
         for (int i = 0; i < SIZE; i++) {
             a[i] = 850.0f * i + 22222.22f;
             b[i] = -12345.678f;
@@ -64,7 +126,7 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
 
     // ---------------- Arithmetic ----------------
     @Test
-    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse", "true"},
+    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse", "true", "rvv", "true"},
         counts = {IRNode.NEG_VF, ">0"})
     public float[] vectorNeg() {
         float[] res = new float[SIZE];
@@ -75,7 +137,7 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
     }
 
     @Test
-    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse", "true"},
+    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse", "true", "rvv", "true"},
         counts = {IRNode.ABS_VF, ">0"})
     public float[] vectorAbs() {
         float[] res = new float[SIZE];
@@ -86,7 +148,7 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
     }
 
     @Test
-    @IR(applyIfCPUFeatureOr = {"asimd", "true", "avx", "true"},
+    @IR(applyIfCPUFeatureOr = {"asimd", "true", "avx", "true", "rvv", "true"},
         counts = {IRNode.SQRT_VF, ">0"})
     public float[] vectorSqrt() {
         float[] res = new float[SIZE];
@@ -97,7 +159,7 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
     }
 
     @Test
-    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse2", "true"},
+    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse2", "true", "rvv", "true"},
         counts = {IRNode.ADD_VF, ">0"})
     public float[] vectorAdd() {
         float[] res = new float[SIZE];
@@ -108,7 +170,7 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
     }
 
     @Test
-    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse2", "true"},
+    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse2", "true", "rvv", "true"},
         counts = {IRNode.SUB_VF, ">0"})
     public float[] vectorSub() {
         float[] res = new float[SIZE];
@@ -119,7 +181,7 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
     }
 
     @Test
-    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse2", "true"},
+    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse2", "true", "rvv", "true"},
         counts = {IRNode.MUL_VF, ">0"})
     public float[] vectorMul() {
         float[] res = new float[SIZE];
@@ -130,7 +192,7 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
     }
 
     @Test
-    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse2", "true"},
+    @IR(applyIfCPUFeatureOr = {"asimd", "true", "sse2", "true", "rvv", "true"},
         counts = {IRNode.DIV_VF, ">0"})
     public float[] vectorDiv() {
         float[] res = new float[SIZE];
@@ -141,23 +203,23 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
     }
 
     @Test
-    @IR(applyIfCPUFeatureOr = {"asimd", "true", "avx", "true"},
+    @IR(applyIfCPUFeatureOr = {"asimd", "true", "avx", "true", "rvv", "true"},
         counts = {IRNode.MAX_VF, ">0"})
     public float[] vectorMax() {
         float[] res = new float[SIZE];
         for (int i = 0; i < SIZE; i++) {
-            res[i] = Math.max(a[i], b[i]);
+            res[i] = Math.max(d[i], e[i]);
         }
         return res;
     }
 
     @Test
-    @IR(applyIfCPUFeatureOr = {"asimd", "true", "avx", "true"},
+    @IR(applyIfCPUFeatureOr = {"asimd", "true", "avx", "true", "rvv", "true"},
         counts = {IRNode.MIN_VF, ">0"})
     public float[] vectorMin() {
         float[] res = new float[SIZE];
         for (int i = 0; i < SIZE; i++) {
-            res[i] = Math.min(a[i], b[i]);
+            res[i] = Math.min(d[i], e[i]);
         }
         return res;
     }
@@ -167,6 +229,8 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
         counts = {IRNode.FMA_VF, ">0", IRNode.VFMLA, ">0"})
     @IR(applyIfCPUFeatureAnd = {"fma", "true", "avx", "true"},
         counts = {IRNode.FMA_VF, ">0"})
+    @IR(applyIfCPUFeature = {"rvv", "true"},
+        counts = {IRNode.FMA_VF, ">0"})
     public float[] vectorMulAdd() {
         float[] res = new float[SIZE];
         for (int i = 0; i < SIZE; i++) {
@@ -180,6 +244,8 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
         counts = {IRNode.FMA_VF, ">0", IRNode.VFMLS, ">0"})
     @IR(applyIfCPUFeatureAnd = {"fma", "true", "avx", "true"},
         counts = {IRNode.FMA_VF, ">0"})
+    @IR(applyIfCPUFeature = {"rvv", "true"},
+        counts = {IRNode.FMA_VF, ">0"})
     public float[] vectorMulSub1() {
         float[] res = new float[SIZE];
         for (int i = 0; i < SIZE; i++) {
@@ -193,6 +259,8 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
         counts = {IRNode.FMA_VF, ">0", IRNode.VFMLS, ">0"})
     @IR(applyIfCPUFeatureAnd = {"fma", "true", "avx", "true"},
         counts = {IRNode.FMA_VF, ">0"})
+    @IR(applyIfCPUFeature = {"rvv", "true"},
+        counts = {IRNode.FMA_VF, ">0"})
     public float[] vectorMulSub2() {
         float[] res = new float[SIZE];
         for (int i = 0; i < SIZE; i++) {
@@ -208,6 +276,8 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
         counts = {IRNode.VFNMLA, ">0"})
     @IR(applyIfCPUFeatureAnd = {"fma", "true", "avx", "true"},
         counts = {IRNode.FMA_VF, ">0"})
+    @IR(applyIfCPUFeature = {"rvv", "true"},
+        counts = {IRNode.FMA_VF, ">0"})
     public float[] vectorNegateMulAdd1() {
         float[] res = new float[SIZE];
         for (int i = 0; i < SIZE; i++) {
@@ -223,6 +293,8 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
         counts = {IRNode.VFNMLA, ">0"})
     @IR(applyIfCPUFeatureAnd = {"fma", "true", "avx", "true"},
         counts = {IRNode.FMA_VF, ">0"})
+    @IR(applyIfCPUFeature = {"rvv", "true"},
+        counts = {IRNode.FMA_VF, ">0"})
     public float[] vectorNegateMulAdd2() {
         float[] res = new float[SIZE];
         for (int i = 0; i < SIZE; i++) {
@@ -236,6 +308,8 @@ public class BasicFloatOpTest extends VectorizationTestRunner {
         counts = {IRNode.FMA_VF, ">0"})
     @IR(applyIfCPUFeatureAnd = {"fma", "true", "avx", "true"},
         counts = {IRNode.FMA_VF, ">0"})
+    @IR(applyIfCPUFeature = {"rvv", "true"},
+        counts = {IRNode.FMA_VF, ">0"})
     public float[] vectorNegateMulSub() {
         float[] res = new float[SIZE];
         for (int i = 0; i < SIZE; i++) {
