@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,7 @@ package gc.z;
 
 /*
  * @test TestPageCacheFlush
- * @requires vm.gc.ZGenerational
+ * @requires vm.gc.Z
  * @summary Test ZGC page cache flushing
  * @library /test/lib
  * @run driver gc.z.TestPageCacheFlush
@@ -68,16 +68,15 @@ public class TestPageCacheFlush {
     }
 
     public static void main(String[] args) throws Exception {
-        ProcessTools.executeProcess(ProcessTools.createJavaProcessBuilder(
-                                    "-XX:+UseZGC",
-                                    "-XX:+ZGenerational",
-                                    "-Xms128M",
-                                    "-Xmx128M",
-                                    "-Xlog:gc,gc+init,gc+heap=debug",
-                                    Test.class.getName()))
-                    .outputTo(System.out)
-                    .errorTo(System.out)
-                    .shouldContain("Page Cache Flushed:")
-                    .shouldHaveExitValue(0);
+        ProcessTools.executeTestJava(
+            "-XX:+UseZGC",
+            "-Xms128M",
+            "-Xmx128M",
+            "-Xlog:gc,gc+init,gc+heap=debug",
+            Test.class.getName())
+                .outputTo(System.out)
+                .errorTo(System.out)
+                .shouldContain("Page Cache Flushed:")
+                .shouldHaveExitValue(0);
     }
 }
