@@ -41,7 +41,7 @@ public class ObsoleteFlagErrorMessage {
     String flag = "DummyObsoleteTestFlag";
 
     // Case 1: Newly obsolete flags with extra junk appended should not be treated as newly obsolete (8060449)
-    ProcessBuilder pb = ProcessTools.createJavaProcessBuilder(
+    ProcessBuilder pb = ProcessTools.createLimitedTestJavaProcessBuilder(
         "-XX:" + flag + "PlusJunk", "-version");
 
     OutputAnalyzer output = new OutputAnalyzer(pb.start());
@@ -49,7 +49,7 @@ public class ObsoleteFlagErrorMessage {
     output.shouldHaveExitValue(1);
 
     // Case 2: Newly obsolete flags should be recognized as newly obsolete (8073989)
-    ProcessBuilder pb2 = ProcessTools.createJavaProcessBuilder(
+    ProcessBuilder pb2 = ProcessTools.createLimitedTestJavaProcessBuilder(
         "-XX:+" + flag, "-version");
 
     OutputAnalyzer output2 = new OutputAnalyzer(pb2.start());
