@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -111,7 +111,7 @@ public class WrongTopClasspath extends DynamicArchiveTestBase {
                 "assertNotShared:GenericTestApp") // but top archive is not useable
           .assertNormalExit(output -> {
               output.shouldContain(topArchiveMsg);
-              output.shouldMatch("A jar file is not the one used while building the shared archive file:.*GenericTestApp.jar");
-              output.shouldMatch(".warning..cds.*GenericTestApp.jar timestamp has changed.");});
+              output.shouldMatch("This file is not the one used while building the shared archive file:.*GenericTestApp.jar");
+              output.shouldMatch(".warning..cds.*GenericTestApp.jar.*timestamp has changed");});
     }
 }
