@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -265,7 +265,7 @@ public class redefineclasses027 {
     }
 
     void displayVMStatus() {
-        ThreadReference thrd = debugee.mainThread();
+        ThreadReference thrd = debugee.threadByName("main");
         if (thrd.isSuspended()) {
             statDebugee = "Debugee is suspended";
             display("\n\n<<<" + statDebugee + ">>>");
