@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -247,7 +247,15 @@ public class instancefilter005 extends JDIBase {
 
         log2("      received: ClassPrepareEvent for debuggeeClass");
 
-        setupBreakpointForCommunication(debuggeeClass);
+        String bPointMethod = "methodForCommunication";
+        String lineForComm  = "lineForComm";
+
+        ThreadReference   mainThread = debuggee.threadByNameOrThrow("main");
+
+        BreakpointRequest bpRequest = settingBreakpoint(mainThread,
+                                             debuggeeClass,
+                                            bPointMethod, lineForComm, "zero");
+        bpRequest.enable();
 
     //------------------------------------------------------  testing section
 
