@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -49,7 +49,6 @@ import java.util.concurrent.Executor;
 import java.util.concurrent.Flow;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.stream.IntStream;
-import java.time.Instant;
 import java.time.temporal.ChronoUnit;
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLParameters;
@@ -57,6 +56,9 @@ import java.net.http.HttpClient;
 import java.net.http.HttpRequest;
 import java.net.http.HttpResponse;
 import jdk.internal.net.http.common.FlowTube;
+import jdk.internal.net.http.common.TimeSource;
+import jdk.internal.net.http.common.TimeLine;
+import jdk.internal.net.http.common.Deadline;
 
 /**
  * @summary Verifies that the ConnectionPool correctly handle
@@ -94,14 +96,15 @@ public class ConnectionPoolTest {
     }
 
     public static void testCacheCleaners() throws Exception {
-        ConnectionPool pool = new ConnectionPool(666);
+        final TimeLine timeLine = TimeSource.source();
+        ConnectionPool pool = new ConnectionPool(666, timeLine);
         HttpClient client = new HttpClientStub(pool);
         InetSocketAddress proxy = InetSocketAddress.createUnresolved("bar", 80);
         System.out.println("Adding 20 connections to pool");
         Random random = new Random();
 
         final int count = 20;
-        Instant now = Instant.now().truncatedTo(ChronoUnit.SECONDS);
+        Deadline now = timeLine.instant().truncatedTo(ChronoUnit.SECONDS);
         int[] keepAlives = new int[count];
         HttpConnectionStub[] connections = new HttpConnectionStub[count];
         long purge = pool.purgeExpiredConnectionsAndReturnNextDeadline(now);
@@ -143,7 +146,7 @@ public class ConnectionPoolTest {
         }
         purge = mean * 1000;
         System.out.println("start purging at " + purge + " ms");
-        Instant next = now;
+        Deadline next = now;
         do {
            System.out.println("next purge is in " + purge + " ms");
            next = next.plus(purge, ChronoUnit.MILLIS);
@@ -180,7 +183,7 @@ public class ConnectionPoolTest {
         Random random = new Random();
 
         final int count = 20;
-        Instant now = Instant.now().truncatedTo(ChronoUnit.SECONDS);
+        Deadline now = TimeSource.now().truncatedTo(ChronoUnit.SECONDS);
         int[] keepAlives = new int[count];
         HttpConnectionStub[] connections = new HttpConnectionStub[count];
         long purge = pool.purgeExpiredConnectionsAndReturnNextDeadline(now);
