@@ -31,17 +31,13 @@
  * @library /test/lib
  * @build jdk.test.lib.NetworkConfiguration
  *        jdk.test.lib.Platform
- *        jtreg.SkippedException
  * @run main TcpTest -d
- * @run main/othervm -Djdk.net.usePlainSocketImpl TcpTest -d
  */
 
 import java.net.*;
 import java.io.*;
 import java.util.concurrent.TimeUnit;
 
-import jtreg.SkippedException;
-
 public class TcpTest extends Tests {
     static ServerSocket server, server1, server2;
     static Socket c1, c2, c3, s1, s2, s3;
@@ -66,10 +62,12 @@ public class TcpTest extends Tests {
     public static void main (String[] args) throws Exception {
         checkDebug(args);
         if (ia4addr == null) {
-            throw new SkippedException("No IPV4 addresses: exiting test");
+            System.out.println ("No IPV4 addresses: exiting test");
+            return;
         }
         if (ia6addr == null) {
-            throw new SkippedException("No IPV6 addresses: exiting test");
+            System.out.println ("No IPV6 addresses: exiting test");
+            return;
         }
         dprintln ("Local Addresses");
         dprintln (ia4addr.toString());
