@@ -42,6 +42,10 @@ ternary_double_broadcast_masked="Ternary-Double-Broadcast-Masked-op"
 ternary_scalar="Ternary-Scalar-op"
 binary="Binary-op"
 binary_masked="Binary-Masked-op"
+binary_memop="Binary-mem-op"
+binary_masked_memop="Binary-Masked-mem-op"
+saturating_binary="SaturatingBinary-op"
+saturating_binary_masked="SaturatingBinary-Masked-op"
 binary_broadcast="Binary-Broadcast-op"
 binary_broadcast_masked="Binary-Broadcast-Masked-op"
 binary_broadcast_long="Binary-Broadcast-Long-op"
@@ -253,6 +257,12 @@ function gen_binary_alu_op {
   gen_op_tmpl $binary_masked "$@"
 }
 
+function gen_binary_alu_mem_op {
+  echo "Generating binary op $1 ($2)..."
+  gen_op_tmpl $binary_memop "$@"
+  gen_op_tmpl $binary_masked_memop "$@"
+}
+
 function gen_binary_alu_bcst_op {
   echo "Generating binary broadcast op $1 ($2)..."
   gen_op_tmpl $binary_broadcast "$@"
@@ -310,6 +320,12 @@ function gen_binary_op {
   gen_op_tmpl $binary_masked "$@"
 }
 
+function gen_saturating_binary_op {
+  echo "Generating binary op $1 ($2)..."
+  gen_op_tmpl $saturating_binary "$@"
+  gen_op_tmpl $saturating_binary_masked "$@"
+}
+
 function gen_binary_op_no_masked {
   echo "Generating binary op $1 ($2)..."
 #  gen_op_tmpl $binary_scalar "$@"
@@ -456,9 +472,19 @@ gen_shift_cst_op  "ASHR" "(a >> CONST_SHIFT)" "BITWISE"
 gen_shift_cst_op  "ROR" "ROR_scalar(a, CONST_SHIFT)" "BITWISE"
 gen_shift_cst_op  "ROL" "ROL_scalar(a, CONST_SHIFT)" "BITWISE"
 
+# Binary operation with one memory operand
+gen_binary_alu_mem_op "MIN+min+withMask", "Math.min(a, b)"
+gen_binary_alu_mem_op "MAX+max+withMask", "Math.max(a, b)"
+
 # Masked reductions.
 gen_binary_op_no_masked "MIN+min" "Math.min(a, b)"
 gen_binary_op_no_masked "MAX+max" "Math.max(a, b)"
+gen_binary_op "UMIN" "VectorMath.minUnsigned(a, b)" "BITWISE"
+gen_binary_op "UMAX" "VectorMath.maxUnsigned(a, b)" "BITWISE"
+gen_saturating_binary_op "SADD" "VectorMath.addSaturating(a, b)" "BITWISE"
+gen_saturating_binary_op "SSUB" "VectorMath.subSaturating(a, b)" "BITWISE"
+gen_saturating_binary_op "SUADD" "VectorMath.addSaturatingUnsigned(a, b)" "BITWISE"
+gen_saturating_binary_op "SUSUB" "VectorMath.subSaturatingUnsigned(a, b)" "BITWISE"
 gen_binary_bcst_op_no_masked "MIN+min" "Math.min(a, b)"
 gen_binary_bcst_op_no_masked "MAX+max" "Math.max(a, b)"
 
@@ -470,6 +496,8 @@ gen_reduction_op "ADD" "+" "" "0"
 gen_reduction_op "MUL" "*" "" "1"
 gen_reduction_op_func "MIN" "(\$type\$) Math.min" "" "\$Wideboxtype\$.\$MaxValue\$"
 gen_reduction_op_func "MAX" "(\$type\$) Math.max" "" "\$Wideboxtype\$.\$MinValue\$"
+gen_reduction_op_func "UMIN" "(\$type\$) VectorMath.minUnsigned" "BITWISE" "\$Wideboxtype\$.\$MaxValue\$"
+gen_reduction_op_func "UMAX" "(\$type\$) VectorMath.maxUnsigned" "BITWISE" "\$Wideboxtype\$.\$MinValue\$"
 gen_reduction_op_func "FIRST_NONZERO" "firstNonZero" "" "(\$type\$) 0"
 
 # Boolean reductions.
@@ -494,10 +522,10 @@ gen_compare_op "NE" "neq"
 gen_compare_op "LE" "le"
 gen_compare_op "GE" "ge"
 
-gen_compare_op "UNSIGNED_LT" "ult" "BITWISE"
-gen_compare_op "UNSIGNED_GT" "ugt" "BITWISE"
-gen_compare_op "UNSIGNED_LE" "ule" "BITWISE"
-gen_compare_op "UNSIGNED_GE" "uge" "BITWISE"
+gen_compare_op "ULT" "ult" "BITWISE"
+gen_compare_op "UGT" "ugt" "BITWISE"
+gen_compare_op "ULE" "ule" "BITWISE"
+gen_compare_op "UGE" "uge" "BITWISE"
 
 
 gen_compare_bcst_op "LT" "<"
