@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -39,7 +39,7 @@ import jdk.test.lib.jfr.EventNames;
 /**
  * @test
  * @summary Set destination to an existing file. File should be overwritten.
- * @key jfr
+ * @requires vm.flagless
  * @requires vm.hasJFR
  * @library /test/lib
  * @run main/othervm jdk.jfr.api.recording.destination.TestDestFileExist
@@ -59,7 +59,7 @@ public class TestDestFileExist {
         r.stop();
         List<RecordedEvent> events = RecordingFile.readAllEvents(dest);
         Asserts.assertFalse(events.isEmpty(), "No events found");
-        System.out.println(events.iterator().next());
+        System.out.println(events.getFirst());
         r.close();
     }
 
