@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,19 +32,11 @@ import jdk.test.lib.jfr.EventNames;
 import jdk.test.lib.jfr.Events;
 
 /**
- * @test id=ZSinglegen
- * @requires vm.hasJFR & vm.gc.ZSinglegen
+ * @test
+ * @requires vm.hasJFR & vm.gc.Z
  * @key jfr
  * @library /test/lib /test/jdk /test/hotspot/jtreg
- * @run main/othervm -XX:+UseZGC -XX:-ZGenerational -Xmx32M -Xlog:gc*:gc.log::filecount=0 jdk.jfr.event.gc.detailed.TestZAllocationStallEvent
- */
-
-/**
- * @test id=ZGenerational
- * @requires vm.hasJFR & vm.gc.ZGenerational
- * @key jfr
- * @library /test/lib /test/jdk /test/hotspot/jtreg
- * @run main/othervm -XX:+UseZGC -XX:+ZGenerational -Xmx32M -Xlog:gc*:gc.log::filecount=0 jdk.jfr.event.gc.detailed.TestZAllocationStallEvent
+ * @run main/othervm -XX:+UseZGC -Xmx32M -Xlog:gc*:gc.log::filecount=0 jdk.jfr.event.gc.detailed.TestZAllocationStallEvent
  */
 
 public class TestZAllocationStallEvent {
@@ -56,7 +48,7 @@ public class TestZAllocationStallEvent {
 
             // Allocate many large objects quickly, to outrun the GC
             for (int i = 0; i < 100; i++) {
-                blackHole(new byte[16 * 1024 * 1024]);
+                blackHole(new byte[4 * 1024 * 1024]);
             }
 
             recording.stop();
@@ -65,7 +57,7 @@ public class TestZAllocationStallEvent {
             List<RecordedEvent> events = Events.fromRecording(recording);
             System.out.println("Events: " + events.size());
             Events.hasEvents(events);
-            for (RecordedEvent event : Events.fromRecording(recording)) {
+            for (RecordedEvent event : events) {
                 Events.assertField(event, "size").atLeast(2L * 1024 * 1024);
             }
         }
