@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2022, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,7 +28,7 @@ import jdk.jfr.Registered;
 @Registered(true)
 public class MyCommitRegisteredTrueEvent extends E implements Runnable {
     public void myCommit() {
-        PlaceholderEventWriterFactory.getEventWriter(4711L);
+        PlaceholderEventWriter.getEventWriter();
         throw new RuntimeException("Should not reach here");
     }
 
