@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  */
 package jdk.jpackage.test;
 
-import java.io.File;
 import java.io.IOException;
 import java.nio.file.Files;
 import java.nio.file.Path;
@@ -41,9 +40,9 @@ import java.util.regex.Pattern;
 import java.util.stream.Collectors;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipFile;
-import jdk.jpackage.test.Functional.ThrowingConsumer;
-import jdk.jpackage.test.Functional.ThrowingFunction;
-import jdk.jpackage.test.Functional.ThrowingSupplier;
+import jdk.jpackage.internal.util.function.ThrowingConsumer;
+import jdk.jpackage.internal.util.function.ThrowingFunction;
+import jdk.jpackage.internal.util.function.ThrowingSupplier;
 
 public final class HelloApp {
 
@@ -57,15 +56,10 @@ public final class HelloApp {
 
     private JarBuilder prepareSources(Path srcDir) throws IOException {
         final String srcClassName = appDesc.srcClassName();
-
-        final String qualifiedClassName = appDesc.className();
-
-        final String className = qualifiedClassName.substring(
-                qualifiedClassName.lastIndexOf('.') + 1);
+        final String className = appDesc.shortClassName();
         final String packageName = appDesc.packageName();
 
-        final Path srcFile = srcDir.resolve(Path.of(String.join(
-                File.separator, qualifiedClassName.split("\\.")) + ".java"));
+        final Path srcFile = srcDir.resolve(appDesc.classNameAsPath(".java"));
         Files.createDirectories(srcFile.getParent());
 
         JarBuilder jarBuilder = createJarBuilder().addSourceFile(srcFile);
@@ -317,20 +311,23 @@ public final class HelloApp {
 
     public static void executeLauncherAndVerifyOutput(JPackageCommand cmd,
             String... args) {
-        AppOutputVerifier av = getVerifier(cmd, args);
+        AppOutputVerifier av = assertMainLauncher(cmd, args);
         if (av != null) {
-            // when running app launchers, clear users environment
-            av.executeAndVerifyOutput(true, args);
+            av.executeAndVerifyOutput(args);
         }
     }
 
     public static Executor.Result executeLauncher(JPackageCommand cmd,
             String... args) {
-        AppOutputVerifier av = getVerifier(cmd, args);
-        return av.executeOnly(true, args);
+        AppOutputVerifier av = assertMainLauncher(cmd, args);
+        if (av != null) {
+            return av.saveOutput(true).execute(args);
+        } else {
+            return null;
+        }
     }
 
-    private static AppOutputVerifier getVerifier(JPackageCommand cmd,
+    public static AppOutputVerifier assertMainLauncher(JPackageCommand cmd,
             String... args) {
         final Path launcherPath = cmd.appLauncherPath();
         if (!cmd.canRunLauncher(String.format("Not running [%s] launcher",
@@ -348,7 +345,7 @@ public final class HelloApp {
     }
 
 
-    public final static class AppOutputVerifier {
+    public static final class AppOutputVerifier {
         AppOutputVerifier(Path helloAppLauncher) {
             this.launcherPath = helloAppLauncher;
             this.outputFilePath = TKit.workDir().resolve(OUTPUT_FILENAME);
@@ -356,6 +353,16 @@ public final class HelloApp {
             this.defaultLauncherArgs = new ArrayList<>();
         }
 
+        public AppOutputVerifier saveOutput(boolean v) {
+            saveOutput = v;
+            return this;
+        }
+
+        public AppOutputVerifier expectedExitCode(int v) {
+            expectedExitCode = v;
+            return this;
+        }
+
         public AppOutputVerifier addDefaultArguments(String... v) {
             return addDefaultArguments(List.of(v));
         }
@@ -372,6 +379,8 @@ public final class HelloApp {
                 outputFilePath = Path.of(value);
             } else if ("jpackage.test.exitCode".equals(name)) {
                 expectedExitCode = Integer.parseInt(value);
+            } else if ("jpackage.test.noexit".equals(name)) {
+                launcherNoExit = Boolean.parseBoolean(value);
             }
             return this;
         }
@@ -415,36 +424,16 @@ public final class HelloApp {
         }
 
         public void executeAndVerifyOutput(String... args) {
-            executeAndVerifyOutput(false, args);
+            execute(args);
+            verifyOutput(args);
         }
 
-        public void executeAndVerifyOutput(boolean removePath,
-                List<String> launcherArgs, List<String> appArgs) {
-            final int attempts = 3;
-            final int waitBetweenAttemptsSeconds = 5;
-            getExecutor(launcherArgs.toArray(new String[0])).dumpOutput().setRemovePath(
-                    removePath).executeAndRepeatUntilExitCode(expectedExitCode,
-                            attempts, waitBetweenAttemptsSeconds);
-            verifyOutputFile(outputFilePath, appArgs, params);
-        }
-
-        public void executeAndVerifyOutput(boolean removePath, String... args) {
-            final List<String> launcherArgs = List.of(args);
-            final List<String> appArgs;
-            if (launcherArgs.isEmpty()) {
-                appArgs = defaultLauncherArgs;
+        public Executor.Result execute(String... args) {
+            if (launcherNoExit) {
+                return getExecutor(args).executeWithoutExitCodeCheck();
             } else {
-                appArgs = launcherArgs;
+                return HelloApp.execute(expectedExitCode, getExecutor(args));
             }
-
-            executeAndVerifyOutput(removePath, launcherArgs, appArgs);
-        }
-
-        public Executor.Result executeOnly(boolean removePath, String...args) {
-            return getExecutor(args)
-                    .saveOutput()
-                    .setRemovePath(removePath)
-                    .executeWithoutExitCodeCheck();
         }
 
         private Executor getExecutor(String...args) {
@@ -453,7 +442,7 @@ public final class HelloApp {
             Path outputFile = TKit.workDir().resolve(OUTPUT_FILENAME);
             ThrowingFunction.toFunction(Files::deleteIfExists).apply(outputFile);
 
-            final Path executablePath;
+            Path executablePath;
             if (launcherPath.isAbsolute()) {
                 executablePath = launcherPath;
             } else {
@@ -461,13 +450,27 @@ public final class HelloApp {
                 executablePath = Path.of(".").resolve(launcherPath.normalize());
             }
 
-            final List<String> launcherArgs = List.of(args);
-            return new Executor()
+            if (TKit.isWindows()) {
+                var absExecutablePath = executablePath.toAbsolutePath().normalize();
+                var shortPath = WindowsHelper.toShortPath(absExecutablePath);
+                if (shortPath.isPresent()) {
+                    TKit.trace(String.format("Will run [%s] as [%s]", executablePath, shortPath.get()));
+                    executablePath = shortPath.get();
+                }
+            }
+
+            final var executor = new Executor()
                     .setDirectory(outputFile.getParent())
+                    .saveOutput(saveOutput)
+                    .dumpOutput()
                     .setExecutable(executablePath)
-                    .addArguments(launcherArgs);
+                    .addArguments(List.of(args));
+
+            return configureEnvironment(executor);
         }
 
+        private boolean launcherNoExit;
+        private boolean saveOutput;
         private final Path launcherPath;
         private Path outputFilePath;
         private int expectedExitCode;
@@ -479,13 +482,39 @@ public final class HelloApp {
         return new AppOutputVerifier(helloAppLauncher);
     }
 
-    final static String OUTPUT_FILENAME = "appOutput.txt";
+    public static Executor.Result configureAndExecute(int expectedExitCode, Executor executor) {
+        return execute(expectedExitCode, configureEnvironment(executor));
+    }
+
+    private static Executor.Result execute(int expectedExitCode, Executor executor) {
+        if (TKit.isLinux()) {
+            final int attempts = 3;
+            final int waitBetweenAttemptsSeconds = 5;
+            return executor.executeAndRepeatUntilExitCode(expectedExitCode, attempts,
+                    waitBetweenAttemptsSeconds);
+        } else {
+            return executor.execute(expectedExitCode);
+        }
+    }
+
+    private static Executor configureEnvironment(Executor executor) {
+        if (CLEAR_JAVA_ENV_VARS) {
+            executor.removeEnvVar("JAVA_TOOL_OPTIONS");
+            executor.removeEnvVar("_JAVA_OPTIONS");
+        }
+        return executor;
+    }
+
+    static final String OUTPUT_FILENAME = "appOutput.txt";
 
     private final JavaAppDesc appDesc;
 
     private static final Path HELLO_JAVA = TKit.TEST_SRC_ROOT.resolve(
             "apps/Hello.java");
 
-    private final static String CLASS_NAME = HELLO_JAVA.getFileName().toString().split(
+    private static final String CLASS_NAME = HELLO_JAVA.getFileName().toString().split(
             "\\.", 2)[0];
+
+    private static final boolean CLEAR_JAVA_ENV_VARS = Optional.ofNullable(
+            TKit.getConfigProperty("clear-app-launcher-java-env-vars")).map(Boolean::parseBoolean).orElse(false);
 }
