@@ -23,5 +23,18 @@
 
 package listpkg;
 
-public class MyList implements List {
+/**
+ * A class.
+ */
+public abstract class MyList implements List {
+
+    /**
+     * Field name containing a digit.
+     */
+    public static final int M_3X = 2;
+
+    /**
+     * Method name containing digits.
+     */
+    public void abc123xyz() {}
 }
