@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,7 +23,7 @@
 
 /*
  * @test
- * @bug 8205418 8207229 8207230 8230847 8245786 8247334 8248641 8240658 8246774
+ * @bug 8205418 8207229 8207230 8230847 8245786 8247334 8248641 8240658 8246774 8274347
  * @summary Test the outcomes from Trees.getScope
  * @modules jdk.compiler/com.sun.tools.javac.api
  *          jdk.compiler/com.sun.tools.javac.comp
@@ -73,6 +73,9 @@ import com.sun.tools.javac.tree.JCTree.JCCase;
 import com.sun.tools.javac.tree.JCTree.JCStatement;
 import com.sun.tools.javac.util.Context;
 import com.sun.tools.javac.util.Context.Factory;
+import javax.lang.model.element.TypeElement;
+import javax.lang.model.util.ElementFilter;
+import javax.tools.JavaFileObject;
 
 import static javax.tools.JavaFileObject.Kind.SOURCE;
 
@@ -88,6 +91,8 @@ public class TestGetScopeResult {
         new TestGetScopeResult().testRecord();
         new TestGetScopeResult().testLocalRecordAnnotation();
         new TestGetScopeResult().testRuleCases();
+        new TestGetScopeResult().testNestedSwitchExpression();
+        new TestGetScopeResult().testModuleImportScope();
     }
 
     public void run() throws IOException {
@@ -147,7 +152,7 @@ public class TestGetScopeResult {
                multipleCandidates2);
 
         String[] implicitExplicitConflict1 = {
-            ":t",
+            "<error>:t",
             "s:java.lang.String",
             "super:java.lang.Object",
             "this:Test"
@@ -158,7 +163,7 @@ public class TestGetScopeResult {
 
         String[] implicitExplicitConflict2 = {
             "s:none",
-            ":t",
+            "<error>:t",
             "super:java.lang.Object",
             "this:Test"
         };
@@ -168,7 +173,7 @@ public class TestGetScopeResult {
 
         String[] noFunctionInterface = {
             "s:none",
-            ":t",
+            "<error>:t",
             "super:java.lang.Object",
             "this:Test"
         };
@@ -751,6 +756,135 @@ public class TestGetScopeResult {
         }
     }
 
+    void testNestedSwitchExpression() throws IOException {
+        JavacTool c = JavacTool.create();
+        try (StandardJavaFileManager fm = c.getStandardFileManager(null, null, null)) {
+            String code = """
+                          class Test {
+                              void t(Object o1, Object o2) {
+                                  System.err.println(switch (o1) {
+                                    case String s -> switch (j) {
+                                        case Integer i -> {
+                                            int scopeHere;
+                                            yield "";
+                                        }
+                                        default -> "";
+                                    };
+                                    default -> "";
+                                  });
+                              }
+                          }
+                          """;
+            class MyFileObject extends SimpleJavaFileObject {
+                MyFileObject() {
+                    super(URI.create("myfo:///Test.java"), SOURCE);
+                }
+                @Override
+                public String getCharContent(boolean ignoreEncodingErrors) {
+                    return code;
+                }
+            }
+            Context ctx = new Context();
+            TestAnalyzer.preRegister(ctx);
+            JavacTask t = (JavacTask) c.getTask(null, fm, null, null, null,
+                                                List.of(new MyFileObject()), ctx);
+            CompilationUnitTree cut = t.parse().iterator().next();
+            t.analyze();
+
+            List<List<String>> actual = new ArrayList<>();
+
+            new TreePathScanner<Void, Void>() {
+                @Override
+                public Void visitVariable(VariableTree node, Void p) {
+                    if (node.getName().contentEquals("scopeHere")) {
+                        Scope scope = Trees.instance(t).getScope(getCurrentPath());
+                        actual.add(dumpScope(scope));
+                        JCTree body = getCaseBody(scope);
+                        if (body == null) {
+                            throw new AssertionError("Unexpected null body.");
+                        }
+                    }
+                    return super.visitVariable(node, p);
+                }
+                JCTree getCaseBody(Scope scope) {
+                    return ((JCCase) ((JavacScope) scope).getEnv().next.next.tree).body;
+                }
+            }.scan(cut, null);
+
+            List<List<String>> expected =
+                    List.of(List.of("scopeHere:int",
+                                    "i:java.lang.Integer",
+                                    "s:java.lang.String",
+                                    "o2:java.lang.Object",
+                                    "o1:java.lang.Object",
+                                    "super:java.lang.Object",
+                                    "this:Test"
+                                ));
+
+            if (!expected.equals(actual)) {
+                throw new AssertionError("Unexpected Scope content: " + actual);
+            }
+        }
+    }
+
+    void testModuleImportScope() throws IOException {
+        JavacTool c = JavacTool.create();
+        try (StandardJavaFileManager fm = c.getStandardFileManager(null, null, null)) {
+            String code = """
+                          import module java.compiler;
+                          import java.util.*;
+                          import java.lang.System;
+                          class Test {
+                          }
+                          """;
+            Context ctx = new Context();
+            TestAnalyzer.preRegister(ctx);
+            JavaFileObject input =
+                    SimpleJavaFileObject.forSource(URI.create("myfo:///Test.java"), code);
+            JavacTask t = (JavacTask) c.getTask(null, fm, null, null, null,
+                                                List.of(input),
+                                                ctx);
+            CompilationUnitTree cut = t.parse().iterator().next();
+            t.analyze();
+
+            TreePath topLevelClass = new TreePath(new TreePath(cut), cut.getTypeDecls().get(0));
+            Scope scope = Trees.instance(t).getScope(topLevelClass);
+
+            if (scope.getEnclosingClass() == null) {
+                throw new AssertionError("Expected an enclosing class.");
+            }
+
+            scope = scope.getEnclosingScope();
+
+            if (scope.getEnclosingClass() != null) {
+                throw new AssertionError("Did not expect an enclosing class.");
+            }
+
+            asssertScopeContainsTypeWithFQN(scope, "java.lang.System");
+            asssertScopeContainsTypeWithFQN(scope, "Test");
+
+            scope = scope.getEnclosingScope();
+
+            if (scope.getEnclosingClass() != null) {
+                throw new AssertionError("Did not expect an enclosing class.");
+            }
+
+            asssertScopeContainsTypeWithFQN(scope, "java.util.List");
+
+            scope = scope.getEnclosingScope();
+
+            if (scope.getEnclosingClass() != null) {
+                throw new AssertionError("Did not expect an enclosing class.");
+            }
+
+            asssertScopeContainsTypeWithFQN(scope, "javax.tools.ToolProvider");
+
+            if (scope.getEnclosingScope() != null) {
+                throw new AssertionError("Did not expect an enclosing scope.");
+            }
+        }
+    }
+
     private List<String> dumpScope(Scope scope) {
         List<String> content = new ArrayList<>();
         while (scope.getEnclosingClass() != null) {
@@ -761,4 +895,17 @@ public class TestGetScopeResult {
         }
         return content;
     }
+
+    private void asssertScopeContainsTypeWithFQN(Scope scope, String fqn) {
+        for (TypeElement type : ElementFilter.typesIn(scope.getLocalElements())) {
+            if (type.getQualifiedName().contentEquals(fqn)) {
+                return ;
+            }
+        }
+
+        throw new AssertionError("Expected to find: " + fqn +
+                                 " in: " + scope.getLocalElements() +
+                                 ", but it is missing.");
+    }
+
 }
