@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,14 +23,12 @@
 
 /*
  * @test
- * @bug 8192920 8204588 8246774 8248843 8268869 8235876 8328339 8335896
+ * @bug 8192920 8204588 8246774 8248843 8268869 8235876 8328339 8335896 8344706
  * @summary Test source launcher
  * @library /tools/lib
- * @enablePreview
  * @modules jdk.compiler/com.sun.tools.javac.api
  *          jdk.compiler/com.sun.tools.javac.launcher
  *          jdk.compiler/com.sun.tools.javac.main
- *          java.base/jdk.internal.classfile.impl
  *          java.base/jdk.internal.module
  * @build toolbox.JavaTask toolbox.JavacTask toolbox.TestRunner toolbox.ToolBox
  * @run main SourceLauncherTest
@@ -240,25 +238,6 @@ public class SourceLauncherTest extends TestRunner {
         checkEqual("stdout", log.trim(), file.toAbsolutePath().toUri().toURL().toString());
     }
 
-    @Test
-    public void testSecurityManager(Path base) throws IOException {
-        Path sourceFile = base.resolve("HelloWorld.java");
-        tb.writeJavaFiles(base,
-                "class HelloWorld {\n" +
-                        "    public static void main(String... args) {\n" +
-                        "        System.out.println(\"Hello World!\");\n" +
-                        "    }\n" +
-                        "}");
-
-        String log = new JavaTask(tb)
-                .vmOptions("-Djava.security.manager=default")
-                .className(sourceFile.toString())
-                .run(Task.Expect.FAIL)
-                .getOutput(Task.OutputKind.STDERR);
-        checkContains("stderr", log,
-                "error: cannot use source-code launcher with a security manager enabled");
-    }
-
     @Test
     public void testSystemProperty(Path base) throws IOException {
         tb.writeJavaFiles(base,
@@ -304,6 +283,46 @@ public class SourceLauncherTest extends TestRunner {
         checkNull("exception", r.exception);
     }
 
+
+    @Test
+    public void testMainNoParams(Path base) throws IOException {
+        tb.writeJavaFiles(base,
+            "package hello;\n" +
+            "import java.util.Arrays;\n" +
+            "class World {\n" +
+            "    public static void main(String... args) {\n" +
+            "        System.out.println(\"Hello World! \" + Arrays.toString(args));\n" +
+            "    }\n" +
+            "}");
+        testSuccess(base.resolve("hello").resolve("World.java"), "Hello World! [1, 2, 3]\n");
+    }
+
+    @Test
+    public void testMainNotPublic(Path base) throws IOException {
+        tb.writeJavaFiles(base,
+            "package hello;\n" +
+            "import java.util.Arrays;\n" +
+            "class World {\n" +
+            "    static void main(String... args) {\n" +
+            "        System.out.println(\"Hello World! \" + Arrays.toString(args));\n" +
+            "    }\n" +
+            "}");
+        testSuccess(base.resolve("hello").resolve("World.java"), "Hello World! [1, 2, 3]\n");
+    }
+
+    @Test
+    public void testMainNotStatic(Path base) throws IOException {
+        tb.writeJavaFiles(base,
+            "package hello;\n" +
+            "import java.util.Arrays;\n" +
+            "class World {\n" +
+            "    public void main(String... args) {\n" +
+            "        System.out.println(\"Hello World! \" + Arrays.toString(args));\n" +
+            "    }\n" +
+            "}");
+        testSuccess(base.resolve("hello").resolve("World.java"), "Hello World! [1, 2, 3]\n");
+    }
+
     /*
      * Negative tests: such as cannot find or execute main method.
      */
@@ -323,7 +342,7 @@ public class SourceLauncherTest extends TestRunner {
             file + ":1: error: illegal character: '#'\n" +
             "#!/usr/bin/java --source " + thisVersion + "\n" +
             "^\n" +
-            file + ":1: error: class, interface, enum, or record expected\n" +
+            file + ":1: error: class, interface, annotation type, enum, record, method or field expected\n" +
             "#!/usr/bin/java --source " + thisVersion + "\n" +
             "  ^\n" +
             "2 errors\n",
@@ -537,7 +556,7 @@ public class SourceLauncherTest extends TestRunner {
             file + ":1: error: illegal character: '#'\n" +
             "#/usr/bin/java --source " + thisVersion + "\n" +
             "^\n" +
-            file + ":1: error: class, interface, enum, or record expected\n" +
+            file + ":1: error: class, interface, annotation type, enum, record, method or field expected\n" +
             "#/usr/bin/java --source " + thisVersion + "\n" +
             "  ^\n" +
             "2 errors\n",
@@ -585,30 +604,14 @@ public class SourceLauncherTest extends TestRunner {
                 "error: can't find main(String[]) method in class: NoMain");
     }
 
-    //@Test temporary disabled as enabled preview allows no-param main
+    @Test
     public void testMainBadParams(Path base) throws IOException {
         tb.writeJavaFiles(base,
-                "class BadParams { public static void main() { } }");
+                "class BadParams { public static void main(int n) { } }");
         testError(base.resolve("BadParams.java"), "",
                 "error: can't find main(String[]) method in class: BadParams");
     }
 
-    //@Test temporary disabled as enabled preview allows non-public main
-    public void testMainNotPublic(Path base) throws IOException {
-        tb.writeJavaFiles(base,
-                "class NotPublic { static void main(String... args) { } }");
-        testError(base.resolve("NotPublic.java"), "",
-                "error: can't find main(String[]) method in class: NotPublic");
-    }
-
-    //@Test temporary disabled as enabled preview allows non-static main
-    public void testMainNotStatic(Path base) throws IOException {
-        tb.writeJavaFiles(base,
-                "class NotStatic { public void main(String... args) { } }");
-        testError(base.resolve("NotStatic.java"), "",
-                "error: can't find main(String[]) method in class: NotStatic");
-    }
-
     @Test
     public void testMainNotVoid(Path base) throws IOException {
         tb.writeJavaFiles(base,
