@@ -23,10 +23,6 @@
 
 package jdk.test.lib;
 
-import java.io.IOException;
-import java.io.UncheckedIOException;
-import java.nio.file.Files;
-import java.nio.file.Path;
 import java.util.Arrays;
 import java.util.HexFormat;
 import java.util.Objects;
@@ -238,59 +234,58 @@ public class Asserts {
     }
 
     /**
-     * Asserts that {@code lhs} is the same byte array as {@code rhs}.
+     * Asserts that {@code actual} has the same content as {@code expected}.
      *
-     * @param lhs The left hand side of the comparison.
-     * @param rhs The right hand side of the comparison.
+     * @param expected The expected value
+     * @param actual The actual value
      * @throws RuntimeException if the assertion is not true.
      * @see #assertEqualsByteArray(byte[], byte[], String)
      */
-    public static void assertEqualsByteArray(byte[] lhs, byte[] rhs) {
-        assertEqualsByteArray(lhs, rhs, null);
+    public static void assertEqualsByteArray(byte[] expected, byte[] actual) {
+        assertEqualsByteArray(expected, actual, null);
     }
 
     /**
-     * Asserts that {@code lhs} is not the same byte array as {@code rhs}.
+     * Asserts that {@code actual} does not have the same content as {@code unexpected}.
      *
-     * @param lhs The left hand side of the comparison.
-     * @param rhs The right hand side of the comparison.
+     * @param unexpected The unexpected value
+     * @param actual The actual value
      * @throws RuntimeException if the assertion is not true.
      * @see #assertNotEqualsByteArray(byte[], byte[], String)
      */
-    public static void assertNotEqualsByteArray(byte[] lhs, byte[] rhs) {
-        assertNotEqualsByteArray(lhs, rhs, null);
+    public static void assertNotEqualsByteArray(byte[] unexpected, byte[] actual) {
+        assertNotEqualsByteArray(unexpected, actual, null);
     }
 
     /**
-     * Asserts that {@code lhs} is the same byte array as {@code rhs}.
+     * Asserts that {@code actual} is the same byte array as {@code expected}.
      *
-     * @param lhs The left hand side of the comparison.
-     * @param rhs The right hand side of the comparison.
+     * @param expected The expected value
+     * @param actual The actual value
      * @param msg A description of the assumption; {@code null} for a default message.
      * @throws RuntimeException if the assertion is not true.
      */
-    public static void assertEqualsByteArray(byte[] lhs, byte[] rhs, String msg) {
-        if (!Arrays.equals(lhs, rhs)) {
+    public static void assertEqualsByteArray(byte[] expected, byte[] actual, String msg) {
+        if (!Arrays.equals(expected, actual)) {
             msg = Objects.toString(msg, "assertEqualsByteArray")
-                    + ": expected " + HexFormat.of().formatHex(lhs)
-                    + " to equal " + HexFormat.of().formatHex(rhs);
+                    + ": expected " + HexFormat.of().formatHex(expected)
+                    + " but was " + HexFormat.of().formatHex(actual);
             fail(msg);
         }
     }
 
     /**
-     * Asserts that {@code lhs} is not the same byte array as {@code rhs}.
+     * Asserts that {@code actual} is not the same byte array as {@code unexpected}.
      *
-     * @param lhs The left hand side of the comparison.
-     * @param rhs The right hand side of the comparison.
+     * @param unexpected The unexpected value
+     * @param actual The actual value
      * @param msg A description of the assumption; {@code null} for a default message.
      * @throws RuntimeException if the assertion is not true.
      */
-    public static void assertNotEqualsByteArray(byte[] lhs, byte[] rhs, String msg) {
-        if (Arrays.equals(lhs, rhs)) {
+    public static void assertNotEqualsByteArray(byte[] unexpected, byte[] actual, String msg) {
+        if (Arrays.equals(unexpected, actual)) {
             msg = Objects.toString(msg, "assertNotEqualsByteArray")
-                    + ": expected " + HexFormat.of().formatHex(lhs)
-                    + " to not equal " + HexFormat.of().formatHex(rhs);
+                    + ": expected not equals but was " + HexFormat.of().formatHex(actual);
             fail(msg);
         }
     }
@@ -408,50 +403,49 @@ public class Asserts {
     /**
      * Shorthand for {@link #assertNotEquals(Object, Object)}.
      *
-     * @param lhs The left hand side of the comparison.
-     * @param rhs The right hand side of the comparison.
+     * @param unexpected The unexpected value
+     * @param actual The actual value
      * @see #assertNotEquals(Object, Object)
      */
-    public static void assertNE(Object lhs, Object rhs) {
-        assertNotEquals(lhs, rhs);
+    public static void assertNE(Object unexpected, Object actual) {
+        assertNotEquals(unexpected, actual);
     }
 
     /**
      * Shorthand for {@link #assertNotEquals(Object, Object, String)}.
      *
-     * @param lhs The left hand side of the comparison.
-     * @param rhs The right hand side of the comparison.
+     * @param unexpected The unexpected value
+     * @param actual The actual value
      * @param msg A description of the assumption; {@code null} for a default message.
      * @see #assertNotEquals(Object, Object, String)
      */
-    public static void assertNE(Object lhs, Object rhs, String msg) {
-        assertNotEquals(lhs, rhs, msg);
+    public static void assertNE(Object unexpected, Object actual, String msg) {
+        assertNotEquals(unexpected, actual, msg);
     }
 
     /**
      * Calls {@link #assertNotEquals(Object, Object, String)} with a default message.
      *
-     * @param lhs The left hand side of the comparison.
-     * @param rhs The right hand side of the comparison.
+     * @param unexpected The unexpected value
+     * @param actual The actual value
      * @see #assertNotEquals(Object, Object, String)
      */
-    public static void assertNotEquals(Object lhs, Object rhs) {
-        assertNotEquals(lhs, rhs, null);
+    public static void assertNotEquals(Object unexpected, Object actual) {
+        assertNotEquals(unexpected, actual, null);
     }
 
     /**
-     * Asserts that {@code lhs} is not equal to {@code rhs}.
+     * Asserts that {@code actual} is not equal to {@code unexpected}.
      *
-     * @param lhs The left hand side of the comparison.
-     * @param rhs The right hand side of the comparison.
+     * @param unexpected The unexpected value
+     * @param actual The actual value
      * @param msg A description of the assumption; {@code null} for a default message.
      * @throws RuntimeException if the assertion is not true.
      */
-    public static void assertNotEquals(Object lhs, Object rhs, String msg) {
-        if ((lhs == rhs) || (lhs != null && lhs.equals(rhs))) {
+    public static void assertNotEquals(Object unexpected, Object actual, String msg) {
+        if ((unexpected == actual) || (unexpected != null && unexpected.equals(actual))) {
             msg = Objects.toString(msg, "assertNotEquals")
-                    + ": expected " + Objects.toString(lhs)
-                    + " to not equal " + Objects.toString(rhs);
+                    + ": expected not equals but was " + Objects.toString(actual);
             fail(msg);
         }
     }
@@ -613,28 +607,6 @@ public class Asserts {
         }
     }
 
-    /**
-     * Asserts that contents of two files are equal.
-     *
-     * @param f1 The path of the first file to compare
-     * @param f2 The path of the second file to compare
-     * @throws RuntimeException on mismatch or I/O failure
-     */
-    public static void assertFileContentsEqual(Path f1, Path f2) {
-        long mismatchIndex = 0;
-        try {
-            mismatchIndex = Files.mismatch(f1, f2);
-        } catch (IOException exception) {
-            throw new UncheckedIOException(exception);
-        }
-        if (mismatchIndex >= 0) {
-            String message = String.format(
-                    "Contents of files '%s' and '%s' mismatch at index %d",
-                    f1, f2, mismatchIndex);
-            fail(message);
-        }
-    }
-
     /**
      * A functional interface for executing tests in assertThrownException
      */
