@@ -23,8 +23,6 @@
 
 package jdk.test.lib.artifacts;
 
-import jtreg.SkippedException;
-
 import java.nio.file.Path;
 import java.util.HashMap;
 import java.util.Map;
@@ -70,38 +68,6 @@ public class ArtifactResolver {
         return  manager.resolve(name, artifactDescription, unpack);
     }
 
-    /**
-     * Retrieve an artifact/library/file from a repository or local file system.
-     * <p>
-     * Artifacts are defined with the {@link jdk.test.lib.artifacts.Artifact}
-     * annotation.
-     * <p>
-     * If you have a local version of a dependency that you want to use, you can
-     * specify that by setting the system property:
-     * <code>jdk.test.lib.artifacts.ARTIFACT_NAME</code>. Where ARTIFACT_NAME
-     * is the name field of the Artifact annotation.
-     * <p>
-     * Generally, tests that use this method should be run with <code>make test</code>.
-     * However, tests can also be run with <code>jtreg</code> but you must have a
-     * local copy of the artifact and the system property must be set as specified
-     * above.
-     *
-     * @param klass a class annotated with {@link jdk.test.lib.artifacts.Artifact}
-     * @return the local path to the artifact. If the artifact is a compressed
-     * file that gets unpacked, this path will point to the root
-     * directory of the uncompressed file(s).
-     * @throws SkippedException thrown if the artifact cannot be found
-     */
-    public static Path fetchOne(Class<?> klass) {
-        try {
-            return ArtifactResolver.resolve(klass).entrySet().stream()
-                    .findAny().get().getValue();
-        } catch (ArtifactResolverException e) {
-            Artifact artifact = klass.getAnnotation(Artifact.class);
-            throw new SkippedException("Cannot find the artifact " + artifact.name(), e);
-        }
-    }
-
     private static String artifactName(Artifact artifact) {
         // Format of the artifact name is <organization>.<name>-<revision>(-<classifier>)
         String name = String.format("%s.%s-%s", artifact.organization(), artifact.name(), artifact.revision());
