@@ -29,18 +29,11 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.function.BiFunction;
 import java.util.function.Function;
-import java.security.BasicPermission;
 import java.util.Objects;
 
 import jdk.test.whitebox.parser.DiagnosticCommand;
 
 public class WhiteBox {
-  @SuppressWarnings("serial")
-  public static class WhiteBoxPermission extends BasicPermission {
-    public WhiteBoxPermission(String s) {
-      super(s);
-    }
-  }
 
   private WhiteBox() {}
   private static final WhiteBox instance = new WhiteBox();
@@ -55,11 +48,6 @@ public class WhiteBox {
    * untrusted code.
    */
   public synchronized static WhiteBox getWhiteBox() {
-    @SuppressWarnings("removal")
-    SecurityManager sm = System.getSecurityManager();
-    if (sm != null) {
-      sm.checkPermission(new WhiteBoxPermission("getInstance"));
-    }
     return instance;
   }
 
@@ -104,6 +92,8 @@ public class WhiteBox {
   // printed by the VM.
   public native String printString(String str, int maxLength);
 
+  public native void lockAndStuckInSafepoint();
+
   public int countAliveClasses(String name) {
     // Make sure class name is in the correct format
     return countAliveClasses0(name.replace('.', '/'));
@@ -540,15 +530,19 @@ public class WhiteBox {
   public native void destroyMetaspaceTestContext(long context);
   public native void purgeMetaspaceTestContext(long context);
   public native void printMetaspaceTestContext(long context);
-  public native long getTotalCommittedWordsInMetaspaceTestContext(long context);
-  public native long getTotalUsedWordsInMetaspaceTestContext(long context);
+  public native long getTotalCommittedBytesInMetaspaceTestContext(long context);
+  public native long getTotalUsedBytesInMetaspaceTestContext(long context);
   public native long createArenaInTestContext(long context, boolean is_micro);
   public native void destroyMetaspaceTestArena(long arena);
-  public native long allocateFromMetaspaceTestArena(long arena, long word_size);
-  public native void deallocateToMetaspaceTestArena(long arena, long p, long word_size);
+  public native long allocateFromMetaspaceTestArena(long arena, long size);
+  public native void deallocateToMetaspaceTestArena(long arena, long p, long size);
 
   public native long maxMetaspaceAllocationSize();
 
+  // Word size measured in bytes
+  public native long wordSize();
+  public native long rootChunkWordSize();
+
   // Don't use these methods directly
   // Use jdk.test.whitebox.gc.GC class instead.
   public native boolean isGCSupported(int name);
@@ -777,9 +771,6 @@ public class WhiteBox {
   // Resolved Method Table
   public native long resolvedMethodItemsCount();
 
-  // Protection Domain Table
-  public native int protectionDomainRemovedCount();
-
   public native int getKlassMetadataSize(Class<?> c);
 
   // ThreadSMR GC safety check for threadObj
@@ -795,10 +786,6 @@ public class WhiteBox {
 
   public native void waitUnsafe(int time_ms);
 
-  public native void lockCritical();
-
-  public native void unlockCritical();
-
   public native void pinObject(Object o);
 
   public native void unpinObject(Object o);
@@ -807,4 +794,6 @@ public class WhiteBox {
 
   public native void preTouchMemory(long addr, long size);
   public native long rss();
+
+  public native boolean isStatic();
 }
